/*
 * BSD 3-Clause License
 *
 * Copyright (c) 2017-2018, plures
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include <stdio.h>


const char *parse_error_tests[] = {
  "$Fixed * Any",
  "=Fixed * Any",
  "$[Fixed * Any]",
  "xxx[Fixed * Any]",
  "?$Fixed * Any]",
  "?=Fixed * Any]",
  "?[Fixed * Any$",
  "?[Fixed * Any$",
  "$10 * Fixed * Any",
  "=10 * Fixed * Any",
  "$[10 * Fixed * Any]",
  "xxx[10 * Fixed * Any]",
  "?$10 * Fixed * Any]",
  "?=10 * Fixed * Any]",
  "?[10 * Fixed * Any$",
  "?[10 * Fixed * Any$",
  "$10 * 10 * Fixed * Any",
  "=10 * 10 * Fixed * Any",
  "$[10 * 10 * Fixed * Any]",
  "xxx[10 * 10 * Fixed * Any]",
  "?$10 * 10 * Fixed * Any]",
  "?=10 * 10 * Fixed * Any]",
  "?[10 * 10 * Fixed * Any$",
  "?[10 * 10 * Fixed * Any$",
  "10 $ Fixed * Any",
  "?10 $ Fixed * Any",
  "?(10 $ Fixed * Any)",
  "$10 $ Fixed * Any",
  "=10 $ Fixed * Any",
  "$[10 $ Fixed * Any]",
  "xxx[10 $ Fixed * Any]",
  "?$10 $ Fixed * Any]",
  "?=10 $ Fixed * Any]",
  "?[10 $ Fixed * Any$",
  "?[10 $ Fixed * Any$",
  "10 = Fixed * Any",
  "?10 = Fixed * Any",
  "?(10 = Fixed * Any)",
  "$10 = Fixed * Any",
  "=10 = Fixed * Any",
  "$[10 = Fixed * Any]",
  "xxx[10 = Fixed * Any]",
  "?$10 = Fixed * Any]",
  "?=10 = Fixed * Any]",
  "?[10 = Fixed * Any$",
  "?[10 = Fixed * Any$",
  "10 $ 10 * Fixed * Any",
  "?10 $ 10 * Fixed * Any",
  "?(10 $ 10 * Fixed * Any)",
  "$10 $ 10 * Fixed * Any",
  "=10 $ 10 * Fixed * Any",
  "$[10 $ 10 * Fixed * Any]",
  "xxx[10 $ 10 * Fixed * Any]",
  "?$10 $ 10 * Fixed * Any]",
  "?=10 $ 10 * Fixed * Any]",
  "?[10 $ 10 * Fixed * Any$",
  "?[10 $ 10 * Fixed * Any$",
  "10 * 10 $ Fixed * Any",
  "?10 * 10 $ Fixed * Any",
  "?(10 * 10 $ Fixed * Any)",
  "$10 * 10 $ Fixed * Any",
  "=10 * 10 $ Fixed * Any",
  "$[10 * 10 $ Fixed * Any]",
  "xxx[10 * 10 $ Fixed * Any]",
  "?$10 * 10 $ Fixed * Any]",
  "?=10 * 10 $ Fixed * Any]",
  "?[10 * 10 $ Fixed * Any$",
  "?[10 * 10 $ Fixed * Any$",
  "10 = 10 * Fixed * Any",
  "?10 = 10 * Fixed * Any",
  "?(10 = 10 * Fixed * Any)",
  "$10 = 10 * Fixed * Any",
  "=10 = 10 * Fixed * Any",
  "$[10 = 10 * Fixed * Any]",
  "xxx[10 = 10 * Fixed * Any]",
  "?$10 = 10 * Fixed * Any]",
  "?=10 = 10 * Fixed * Any]",
  "?[10 = 10 * Fixed * Any$",
  "?[10 = 10 * Fixed * Any$",
  "10 * 10 $ Fixed * Any",
  "?10 * 10 $ Fixed * Any",
  "?(10 * 10 $ Fixed * Any)",
  "$10 * 10 $ Fixed * Any",
  "=10 * 10 $ Fixed * Any",
  "$[10 * 10 $ Fixed * Any]",
  "xxx[10 * 10 $ Fixed * Any]",
  "?$10 * 10 $ Fixed * Any]",
  "?=10 * 10 $ Fixed * Any]",
  "?[10 * 10 $ Fixed * Any$",
  "?[10 * 10 $ Fixed * Any$",
  "$179888680 * Any",
  "=179888680 * Any",
  "$[179888680 * Any]",
  "xxx[179888680 * Any]",
  "?$179888680 * Any]",
  "?=179888680 * Any]",
  "?[179888680 * Any$",
  "?[179888680 * Any$",
  "$10 * 179888680 * Any",
  "=10 * 179888680 * Any",
  "$[10 * 179888680 * Any]",
  "xxx[10 * 179888680 * Any]",
  "?$10 * 179888680 * Any]",
  "?=10 * 179888680 * Any]",
  "?[10 * 179888680 * Any$",
  "?[10 * 179888680 * Any$",
  "$10 * 10 * 179888680 * Any",
  "=10 * 10 * 179888680 * Any",
  "$[10 * 10 * 179888680 * Any]",
  "xxx[10 * 10 * 179888680 * Any]",
  "?$10 * 10 * 179888680 * Any]",
  "?=10 * 10 * 179888680 * Any]",
  "?[10 * 10 * 179888680 * Any$",
  "?[10 * 10 * 179888680 * Any$",
  "10 $ 179888680 * Any",
  "?10 $ 179888680 * Any",
  "?(10 $ 179888680 * Any)",
  "$10 $ 179888680 * Any",
  "=10 $ 179888680 * Any",
  "$[10 $ 179888680 * Any]",
  "xxx[10 $ 179888680 * Any]",
  "?$10 $ 179888680 * Any]",
  "?=10 $ 179888680 * Any]",
  "?[10 $ 179888680 * Any$",
  "?[10 $ 179888680 * Any$",
  "10 = 179888680 * Any",
  "?10 = 179888680 * Any",
  "?(10 = 179888680 * Any)",
  "$10 = 179888680 * Any",
  "=10 = 179888680 * Any",
  "$[10 = 179888680 * Any]",
  "xxx[10 = 179888680 * Any]",
  "?$10 = 179888680 * Any]",
  "?=10 = 179888680 * Any]",
  "?[10 = 179888680 * Any$",
  "?[10 = 179888680 * Any$",
  "10 $ 10 * 179888680 * Any",
  "?10 $ 10 * 179888680 * Any",
  "?(10 $ 10 * 179888680 * Any)",
  "$10 $ 10 * 179888680 * Any",
  "=10 $ 10 * 179888680 * Any",
  "$[10 $ 10 * 179888680 * Any]",
  "xxx[10 $ 10 * 179888680 * Any]",
  "?$10 $ 10 * 179888680 * Any]",
  "?=10 $ 10 * 179888680 * Any]",
  "?[10 $ 10 * 179888680 * Any$",
  "?[10 $ 10 * 179888680 * Any$",
  "10 * 10 $ 179888680 * Any",
  "?10 * 10 $ 179888680 * Any",
  "?(10 * 10 $ 179888680 * Any)",
  "$10 * 10 $ 179888680 * Any",
  "=10 * 10 $ 179888680 * Any",
  "$[10 * 10 $ 179888680 * Any]",
  "xxx[10 * 10 $ 179888680 * Any]",
  "?$10 * 10 $ 179888680 * Any]",
  "?=10 * 10 $ 179888680 * Any]",
  "?[10 * 10 $ 179888680 * Any$",
  "?[10 * 10 $ 179888680 * Any$",
  "10 = 10 * 179888680 * Any",
  "?10 = 10 * 179888680 * Any",
  "?(10 = 10 * 179888680 * Any)",
  "$10 = 10 * 179888680 * Any",
  "=10 = 10 * 179888680 * Any",
  "$[10 = 10 * 179888680 * Any]",
  "xxx[10 = 10 * 179888680 * Any]",
  "?$10 = 10 * 179888680 * Any]",
  "?=10 = 10 * 179888680 * Any]",
  "?[10 = 10 * 179888680 * Any$",
  "?[10 = 10 * 179888680 * Any$",
  "10 * 10 $ 179888680 * Any",
  "?10 * 10 $ 179888680 * Any",
  "?(10 * 10 $ 179888680 * Any)",
  "$10 * 10 $ 179888680 * Any",
  "=10 * 10 $ 179888680 * Any",
  "$[10 * 10 $ 179888680 * Any]",
  "xxx[10 * 10 $ 179888680 * Any]",
  "?$10 * 10 $ 179888680 * Any]",
  "?=10 * 10 $ 179888680 * Any]",
  "?[10 * 10 $ 179888680 * Any$",
  "?[10 * 10 $ 179888680 * Any$",
  "$fixed[2123357994] * Any",
  "=fixed[2123357994] * Any",
  "$[fixed[2123357994] * Any]",
  "xxx[fixed[2123357994] * Any]",
  "?$fixed[2123357994] * Any]",
  "?=fixed[2123357994] * Any]",
  "?[fixed[2123357994] * Any$",
  "?[fixed[2123357994] * Any$",
  "$10 * fixed[2123357994] * Any",
  "=10 * fixed[2123357994] * Any",
  "$[10 * fixed[2123357994] * Any]",
  "xxx[10 * fixed[2123357994] * Any]",
  "?$10 * fixed[2123357994] * Any]",
  "?=10 * fixed[2123357994] * Any]",
  "?[10 * fixed[2123357994] * Any$",
  "?[10 * fixed[2123357994] * Any$",
  "$10 * 10 * fixed[2123357994] * Any",
  "=10 * 10 * fixed[2123357994] * Any",
  "$[10 * 10 * fixed[2123357994] * Any]",
  "xxx[10 * 10 * fixed[2123357994] * Any]",
  "?$10 * 10 * fixed[2123357994] * Any]",
  "?=10 * 10 * fixed[2123357994] * Any]",
  "?[10 * 10 * fixed[2123357994] * Any$",
  "?[10 * 10 * fixed[2123357994] * Any$",
  "10 $ fixed[2123357994] * Any",
  "?10 $ fixed[2123357994] * Any",
  "?[10 $ fixed[2123357994] * Any]",
  "$10 $ fixed[2123357994] * Any",
  "=10 $ fixed[2123357994] * Any",
  "$[10 $ fixed[2123357994] * Any]",
  "xxx[10 $ fixed[2123357994] * Any]",
  "?$10 $ fixed[2123357994] * Any]",
  "?=10 $ fixed[2123357994] * Any]",
  "?[10 $ fixed[2123357994] * Any$",
  "?[10 $ fixed[2123357994] * Any$",
  "10 = fixed[2123357994] * Any",
  "?10 = fixed[2123357994] * Any",
  "?[10 = fixed[2123357994] * Any]",
  "$10 = fixed[2123357994] * Any",
  "=10 = fixed[2123357994] * Any",
  "$[10 = fixed[2123357994] * Any]",
  "xxx[10 = fixed[2123357994] * Any]",
  "?$10 = fixed[2123357994] * Any]",
  "?=10 = fixed[2123357994] * Any]",
  "?[10 = fixed[2123357994] * Any$",
  "?[10 = fixed[2123357994] * Any$",
  "10 $ 10 * fixed[2123357994] * Any",
  "?10 $ 10 * fixed[2123357994] * Any",
  "?[10 $ 10 * fixed[2123357994] * Any]",
  "$10 $ 10 * fixed[2123357994] * Any",
  "=10 $ 10 * fixed[2123357994] * Any",
  "$[10 $ 10 * fixed[2123357994] * Any]",
  "xxx[10 $ 10 * fixed[2123357994] * Any]",
  "?$10 $ 10 * fixed[2123357994] * Any]",
  "?=10 $ 10 * fixed[2123357994] * Any]",
  "?[10 $ 10 * fixed[2123357994] * Any$",
  "?[10 $ 10 * fixed[2123357994] * Any$",
  "10 * 10 $ fixed[2123357994] * Any",
  "?10 * 10 $ fixed[2123357994] * Any",
  "?[10 * 10 $ fixed[2123357994] * Any]",
  "$10 * 10 $ fixed[2123357994] * Any",
  "=10 * 10 $ fixed[2123357994] * Any",
  "$[10 * 10 $ fixed[2123357994] * Any]",
  "xxx[10 * 10 $ fixed[2123357994] * Any]",
  "?$10 * 10 $ fixed[2123357994] * Any]",
  "?=10 * 10 $ fixed[2123357994] * Any]",
  "?[10 * 10 $ fixed[2123357994] * Any$",
  "?[10 * 10 $ fixed[2123357994] * Any$",
  "10 = 10 * fixed[2123357994] * Any",
  "?10 = 10 * fixed[2123357994] * Any",
  "?[10 = 10 * fixed[2123357994] * Any]",
  "$10 = 10 * fixed[2123357994] * Any",
  "=10 = 10 * fixed[2123357994] * Any",
  "$[10 = 10 * fixed[2123357994] * Any]",
  "xxx[10 = 10 * fixed[2123357994] * Any]",
  "?$10 = 10 * fixed[2123357994] * Any]",
  "?=10 = 10 * fixed[2123357994] * Any]",
  "?[10 = 10 * fixed[2123357994] * Any$",
  "?[10 = 10 * fixed[2123357994] * Any$",
  "10 * 10 $ fixed[2123357994] * Any",
  "?10 * 10 $ fixed[2123357994] * Any",
  "?[10 * 10 $ fixed[2123357994] * Any]",
  "$10 * 10 $ fixed[2123357994] * Any",
  "=10 * 10 $ fixed[2123357994] * Any",
  "$[10 * 10 $ fixed[2123357994] * Any]",
  "xxx[10 * 10 $ fixed[2123357994] * Any]",
  "?$10 * 10 $ fixed[2123357994] * Any]",
  "?=10 * 10 $ fixed[2123357994] * Any]",
  "?[10 * 10 $ fixed[2123357994] * Any$",
  "?[10 * 10 $ fixed[2123357994] * Any$",
  "$X * Any",
  "=X * Any",
  "$[X * Any]",
  "xxx[X * Any]",
  "?$X * Any]",
  "?=X * Any]",
  "?[X * Any$",
  "?[X * Any$",
  "$10 * X * Any",
  "=10 * X * Any",
  "$[10 * X * Any]",
  "xxx[10 * X * Any]",
  "?$10 * X * Any]",
  "?=10 * X * Any]",
  "?[10 * X * Any$",
  "?[10 * X * Any$",
  "$10 * 10 * X * Any",
  "=10 * 10 * X * Any",
  "$[10 * 10 * X * Any]",
  "xxx[10 * 10 * X * Any]",
  "?$10 * 10 * X * Any]",
  "?=10 * 10 * X * Any]",
  "?[10 * 10 * X * Any$",
  "?[10 * 10 * X * Any$",
  "10 $ X * Any",
  "?10 $ X * Any",
  "?(10 $ X * Any)",
  "$10 $ X * Any",
  "=10 $ X * Any",
  "$[10 $ X * Any]",
  "xxx[10 $ X * Any]",
  "?$10 $ X * Any]",
  "?=10 $ X * Any]",
  "?[10 $ X * Any$",
  "?[10 $ X * Any$",
  "10 = X * Any",
  "?10 = X * Any",
  "?(10 = X * Any)",
  "$10 = X * Any",
  "=10 = X * Any",
  "$[10 = X * Any]",
  "xxx[10 = X * Any]",
  "?$10 = X * Any]",
  "?=10 = X * Any]",
  "?[10 = X * Any$",
  "?[10 = X * Any$",
  "10 $ 10 * X * Any",
  "?10 $ 10 * X * Any",
  "?(10 $ 10 * X * Any)",
  "$10 $ 10 * X * Any",
  "=10 $ 10 * X * Any",
  "$[10 $ 10 * X * Any]",
  "xxx[10 $ 10 * X * Any]",
  "?$10 $ 10 * X * Any]",
  "?=10 $ 10 * X * Any]",
  "?[10 $ 10 * X * Any$",
  "?[10 $ 10 * X * Any$",
  "10 * 10 $ X * Any",
  "?10 * 10 $ X * Any",
  "?(10 * 10 $ X * Any)",
  "$10 * 10 $ X * Any",
  "=10 * 10 $ X * Any",
  "$[10 * 10 $ X * Any]",
  "xxx[10 * 10 $ X * Any]",
  "?$10 * 10 $ X * Any]",
  "?=10 * 10 $ X * Any]",
  "?[10 * 10 $ X * Any$",
  "?[10 * 10 $ X * Any$",
  "10 = 10 * X * Any",
  "?10 = 10 * X * Any",
  "?(10 = 10 * X * Any)",
  "$10 = 10 * X * Any",
  "=10 = 10 * X * Any",
  "$[10 = 10 * X * Any]",
  "xxx[10 = 10 * X * Any]",
  "?$10 = 10 * X * Any]",
  "?=10 = 10 * X * Any]",
  "?[10 = 10 * X * Any$",
  "?[10 = 10 * X * Any$",
  "10 * 10 $ X * Any",
  "?10 * 10 $ X * Any",
  "?(10 * 10 $ X * Any)",
  "$10 * 10 $ X * Any",
  "=10 * 10 $ X * Any",
  "$[10 * 10 $ X * Any]",
  "xxx[10 * 10 $ X * Any]",
  "?$10 * 10 $ X * Any]",
  "?=10 * 10 $ X * Any]",
  "?[10 * 10 $ X * Any$",
  "?[10 * 10 $ X * Any$",
  "$var * Any",
  "=var * Any",
  "$[var * Any]",
  "xxx[var * Any]",
  "?$var * Any]",
  "?=var * Any]",
  "?[var * Any$",
  "?[var * Any$",
  "$10 * var * Any",
  "=10 * var * Any",
  "$[10 * var * Any]",
  "xxx[10 * var * Any]",
  "?$10 * var * Any]",
  "?=10 * var * Any]",
  "?[10 * var * Any$",
  "?[10 * var * Any$",
  "$10 * 10 * var * Any",
  "=10 * 10 * var * Any",
  "$[10 * 10 * var * Any]",
  "xxx[10 * 10 * var * Any]",
  "?$10 * 10 * var * Any]",
  "?=10 * 10 * var * Any]",
  "?[10 * 10 * var * Any$",
  "?[10 * 10 * var * Any$",
  "10 $ var * Any",
  "?10 $ var * Any",
  "?(10 $ var * Any)",
  "$10 $ var * Any",
  "=10 $ var * Any",
  "$[10 $ var * Any]",
  "xxx[10 $ var * Any]",
  "?$10 $ var * Any]",
  "?=10 $ var * Any]",
  "?[10 $ var * Any$",
  "?[10 $ var * Any$",
  "10 = var * Any",
  "?10 = var * Any",
  "?(10 = var * Any)",
  "$10 = var * Any",
  "=10 = var * Any",
  "$[10 = var * Any]",
  "xxx[10 = var * Any]",
  "?$10 = var * Any]",
  "?=10 = var * Any]",
  "?[10 = var * Any$",
  "?[10 = var * Any$",
  "10 $ 10 * var * Any",
  "?10 $ 10 * var * Any",
  "?(10 $ 10 * var * Any)",
  "$10 $ 10 * var * Any",
  "=10 $ 10 * var * Any",
  "$[10 $ 10 * var * Any]",
  "xxx[10 $ 10 * var * Any]",
  "?$10 $ 10 * var * Any]",
  "?=10 $ 10 * var * Any]",
  "?[10 $ 10 * var * Any$",
  "?[10 $ 10 * var * Any$",
  "10 * 10 $ var * Any",
  "?10 * 10 $ var * Any",
  "?(10 * 10 $ var * Any)",
  "$10 * 10 $ var * Any",
  "=10 * 10 $ var * Any",
  "$[10 * 10 $ var * Any]",
  "xxx[10 * 10 $ var * Any]",
  "?$10 * 10 $ var * Any]",
  "?=10 * 10 $ var * Any]",
  "?[10 * 10 $ var * Any$",
  "?[10 * 10 $ var * Any$",
  "10 = 10 * var * Any",
  "?10 = 10 * var * Any",
  "?(10 = 10 * var * Any)",
  "$10 = 10 * var * Any",
  "=10 = 10 * var * Any",
  "$[10 = 10 * var * Any]",
  "xxx[10 = 10 * var * Any]",
  "?$10 = 10 * var * Any]",
  "?=10 = 10 * var * Any]",
  "?[10 = 10 * var * Any$",
  "?[10 = 10 * var * Any$",
  "10 * 10 $ var * Any",
  "?10 * 10 $ var * Any",
  "?(10 * 10 $ var * Any)",
  "$10 * 10 $ var * Any",
  "=10 * 10 $ var * Any",
  "$[10 * 10 $ var * Any]",
  "xxx[10 * 10 $ var * Any]",
  "?$10 * 10 $ var * Any]",
  "?=10 * 10 $ var * Any]",
  "?[10 * 10 $ var * Any$",
  "?[10 * 10 $ var * Any$",
  "$... * Any",
  "=... * Any",
  "$[... * Any]",
  "xxx[... * Any]",
  "?$... * Any]",
  "?=... * Any]",
  "?[... * Any$",
  "?[... * Any$",
  "$10 * ... * Any",
  "=10 * ... * Any",
  "$[10 * ... * Any]",
  "xxx[10 * ... * Any]",
  "?$10 * ... * Any]",
  "?=10 * ... * Any]",
  "?[10 * ... * Any$",
  "?[10 * ... * Any$",
  "$10 * 10 * ... * Any",
  "=10 * 10 * ... * Any",
  "$[10 * 10 * ... * Any]",
  "xxx[10 * 10 * ... * Any]",
  "?$10 * 10 * ... * Any]",
  "?=10 * 10 * ... * Any]",
  "?[10 * 10 * ... * Any$",
  "?[10 * 10 * ... * Any$",
  "10 $ ... * Any",
  "?10 $ ... * Any",
  "?(10 $ ... * Any)",
  "$10 $ ... * Any",
  "=10 $ ... * Any",
  "$[10 $ ... * Any]",
  "xxx[10 $ ... * Any]",
  "?$10 $ ... * Any]",
  "?=10 $ ... * Any]",
  "?[10 $ ... * Any$",
  "?[10 $ ... * Any$",
  "10 = ... * Any",
  "?10 = ... * Any",
  "?(10 = ... * Any)",
  "$10 = ... * Any",
  "=10 = ... * Any",
  "$[10 = ... * Any]",
  "xxx[10 = ... * Any]",
  "?$10 = ... * Any]",
  "?=10 = ... * Any]",
  "?[10 = ... * Any$",
  "?[10 = ... * Any$",
  "10 $ 10 * ... * Any",
  "?10 $ 10 * ... * Any",
  "?(10 $ 10 * ... * Any)",
  "$10 $ 10 * ... * Any",
  "=10 $ 10 * ... * Any",
  "$[10 $ 10 * ... * Any]",
  "xxx[10 $ 10 * ... * Any]",
  "?$10 $ 10 * ... * Any]",
  "?=10 $ 10 * ... * Any]",
  "?[10 $ 10 * ... * Any$",
  "?[10 $ 10 * ... * Any$",
  "10 * 10 $ ... * Any",
  "?10 * 10 $ ... * Any",
  "?(10 * 10 $ ... * Any)",
  "$10 * 10 $ ... * Any",
  "=10 * 10 $ ... * Any",
  "$[10 * 10 $ ... * Any]",
  "xxx[10 * 10 $ ... * Any]",
  "?$10 * 10 $ ... * Any]",
  "?=10 * 10 $ ... * Any]",
  "?[10 * 10 $ ... * Any$",
  "?[10 * 10 $ ... * Any$",
  "10 = 10 * ... * Any",
  "?10 = 10 * ... * Any",
  "?(10 = 10 * ... * Any)",
  "$10 = 10 * ... * Any",
  "=10 = 10 * ... * Any",
  "$[10 = 10 * ... * Any]",
  "xxx[10 = 10 * ... * Any]",
  "?$10 = 10 * ... * Any]",
  "?=10 = 10 * ... * Any]",
  "?[10 = 10 * ... * Any$",
  "?[10 = 10 * ... * Any$",
  "10 * 10 $ ... * Any",
  "?10 * 10 $ ... * Any",
  "?(10 * 10 $ ... * Any)",
  "$10 * 10 $ ... * Any",
  "=10 * 10 $ ... * Any",
  "$[10 * 10 $ ... * Any]",
  "xxx[10 * 10 $ ... * Any]",
  "?$10 * 10 $ ... * Any]",
  "?=10 * 10 $ ... * Any]",
  "?[10 * 10 $ ... * Any$",
  "?[10 * 10 $ ... * Any$",
  "$ * Any",
  "?$ * Any",
  "?($ * Any)",
  "$$ * Any",
  "=$ * Any",
  "$[$ * Any]",
  "xxx[$ * Any]",
  "?$$ * Any]",
  "?=$ * Any]",
  "?[$ * Any$",
  "?[$ * Any$",
  "10 * $ * Any",
  "?10 * $ * Any",
  "?(10 * $ * Any)",
  "$10 * $ * Any",
  "=10 * $ * Any",
  "$[10 * $ * Any]",
  "xxx[10 * $ * Any]",
  "?$10 * $ * Any]",
  "?=10 * $ * Any]",
  "?[10 * $ * Any$",
  "?[10 * $ * Any$",
  "10 * 10 * $ * Any",
  "?10 * 10 * $ * Any",
  "?(10 * 10 * $ * Any)",
  "$10 * 10 * $ * Any",
  "=10 * 10 * $ * Any",
  "$[10 * 10 * $ * Any]",
  "xxx[10 * 10 * $ * Any]",
  "?$10 * 10 * $ * Any]",
  "?=10 * 10 * $ * Any]",
  "?[10 * 10 * $ * Any$",
  "?[10 * 10 * $ * Any$",
  "10 $ $ * Any",
  "?10 $ $ * Any",
  "?(10 $ $ * Any)",
  "$10 $ $ * Any",
  "=10 $ $ * Any",
  "$[10 $ $ * Any]",
  "xxx[10 $ $ * Any]",
  "?$10 $ $ * Any]",
  "?=10 $ $ * Any]",
  "?[10 $ $ * Any$",
  "?[10 $ $ * Any$",
  "10 = $ * Any",
  "?10 = $ * Any",
  "?(10 = $ * Any)",
  "$10 = $ * Any",
  "=10 = $ * Any",
  "$[10 = $ * Any]",
  "xxx[10 = $ * Any]",
  "?$10 = $ * Any]",
  "?=10 = $ * Any]",
  "?[10 = $ * Any$",
  "?[10 = $ * Any$",
  "10 $ 10 * $ * Any",
  "?10 $ 10 * $ * Any",
  "?(10 $ 10 * $ * Any)",
  "$10 $ 10 * $ * Any",
  "=10 $ 10 * $ * Any",
  "$[10 $ 10 * $ * Any]",
  "xxx[10 $ 10 * $ * Any]",
  "?$10 $ 10 * $ * Any]",
  "?=10 $ 10 * $ * Any]",
  "?[10 $ 10 * $ * Any$",
  "?[10 $ 10 * $ * Any$",
  "10 * 10 $ $ * Any",
  "?10 * 10 $ $ * Any",
  "?(10 * 10 $ $ * Any)",
  "$10 * 10 $ $ * Any",
  "=10 * 10 $ $ * Any",
  "$[10 * 10 $ $ * Any]",
  "xxx[10 * 10 $ $ * Any]",
  "?$10 * 10 $ $ * Any]",
  "?=10 * 10 $ $ * Any]",
  "?[10 * 10 $ $ * Any$",
  "?[10 * 10 $ $ * Any$",
  "10 = 10 * $ * Any",
  "?10 = 10 * $ * Any",
  "?(10 = 10 * $ * Any)",
  "$10 = 10 * $ * Any",
  "=10 = 10 * $ * Any",
  "$[10 = 10 * $ * Any]",
  "xxx[10 = 10 * $ * Any]",
  "?$10 = 10 * $ * Any]",
  "?=10 = 10 * $ * Any]",
  "?[10 = 10 * $ * Any$",
  "?[10 = 10 * $ * Any$",
  "10 * 10 $ $ * Any",
  "?10 * 10 $ $ * Any",
  "?(10 * 10 $ $ * Any)",
  "$10 * 10 $ $ * Any",
  "=10 * 10 $ $ * Any",
  "$[10 * 10 $ $ * Any]",
  "xxx[10 * 10 $ $ * Any]",
  "?$10 * 10 $ $ * Any]",
  "?=10 * 10 $ $ * Any]",
  "?[10 * 10 $ $ * Any$",
  "?[10 * 10 $ $ * Any$",
  "$10 * Any",
  "=10 * Any",
  "$[10 * Any]",
  "xxx[10 * Any]",
  "?$10 * Any]",
  "?=10 * Any]",
  "?[10 * Any$",
  "?[10 * Any$",
  "$Any",
  "=Any",
  "$[Any]",
  "xxx[Any]",
  "?$Any]",
  "?=Any]",
  "?[Any$",
  "?[Any$",
  "10 $ Any",
  "?10 $ Any",
  "?(10 $ Any)",
  "$10 $ Any",
  "=10 $ Any",
  "$[10 $ Any]",
  "xxx[10 $ Any]",
  "?$10 $ Any]",
  "?=10 $ Any]",
  "?[10 $ Any$",
  "?[10 $ Any$",
  "10 = Any",
  "?10 = Any",
  "?(10 = Any)",
  "$10 = Any",
  "=10 = Any",
  "$[10 = Any]",
  "xxx[10 = Any]",
  "?$10 = Any]",
  "?=10 = Any]",
  "?[10 = Any$",
  "?[10 = Any$",
  "$10 * ScalarKind",
  "=10 * ScalarKind",
  "$[10 * ScalarKind]",
  "xxx[10 * ScalarKind]",
  "?$10 * ScalarKind]",
  "?=10 * ScalarKind]",
  "?[10 * ScalarKind$",
  "?[10 * ScalarKind$",
  "$ScalarKind",
  "=ScalarKind",
  "$[ScalarKind]",
  "xxx[ScalarKind]",
  "?$ScalarKind]",
  "?=ScalarKind]",
  "?[ScalarKind$",
  "?[ScalarKind$",
  "10 $ ScalarKind",
  "?10 $ ScalarKind",
  "?(10 $ ScalarKind)",
  "$10 $ ScalarKind",
  "=10 $ ScalarKind",
  "$[10 $ ScalarKind]",
  "xxx[10 $ ScalarKind]",
  "?$10 $ ScalarKind]",
  "?=10 $ ScalarKind]",
  "?[10 $ ScalarKind$",
  "?[10 $ ScalarKind$",
  "10 = ScalarKind",
  "?10 = ScalarKind",
  "?(10 = ScalarKind)",
  "$10 = ScalarKind",
  "=10 = ScalarKind",
  "$[10 = ScalarKind]",
  "xxx[10 = ScalarKind]",
  "?$10 = ScalarKind]",
  "?=10 = ScalarKind]",
  "?[10 = ScalarKind$",
  "?[10 = ScalarKind$",
  "$10 * void",
  "=10 * void",
  "$[10 * void]",
  "xxx[10 * void]",
  "?$10 * void]",
  "?=10 * void]",
  "?[10 * void$",
  "?[10 * void$",
  "$void",
  "=void",
  "$[void]",
  "xxx[void]",
  "?$void]",
  "?=void]",
  "?[void$",
  "?[void$",
  "10 $ void",
  "?10 $ void",
  "?(10 $ void)",
  "$10 $ void",
  "=10 $ void",
  "$[10 $ void]",
  "xxx[10 $ void]",
  "?$10 $ void]",
  "?=10 $ void]",
  "?[10 $ void$",
  "?[10 $ void$",
  "10 = void",
  "?10 = void",
  "?(10 = void)",
  "$10 = void",
  "=10 = void",
  "$[10 = void]",
  "xxx[10 = void]",
  "?$10 = void]",
  "?=10 = void]",
  "?[10 = void$",
  "?[10 = void$",
  "$10 * bool",
  "=10 * bool",
  "$[10 * bool]",
  "xxx[10 * bool]",
  "?$10 * bool]",
  "?=10 * bool]",
  "?[10 * bool$",
  "?[10 * bool$",
  "$bool",
  "^bool",
  "$[bool]",
  "xxx[bool]",
  "?$bool]",
  "?^bool]",
  "?[bool$",
  "?[bool$",
  "10 $ bool",
  "?10 $ bool",
  "?(10 $ bool)",
  "$10 $ bool",
  "=10 $ bool",
  "$[10 $ bool]",
  "xxx[10 $ bool]",
  "?$10 $ bool]",
  "?=10 $ bool]",
  "?[10 $ bool$",
  "?[10 $ bool$",
  "10 = bool",
  "?10 = bool",
  "?(10 = bool)",
  "$10 = bool",
  "=10 = bool",
  "$[10 = bool]",
  "xxx[10 = bool]",
  "?$10 = bool]",
  "?=10 = bool]",
  "?[10 = bool$",
  "?[10 = bool$",
  "$10 * SignedKind",
  "=10 * SignedKind",
  "$[10 * SignedKind]",
  "xxx[10 * SignedKind]",
  "?$10 * SignedKind]",
  "?=10 * SignedKind]",
  "?[10 * SignedKind$",
  "?[10 * SignedKind$",
  "$SignedKind",
  "=SignedKind",
  "$[SignedKind]",
  "xxx[SignedKind]",
  "?$SignedKind]",
  "?=SignedKind]",
  "?[SignedKind$",
  "?[SignedKind$",
  "10 $ SignedKind",
  "?10 $ SignedKind",
  "?(10 $ SignedKind)",
  "$10 $ SignedKind",
  "=10 $ SignedKind",
  "$[10 $ SignedKind]",
  "xxx[10 $ SignedKind]",
  "?$10 $ SignedKind]",
  "?=10 $ SignedKind]",
  "?[10 $ SignedKind$",
  "?[10 $ SignedKind$",
  "10 = SignedKind",
  "?10 = SignedKind",
  "?(10 = SignedKind)",
  "$10 = SignedKind",
  "=10 = SignedKind",
  "$[10 = SignedKind]",
  "xxx[10 = SignedKind]",
  "?$10 = SignedKind]",
  "?=10 = SignedKind]",
  "?[10 = SignedKind$",
  "?[10 = SignedKind$",
  "$10 * int8",
  "=10 * int8",
  "$[10 * int8]",
  "xxx[10 * int8]",
  "?$10 * int8]",
  "?=10 * int8]",
  "?[10 * int8$",
  "?[10 * int8$",
  "$int8",
  "^int8",
  "$[int8]",
  "xxx[int8]",
  "?$int8]",
  "?^int8]",
  "?[int8$",
  "?[int8$",
  "10 $ int8",
  "?10 $ int8",
  "?(10 $ int8)",
  "$10 $ int8",
  "=10 $ int8",
  "$[10 $ int8]",
  "xxx[10 $ int8]",
  "?$10 $ int8]",
  "?=10 $ int8]",
  "?[10 $ int8$",
  "?[10 $ int8$",
  "10 = int8",
  "?10 = int8",
  "?(10 = int8)",
  "$10 = int8",
  "=10 = int8",
  "$[10 = int8]",
  "xxx[10 = int8]",
  "?$10 = int8]",
  "?=10 = int8]",
  "?[10 = int8$",
  "?[10 = int8$",
  "$10 * int16",
  "=10 * int16",
  "$[10 * int16]",
  "xxx[10 * int16]",
  "?$10 * int16]",
  "?=10 * int16]",
  "?[10 * int16$",
  "?[10 * int16$",
  "$int16",
  "^int16",
  "$[int16]",
  "xxx[int16]",
  "?$int16]",
  "?^int16]",
  "?[int16$",
  "?[int16$",
  "10 $ int16",
  "?10 $ int16",
  "?(10 $ int16)",
  "$10 $ int16",
  "=10 $ int16",
  "$[10 $ int16]",
  "xxx[10 $ int16]",
  "?$10 $ int16]",
  "?=10 $ int16]",
  "?[10 $ int16$",
  "?[10 $ int16$",
  "10 = int16",
  "?10 = int16",
  "?(10 = int16)",
  "$10 = int16",
  "=10 = int16",
  "$[10 = int16]",
  "xxx[10 = int16]",
  "?$10 = int16]",
  "?=10 = int16]",
  "?[10 = int16$",
  "?[10 = int16$",
  "$10 * int32",
  "=10 * int32",
  "$[10 * int32]",
  "xxx[10 * int32]",
  "?$10 * int32]",
  "?=10 * int32]",
  "?[10 * int32$",
  "?[10 * int32$",
  "$int32",
  "^int32",
  "$[int32]",
  "xxx[int32]",
  "?$int32]",
  "?^int32]",
  "?[int32$",
  "?[int32$",
  "10 $ int32",
  "?10 $ int32",
  "?(10 $ int32)",
  "$10 $ int32",
  "=10 $ int32",
  "$[10 $ int32]",
  "xxx[10 $ int32]",
  "?$10 $ int32]",
  "?=10 $ int32]",
  "?[10 $ int32$",
  "?[10 $ int32$",
  "10 = int32",
  "?10 = int32",
  "?(10 = int32)",
  "$10 = int32",
  "=10 = int32",
  "$[10 = int32]",
  "xxx[10 = int32]",
  "?$10 = int32]",
  "?=10 = int32]",
  "?[10 = int32$",
  "?[10 = int32$",
  "$10 * int64",
  "=10 * int64",
  "$[10 * int64]",
  "xxx[10 * int64]",
  "?$10 * int64]",
  "?=10 * int64]",
  "?[10 * int64$",
  "?[10 * int64$",
  "$int64",
  "^int64",
  "$[int64]",
  "xxx[int64]",
  "?$int64]",
  "?^int64]",
  "?[int64$",
  "?[int64$",
  "10 $ int64",
  "?10 $ int64",
  "?(10 $ int64)",
  "$10 $ int64",
  "=10 $ int64",
  "$[10 $ int64]",
  "xxx[10 $ int64]",
  "?$10 $ int64]",
  "?=10 $ int64]",
  "?[10 $ int64$",
  "?[10 $ int64$",
  "10 = int64",
  "?10 = int64",
  "?(10 = int64)",
  "$10 = int64",
  "=10 = int64",
  "$[10 = int64]",
  "xxx[10 = int64]",
  "?$10 = int64]",
  "?=10 = int64]",
  "?[10 = int64$",
  "?[10 = int64$",
  "$10 * UnsignedKind",
  "=10 * UnsignedKind",
  "$[10 * UnsignedKind]",
  "xxx[10 * UnsignedKind]",
  "?$10 * UnsignedKind]",
  "?=10 * UnsignedKind]",
  "?[10 * UnsignedKind$",
  "?[10 * UnsignedKind$",
  "$UnsignedKind",
  "=UnsignedKind",
  "$[UnsignedKind]",
  "xxx[UnsignedKind]",
  "?$UnsignedKind]",
  "?=UnsignedKind]",
  "?[UnsignedKind$",
  "?[UnsignedKind$",
  "10 $ UnsignedKind",
  "?10 $ UnsignedKind",
  "?(10 $ UnsignedKind)",
  "$10 $ UnsignedKind",
  "=10 $ UnsignedKind",
  "$[10 $ UnsignedKind]",
  "xxx[10 $ UnsignedKind]",
  "?$10 $ UnsignedKind]",
  "?=10 $ UnsignedKind]",
  "?[10 $ UnsignedKind$",
  "?[10 $ UnsignedKind$",
  "10 = UnsignedKind",
  "?10 = UnsignedKind",
  "?(10 = UnsignedKind)",
  "$10 = UnsignedKind",
  "=10 = UnsignedKind",
  "$[10 = UnsignedKind]",
  "xxx[10 = UnsignedKind]",
  "?$10 = UnsignedKind]",
  "?=10 = UnsignedKind]",
  "?[10 = UnsignedKind$",
  "?[10 = UnsignedKind$",
  "$10 * uint8",
  "=10 * uint8",
  "$[10 * uint8]",
  "xxx[10 * uint8]",
  "?$10 * uint8]",
  "?=10 * uint8]",
  "?[10 * uint8$",
  "?[10 * uint8$",
  "$uint8",
  "^uint8",
  "$[uint8]",
  "xxx[uint8]",
  "?$uint8]",
  "?^uint8]",
  "?[uint8$",
  "?[uint8$",
  "10 $ uint8",
  "?10 $ uint8",
  "?(10 $ uint8)",
  "$10 $ uint8",
  "=10 $ uint8",
  "$[10 $ uint8]",
  "xxx[10 $ uint8]",
  "?$10 $ uint8]",
  "?=10 $ uint8]",
  "?[10 $ uint8$",
  "?[10 $ uint8$",
  "10 = uint8",
  "?10 = uint8",
  "?(10 = uint8)",
  "$10 = uint8",
  "=10 = uint8",
  "$[10 = uint8]",
  "xxx[10 = uint8]",
  "?$10 = uint8]",
  "?=10 = uint8]",
  "?[10 = uint8$",
  "?[10 = uint8$",
  "$10 * uint16",
  "=10 * uint16",
  "$[10 * uint16]",
  "xxx[10 * uint16]",
  "?$10 * uint16]",
  "?=10 * uint16]",
  "?[10 * uint16$",
  "?[10 * uint16$",
  "$uint16",
  "^uint16",
  "$[uint16]",
  "xxx[uint16]",
  "?$uint16]",
  "?^uint16]",
  "?[uint16$",
  "?[uint16$",
  "10 $ uint16",
  "?10 $ uint16",
  "?(10 $ uint16)",
  "$10 $ uint16",
  "=10 $ uint16",
  "$[10 $ uint16]",
  "xxx[10 $ uint16]",
  "?$10 $ uint16]",
  "?=10 $ uint16]",
  "?[10 $ uint16$",
  "?[10 $ uint16$",
  "10 = uint16",
  "?10 = uint16",
  "?(10 = uint16)",
  "$10 = uint16",
  "=10 = uint16",
  "$[10 = uint16]",
  "xxx[10 = uint16]",
  "?$10 = uint16]",
  "?=10 = uint16]",
  "?[10 = uint16$",
  "?[10 = uint16$",
  "$10 * uint32",
  "=10 * uint32",
  "$[10 * uint32]",
  "xxx[10 * uint32]",
  "?$10 * uint32]",
  "?=10 * uint32]",
  "?[10 * uint32$",
  "?[10 * uint32$",
  "$uint32",
  "^uint32",
  "$[uint32]",
  "xxx[uint32]",
  "?$uint32]",
  "?^uint32]",
  "?[uint32$",
  "?[uint32$",
  "10 $ uint32",
  "?10 $ uint32",
  "?(10 $ uint32)",
  "$10 $ uint32",
  "=10 $ uint32",
  "$[10 $ uint32]",
  "xxx[10 $ uint32]",
  "?$10 $ uint32]",
  "?=10 $ uint32]",
  "?[10 $ uint32$",
  "?[10 $ uint32$",
  "10 = uint32",
  "?10 = uint32",
  "?(10 = uint32)",
  "$10 = uint32",
  "=10 = uint32",
  "$[10 = uint32]",
  "xxx[10 = uint32]",
  "?$10 = uint32]",
  "?=10 = uint32]",
  "?[10 = uint32$",
  "?[10 = uint32$",
  "$10 * uint64",
  "=10 * uint64",
  "$[10 * uint64]",
  "xxx[10 * uint64]",
  "?$10 * uint64]",
  "?=10 * uint64]",
  "?[10 * uint64$",
  "?[10 * uint64$",
  "$uint64",
  "^uint64",
  "$[uint64]",
  "xxx[uint64]",
  "?$uint64]",
  "?^uint64]",
  "?[uint64$",
  "?[uint64$",
  "10 $ uint64",
  "?10 $ uint64",
  "?(10 $ uint64)",
  "$10 $ uint64",
  "=10 $ uint64",
  "$[10 $ uint64]",
  "xxx[10 $ uint64]",
  "?$10 $ uint64]",
  "?=10 $ uint64]",
  "?[10 $ uint64$",
  "?[10 $ uint64$",
  "10 = uint64",
  "?10 = uint64",
  "?(10 = uint64)",
  "$10 = uint64",
  "=10 = uint64",
  "$[10 = uint64]",
  "xxx[10 = uint64]",
  "?$10 = uint64]",
  "?=10 = uint64]",
  "?[10 = uint64$",
  "?[10 = uint64$",
  "$10 * FloatKind",
  "=10 * FloatKind",
  "$[10 * FloatKind]",
  "xxx[10 * FloatKind]",
  "?$10 * FloatKind]",
  "?=10 * FloatKind]",
  "?[10 * FloatKind$",
  "?[10 * FloatKind$",
  "$FloatKind",
  "=FloatKind",
  "$[FloatKind]",
  "xxx[FloatKind]",
  "?$FloatKind]",
  "?=FloatKind]",
  "?[FloatKind$",
  "?[FloatKind$",
  "10 $ FloatKind",
  "?10 $ FloatKind",
  "?(10 $ FloatKind)",
  "$10 $ FloatKind",
  "=10 $ FloatKind",
  "$[10 $ FloatKind]",
  "xxx[10 $ FloatKind]",
  "?$10 $ FloatKind]",
  "?=10 $ FloatKind]",
  "?[10 $ FloatKind$",
  "?[10 $ FloatKind$",
  "10 = FloatKind",
  "?10 = FloatKind",
  "?(10 = FloatKind)",
  "$10 = FloatKind",
  "=10 = FloatKind",
  "$[10 = FloatKind]",
  "xxx[10 = FloatKind]",
  "?$10 = FloatKind]",
  "?=10 = FloatKind]",
  "?[10 = FloatKind$",
  "?[10 = FloatKind$",
  "$10 * float32",
  "=10 * float32",
  "$[10 * float32]",
  "xxx[10 * float32]",
  "?$10 * float32]",
  "?=10 * float32]",
  "?[10 * float32$",
  "?[10 * float32$",
  "$float32",
  "^float32",
  "$[float32]",
  "xxx[float32]",
  "?$float32]",
  "?^float32]",
  "?[float32$",
  "?[float32$",
  "10 $ float32",
  "?10 $ float32",
  "?(10 $ float32)",
  "$10 $ float32",
  "=10 $ float32",
  "$[10 $ float32]",
  "xxx[10 $ float32]",
  "?$10 $ float32]",
  "?=10 $ float32]",
  "?[10 $ float32$",
  "?[10 $ float32$",
  "10 = float32",
  "?10 = float32",
  "?(10 = float32)",
  "$10 = float32",
  "=10 = float32",
  "$[10 = float32]",
  "xxx[10 = float32]",
  "?$10 = float32]",
  "?=10 = float32]",
  "?[10 = float32$",
  "?[10 = float32$",
  "$10 * float64",
  "=10 * float64",
  "$[10 * float64]",
  "xxx[10 * float64]",
  "?$10 * float64]",
  "?=10 * float64]",
  "?[10 * float64$",
  "?[10 * float64$",
  "$float64",
  "^float64",
  "$[float64]",
  "xxx[float64]",
  "?$float64]",
  "?^float64]",
  "?[float64$",
  "?[float64$",
  "10 $ float64",
  "?10 $ float64",
  "?(10 $ float64)",
  "$10 $ float64",
  "=10 $ float64",
  "$[10 $ float64]",
  "xxx[10 $ float64]",
  "?$10 $ float64]",
  "?=10 $ float64]",
  "?[10 $ float64$",
  "?[10 $ float64$",
  "10 = float64",
  "?10 = float64",
  "?(10 = float64)",
  "$10 = float64",
  "=10 = float64",
  "$[10 = float64]",
  "xxx[10 = float64]",
  "?$10 = float64]",
  "?=10 = float64]",
  "?[10 = float64$",
  "?[10 = float64$",
  "$10 * ComplexKind",
  "=10 * ComplexKind",
  "$[10 * ComplexKind]",
  "xxx[10 * ComplexKind]",
  "?$10 * ComplexKind]",
  "?=10 * ComplexKind]",
  "?[10 * ComplexKind$",
  "?[10 * ComplexKind$",
  "$ComplexKind",
  "=ComplexKind",
  "$[ComplexKind]",
  "xxx[ComplexKind]",
  "?$ComplexKind]",
  "?=ComplexKind]",
  "?[ComplexKind$",
  "?[ComplexKind$",
  "10 $ ComplexKind",
  "?10 $ ComplexKind",
  "?(10 $ ComplexKind)",
  "$10 $ ComplexKind",
  "=10 $ ComplexKind",
  "$[10 $ ComplexKind]",
  "xxx[10 $ ComplexKind]",
  "?$10 $ ComplexKind]",
  "?=10 $ ComplexKind]",
  "?[10 $ ComplexKind$",
  "?[10 $ ComplexKind$",
  "10 = ComplexKind",
  "?10 = ComplexKind",
  "?(10 = ComplexKind)",
  "$10 = ComplexKind",
  "=10 = ComplexKind",
  "$[10 = ComplexKind]",
  "xxx[10 = ComplexKind]",
  "?$10 = ComplexKind]",
  "?=10 = ComplexKind]",
  "?[10 = ComplexKind$",
  "?[10 = ComplexKind$",
  "$10 * complex64",
  "=10 * complex64",
  "$[10 * complex64]",
  "xxx[10 * complex64]",
  "?$10 * complex64]",
  "?=10 * complex64]",
  "?[10 * complex64$",
  "?[10 * complex64$",
  "$complex64",
  "^complex64",
  "$[complex64]",
  "xxx[complex64]",
  "?$complex64]",
  "?^complex64]",
  "?[complex64$",
  "?[complex64$",
  "10 $ complex64",
  "?10 $ complex64",
  "?(10 $ complex64)",
  "$10 $ complex64",
  "=10 $ complex64",
  "$[10 $ complex64]",
  "xxx[10 $ complex64]",
  "?$10 $ complex64]",
  "?=10 $ complex64]",
  "?[10 $ complex64$",
  "?[10 $ complex64$",
  "10 = complex64",
  "?10 = complex64",
  "?(10 = complex64)",
  "$10 = complex64",
  "=10 = complex64",
  "$[10 = complex64]",
  "xxx[10 = complex64]",
  "?$10 = complex64]",
  "?=10 = complex64]",
  "?[10 = complex64$",
  "?[10 = complex64$",
  "$10 * complex128",
  "=10 * complex128",
  "$[10 * complex128]",
  "xxx[10 * complex128]",
  "?$10 * complex128]",
  "?=10 * complex128]",
  "?[10 * complex128$",
  "?[10 * complex128$",
  "$complex128",
  "^complex128",
  "$[complex128]",
  "xxx[complex128]",
  "?$complex128]",
  "?^complex128]",
  "?[complex128$",
  "?[complex128$",
  "10 $ complex128",
  "?10 $ complex128",
  "?(10 $ complex128)",
  "$10 $ complex128",
  "=10 $ complex128",
  "$[10 $ complex128]",
  "xxx[10 $ complex128]",
  "?$10 $ complex128]",
  "?=10 $ complex128]",
  "?[10 $ complex128$",
  "?[10 $ complex128$",
  "10 = complex128",
  "?10 = complex128",
  "?(10 = complex128)",
  "$10 = complex128",
  "=10 = complex128",
  "$[10 = complex128]",
  "xxx[10 = complex128]",
  "?$10 = complex128]",
  "?=10 = complex128]",
  "?[10 = complex128$",
  "?[10 = complex128$",
  "$10 * int",
  "=10 * int",
  "$[10 * int]",
  "xxx[10 * int]",
  "?$10 * int]",
  "?=10 * int]",
  "?[10 * int$",
  "?[10 * int$",
  "$int",
  "^int",
  "$[int]",
  "xxx[int]",
  "?$int]",
  "?^int]",
  "?[int$",
  "?[int$",
  "10 $ int",
  "?10 $ int",
  "?(10 $ int)",
  "$10 $ int",
  "=10 $ int",
  "$[10 $ int]",
  "xxx[10 $ int]",
  "?$10 $ int]",
  "?=10 $ int]",
  "?[10 $ int$",
  "?[10 $ int$",
  "10 = int",
  "?10 = int",
  "?(10 = int)",
  "$10 = int",
  "=10 = int",
  "$[10 = int]",
  "xxx[10 = int]",
  "?$10 = int]",
  "?=10 = int]",
  "?[10 = int$",
  "?[10 = int$",
  "$10 * float64",
  "=10 * float64",
  "$[10 * float64]",
  "xxx[10 * float64]",
  "?$10 * float64]",
  "?=10 * float64]",
  "?[10 * float64$",
  "?[10 * float64$",
  "$float64",
  "^float64",
  "$[float64]",
  "xxx[float64]",
  "?$float64]",
  "?^float64]",
  "?[float64$",
  "?[float64$",
  "10 $ float64",
  "?10 $ float64",
  "?(10 $ float64)",
  "$10 $ float64",
  "=10 $ float64",
  "$[10 $ float64]",
  "xxx[10 $ float64]",
  "?$10 $ float64]",
  "?=10 $ float64]",
  "?[10 $ float64$",
  "?[10 $ float64$",
  "10 = float64",
  "?10 = float64",
  "?(10 = float64)",
  "$10 = float64",
  "=10 = float64",
  "$[10 = float64]",
  "xxx[10 = float64]",
  "?$10 = float64]",
  "?=10 = float64]",
  "?[10 = float64$",
  "?[10 = float64$",
  "$10 * complex",
  "=10 * complex",
  "$[10 * complex]",
  "xxx[10 * complex]",
  "?$10 * complex]",
  "?=10 * complex]",
  "?[10 * complex$",
  "?[10 * complex$",
  "$complex",
  "^complex",
  "$[complex]",
  "xxx[complex]",
  "?$complex]",
  "?^complex]",
  "?[complex$",
  "?[complex$",
  "10 $ complex",
  "?10 $ complex",
  "?(10 $ complex)",
  "$10 $ complex",
  "=10 $ complex",
  "$[10 $ complex]",
  "xxx[10 $ complex]",
  "?$10 $ complex]",
  "?=10 $ complex]",
  "?[10 $ complex$",
  "?[10 $ complex$",
  "10 = complex",
  "?10 = complex",
  "?(10 = complex)",
  "$10 = complex",
  "=10 = complex",
  "$[10 = complex]",
  "xxx[10 = complex]",
  "?$10 = complex]",
  "?=10 = complex]",
  "?[10 = complex$",
  "?[10 = complex$",
  "$10 * intptr",
  "=10 * intptr",
  "$[10 * intptr]",
  "xxx[10 * intptr]",
  "?$10 * intptr]",
  "?=10 * intptr]",
  "?[10 * intptr$",
  "?[10 * intptr$",
  "$intptr",
  "^intptr",
  "$[intptr]",
  "xxx[intptr]",
  "?$intptr]",
  "?^intptr]",
  "?[intptr$",
  "?[intptr$",
  "10 $ intptr",
  "?10 $ intptr",
  "?(10 $ intptr)",
  "$10 $ intptr",
  "=10 $ intptr",
  "$[10 $ intptr]",
  "xxx[10 $ intptr]",
  "?$10 $ intptr]",
  "?=10 $ intptr]",
  "?[10 $ intptr$",
  "?[10 $ intptr$",
  "10 = intptr",
  "?10 = intptr",
  "?(10 = intptr)",
  "$10 = intptr",
  "=10 = intptr",
  "$[10 = intptr]",
  "xxx[10 = intptr]",
  "?$10 = intptr]",
  "?=10 = intptr]",
  "?[10 = intptr$",
  "?[10 = intptr$",
  "$10 * uintptr",
  "=10 * uintptr",
  "$[10 * uintptr]",
  "xxx[10 * uintptr]",
  "?$10 * uintptr]",
  "?=10 * uintptr]",
  "?[10 * uintptr$",
  "?[10 * uintptr$",
  "$uintptr",
  "^uintptr",
  "$[uintptr]",
  "xxx[uintptr]",
  "?$uintptr]",
  "?^uintptr]",
  "?[uintptr$",
  "?[uintptr$",
  "10 $ uintptr",
  "?10 $ uintptr",
  "?(10 $ uintptr)",
  "$10 $ uintptr",
  "=10 $ uintptr",
  "$[10 $ uintptr]",
  "xxx[10 $ uintptr]",
  "?$10 $ uintptr]",
  "?=10 $ uintptr]",
  "?[10 $ uintptr$",
  "?[10 $ uintptr$",
  "10 = uintptr",
  "?10 = uintptr",
  "?(10 = uintptr)",
  "$10 = uintptr",
  "=10 = uintptr",
  "$[10 = uintptr]",
  "xxx[10 = uintptr]",
  "?$10 = uintptr]",
  "?=10 = uintptr]",
  "?[10 = uintptr$",
  "?[10 = uintptr$",
  "$10 * char",
  "=10 * char",
  "$[10 * char]",
  "xxx[10 * char]",
  "?$10 * char]",
  "?=10 * char]",
  "?[10 * char$",
  "?[10 * char$",
  "$char",
  "=char",
  "$[char]",
  "xxx[char]",
  "?$char]",
  "?=char]",
  "?[char$",
  "?[char$",
  "10 $ char",
  "?10 $ char",
  "?(10 $ char)",
  "$10 $ char",
  "=10 $ char",
  "$[10 $ char]",
  "xxx[10 $ char]",
  "?$10 $ char]",
  "?=10 $ char]",
  "?[10 $ char$",
  "?[10 $ char$",
  "10 = char",
  "?10 = char",
  "?(10 = char)",
  "$10 = char",
  "=10 = char",
  "$[10 = char]",
  "xxx[10 = char]",
  "?$10 = char]",
  "?=10 = char]",
  "?[10 = char$",
  "?[10 = char$",
  "$10 * char('A')",
  "=10 * char('A')",
  "$[10 * char('A'])",
  "xxx[10 * char('A'])",
  "?$10 * char('A'])",
  "?=10 * char('A'])",
  "?[10 * char('A')$",
  "?[10 * char('A')$",
  "$char('A')",
  "=char('A')",
  "$[char('A'])",
  "xxx[char('A'])",
  "?$char('A'])",
  "?=char('A'])",
  "?[char('A')$",
  "?[char('A')$",
  "10 $ char('A')",
  "?10 $ char('A')",
  "?(10 $ char('A'))",
  "$10 $ char('A')",
  "=10 $ char('A')",
  "$[10 $ char('A'])",
  "xxx[10 $ char('A'])",
  "?$10 $ char('A'])",
  "?=10 $ char('A'])",
  "?[10 $ char('A')$",
  "?[10 $ char('A')$",
  "10 = char('A')",
  "?10 = char('A')",
  "?(10 = char('A'))",
  "$10 = char('A')",
  "=10 = char('A')",
  "$[10 = char('A'])",
  "xxx[10 = char('A'])",
  "?$10 = char('A'])",
  "?=10 = char('A'])",
  "?[10 = char('A')$",
  "?[10 = char('A')$",
  "$10 * char('ascii')",
  "=10 * char('ascii')",
  "$[10 * char('ascii'])",
  "xxx[10 * char('ascii'])",
  "?$10 * char('ascii'])",
  "?=10 * char('ascii'])",
  "?[10 * char('ascii')$",
  "?[10 * char('ascii')$",
  "$char('ascii')",
  "=char('ascii')",
  "$[char('ascii'])",
  "xxx[char('ascii'])",
  "?$char('ascii'])",
  "?=char('ascii'])",
  "?[char('ascii')$",
  "?[char('ascii')$",
  "10 $ char('ascii')",
  "?10 $ char('ascii')",
  "?(10 $ char('ascii'))",
  "$10 $ char('ascii')",
  "=10 $ char('ascii')",
  "$[10 $ char('ascii'])",
  "xxx[10 $ char('ascii'])",
  "?$10 $ char('ascii'])",
  "?=10 $ char('ascii'])",
  "?[10 $ char('ascii')$",
  "?[10 $ char('ascii')$",
  "10 = char('ascii')",
  "?10 = char('ascii')",
  "?(10 = char('ascii'))",
  "$10 = char('ascii')",
  "=10 = char('ascii')",
  "$[10 = char('ascii'])",
  "xxx[10 = char('ascii'])",
  "?$10 = char('ascii'])",
  "?=10 = char('ascii'])",
  "?[10 = char('ascii')$",
  "?[10 = char('ascii')$",
  "$10 * char('us-ascii')",
  "=10 * char('us-ascii')",
  "$[10 * char('us-ascii'])",
  "xxx[10 * char('us-ascii'])",
  "?$10 * char('us-ascii'])",
  "?=10 * char('us-ascii'])",
  "?[10 * char('us-ascii')$",
  "?[10 * char('us-ascii')$",
  "$char('us-ascii')",
  "=char('us-ascii')",
  "$[char('us-ascii'])",
  "xxx[char('us-ascii'])",
  "?$char('us-ascii'])",
  "?=char('us-ascii'])",
  "?[char('us-ascii')$",
  "?[char('us-ascii')$",
  "10 $ char('us-ascii')",
  "?10 $ char('us-ascii')",
  "?(10 $ char('us-ascii'))",
  "$10 $ char('us-ascii')",
  "=10 $ char('us-ascii')",
  "$[10 $ char('us-ascii'])",
  "xxx[10 $ char('us-ascii'])",
  "?$10 $ char('us-ascii'])",
  "?=10 $ char('us-ascii'])",
  "?[10 $ char('us-ascii')$",
  "?[10 $ char('us-ascii')$",
  "10 = char('us-ascii')",
  "?10 = char('us-ascii')",
  "?(10 = char('us-ascii'))",
  "$10 = char('us-ascii')",
  "=10 = char('us-ascii')",
  "$[10 = char('us-ascii'])",
  "xxx[10 = char('us-ascii'])",
  "?$10 = char('us-ascii'])",
  "?=10 = char('us-ascii'])",
  "?[10 = char('us-ascii')$",
  "?[10 = char('us-ascii')$",
  "$10 * char('U8')",
  "=10 * char('U8')",
  "$[10 * char('U8'])",
  "xxx[10 * char('U8'])",
  "?$10 * char('U8'])",
  "?=10 * char('U8'])",
  "?[10 * char('U8')$",
  "?[10 * char('U8')$",
  "$char('U8')",
  "=char('U8')",
  "$[char('U8'])",
  "xxx[char('U8'])",
  "?$char('U8'])",
  "?=char('U8'])",
  "?[char('U8')$",
  "?[char('U8')$",
  "10 $ char('U8')",
  "?10 $ char('U8')",
  "?(10 $ char('U8'))",
  "$10 $ char('U8')",
  "=10 $ char('U8')",
  "$[10 $ char('U8'])",
  "xxx[10 $ char('U8'])",
  "?$10 $ char('U8'])",
  "?=10 $ char('U8'])",
  "?[10 $ char('U8')$",
  "?[10 $ char('U8')$",
  "10 = char('U8')",
  "?10 = char('U8')",
  "?(10 = char('U8'))",
  "$10 = char('U8')",
  "=10 = char('U8')",
  "$[10 = char('U8'])",
  "xxx[10 = char('U8'])",
  "?$10 = char('U8'])",
  "?=10 = char('U8'])",
  "?[10 = char('U8')$",
  "?[10 = char('U8')$",
  "$10 * char('utf8')",
  "=10 * char('utf8')",
  "$[10 * char('utf8'])",
  "xxx[10 * char('utf8'])",
  "?$10 * char('utf8'])",
  "?=10 * char('utf8'])",
  "?[10 * char('utf8')$",
  "?[10 * char('utf8')$",
  "$char('utf8')",
  "=char('utf8')",
  "$[char('utf8'])",
  "xxx[char('utf8'])",
  "?$char('utf8'])",
  "?=char('utf8'])",
  "?[char('utf8')$",
  "?[char('utf8')$",
  "10 $ char('utf8')",
  "?10 $ char('utf8')",
  "?(10 $ char('utf8'))",
  "$10 $ char('utf8')",
  "=10 $ char('utf8')",
  "$[10 $ char('utf8'])",
  "xxx[10 $ char('utf8'])",
  "?$10 $ char('utf8'])",
  "?=10 $ char('utf8'])",
  "?[10 $ char('utf8')$",
  "?[10 $ char('utf8')$",
  "10 = char('utf8')",
  "?10 = char('utf8')",
  "?(10 = char('utf8'))",
  "$10 = char('utf8')",
  "=10 = char('utf8')",
  "$[10 = char('utf8'])",
  "xxx[10 = char('utf8'])",
  "?$10 = char('utf8'])",
  "?=10 = char('utf8'])",
  "?[10 = char('utf8')$",
  "?[10 = char('utf8')$",
  "$10 * char('utf-8')",
  "=10 * char('utf-8')",
  "$[10 * char('utf-8'])",
  "xxx[10 * char('utf-8'])",
  "?$10 * char('utf-8'])",
  "?=10 * char('utf-8'])",
  "?[10 * char('utf-8')$",
  "?[10 * char('utf-8')$",
  "$char('utf-8')",
  "=char('utf-8')",
  "$[char('utf-8'])",
  "xxx[char('utf-8'])",
  "?$char('utf-8'])",
  "?=char('utf-8'])",
  "?[char('utf-8')$",
  "?[char('utf-8')$",
  "10 $ char('utf-8')",
  "?10 $ char('utf-8')",
  "?(10 $ char('utf-8'))",
  "$10 $ char('utf-8')",
  "=10 $ char('utf-8')",
  "$[10 $ char('utf-8'])",
  "xxx[10 $ char('utf-8'])",
  "?$10 $ char('utf-8'])",
  "?=10 $ char('utf-8'])",
  "?[10 $ char('utf-8')$",
  "?[10 $ char('utf-8')$",
  "10 = char('utf-8')",
  "?10 = char('utf-8')",
  "?(10 = char('utf-8'))",
  "$10 = char('utf-8')",
  "=10 = char('utf-8')",
  "$[10 = char('utf-8'])",
  "xxx[10 = char('utf-8'])",
  "?$10 = char('utf-8'])",
  "?=10 = char('utf-8'])",
  "?[10 = char('utf-8')$",
  "?[10 = char('utf-8')$",
  "$10 * char('U16')",
  "=10 * char('U16')",
  "$[10 * char('U16'])",
  "xxx[10 * char('U16'])",
  "?$10 * char('U16'])",
  "?=10 * char('U16'])",
  "?[10 * char('U16')$",
  "?[10 * char('U16')$",
  "$char('U16')",
  "=char('U16')",
  "$[char('U16'])",
  "xxx[char('U16'])",
  "?$char('U16'])",
  "?=char('U16'])",
  "?[char('U16')$",
  "?[char('U16')$",
  "10 $ char('U16')",
  "?10 $ char('U16')",
  "?(10 $ char('U16'))",
  "$10 $ char('U16')",
  "=10 $ char('U16')",
  "$[10 $ char('U16'])",
  "xxx[10 $ char('U16'])",
  "?$10 $ char('U16'])",
  "?=10 $ char('U16'])",
  "?[10 $ char('U16')$",
  "?[10 $ char('U16')$",
  "10 = char('U16')",
  "?10 = char('U16')",
  "?(10 = char('U16'))",
  "$10 = char('U16')",
  "=10 = char('U16')",
  "$[10 = char('U16'])",
  "xxx[10 = char('U16'])",
  "?$10 = char('U16'])",
  "?=10 = char('U16'])",
  "?[10 = char('U16')$",
  "?[10 = char('U16')$",
  "$10 * char('utf16')",
  "=10 * char('utf16')",
  "$[10 * char('utf16'])",
  "xxx[10 * char('utf16'])",
  "?$10 * char('utf16'])",
  "?=10 * char('utf16'])",
  "?[10 * char('utf16')$",
  "?[10 * char('utf16')$",
  "$char('utf16')",
  "=char('utf16')",
  "$[char('utf16'])",
  "xxx[char('utf16'])",
  "?$char('utf16'])",
  "?=char('utf16'])",
  "?[char('utf16')$",
  "?[char('utf16')$",
  "10 $ char('utf16')",
  "?10 $ char('utf16')",
  "?(10 $ char('utf16'))",
  "$10 $ char('utf16')",
  "=10 $ char('utf16')",
  "$[10 $ char('utf16'])",
  "xxx[10 $ char('utf16'])",
  "?$10 $ char('utf16'])",
  "?=10 $ char('utf16'])",
  "?[10 $ char('utf16')$",
  "?[10 $ char('utf16')$",
  "10 = char('utf16')",
  "?10 = char('utf16')",
  "?(10 = char('utf16'))",
  "$10 = char('utf16')",
  "=10 = char('utf16')",
  "$[10 = char('utf16'])",
  "xxx[10 = char('utf16'])",
  "?$10 = char('utf16'])",
  "?=10 = char('utf16'])",
  "?[10 = char('utf16')$",
  "?[10 = char('utf16')$",
  "$10 * char('utf-16')",
  "=10 * char('utf-16')",
  "$[10 * char('utf-16'])",
  "xxx[10 * char('utf-16'])",
  "?$10 * char('utf-16'])",
  "?=10 * char('utf-16'])",
  "?[10 * char('utf-16')$",
  "?[10 * char('utf-16')$",
  "$char('utf-16')",
  "=char('utf-16')",
  "$[char('utf-16'])",
  "xxx[char('utf-16'])",
  "?$char('utf-16'])",
  "?=char('utf-16'])",
  "?[char('utf-16')$",
  "?[char('utf-16')$",
  "10 $ char('utf-16')",
  "?10 $ char('utf-16')",
  "?(10 $ char('utf-16'))",
  "$10 $ char('utf-16')",
  "=10 $ char('utf-16')",
  "$[10 $ char('utf-16'])",
  "xxx[10 $ char('utf-16'])",
  "?$10 $ char('utf-16'])",
  "?=10 $ char('utf-16'])",
  "?[10 $ char('utf-16')$",
  "?[10 $ char('utf-16')$",
  "10 = char('utf-16')",
  "?10 = char('utf-16')",
  "?(10 = char('utf-16'))",
  "$10 = char('utf-16')",
  "=10 = char('utf-16')",
  "$[10 = char('utf-16'])",
  "xxx[10 = char('utf-16'])",
  "?$10 = char('utf-16'])",
  "?=10 = char('utf-16'])",
  "?[10 = char('utf-16')$",
  "?[10 = char('utf-16')$",
  "$10 * char('U32')",
  "=10 * char('U32')",
  "$[10 * char('U32'])",
  "xxx[10 * char('U32'])",
  "?$10 * char('U32'])",
  "?=10 * char('U32'])",
  "?[10 * char('U32')$",
  "?[10 * char('U32')$",
  "$char('U32')",
  "=char('U32')",
  "$[char('U32'])",
  "xxx[char('U32'])",
  "?$char('U32'])",
  "?=char('U32'])",
  "?[char('U32')$",
  "?[char('U32')$",
  "10 $ char('U32')",
  "?10 $ char('U32')",
  "?(10 $ char('U32'))",
  "$10 $ char('U32')",
  "=10 $ char('U32')",
  "$[10 $ char('U32'])",
  "xxx[10 $ char('U32'])",
  "?$10 $ char('U32'])",
  "?=10 $ char('U32'])",
  "?[10 $ char('U32')$",
  "?[10 $ char('U32')$",
  "10 = char('U32')",
  "?10 = char('U32')",
  "?(10 = char('U32'))",
  "$10 = char('U32')",
  "=10 = char('U32')",
  "$[10 = char('U32'])",
  "xxx[10 = char('U32'])",
  "?$10 = char('U32'])",
  "?=10 = char('U32'])",
  "?[10 = char('U32')$",
  "?[10 = char('U32')$",
  "$10 * char('utf32')",
  "=10 * char('utf32')",
  "$[10 * char('utf32'])",
  "xxx[10 * char('utf32'])",
  "?$10 * char('utf32'])",
  "?=10 * char('utf32'])",
  "?[10 * char('utf32')$",
  "?[10 * char('utf32')$",
  "$char('utf32')",
  "=char('utf32')",
  "$[char('utf32'])",
  "xxx[char('utf32'])",
  "?$char('utf32'])",
  "?=char('utf32'])",
  "?[char('utf32')$",
  "?[char('utf32')$",
  "10 $ char('utf32')",
  "?10 $ char('utf32')",
  "?(10 $ char('utf32'))",
  "$10 $ char('utf32')",
  "=10 $ char('utf32')",
  "$[10 $ char('utf32'])",
  "xxx[10 $ char('utf32'])",
  "?$10 $ char('utf32'])",
  "?=10 $ char('utf32'])",
  "?[10 $ char('utf32')$",
  "?[10 $ char('utf32')$",
  "10 = char('utf32')",
  "?10 = char('utf32')",
  "?(10 = char('utf32'))",
  "$10 = char('utf32')",
  "=10 = char('utf32')",
  "$[10 = char('utf32'])",
  "xxx[10 = char('utf32'])",
  "?$10 = char('utf32'])",
  "?=10 = char('utf32'])",
  "?[10 = char('utf32')$",
  "?[10 = char('utf32')$",
  "$10 * char('utf-32')",
  "=10 * char('utf-32')",
  "$[10 * char('utf-32'])",
  "xxx[10 * char('utf-32'])",
  "?$10 * char('utf-32'])",
  "?=10 * char('utf-32'])",
  "?[10 * char('utf-32')$",
  "?[10 * char('utf-32')$",
  "$char('utf-32')",
  "=char('utf-32')",
  "$[char('utf-32'])",
  "xxx[char('utf-32'])",
  "?$char('utf-32'])",
  "?=char('utf-32'])",
  "?[char('utf-32')$",
  "?[char('utf-32')$",
  "10 $ char('utf-32')",
  "?10 $ char('utf-32')",
  "?(10 $ char('utf-32'))",
  "$10 $ char('utf-32')",
  "=10 $ char('utf-32')",
  "$[10 $ char('utf-32'])",
  "xxx[10 $ char('utf-32'])",
  "?$10 $ char('utf-32'])",
  "?=10 $ char('utf-32'])",
  "?[10 $ char('utf-32')$",
  "?[10 $ char('utf-32')$",
  "10 = char('utf-32')",
  "?10 = char('utf-32')",
  "?(10 = char('utf-32'))",
  "$10 = char('utf-32')",
  "=10 = char('utf-32')",
  "$[10 = char('utf-32'])",
  "xxx[10 = char('utf-32'])",
  "?$10 = char('utf-32'])",
  "?=10 = char('utf-32'])",
  "?[10 = char('utf-32')$",
  "?[10 = char('utf-32')$",
  "$10 * char('ucs2')",
  "=10 * char('ucs2')",
  "$[10 * char('ucs2'])",
  "xxx[10 * char('ucs2'])",
  "?$10 * char('ucs2'])",
  "?=10 * char('ucs2'])",
  "?[10 * char('ucs2')$",
  "?[10 * char('ucs2')$",
  "$char('ucs2')",
  "=char('ucs2')",
  "$[char('ucs2'])",
  "xxx[char('ucs2'])",
  "?$char('ucs2'])",
  "?=char('ucs2'])",
  "?[char('ucs2')$",
  "?[char('ucs2')$",
  "10 $ char('ucs2')",
  "?10 $ char('ucs2')",
  "?(10 $ char('ucs2'))",
  "$10 $ char('ucs2')",
  "=10 $ char('ucs2')",
  "$[10 $ char('ucs2'])",
  "xxx[10 $ char('ucs2'])",
  "?$10 $ char('ucs2'])",
  "?=10 $ char('ucs2'])",
  "?[10 $ char('ucs2')$",
  "?[10 $ char('ucs2')$",
  "10 = char('ucs2')",
  "?10 = char('ucs2')",
  "?(10 = char('ucs2'))",
  "$10 = char('ucs2')",
  "=10 = char('ucs2')",
  "$[10 = char('ucs2'])",
  "xxx[10 = char('ucs2'])",
  "?$10 = char('ucs2'])",
  "?=10 = char('ucs2'])",
  "?[10 = char('ucs2')$",
  "?[10 = char('ucs2')$",
  "$10 * char('ucs-2')",
  "=10 * char('ucs-2')",
  "$[10 * char('ucs-2'])",
  "xxx[10 * char('ucs-2'])",
  "?$10 * char('ucs-2'])",
  "?=10 * char('ucs-2'])",
  "?[10 * char('ucs-2')$",
  "?[10 * char('ucs-2')$",
  "$char('ucs-2')",
  "=char('ucs-2')",
  "$[char('ucs-2'])",
  "xxx[char('ucs-2'])",
  "?$char('ucs-2'])",
  "?=char('ucs-2'])",
  "?[char('ucs-2')$",
  "?[char('ucs-2')$",
  "10 $ char('ucs-2')",
  "?10 $ char('ucs-2')",
  "?(10 $ char('ucs-2'))",
  "$10 $ char('ucs-2')",
  "=10 $ char('ucs-2')",
  "$[10 $ char('ucs-2'])",
  "xxx[10 $ char('ucs-2'])",
  "?$10 $ char('ucs-2'])",
  "?=10 $ char('ucs-2'])",
  "?[10 $ char('ucs-2')$",
  "?[10 $ char('ucs-2')$",
  "10 = char('ucs-2')",
  "?10 = char('ucs-2')",
  "?(10 = char('ucs-2'))",
  "$10 = char('ucs-2')",
  "=10 = char('ucs-2')",
  "$[10 = char('ucs-2'])",
  "xxx[10 = char('ucs-2'])",
  "?$10 = char('ucs-2'])",
  "?=10 = char('ucs-2'])",
  "?[10 = char('ucs-2')$",
  "?[10 = char('ucs-2')$",
  "$10 * char('ucs_2')",
  "=10 * char('ucs_2')",
  "$[10 * char('ucs_2'])",
  "xxx[10 * char('ucs_2'])",
  "?$10 * char('ucs_2'])",
  "?=10 * char('ucs_2'])",
  "?[10 * char('ucs_2')$",
  "?[10 * char('ucs_2')$",
  "$char('ucs_2')",
  "=char('ucs_2')",
  "$[char('ucs_2'])",
  "xxx[char('ucs_2'])",
  "?$char('ucs_2'])",
  "?=char('ucs_2'])",
  "?[char('ucs_2')$",
  "?[char('ucs_2')$",
  "10 $ char('ucs_2')",
  "?10 $ char('ucs_2')",
  "?(10 $ char('ucs_2'))",
  "$10 $ char('ucs_2')",
  "=10 $ char('ucs_2')",
  "$[10 $ char('ucs_2'])",
  "xxx[10 $ char('ucs_2'])",
  "?$10 $ char('ucs_2'])",
  "?=10 $ char('ucs_2'])",
  "?[10 $ char('ucs_2')$",
  "?[10 $ char('ucs_2')$",
  "10 = char('ucs_2')",
  "?10 = char('ucs_2')",
  "?(10 = char('ucs_2'))",
  "$10 = char('ucs_2')",
  "=10 = char('ucs_2')",
  "$[10 = char('ucs_2'])",
  "xxx[10 = char('ucs_2'])",
  "?$10 = char('ucs_2'])",
  "?=10 = char('ucs_2'])",
  "?[10 = char('ucs_2')$",
  "?[10 = char('ucs_2')$",
  "10 * $['ucs_2']",
  "?10 * $['ucs_2']",
  "?[10 * $['ucs_2']]",
  "$10 * $['ucs_2']",
  "=10 * $['ucs_2']",
  "$[10 * $['ucs_2']]",
  "xxx[10 * $['ucs_2']]",
  "?$10 * $['ucs_2']]",
  "?=10 * $['ucs_2']]",
  "?[10 * $['ucs_2']$",
  "?[10 * $['ucs_2']$",
  "$['ucs_2']",
  "?$['ucs_2']",
  "?[$['ucs_2']]",
  "$$['ucs_2']",
  "=$['ucs_2']",
  "$[$['ucs_2']]",
  "xxx[$['ucs_2']]",
  "?$$['ucs_2']]",
  "?=$['ucs_2']]",
  "?[$['ucs_2']$",
  "?[$['ucs_2']$",
  "10 $ $['ucs_2']",
  "?10 $ $['ucs_2']",
  "?[10 $ $['ucs_2']]",
  "$10 $ $['ucs_2']",
  "=10 $ $['ucs_2']",
  "$[10 $ $['ucs_2']]",
  "xxx[10 $ $['ucs_2']]",
  "?$10 $ $['ucs_2']]",
  "?=10 $ $['ucs_2']]",
  "?[10 $ $['ucs_2']$",
  "?[10 $ $['ucs_2']$",
  "10 = $['ucs_2']",
  "?10 = $['ucs_2']",
  "?[10 = $['ucs_2']]",
  "$10 = $['ucs_2']",
  "=10 = $['ucs_2']",
  "$[10 = $['ucs_2']]",
  "xxx[10 = $['ucs_2']]",
  "?$10 = $['ucs_2']]",
  "?=10 = $['ucs_2']]",
  "?[10 = $['ucs_2']$",
  "?[10 = $['ucs_2']$",
  "10 * xxx['ucs_2']",
  "?10 * xxx['ucs_2']",
  "?[10 * xxx['ucs_2']]",
  "$10 * xxx['ucs_2']",
  "=10 * xxx['ucs_2']",
  "$[10 * xxx['ucs_2']]",
  "xxx[10 * xxx['ucs_2']]",
  "?$10 * xxx['ucs_2']]",
  "?=10 * xxx['ucs_2']]",
  "?[10 * xxx['ucs_2']$",
  "?[10 * xxx['ucs_2']$",
  "xxx['ucs_2']",
  "?xxx['ucs_2']",
  "?[xxx['ucs_2']]",
  "$xxx['ucs_2']",
  "=xxx['ucs_2']",
  "$[xxx['ucs_2']]",
  "xxx[xxx['ucs_2']]",
  "?$xxx['ucs_2']]",
  "?=xxx['ucs_2']]",
  "?[xxx['ucs_2']$",
  "?[xxx['ucs_2']$",
  "10 $ xxx['ucs_2']",
  "?10 $ xxx['ucs_2']",
  "?[10 $ xxx['ucs_2']]",
  "$10 $ xxx['ucs_2']",
  "=10 $ xxx['ucs_2']",
  "$[10 $ xxx['ucs_2']]",
  "xxx[10 $ xxx['ucs_2']]",
  "?$10 $ xxx['ucs_2']]",
  "?=10 $ xxx['ucs_2']]",
  "?[10 $ xxx['ucs_2']$",
  "?[10 $ xxx['ucs_2']$",
  "10 = xxx['ucs_2']",
  "?10 = xxx['ucs_2']",
  "?[10 = xxx['ucs_2']]",
  "$10 = xxx['ucs_2']",
  "=10 = xxx['ucs_2']",
  "$[10 = xxx['ucs_2']]",
  "xxx[10 = xxx['ucs_2']]",
  "?$10 = xxx['ucs_2']]",
  "?=10 = xxx['ucs_2']]",
  "?[10 = xxx['ucs_2']$",
  "?[10 = xxx['ucs_2']$",
  "10 * char$'ucs_2']",
  "?10 * char$'ucs_2']",
  "?(10 * char$'ucs_2'])",
  "$10 * char$'ucs_2']",
  "=10 * char$'ucs_2']",
  "$[10 * char$'ucs_2']]",
  "xxx[10 * char$'ucs_2']]",
  "?$10 * char$'ucs_2']]",
  "?=10 * char$'ucs_2']]",
  "?(10 * char$'ucs_2')$",
  "?(10 * char$'ucs_2')$",
  "char$'ucs_2']",
  "?char$'ucs_2']",
  "?(char$'ucs_2'])",
  "$char$'ucs_2']",
  "=char$'ucs_2']",
  "$[char$'ucs_2']]",
  "xxx[char$'ucs_2']]",
  "?$char$'ucs_2']]",
  "?=char$'ucs_2']]",
  "?(char$'ucs_2')$",
  "?(char$'ucs_2')$",
  "10 $ char$'ucs_2']",
  "?10 $ char$'ucs_2']",
  "?(10 $ char$'ucs_2'])",
  "$10 $ char$'ucs_2']",
  "=10 $ char$'ucs_2']",
  "$[10 $ char$'ucs_2']]",
  "xxx[10 $ char$'ucs_2']]",
  "?$10 $ char$'ucs_2']]",
  "?=10 $ char$'ucs_2']]",
  "?(10 $ char$'ucs_2')$",
  "?(10 $ char$'ucs_2')$",
  "10 = char$'ucs_2']",
  "?10 = char$'ucs_2']",
  "?(10 = char$'ucs_2'])",
  "$10 = char$'ucs_2']",
  "=10 = char$'ucs_2']",
  "$[10 = char$'ucs_2']]",
  "xxx[10 = char$'ucs_2']]",
  "?$10 = char$'ucs_2']]",
  "?=10 = char$'ucs_2']]",
  "?(10 = char$'ucs_2')$",
  "?(10 = char$'ucs_2')$",
  "10 * char='ucs_2']",
  "?10 * char='ucs_2']",
  "?(10 * char='ucs_2'])",
  "$10 * char='ucs_2']",
  "=10 * char='ucs_2']",
  "$[10 * char='ucs_2']]",
  "xxx[10 * char='ucs_2']]",
  "?$10 * char='ucs_2']]",
  "?=10 * char='ucs_2']]",
  "?(10 * char='ucs_2')$",
  "?(10 * char='ucs_2')$",
  "char='ucs_2']",
  "?char='ucs_2']",
  "?(char='ucs_2'])",
  "$char='ucs_2']",
  "=char='ucs_2']",
  "$[char='ucs_2']]",
  "xxx[char='ucs_2']]",
  "?$char='ucs_2']]",
  "?=char='ucs_2']]",
  "?(char='ucs_2')$",
  "?(char='ucs_2')$",
  "10 $ char='ucs_2']",
  "?10 $ char='ucs_2']",
  "?(10 $ char='ucs_2'])",
  "$10 $ char='ucs_2']",
  "=10 $ char='ucs_2']",
  "$[10 $ char='ucs_2']]",
  "xxx[10 $ char='ucs_2']]",
  "?$10 $ char='ucs_2']]",
  "?=10 $ char='ucs_2']]",
  "?(10 $ char='ucs_2')$",
  "?(10 $ char='ucs_2')$",
  "10 = char='ucs_2']",
  "?10 = char='ucs_2']",
  "?(10 = char='ucs_2'])",
  "$10 = char='ucs_2']",
  "=10 = char='ucs_2']",
  "$[10 = char='ucs_2']]",
  "xxx[10 = char='ucs_2']]",
  "?$10 = char='ucs_2']]",
  "?=10 = char='ucs_2']]",
  "?(10 = char='ucs_2')$",
  "?(10 = char='ucs_2')$",
  "10 * char($)",
  "?10 * char($)",
  "?(10 * char($))",
  "$10 * char($)",
  "=10 * char($)",
  "$[10 * char($])",
  "xxx[10 * char($])",
  "?$10 * char($])",
  "?=10 * char($])",
  "?[10 * char($)$",
  "?[10 * char($)$",
  "char($)",
  "?char($)",
  "?(char($))",
  "$char($)",
  "=char($)",
  "$[char($])",
  "xxx[char($])",
  "?$char($])",
  "?=char($])",
  "?[char($)$",
  "?[char($)$",
  "10 $ char($)",
  "?10 $ char($)",
  "?(10 $ char($))",
  "$10 $ char($)",
  "=10 $ char($)",
  "$[10 $ char($])",
  "xxx[10 $ char($])",
  "?$10 $ char($])",
  "?=10 $ char($])",
  "?[10 $ char($)$",
  "?[10 $ char($)$",
  "10 = char($)",
  "?10 = char($)",
  "?(10 = char($))",
  "$10 = char($)",
  "=10 = char($)",
  "$[10 = char($])",
  "xxx[10 = char($])",
  "?$10 = char($])",
  "?=10 = char($])",
  "?[10 = char($)$",
  "?[10 = char($)$",
  "10 * char('')",
  "?10 * char('')",
  "?(10 * char(''))",
  "$10 * char('')",
  "=10 * char('')",
  "$[10 * char(''])",
  "xxx[10 * char(''])",
  "?$10 * char(''])",
  "?=10 * char(''])",
  "?[10 * char('')$",
  "?[10 * char('')$",
  "char('')",
  "?char('')",
  "?(char(''))",
  "$char('')",
  "=char('')",
  "$[char(''])",
  "xxx[char(''])",
  "?$char(''])",
  "?=char(''])",
  "?[char('')$",
  "?[char('')$",
  "10 $ char('')",
  "?10 $ char('')",
  "?(10 $ char(''))",
  "$10 $ char('')",
  "=10 $ char('')",
  "$[10 $ char(''])",
  "xxx[10 $ char(''])",
  "?$10 $ char(''])",
  "?=10 $ char(''])",
  "?[10 $ char('')$",
  "?[10 $ char('')$",
  "10 = char('')",
  "?10 = char('')",
  "?(10 = char(''))",
  "$10 = char('')",
  "=10 = char('')",
  "$[10 = char(''])",
  "xxx[10 = char(''])",
  "?$10 = char(''])",
  "?=10 = char(''])",
  "?[10 = char('')$",
  "?[10 = char('')$",
  "10 * char('xxx')",
  "?10 * char('xxx')",
  "?(10 * char('xxx'))",
  "$10 * char('xxx')",
  "=10 * char('xxx')",
  "$[10 * char('xxx'])",
  "xxx[10 * char('xxx'])",
  "?$10 * char('xxx'])",
  "?=10 * char('xxx'])",
  "?[10 * char('xxx')$",
  "?[10 * char('xxx')$",
  "char('xxx')",
  "?char('xxx')",
  "?(char('xxx'))",
  "$char('xxx')",
  "=char('xxx')",
  "$[char('xxx'])",
  "xxx[char('xxx'])",
  "?$char('xxx'])",
  "?=char('xxx'])",
  "?[char('xxx')$",
  "?[char('xxx')$",
  "10 $ char('xxx')",
  "?10 $ char('xxx')",
  "?(10 $ char('xxx'))",
  "$10 $ char('xxx')",
  "=10 $ char('xxx')",
  "$[10 $ char('xxx'])",
  "xxx[10 $ char('xxx'])",
  "?$10 $ char('xxx'])",
  "?=10 $ char('xxx'])",
  "?[10 $ char('xxx')$",
  "?[10 $ char('xxx')$",
  "10 = char('xxx')",
  "?10 = char('xxx')",
  "?(10 = char('xxx'))",
  "$10 = char('xxx')",
  "=10 = char('xxx')",
  "$[10 = char('xxx'])",
  "xxx[10 = char('xxx'])",
  "?$10 = char('xxx'])",
  "?=10 = char('xxx'])",
  "?[10 = char('xxx')$",
  "?[10 = char('xxx')$",
  "10 * char['utf8'$",
  "?10 * char['utf8'$",
  "?[10 * char('utf8'$)",
  "$10 * char['utf8'$",
  "=10 * char['utf8'$",
  "$[10 * char('utf8'$)",
  "xxx[10 * char('utf8'$)",
  "?$10 * char('utf8'$)",
  "?=10 * char('utf8'$)",
  "?[10 * char['utf8'$$",
  "?[10 * char['utf8'$$",
  "char['utf8'$",
  "?char['utf8'$",
  "?[char('utf8'$)",
  "$char['utf8'$",
  "=char['utf8'$",
  "$[char('utf8'$)",
  "xxx[char('utf8'$)",
  "?$char('utf8'$)",
  "?=char('utf8'$)",
  "?[char['utf8'$$",
  "?[char['utf8'$$",
  "10 $ char['utf8'$",
  "?10 $ char['utf8'$",
  "?[10 $ char('utf8'$)",
  "$10 $ char['utf8'$",
  "=10 $ char['utf8'$",
  "$[10 $ char('utf8'$)",
  "xxx[10 $ char('utf8'$)",
  "?$10 $ char('utf8'$)",
  "?=10 $ char('utf8'$)",
  "?[10 $ char['utf8'$$",
  "?[10 $ char['utf8'$$",
  "10 = char['utf8'$",
  "?10 = char['utf8'$",
  "?[10 = char('utf8'$)",
  "$10 = char['utf8'$",
  "=10 = char['utf8'$",
  "$[10 = char('utf8'$)",
  "xxx[10 = char('utf8'$)",
  "?$10 = char('utf8'$)",
  "?=10 = char('utf8'$)",
  "?[10 = char['utf8'$$",
  "?[10 = char['utf8'$$",
  "10 * char['utf8'$",
  "?10 * char['utf8'$",
  "?[10 * char('utf8'$)",
  "$10 * char['utf8'$",
  "=10 * char['utf8'$",
  "$[10 * char('utf8'$)",
  "xxx[10 * char('utf8'$)",
  "?$10 * char('utf8'$)",
  "?=10 * char('utf8'$)",
  "?[10 * char['utf8'$$",
  "?[10 * char['utf8'$$",
  "char['utf8'$",
  "?char['utf8'$",
  "?[char('utf8'$)",
  "$char['utf8'$",
  "=char['utf8'$",
  "$[char('utf8'$)",
  "xxx[char('utf8'$)",
  "?$char('utf8'$)",
  "?=char('utf8'$)",
  "?[char['utf8'$$",
  "?[char['utf8'$$",
  "10 $ char['utf8'$",
  "?10 $ char['utf8'$",
  "?[10 $ char('utf8'$)",
  "$10 $ char['utf8'$",
  "=10 $ char['utf8'$",
  "$[10 $ char('utf8'$)",
  "xxx[10 $ char('utf8'$)",
  "?$10 $ char('utf8'$)",
  "?=10 $ char('utf8'$)",
  "?[10 $ char['utf8'$$",
  "?[10 $ char['utf8'$$",
  "10 = char['utf8'$",
  "?10 = char['utf8'$",
  "?[10 = char('utf8'$)",
  "$10 = char['utf8'$",
  "=10 = char['utf8'$",
  "$[10 = char('utf8'$)",
  "xxx[10 = char('utf8'$)",
  "?$10 = char('utf8'$)",
  "?=10 = char('utf8'$)",
  "?[10 = char['utf8'$$",
  "?[10 = char['utf8'$$",
  "$10 * string",
  "=10 * string",
  "$[10 * string]",
  "xxx[10 * string]",
  "?$10 * string]",
  "?=10 * string]",
  "?[10 * string$",
  "?[10 * string$",
  "$string",
  "=string",
  "$[string]",
  "xxx[string]",
  "?$string]",
  "?=string]",
  "?[string$",
  "?[string$",
  "10 $ string",
  "?10 $ string",
  "?(10 $ string)",
  "$10 $ string",
  "=10 $ string",
  "$[10 $ string]",
  "xxx[10 $ string]",
  "?$10 $ string]",
  "?=10 $ string]",
  "?[10 $ string$",
  "?[10 $ string$",
  "10 = string",
  "?10 = string",
  "?(10 = string)",
  "$10 = string",
  "=10 = string",
  "$[10 = string]",
  "xxx[10 = string]",
  "?$10 = string]",
  "?=10 = string]",
  "?[10 = string$",
  "?[10 = string$",
  "$10 * FixedStringKind",
  "=10 * FixedStringKind",
  "$[10 * FixedStringKind]",
  "xxx[10 * FixedStringKind]",
  "?$10 * FixedStringKind]",
  "?=10 * FixedStringKind]",
  "?[10 * FixedStringKind$",
  "?[10 * FixedStringKind$",
  "$FixedStringKind",
  "=FixedStringKind",
  "$[FixedStringKind]",
  "xxx[FixedStringKind]",
  "?$FixedStringKind]",
  "?=FixedStringKind]",
  "?[FixedStringKind$",
  "?[FixedStringKind$",
  "10 $ FixedStringKind",
  "?10 $ FixedStringKind",
  "?(10 $ FixedStringKind)",
  "$10 $ FixedStringKind",
  "=10 $ FixedStringKind",
  "$[10 $ FixedStringKind]",
  "xxx[10 $ FixedStringKind]",
  "?$10 $ FixedStringKind]",
  "?=10 $ FixedStringKind]",
  "?[10 $ FixedStringKind$",
  "?[10 $ FixedStringKind$",
  "10 = FixedStringKind",
  "?10 = FixedStringKind",
  "?(10 = FixedStringKind)",
  "$10 = FixedStringKind",
  "=10 = FixedStringKind",
  "$[10 = FixedStringKind]",
  "xxx[10 = FixedStringKind]",
  "?$10 = FixedStringKind]",
  "?=10 = FixedStringKind]",
  "?[10 = FixedStringKind$",
  "?[10 = FixedStringKind$",
  "$10 * fixed_string(2356491620)",
  "=10 * fixed_string(2356491620)",
  "$[10 * fixed_string(2356491620])",
  "xxx[10 * fixed_string(2356491620])",
  "?$10 * fixed_string(2356491620])",
  "?=10 * fixed_string(2356491620])",
  "?[10 * fixed_string(2356491620)$",
  "?[10 * fixed_string(2356491620)$",
  "$fixed_string(2356491620)",
  "=fixed_string(2356491620)",
  "$[fixed_string(2356491620])",
  "xxx[fixed_string(2356491620])",
  "?$fixed_string(2356491620])",
  "?=fixed_string(2356491620])",
  "?[fixed_string(2356491620)$",
  "?[fixed_string(2356491620)$",
  "10 $ fixed_string(2356491620)",
  "?10 $ fixed_string(2356491620)",
  "?(10 $ fixed_string(2356491620))",
  "$10 $ fixed_string(2356491620)",
  "=10 $ fixed_string(2356491620)",
  "$[10 $ fixed_string(2356491620])",
  "xxx[10 $ fixed_string(2356491620])",
  "?$10 $ fixed_string(2356491620])",
  "?=10 $ fixed_string(2356491620])",
  "?[10 $ fixed_string(2356491620)$",
  "?[10 $ fixed_string(2356491620)$",
  "10 = fixed_string(2356491620)",
  "?10 = fixed_string(2356491620)",
  "?(10 = fixed_string(2356491620))",
  "$10 = fixed_string(2356491620)",
  "=10 = fixed_string(2356491620)",
  "$[10 = fixed_string(2356491620])",
  "xxx[10 = fixed_string(2356491620])",
  "?$10 = fixed_string(2356491620])",
  "?=10 = fixed_string(2356491620])",
  "?[10 = fixed_string(2356491620)$",
  "?[10 = fixed_string(2356491620)$",
  "$10 * fixed_string(2588335668, 'A')",
  "=10 * fixed_string(2588335668, 'A')",
  "$[10 * fixed_string(2588335668, 'A'])",
  "xxx[10 * fixed_string(2588335668, 'A'])",
  "?$10 * fixed_string(2588335668, 'A'])",
  "?=10 * fixed_string(2588335668, 'A'])",
  "?[10 * fixed_string(2588335668, 'A')$",
  "?[10 * fixed_string(2588335668, 'A')$",
  "$fixed_string(2588335668, 'A')",
  "=fixed_string(2588335668, 'A')",
  "$[fixed_string(2588335668, 'A'])",
  "xxx[fixed_string(2588335668, 'A'])",
  "?$fixed_string(2588335668, 'A'])",
  "?=fixed_string(2588335668, 'A'])",
  "?[fixed_string(2588335668, 'A')$",
  "?[fixed_string(2588335668, 'A')$",
  "10 $ fixed_string(2588335668, 'A')",
  "?10 $ fixed_string(2588335668, 'A')",
  "?(10 $ fixed_string(2588335668, 'A'))",
  "$10 $ fixed_string(2588335668, 'A')",
  "=10 $ fixed_string(2588335668, 'A')",
  "$[10 $ fixed_string(2588335668, 'A'])",
  "xxx[10 $ fixed_string(2588335668, 'A'])",
  "?$10 $ fixed_string(2588335668, 'A'])",
  "?=10 $ fixed_string(2588335668, 'A'])",
  "?[10 $ fixed_string(2588335668, 'A')$",
  "?[10 $ fixed_string(2588335668, 'A')$",
  "10 = fixed_string(2588335668, 'A')",
  "?10 = fixed_string(2588335668, 'A')",
  "?(10 = fixed_string(2588335668, 'A'))",
  "$10 = fixed_string(2588335668, 'A')",
  "=10 = fixed_string(2588335668, 'A')",
  "$[10 = fixed_string(2588335668, 'A'])",
  "xxx[10 = fixed_string(2588335668, 'A'])",
  "?$10 = fixed_string(2588335668, 'A'])",
  "?=10 = fixed_string(2588335668, 'A'])",
  "?[10 = fixed_string(2588335668, 'A')$",
  "?[10 = fixed_string(2588335668, 'A')$",
  "$10 * fixed_string(315657682, 'ascii')",
  "=10 * fixed_string(315657682, 'ascii')",
  "$[10 * fixed_string(315657682, 'ascii'])",
  "xxx[10 * fixed_string(315657682, 'ascii'])",
  "?$10 * fixed_string(315657682, 'ascii'])",
  "?=10 * fixed_string(315657682, 'ascii'])",
  "?[10 * fixed_string(315657682, 'ascii')$",
  "?[10 * fixed_string(315657682, 'ascii')$",
  "$fixed_string(315657682, 'ascii')",
  "=fixed_string(315657682, 'ascii')",
  "$[fixed_string(315657682, 'ascii'])",
  "xxx[fixed_string(315657682, 'ascii'])",
  "?$fixed_string(315657682, 'ascii'])",
  "?=fixed_string(315657682, 'ascii'])",
  "?[fixed_string(315657682, 'ascii')$",
  "?[fixed_string(315657682, 'ascii')$",
  "10 $ fixed_string(315657682, 'ascii')",
  "?10 $ fixed_string(315657682, 'ascii')",
  "?(10 $ fixed_string(315657682, 'ascii'))",
  "$10 $ fixed_string(315657682, 'ascii')",
  "=10 $ fixed_string(315657682, 'ascii')",
  "$[10 $ fixed_string(315657682, 'ascii'])",
  "xxx[10 $ fixed_string(315657682, 'ascii'])",
  "?$10 $ fixed_string(315657682, 'ascii'])",
  "?=10 $ fixed_string(315657682, 'ascii'])",
  "?[10 $ fixed_string(315657682, 'ascii')$",
  "?[10 $ fixed_string(315657682, 'ascii')$",
  "10 = fixed_string(315657682, 'ascii')",
  "?10 = fixed_string(315657682, 'ascii')",
  "?(10 = fixed_string(315657682, 'ascii'))",
  "$10 = fixed_string(315657682, 'ascii')",
  "=10 = fixed_string(315657682, 'ascii')",
  "$[10 = fixed_string(315657682, 'ascii'])",
  "xxx[10 = fixed_string(315657682, 'ascii'])",
  "?$10 = fixed_string(315657682, 'ascii'])",
  "?=10 = fixed_string(315657682, 'ascii'])",
  "?[10 = fixed_string(315657682, 'ascii')$",
  "?[10 = fixed_string(315657682, 'ascii')$",
  "$10 * fixed_string(3109418515, 'us-ascii')",
  "=10 * fixed_string(3109418515, 'us-ascii')",
  "$[10 * fixed_string(3109418515, 'us-ascii'])",
  "xxx[10 * fixed_string(3109418515, 'us-ascii'])",
  "?$10 * fixed_string(3109418515, 'us-ascii'])",
  "?=10 * fixed_string(3109418515, 'us-ascii'])",
  "?[10 * fixed_string(3109418515, 'us-ascii')$",
  "?[10 * fixed_string(3109418515, 'us-ascii')$",
  "$fixed_string(3109418515, 'us-ascii')",
  "=fixed_string(3109418515, 'us-ascii')",
  "$[fixed_string(3109418515, 'us-ascii'])",
  "xxx[fixed_string(3109418515, 'us-ascii'])",
  "?$fixed_string(3109418515, 'us-ascii'])",
  "?=fixed_string(3109418515, 'us-ascii'])",
  "?[fixed_string(3109418515, 'us-ascii')$",
  "?[fixed_string(3109418515, 'us-ascii')$",
  "10 $ fixed_string(3109418515, 'us-ascii')",
  "?10 $ fixed_string(3109418515, 'us-ascii')",
  "?(10 $ fixed_string(3109418515, 'us-ascii'))",
  "$10 $ fixed_string(3109418515, 'us-ascii')",
  "=10 $ fixed_string(3109418515, 'us-ascii')",
  "$[10 $ fixed_string(3109418515, 'us-ascii'])",
  "xxx[10 $ fixed_string(3109418515, 'us-ascii'])",
  "?$10 $ fixed_string(3109418515, 'us-ascii'])",
  "?=10 $ fixed_string(3109418515, 'us-ascii'])",
  "?[10 $ fixed_string(3109418515, 'us-ascii')$",
  "?[10 $ fixed_string(3109418515, 'us-ascii')$",
  "10 = fixed_string(3109418515, 'us-ascii')",
  "?10 = fixed_string(3109418515, 'us-ascii')",
  "?(10 = fixed_string(3109418515, 'us-ascii'))",
  "$10 = fixed_string(3109418515, 'us-ascii')",
  "=10 = fixed_string(3109418515, 'us-ascii')",
  "$[10 = fixed_string(3109418515, 'us-ascii'])",
  "xxx[10 = fixed_string(3109418515, 'us-ascii'])",
  "?$10 = fixed_string(3109418515, 'us-ascii'])",
  "?=10 = fixed_string(3109418515, 'us-ascii'])",
  "?[10 = fixed_string(3109418515, 'us-ascii')$",
  "?[10 = fixed_string(3109418515, 'us-ascii')$",
  "$10 * fixed_string(3894583270, 'U8')",
  "=10 * fixed_string(3894583270, 'U8')",
  "$[10 * fixed_string(3894583270, 'U8'])",
  "xxx[10 * fixed_string(3894583270, 'U8'])",
  "?$10 * fixed_string(3894583270, 'U8'])",
  "?=10 * fixed_string(3894583270, 'U8'])",
  "?[10 * fixed_string(3894583270, 'U8')$",
  "?[10 * fixed_string(3894583270, 'U8')$",
  "$fixed_string(3894583270, 'U8')",
  "=fixed_string(3894583270, 'U8')",
  "$[fixed_string(3894583270, 'U8'])",
  "xxx[fixed_string(3894583270, 'U8'])",
  "?$fixed_string(3894583270, 'U8'])",
  "?=fixed_string(3894583270, 'U8'])",
  "?[fixed_string(3894583270, 'U8')$",
  "?[fixed_string(3894583270, 'U8')$",
  "10 $ fixed_string(3894583270, 'U8')",
  "?10 $ fixed_string(3894583270, 'U8')",
  "?(10 $ fixed_string(3894583270, 'U8'))",
  "$10 $ fixed_string(3894583270, 'U8')",
  "=10 $ fixed_string(3894583270, 'U8')",
  "$[10 $ fixed_string(3894583270, 'U8'])",
  "xxx[10 $ fixed_string(3894583270, 'U8'])",
  "?$10 $ fixed_string(3894583270, 'U8'])",
  "?=10 $ fixed_string(3894583270, 'U8'])",
  "?[10 $ fixed_string(3894583270, 'U8')$",
  "?[10 $ fixed_string(3894583270, 'U8')$",
  "10 = fixed_string(3894583270, 'U8')",
  "?10 = fixed_string(3894583270, 'U8')",
  "?(10 = fixed_string(3894583270, 'U8'))",
  "$10 = fixed_string(3894583270, 'U8')",
  "=10 = fixed_string(3894583270, 'U8')",
  "$[10 = fixed_string(3894583270, 'U8'])",
  "xxx[10 = fixed_string(3894583270, 'U8'])",
  "?$10 = fixed_string(3894583270, 'U8'])",
  "?=10 = fixed_string(3894583270, 'U8'])",
  "?[10 = fixed_string(3894583270, 'U8')$",
  "?[10 = fixed_string(3894583270, 'U8')$",
  "$10 * fixed_string(1878160692, 'utf8')",
  "=10 * fixed_string(1878160692, 'utf8')",
  "$[10 * fixed_string(1878160692, 'utf8'])",
  "xxx[10 * fixed_string(1878160692, 'utf8'])",
  "?$10 * fixed_string(1878160692, 'utf8'])",
  "?=10 * fixed_string(1878160692, 'utf8'])",
  "?[10 * fixed_string(1878160692, 'utf8')$",
  "?[10 * fixed_string(1878160692, 'utf8')$",
  "$fixed_string(1878160692, 'utf8')",
  "=fixed_string(1878160692, 'utf8')",
  "$[fixed_string(1878160692, 'utf8'])",
  "xxx[fixed_string(1878160692, 'utf8'])",
  "?$fixed_string(1878160692, 'utf8'])",
  "?=fixed_string(1878160692, 'utf8'])",
  "?[fixed_string(1878160692, 'utf8')$",
  "?[fixed_string(1878160692, 'utf8')$",
  "10 $ fixed_string(1878160692, 'utf8')",
  "?10 $ fixed_string(1878160692, 'utf8')",
  "?(10 $ fixed_string(1878160692, 'utf8'))",
  "$10 $ fixed_string(1878160692, 'utf8')",
  "=10 $ fixed_string(1878160692, 'utf8')",
  "$[10 $ fixed_string(1878160692, 'utf8'])",
  "xxx[10 $ fixed_string(1878160692, 'utf8'])",
  "?$10 $ fixed_string(1878160692, 'utf8'])",
  "?=10 $ fixed_string(1878160692, 'utf8'])",
  "?[10 $ fixed_string(1878160692, 'utf8')$",
  "?[10 $ fixed_string(1878160692, 'utf8')$",
  "10 = fixed_string(1878160692, 'utf8')",
  "?10 = fixed_string(1878160692, 'utf8')",
  "?(10 = fixed_string(1878160692, 'utf8'))",
  "$10 = fixed_string(1878160692, 'utf8')",
  "=10 = fixed_string(1878160692, 'utf8')",
  "$[10 = fixed_string(1878160692, 'utf8'])",
  "xxx[10 = fixed_string(1878160692, 'utf8'])",
  "?$10 = fixed_string(1878160692, 'utf8'])",
  "?=10 = fixed_string(1878160692, 'utf8'])",
  "?[10 = fixed_string(1878160692, 'utf8')$",
  "?[10 = fixed_string(1878160692, 'utf8')$",
  "$10 * fixed_string(3619200656, 'utf-8')",
  "=10 * fixed_string(3619200656, 'utf-8')",
  "$[10 * fixed_string(3619200656, 'utf-8'])",
  "xxx[10 * fixed_string(3619200656, 'utf-8'])",
  "?$10 * fixed_string(3619200656, 'utf-8'])",
  "?=10 * fixed_string(3619200656, 'utf-8'])",
  "?[10 * fixed_string(3619200656, 'utf-8')$",
  "?[10 * fixed_string(3619200656, 'utf-8')$",
  "$fixed_string(3619200656, 'utf-8')",
  "=fixed_string(3619200656, 'utf-8')",
  "$[fixed_string(3619200656, 'utf-8'])",
  "xxx[fixed_string(3619200656, 'utf-8'])",
  "?$fixed_string(3619200656, 'utf-8'])",
  "?=fixed_string(3619200656, 'utf-8'])",
  "?[fixed_string(3619200656, 'utf-8')$",
  "?[fixed_string(3619200656, 'utf-8')$",
  "10 $ fixed_string(3619200656, 'utf-8')",
  "?10 $ fixed_string(3619200656, 'utf-8')",
  "?(10 $ fixed_string(3619200656, 'utf-8'))",
  "$10 $ fixed_string(3619200656, 'utf-8')",
  "=10 $ fixed_string(3619200656, 'utf-8')",
  "$[10 $ fixed_string(3619200656, 'utf-8'])",
  "xxx[10 $ fixed_string(3619200656, 'utf-8'])",
  "?$10 $ fixed_string(3619200656, 'utf-8'])",
  "?=10 $ fixed_string(3619200656, 'utf-8'])",
  "?[10 $ fixed_string(3619200656, 'utf-8')$",
  "?[10 $ fixed_string(3619200656, 'utf-8')$",
  "10 = fixed_string(3619200656, 'utf-8')",
  "?10 = fixed_string(3619200656, 'utf-8')",
  "?(10 = fixed_string(3619200656, 'utf-8'))",
  "$10 = fixed_string(3619200656, 'utf-8')",
  "=10 = fixed_string(3619200656, 'utf-8')",
  "$[10 = fixed_string(3619200656, 'utf-8'])",
  "xxx[10 = fixed_string(3619200656, 'utf-8'])",
  "?$10 = fixed_string(3619200656, 'utf-8'])",
  "?=10 = fixed_string(3619200656, 'utf-8'])",
  "?[10 = fixed_string(3619200656, 'utf-8')$",
  "?[10 = fixed_string(3619200656, 'utf-8')$",
  "$10 * fixed_string(675261477, 'U16')",
  "=10 * fixed_string(675261477, 'U16')",
  "$[10 * fixed_string(675261477, 'U16'])",
  "xxx[10 * fixed_string(675261477, 'U16'])",
  "?$10 * fixed_string(675261477, 'U16'])",
  "?=10 * fixed_string(675261477, 'U16'])",
  "?[10 * fixed_string(675261477, 'U16')$",
  "?[10 * fixed_string(675261477, 'U16')$",
  "$fixed_string(675261477, 'U16')",
  "=fixed_string(675261477, 'U16')",
  "$[fixed_string(675261477, 'U16'])",
  "xxx[fixed_string(675261477, 'U16'])",
  "?$fixed_string(675261477, 'U16'])",
  "?=fixed_string(675261477, 'U16'])",
  "?[fixed_string(675261477, 'U16')$",
  "?[fixed_string(675261477, 'U16')$",
  "10 $ fixed_string(675261477, 'U16')",
  "?10 $ fixed_string(675261477, 'U16')",
  "?(10 $ fixed_string(675261477, 'U16'))",
  "$10 $ fixed_string(675261477, 'U16')",
  "=10 $ fixed_string(675261477, 'U16')",
  "$[10 $ fixed_string(675261477, 'U16'])",
  "xxx[10 $ fixed_string(675261477, 'U16'])",
  "?$10 $ fixed_string(675261477, 'U16'])",
  "?=10 $ fixed_string(675261477, 'U16'])",
  "?[10 $ fixed_string(675261477, 'U16')$",
  "?[10 $ fixed_string(675261477, 'U16')$",
  "10 = fixed_string(675261477, 'U16')",
  "?10 = fixed_string(675261477, 'U16')",
  "?(10 = fixed_string(675261477, 'U16'))",
  "$10 = fixed_string(675261477, 'U16')",
  "=10 = fixed_string(675261477, 'U16')",
  "$[10 = fixed_string(675261477, 'U16'])",
  "xxx[10 = fixed_string(675261477, 'U16'])",
  "?$10 = fixed_string(675261477, 'U16'])",
  "?=10 = fixed_string(675261477, 'U16'])",
  "?[10 = fixed_string(675261477, 'U16')$",
  "?[10 = fixed_string(675261477, 'U16')$",
  "$10 * fixed_string(2010978336, 'utf16')",
  "=10 * fixed_string(2010978336, 'utf16')",
  "$[10 * fixed_string(2010978336, 'utf16'])",
  "xxx[10 * fixed_string(2010978336, 'utf16'])",
  "?$10 * fixed_string(2010978336, 'utf16'])",
  "?=10 * fixed_string(2010978336, 'utf16'])",
  "?[10 * fixed_string(2010978336, 'utf16')$",
  "?[10 * fixed_string(2010978336, 'utf16')$",
  "$fixed_string(2010978336, 'utf16')",
  "=fixed_string(2010978336, 'utf16')",
  "$[fixed_string(2010978336, 'utf16'])",
  "xxx[fixed_string(2010978336, 'utf16'])",
  "?$fixed_string(2010978336, 'utf16'])",
  "?=fixed_string(2010978336, 'utf16'])",
  "?[fixed_string(2010978336, 'utf16')$",
  "?[fixed_string(2010978336, 'utf16')$",
  "10 $ fixed_string(2010978336, 'utf16')",
  "?10 $ fixed_string(2010978336, 'utf16')",
  "?(10 $ fixed_string(2010978336, 'utf16'))",
  "$10 $ fixed_string(2010978336, 'utf16')",
  "=10 $ fixed_string(2010978336, 'utf16')",
  "$[10 $ fixed_string(2010978336, 'utf16'])",
  "xxx[10 $ fixed_string(2010978336, 'utf16'])",
  "?$10 $ fixed_string(2010978336, 'utf16'])",
  "?=10 $ fixed_string(2010978336, 'utf16'])",
  "?[10 $ fixed_string(2010978336, 'utf16')$",
  "?[10 $ fixed_string(2010978336, 'utf16')$",
  "10 = fixed_string(2010978336, 'utf16')",
  "?10 = fixed_string(2010978336, 'utf16')",
  "?(10 = fixed_string(2010978336, 'utf16'))",
  "$10 = fixed_string(2010978336, 'utf16')",
  "=10 = fixed_string(2010978336, 'utf16')",
  "$[10 = fixed_string(2010978336, 'utf16'])",
  "xxx[10 = fixed_string(2010978336, 'utf16'])",
  "?$10 = fixed_string(2010978336, 'utf16'])",
  "?=10 = fixed_string(2010978336, 'utf16'])",
  "?[10 = fixed_string(2010978336, 'utf16')$",
  "?[10 = fixed_string(2010978336, 'utf16')$",
  "$10 * fixed_string(2461628726, 'utf-16')",
  "=10 * fixed_string(2461628726, 'utf-16')",
  "$[10 * fixed_string(2461628726, 'utf-16'])",
  "xxx[10 * fixed_string(2461628726, 'utf-16'])",
  "?$10 * fixed_string(2461628726, 'utf-16'])",
  "?=10 * fixed_string(2461628726, 'utf-16'])",
  "?[10 * fixed_string(2461628726, 'utf-16')$",
  "?[10 * fixed_string(2461628726, 'utf-16')$",
  "$fixed_string(2461628726, 'utf-16')",
  "=fixed_string(2461628726, 'utf-16')",
  "$[fixed_string(2461628726, 'utf-16'])",
  "xxx[fixed_string(2461628726, 'utf-16'])",
  "?$fixed_string(2461628726, 'utf-16'])",
  "?=fixed_string(2461628726, 'utf-16'])",
  "?[fixed_string(2461628726, 'utf-16')$",
  "?[fixed_string(2461628726, 'utf-16')$",
  "10 $ fixed_string(2461628726, 'utf-16')",
  "?10 $ fixed_string(2461628726, 'utf-16')",
  "?(10 $ fixed_string(2461628726, 'utf-16'))",
  "$10 $ fixed_string(2461628726, 'utf-16')",
  "=10 $ fixed_string(2461628726, 'utf-16')",
  "$[10 $ fixed_string(2461628726, 'utf-16'])",
  "xxx[10 $ fixed_string(2461628726, 'utf-16'])",
  "?$10 $ fixed_string(2461628726, 'utf-16'])",
  "?=10 $ fixed_string(2461628726, 'utf-16'])",
  "?[10 $ fixed_string(2461628726, 'utf-16')$",
  "?[10 $ fixed_string(2461628726, 'utf-16')$",
  "10 = fixed_string(2461628726, 'utf-16')",
  "?10 = fixed_string(2461628726, 'utf-16')",
  "?(10 = fixed_string(2461628726, 'utf-16'))",
  "$10 = fixed_string(2461628726, 'utf-16')",
  "=10 = fixed_string(2461628726, 'utf-16')",
  "$[10 = fixed_string(2461628726, 'utf-16'])",
  "xxx[10 = fixed_string(2461628726, 'utf-16'])",
  "?$10 = fixed_string(2461628726, 'utf-16'])",
  "?=10 = fixed_string(2461628726, 'utf-16'])",
  "?[10 = fixed_string(2461628726, 'utf-16')$",
  "?[10 = fixed_string(2461628726, 'utf-16')$",
  "$10 * fixed_string(2248049116, 'U32')",
  "=10 * fixed_string(2248049116, 'U32')",
  "$[10 * fixed_string(2248049116, 'U32'])",
  "xxx[10 * fixed_string(2248049116, 'U32'])",
  "?$10 * fixed_string(2248049116, 'U32'])",
  "?=10 * fixed_string(2248049116, 'U32'])",
  "?[10 * fixed_string(2248049116, 'U32')$",
  "?[10 * fixed_string(2248049116, 'U32')$",
  "$fixed_string(2248049116, 'U32')",
  "=fixed_string(2248049116, 'U32')",
  "$[fixed_string(2248049116, 'U32'])",
  "xxx[fixed_string(2248049116, 'U32'])",
  "?$fixed_string(2248049116, 'U32'])",
  "?=fixed_string(2248049116, 'U32'])",
  "?[fixed_string(2248049116, 'U32')$",
  "?[fixed_string(2248049116, 'U32')$",
  "10 $ fixed_string(2248049116, 'U32')",
  "?10 $ fixed_string(2248049116, 'U32')",
  "?(10 $ fixed_string(2248049116, 'U32'))",
  "$10 $ fixed_string(2248049116, 'U32')",
  "=10 $ fixed_string(2248049116, 'U32')",
  "$[10 $ fixed_string(2248049116, 'U32'])",
  "xxx[10 $ fixed_string(2248049116, 'U32'])",
  "?$10 $ fixed_string(2248049116, 'U32'])",
  "?=10 $ fixed_string(2248049116, 'U32'])",
  "?[10 $ fixed_string(2248049116, 'U32')$",
  "?[10 $ fixed_string(2248049116, 'U32')$",
  "10 = fixed_string(2248049116, 'U32')",
  "?10 = fixed_string(2248049116, 'U32')",
  "?(10 = fixed_string(2248049116, 'U32'))",
  "$10 = fixed_string(2248049116, 'U32')",
  "=10 = fixed_string(2248049116, 'U32')",
  "$[10 = fixed_string(2248049116, 'U32'])",
  "xxx[10 = fixed_string(2248049116, 'U32'])",
  "?$10 = fixed_string(2248049116, 'U32'])",
  "?=10 = fixed_string(2248049116, 'U32'])",
  "?[10 = fixed_string(2248049116, 'U32')$",
  "?[10 = fixed_string(2248049116, 'U32')$",
  "$10 * fixed_string(19652890, 'utf32')",
  "=10 * fixed_string(19652890, 'utf32')",
  "$[10 * fixed_string(19652890, 'utf32'])",
  "xxx[10 * fixed_string(19652890, 'utf32'])",
  "?$10 * fixed_string(19652890, 'utf32'])",
  "?=10 * fixed_string(19652890, 'utf32'])",
  "?[10 * fixed_string(19652890, 'utf32')$",
  "?[10 * fixed_string(19652890, 'utf32')$",
  "$fixed_string(19652890, 'utf32')",
  "=fixed_string(19652890, 'utf32')",
  "$[fixed_string(19652890, 'utf32'])",
  "xxx[fixed_string(19652890, 'utf32'])",
  "?$fixed_string(19652890, 'utf32'])",
  "?=fixed_string(19652890, 'utf32'])",
  "?[fixed_string(19652890, 'utf32')$",
  "?[fixed_string(19652890, 'utf32')$",
  "10 $ fixed_string(19652890, 'utf32')",
  "?10 $ fixed_string(19652890, 'utf32')",
  "?(10 $ fixed_string(19652890, 'utf32'))",
  "$10 $ fixed_string(19652890, 'utf32')",
  "=10 $ fixed_string(19652890, 'utf32')",
  "$[10 $ fixed_string(19652890, 'utf32'])",
  "xxx[10 $ fixed_string(19652890, 'utf32'])",
  "?$10 $ fixed_string(19652890, 'utf32'])",
  "?=10 $ fixed_string(19652890, 'utf32'])",
  "?[10 $ fixed_string(19652890, 'utf32')$",
  "?[10 $ fixed_string(19652890, 'utf32')$",
  "10 = fixed_string(19652890, 'utf32')",
  "?10 = fixed_string(19652890, 'utf32')",
  "?(10 = fixed_string(19652890, 'utf32'))",
  "$10 = fixed_string(19652890, 'utf32')",
  "=10 = fixed_string(19652890, 'utf32')",
  "$[10 = fixed_string(19652890, 'utf32'])",
  "xxx[10 = fixed_string(19652890, 'utf32'])",
  "?$10 = fixed_string(19652890, 'utf32'])",
  "?=10 = fixed_string(19652890, 'utf32'])",
  "?[10 = fixed_string(19652890, 'utf32')$",
  "?[10 = fixed_string(19652890, 'utf32')$",
  "$10 * fixed_string(1199770379, 'utf-32')",
  "=10 * fixed_string(1199770379, 'utf-32')",
  "$[10 * fixed_string(1199770379, 'utf-32'])",
  "xxx[10 * fixed_string(1199770379, 'utf-32'])",
  "?$10 * fixed_string(1199770379, 'utf-32'])",
  "?=10 * fixed_string(1199770379, 'utf-32'])",
  "?[10 * fixed_string(1199770379, 'utf-32')$",
  "?[10 * fixed_string(1199770379, 'utf-32')$",
  "$fixed_string(1199770379, 'utf-32')",
  "=fixed_string(1199770379, 'utf-32')",
  "$[fixed_string(1199770379, 'utf-32'])",
  "xxx[fixed_string(1199770379, 'utf-32'])",
  "?$fixed_string(1199770379, 'utf-32'])",
  "?=fixed_string(1199770379, 'utf-32'])",
  "?[fixed_string(1199770379, 'utf-32')$",
  "?[fixed_string(1199770379, 'utf-32')$",
  "10 $ fixed_string(1199770379, 'utf-32')",
  "?10 $ fixed_string(1199770379, 'utf-32')",
  "?(10 $ fixed_string(1199770379, 'utf-32'))",
  "$10 $ fixed_string(1199770379, 'utf-32')",
  "=10 $ fixed_string(1199770379, 'utf-32')",
  "$[10 $ fixed_string(1199770379, 'utf-32'])",
  "xxx[10 $ fixed_string(1199770379, 'utf-32'])",
  "?$10 $ fixed_string(1199770379, 'utf-32'])",
  "?=10 $ fixed_string(1199770379, 'utf-32'])",
  "?[10 $ fixed_string(1199770379, 'utf-32')$",
  "?[10 $ fixed_string(1199770379, 'utf-32')$",
  "10 = fixed_string(1199770379, 'utf-32')",
  "?10 = fixed_string(1199770379, 'utf-32')",
  "?(10 = fixed_string(1199770379, 'utf-32'))",
  "$10 = fixed_string(1199770379, 'utf-32')",
  "=10 = fixed_string(1199770379, 'utf-32')",
  "$[10 = fixed_string(1199770379, 'utf-32'])",
  "xxx[10 = fixed_string(1199770379, 'utf-32'])",
  "?$10 = fixed_string(1199770379, 'utf-32'])",
  "?=10 = fixed_string(1199770379, 'utf-32'])",
  "?[10 = fixed_string(1199770379, 'utf-32')$",
  "?[10 = fixed_string(1199770379, 'utf-32')$",
  "$10 * fixed_string(3528885699, 'ucs2')",
  "=10 * fixed_string(3528885699, 'ucs2')",
  "$[10 * fixed_string(3528885699, 'ucs2'])",
  "xxx[10 * fixed_string(3528885699, 'ucs2'])",
  "?$10 * fixed_string(3528885699, 'ucs2'])",
  "?=10 * fixed_string(3528885699, 'ucs2'])",
  "?[10 * fixed_string(3528885699, 'ucs2')$",
  "?[10 * fixed_string(3528885699, 'ucs2')$",
  "$fixed_string(3528885699, 'ucs2')",
  "=fixed_string(3528885699, 'ucs2')",
  "$[fixed_string(3528885699, 'ucs2'])",
  "xxx[fixed_string(3528885699, 'ucs2'])",
  "?$fixed_string(3528885699, 'ucs2'])",
  "?=fixed_string(3528885699, 'ucs2'])",
  "?[fixed_string(3528885699, 'ucs2')$",
  "?[fixed_string(3528885699, 'ucs2')$",
  "10 $ fixed_string(3528885699, 'ucs2')",
  "?10 $ fixed_string(3528885699, 'ucs2')",
  "?(10 $ fixed_string(3528885699, 'ucs2'))",
  "$10 $ fixed_string(3528885699, 'ucs2')",
  "=10 $ fixed_string(3528885699, 'ucs2')",
  "$[10 $ fixed_string(3528885699, 'ucs2'])",
  "xxx[10 $ fixed_string(3528885699, 'ucs2'])",
  "?$10 $ fixed_string(3528885699, 'ucs2'])",
  "?=10 $ fixed_string(3528885699, 'ucs2'])",
  "?[10 $ fixed_string(3528885699, 'ucs2')$",
  "?[10 $ fixed_string(3528885699, 'ucs2')$",
  "10 = fixed_string(3528885699, 'ucs2')",
  "?10 = fixed_string(3528885699, 'ucs2')",
  "?(10 = fixed_string(3528885699, 'ucs2'))",
  "$10 = fixed_string(3528885699, 'ucs2')",
  "=10 = fixed_string(3528885699, 'ucs2')",
  "$[10 = fixed_string(3528885699, 'ucs2'])",
  "xxx[10 = fixed_string(3528885699, 'ucs2'])",
  "?$10 = fixed_string(3528885699, 'ucs2'])",
  "?=10 = fixed_string(3528885699, 'ucs2'])",
  "?[10 = fixed_string(3528885699, 'ucs2')$",
  "?[10 = fixed_string(3528885699, 'ucs2')$",
  "$10 * fixed_string(402273181, 'ucs-2')",
  "=10 * fixed_string(402273181, 'ucs-2')",
  "$[10 * fixed_string(402273181, 'ucs-2'])",
  "xxx[10 * fixed_string(402273181, 'ucs-2'])",
  "?$10 * fixed_string(402273181, 'ucs-2'])",
  "?=10 * fixed_string(402273181, 'ucs-2'])",
  "?[10 * fixed_string(402273181, 'ucs-2')$",
  "?[10 * fixed_string(402273181, 'ucs-2')$",
  "$fixed_string(402273181, 'ucs-2')",
  "=fixed_string(402273181, 'ucs-2')",
  "$[fixed_string(402273181, 'ucs-2'])",
  "xxx[fixed_string(402273181, 'ucs-2'])",
  "?$fixed_string(402273181, 'ucs-2'])",
  "?=fixed_string(402273181, 'ucs-2'])",
  "?[fixed_string(402273181, 'ucs-2')$",
  "?[fixed_string(402273181, 'ucs-2')$",
  "10 $ fixed_string(402273181, 'ucs-2')",
  "?10 $ fixed_string(402273181, 'ucs-2')",
  "?(10 $ fixed_string(402273181, 'ucs-2'))",
  "$10 $ fixed_string(402273181, 'ucs-2')",
  "=10 $ fixed_string(402273181, 'ucs-2')",
  "$[10 $ fixed_string(402273181, 'ucs-2'])",
  "xxx[10 $ fixed_string(402273181, 'ucs-2'])",
  "?$10 $ fixed_string(402273181, 'ucs-2'])",
  "?=10 $ fixed_string(402273181, 'ucs-2'])",
  "?[10 $ fixed_string(402273181, 'ucs-2')$",
  "?[10 $ fixed_string(402273181, 'ucs-2')$",
  "10 = fixed_string(402273181, 'ucs-2')",
  "?10 = fixed_string(402273181, 'ucs-2')",
  "?(10 = fixed_string(402273181, 'ucs-2'))",
  "$10 = fixed_string(402273181, 'ucs-2')",
  "=10 = fixed_string(402273181, 'ucs-2')",
  "$[10 = fixed_string(402273181, 'ucs-2'])",
  "xxx[10 = fixed_string(402273181, 'ucs-2'])",
  "?$10 = fixed_string(402273181, 'ucs-2'])",
  "?=10 = fixed_string(402273181, 'ucs-2'])",
  "?[10 = fixed_string(402273181, 'ucs-2')$",
  "?[10 = fixed_string(402273181, 'ucs-2')$",
  "$10 * fixed_string(664657071, 'ucs_2')",
  "=10 * fixed_string(664657071, 'ucs_2')",
  "$[10 * fixed_string(664657071, 'ucs_2'])",
  "xxx[10 * fixed_string(664657071, 'ucs_2'])",
  "?$10 * fixed_string(664657071, 'ucs_2'])",
  "?=10 * fixed_string(664657071, 'ucs_2'])",
  "?[10 * fixed_string(664657071, 'ucs_2')$",
  "?[10 * fixed_string(664657071, 'ucs_2')$",
  "$fixed_string(664657071, 'ucs_2')",
  "=fixed_string(664657071, 'ucs_2')",
  "$[fixed_string(664657071, 'ucs_2'])",
  "xxx[fixed_string(664657071, 'ucs_2'])",
  "?$fixed_string(664657071, 'ucs_2'])",
  "?=fixed_string(664657071, 'ucs_2'])",
  "?[fixed_string(664657071, 'ucs_2')$",
  "?[fixed_string(664657071, 'ucs_2')$",
  "10 $ fixed_string(664657071, 'ucs_2')",
  "?10 $ fixed_string(664657071, 'ucs_2')",
  "?(10 $ fixed_string(664657071, 'ucs_2'))",
  "$10 $ fixed_string(664657071, 'ucs_2')",
  "=10 $ fixed_string(664657071, 'ucs_2')",
  "$[10 $ fixed_string(664657071, 'ucs_2'])",
  "xxx[10 $ fixed_string(664657071, 'ucs_2'])",
  "?$10 $ fixed_string(664657071, 'ucs_2'])",
  "?=10 $ fixed_string(664657071, 'ucs_2'])",
  "?[10 $ fixed_string(664657071, 'ucs_2')$",
  "?[10 $ fixed_string(664657071, 'ucs_2')$",
  "10 = fixed_string(664657071, 'ucs_2')",
  "?10 = fixed_string(664657071, 'ucs_2')",
  "?(10 = fixed_string(664657071, 'ucs_2'))",
  "$10 = fixed_string(664657071, 'ucs_2')",
  "=10 = fixed_string(664657071, 'ucs_2')",
  "$[10 = fixed_string(664657071, 'ucs_2'])",
  "xxx[10 = fixed_string(664657071, 'ucs_2'])",
  "?$10 = fixed_string(664657071, 'ucs_2'])",
  "?=10 = fixed_string(664657071, 'ucs_2'])",
  "?[10 = fixed_string(664657071, 'ucs_2')$",
  "?[10 = fixed_string(664657071, 'ucs_2')$",
  "10 * $[519876065, 'ucs_2']",
  "?10 * $[519876065, 'ucs_2']",
  "?[10 * $[519876065, 'ucs_2']]",
  "$10 * $[519876065, 'ucs_2']",
  "=10 * $[519876065, 'ucs_2']",
  "$[10 * $[519876065, 'ucs_2']]",
  "xxx[10 * $[519876065, 'ucs_2']]",
  "?$10 * $[519876065, 'ucs_2']]",
  "?=10 * $[519876065, 'ucs_2']]",
  "?[10 * $[519876065, 'ucs_2']$",
  "?[10 * $[519876065, 'ucs_2']$",
  "$[519876065, 'ucs_2']",
  "?$[519876065, 'ucs_2']",
  "?[$[519876065, 'ucs_2']]",
  "$$[519876065, 'ucs_2']",
  "=$[519876065, 'ucs_2']",
  "$[$[519876065, 'ucs_2']]",
  "xxx[$[519876065, 'ucs_2']]",
  "?$$[519876065, 'ucs_2']]",
  "?=$[519876065, 'ucs_2']]",
  "?[$[519876065, 'ucs_2']$",
  "?[$[519876065, 'ucs_2']$",
  "10 $ $[519876065, 'ucs_2']",
  "?10 $ $[519876065, 'ucs_2']",
  "?[10 $ $[519876065, 'ucs_2']]",
  "$10 $ $[519876065, 'ucs_2']",
  "=10 $ $[519876065, 'ucs_2']",
  "$[10 $ $[519876065, 'ucs_2']]",
  "xxx[10 $ $[519876065, 'ucs_2']]",
  "?$10 $ $[519876065, 'ucs_2']]",
  "?=10 $ $[519876065, 'ucs_2']]",
  "?[10 $ $[519876065, 'ucs_2']$",
  "?[10 $ $[519876065, 'ucs_2']$",
  "10 = $[519876065, 'ucs_2']",
  "?10 = $[519876065, 'ucs_2']",
  "?[10 = $[519876065, 'ucs_2']]",
  "$10 = $[519876065, 'ucs_2']",
  "=10 = $[519876065, 'ucs_2']",
  "$[10 = $[519876065, 'ucs_2']]",
  "xxx[10 = $[519876065, 'ucs_2']]",
  "?$10 = $[519876065, 'ucs_2']]",
  "?=10 = $[519876065, 'ucs_2']]",
  "?[10 = $[519876065, 'ucs_2']$",
  "?[10 = $[519876065, 'ucs_2']$",
  "10 * xxx[3860868500, 'ucs_2']",
  "?10 * xxx[3860868500, 'ucs_2']",
  "?[10 * xxx[3860868500, 'ucs_2']]",
  "$10 * xxx[3860868500, 'ucs_2']",
  "=10 * xxx[3860868500, 'ucs_2']",
  "$[10 * xxx[3860868500, 'ucs_2']]",
  "xxx[10 * xxx[3860868500, 'ucs_2']]",
  "?$10 * xxx[3860868500, 'ucs_2']]",
  "?=10 * xxx[3860868500, 'ucs_2']]",
  "?[10 * xxx[3860868500, 'ucs_2']$",
  "?[10 * xxx[3860868500, 'ucs_2']$",
  "xxx[3860868500, 'ucs_2']",
  "?xxx[3860868500, 'ucs_2']",
  "?[xxx[3860868500, 'ucs_2']]",
  "$xxx[3860868500, 'ucs_2']",
  "=xxx[3860868500, 'ucs_2']",
  "$[xxx[3860868500, 'ucs_2']]",
  "xxx[xxx[3860868500, 'ucs_2']]",
  "?$xxx[3860868500, 'ucs_2']]",
  "?=xxx[3860868500, 'ucs_2']]",
  "?[xxx[3860868500, 'ucs_2']$",
  "?[xxx[3860868500, 'ucs_2']$",
  "10 $ xxx[3860868500, 'ucs_2']",
  "?10 $ xxx[3860868500, 'ucs_2']",
  "?[10 $ xxx[3860868500, 'ucs_2']]",
  "$10 $ xxx[3860868500, 'ucs_2']",
  "=10 $ xxx[3860868500, 'ucs_2']",
  "$[10 $ xxx[3860868500, 'ucs_2']]",
  "xxx[10 $ xxx[3860868500, 'ucs_2']]",
  "?$10 $ xxx[3860868500, 'ucs_2']]",
  "?=10 $ xxx[3860868500, 'ucs_2']]",
  "?[10 $ xxx[3860868500, 'ucs_2']$",
  "?[10 $ xxx[3860868500, 'ucs_2']$",
  "10 = xxx[3860868500, 'ucs_2']",
  "?10 = xxx[3860868500, 'ucs_2']",
  "?[10 = xxx[3860868500, 'ucs_2']]",
  "$10 = xxx[3860868500, 'ucs_2']",
  "=10 = xxx[3860868500, 'ucs_2']",
  "$[10 = xxx[3860868500, 'ucs_2']]",
  "xxx[10 = xxx[3860868500, 'ucs_2']]",
  "?$10 = xxx[3860868500, 'ucs_2']]",
  "?=10 = xxx[3860868500, 'ucs_2']]",
  "?[10 = xxx[3860868500, 'ucs_2']$",
  "?[10 = xxx[3860868500, 'ucs_2']$",
  "10 * fixed_string$1672451972, 'ucs_2']",
  "?10 * fixed_string$1672451972, 'ucs_2']",
  "?(10 * fixed_string$1672451972, 'ucs_2'])",
  "$10 * fixed_string$1672451972, 'ucs_2']",
  "=10 * fixed_string$1672451972, 'ucs_2']",
  "$[10 * fixed_string$1672451972, 'ucs_2']]",
  "xxx[10 * fixed_string$1672451972, 'ucs_2']]",
  "?$10 * fixed_string$1672451972, 'ucs_2']]",
  "?=10 * fixed_string$1672451972, 'ucs_2']]",
  "?(10 * fixed_string$1672451972, 'ucs_2')$",
  "?(10 * fixed_string$1672451972, 'ucs_2')$",
  "fixed_string$1672451972, 'ucs_2']",
  "?fixed_string$1672451972, 'ucs_2']",
  "?(fixed_string$1672451972, 'ucs_2'])",
  "$fixed_string$1672451972, 'ucs_2']",
  "=fixed_string$1672451972, 'ucs_2']",
  "$[fixed_string$1672451972, 'ucs_2']]",
  "xxx[fixed_string$1672451972, 'ucs_2']]",
  "?$fixed_string$1672451972, 'ucs_2']]",
  "?=fixed_string$1672451972, 'ucs_2']]",
  "?(fixed_string$1672451972, 'ucs_2')$",
  "?(fixed_string$1672451972, 'ucs_2')$",
  "10 $ fixed_string$1672451972, 'ucs_2']",
  "?10 $ fixed_string$1672451972, 'ucs_2']",
  "?(10 $ fixed_string$1672451972, 'ucs_2'])",
  "$10 $ fixed_string$1672451972, 'ucs_2']",
  "=10 $ fixed_string$1672451972, 'ucs_2']",
  "$[10 $ fixed_string$1672451972, 'ucs_2']]",
  "xxx[10 $ fixed_string$1672451972, 'ucs_2']]",
  "?$10 $ fixed_string$1672451972, 'ucs_2']]",
  "?=10 $ fixed_string$1672451972, 'ucs_2']]",
  "?(10 $ fixed_string$1672451972, 'ucs_2')$",
  "?(10 $ fixed_string$1672451972, 'ucs_2')$",
  "10 = fixed_string$1672451972, 'ucs_2']",
  "?10 = fixed_string$1672451972, 'ucs_2']",
  "?(10 = fixed_string$1672451972, 'ucs_2'])",
  "$10 = fixed_string$1672451972, 'ucs_2']",
  "=10 = fixed_string$1672451972, 'ucs_2']",
  "$[10 = fixed_string$1672451972, 'ucs_2']]",
  "xxx[10 = fixed_string$1672451972, 'ucs_2']]",
  "?$10 = fixed_string$1672451972, 'ucs_2']]",
  "?=10 = fixed_string$1672451972, 'ucs_2']]",
  "?(10 = fixed_string$1672451972, 'ucs_2')$",
  "?(10 = fixed_string$1672451972, 'ucs_2')$",
  "10 * fixed_string=2587803306, 'ucs_2']",
  "?10 * fixed_string=2587803306, 'ucs_2']",
  "?(10 * fixed_string=2587803306, 'ucs_2'])",
  "$10 * fixed_string=2587803306, 'ucs_2']",
  "=10 * fixed_string=2587803306, 'ucs_2']",
  "$[10 * fixed_string=2587803306, 'ucs_2']]",
  "xxx[10 * fixed_string=2587803306, 'ucs_2']]",
  "?$10 * fixed_string=2587803306, 'ucs_2']]",
  "?=10 * fixed_string=2587803306, 'ucs_2']]",
  "?(10 * fixed_string=2587803306, 'ucs_2')$",
  "?(10 * fixed_string=2587803306, 'ucs_2')$",
  "fixed_string=2587803306, 'ucs_2']",
  "?fixed_string=2587803306, 'ucs_2']",
  "?(fixed_string=2587803306, 'ucs_2'])",
  "$fixed_string=2587803306, 'ucs_2']",
  "=fixed_string=2587803306, 'ucs_2']",
  "$[fixed_string=2587803306, 'ucs_2']]",
  "xxx[fixed_string=2587803306, 'ucs_2']]",
  "?$fixed_string=2587803306, 'ucs_2']]",
  "?=fixed_string=2587803306, 'ucs_2']]",
  "?(fixed_string=2587803306, 'ucs_2')$",
  "?(fixed_string=2587803306, 'ucs_2')$",
  "10 $ fixed_string=2587803306, 'ucs_2']",
  "?10 $ fixed_string=2587803306, 'ucs_2']",
  "?(10 $ fixed_string=2587803306, 'ucs_2'])",
  "$10 $ fixed_string=2587803306, 'ucs_2']",
  "=10 $ fixed_string=2587803306, 'ucs_2']",
  "$[10 $ fixed_string=2587803306, 'ucs_2']]",
  "xxx[10 $ fixed_string=2587803306, 'ucs_2']]",
  "?$10 $ fixed_string=2587803306, 'ucs_2']]",
  "?=10 $ fixed_string=2587803306, 'ucs_2']]",
  "?(10 $ fixed_string=2587803306, 'ucs_2')$",
  "?(10 $ fixed_string=2587803306, 'ucs_2')$",
  "10 = fixed_string=2587803306, 'ucs_2']",
  "?10 = fixed_string=2587803306, 'ucs_2']",
  "?(10 = fixed_string=2587803306, 'ucs_2'])",
  "$10 = fixed_string=2587803306, 'ucs_2']",
  "=10 = fixed_string=2587803306, 'ucs_2']",
  "$[10 = fixed_string=2587803306, 'ucs_2']]",
  "xxx[10 = fixed_string=2587803306, 'ucs_2']]",
  "?$10 = fixed_string=2587803306, 'ucs_2']]",
  "?=10 = fixed_string=2587803306, 'ucs_2']]",
  "?(10 = fixed_string=2587803306, 'ucs_2')$",
  "?(10 = fixed_string=2587803306, 'ucs_2')$",
  "10 * fixed_string($, 'ucs_2')",
  "?10 * fixed_string($, 'ucs_2')",
  "?(10 * fixed_string($, 'ucs_2'))",
  "$10 * fixed_string($, 'ucs_2')",
  "=10 * fixed_string($, 'ucs_2')",
  "$[10 * fixed_string($, 'ucs_2'])",
  "xxx[10 * fixed_string($, 'ucs_2'])",
  "?$10 * fixed_string($, 'ucs_2'])",
  "?=10 * fixed_string($, 'ucs_2'])",
  "?[10 * fixed_string($, 'ucs_2')$",
  "?[10 * fixed_string($, 'ucs_2')$",
  "fixed_string($, 'ucs_2')",
  "?fixed_string($, 'ucs_2')",
  "?(fixed_string($, 'ucs_2'))",
  "$fixed_string($, 'ucs_2')",
  "=fixed_string($, 'ucs_2')",
  "$[fixed_string($, 'ucs_2'])",
  "xxx[fixed_string($, 'ucs_2'])",
  "?$fixed_string($, 'ucs_2'])",
  "?=fixed_string($, 'ucs_2'])",
  "?[fixed_string($, 'ucs_2')$",
  "?[fixed_string($, 'ucs_2')$",
  "10 $ fixed_string($, 'ucs_2')",
  "?10 $ fixed_string($, 'ucs_2')",
  "?(10 $ fixed_string($, 'ucs_2'))",
  "$10 $ fixed_string($, 'ucs_2')",
  "=10 $ fixed_string($, 'ucs_2')",
  "$[10 $ fixed_string($, 'ucs_2'])",
  "xxx[10 $ fixed_string($, 'ucs_2'])",
  "?$10 $ fixed_string($, 'ucs_2'])",
  "?=10 $ fixed_string($, 'ucs_2'])",
  "?[10 $ fixed_string($, 'ucs_2')$",
  "?[10 $ fixed_string($, 'ucs_2')$",
  "10 = fixed_string($, 'ucs_2')",
  "?10 = fixed_string($, 'ucs_2')",
  "?(10 = fixed_string($, 'ucs_2'))",
  "$10 = fixed_string($, 'ucs_2')",
  "=10 = fixed_string($, 'ucs_2')",
  "$[10 = fixed_string($, 'ucs_2'])",
  "xxx[10 = fixed_string($, 'ucs_2'])",
  "?$10 = fixed_string($, 'ucs_2'])",
  "?=10 = fixed_string($, 'ucs_2'])",
  "?[10 = fixed_string($, 'ucs_2')$",
  "?[10 = fixed_string($, 'ucs_2')$",
  "10 * fixed_string('', 'ucs_2')",
  "?10 * fixed_string('', 'ucs_2')",
  "?(10 * fixed_string('', 'ucs_2'))",
  "$10 * fixed_string('', 'ucs_2')",
  "=10 * fixed_string('', 'ucs_2')",
  "$[10 * fixed_string('', 'ucs_2'])",
  "xxx[10 * fixed_string('', 'ucs_2'])",
  "?$10 * fixed_string('', 'ucs_2'])",
  "?=10 * fixed_string('', 'ucs_2'])",
  "?[10 * fixed_string('', 'ucs_2')$",
  "?[10 * fixed_string('', 'ucs_2')$",
  "fixed_string('', 'ucs_2')",
  "?fixed_string('', 'ucs_2')",
  "?(fixed_string('', 'ucs_2'))",
  "$fixed_string('', 'ucs_2')",
  "=fixed_string('', 'ucs_2')",
  "$[fixed_string('', 'ucs_2'])",
  "xxx[fixed_string('', 'ucs_2'])",
  "?$fixed_string('', 'ucs_2'])",
  "?=fixed_string('', 'ucs_2'])",
  "?[fixed_string('', 'ucs_2')$",
  "?[fixed_string('', 'ucs_2')$",
  "10 $ fixed_string('', 'ucs_2')",
  "?10 $ fixed_string('', 'ucs_2')",
  "?(10 $ fixed_string('', 'ucs_2'))",
  "$10 $ fixed_string('', 'ucs_2')",
  "=10 $ fixed_string('', 'ucs_2')",
  "$[10 $ fixed_string('', 'ucs_2'])",
  "xxx[10 $ fixed_string('', 'ucs_2'])",
  "?$10 $ fixed_string('', 'ucs_2'])",
  "?=10 $ fixed_string('', 'ucs_2'])",
  "?[10 $ fixed_string('', 'ucs_2')$",
  "?[10 $ fixed_string('', 'ucs_2')$",
  "10 = fixed_string('', 'ucs_2')",
  "?10 = fixed_string('', 'ucs_2')",
  "?(10 = fixed_string('', 'ucs_2'))",
  "$10 = fixed_string('', 'ucs_2')",
  "=10 = fixed_string('', 'ucs_2')",
  "$[10 = fixed_string('', 'ucs_2'])",
  "xxx[10 = fixed_string('', 'ucs_2'])",
  "?$10 = fixed_string('', 'ucs_2'])",
  "?=10 = fixed_string('', 'ucs_2'])",
  "?[10 = fixed_string('', 'ucs_2')$",
  "?[10 = fixed_string('', 'ucs_2')$",
  "10 * fixed_string('xxx', 'ucs_2')",
  "?10 * fixed_string('xxx', 'ucs_2')",
  "?(10 * fixed_string('xxx', 'ucs_2'))",
  "$10 * fixed_string('xxx', 'ucs_2')",
  "=10 * fixed_string('xxx', 'ucs_2')",
  "$[10 * fixed_string('xxx', 'ucs_2'])",
  "xxx[10 * fixed_string('xxx', 'ucs_2'])",
  "?$10 * fixed_string('xxx', 'ucs_2'])",
  "?=10 * fixed_string('xxx', 'ucs_2'])",
  "?[10 * fixed_string('xxx', 'ucs_2')$",
  "?[10 * fixed_string('xxx', 'ucs_2')$",
  "fixed_string('xxx', 'ucs_2')",
  "?fixed_string('xxx', 'ucs_2')",
  "?(fixed_string('xxx', 'ucs_2'))",
  "$fixed_string('xxx', 'ucs_2')",
  "=fixed_string('xxx', 'ucs_2')",
  "$[fixed_string('xxx', 'ucs_2'])",
  "xxx[fixed_string('xxx', 'ucs_2'])",
  "?$fixed_string('xxx', 'ucs_2'])",
  "?=fixed_string('xxx', 'ucs_2'])",
  "?[fixed_string('xxx', 'ucs_2')$",
  "?[fixed_string('xxx', 'ucs_2')$",
  "10 $ fixed_string('xxx', 'ucs_2')",
  "?10 $ fixed_string('xxx', 'ucs_2')",
  "?(10 $ fixed_string('xxx', 'ucs_2'))",
  "$10 $ fixed_string('xxx', 'ucs_2')",
  "=10 $ fixed_string('xxx', 'ucs_2')",
  "$[10 $ fixed_string('xxx', 'ucs_2'])",
  "xxx[10 $ fixed_string('xxx', 'ucs_2'])",
  "?$10 $ fixed_string('xxx', 'ucs_2'])",
  "?=10 $ fixed_string('xxx', 'ucs_2'])",
  "?[10 $ fixed_string('xxx', 'ucs_2')$",
  "?[10 $ fixed_string('xxx', 'ucs_2')$",
  "10 = fixed_string('xxx', 'ucs_2')",
  "?10 = fixed_string('xxx', 'ucs_2')",
  "?(10 = fixed_string('xxx', 'ucs_2'))",
  "$10 = fixed_string('xxx', 'ucs_2')",
  "=10 = fixed_string('xxx', 'ucs_2')",
  "$[10 = fixed_string('xxx', 'ucs_2'])",
  "xxx[10 = fixed_string('xxx', 'ucs_2'])",
  "?$10 = fixed_string('xxx', 'ucs_2'])",
  "?=10 = fixed_string('xxx', 'ucs_2'])",
  "?[10 = fixed_string('xxx', 'ucs_2')$",
  "?[10 = fixed_string('xxx', 'ucs_2')$",
  "10 * fixed_string(1$ 'ucs_2')",
  "?10 * fixed_string(1$ 'ucs_2')",
  "?(10 * fixed_string(1$ 'ucs_2'))",
  "$10 * fixed_string(1$ 'ucs_2')",
  "=10 * fixed_string(1$ 'ucs_2')",
  "$[10 * fixed_string(1$ 'ucs_2'])",
  "xxx[10 * fixed_string(1$ 'ucs_2'])",
  "?$10 * fixed_string(1$ 'ucs_2'])",
  "?=10 * fixed_string(1$ 'ucs_2'])",
  "?[10 * fixed_string(1$ 'ucs_2')$",
  "?[10 * fixed_string(1$ 'ucs_2')$",
  "fixed_string(1$ 'ucs_2')",
  "?fixed_string(1$ 'ucs_2')",
  "?(fixed_string(1$ 'ucs_2'))",
  "$fixed_string(1$ 'ucs_2')",
  "=fixed_string(1$ 'ucs_2')",
  "$[fixed_string(1$ 'ucs_2'])",
  "xxx[fixed_string(1$ 'ucs_2'])",
  "?$fixed_string(1$ 'ucs_2'])",
  "?=fixed_string(1$ 'ucs_2'])",
  "?[fixed_string(1$ 'ucs_2')$",
  "?[fixed_string(1$ 'ucs_2')$",
  "10 $ fixed_string(1$ 'ucs_2')",
  "?10 $ fixed_string(1$ 'ucs_2')",
  "?(10 $ fixed_string(1$ 'ucs_2'))",
  "$10 $ fixed_string(1$ 'ucs_2')",
  "=10 $ fixed_string(1$ 'ucs_2')",
  "$[10 $ fixed_string(1$ 'ucs_2'])",
  "xxx[10 $ fixed_string(1$ 'ucs_2'])",
  "?$10 $ fixed_string(1$ 'ucs_2'])",
  "?=10 $ fixed_string(1$ 'ucs_2'])",
  "?[10 $ fixed_string(1$ 'ucs_2')$",
  "?[10 $ fixed_string(1$ 'ucs_2')$",
  "10 = fixed_string(1$ 'ucs_2')",
  "?10 = fixed_string(1$ 'ucs_2')",
  "?(10 = fixed_string(1$ 'ucs_2'))",
  "$10 = fixed_string(1$ 'ucs_2')",
  "=10 = fixed_string(1$ 'ucs_2')",
  "$[10 = fixed_string(1$ 'ucs_2'])",
  "xxx[10 = fixed_string(1$ 'ucs_2'])",
  "?$10 = fixed_string(1$ 'ucs_2'])",
  "?=10 = fixed_string(1$ 'ucs_2'])",
  "?[10 = fixed_string(1$ 'ucs_2')$",
  "?[10 = fixed_string(1$ 'ucs_2')$",
  "10 * fixed_string(1::: 'ucs_2')",
  "?10 * fixed_string(1::: 'ucs_2')",
  "?(10 * fixed_string(1::: 'ucs_2'))",
  "$10 * fixed_string(1::: 'ucs_2')",
  "=10 * fixed_string(1::: 'ucs_2')",
  "$[10 * fixed_string(1::: 'ucs_2'])",
  "xxx[10 * fixed_string(1::: 'ucs_2'])",
  "?$10 * fixed_string(1::: 'ucs_2'])",
  "?=10 * fixed_string(1::: 'ucs_2'])",
  "?[10 * fixed_string(1::: 'ucs_2')$",
  "?[10 * fixed_string(1::: 'ucs_2')$",
  "fixed_string(1::: 'ucs_2')",
  "?fixed_string(1::: 'ucs_2')",
  "?(fixed_string(1::: 'ucs_2'))",
  "$fixed_string(1::: 'ucs_2')",
  "=fixed_string(1::: 'ucs_2')",
  "$[fixed_string(1::: 'ucs_2'])",
  "xxx[fixed_string(1::: 'ucs_2'])",
  "?$fixed_string(1::: 'ucs_2'])",
  "?=fixed_string(1::: 'ucs_2'])",
  "?[fixed_string(1::: 'ucs_2')$",
  "?[fixed_string(1::: 'ucs_2')$",
  "10 $ fixed_string(1::: 'ucs_2')",
  "?10 $ fixed_string(1::: 'ucs_2')",
  "?(10 $ fixed_string(1::: 'ucs_2'))",
  "$10 $ fixed_string(1::: 'ucs_2')",
  "=10 $ fixed_string(1::: 'ucs_2')",
  "$[10 $ fixed_string(1::: 'ucs_2'])",
  "xxx[10 $ fixed_string(1::: 'ucs_2'])",
  "?$10 $ fixed_string(1::: 'ucs_2'])",
  "?=10 $ fixed_string(1::: 'ucs_2'])",
  "?[10 $ fixed_string(1::: 'ucs_2')$",
  "?[10 $ fixed_string(1::: 'ucs_2')$",
  "10 = fixed_string(1::: 'ucs_2')",
  "?10 = fixed_string(1::: 'ucs_2')",
  "?(10 = fixed_string(1::: 'ucs_2'))",
  "$10 = fixed_string(1::: 'ucs_2')",
  "=10 = fixed_string(1::: 'ucs_2')",
  "$[10 = fixed_string(1::: 'ucs_2'])",
  "xxx[10 = fixed_string(1::: 'ucs_2'])",
  "?$10 = fixed_string(1::: 'ucs_2'])",
  "?=10 = fixed_string(1::: 'ucs_2'])",
  "?[10 = fixed_string(1::: 'ucs_2')$",
  "?[10 = fixed_string(1::: 'ucs_2')$",
  "10 * fixed_string(1, $)",
  "?10 * fixed_string(1, $)",
  "?(10 * fixed_string(1, $))",
  "$10 * fixed_string(1, $)",
  "=10 * fixed_string(1, $)",
  "$[10 * fixed_string(1, $])",
  "xxx[10 * fixed_string(1, $])",
  "?$10 * fixed_string(1, $])",
  "?=10 * fixed_string(1, $])",
  "?[10 * fixed_string(1, $)$",
  "?[10 * fixed_string(1, $)$",
  "fixed_string(1, $)",
  "?fixed_string(1, $)",
  "?(fixed_string(1, $))",
  "$fixed_string(1, $)",
  "=fixed_string(1, $)",
  "$[fixed_string(1, $])",
  "xxx[fixed_string(1, $])",
  "?$fixed_string(1, $])",
  "?=fixed_string(1, $])",
  "?[fixed_string(1, $)$",
  "?[fixed_string(1, $)$",
  "10 $ fixed_string(1, $)",
  "?10 $ fixed_string(1, $)",
  "?(10 $ fixed_string(1, $))",
  "$10 $ fixed_string(1, $)",
  "=10 $ fixed_string(1, $)",
  "$[10 $ fixed_string(1, $])",
  "xxx[10 $ fixed_string(1, $])",
  "?$10 $ fixed_string(1, $])",
  "?=10 $ fixed_string(1, $])",
  "?[10 $ fixed_string(1, $)$",
  "?[10 $ fixed_string(1, $)$",
  "10 = fixed_string(1, $)",
  "?10 = fixed_string(1, $)",
  "?(10 = fixed_string(1, $))",
  "$10 = fixed_string(1, $)",
  "=10 = fixed_string(1, $)",
  "$[10 = fixed_string(1, $])",
  "xxx[10 = fixed_string(1, $])",
  "?$10 = fixed_string(1, $])",
  "?=10 = fixed_string(1, $])",
  "?[10 = fixed_string(1, $)$",
  "?[10 = fixed_string(1, $)$",
  "10 * fixed_string(1, '')",
  "?10 * fixed_string(1, '')",
  "?(10 * fixed_string(1, ''))",
  "$10 * fixed_string(1, '')",
  "=10 * fixed_string(1, '')",
  "$[10 * fixed_string(1, ''])",
  "xxx[10 * fixed_string(1, ''])",
  "?$10 * fixed_string(1, ''])",
  "?=10 * fixed_string(1, ''])",
  "?[10 * fixed_string(1, '')$",
  "?[10 * fixed_string(1, '')$",
  "fixed_string(1, '')",
  "?fixed_string(1, '')",
  "?(fixed_string(1, ''))",
  "$fixed_string(1, '')",
  "=fixed_string(1, '')",
  "$[fixed_string(1, ''])",
  "xxx[fixed_string(1, ''])",
  "?$fixed_string(1, ''])",
  "?=fixed_string(1, ''])",
  "?[fixed_string(1, '')$",
  "?[fixed_string(1, '')$",
  "10 $ fixed_string(1, '')",
  "?10 $ fixed_string(1, '')",
  "?(10 $ fixed_string(1, ''))",
  "$10 $ fixed_string(1, '')",
  "=10 $ fixed_string(1, '')",
  "$[10 $ fixed_string(1, ''])",
  "xxx[10 $ fixed_string(1, ''])",
  "?$10 $ fixed_string(1, ''])",
  "?=10 $ fixed_string(1, ''])",
  "?[10 $ fixed_string(1, '')$",
  "?[10 $ fixed_string(1, '')$",
  "10 = fixed_string(1, '')",
  "?10 = fixed_string(1, '')",
  "?(10 = fixed_string(1, ''))",
  "$10 = fixed_string(1, '')",
  "=10 = fixed_string(1, '')",
  "$[10 = fixed_string(1, ''])",
  "xxx[10 = fixed_string(1, ''])",
  "?$10 = fixed_string(1, ''])",
  "?=10 = fixed_string(1, ''])",
  "?[10 = fixed_string(1, '')$",
  "?[10 = fixed_string(1, '')$",
  "10 * fixed_string(1, 'xxx')",
  "?10 * fixed_string(1, 'xxx')",
  "?(10 * fixed_string(1, 'xxx'))",
  "$10 * fixed_string(1, 'xxx')",
  "=10 * fixed_string(1, 'xxx')",
  "$[10 * fixed_string(1, 'xxx'])",
  "xxx[10 * fixed_string(1, 'xxx'])",
  "?$10 * fixed_string(1, 'xxx'])",
  "?=10 * fixed_string(1, 'xxx'])",
  "?[10 * fixed_string(1, 'xxx')$",
  "?[10 * fixed_string(1, 'xxx')$",
  "fixed_string(1, 'xxx')",
  "?fixed_string(1, 'xxx')",
  "?(fixed_string(1, 'xxx'))",
  "$fixed_string(1, 'xxx')",
  "=fixed_string(1, 'xxx')",
  "$[fixed_string(1, 'xxx'])",
  "xxx[fixed_string(1, 'xxx'])",
  "?$fixed_string(1, 'xxx'])",
  "?=fixed_string(1, 'xxx'])",
  "?[fixed_string(1, 'xxx')$",
  "?[fixed_string(1, 'xxx')$",
  "10 $ fixed_string(1, 'xxx')",
  "?10 $ fixed_string(1, 'xxx')",
  "?(10 $ fixed_string(1, 'xxx'))",
  "$10 $ fixed_string(1, 'xxx')",
  "=10 $ fixed_string(1, 'xxx')",
  "$[10 $ fixed_string(1, 'xxx'])",
  "xxx[10 $ fixed_string(1, 'xxx'])",
  "?$10 $ fixed_string(1, 'xxx'])",
  "?=10 $ fixed_string(1, 'xxx'])",
  "?[10 $ fixed_string(1, 'xxx')$",
  "?[10 $ fixed_string(1, 'xxx')$",
  "10 = fixed_string(1, 'xxx')",
  "?10 = fixed_string(1, 'xxx')",
  "?(10 = fixed_string(1, 'xxx'))",
  "$10 = fixed_string(1, 'xxx')",
  "=10 = fixed_string(1, 'xxx')",
  "$[10 = fixed_string(1, 'xxx'])",
  "xxx[10 = fixed_string(1, 'xxx'])",
  "?$10 = fixed_string(1, 'xxx'])",
  "?=10 = fixed_string(1, 'xxx'])",
  "?[10 = fixed_string(1, 'xxx')$",
  "?[10 = fixed_string(1, 'xxx')$",
  "10 * fixed_string[1, 'ucs_2'$",
  "?10 * fixed_string[1, 'ucs_2'$",
  "?[10 * fixed_string(1, 'ucs_2'$)",
  "$10 * fixed_string[1, 'ucs_2'$",
  "=10 * fixed_string[1, 'ucs_2'$",
  "$[10 * fixed_string(1, 'ucs_2'$)",
  "xxx[10 * fixed_string(1, 'ucs_2'$)",
  "?$10 * fixed_string(1, 'ucs_2'$)",
  "?=10 * fixed_string(1, 'ucs_2'$)",
  "?[10 * fixed_string[1, 'ucs_2'$$",
  "?[10 * fixed_string[1, 'ucs_2'$$",
  "fixed_string[1, 'ucs_2'$",
  "?fixed_string[1, 'ucs_2'$",
  "?[fixed_string(1, 'ucs_2'$)",
  "$fixed_string[1, 'ucs_2'$",
  "=fixed_string[1, 'ucs_2'$",
  "$[fixed_string(1, 'ucs_2'$)",
  "xxx[fixed_string(1, 'ucs_2'$)",
  "?$fixed_string(1, 'ucs_2'$)",
  "?=fixed_string(1, 'ucs_2'$)",
  "?[fixed_string[1, 'ucs_2'$$",
  "?[fixed_string[1, 'ucs_2'$$",
  "10 $ fixed_string[1, 'ucs_2'$",
  "?10 $ fixed_string[1, 'ucs_2'$",
  "?[10 $ fixed_string(1, 'ucs_2'$)",
  "$10 $ fixed_string[1, 'ucs_2'$",
  "=10 $ fixed_string[1, 'ucs_2'$",
  "$[10 $ fixed_string(1, 'ucs_2'$)",
  "xxx[10 $ fixed_string(1, 'ucs_2'$)",
  "?$10 $ fixed_string(1, 'ucs_2'$)",
  "?=10 $ fixed_string(1, 'ucs_2'$)",
  "?[10 $ fixed_string[1, 'ucs_2'$$",
  "?[10 $ fixed_string[1, 'ucs_2'$$",
  "10 = fixed_string[1, 'ucs_2'$",
  "?10 = fixed_string[1, 'ucs_2'$",
  "?[10 = fixed_string(1, 'ucs_2'$)",
  "$10 = fixed_string[1, 'ucs_2'$",
  "=10 = fixed_string[1, 'ucs_2'$",
  "$[10 = fixed_string(1, 'ucs_2'$)",
  "xxx[10 = fixed_string(1, 'ucs_2'$)",
  "?$10 = fixed_string(1, 'ucs_2'$)",
  "?=10 = fixed_string(1, 'ucs_2'$)",
  "?[10 = fixed_string[1, 'ucs_2'$$",
  "?[10 = fixed_string[1, 'ucs_2'$$",
  "10 * fixed_string[1, 'ucs_2'$",
  "?10 * fixed_string[1, 'ucs_2'$",
  "?[10 * fixed_string(1, 'ucs_2'$)",
  "$10 * fixed_string[1, 'ucs_2'$",
  "=10 * fixed_string[1, 'ucs_2'$",
  "$[10 * fixed_string(1, 'ucs_2'$)",
  "xxx[10 * fixed_string(1, 'ucs_2'$)",
  "?$10 * fixed_string(1, 'ucs_2'$)",
  "?=10 * fixed_string(1, 'ucs_2'$)",
  "?[10 * fixed_string[1, 'ucs_2'$$",
  "?[10 * fixed_string[1, 'ucs_2'$$",
  "fixed_string[1, 'ucs_2'$",
  "?fixed_string[1, 'ucs_2'$",
  "?[fixed_string(1, 'ucs_2'$)",
  "$fixed_string[1, 'ucs_2'$",
  "=fixed_string[1, 'ucs_2'$",
  "$[fixed_string(1, 'ucs_2'$)",
  "xxx[fixed_string(1, 'ucs_2'$)",
  "?$fixed_string(1, 'ucs_2'$)",
  "?=fixed_string(1, 'ucs_2'$)",
  "?[fixed_string[1, 'ucs_2'$$",
  "?[fixed_string[1, 'ucs_2'$$",
  "10 $ fixed_string[1, 'ucs_2'$",
  "?10 $ fixed_string[1, 'ucs_2'$",
  "?[10 $ fixed_string(1, 'ucs_2'$)",
  "$10 $ fixed_string[1, 'ucs_2'$",
  "=10 $ fixed_string[1, 'ucs_2'$",
  "$[10 $ fixed_string(1, 'ucs_2'$)",
  "xxx[10 $ fixed_string(1, 'ucs_2'$)",
  "?$10 $ fixed_string(1, 'ucs_2'$)",
  "?=10 $ fixed_string(1, 'ucs_2'$)",
  "?[10 $ fixed_string[1, 'ucs_2'$$",
  "?[10 $ fixed_string[1, 'ucs_2'$$",
  "10 = fixed_string[1, 'ucs_2'$",
  "?10 = fixed_string[1, 'ucs_2'$",
  "?[10 = fixed_string(1, 'ucs_2'$)",
  "$10 = fixed_string[1, 'ucs_2'$",
  "=10 = fixed_string[1, 'ucs_2'$",
  "$[10 = fixed_string(1, 'ucs_2'$)",
  "xxx[10 = fixed_string(1, 'ucs_2'$)",
  "?$10 = fixed_string(1, 'ucs_2'$)",
  "?=10 = fixed_string(1, 'ucs_2'$)",
  "?[10 = fixed_string[1, 'ucs_2'$$",
  "?[10 = fixed_string[1, 'ucs_2'$$",
  "$10 * bytes(align=10)",
  "=10 * bytes(align=10)",
  "$[10 * bytes(align=10])",
  "xxx[10 * bytes(align=10])",
  "?$10 * bytes(align=10])",
  "?=10 * bytes(align=10])",
  "?[10 * bytes(align=10)$",
  "?[10 * bytes(align=10)$",
  "$bytes(align=10)",
  "=bytes(align=10)",
  "$[bytes(align=10])",
  "xxx[bytes(align=10])",
  "?$bytes(align=10])",
  "?=bytes(align=10])",
  "?[bytes(align=10)$",
  "?[bytes(align=10)$",
  "10 $ bytes(align=10)",
  "?10 $ bytes(align=10)",
  "?(10 $ bytes(align=10))",
  "$10 $ bytes(align=10)",
  "=10 $ bytes(align=10)",
  "$[10 $ bytes(align=10])",
  "xxx[10 $ bytes(align=10])",
  "?$10 $ bytes(align=10])",
  "?=10 $ bytes(align=10])",
  "?[10 $ bytes(align=10)$",
  "?[10 $ bytes(align=10)$",
  "10 = bytes(align=10)",
  "?10 = bytes(align=10)",
  "?(10 = bytes(align=10))",
  "$10 = bytes(align=10)",
  "=10 = bytes(align=10)",
  "$[10 = bytes(align=10])",
  "xxx[10 = bytes(align=10])",
  "?$10 = bytes(align=10])",
  "?=10 = bytes(align=10])",
  "?[10 = bytes(align=10)$",
  "?[10 = bytes(align=10)$",
  "10 * $[align=2]",
  "?10 * $[align=2]",
  "?[10 * $[align=2]]",
  "$10 * $[align=2]",
  "=10 * $[align=2]",
  "$[10 * $[align=2]]",
  "xxx[10 * $[align=2]]",
  "?$10 * $[align=2]]",
  "?=10 * $[align=2]]",
  "?[10 * $[align=2]$",
  "?[10 * $[align=2]$",
  "$[align=2]",
  "?$[align=2]",
  "?[$[align=2]]",
  "$$[align=2]",
  "=$[align=2]",
  "$[$[align=2]]",
  "xxx[$[align=2]]",
  "?$$[align=2]]",
  "?=$[align=2]]",
  "?[$[align=2]$",
  "?[$[align=2]$",
  "10 $ $[align=2]",
  "?10 $ $[align=2]",
  "?[10 $ $[align=2]]",
  "$10 $ $[align=2]",
  "=10 $ $[align=2]",
  "$[10 $ $[align=2]]",
  "xxx[10 $ $[align=2]]",
  "?$10 $ $[align=2]]",
  "?=10 $ $[align=2]]",
  "?[10 $ $[align=2]$",
  "?[10 $ $[align=2]$",
  "10 = $[align=2]",
  "?10 = $[align=2]",
  "?[10 = $[align=2]]",
  "$10 = $[align=2]",
  "=10 = $[align=2]",
  "$[10 = $[align=2]]",
  "xxx[10 = $[align=2]]",
  "?$10 = $[align=2]]",
  "?=10 = $[align=2]]",
  "?[10 = $[align=2]$",
  "?[10 = $[align=2]$",
  "10 * xxx[align=8]",
  "?10 * xxx[align=8]",
  "?[10 * xxx[align=8]]",
  "$10 * xxx[align=8]",
  "=10 * xxx[align=8]",
  "$[10 * xxx[align=8]]",
  "xxx[10 * xxx[align=8]]",
  "?$10 * xxx[align=8]]",
  "?=10 * xxx[align=8]]",
  "?[10 * xxx[align=8]$",
  "?[10 * xxx[align=8]$",
  "xxx[align=8]",
  "?xxx[align=8]",
  "?[xxx[align=8]]",
  "$xxx[align=8]",
  "=xxx[align=8]",
  "$[xxx[align=8]]",
  "xxx[xxx[align=8]]",
  "?$xxx[align=8]]",
  "?=xxx[align=8]]",
  "?[xxx[align=8]$",
  "?[xxx[align=8]$",
  "10 $ xxx[align=8]",
  "?10 $ xxx[align=8]",
  "?[10 $ xxx[align=8]]",
  "$10 $ xxx[align=8]",
  "=10 $ xxx[align=8]",
  "$[10 $ xxx[align=8]]",
  "xxx[10 $ xxx[align=8]]",
  "?$10 $ xxx[align=8]]",
  "?=10 $ xxx[align=8]]",
  "?[10 $ xxx[align=8]$",
  "?[10 $ xxx[align=8]$",
  "10 = xxx[align=8]",
  "?10 = xxx[align=8]",
  "?[10 = xxx[align=8]]",
  "$10 = xxx[align=8]",
  "=10 = xxx[align=8]",
  "$[10 = xxx[align=8]]",
  "xxx[10 = xxx[align=8]]",
  "?$10 = xxx[align=8]]",
  "?=10 = xxx[align=8]]",
  "?[10 = xxx[align=8]$",
  "?[10 = xxx[align=8]$",
  "10 * bytes$align=8]",
  "?10 * bytes$align=8]",
  "?(10 * bytes$align=8])",
  "$10 * bytes$align=8]",
  "=10 * bytes$align=8]",
  "$[10 * bytes$align=8]]",
  "xxx[10 * bytes$align=8]]",
  "?$10 * bytes$align=8]]",
  "?=10 * bytes$align=8]]",
  "?(10 * bytes$align=8)$",
  "?(10 * bytes$align=8)$",
  "bytes$align=8]",
  "?bytes$align=8]",
  "?(bytes$align=8])",
  "$bytes$align=8]",
  "=bytes$align=8]",
  "$[bytes$align=8]]",
  "xxx[bytes$align=8]]",
  "?$bytes$align=8]]",
  "?=bytes$align=8]]",
  "?(bytes$align=8)$",
  "?(bytes$align=8)$",
  "10 $ bytes$align=8]",
  "?10 $ bytes$align=8]",
  "?(10 $ bytes$align=8])",
  "$10 $ bytes$align=8]",
  "=10 $ bytes$align=8]",
  "$[10 $ bytes$align=8]]",
  "xxx[10 $ bytes$align=8]]",
  "?$10 $ bytes$align=8]]",
  "?=10 $ bytes$align=8]]",
  "?(10 $ bytes$align=8)$",
  "?(10 $ bytes$align=8)$",
  "10 = bytes$align=8]",
  "?10 = bytes$align=8]",
  "?(10 = bytes$align=8])",
  "$10 = bytes$align=8]",
  "=10 = bytes$align=8]",
  "$[10 = bytes$align=8]]",
  "xxx[10 = bytes$align=8]]",
  "?$10 = bytes$align=8]]",
  "?=10 = bytes$align=8]]",
  "?(10 = bytes$align=8)$",
  "?(10 = bytes$align=8)$",
  "10 * bytes=align=16]",
  "?10 * bytes=align=16]",
  "?(10 * bytes=align=16])",
  "$10 * bytes=align=16]",
  "=10 * bytes=align=16]",
  "$[10 * bytes=align=16]]",
  "xxx[10 * bytes=align=16]]",
  "?$10 * bytes=align=16]]",
  "?=10 * bytes=align=16]]",
  "?(10 * bytes=align=16)$",
  "?(10 * bytes=align=16)$",
  "bytes=align=16]",
  "?bytes=align=16]",
  "?(bytes=align=16])",
  "$bytes=align=16]",
  "=bytes=align=16]",
  "$[bytes=align=16]]",
  "xxx[bytes=align=16]]",
  "?$bytes=align=16]]",
  "?=bytes=align=16]]",
  "?(bytes=align=16)$",
  "?(bytes=align=16)$",
  "10 $ bytes=align=16]",
  "?10 $ bytes=align=16]",
  "?(10 $ bytes=align=16])",
  "$10 $ bytes=align=16]",
  "=10 $ bytes=align=16]",
  "$[10 $ bytes=align=16]]",
  "xxx[10 $ bytes=align=16]]",
  "?$10 $ bytes=align=16]]",
  "?=10 $ bytes=align=16]]",
  "?(10 $ bytes=align=16)$",
  "?(10 $ bytes=align=16)$",
  "10 = bytes=align=16]",
  "?10 = bytes=align=16]",
  "?(10 = bytes=align=16])",
  "$10 = bytes=align=16]",
  "=10 = bytes=align=16]",
  "$[10 = bytes=align=16]]",
  "xxx[10 = bytes=align=16]]",
  "?$10 = bytes=align=16]]",
  "?=10 = bytes=align=16]]",
  "?(10 = bytes=align=16)$",
  "?(10 = bytes=align=16)$",
  "10 * bytes($=29)",
  "?10 * bytes($=29)",
  "?(10 * bytes($=29))",
  "$10 * bytes($=29)",
  "=10 * bytes($=29)",
  "$[10 * bytes($=29])",
  "xxx[10 * bytes($=29])",
  "?$10 * bytes($=29])",
  "?=10 * bytes($=29])",
  "?[10 * bytes($=29)$",
  "?[10 * bytes($=29)$",
  "bytes($=29)",
  "?bytes($=29)",
  "?(bytes($=29))",
  "$bytes($=29)",
  "=bytes($=29)",
  "$[bytes($=29])",
  "xxx[bytes($=29])",
  "?$bytes($=29])",
  "?=bytes($=29])",
  "?[bytes($=29)$",
  "?[bytes($=29)$",
  "10 $ bytes($=29)",
  "?10 $ bytes($=29)",
  "?(10 $ bytes($=29))",
  "$10 $ bytes($=29)",
  "=10 $ bytes($=29)",
  "$[10 $ bytes($=29])",
  "xxx[10 $ bytes($=29])",
  "?$10 $ bytes($=29])",
  "?=10 $ bytes($=29])",
  "?[10 $ bytes($=29)$",
  "?[10 $ bytes($=29)$",
  "10 = bytes($=29)",
  "?10 = bytes($=29)",
  "?(10 = bytes($=29))",
  "$10 = bytes($=29)",
  "=10 = bytes($=29)",
  "$[10 = bytes($=29])",
  "xxx[10 = bytes($=29])",
  "?$10 = bytes($=29])",
  "?=10 = bytes($=29])",
  "?[10 = bytes($=29)$",
  "?[10 = bytes($=29)$",
  "10 * bytes(xxx=76)",
  "?10 * bytes(xxx=76)",
  "?(10 * bytes(xxx=76))",
  "$10 * bytes(xxx=76)",
  "=10 * bytes(xxx=76)",
  "$[10 * bytes(xxx=76])",
  "xxx[10 * bytes(xxx=76])",
  "?$10 * bytes(xxx=76])",
  "?=10 * bytes(xxx=76])",
  "?[10 * bytes(xxx=76)$",
  "?[10 * bytes(xxx=76)$",
  "bytes(xxx=76)",
  "?bytes(xxx=76)",
  "?(bytes(xxx=76))",
  "$bytes(xxx=76)",
  "=bytes(xxx=76)",
  "$[bytes(xxx=76])",
  "xxx[bytes(xxx=76])",
  "?$bytes(xxx=76])",
  "?=bytes(xxx=76])",
  "?[bytes(xxx=76)$",
  "?[bytes(xxx=76)$",
  "10 $ bytes(xxx=76)",
  "?10 $ bytes(xxx=76)",
  "?(10 $ bytes(xxx=76))",
  "$10 $ bytes(xxx=76)",
  "=10 $ bytes(xxx=76)",
  "$[10 $ bytes(xxx=76])",
  "xxx[10 $ bytes(xxx=76])",
  "?$10 $ bytes(xxx=76])",
  "?=10 $ bytes(xxx=76])",
  "?[10 $ bytes(xxx=76)$",
  "?[10 $ bytes(xxx=76)$",
  "10 = bytes(xxx=76)",
  "?10 = bytes(xxx=76)",
  "?(10 = bytes(xxx=76))",
  "$10 = bytes(xxx=76)",
  "=10 = bytes(xxx=76)",
  "$[10 = bytes(xxx=76])",
  "xxx[10 = bytes(xxx=76])",
  "?$10 = bytes(xxx=76])",
  "?=10 = bytes(xxx=76])",
  "?[10 = bytes(xxx=76)$",
  "?[10 = bytes(xxx=76)$",
  "10 * bytes(align$66)",
  "?10 * bytes(align$66)",
  "?(10 * bytes(align$66))",
  "$10 * bytes(align$66)",
  "=10 * bytes(align$66)",
  "$[10 * bytes(align$66])",
  "xxx[10 * bytes(align$66])",
  "?$10 * bytes(align$66])",
  "?=10 * bytes(align$66])",
  "?[10 * bytes(align$66)$",
  "?[10 * bytes(align$66)$",
  "bytes(align$66)",
  "?bytes(align$66)",
  "?(bytes(align$66))",
  "$bytes(align$66)",
  "=bytes(align$66)",
  "$[bytes(align$66])",
  "xxx[bytes(align$66])",
  "?$bytes(align$66])",
  "?=bytes(align$66])",
  "?[bytes(align$66)$",
  "?[bytes(align$66)$",
  "10 $ bytes(align$66)",
  "?10 $ bytes(align$66)",
  "?(10 $ bytes(align$66))",
  "$10 $ bytes(align$66)",
  "=10 $ bytes(align$66)",
  "$[10 $ bytes(align$66])",
  "xxx[10 $ bytes(align$66])",
  "?$10 $ bytes(align$66])",
  "?=10 $ bytes(align$66])",
  "?[10 $ bytes(align$66)$",
  "?[10 $ bytes(align$66)$",
  "10 = bytes(align$66)",
  "?10 = bytes(align$66)",
  "?(10 = bytes(align$66))",
  "$10 = bytes(align$66)",
  "=10 = bytes(align$66)",
  "$[10 = bytes(align$66])",
  "xxx[10 = bytes(align$66])",
  "?$10 = bytes(align$66])",
  "?=10 = bytes(align$66])",
  "?[10 = bytes(align$66)$",
  "?[10 = bytes(align$66)$",
  "10 * bytes(align''89)",
  "?10 * bytes(align''89)",
  "?(10 * bytes(align''89))",
  "$10 * bytes(align''89)",
  "=10 * bytes(align''89)",
  "$[10 * bytes(align''89])",
  "xxx[10 * bytes(align''89])",
  "?$10 * bytes(align''89])",
  "?=10 * bytes(align''89])",
  "?[10 * bytes(align''89)$",
  "?[10 * bytes(align''89)$",
  "bytes(align''89)",
  "?bytes(align''89)",
  "?(bytes(align''89))",
  "$bytes(align''89)",
  "=bytes(align''89)",
  "$[bytes(align''89])",
  "xxx[bytes(align''89])",
  "?$bytes(align''89])",
  "?=bytes(align''89])",
  "?[bytes(align''89)$",
  "?[bytes(align''89)$",
  "10 $ bytes(align''89)",
  "?10 $ bytes(align''89)",
  "?(10 $ bytes(align''89))",
  "$10 $ bytes(align''89)",
  "=10 $ bytes(align''89)",
  "$[10 $ bytes(align''89])",
  "xxx[10 $ bytes(align''89])",
  "?$10 $ bytes(align''89])",
  "?=10 $ bytes(align''89])",
  "?[10 $ bytes(align''89)$",
  "?[10 $ bytes(align''89)$",
  "10 = bytes(align''89)",
  "?10 = bytes(align''89)",
  "?(10 = bytes(align''89))",
  "$10 = bytes(align''89)",
  "=10 = bytes(align''89)",
  "$[10 = bytes(align''89])",
  "xxx[10 = bytes(align''89])",
  "?$10 = bytes(align''89])",
  "?=10 = bytes(align''89])",
  "?[10 = bytes(align''89)$",
  "?[10 = bytes(align''89)$",
  "10 * bytes(align'xxx'156)",
  "?10 * bytes(align'xxx'156)",
  "?(10 * bytes(align'xxx'156))",
  "$10 * bytes(align'xxx'156)",
  "=10 * bytes(align'xxx'156)",
  "$[10 * bytes(align'xxx'156])",
  "xxx[10 * bytes(align'xxx'156])",
  "?$10 * bytes(align'xxx'156])",
  "?=10 * bytes(align'xxx'156])",
  "?[10 * bytes(align'xxx'156)$",
  "?[10 * bytes(align'xxx'156)$",
  "bytes(align'xxx'156)",
  "?bytes(align'xxx'156)",
  "?(bytes(align'xxx'156))",
  "$bytes(align'xxx'156)",
  "=bytes(align'xxx'156)",
  "$[bytes(align'xxx'156])",
  "xxx[bytes(align'xxx'156])",
  "?$bytes(align'xxx'156])",
  "?=bytes(align'xxx'156])",
  "?[bytes(align'xxx'156)$",
  "?[bytes(align'xxx'156)$",
  "10 $ bytes(align'xxx'156)",
  "?10 $ bytes(align'xxx'156)",
  "?(10 $ bytes(align'xxx'156))",
  "$10 $ bytes(align'xxx'156)",
  "=10 $ bytes(align'xxx'156)",
  "$[10 $ bytes(align'xxx'156])",
  "xxx[10 $ bytes(align'xxx'156])",
  "?$10 $ bytes(align'xxx'156])",
  "?=10 $ bytes(align'xxx'156])",
  "?[10 $ bytes(align'xxx'156)$",
  "?[10 $ bytes(align'xxx'156)$",
  "10 = bytes(align'xxx'156)",
  "?10 = bytes(align'xxx'156)",
  "?(10 = bytes(align'xxx'156))",
  "$10 = bytes(align'xxx'156)",
  "=10 = bytes(align'xxx'156)",
  "$[10 = bytes(align'xxx'156])",
  "xxx[10 = bytes(align'xxx'156])",
  "?$10 = bytes(align'xxx'156])",
  "?=10 = bytes(align'xxx'156])",
  "?[10 = bytes(align'xxx'156)$",
  "?[10 = bytes(align'xxx'156)$",
  "10 * bytes(align=$)",
  "?10 * bytes(align=$)",
  "?(10 * bytes(align=$))",
  "$10 * bytes(align=$)",
  "=10 * bytes(align=$)",
  "$[10 * bytes(align=$])",
  "xxx[10 * bytes(align=$])",
  "?$10 * bytes(align=$])",
  "?=10 * bytes(align=$])",
  "?[10 * bytes(align=$)$",
  "?[10 * bytes(align=$)$",
  "bytes(align=$)",
  "?bytes(align=$)",
  "?(bytes(align=$))",
  "$bytes(align=$)",
  "=bytes(align=$)",
  "$[bytes(align=$])",
  "xxx[bytes(align=$])",
  "?$bytes(align=$])",
  "?=bytes(align=$])",
  "?[bytes(align=$)$",
  "?[bytes(align=$)$",
  "10 $ bytes(align=$)",
  "?10 $ bytes(align=$)",
  "?(10 $ bytes(align=$))",
  "$10 $ bytes(align=$)",
  "=10 $ bytes(align=$)",
  "$[10 $ bytes(align=$])",
  "xxx[10 $ bytes(align=$])",
  "?$10 $ bytes(align=$])",
  "?=10 $ bytes(align=$])",
  "?[10 $ bytes(align=$)$",
  "?[10 $ bytes(align=$)$",
  "10 = bytes(align=$)",
  "?10 = bytes(align=$)",
  "?(10 = bytes(align=$))",
  "$10 = bytes(align=$)",
  "=10 = bytes(align=$)",
  "$[10 = bytes(align=$])",
  "xxx[10 = bytes(align=$])",
  "?$10 = bytes(align=$])",
  "?=10 = bytes(align=$])",
  "?[10 = bytes(align=$)$",
  "?[10 = bytes(align=$)$",
  "10 * bytes(align=xxx)",
  "?10 * bytes(align=xxx)",
  "?(10 * bytes(align=xxx))",
  "$10 * bytes(align=xxx)",
  "=10 * bytes(align=xxx)",
  "$[10 * bytes(align=xxx])",
  "xxx[10 * bytes(align=xxx])",
  "?$10 * bytes(align=xxx])",
  "?=10 * bytes(align=xxx])",
  "?[10 * bytes(align=xxx)$",
  "?[10 * bytes(align=xxx)$",
  "bytes(align=xxx)",
  "?bytes(align=xxx)",
  "?(bytes(align=xxx))",
  "$bytes(align=xxx)",
  "=bytes(align=xxx)",
  "$[bytes(align=xxx])",
  "xxx[bytes(align=xxx])",
  "?$bytes(align=xxx])",
  "?=bytes(align=xxx])",
  "?[bytes(align=xxx)$",
  "?[bytes(align=xxx)$",
  "10 $ bytes(align=xxx)",
  "?10 $ bytes(align=xxx)",
  "?(10 $ bytes(align=xxx))",
  "$10 $ bytes(align=xxx)",
  "=10 $ bytes(align=xxx)",
  "$[10 $ bytes(align=xxx])",
  "xxx[10 $ bytes(align=xxx])",
  "?$10 $ bytes(align=xxx])",
  "?=10 $ bytes(align=xxx])",
  "?[10 $ bytes(align=xxx)$",
  "?[10 $ bytes(align=xxx)$",
  "10 = bytes(align=xxx)",
  "?10 = bytes(align=xxx)",
  "?(10 = bytes(align=xxx))",
  "$10 = bytes(align=xxx)",
  "=10 = bytes(align=xxx)",
  "$[10 = bytes(align=xxx])",
  "xxx[10 = bytes(align=xxx])",
  "?$10 = bytes(align=xxx])",
  "?=10 = bytes(align=xxx])",
  "?[10 = bytes(align=xxx)$",
  "?[10 = bytes(align=xxx)$",
  "10 * bytes[align=1$",
  "?10 * bytes[align=1$",
  "?[10 * bytes(align=1$)",
  "$10 * bytes[align=1$",
  "=10 * bytes[align=1$",
  "$[10 * bytes(align=1$)",
  "xxx[10 * bytes(align=1$)",
  "?$10 * bytes(align=1$)",
  "?=10 * bytes(align=1$)",
  "?[10 * bytes[align=1$$",
  "?[10 * bytes[align=1$$",
  "bytes[align=1$",
  "?bytes[align=1$",
  "?[bytes(align=1$)",
  "$bytes[align=1$",
  "=bytes[align=1$",
  "$[bytes(align=1$)",
  "xxx[bytes(align=1$)",
  "?$bytes(align=1$)",
  "?=bytes(align=1$)",
  "?[bytes[align=1$$",
  "?[bytes[align=1$$",
  "10 $ bytes[align=1$",
  "?10 $ bytes[align=1$",
  "?[10 $ bytes(align=1$)",
  "$10 $ bytes[align=1$",
  "=10 $ bytes[align=1$",
  "$[10 $ bytes(align=1$)",
  "xxx[10 $ bytes(align=1$)",
  "?$10 $ bytes(align=1$)",
  "?=10 $ bytes(align=1$)",
  "?[10 $ bytes[align=1$$",
  "?[10 $ bytes[align=1$$",
  "10 = bytes[align=1$",
  "?10 = bytes[align=1$",
  "?[10 = bytes(align=1$)",
  "$10 = bytes[align=1$",
  "=10 = bytes[align=1$",
  "$[10 = bytes(align=1$)",
  "xxx[10 = bytes(align=1$)",
  "?$10 = bytes(align=1$)",
  "?=10 = bytes(align=1$)",
  "?[10 = bytes[align=1$$",
  "?[10 = bytes[align=1$$",
  "10 * bytes[align=1$",
  "?10 * bytes[align=1$",
  "?[10 * bytes(align=1$)",
  "$10 * bytes[align=1$",
  "=10 * bytes[align=1$",
  "$[10 * bytes(align=1$)",
  "xxx[10 * bytes(align=1$)",
  "?$10 * bytes(align=1$)",
  "?=10 * bytes(align=1$)",
  "?[10 * bytes[align=1$$",
  "?[10 * bytes[align=1$$",
  "bytes[align=1$",
  "?bytes[align=1$",
  "?[bytes(align=1$)",
  "$bytes[align=1$",
  "=bytes[align=1$",
  "$[bytes(align=1$)",
  "xxx[bytes(align=1$)",
  "?$bytes(align=1$)",
  "?=bytes(align=1$)",
  "?[bytes[align=1$$",
  "?[bytes[align=1$$",
  "10 $ bytes[align=1$",
  "?10 $ bytes[align=1$",
  "?[10 $ bytes(align=1$)",
  "$10 $ bytes[align=1$",
  "=10 $ bytes[align=1$",
  "$[10 $ bytes(align=1$)",
  "xxx[10 $ bytes(align=1$)",
  "?$10 $ bytes(align=1$)",
  "?=10 $ bytes(align=1$)",
  "?[10 $ bytes[align=1$$",
  "?[10 $ bytes[align=1$$",
  "10 = bytes[align=1$",
  "?10 = bytes[align=1$",
  "?[10 = bytes(align=1$)",
  "$10 = bytes[align=1$",
  "=10 = bytes[align=1$",
  "$[10 = bytes(align=1$)",
  "xxx[10 = bytes(align=1$)",
  "?$10 = bytes(align=1$)",
  "?=10 = bytes(align=1$)",
  "?[10 = bytes[align=1$$",
  "?[10 = bytes[align=1$$",
  "$10 * FixedBytesKind",
  "=10 * FixedBytesKind",
  "$[10 * FixedBytesKind]",
  "xxx[10 * FixedBytesKind]",
  "?$10 * FixedBytesKind]",
  "?=10 * FixedBytesKind]",
  "?[10 * FixedBytesKind$",
  "?[10 * FixedBytesKind$",
  "$FixedBytesKind",
  "=FixedBytesKind",
  "$[FixedBytesKind]",
  "xxx[FixedBytesKind]",
  "?$FixedBytesKind]",
  "?=FixedBytesKind]",
  "?[FixedBytesKind$",
  "?[FixedBytesKind$",
  "10 $ FixedBytesKind",
  "?10 $ FixedBytesKind",
  "?(10 $ FixedBytesKind)",
  "$10 $ FixedBytesKind",
  "=10 $ FixedBytesKind",
  "$[10 $ FixedBytesKind]",
  "xxx[10 $ FixedBytesKind]",
  "?$10 $ FixedBytesKind]",
  "?=10 $ FixedBytesKind]",
  "?[10 $ FixedBytesKind$",
  "?[10 $ FixedBytesKind$",
  "10 = FixedBytesKind",
  "?10 = FixedBytesKind",
  "?(10 = FixedBytesKind)",
  "$10 = FixedBytesKind",
  "=10 = FixedBytesKind",
  "$[10 = FixedBytesKind]",
  "xxx[10 = FixedBytesKind]",
  "?$10 = FixedBytesKind]",
  "?=10 = FixedBytesKind]",
  "?[10 = FixedBytesKind$",
  "?[10 = FixedBytesKind$",
  "$10 * fixed_bytes(size=2130818740, align=4)",
  "=10 * fixed_bytes(size=2130818740, align=4)",
  "$[10 * fixed_bytes(size=2130818740, align=4])",
  "xxx[10 * fixed_bytes(size=2130818740, align=4])",
  "?$10 * fixed_bytes(size=2130818740, align=4])",
  "?=10 * fixed_bytes(size=2130818740, align=4])",
  "?[10 * fixed_bytes(size=2130818740, align=4)$",
  "?[10 * fixed_bytes(size=2130818740, align=4)$",
  "$fixed_bytes(size=2130818740, align=4)",
  "=fixed_bytes(size=2130818740, align=4)",
  "$[fixed_bytes(size=2130818740, align=4])",
  "xxx[fixed_bytes(size=2130818740, align=4])",
  "?$fixed_bytes(size=2130818740, align=4])",
  "?=fixed_bytes(size=2130818740, align=4])",
  "?[fixed_bytes(size=2130818740, align=4)$",
  "?[fixed_bytes(size=2130818740, align=4)$",
  "10 $ fixed_bytes(size=2130818740, align=4)",
  "?10 $ fixed_bytes(size=2130818740, align=4)",
  "?(10 $ fixed_bytes(size=2130818740, align=4))",
  "$10 $ fixed_bytes(size=2130818740, align=4)",
  "=10 $ fixed_bytes(size=2130818740, align=4)",
  "$[10 $ fixed_bytes(size=2130818740, align=4])",
  "xxx[10 $ fixed_bytes(size=2130818740, align=4])",
  "?$10 $ fixed_bytes(size=2130818740, align=4])",
  "?=10 $ fixed_bytes(size=2130818740, align=4])",
  "?[10 $ fixed_bytes(size=2130818740, align=4)$",
  "?[10 $ fixed_bytes(size=2130818740, align=4)$",
  "10 = fixed_bytes(size=2130818740, align=4)",
  "?10 = fixed_bytes(size=2130818740, align=4)",
  "?(10 = fixed_bytes(size=2130818740, align=4))",
  "$10 = fixed_bytes(size=2130818740, align=4)",
  "=10 = fixed_bytes(size=2130818740, align=4)",
  "$[10 = fixed_bytes(size=2130818740, align=4])",
  "xxx[10 = fixed_bytes(size=2130818740, align=4])",
  "?$10 = fixed_bytes(size=2130818740, align=4])",
  "?=10 = fixed_bytes(size=2130818740, align=4])",
  "?[10 = fixed_bytes(size=2130818740, align=4)$",
  "?[10 = fixed_bytes(size=2130818740, align=4)$",
  "10 * $[1571175992, align=4]",
  "?10 * $[1571175992, align=4]",
  "?[10 * $[1571175992, align=4]]",
  "$10 * $[1571175992, align=4]",
  "=10 * $[1571175992, align=4]",
  "$[10 * $[1571175992, align=4]]",
  "xxx[10 * $[1571175992, align=4]]",
  "?$10 * $[1571175992, align=4]]",
  "?=10 * $[1571175992, align=4]]",
  "?[10 * $[1571175992, align=4]$",
  "?[10 * $[1571175992, align=4]$",
  "$[1571175992, align=4]",
  "?$[1571175992, align=4]",
  "?[$[1571175992, align=4]]",
  "$$[1571175992, align=4]",
  "=$[1571175992, align=4]",
  "$[$[1571175992, align=4]]",
  "xxx[$[1571175992, align=4]]",
  "?$$[1571175992, align=4]]",
  "?=$[1571175992, align=4]]",
  "?[$[1571175992, align=4]$",
  "?[$[1571175992, align=4]$",
  "10 $ $[1571175992, align=4]",
  "?10 $ $[1571175992, align=4]",
  "?[10 $ $[1571175992, align=4]]",
  "$10 $ $[1571175992, align=4]",
  "=10 $ $[1571175992, align=4]",
  "$[10 $ $[1571175992, align=4]]",
  "xxx[10 $ $[1571175992, align=4]]",
  "?$10 $ $[1571175992, align=4]]",
  "?=10 $ $[1571175992, align=4]]",
  "?[10 $ $[1571175992, align=4]$",
  "?[10 $ $[1571175992, align=4]$",
  "10 = $[1571175992, align=4]",
  "?10 = $[1571175992, align=4]",
  "?[10 = $[1571175992, align=4]]",
  "$10 = $[1571175992, align=4]",
  "=10 = $[1571175992, align=4]",
  "$[10 = $[1571175992, align=4]]",
  "xxx[10 = $[1571175992, align=4]]",
  "?$10 = $[1571175992, align=4]]",
  "?=10 = $[1571175992, align=4]]",
  "?[10 = $[1571175992, align=4]$",
  "?[10 = $[1571175992, align=4]$",
  "10 * xxx[1003886228, align=4]",
  "?10 * xxx[1003886228, align=4]",
  "?[10 * xxx[1003886228, align=4]]",
  "$10 * xxx[1003886228, align=4]",
  "=10 * xxx[1003886228, align=4]",
  "$[10 * xxx[1003886228, align=4]]",
  "xxx[10 * xxx[1003886228, align=4]]",
  "?$10 * xxx[1003886228, align=4]]",
  "?=10 * xxx[1003886228, align=4]]",
  "?[10 * xxx[1003886228, align=4]$",
  "?[10 * xxx[1003886228, align=4]$",
  "xxx[1003886228, align=4]",
  "?xxx[1003886228, align=4]",
  "?[xxx[1003886228, align=4]]",
  "$xxx[1003886228, align=4]",
  "=xxx[1003886228, align=4]",
  "$[xxx[1003886228, align=4]]",
  "xxx[xxx[1003886228, align=4]]",
  "?$xxx[1003886228, align=4]]",
  "?=xxx[1003886228, align=4]]",
  "?[xxx[1003886228, align=4]$",
  "?[xxx[1003886228, align=4]$",
  "10 $ xxx[1003886228, align=4]",
  "?10 $ xxx[1003886228, align=4]",
  "?[10 $ xxx[1003886228, align=4]]",
  "$10 $ xxx[1003886228, align=4]",
  "=10 $ xxx[1003886228, align=4]",
  "$[10 $ xxx[1003886228, align=4]]",
  "xxx[10 $ xxx[1003886228, align=4]]",
  "?$10 $ xxx[1003886228, align=4]]",
  "?=10 $ xxx[1003886228, align=4]]",
  "?[10 $ xxx[1003886228, align=4]$",
  "?[10 $ xxx[1003886228, align=4]$",
  "10 = xxx[1003886228, align=4]",
  "?10 = xxx[1003886228, align=4]",
  "?[10 = xxx[1003886228, align=4]]",
  "$10 = xxx[1003886228, align=4]",
  "=10 = xxx[1003886228, align=4]",
  "$[10 = xxx[1003886228, align=4]]",
  "xxx[10 = xxx[1003886228, align=4]]",
  "?$10 = xxx[1003886228, align=4]]",
  "?=10 = xxx[1003886228, align=4]]",
  "?[10 = xxx[1003886228, align=4]$",
  "?[10 = xxx[1003886228, align=4]$",
  "10 * fixed_bytes$1497937318, align=4]",
  "?10 * fixed_bytes$1497937318, align=4]",
  "?(10 * fixed_bytes$1497937318, align=4])",
  "$10 * fixed_bytes$1497937318, align=4]",
  "=10 * fixed_bytes$1497937318, align=4]",
  "$[10 * fixed_bytes$1497937318, align=4]]",
  "xxx[10 * fixed_bytes$1497937318, align=4]]",
  "?$10 * fixed_bytes$1497937318, align=4]]",
  "?=10 * fixed_bytes$1497937318, align=4]]",
  "?(10 * fixed_bytes$1497937318, align=4)$",
  "?(10 * fixed_bytes$1497937318, align=4)$",
  "fixed_bytes$1497937318, align=4]",
  "?fixed_bytes$1497937318, align=4]",
  "?(fixed_bytes$1497937318, align=4])",
  "$fixed_bytes$1497937318, align=4]",
  "=fixed_bytes$1497937318, align=4]",
  "$[fixed_bytes$1497937318, align=4]]",
  "xxx[fixed_bytes$1497937318, align=4]]",
  "?$fixed_bytes$1497937318, align=4]]",
  "?=fixed_bytes$1497937318, align=4]]",
  "?(fixed_bytes$1497937318, align=4)$",
  "?(fixed_bytes$1497937318, align=4)$",
  "10 $ fixed_bytes$1497937318, align=4]",
  "?10 $ fixed_bytes$1497937318, align=4]",
  "?(10 $ fixed_bytes$1497937318, align=4])",
  "$10 $ fixed_bytes$1497937318, align=4]",
  "=10 $ fixed_bytes$1497937318, align=4]",
  "$[10 $ fixed_bytes$1497937318, align=4]]",
  "xxx[10 $ fixed_bytes$1497937318, align=4]]",
  "?$10 $ fixed_bytes$1497937318, align=4]]",
  "?=10 $ fixed_bytes$1497937318, align=4]]",
  "?(10 $ fixed_bytes$1497937318, align=4)$",
  "?(10 $ fixed_bytes$1497937318, align=4)$",
  "10 = fixed_bytes$1497937318, align=4]",
  "?10 = fixed_bytes$1497937318, align=4]",
  "?(10 = fixed_bytes$1497937318, align=4])",
  "$10 = fixed_bytes$1497937318, align=4]",
  "=10 = fixed_bytes$1497937318, align=4]",
  "$[10 = fixed_bytes$1497937318, align=4]]",
  "xxx[10 = fixed_bytes$1497937318, align=4]]",
  "?$10 = fixed_bytes$1497937318, align=4]]",
  "?=10 = fixed_bytes$1497937318, align=4]]",
  "?(10 = fixed_bytes$1497937318, align=4)$",
  "?(10 = fixed_bytes$1497937318, align=4)$",
  "10 * fixed_bytes=1996527793, align=4]",
  "?10 * fixed_bytes=1996527793, align=4]",
  "?(10 * fixed_bytes=1996527793, align=4])",
  "$10 * fixed_bytes=1996527793, align=4]",
  "=10 * fixed_bytes=1996527793, align=4]",
  "$[10 * fixed_bytes=1996527793, align=4]]",
  "xxx[10 * fixed_bytes=1996527793, align=4]]",
  "?$10 * fixed_bytes=1996527793, align=4]]",
  "?=10 * fixed_bytes=1996527793, align=4]]",
  "?(10 * fixed_bytes=1996527793, align=4)$",
  "?(10 * fixed_bytes=1996527793, align=4)$",
  "fixed_bytes=1996527793, align=4]",
  "?fixed_bytes=1996527793, align=4]",
  "?(fixed_bytes=1996527793, align=4])",
  "$fixed_bytes=1996527793, align=4]",
  "=fixed_bytes=1996527793, align=4]",
  "$[fixed_bytes=1996527793, align=4]]",
  "xxx[fixed_bytes=1996527793, align=4]]",
  "?$fixed_bytes=1996527793, align=4]]",
  "?=fixed_bytes=1996527793, align=4]]",
  "?(fixed_bytes=1996527793, align=4)$",
  "?(fixed_bytes=1996527793, align=4)$",
  "10 $ fixed_bytes=1996527793, align=4]",
  "?10 $ fixed_bytes=1996527793, align=4]",
  "?(10 $ fixed_bytes=1996527793, align=4])",
  "$10 $ fixed_bytes=1996527793, align=4]",
  "=10 $ fixed_bytes=1996527793, align=4]",
  "$[10 $ fixed_bytes=1996527793, align=4]]",
  "xxx[10 $ fixed_bytes=1996527793, align=4]]",
  "?$10 $ fixed_bytes=1996527793, align=4]]",
  "?=10 $ fixed_bytes=1996527793, align=4]]",
  "?(10 $ fixed_bytes=1996527793, align=4)$",
  "?(10 $ fixed_bytes=1996527793, align=4)$",
  "10 = fixed_bytes=1996527793, align=4]",
  "?10 = fixed_bytes=1996527793, align=4]",
  "?(10 = fixed_bytes=1996527793, align=4])",
  "$10 = fixed_bytes=1996527793, align=4]",
  "=10 = fixed_bytes=1996527793, align=4]",
  "$[10 = fixed_bytes=1996527793, align=4]]",
  "xxx[10 = fixed_bytes=1996527793, align=4]]",
  "?$10 = fixed_bytes=1996527793, align=4]]",
  "?=10 = fixed_bytes=1996527793, align=4]]",
  "?(10 = fixed_bytes=1996527793, align=4)$",
  "?(10 = fixed_bytes=1996527793, align=4)$",
  "10 * fixed_bytes($, align=4)",
  "?10 * fixed_bytes($, align=4)",
  "?(10 * fixed_bytes($, align=4))",
  "$10 * fixed_bytes($, align=4)",
  "=10 * fixed_bytes($, align=4)",
  "$[10 * fixed_bytes($, align=4])",
  "xxx[10 * fixed_bytes($, align=4])",
  "?$10 * fixed_bytes($, align=4])",
  "?=10 * fixed_bytes($, align=4])",
  "?[10 * fixed_bytes($, align=4)$",
  "?[10 * fixed_bytes($, align=4)$",
  "fixed_bytes($, align=4)",
  "?fixed_bytes($, align=4)",
  "?(fixed_bytes($, align=4))",
  "$fixed_bytes($, align=4)",
  "=fixed_bytes($, align=4)",
  "$[fixed_bytes($, align=4])",
  "xxx[fixed_bytes($, align=4])",
  "?$fixed_bytes($, align=4])",
  "?=fixed_bytes($, align=4])",
  "?[fixed_bytes($, align=4)$",
  "?[fixed_bytes($, align=4)$",
  "10 $ fixed_bytes($, align=4)",
  "?10 $ fixed_bytes($, align=4)",
  "?(10 $ fixed_bytes($, align=4))",
  "$10 $ fixed_bytes($, align=4)",
  "=10 $ fixed_bytes($, align=4)",
  "$[10 $ fixed_bytes($, align=4])",
  "xxx[10 $ fixed_bytes($, align=4])",
  "?$10 $ fixed_bytes($, align=4])",
  "?=10 $ fixed_bytes($, align=4])",
  "?[10 $ fixed_bytes($, align=4)$",
  "?[10 $ fixed_bytes($, align=4)$",
  "10 = fixed_bytes($, align=4)",
  "?10 = fixed_bytes($, align=4)",
  "?(10 = fixed_bytes($, align=4))",
  "$10 = fixed_bytes($, align=4)",
  "=10 = fixed_bytes($, align=4)",
  "$[10 = fixed_bytes($, align=4])",
  "xxx[10 = fixed_bytes($, align=4])",
  "?$10 = fixed_bytes($, align=4])",
  "?=10 = fixed_bytes($, align=4])",
  "?[10 = fixed_bytes($, align=4)$",
  "?[10 = fixed_bytes($, align=4)$",
  "10 * fixed_bytes(=, align=4)",
  "?10 * fixed_bytes(=, align=4)",
  "?(10 * fixed_bytes(=, align=4))",
  "$10 * fixed_bytes(=, align=4)",
  "=10 * fixed_bytes(=, align=4)",
  "$[10 * fixed_bytes(=, align=4])",
  "xxx[10 * fixed_bytes(=, align=4])",
  "?$10 * fixed_bytes(=, align=4])",
  "?=10 * fixed_bytes(=, align=4])",
  "?[10 * fixed_bytes(=, align=4)$",
  "?[10 * fixed_bytes(=, align=4)$",
  "fixed_bytes(=, align=4)",
  "?fixed_bytes(=, align=4)",
  "?(fixed_bytes(=, align=4))",
  "$fixed_bytes(=, align=4)",
  "=fixed_bytes(=, align=4)",
  "$[fixed_bytes(=, align=4])",
  "xxx[fixed_bytes(=, align=4])",
  "?$fixed_bytes(=, align=4])",
  "?=fixed_bytes(=, align=4])",
  "?[fixed_bytes(=, align=4)$",
  "?[fixed_bytes(=, align=4)$",
  "10 $ fixed_bytes(=, align=4)",
  "?10 $ fixed_bytes(=, align=4)",
  "?(10 $ fixed_bytes(=, align=4))",
  "$10 $ fixed_bytes(=, align=4)",
  "=10 $ fixed_bytes(=, align=4)",
  "$[10 $ fixed_bytes(=, align=4])",
  "xxx[10 $ fixed_bytes(=, align=4])",
  "?$10 $ fixed_bytes(=, align=4])",
  "?=10 $ fixed_bytes(=, align=4])",
  "?[10 $ fixed_bytes(=, align=4)$",
  "?[10 $ fixed_bytes(=, align=4)$",
  "10 = fixed_bytes(=, align=4)",
  "?10 = fixed_bytes(=, align=4)",
  "?(10 = fixed_bytes(=, align=4))",
  "$10 = fixed_bytes(=, align=4)",
  "=10 = fixed_bytes(=, align=4)",
  "$[10 = fixed_bytes(=, align=4])",
  "xxx[10 = fixed_bytes(=, align=4])",
  "?$10 = fixed_bytes(=, align=4])",
  "?=10 = fixed_bytes(=, align=4])",
  "?[10 = fixed_bytes(=, align=4)$",
  "?[10 = fixed_bytes(=, align=4)$",
  "10 * fixed_bytes(10000000000000000000000000000000000000000000000, align=4)",
  "?10 * fixed_bytes(10000000000000000000000000000000000000000000000, align=4)",
  "?(10 * fixed_bytes(10000000000000000000000000000000000000000000000, align=4))",
  "$10 * fixed_bytes(10000000000000000000000000000000000000000000000, align=4)",
  "=10 * fixed_bytes(10000000000000000000000000000000000000000000000, align=4)",
  "$[10 * fixed_bytes(10000000000000000000000000000000000000000000000, align=4])",
  "xxx[10 * fixed_bytes(10000000000000000000000000000000000000000000000, align=4])",
  "?$10 * fixed_bytes(10000000000000000000000000000000000000000000000, align=4])",
  "?=10 * fixed_bytes(10000000000000000000000000000000000000000000000, align=4])",
  "?[10 * fixed_bytes(10000000000000000000000000000000000000000000000, align=4)$",
  "?[10 * fixed_bytes(10000000000000000000000000000000000000000000000, align=4)$",
  "fixed_bytes(10000000000000000000000000000000000000000000000, align=4)",
  "?fixed_bytes(10000000000000000000000000000000000000000000000, align=4)",
  "?(fixed_bytes(10000000000000000000000000000000000000000000000, align=4))",
  "$fixed_bytes(10000000000000000000000000000000000000000000000, align=4)",
  "=fixed_bytes(10000000000000000000000000000000000000000000000, align=4)",
  "$[fixed_bytes(10000000000000000000000000000000000000000000000, align=4])",
  "xxx[fixed_bytes(10000000000000000000000000000000000000000000000, align=4])",
  "?$fixed_bytes(10000000000000000000000000000000000000000000000, align=4])",
  "?=fixed_bytes(10000000000000000000000000000000000000000000000, align=4])",
  "?[fixed_bytes(10000000000000000000000000000000000000000000000, align=4)$",
  "?[fixed_bytes(10000000000000000000000000000000000000000000000, align=4)$",
  "10 $ fixed_bytes(10000000000000000000000000000000000000000000000, align=4)",
  "?10 $ fixed_bytes(10000000000000000000000000000000000000000000000, align=4)",
  "?(10 $ fixed_bytes(10000000000000000000000000000000000000000000000, align=4))",
  "$10 $ fixed_bytes(10000000000000000000000000000000000000000000000, align=4)",
  "=10 $ fixed_bytes(10000000000000000000000000000000000000000000000, align=4)",
  "$[10 $ fixed_bytes(10000000000000000000000000000000000000000000000, align=4])",
  "xxx[10 $ fixed_bytes(10000000000000000000000000000000000000000000000, align=4])",
  "?$10 $ fixed_bytes(10000000000000000000000000000000000000000000000, align=4])",
  "?=10 $ fixed_bytes(10000000000000000000000000000000000000000000000, align=4])",
  "?[10 $ fixed_bytes(10000000000000000000000000000000000000000000000, align=4)$",
  "?[10 $ fixed_bytes(10000000000000000000000000000000000000000000000, align=4)$",
  "10 = fixed_bytes(10000000000000000000000000000000000000000000000, align=4)",
  "?10 = fixed_bytes(10000000000000000000000000000000000000000000000, align=4)",
  "?(10 = fixed_bytes(10000000000000000000000000000000000000000000000, align=4))",
  "$10 = fixed_bytes(10000000000000000000000000000000000000000000000, align=4)",
  "=10 = fixed_bytes(10000000000000000000000000000000000000000000000, align=4)",
  "$[10 = fixed_bytes(10000000000000000000000000000000000000000000000, align=4])",
  "xxx[10 = fixed_bytes(10000000000000000000000000000000000000000000000, align=4])",
  "?$10 = fixed_bytes(10000000000000000000000000000000000000000000000, align=4])",
  "?=10 = fixed_bytes(10000000000000000000000000000000000000000000000, align=4])",
  "?[10 = fixed_bytes(10000000000000000000000000000000000000000000000, align=4)$",
  "?[10 = fixed_bytes(10000000000000000000000000000000000000000000000, align=4)$",
  "10 * fixed_bytes(1.2932, align=4)",
  "?10 * fixed_bytes(1.2932, align=4)",
  "?(10 * fixed_bytes(1.2932, align=4))",
  "$10 * fixed_bytes(1.2932, align=4)",
  "=10 * fixed_bytes(1.2932, align=4)",
  "$[10 * fixed_bytes(1.2932, align=4])",
  "xxx[10 * fixed_bytes(1.2932, align=4])",
  "?$10 * fixed_bytes(1.2932, align=4])",
  "?=10 * fixed_bytes(1.2932, align=4])",
  "?[10 * fixed_bytes(1.2932, align=4)$",
  "?[10 * fixed_bytes(1.2932, align=4)$",
  "fixed_bytes(1.2932, align=4)",
  "?fixed_bytes(1.2932, align=4)",
  "?(fixed_bytes(1.2932, align=4))",
  "$fixed_bytes(1.2932, align=4)",
  "=fixed_bytes(1.2932, align=4)",
  "$[fixed_bytes(1.2932, align=4])",
  "xxx[fixed_bytes(1.2932, align=4])",
  "?$fixed_bytes(1.2932, align=4])",
  "?=fixed_bytes(1.2932, align=4])",
  "?[fixed_bytes(1.2932, align=4)$",
  "?[fixed_bytes(1.2932, align=4)$",
  "10 $ fixed_bytes(1.2932, align=4)",
  "?10 $ fixed_bytes(1.2932, align=4)",
  "?(10 $ fixed_bytes(1.2932, align=4))",
  "$10 $ fixed_bytes(1.2932, align=4)",
  "=10 $ fixed_bytes(1.2932, align=4)",
  "$[10 $ fixed_bytes(1.2932, align=4])",
  "xxx[10 $ fixed_bytes(1.2932, align=4])",
  "?$10 $ fixed_bytes(1.2932, align=4])",
  "?=10 $ fixed_bytes(1.2932, align=4])",
  "?[10 $ fixed_bytes(1.2932, align=4)$",
  "?[10 $ fixed_bytes(1.2932, align=4)$",
  "10 = fixed_bytes(1.2932, align=4)",
  "?10 = fixed_bytes(1.2932, align=4)",
  "?(10 = fixed_bytes(1.2932, align=4))",
  "$10 = fixed_bytes(1.2932, align=4)",
  "=10 = fixed_bytes(1.2932, align=4)",
  "$[10 = fixed_bytes(1.2932, align=4])",
  "xxx[10 = fixed_bytes(1.2932, align=4])",
  "?$10 = fixed_bytes(1.2932, align=4])",
  "?=10 = fixed_bytes(1.2932, align=4])",
  "?[10 = fixed_bytes(1.2932, align=4)$",
  "?[10 = fixed_bytes(1.2932, align=4)$",
  "10 * fixed_bytes(2312411748$align=4)",
  "?10 * fixed_bytes(2312411748$align=4)",
  "?(10 * fixed_bytes(2312411748$align=4))",
  "$10 * fixed_bytes(2312411748$align=4)",
  "=10 * fixed_bytes(2312411748$align=4)",
  "$[10 * fixed_bytes(2312411748$align=4])",
  "xxx[10 * fixed_bytes(2312411748$align=4])",
  "?$10 * fixed_bytes(2312411748$align=4])",
  "?=10 * fixed_bytes(2312411748$align=4])",
  "?[10 * fixed_bytes(2312411748$align=4)$",
  "?[10 * fixed_bytes(2312411748$align=4)$",
  "fixed_bytes(2312411748$align=4)",
  "?fixed_bytes(2312411748$align=4)",
  "?(fixed_bytes(2312411748$align=4))",
  "$fixed_bytes(2312411748$align=4)",
  "=fixed_bytes(2312411748$align=4)",
  "$[fixed_bytes(2312411748$align=4])",
  "xxx[fixed_bytes(2312411748$align=4])",
  "?$fixed_bytes(2312411748$align=4])",
  "?=fixed_bytes(2312411748$align=4])",
  "?[fixed_bytes(2312411748$align=4)$",
  "?[fixed_bytes(2312411748$align=4)$",
  "10 $ fixed_bytes(2312411748$align=4)",
  "?10 $ fixed_bytes(2312411748$align=4)",
  "?(10 $ fixed_bytes(2312411748$align=4))",
  "$10 $ fixed_bytes(2312411748$align=4)",
  "=10 $ fixed_bytes(2312411748$align=4)",
  "$[10 $ fixed_bytes(2312411748$align=4])",
  "xxx[10 $ fixed_bytes(2312411748$align=4])",
  "?$10 $ fixed_bytes(2312411748$align=4])",
  "?=10 $ fixed_bytes(2312411748$align=4])",
  "?[10 $ fixed_bytes(2312411748$align=4)$",
  "?[10 $ fixed_bytes(2312411748$align=4)$",
  "10 = fixed_bytes(2312411748$align=4)",
  "?10 = fixed_bytes(2312411748$align=4)",
  "?(10 = fixed_bytes(2312411748$align=4))",
  "$10 = fixed_bytes(2312411748$align=4)",
  "=10 = fixed_bytes(2312411748$align=4)",
  "$[10 = fixed_bytes(2312411748$align=4])",
  "xxx[10 = fixed_bytes(2312411748$align=4])",
  "?$10 = fixed_bytes(2312411748$align=4])",
  "?=10 = fixed_bytes(2312411748$align=4])",
  "?[10 = fixed_bytes(2312411748$align=4)$",
  "?[10 = fixed_bytes(2312411748$align=4)$",
  "10 * fixed_bytes(3391780523=align=4)",
  "?10 * fixed_bytes(3391780523=align=4)",
  "?(10 * fixed_bytes(3391780523=align=4))",
  "$10 * fixed_bytes(3391780523=align=4)",
  "=10 * fixed_bytes(3391780523=align=4)",
  "$[10 * fixed_bytes(3391780523=align=4])",
  "xxx[10 * fixed_bytes(3391780523=align=4])",
  "?$10 * fixed_bytes(3391780523=align=4])",
  "?=10 * fixed_bytes(3391780523=align=4])",
  "?[10 * fixed_bytes(3391780523=align=4)$",
  "?[10 * fixed_bytes(3391780523=align=4)$",
  "fixed_bytes(3391780523=align=4)",
  "?fixed_bytes(3391780523=align=4)",
  "?(fixed_bytes(3391780523=align=4))",
  "$fixed_bytes(3391780523=align=4)",
  "=fixed_bytes(3391780523=align=4)",
  "$[fixed_bytes(3391780523=align=4])",
  "xxx[fixed_bytes(3391780523=align=4])",
  "?$fixed_bytes(3391780523=align=4])",
  "?=fixed_bytes(3391780523=align=4])",
  "?[fixed_bytes(3391780523=align=4)$",
  "?[fixed_bytes(3391780523=align=4)$",
  "10 $ fixed_bytes(3391780523=align=4)",
  "?10 $ fixed_bytes(3391780523=align=4)",
  "?(10 $ fixed_bytes(3391780523=align=4))",
  "$10 $ fixed_bytes(3391780523=align=4)",
  "=10 $ fixed_bytes(3391780523=align=4)",
  "$[10 $ fixed_bytes(3391780523=align=4])",
  "xxx[10 $ fixed_bytes(3391780523=align=4])",
  "?$10 $ fixed_bytes(3391780523=align=4])",
  "?=10 $ fixed_bytes(3391780523=align=4])",
  "?[10 $ fixed_bytes(3391780523=align=4)$",
  "?[10 $ fixed_bytes(3391780523=align=4)$",
  "10 = fixed_bytes(3391780523=align=4)",
  "?10 = fixed_bytes(3391780523=align=4)",
  "?(10 = fixed_bytes(3391780523=align=4))",
  "$10 = fixed_bytes(3391780523=align=4)",
  "=10 = fixed_bytes(3391780523=align=4)",
  "$[10 = fixed_bytes(3391780523=align=4])",
  "xxx[10 = fixed_bytes(3391780523=align=4])",
  "?$10 = fixed_bytes(3391780523=align=4])",
  "?=10 = fixed_bytes(3391780523=align=4])",
  "?[10 = fixed_bytes(3391780523=align=4)$",
  "?[10 = fixed_bytes(3391780523=align=4)$",
  "10 * fixed_bytes(3799460819, $=180)",
  "?10 * fixed_bytes(3799460819, $=180)",
  "?(10 * fixed_bytes(3799460819, $=180))",
  "$10 * fixed_bytes(3799460819, $=180)",
  "=10 * fixed_bytes(3799460819, $=180)",
  "$[10 * fixed_bytes(3799460819, $=180])",
  "xxx[10 * fixed_bytes(3799460819, $=180])",
  "?$10 * fixed_bytes(3799460819, $=180])",
  "?=10 * fixed_bytes(3799460819, $=180])",
  "?[10 * fixed_bytes(3799460819, $=180)$",
  "?[10 * fixed_bytes(3799460819, $=180)$",
  "fixed_bytes(3799460819, $=180)",
  "?fixed_bytes(3799460819, $=180)",
  "?(fixed_bytes(3799460819, $=180))",
  "$fixed_bytes(3799460819, $=180)",
  "=fixed_bytes(3799460819, $=180)",
  "$[fixed_bytes(3799460819, $=180])",
  "xxx[fixed_bytes(3799460819, $=180])",
  "?$fixed_bytes(3799460819, $=180])",
  "?=fixed_bytes(3799460819, $=180])",
  "?[fixed_bytes(3799460819, $=180)$",
  "?[fixed_bytes(3799460819, $=180)$",
  "10 $ fixed_bytes(3799460819, $=180)",
  "?10 $ fixed_bytes(3799460819, $=180)",
  "?(10 $ fixed_bytes(3799460819, $=180))",
  "$10 $ fixed_bytes(3799460819, $=180)",
  "=10 $ fixed_bytes(3799460819, $=180)",
  "$[10 $ fixed_bytes(3799460819, $=180])",
  "xxx[10 $ fixed_bytes(3799460819, $=180])",
  "?$10 $ fixed_bytes(3799460819, $=180])",
  "?=10 $ fixed_bytes(3799460819, $=180])",
  "?[10 $ fixed_bytes(3799460819, $=180)$",
  "?[10 $ fixed_bytes(3799460819, $=180)$",
  "10 = fixed_bytes(3799460819, $=180)",
  "?10 = fixed_bytes(3799460819, $=180)",
  "?(10 = fixed_bytes(3799460819, $=180))",
  "$10 = fixed_bytes(3799460819, $=180)",
  "=10 = fixed_bytes(3799460819, $=180)",
  "$[10 = fixed_bytes(3799460819, $=180])",
  "xxx[10 = fixed_bytes(3799460819, $=180])",
  "?$10 = fixed_bytes(3799460819, $=180])",
  "?=10 = fixed_bytes(3799460819, $=180])",
  "?[10 = fixed_bytes(3799460819, $=180)$",
  "?[10 = fixed_bytes(3799460819, $=180)$",
  "10 * fixed_bytes(120796144, xxx=160)",
  "?10 * fixed_bytes(120796144, xxx=160)",
  "?(10 * fixed_bytes(120796144, xxx=160))",
  "$10 * fixed_bytes(120796144, xxx=160)",
  "=10 * fixed_bytes(120796144, xxx=160)",
  "$[10 * fixed_bytes(120796144, xxx=160])",
  "xxx[10 * fixed_bytes(120796144, xxx=160])",
  "?$10 * fixed_bytes(120796144, xxx=160])",
  "?=10 * fixed_bytes(120796144, xxx=160])",
  "?[10 * fixed_bytes(120796144, xxx=160)$",
  "?[10 * fixed_bytes(120796144, xxx=160)$",
  "fixed_bytes(120796144, xxx=160)",
  "?fixed_bytes(120796144, xxx=160)",
  "?(fixed_bytes(120796144, xxx=160))",
  "$fixed_bytes(120796144, xxx=160)",
  "=fixed_bytes(120796144, xxx=160)",
  "$[fixed_bytes(120796144, xxx=160])",
  "xxx[fixed_bytes(120796144, xxx=160])",
  "?$fixed_bytes(120796144, xxx=160])",
  "?=fixed_bytes(120796144, xxx=160])",
  "?[fixed_bytes(120796144, xxx=160)$",
  "?[fixed_bytes(120796144, xxx=160)$",
  "10 $ fixed_bytes(120796144, xxx=160)",
  "?10 $ fixed_bytes(120796144, xxx=160)",
  "?(10 $ fixed_bytes(120796144, xxx=160))",
  "$10 $ fixed_bytes(120796144, xxx=160)",
  "=10 $ fixed_bytes(120796144, xxx=160)",
  "$[10 $ fixed_bytes(120796144, xxx=160])",
  "xxx[10 $ fixed_bytes(120796144, xxx=160])",
  "?$10 $ fixed_bytes(120796144, xxx=160])",
  "?=10 $ fixed_bytes(120796144, xxx=160])",
  "?[10 $ fixed_bytes(120796144, xxx=160)$",
  "?[10 $ fixed_bytes(120796144, xxx=160)$",
  "10 = fixed_bytes(120796144, xxx=160)",
  "?10 = fixed_bytes(120796144, xxx=160)",
  "?(10 = fixed_bytes(120796144, xxx=160))",
  "$10 = fixed_bytes(120796144, xxx=160)",
  "=10 = fixed_bytes(120796144, xxx=160)",
  "$[10 = fixed_bytes(120796144, xxx=160])",
  "xxx[10 = fixed_bytes(120796144, xxx=160])",
  "?$10 = fixed_bytes(120796144, xxx=160])",
  "?=10 = fixed_bytes(120796144, xxx=160])",
  "?[10 = fixed_bytes(120796144, xxx=160)$",
  "?[10 = fixed_bytes(120796144, xxx=160)$",
  "10 * fixed_bytes(2132477446, align$177)",
  "?10 * fixed_bytes(2132477446, align$177)",
  "?(10 * fixed_bytes(2132477446, align$177))",
  "$10 * fixed_bytes(2132477446, align$177)",
  "=10 * fixed_bytes(2132477446, align$177)",
  "$[10 * fixed_bytes(2132477446, align$177])",
  "xxx[10 * fixed_bytes(2132477446, align$177])",
  "?$10 * fixed_bytes(2132477446, align$177])",
  "?=10 * fixed_bytes(2132477446, align$177])",
  "?[10 * fixed_bytes(2132477446, align$177)$",
  "?[10 * fixed_bytes(2132477446, align$177)$",
  "fixed_bytes(2132477446, align$177)",
  "?fixed_bytes(2132477446, align$177)",
  "?(fixed_bytes(2132477446, align$177))",
  "$fixed_bytes(2132477446, align$177)",
  "=fixed_bytes(2132477446, align$177)",
  "$[fixed_bytes(2132477446, align$177])",
  "xxx[fixed_bytes(2132477446, align$177])",
  "?$fixed_bytes(2132477446, align$177])",
  "?=fixed_bytes(2132477446, align$177])",
  "?[fixed_bytes(2132477446, align$177)$",
  "?[fixed_bytes(2132477446, align$177)$",
  "10 $ fixed_bytes(2132477446, align$177)",
  "?10 $ fixed_bytes(2132477446, align$177)",
  "?(10 $ fixed_bytes(2132477446, align$177))",
  "$10 $ fixed_bytes(2132477446, align$177)",
  "=10 $ fixed_bytes(2132477446, align$177)",
  "$[10 $ fixed_bytes(2132477446, align$177])",
  "xxx[10 $ fixed_bytes(2132477446, align$177])",
  "?$10 $ fixed_bytes(2132477446, align$177])",
  "?=10 $ fixed_bytes(2132477446, align$177])",
  "?[10 $ fixed_bytes(2132477446, align$177)$",
  "?[10 $ fixed_bytes(2132477446, align$177)$",
  "10 = fixed_bytes(2132477446, align$177)",
  "?10 = fixed_bytes(2132477446, align$177)",
  "?(10 = fixed_bytes(2132477446, align$177))",
  "$10 = fixed_bytes(2132477446, align$177)",
  "=10 = fixed_bytes(2132477446, align$177)",
  "$[10 = fixed_bytes(2132477446, align$177])",
  "xxx[10 = fixed_bytes(2132477446, align$177])",
  "?$10 = fixed_bytes(2132477446, align$177])",
  "?=10 = fixed_bytes(2132477446, align$177])",
  "?[10 = fixed_bytes(2132477446, align$177)$",
  "?[10 = fixed_bytes(2132477446, align$177)$",
  "10 * fixed_bytes(4221343387, align==183)",
  "?10 * fixed_bytes(4221343387, align==183)",
  "?(10 * fixed_bytes(4221343387, align==183))",
  "$10 * fixed_bytes(4221343387, align==183)",
  "=10 * fixed_bytes(4221343387, align==183)",
  "$[10 * fixed_bytes(4221343387, align==183])",
  "xxx[10 * fixed_bytes(4221343387, align==183])",
  "?$10 * fixed_bytes(4221343387, align==183])",
  "?=10 * fixed_bytes(4221343387, align==183])",
  "?[10 * fixed_bytes(4221343387, align==183)$",
  "?[10 * fixed_bytes(4221343387, align==183)$",
  "fixed_bytes(4221343387, align==183)",
  "?fixed_bytes(4221343387, align==183)",
  "?(fixed_bytes(4221343387, align==183))",
  "$fixed_bytes(4221343387, align==183)",
  "=fixed_bytes(4221343387, align==183)",
  "$[fixed_bytes(4221343387, align==183])",
  "xxx[fixed_bytes(4221343387, align==183])",
  "?$fixed_bytes(4221343387, align==183])",
  "?=fixed_bytes(4221343387, align==183])",
  "?[fixed_bytes(4221343387, align==183)$",
  "?[fixed_bytes(4221343387, align==183)$",
  "10 $ fixed_bytes(4221343387, align==183)",
  "?10 $ fixed_bytes(4221343387, align==183)",
  "?(10 $ fixed_bytes(4221343387, align==183))",
  "$10 $ fixed_bytes(4221343387, align==183)",
  "=10 $ fixed_bytes(4221343387, align==183)",
  "$[10 $ fixed_bytes(4221343387, align==183])",
  "xxx[10 $ fixed_bytes(4221343387, align==183])",
  "?$10 $ fixed_bytes(4221343387, align==183])",
  "?=10 $ fixed_bytes(4221343387, align==183])",
  "?[10 $ fixed_bytes(4221343387, align==183)$",
  "?[10 $ fixed_bytes(4221343387, align==183)$",
  "10 = fixed_bytes(4221343387, align==183)",
  "?10 = fixed_bytes(4221343387, align==183)",
  "?(10 = fixed_bytes(4221343387, align==183))",
  "$10 = fixed_bytes(4221343387, align==183)",
  "=10 = fixed_bytes(4221343387, align==183)",
  "$[10 = fixed_bytes(4221343387, align==183])",
  "xxx[10 = fixed_bytes(4221343387, align==183])",
  "?$10 = fixed_bytes(4221343387, align==183])",
  "?=10 = fixed_bytes(4221343387, align==183])",
  "?[10 = fixed_bytes(4221343387, align==183)$",
  "?[10 = fixed_bytes(4221343387, align==183)$",
  "10 * fixed_bytes(1598058055, align=10000000000000000000000000000000000000000000000000000)",
  "?10 * fixed_bytes(1598058055, align=10000000000000000000000000000000000000000000000000000)",
  "?(10 * fixed_bytes(1598058055, align=10000000000000000000000000000000000000000000000000000))",
  "$10 * fixed_bytes(1598058055, align=10000000000000000000000000000000000000000000000000000)",
  "=10 * fixed_bytes(1598058055, align=10000000000000000000000000000000000000000000000000000)",
  "$[10 * fixed_bytes(1598058055, align=10000000000000000000000000000000000000000000000000000])",
  "xxx[10 * fixed_bytes(1598058055, align=10000000000000000000000000000000000000000000000000000])",
  "?$10 * fixed_bytes(1598058055, align=10000000000000000000000000000000000000000000000000000])",
  "?=10 * fixed_bytes(1598058055, align=10000000000000000000000000000000000000000000000000000])",
  "?[10 * fixed_bytes(1598058055, align=10000000000000000000000000000000000000000000000000000)$",
  "?[10 * fixed_bytes(1598058055, align=10000000000000000000000000000000000000000000000000000)$",
  "fixed_bytes(1598058055, align=10000000000000000000000000000000000000000000000000000)",
  "?fixed_bytes(1598058055, align=10000000000000000000000000000000000000000000000000000)",
  "?(fixed_bytes(1598058055, align=10000000000000000000000000000000000000000000000000000))",
  "$fixed_bytes(1598058055, align=10000000000000000000000000000000000000000000000000000)",
  "=fixed_bytes(1598058055, align=10000000000000000000000000000000000000000000000000000)",
  "$[fixed_bytes(1598058055, align=10000000000000000000000000000000000000000000000000000])",
  "xxx[fixed_bytes(1598058055, align=10000000000000000000000000000000000000000000000000000])",
  "?$fixed_bytes(1598058055, align=10000000000000000000000000000000000000000000000000000])",
  "?=fixed_bytes(1598058055, align=10000000000000000000000000000000000000000000000000000])",
  "?[fixed_bytes(1598058055, align=10000000000000000000000000000000000000000000000000000)$",
  "?[fixed_bytes(1598058055, align=10000000000000000000000000000000000000000000000000000)$",
  "10 $ fixed_bytes(1598058055, align=10000000000000000000000000000000000000000000000000000)",
  "?10 $ fixed_bytes(1598058055, align=10000000000000000000000000000000000000000000000000000)",
  "?(10 $ fixed_bytes(1598058055, align=10000000000000000000000000000000000000000000000000000))",
  "$10 $ fixed_bytes(1598058055, align=10000000000000000000000000000000000000000000000000000)",
  "=10 $ fixed_bytes(1598058055, align=10000000000000000000000000000000000000000000000000000)",
  "$[10 $ fixed_bytes(1598058055, align=10000000000000000000000000000000000000000000000000000])",
  "xxx[10 $ fixed_bytes(1598058055, align=10000000000000000000000000000000000000000000000000000])",
  "?$10 $ fixed_bytes(1598058055, align=10000000000000000000000000000000000000000000000000000])",
  "?=10 $ fixed_bytes(1598058055, align=10000000000000000000000000000000000000000000000000000])",
  "?[10 $ fixed_bytes(1598058055, align=10000000000000000000000000000000000000000000000000000)$",
  "?[10 $ fixed_bytes(1598058055, align=10000000000000000000000000000000000000000000000000000)$",
  "10 = fixed_bytes(1598058055, align=10000000000000000000000000000000000000000000000000000)",
  "?10 = fixed_bytes(1598058055, align=10000000000000000000000000000000000000000000000000000)",
  "?(10 = fixed_bytes(1598058055, align=10000000000000000000000000000000000000000000000000000))",
  "$10 = fixed_bytes(1598058055, align=10000000000000000000000000000000000000000000000000000)",
  "=10 = fixed_bytes(1598058055, align=10000000000000000000000000000000000000000000000000000)",
  "$[10 = fixed_bytes(1598058055, align=10000000000000000000000000000000000000000000000000000])",
  "xxx[10 = fixed_bytes(1598058055, align=10000000000000000000000000000000000000000000000000000])",
  "?$10 = fixed_bytes(1598058055, align=10000000000000000000000000000000000000000000000000000])",
  "?=10 = fixed_bytes(1598058055, align=10000000000000000000000000000000000000000000000000000])",
  "?[10 = fixed_bytes(1598058055, align=10000000000000000000000000000000000000000000000000000)$",
  "?[10 = fixed_bytes(1598058055, align=10000000000000000000000000000000000000000000000000000)$",
  "10 * fixed_bytes(3169422723, align='')",
  "?10 * fixed_bytes(3169422723, align='')",
  "?(10 * fixed_bytes(3169422723, align=''))",
  "$10 * fixed_bytes(3169422723, align='')",
  "=10 * fixed_bytes(3169422723, align='')",
  "$[10 * fixed_bytes(3169422723, align=''])",
  "xxx[10 * fixed_bytes(3169422723, align=''])",
  "?$10 * fixed_bytes(3169422723, align=''])",
  "?=10 * fixed_bytes(3169422723, align=''])",
  "?[10 * fixed_bytes(3169422723, align='')$",
  "?[10 * fixed_bytes(3169422723, align='')$",
  "fixed_bytes(3169422723, align='')",
  "?fixed_bytes(3169422723, align='')",
  "?(fixed_bytes(3169422723, align=''))",
  "$fixed_bytes(3169422723, align='')",
  "=fixed_bytes(3169422723, align='')",
  "$[fixed_bytes(3169422723, align=''])",
  "xxx[fixed_bytes(3169422723, align=''])",
  "?$fixed_bytes(3169422723, align=''])",
  "?=fixed_bytes(3169422723, align=''])",
  "?[fixed_bytes(3169422723, align='')$",
  "?[fixed_bytes(3169422723, align='')$",
  "10 $ fixed_bytes(3169422723, align='')",
  "?10 $ fixed_bytes(3169422723, align='')",
  "?(10 $ fixed_bytes(3169422723, align=''))",
  "$10 $ fixed_bytes(3169422723, align='')",
  "=10 $ fixed_bytes(3169422723, align='')",
  "$[10 $ fixed_bytes(3169422723, align=''])",
  "xxx[10 $ fixed_bytes(3169422723, align=''])",
  "?$10 $ fixed_bytes(3169422723, align=''])",
  "?=10 $ fixed_bytes(3169422723, align=''])",
  "?[10 $ fixed_bytes(3169422723, align='')$",
  "?[10 $ fixed_bytes(3169422723, align='')$",
  "10 = fixed_bytes(3169422723, align='')",
  "?10 = fixed_bytes(3169422723, align='')",
  "?(10 = fixed_bytes(3169422723, align=''))",
  "$10 = fixed_bytes(3169422723, align='')",
  "=10 = fixed_bytes(3169422723, align='')",
  "$[10 = fixed_bytes(3169422723, align=''])",
  "xxx[10 = fixed_bytes(3169422723, align=''])",
  "?$10 = fixed_bytes(3169422723, align=''])",
  "?=10 = fixed_bytes(3169422723, align=''])",
  "?[10 = fixed_bytes(3169422723, align='')$",
  "?[10 = fixed_bytes(3169422723, align='')$",
  "10 * fixed_bytes[219083220, align=25$",
  "?10 * fixed_bytes[219083220, align=25$",
  "?[10 * fixed_bytes(219083220, align=25$)",
  "$10 * fixed_bytes[219083220, align=25$",
  "=10 * fixed_bytes[219083220, align=25$",
  "$[10 * fixed_bytes(219083220, align=25$)",
  "xxx[10 * fixed_bytes(219083220, align=25$)",
  "?$10 * fixed_bytes(219083220, align=25$)",
  "?=10 * fixed_bytes(219083220, align=25$)",
  "?[10 * fixed_bytes[219083220, align=25$$",
  "?[10 * fixed_bytes[219083220, align=25$$",
  "fixed_bytes[219083220, align=25$",
  "?fixed_bytes[219083220, align=25$",
  "?[fixed_bytes(219083220, align=25$)",
  "$fixed_bytes[219083220, align=25$",
  "=fixed_bytes[219083220, align=25$",
  "$[fixed_bytes(219083220, align=25$)",
  "xxx[fixed_bytes(219083220, align=25$)",
  "?$fixed_bytes(219083220, align=25$)",
  "?=fixed_bytes(219083220, align=25$)",
  "?[fixed_bytes[219083220, align=25$$",
  "?[fixed_bytes[219083220, align=25$$",
  "10 $ fixed_bytes[219083220, align=25$",
  "?10 $ fixed_bytes[219083220, align=25$",
  "?[10 $ fixed_bytes(219083220, align=25$)",
  "$10 $ fixed_bytes[219083220, align=25$",
  "=10 $ fixed_bytes[219083220, align=25$",
  "$[10 $ fixed_bytes(219083220, align=25$)",
  "xxx[10 $ fixed_bytes(219083220, align=25$)",
  "?$10 $ fixed_bytes(219083220, align=25$)",
  "?=10 $ fixed_bytes(219083220, align=25$)",
  "?[10 $ fixed_bytes[219083220, align=25$$",
  "?[10 $ fixed_bytes[219083220, align=25$$",
  "10 = fixed_bytes[219083220, align=25$",
  "?10 = fixed_bytes[219083220, align=25$",
  "?[10 = fixed_bytes(219083220, align=25$)",
  "$10 = fixed_bytes[219083220, align=25$",
  "=10 = fixed_bytes[219083220, align=25$",
  "$[10 = fixed_bytes(219083220, align=25$)",
  "xxx[10 = fixed_bytes(219083220, align=25$)",
  "?$10 = fixed_bytes(219083220, align=25$)",
  "?=10 = fixed_bytes(219083220, align=25$)",
  "?[10 = fixed_bytes[219083220, align=25$$",
  "?[10 = fixed_bytes[219083220, align=25$$",
  "$10 * categorical[36]",
  "=10 * categorical[36]",
  "$[10 * categorical[36]]",
  "xxx[10 * categorical[36]]",
  "?$10 * categorical[36]]",
  "?=10 * categorical[36]]",
  "?[10 * categorical[36]$",
  "?[10 * categorical[36]$",
  "$categorical[36]",
  "=categorical[36]",
  "$[categorical[36]]",
  "xxx[categorical[36]]",
  "?$categorical[36]]",
  "?=categorical[36]]",
  "?[categorical[36]$",
  "?[categorical[36]$",
  "10 $ categorical[36]",
  "?10 $ categorical[36]",
  "?[10 $ categorical[36]]",
  "$10 $ categorical[36]",
  "=10 $ categorical[36]",
  "$[10 $ categorical[36]]",
  "xxx[10 $ categorical[36]]",
  "?$10 $ categorical[36]]",
  "?=10 $ categorical[36]]",
  "?[10 $ categorical[36]$",
  "?[10 $ categorical[36]$",
  "10 = categorical[36]",
  "?10 = categorical[36]",
  "?[10 = categorical[36]]",
  "$10 = categorical[36]",
  "=10 = categorical[36]",
  "$[10 = categorical[36]]",
  "xxx[10 = categorical[36]]",
  "?$10 = categorical[36]]",
  "?=10 = categorical[36]]",
  "?[10 = categorical[36]$",
  "?[10 = categorical[36]$",
  "$10 * categorical[10, 36]",
  "=10 * categorical[10, 36]",
  "$[10 * categorical[10, 36]]",
  "xxx[10 * categorical[10, 36]]",
  "?$10 * categorical[10, 36]]",
  "?=10 * categorical[10, 36]]",
  "?[10 * categorical[10, 36]$",
  "?[10 * categorical[10, 36]$",
  "$categorical[10, 36]",
  "=categorical[10, 36]",
  "$[categorical[10, 36]]",
  "xxx[categorical[10, 36]]",
  "?$categorical[10, 36]]",
  "?=categorical[10, 36]]",
  "?[categorical[10, 36]$",
  "?[categorical[10, 36]$",
  "10 $ categorical[10, 36]",
  "?10 $ categorical[10, 36]",
  "?[10 $ categorical[10, 36]]",
  "$10 $ categorical[10, 36]",
  "=10 $ categorical[10, 36]",
  "$[10 $ categorical[10, 36]]",
  "xxx[10 $ categorical[10, 36]]",
  "?$10 $ categorical[10, 36]]",
  "?=10 $ categorical[10, 36]]",
  "?[10 $ categorical[10, 36]$",
  "?[10 $ categorical[10, 36]$",
  "10 = categorical[10, 36]",
  "?10 = categorical[10, 36]",
  "?[10 = categorical[10, 36]]",
  "$10 = categorical[10, 36]",
  "=10 = categorical[10, 36]",
  "$[10 = categorical[10, 36]]",
  "xxx[10 = categorical[10, 36]]",
  "?$10 = categorical[10, 36]]",
  "?=10 = categorical[10, 36]]",
  "?[10 = categorical[10, 36]$",
  "?[10 = categorical[10, 36]$",
  "$10 * categorical[10, 10, 36]",
  "=10 * categorical[10, 10, 36]",
  "$[10 * categorical[10, 10, 36]]",
  "xxx[10 * categorical[10, 10, 36]]",
  "?$10 * categorical[10, 10, 36]]",
  "?=10 * categorical[10, 10, 36]]",
  "?[10 * categorical[10, 10, 36]$",
  "?[10 * categorical[10, 10, 36]$",
  "$categorical[10, 10, 36]",
  "=categorical[10, 10, 36]",
  "$[categorical[10, 10, 36]]",
  "xxx[categorical[10, 10, 36]]",
  "?$categorical[10, 10, 36]]",
  "?=categorical[10, 10, 36]]",
  "?[categorical[10, 10, 36]$",
  "?[categorical[10, 10, 36]$",
  "10 $ categorical[10, 10, 36]",
  "?10 $ categorical[10, 10, 36]",
  "?[10 $ categorical[10, 10, 36]]",
  "$10 $ categorical[10, 10, 36]",
  "=10 $ categorical[10, 10, 36]",
  "$[10 $ categorical[10, 10, 36]]",
  "xxx[10 $ categorical[10, 10, 36]]",
  "?$10 $ categorical[10, 10, 36]]",
  "?=10 $ categorical[10, 10, 36]]",
  "?[10 $ categorical[10, 10, 36]$",
  "?[10 $ categorical[10, 10, 36]$",
  "10 = categorical[10, 10, 36]",
  "?10 = categorical[10, 10, 36]",
  "?[10 = categorical[10, 10, 36]]",
  "$10 = categorical[10, 10, 36]",
  "=10 = categorical[10, 10, 36]",
  "$[10 = categorical[10, 10, 36]]",
  "xxx[10 = categorical[10, 10, 36]]",
  "?$10 = categorical[10, 10, 36]]",
  "?=10 = categorical[10, 10, 36]]",
  "?[10 = categorical[10, 10, 36]$",
  "?[10 = categorical[10, 10, 36]$",
  "$10 * categorical[-17524]",
  "=10 * categorical[-17524]",
  "$[10 * categorical[-17524]]",
  "xxx[10 * categorical[-17524]]",
  "?$10 * categorical[-17524]]",
  "?=10 * categorical[-17524]]",
  "?[10 * categorical[-17524]$",
  "?[10 * categorical[-17524]$",
  "$categorical[-17524]",
  "=categorical[-17524]",
  "$[categorical[-17524]]",
  "xxx[categorical[-17524]]",
  "?$categorical[-17524]]",
  "?=categorical[-17524]]",
  "?[categorical[-17524]$",
  "?[categorical[-17524]$",
  "10 $ categorical[-17524]",
  "?10 $ categorical[-17524]",
  "?[10 $ categorical[-17524]]",
  "$10 $ categorical[-17524]",
  "=10 $ categorical[-17524]",
  "$[10 $ categorical[-17524]]",
  "xxx[10 $ categorical[-17524]]",
  "?$10 $ categorical[-17524]]",
  "?=10 $ categorical[-17524]]",
  "?[10 $ categorical[-17524]$",
  "?[10 $ categorical[-17524]$",
  "10 = categorical[-17524]",
  "?10 = categorical[-17524]",
  "?[10 = categorical[-17524]]",
  "$10 = categorical[-17524]",
  "=10 = categorical[-17524]",
  "$[10 = categorical[-17524]]",
  "xxx[10 = categorical[-17524]]",
  "?$10 = categorical[-17524]]",
  "?=10 = categorical[-17524]]",
  "?[10 = categorical[-17524]$",
  "?[10 = categorical[-17524]$",
  "$10 * categorical[10, -17524]",
  "=10 * categorical[10, -17524]",
  "$[10 * categorical[10, -17524]]",
  "xxx[10 * categorical[10, -17524]]",
  "?$10 * categorical[10, -17524]]",
  "?=10 * categorical[10, -17524]]",
  "?[10 * categorical[10, -17524]$",
  "?[10 * categorical[10, -17524]$",
  "$categorical[10, -17524]",
  "=categorical[10, -17524]",
  "$[categorical[10, -17524]]",
  "xxx[categorical[10, -17524]]",
  "?$categorical[10, -17524]]",
  "?=categorical[10, -17524]]",
  "?[categorical[10, -17524]$",
  "?[categorical[10, -17524]$",
  "10 $ categorical[10, -17524]",
  "?10 $ categorical[10, -17524]",
  "?[10 $ categorical[10, -17524]]",
  "$10 $ categorical[10, -17524]",
  "=10 $ categorical[10, -17524]",
  "$[10 $ categorical[10, -17524]]",
  "xxx[10 $ categorical[10, -17524]]",
  "?$10 $ categorical[10, -17524]]",
  "?=10 $ categorical[10, -17524]]",
  "?[10 $ categorical[10, -17524]$",
  "?[10 $ categorical[10, -17524]$",
  "10 = categorical[10, -17524]",
  "?10 = categorical[10, -17524]",
  "?[10 = categorical[10, -17524]]",
  "$10 = categorical[10, -17524]",
  "=10 = categorical[10, -17524]",
  "$[10 = categorical[10, -17524]]",
  "xxx[10 = categorical[10, -17524]]",
  "?$10 = categorical[10, -17524]]",
  "?=10 = categorical[10, -17524]]",
  "?[10 = categorical[10, -17524]$",
  "?[10 = categorical[10, -17524]$",
  "$10 * categorical[10, 10, -17524]",
  "=10 * categorical[10, 10, -17524]",
  "$[10 * categorical[10, 10, -17524]]",
  "xxx[10 * categorical[10, 10, -17524]]",
  "?$10 * categorical[10, 10, -17524]]",
  "?=10 * categorical[10, 10, -17524]]",
  "?[10 * categorical[10, 10, -17524]$",
  "?[10 * categorical[10, 10, -17524]$",
  "$categorical[10, 10, -17524]",
  "=categorical[10, 10, -17524]",
  "$[categorical[10, 10, -17524]]",
  "xxx[categorical[10, 10, -17524]]",
  "?$categorical[10, 10, -17524]]",
  "?=categorical[10, 10, -17524]]",
  "?[categorical[10, 10, -17524]$",
  "?[categorical[10, 10, -17524]$",
  "10 $ categorical[10, 10, -17524]",
  "?10 $ categorical[10, 10, -17524]",
  "?[10 $ categorical[10, 10, -17524]]",
  "$10 $ categorical[10, 10, -17524]",
  "=10 $ categorical[10, 10, -17524]",
  "$[10 $ categorical[10, 10, -17524]]",
  "xxx[10 $ categorical[10, 10, -17524]]",
  "?$10 $ categorical[10, 10, -17524]]",
  "?=10 $ categorical[10, 10, -17524]]",
  "?[10 $ categorical[10, 10, -17524]$",
  "?[10 $ categorical[10, 10, -17524]$",
  "10 = categorical[10, 10, -17524]",
  "?10 = categorical[10, 10, -17524]",
  "?[10 = categorical[10, 10, -17524]]",
  "$10 = categorical[10, 10, -17524]",
  "=10 = categorical[10, 10, -17524]",
  "$[10 = categorical[10, 10, -17524]]",
  "xxx[10 = categorical[10, 10, -17524]]",
  "?$10 = categorical[10, 10, -17524]]",
  "?=10 = categorical[10, 10, -17524]]",
  "?[10 = categorical[10, 10, -17524]$",
  "?[10 = categorical[10, 10, -17524]$",
  "$10 * categorical[-1338288587]",
  "=10 * categorical[-1338288587]",
  "$[10 * categorical[-1338288587]]",
  "xxx[10 * categorical[-1338288587]]",
  "?$10 * categorical[-1338288587]]",
  "?=10 * categorical[-1338288587]]",
  "?[10 * categorical[-1338288587]$",
  "?[10 * categorical[-1338288587]$",
  "$categorical[-1338288587]",
  "=categorical[-1338288587]",
  "$[categorical[-1338288587]]",
  "xxx[categorical[-1338288587]]",
  "?$categorical[-1338288587]]",
  "?=categorical[-1338288587]]",
  "?[categorical[-1338288587]$",
  "?[categorical[-1338288587]$",
  "10 $ categorical[-1338288587]",
  "?10 $ categorical[-1338288587]",
  "?[10 $ categorical[-1338288587]]",
  "$10 $ categorical[-1338288587]",
  "=10 $ categorical[-1338288587]",
  "$[10 $ categorical[-1338288587]]",
  "xxx[10 $ categorical[-1338288587]]",
  "?$10 $ categorical[-1338288587]]",
  "?=10 $ categorical[-1338288587]]",
  "?[10 $ categorical[-1338288587]$",
  "?[10 $ categorical[-1338288587]$",
  "10 = categorical[-1338288587]",
  "?10 = categorical[-1338288587]",
  "?[10 = categorical[-1338288587]]",
  "$10 = categorical[-1338288587]",
  "=10 = categorical[-1338288587]",
  "$[10 = categorical[-1338288587]]",
  "xxx[10 = categorical[-1338288587]]",
  "?$10 = categorical[-1338288587]]",
  "?=10 = categorical[-1338288587]]",
  "?[10 = categorical[-1338288587]$",
  "?[10 = categorical[-1338288587]$",
  "$10 * categorical[10, -1338288587]",
  "=10 * categorical[10, -1338288587]",
  "$[10 * categorical[10, -1338288587]]",
  "xxx[10 * categorical[10, -1338288587]]",
  "?$10 * categorical[10, -1338288587]]",
  "?=10 * categorical[10, -1338288587]]",
  "?[10 * categorical[10, -1338288587]$",
  "?[10 * categorical[10, -1338288587]$",
  "$categorical[10, -1338288587]",
  "=categorical[10, -1338288587]",
  "$[categorical[10, -1338288587]]",
  "xxx[categorical[10, -1338288587]]",
  "?$categorical[10, -1338288587]]",
  "?=categorical[10, -1338288587]]",
  "?[categorical[10, -1338288587]$",
  "?[categorical[10, -1338288587]$",
  "10 $ categorical[10, -1338288587]",
  "?10 $ categorical[10, -1338288587]",
  "?[10 $ categorical[10, -1338288587]]",
  "$10 $ categorical[10, -1338288587]",
  "=10 $ categorical[10, -1338288587]",
  "$[10 $ categorical[10, -1338288587]]",
  "xxx[10 $ categorical[10, -1338288587]]",
  "?$10 $ categorical[10, -1338288587]]",
  "?=10 $ categorical[10, -1338288587]]",
  "?[10 $ categorical[10, -1338288587]$",
  "?[10 $ categorical[10, -1338288587]$",
  "10 = categorical[10, -1338288587]",
  "?10 = categorical[10, -1338288587]",
  "?[10 = categorical[10, -1338288587]]",
  "$10 = categorical[10, -1338288587]",
  "=10 = categorical[10, -1338288587]",
  "$[10 = categorical[10, -1338288587]]",
  "xxx[10 = categorical[10, -1338288587]]",
  "?$10 = categorical[10, -1338288587]]",
  "?=10 = categorical[10, -1338288587]]",
  "?[10 = categorical[10, -1338288587]$",
  "?[10 = categorical[10, -1338288587]$",
  "$10 * categorical[10, 10, -1338288587]",
  "=10 * categorical[10, 10, -1338288587]",
  "$[10 * categorical[10, 10, -1338288587]]",
  "xxx[10 * categorical[10, 10, -1338288587]]",
  "?$10 * categorical[10, 10, -1338288587]]",
  "?=10 * categorical[10, 10, -1338288587]]",
  "?[10 * categorical[10, 10, -1338288587]$",
  "?[10 * categorical[10, 10, -1338288587]$",
  "$categorical[10, 10, -1338288587]",
  "=categorical[10, 10, -1338288587]",
  "$[categorical[10, 10, -1338288587]]",
  "xxx[categorical[10, 10, -1338288587]]",
  "?$categorical[10, 10, -1338288587]]",
  "?=categorical[10, 10, -1338288587]]",
  "?[categorical[10, 10, -1338288587]$",
  "?[categorical[10, 10, -1338288587]$",
  "10 $ categorical[10, 10, -1338288587]",
  "?10 $ categorical[10, 10, -1338288587]",
  "?[10 $ categorical[10, 10, -1338288587]]",
  "$10 $ categorical[10, 10, -1338288587]",
  "=10 $ categorical[10, 10, -1338288587]",
  "$[10 $ categorical[10, 10, -1338288587]]",
  "xxx[10 $ categorical[10, 10, -1338288587]]",
  "?$10 $ categorical[10, 10, -1338288587]]",
  "?=10 $ categorical[10, 10, -1338288587]]",
  "?[10 $ categorical[10, 10, -1338288587]$",
  "?[10 $ categorical[10, 10, -1338288587]$",
  "10 = categorical[10, 10, -1338288587]",
  "?10 = categorical[10, 10, -1338288587]",
  "?[10 = categorical[10, 10, -1338288587]]",
  "$10 = categorical[10, 10, -1338288587]",
  "=10 = categorical[10, 10, -1338288587]",
  "$[10 = categorical[10, 10, -1338288587]]",
  "xxx[10 = categorical[10, 10, -1338288587]]",
  "?$10 = categorical[10, 10, -1338288587]]",
  "?=10 = categorical[10, 10, -1338288587]]",
  "?[10 = categorical[10, 10, -1338288587]$",
  "?[10 = categorical[10, 10, -1338288587]$",
  "$10 * categorical[1315348478]",
  "=10 * categorical[1315348478]",
  "$[10 * categorical[1315348478]]",
  "xxx[10 * categorical[1315348478]]",
  "?$10 * categorical[1315348478]]",
  "?=10 * categorical[1315348478]]",
  "?[10 * categorical[1315348478]$",
  "?[10 * categorical[1315348478]$",
  "$categorical[1315348478]",
  "=categorical[1315348478]",
  "$[categorical[1315348478]]",
  "xxx[categorical[1315348478]]",
  "?$categorical[1315348478]]",
  "?=categorical[1315348478]]",
  "?[categorical[1315348478]$",
  "?[categorical[1315348478]$",
  "10 $ categorical[1315348478]",
  "?10 $ categorical[1315348478]",
  "?[10 $ categorical[1315348478]]",
  "$10 $ categorical[1315348478]",
  "=10 $ categorical[1315348478]",
  "$[10 $ categorical[1315348478]]",
  "xxx[10 $ categorical[1315348478]]",
  "?$10 $ categorical[1315348478]]",
  "?=10 $ categorical[1315348478]]",
  "?[10 $ categorical[1315348478]$",
  "?[10 $ categorical[1315348478]$",
  "10 = categorical[1315348478]",
  "?10 = categorical[1315348478]",
  "?[10 = categorical[1315348478]]",
  "$10 = categorical[1315348478]",
  "=10 = categorical[1315348478]",
  "$[10 = categorical[1315348478]]",
  "xxx[10 = categorical[1315348478]]",
  "?$10 = categorical[1315348478]]",
  "?=10 = categorical[1315348478]]",
  "?[10 = categorical[1315348478]$",
  "?[10 = categorical[1315348478]$",
  "$10 * categorical[10, 1315348478]",
  "=10 * categorical[10, 1315348478]",
  "$[10 * categorical[10, 1315348478]]",
  "xxx[10 * categorical[10, 1315348478]]",
  "?$10 * categorical[10, 1315348478]]",
  "?=10 * categorical[10, 1315348478]]",
  "?[10 * categorical[10, 1315348478]$",
  "?[10 * categorical[10, 1315348478]$",
  "$categorical[10, 1315348478]",
  "=categorical[10, 1315348478]",
  "$[categorical[10, 1315348478]]",
  "xxx[categorical[10, 1315348478]]",
  "?$categorical[10, 1315348478]]",
  "?=categorical[10, 1315348478]]",
  "?[categorical[10, 1315348478]$",
  "?[categorical[10, 1315348478]$",
  "10 $ categorical[10, 1315348478]",
  "?10 $ categorical[10, 1315348478]",
  "?[10 $ categorical[10, 1315348478]]",
  "$10 $ categorical[10, 1315348478]",
  "=10 $ categorical[10, 1315348478]",
  "$[10 $ categorical[10, 1315348478]]",
  "xxx[10 $ categorical[10, 1315348478]]",
  "?$10 $ categorical[10, 1315348478]]",
  "?=10 $ categorical[10, 1315348478]]",
  "?[10 $ categorical[10, 1315348478]$",
  "?[10 $ categorical[10, 1315348478]$",
  "10 = categorical[10, 1315348478]",
  "?10 = categorical[10, 1315348478]",
  "?[10 = categorical[10, 1315348478]]",
  "$10 = categorical[10, 1315348478]",
  "=10 = categorical[10, 1315348478]",
  "$[10 = categorical[10, 1315348478]]",
  "xxx[10 = categorical[10, 1315348478]]",
  "?$10 = categorical[10, 1315348478]]",
  "?=10 = categorical[10, 1315348478]]",
  "?[10 = categorical[10, 1315348478]$",
  "?[10 = categorical[10, 1315348478]$",
  "$10 * categorical[10, 10, 1315348478]",
  "=10 * categorical[10, 10, 1315348478]",
  "$[10 * categorical[10, 10, 1315348478]]",
  "xxx[10 * categorical[10, 10, 1315348478]]",
  "?$10 * categorical[10, 10, 1315348478]]",
  "?=10 * categorical[10, 10, 1315348478]]",
  "?[10 * categorical[10, 10, 1315348478]$",
  "?[10 * categorical[10, 10, 1315348478]$",
  "$categorical[10, 10, 1315348478]",
  "=categorical[10, 10, 1315348478]",
  "$[categorical[10, 10, 1315348478]]",
  "xxx[categorical[10, 10, 1315348478]]",
  "?$categorical[10, 10, 1315348478]]",
  "?=categorical[10, 10, 1315348478]]",
  "?[categorical[10, 10, 1315348478]$",
  "?[categorical[10, 10, 1315348478]$",
  "10 $ categorical[10, 10, 1315348478]",
  "?10 $ categorical[10, 10, 1315348478]",
  "?[10 $ categorical[10, 10, 1315348478]]",
  "$10 $ categorical[10, 10, 1315348478]",
  "=10 $ categorical[10, 10, 1315348478]",
  "$[10 $ categorical[10, 10, 1315348478]]",
  "xxx[10 $ categorical[10, 10, 1315348478]]",
  "?$10 $ categorical[10, 10, 1315348478]]",
  "?=10 $ categorical[10, 10, 1315348478]]",
  "?[10 $ categorical[10, 10, 1315348478]$",
  "?[10 $ categorical[10, 10, 1315348478]$",
  "10 = categorical[10, 10, 1315348478]",
  "?10 = categorical[10, 10, 1315348478]",
  "?[10 = categorical[10, 10, 1315348478]]",
  "$10 = categorical[10, 10, 1315348478]",
  "=10 = categorical[10, 10, 1315348478]",
  "$[10 = categorical[10, 10, 1315348478]]",
  "xxx[10 = categorical[10, 10, 1315348478]]",
  "?$10 = categorical[10, 10, 1315348478]]",
  "?=10 = categorical[10, 10, 1315348478]]",
  "?[10 = categorical[10, 10, 1315348478]$",
  "?[10 = categorical[10, 10, 1315348478]$",
  "$10 * categorical[158]",
  "=10 * categorical[158]",
  "$[10 * categorical[158]]",
  "xxx[10 * categorical[158]]",
  "?$10 * categorical[158]]",
  "?=10 * categorical[158]]",
  "?[10 * categorical[158]$",
  "?[10 * categorical[158]$",
  "$categorical[158]",
  "=categorical[158]",
  "$[categorical[158]]",
  "xxx[categorical[158]]",
  "?$categorical[158]]",
  "?=categorical[158]]",
  "?[categorical[158]$",
  "?[categorical[158]$",
  "10 $ categorical[158]",
  "?10 $ categorical[158]",
  "?[10 $ categorical[158]]",
  "$10 $ categorical[158]",
  "=10 $ categorical[158]",
  "$[10 $ categorical[158]]",
  "xxx[10 $ categorical[158]]",
  "?$10 $ categorical[158]]",
  "?=10 $ categorical[158]]",
  "?[10 $ categorical[158]$",
  "?[10 $ categorical[158]$",
  "10 = categorical[158]",
  "?10 = categorical[158]",
  "?[10 = categorical[158]]",
  "$10 = categorical[158]",
  "=10 = categorical[158]",
  "$[10 = categorical[158]]",
  "xxx[10 = categorical[158]]",
  "?$10 = categorical[158]]",
  "?=10 = categorical[158]]",
  "?[10 = categorical[158]$",
  "?[10 = categorical[158]$",
  "$10 * categorical[10, 158]",
  "=10 * categorical[10, 158]",
  "$[10 * categorical[10, 158]]",
  "xxx[10 * categorical[10, 158]]",
  "?$10 * categorical[10, 158]]",
  "?=10 * categorical[10, 158]]",
  "?[10 * categorical[10, 158]$",
  "?[10 * categorical[10, 158]$",
  "$categorical[10, 158]",
  "=categorical[10, 158]",
  "$[categorical[10, 158]]",
  "xxx[categorical[10, 158]]",
  "?$categorical[10, 158]]",
  "?=categorical[10, 158]]",
  "?[categorical[10, 158]$",
  "?[categorical[10, 158]$",
  "10 $ categorical[10, 158]",
  "?10 $ categorical[10, 158]",
  "?[10 $ categorical[10, 158]]",
  "$10 $ categorical[10, 158]",
  "=10 $ categorical[10, 158]",
  "$[10 $ categorical[10, 158]]",
  "xxx[10 $ categorical[10, 158]]",
  "?$10 $ categorical[10, 158]]",
  "?=10 $ categorical[10, 158]]",
  "?[10 $ categorical[10, 158]$",
  "?[10 $ categorical[10, 158]$",
  "10 = categorical[10, 158]",
  "?10 = categorical[10, 158]",
  "?[10 = categorical[10, 158]]",
  "$10 = categorical[10, 158]",
  "=10 = categorical[10, 158]",
  "$[10 = categorical[10, 158]]",
  "xxx[10 = categorical[10, 158]]",
  "?$10 = categorical[10, 158]]",
  "?=10 = categorical[10, 158]]",
  "?[10 = categorical[10, 158]$",
  "?[10 = categorical[10, 158]$",
  "$10 * categorical[10, 10, 158]",
  "=10 * categorical[10, 10, 158]",
  "$[10 * categorical[10, 10, 158]]",
  "xxx[10 * categorical[10, 10, 158]]",
  "?$10 * categorical[10, 10, 158]]",
  "?=10 * categorical[10, 10, 158]]",
  "?[10 * categorical[10, 10, 158]$",
  "?[10 * categorical[10, 10, 158]$",
  "$categorical[10, 10, 158]",
  "=categorical[10, 10, 158]",
  "$[categorical[10, 10, 158]]",
  "xxx[categorical[10, 10, 158]]",
  "?$categorical[10, 10, 158]]",
  "?=categorical[10, 10, 158]]",
  "?[categorical[10, 10, 158]$",
  "?[categorical[10, 10, 158]$",
  "10 $ categorical[10, 10, 158]",
  "?10 $ categorical[10, 10, 158]",
  "?[10 $ categorical[10, 10, 158]]",
  "$10 $ categorical[10, 10, 158]",
  "=10 $ categorical[10, 10, 158]",
  "$[10 $ categorical[10, 10, 158]]",
  "xxx[10 $ categorical[10, 10, 158]]",
  "?$10 $ categorical[10, 10, 158]]",
  "?=10 $ categorical[10, 10, 158]]",
  "?[10 $ categorical[10, 10, 158]$",
  "?[10 $ categorical[10, 10, 158]$",
  "10 = categorical[10, 10, 158]",
  "?10 = categorical[10, 10, 158]",
  "?[10 = categorical[10, 10, 158]]",
  "$10 = categorical[10, 10, 158]",
  "=10 = categorical[10, 10, 158]",
  "$[10 = categorical[10, 10, 158]]",
  "xxx[10 = categorical[10, 10, 158]]",
  "?$10 = categorical[10, 10, 158]]",
  "?=10 = categorical[10, 10, 158]]",
  "?[10 = categorical[10, 10, 158]$",
  "?[10 = categorical[10, 10, 158]$",
  "$10 * categorical[58507]",
  "=10 * categorical[58507]",
  "$[10 * categorical[58507]]",
  "xxx[10 * categorical[58507]]",
  "?$10 * categorical[58507]]",
  "?=10 * categorical[58507]]",
  "?[10 * categorical[58507]$",
  "?[10 * categorical[58507]$",
  "$categorical[58507]",
  "=categorical[58507]",
  "$[categorical[58507]]",
  "xxx[categorical[58507]]",
  "?$categorical[58507]]",
  "?=categorical[58507]]",
  "?[categorical[58507]$",
  "?[categorical[58507]$",
  "10 $ categorical[58507]",
  "?10 $ categorical[58507]",
  "?[10 $ categorical[58507]]",
  "$10 $ categorical[58507]",
  "=10 $ categorical[58507]",
  "$[10 $ categorical[58507]]",
  "xxx[10 $ categorical[58507]]",
  "?$10 $ categorical[58507]]",
  "?=10 $ categorical[58507]]",
  "?[10 $ categorical[58507]$",
  "?[10 $ categorical[58507]$",
  "10 = categorical[58507]",
  "?10 = categorical[58507]",
  "?[10 = categorical[58507]]",
  "$10 = categorical[58507]",
  "=10 = categorical[58507]",
  "$[10 = categorical[58507]]",
  "xxx[10 = categorical[58507]]",
  "?$10 = categorical[58507]]",
  "?=10 = categorical[58507]]",
  "?[10 = categorical[58507]$",
  "?[10 = categorical[58507]$",
  "$10 * categorical[10, 58507]",
  "=10 * categorical[10, 58507]",
  "$[10 * categorical[10, 58507]]",
  "xxx[10 * categorical[10, 58507]]",
  "?$10 * categorical[10, 58507]]",
  "?=10 * categorical[10, 58507]]",
  "?[10 * categorical[10, 58507]$",
  "?[10 * categorical[10, 58507]$",
  "$categorical[10, 58507]",
  "=categorical[10, 58507]",
  "$[categorical[10, 58507]]",
  "xxx[categorical[10, 58507]]",
  "?$categorical[10, 58507]]",
  "?=categorical[10, 58507]]",
  "?[categorical[10, 58507]$",
  "?[categorical[10, 58507]$",
  "10 $ categorical[10, 58507]",
  "?10 $ categorical[10, 58507]",
  "?[10 $ categorical[10, 58507]]",
  "$10 $ categorical[10, 58507]",
  "=10 $ categorical[10, 58507]",
  "$[10 $ categorical[10, 58507]]",
  "xxx[10 $ categorical[10, 58507]]",
  "?$10 $ categorical[10, 58507]]",
  "?=10 $ categorical[10, 58507]]",
  "?[10 $ categorical[10, 58507]$",
  "?[10 $ categorical[10, 58507]$",
  "10 = categorical[10, 58507]",
  "?10 = categorical[10, 58507]",
  "?[10 = categorical[10, 58507]]",
  "$10 = categorical[10, 58507]",
  "=10 = categorical[10, 58507]",
  "$[10 = categorical[10, 58507]]",
  "xxx[10 = categorical[10, 58507]]",
  "?$10 = categorical[10, 58507]]",
  "?=10 = categorical[10, 58507]]",
  "?[10 = categorical[10, 58507]$",
  "?[10 = categorical[10, 58507]$",
  "$10 * categorical[10, 10, 58507]",
  "=10 * categorical[10, 10, 58507]",
  "$[10 * categorical[10, 10, 58507]]",
  "xxx[10 * categorical[10, 10, 58507]]",
  "?$10 * categorical[10, 10, 58507]]",
  "?=10 * categorical[10, 10, 58507]]",
  "?[10 * categorical[10, 10, 58507]$",
  "?[10 * categorical[10, 10, 58507]$",
  "$categorical[10, 10, 58507]",
  "=categorical[10, 10, 58507]",
  "$[categorical[10, 10, 58507]]",
  "xxx[categorical[10, 10, 58507]]",
  "?$categorical[10, 10, 58507]]",
  "?=categorical[10, 10, 58507]]",
  "?[categorical[10, 10, 58507]$",
  "?[categorical[10, 10, 58507]$",
  "10 $ categorical[10, 10, 58507]",
  "?10 $ categorical[10, 10, 58507]",
  "?[10 $ categorical[10, 10, 58507]]",
  "$10 $ categorical[10, 10, 58507]",
  "=10 $ categorical[10, 10, 58507]",
  "$[10 $ categorical[10, 10, 58507]]",
  "xxx[10 $ categorical[10, 10, 58507]]",
  "?$10 $ categorical[10, 10, 58507]]",
  "?=10 $ categorical[10, 10, 58507]]",
  "?[10 $ categorical[10, 10, 58507]$",
  "?[10 $ categorical[10, 10, 58507]$",
  "10 = categorical[10, 10, 58507]",
  "?10 = categorical[10, 10, 58507]",
  "?[10 = categorical[10, 10, 58507]]",
  "$10 = categorical[10, 10, 58507]",
  "=10 = categorical[10, 10, 58507]",
  "$[10 = categorical[10, 10, 58507]]",
  "xxx[10 = categorical[10, 10, 58507]]",
  "?$10 = categorical[10, 10, 58507]]",
  "?=10 = categorical[10, 10, 58507]]",
  "?[10 = categorical[10, 10, 58507]$",
  "?[10 = categorical[10, 10, 58507]$",
  "$10 * categorical[2051118756]",
  "=10 * categorical[2051118756]",
  "$[10 * categorical[2051118756]]",
  "xxx[10 * categorical[2051118756]]",
  "?$10 * categorical[2051118756]]",
  "?=10 * categorical[2051118756]]",
  "?[10 * categorical[2051118756]$",
  "?[10 * categorical[2051118756]$",
  "$categorical[2051118756]",
  "=categorical[2051118756]",
  "$[categorical[2051118756]]",
  "xxx[categorical[2051118756]]",
  "?$categorical[2051118756]]",
  "?=categorical[2051118756]]",
  "?[categorical[2051118756]$",
  "?[categorical[2051118756]$",
  "10 $ categorical[2051118756]",
  "?10 $ categorical[2051118756]",
  "?[10 $ categorical[2051118756]]",
  "$10 $ categorical[2051118756]",
  "=10 $ categorical[2051118756]",
  "$[10 $ categorical[2051118756]]",
  "xxx[10 $ categorical[2051118756]]",
  "?$10 $ categorical[2051118756]]",
  "?=10 $ categorical[2051118756]]",
  "?[10 $ categorical[2051118756]$",
  "?[10 $ categorical[2051118756]$",
  "10 = categorical[2051118756]",
  "?10 = categorical[2051118756]",
  "?[10 = categorical[2051118756]]",
  "$10 = categorical[2051118756]",
  "=10 = categorical[2051118756]",
  "$[10 = categorical[2051118756]]",
  "xxx[10 = categorical[2051118756]]",
  "?$10 = categorical[2051118756]]",
  "?=10 = categorical[2051118756]]",
  "?[10 = categorical[2051118756]$",
  "?[10 = categorical[2051118756]$",
  "$10 * categorical[10, 2051118756]",
  "=10 * categorical[10, 2051118756]",
  "$[10 * categorical[10, 2051118756]]",
  "xxx[10 * categorical[10, 2051118756]]",
  "?$10 * categorical[10, 2051118756]]",
  "?=10 * categorical[10, 2051118756]]",
  "?[10 * categorical[10, 2051118756]$",
  "?[10 * categorical[10, 2051118756]$",
  "$categorical[10, 2051118756]",
  "=categorical[10, 2051118756]",
  "$[categorical[10, 2051118756]]",
  "xxx[categorical[10, 2051118756]]",
  "?$categorical[10, 2051118756]]",
  "?=categorical[10, 2051118756]]",
  "?[categorical[10, 2051118756]$",
  "?[categorical[10, 2051118756]$",
  "10 $ categorical[10, 2051118756]",
  "?10 $ categorical[10, 2051118756]",
  "?[10 $ categorical[10, 2051118756]]",
  "$10 $ categorical[10, 2051118756]",
  "=10 $ categorical[10, 2051118756]",
  "$[10 $ categorical[10, 2051118756]]",
  "xxx[10 $ categorical[10, 2051118756]]",
  "?$10 $ categorical[10, 2051118756]]",
  "?=10 $ categorical[10, 2051118756]]",
  "?[10 $ categorical[10, 2051118756]$",
  "?[10 $ categorical[10, 2051118756]$",
  "10 = categorical[10, 2051118756]",
  "?10 = categorical[10, 2051118756]",
  "?[10 = categorical[10, 2051118756]]",
  "$10 = categorical[10, 2051118756]",
  "=10 = categorical[10, 2051118756]",
  "$[10 = categorical[10, 2051118756]]",
  "xxx[10 = categorical[10, 2051118756]]",
  "?$10 = categorical[10, 2051118756]]",
  "?=10 = categorical[10, 2051118756]]",
  "?[10 = categorical[10, 2051118756]$",
  "?[10 = categorical[10, 2051118756]$",
  "$10 * categorical[10, 10, 2051118756]",
  "=10 * categorical[10, 10, 2051118756]",
  "$[10 * categorical[10, 10, 2051118756]]",
  "xxx[10 * categorical[10, 10, 2051118756]]",
  "?$10 * categorical[10, 10, 2051118756]]",
  "?=10 * categorical[10, 10, 2051118756]]",
  "?[10 * categorical[10, 10, 2051118756]$",
  "?[10 * categorical[10, 10, 2051118756]$",
  "$categorical[10, 10, 2051118756]",
  "=categorical[10, 10, 2051118756]",
  "$[categorical[10, 10, 2051118756]]",
  "xxx[categorical[10, 10, 2051118756]]",
  "?$categorical[10, 10, 2051118756]]",
  "?=categorical[10, 10, 2051118756]]",
  "?[categorical[10, 10, 2051118756]$",
  "?[categorical[10, 10, 2051118756]$",
  "10 $ categorical[10, 10, 2051118756]",
  "?10 $ categorical[10, 10, 2051118756]",
  "?[10 $ categorical[10, 10, 2051118756]]",
  "$10 $ categorical[10, 10, 2051118756]",
  "=10 $ categorical[10, 10, 2051118756]",
  "$[10 $ categorical[10, 10, 2051118756]]",
  "xxx[10 $ categorical[10, 10, 2051118756]]",
  "?$10 $ categorical[10, 10, 2051118756]]",
  "?=10 $ categorical[10, 10, 2051118756]]",
  "?[10 $ categorical[10, 10, 2051118756]$",
  "?[10 $ categorical[10, 10, 2051118756]$",
  "10 = categorical[10, 10, 2051118756]",
  "?10 = categorical[10, 10, 2051118756]",
  "?[10 = categorical[10, 10, 2051118756]]",
  "$10 = categorical[10, 10, 2051118756]",
  "=10 = categorical[10, 10, 2051118756]",
  "$[10 = categorical[10, 10, 2051118756]]",
  "xxx[10 = categorical[10, 10, 2051118756]]",
  "?$10 = categorical[10, 10, 2051118756]]",
  "?=10 = categorical[10, 10, 2051118756]]",
  "?[10 = categorical[10, 10, 2051118756]$",
  "?[10 = categorical[10, 10, 2051118756]$",
  "$10 * categorical[3451618092]",
  "=10 * categorical[3451618092]",
  "$[10 * categorical[3451618092]]",
  "xxx[10 * categorical[3451618092]]",
  "?$10 * categorical[3451618092]]",
  "?=10 * categorical[3451618092]]",
  "?[10 * categorical[3451618092]$",
  "?[10 * categorical[3451618092]$",
  "$categorical[3451618092]",
  "=categorical[3451618092]",
  "$[categorical[3451618092]]",
  "xxx[categorical[3451618092]]",
  "?$categorical[3451618092]]",
  "?=categorical[3451618092]]",
  "?[categorical[3451618092]$",
  "?[categorical[3451618092]$",
  "10 $ categorical[3451618092]",
  "?10 $ categorical[3451618092]",
  "?[10 $ categorical[3451618092]]",
  "$10 $ categorical[3451618092]",
  "=10 $ categorical[3451618092]",
  "$[10 $ categorical[3451618092]]",
  "xxx[10 $ categorical[3451618092]]",
  "?$10 $ categorical[3451618092]]",
  "?=10 $ categorical[3451618092]]",
  "?[10 $ categorical[3451618092]$",
  "?[10 $ categorical[3451618092]$",
  "10 = categorical[3451618092]",
  "?10 = categorical[3451618092]",
  "?[10 = categorical[3451618092]]",
  "$10 = categorical[3451618092]",
  "=10 = categorical[3451618092]",
  "$[10 = categorical[3451618092]]",
  "xxx[10 = categorical[3451618092]]",
  "?$10 = categorical[3451618092]]",
  "?=10 = categorical[3451618092]]",
  "?[10 = categorical[3451618092]$",
  "?[10 = categorical[3451618092]$",
  "$10 * categorical[10, 3451618092]",
  "=10 * categorical[10, 3451618092]",
  "$[10 * categorical[10, 3451618092]]",
  "xxx[10 * categorical[10, 3451618092]]",
  "?$10 * categorical[10, 3451618092]]",
  "?=10 * categorical[10, 3451618092]]",
  "?[10 * categorical[10, 3451618092]$",
  "?[10 * categorical[10, 3451618092]$",
  "$categorical[10, 3451618092]",
  "=categorical[10, 3451618092]",
  "$[categorical[10, 3451618092]]",
  "xxx[categorical[10, 3451618092]]",
  "?$categorical[10, 3451618092]]",
  "?=categorical[10, 3451618092]]",
  "?[categorical[10, 3451618092]$",
  "?[categorical[10, 3451618092]$",
  "10 $ categorical[10, 3451618092]",
  "?10 $ categorical[10, 3451618092]",
  "?[10 $ categorical[10, 3451618092]]",
  "$10 $ categorical[10, 3451618092]",
  "=10 $ categorical[10, 3451618092]",
  "$[10 $ categorical[10, 3451618092]]",
  "xxx[10 $ categorical[10, 3451618092]]",
  "?$10 $ categorical[10, 3451618092]]",
  "?=10 $ categorical[10, 3451618092]]",
  "?[10 $ categorical[10, 3451618092]$",
  "?[10 $ categorical[10, 3451618092]$",
  "10 = categorical[10, 3451618092]",
  "?10 = categorical[10, 3451618092]",
  "?[10 = categorical[10, 3451618092]]",
  "$10 = categorical[10, 3451618092]",
  "=10 = categorical[10, 3451618092]",
  "$[10 = categorical[10, 3451618092]]",
  "xxx[10 = categorical[10, 3451618092]]",
  "?$10 = categorical[10, 3451618092]]",
  "?=10 = categorical[10, 3451618092]]",
  "?[10 = categorical[10, 3451618092]$",
  "?[10 = categorical[10, 3451618092]$",
  "$10 * categorical[10, 10, 3451618092]",
  "=10 * categorical[10, 10, 3451618092]",
  "$[10 * categorical[10, 10, 3451618092]]",
  "xxx[10 * categorical[10, 10, 3451618092]]",
  "?$10 * categorical[10, 10, 3451618092]]",
  "?=10 * categorical[10, 10, 3451618092]]",
  "?[10 * categorical[10, 10, 3451618092]$",
  "?[10 * categorical[10, 10, 3451618092]$",
  "$categorical[10, 10, 3451618092]",
  "=categorical[10, 10, 3451618092]",
  "$[categorical[10, 10, 3451618092]]",
  "xxx[categorical[10, 10, 3451618092]]",
  "?$categorical[10, 10, 3451618092]]",
  "?=categorical[10, 10, 3451618092]]",
  "?[categorical[10, 10, 3451618092]$",
  "?[categorical[10, 10, 3451618092]$",
  "10 $ categorical[10, 10, 3451618092]",
  "?10 $ categorical[10, 10, 3451618092]",
  "?[10 $ categorical[10, 10, 3451618092]]",
  "$10 $ categorical[10, 10, 3451618092]",
  "=10 $ categorical[10, 10, 3451618092]",
  "$[10 $ categorical[10, 10, 3451618092]]",
  "xxx[10 $ categorical[10, 10, 3451618092]]",
  "?$10 $ categorical[10, 10, 3451618092]]",
  "?=10 $ categorical[10, 10, 3451618092]]",
  "?[10 $ categorical[10, 10, 3451618092]$",
  "?[10 $ categorical[10, 10, 3451618092]$",
  "10 = categorical[10, 10, 3451618092]",
  "?10 = categorical[10, 10, 3451618092]",
  "?[10 = categorical[10, 10, 3451618092]]",
  "$10 = categorical[10, 10, 3451618092]",
  "=10 = categorical[10, 10, 3451618092]",
  "$[10 = categorical[10, 10, 3451618092]]",
  "xxx[10 = categorical[10, 10, 3451618092]]",
  "?$10 = categorical[10, 10, 3451618092]]",
  "?=10 = categorical[10, 10, 3451618092]]",
  "?[10 = categorical[10, 10, 3451618092]$",
  "?[10 = categorical[10, 10, 3451618092]$",
  "$10 * categorical[562185221.0292504]",
  "=10 * categorical[562185221.0292504]",
  "$[10 * categorical[562185221.0292504]]",
  "xxx[10 * categorical[562185221.0292504]]",
  "?$10 * categorical[562185221.0292504]]",
  "?=10 * categorical[562185221.0292504]]",
  "?[10 * categorical[562185221.0292504]$",
  "?[10 * categorical[562185221.0292504]$",
  "$categorical[562185221.0292504]",
  "=categorical[562185221.0292504]",
  "$[categorical[562185221.0292504]]",
  "xxx[categorical[562185221.0292504]]",
  "?$categorical[562185221.0292504]]",
  "?=categorical[562185221.0292504]]",
  "?[categorical[562185221.0292504]$",
  "?[categorical[562185221.0292504]$",
  "10 $ categorical[562185221.0292504]",
  "?10 $ categorical[562185221.0292504]",
  "?[10 $ categorical[562185221.0292504]]",
  "$10 $ categorical[562185221.0292504]",
  "=10 $ categorical[562185221.0292504]",
  "$[10 $ categorical[562185221.0292504]]",
  "xxx[10 $ categorical[562185221.0292504]]",
  "?$10 $ categorical[562185221.0292504]]",
  "?=10 $ categorical[562185221.0292504]]",
  "?[10 $ categorical[562185221.0292504]$",
  "?[10 $ categorical[562185221.0292504]$",
  "10 = categorical[562185221.0292504]",
  "?10 = categorical[562185221.0292504]",
  "?[10 = categorical[562185221.0292504]]",
  "$10 = categorical[562185221.0292504]",
  "=10 = categorical[562185221.0292504]",
  "$[10 = categorical[562185221.0292504]]",
  "xxx[10 = categorical[562185221.0292504]]",
  "?$10 = categorical[562185221.0292504]]",
  "?=10 = categorical[562185221.0292504]]",
  "?[10 = categorical[562185221.0292504]$",
  "?[10 = categorical[562185221.0292504]$",
  "$10 * categorical[10, 562185221.0292504]",
  "=10 * categorical[10, 562185221.0292504]",
  "$[10 * categorical[10, 562185221.0292504]]",
  "xxx[10 * categorical[10, 562185221.0292504]]",
  "?$10 * categorical[10, 562185221.0292504]]",
  "?=10 * categorical[10, 562185221.0292504]]",
  "?[10 * categorical[10, 562185221.0292504]$",
  "?[10 * categorical[10, 562185221.0292504]$",
  "$categorical[10, 562185221.0292504]",
  "=categorical[10, 562185221.0292504]",
  "$[categorical[10, 562185221.0292504]]",
  "xxx[categorical[10, 562185221.0292504]]",
  "?$categorical[10, 562185221.0292504]]",
  "?=categorical[10, 562185221.0292504]]",
  "?[categorical[10, 562185221.0292504]$",
  "?[categorical[10, 562185221.0292504]$",
  "10 $ categorical[10, 562185221.0292504]",
  "?10 $ categorical[10, 562185221.0292504]",
  "?[10 $ categorical[10, 562185221.0292504]]",
  "$10 $ categorical[10, 562185221.0292504]",
  "=10 $ categorical[10, 562185221.0292504]",
  "$[10 $ categorical[10, 562185221.0292504]]",
  "xxx[10 $ categorical[10, 562185221.0292504]]",
  "?$10 $ categorical[10, 562185221.0292504]]",
  "?=10 $ categorical[10, 562185221.0292504]]",
  "?[10 $ categorical[10, 562185221.0292504]$",
  "?[10 $ categorical[10, 562185221.0292504]$",
  "10 = categorical[10, 562185221.0292504]",
  "?10 = categorical[10, 562185221.0292504]",
  "?[10 = categorical[10, 562185221.0292504]]",
  "$10 = categorical[10, 562185221.0292504]",
  "=10 = categorical[10, 562185221.0292504]",
  "$[10 = categorical[10, 562185221.0292504]]",
  "xxx[10 = categorical[10, 562185221.0292504]]",
  "?$10 = categorical[10, 562185221.0292504]]",
  "?=10 = categorical[10, 562185221.0292504]]",
  "?[10 = categorical[10, 562185221.0292504]$",
  "?[10 = categorical[10, 562185221.0292504]$",
  "$10 * categorical[10, 10, 562185221.0292504]",
  "=10 * categorical[10, 10, 562185221.0292504]",
  "$[10 * categorical[10, 10, 562185221.0292504]]",
  "xxx[10 * categorical[10, 10, 562185221.0292504]]",
  "?$10 * categorical[10, 10, 562185221.0292504]]",
  "?=10 * categorical[10, 10, 562185221.0292504]]",
  "?[10 * categorical[10, 10, 562185221.0292504]$",
  "?[10 * categorical[10, 10, 562185221.0292504]$",
  "$categorical[10, 10, 562185221.0292504]",
  "=categorical[10, 10, 562185221.0292504]",
  "$[categorical[10, 10, 562185221.0292504]]",
  "xxx[categorical[10, 10, 562185221.0292504]]",
  "?$categorical[10, 10, 562185221.0292504]]",
  "?=categorical[10, 10, 562185221.0292504]]",
  "?[categorical[10, 10, 562185221.0292504]$",
  "?[categorical[10, 10, 562185221.0292504]$",
  "10 $ categorical[10, 10, 562185221.0292504]",
  "?10 $ categorical[10, 10, 562185221.0292504]",
  "?[10 $ categorical[10, 10, 562185221.0292504]]",
  "$10 $ categorical[10, 10, 562185221.0292504]",
  "=10 $ categorical[10, 10, 562185221.0292504]",
  "$[10 $ categorical[10, 10, 562185221.0292504]]",
  "xxx[10 $ categorical[10, 10, 562185221.0292504]]",
  "?$10 $ categorical[10, 10, 562185221.0292504]]",
  "?=10 $ categorical[10, 10, 562185221.0292504]]",
  "?[10 $ categorical[10, 10, 562185221.0292504]$",
  "?[10 $ categorical[10, 10, 562185221.0292504]$",
  "10 = categorical[10, 10, 562185221.0292504]",
  "?10 = categorical[10, 10, 562185221.0292504]",
  "?[10 = categorical[10, 10, 562185221.0292504]]",
  "$10 = categorical[10, 10, 562185221.0292504]",
  "=10 = categorical[10, 10, 562185221.0292504]",
  "$[10 = categorical[10, 10, 562185221.0292504]]",
  "xxx[10 = categorical[10, 10, 562185221.0292504]]",
  "?$10 = categorical[10, 10, 562185221.0292504]]",
  "?=10 = categorical[10, 10, 562185221.0292504]]",
  "?[10 = categorical[10, 10, 562185221.0292504]$",
  "?[10 = categorical[10, 10, 562185221.0292504]$",
  "$10 * categorical[170135311.32221362]",
  "=10 * categorical[170135311.32221362]",
  "$[10 * categorical[170135311.32221362]]",
  "xxx[10 * categorical[170135311.32221362]]",
  "?$10 * categorical[170135311.32221362]]",
  "?=10 * categorical[170135311.32221362]]",
  "?[10 * categorical[170135311.32221362]$",
  "?[10 * categorical[170135311.32221362]$",
  "$categorical[170135311.32221362]",
  "=categorical[170135311.32221362]",
  "$[categorical[170135311.32221362]]",
  "xxx[categorical[170135311.32221362]]",
  "?$categorical[170135311.32221362]]",
  "?=categorical[170135311.32221362]]",
  "?[categorical[170135311.32221362]$",
  "?[categorical[170135311.32221362]$",
  "10 $ categorical[170135311.32221362]",
  "?10 $ categorical[170135311.32221362]",
  "?[10 $ categorical[170135311.32221362]]",
  "$10 $ categorical[170135311.32221362]",
  "=10 $ categorical[170135311.32221362]",
  "$[10 $ categorical[170135311.32221362]]",
  "xxx[10 $ categorical[170135311.32221362]]",
  "?$10 $ categorical[170135311.32221362]]",
  "?=10 $ categorical[170135311.32221362]]",
  "?[10 $ categorical[170135311.32221362]$",
  "?[10 $ categorical[170135311.32221362]$",
  "10 = categorical[170135311.32221362]",
  "?10 = categorical[170135311.32221362]",
  "?[10 = categorical[170135311.32221362]]",
  "$10 = categorical[170135311.32221362]",
  "=10 = categorical[170135311.32221362]",
  "$[10 = categorical[170135311.32221362]]",
  "xxx[10 = categorical[170135311.32221362]]",
  "?$10 = categorical[170135311.32221362]]",
  "?=10 = categorical[170135311.32221362]]",
  "?[10 = categorical[170135311.32221362]$",
  "?[10 = categorical[170135311.32221362]$",
  "$10 * categorical[10, 170135311.32221362]",
  "=10 * categorical[10, 170135311.32221362]",
  "$[10 * categorical[10, 170135311.32221362]]",
  "xxx[10 * categorical[10, 170135311.32221362]]",
  "?$10 * categorical[10, 170135311.32221362]]",
  "?=10 * categorical[10, 170135311.32221362]]",
  "?[10 * categorical[10, 170135311.32221362]$",
  "?[10 * categorical[10, 170135311.32221362]$",
  "$categorical[10, 170135311.32221362]",
  "=categorical[10, 170135311.32221362]",
  "$[categorical[10, 170135311.32221362]]",
  "xxx[categorical[10, 170135311.32221362]]",
  "?$categorical[10, 170135311.32221362]]",
  "?=categorical[10, 170135311.32221362]]",
  "?[categorical[10, 170135311.32221362]$",
  "?[categorical[10, 170135311.32221362]$",
  "10 $ categorical[10, 170135311.32221362]",
  "?10 $ categorical[10, 170135311.32221362]",
  "?[10 $ categorical[10, 170135311.32221362]]",
  "$10 $ categorical[10, 170135311.32221362]",
  "=10 $ categorical[10, 170135311.32221362]",
  "$[10 $ categorical[10, 170135311.32221362]]",
  "xxx[10 $ categorical[10, 170135311.32221362]]",
  "?$10 $ categorical[10, 170135311.32221362]]",
  "?=10 $ categorical[10, 170135311.32221362]]",
  "?[10 $ categorical[10, 170135311.32221362]$",
  "?[10 $ categorical[10, 170135311.32221362]$",
  "10 = categorical[10, 170135311.32221362]",
  "?10 = categorical[10, 170135311.32221362]",
  "?[10 = categorical[10, 170135311.32221362]]",
  "$10 = categorical[10, 170135311.32221362]",
  "=10 = categorical[10, 170135311.32221362]",
  "$[10 = categorical[10, 170135311.32221362]]",
  "xxx[10 = categorical[10, 170135311.32221362]]",
  "?$10 = categorical[10, 170135311.32221362]]",
  "?=10 = categorical[10, 170135311.32221362]]",
  "?[10 = categorical[10, 170135311.32221362]$",
  "?[10 = categorical[10, 170135311.32221362]$",
  "$10 * categorical[10, 10, 170135311.32221362]",
  "=10 * categorical[10, 10, 170135311.32221362]",
  "$[10 * categorical[10, 10, 170135311.32221362]]",
  "xxx[10 * categorical[10, 10, 170135311.32221362]]",
  "?$10 * categorical[10, 10, 170135311.32221362]]",
  "?=10 * categorical[10, 10, 170135311.32221362]]",
  "?[10 * categorical[10, 10, 170135311.32221362]$",
  "?[10 * categorical[10, 10, 170135311.32221362]$",
  "$categorical[10, 10, 170135311.32221362]",
  "=categorical[10, 10, 170135311.32221362]",
  "$[categorical[10, 10, 170135311.32221362]]",
  "xxx[categorical[10, 10, 170135311.32221362]]",
  "?$categorical[10, 10, 170135311.32221362]]",
  "?=categorical[10, 10, 170135311.32221362]]",
  "?[categorical[10, 10, 170135311.32221362]$",
  "?[categorical[10, 10, 170135311.32221362]$",
  "10 $ categorical[10, 10, 170135311.32221362]",
  "?10 $ categorical[10, 10, 170135311.32221362]",
  "?[10 $ categorical[10, 10, 170135311.32221362]]",
  "$10 $ categorical[10, 10, 170135311.32221362]",
  "=10 $ categorical[10, 10, 170135311.32221362]",
  "$[10 $ categorical[10, 10, 170135311.32221362]]",
  "xxx[10 $ categorical[10, 10, 170135311.32221362]]",
  "?$10 $ categorical[10, 10, 170135311.32221362]]",
  "?=10 $ categorical[10, 10, 170135311.32221362]]",
  "?[10 $ categorical[10, 10, 170135311.32221362]$",
  "?[10 $ categorical[10, 10, 170135311.32221362]$",
  "10 = categorical[10, 10, 170135311.32221362]",
  "?10 = categorical[10, 10, 170135311.32221362]",
  "?[10 = categorical[10, 10, 170135311.32221362]]",
  "$10 = categorical[10, 10, 170135311.32221362]",
  "=10 = categorical[10, 10, 170135311.32221362]",
  "$[10 = categorical[10, 10, 170135311.32221362]]",
  "xxx[10 = categorical[10, 10, 170135311.32221362]]",
  "?$10 = categorical[10, 10, 170135311.32221362]]",
  "?=10 = categorical[10, 10, 170135311.32221362]]",
  "?[10 = categorical[10, 10, 170135311.32221362]$",
  "?[10 = categorical[10, 10, 170135311.32221362]$",
  "$10 * categorical[\"dUX1\"]",
  "=10 * categorical[\"dUX1\"]",
  "$[10 * categorical[\"dUX1\"]]",
  "xxx[10 * categorical[\"dUX1\"]]",
  "?$10 * categorical[\"dUX1\"]]",
  "?=10 * categorical[\"dUX1\"]]",
  "?[10 * categorical[\"dUX1\"]$",
  "?[10 * categorical[\"dUX1\"]$",
  "$categorical[\"dUX1\"]",
  "=categorical[\"dUX1\"]",
  "$[categorical[\"dUX1\"]]",
  "xxx[categorical[\"dUX1\"]]",
  "?$categorical[\"dUX1\"]]",
  "?=categorical[\"dUX1\"]]",
  "?[categorical[\"dUX1\"]$",
  "?[categorical[\"dUX1\"]$",
  "10 $ categorical[\"dUX1\"]",
  "?10 $ categorical[\"dUX1\"]",
  "?[10 $ categorical[\"dUX1\"]]",
  "$10 $ categorical[\"dUX1\"]",
  "=10 $ categorical[\"dUX1\"]",
  "$[10 $ categorical[\"dUX1\"]]",
  "xxx[10 $ categorical[\"dUX1\"]]",
  "?$10 $ categorical[\"dUX1\"]]",
  "?=10 $ categorical[\"dUX1\"]]",
  "?[10 $ categorical[\"dUX1\"]$",
  "?[10 $ categorical[\"dUX1\"]$",
  "10 = categorical[\"dUX1\"]",
  "?10 = categorical[\"dUX1\"]",
  "?[10 = categorical[\"dUX1\"]]",
  "$10 = categorical[\"dUX1\"]",
  "=10 = categorical[\"dUX1\"]",
  "$[10 = categorical[\"dUX1\"]]",
  "xxx[10 = categorical[\"dUX1\"]]",
  "?$10 = categorical[\"dUX1\"]]",
  "?=10 = categorical[\"dUX1\"]]",
  "?[10 = categorical[\"dUX1\"]$",
  "?[10 = categorical[\"dUX1\"]$",
  "$10 * categorical[10, \"dUX1\"]",
  "=10 * categorical[10, \"dUX1\"]",
  "$[10 * categorical[10, \"dUX1\"]]",
  "xxx[10 * categorical[10, \"dUX1\"]]",
  "?$10 * categorical[10, \"dUX1\"]]",
  "?=10 * categorical[10, \"dUX1\"]]",
  "?[10 * categorical[10, \"dUX1\"]$",
  "?[10 * categorical[10, \"dUX1\"]$",
  "$categorical[10, \"dUX1\"]",
  "=categorical[10, \"dUX1\"]",
  "$[categorical[10, \"dUX1\"]]",
  "xxx[categorical[10, \"dUX1\"]]",
  "?$categorical[10, \"dUX1\"]]",
  "?=categorical[10, \"dUX1\"]]",
  "?[categorical[10, \"dUX1\"]$",
  "?[categorical[10, \"dUX1\"]$",
  "10 $ categorical[10, \"dUX1\"]",
  "?10 $ categorical[10, \"dUX1\"]",
  "?[10 $ categorical[10, \"dUX1\"]]",
  "$10 $ categorical[10, \"dUX1\"]",
  "=10 $ categorical[10, \"dUX1\"]",
  "$[10 $ categorical[10, \"dUX1\"]]",
  "xxx[10 $ categorical[10, \"dUX1\"]]",
  "?$10 $ categorical[10, \"dUX1\"]]",
  "?=10 $ categorical[10, \"dUX1\"]]",
  "?[10 $ categorical[10, \"dUX1\"]$",
  "?[10 $ categorical[10, \"dUX1\"]$",
  "10 = categorical[10, \"dUX1\"]",
  "?10 = categorical[10, \"dUX1\"]",
  "?[10 = categorical[10, \"dUX1\"]]",
  "$10 = categorical[10, \"dUX1\"]",
  "=10 = categorical[10, \"dUX1\"]",
  "$[10 = categorical[10, \"dUX1\"]]",
  "xxx[10 = categorical[10, \"dUX1\"]]",
  "?$10 = categorical[10, \"dUX1\"]]",
  "?=10 = categorical[10, \"dUX1\"]]",
  "?[10 = categorical[10, \"dUX1\"]$",
  "?[10 = categorical[10, \"dUX1\"]$",
  "$10 * categorical[10, 10, \"dUX1\"]",
  "=10 * categorical[10, 10, \"dUX1\"]",
  "$[10 * categorical[10, 10, \"dUX1\"]]",
  "xxx[10 * categorical[10, 10, \"dUX1\"]]",
  "?$10 * categorical[10, 10, \"dUX1\"]]",
  "?=10 * categorical[10, 10, \"dUX1\"]]",
  "?[10 * categorical[10, 10, \"dUX1\"]$",
  "?[10 * categorical[10, 10, \"dUX1\"]$",
  "$categorical[10, 10, \"dUX1\"]",
  "=categorical[10, 10, \"dUX1\"]",
  "$[categorical[10, 10, \"dUX1\"]]",
  "xxx[categorical[10, 10, \"dUX1\"]]",
  "?$categorical[10, 10, \"dUX1\"]]",
  "?=categorical[10, 10, \"dUX1\"]]",
  "?[categorical[10, 10, \"dUX1\"]$",
  "?[categorical[10, 10, \"dUX1\"]$",
  "10 $ categorical[10, 10, \"dUX1\"]",
  "?10 $ categorical[10, 10, \"dUX1\"]",
  "?[10 $ categorical[10, 10, \"dUX1\"]]",
  "$10 $ categorical[10, 10, \"dUX1\"]",
  "=10 $ categorical[10, 10, \"dUX1\"]",
  "$[10 $ categorical[10, 10, \"dUX1\"]]",
  "xxx[10 $ categorical[10, 10, \"dUX1\"]]",
  "?$10 $ categorical[10, 10, \"dUX1\"]]",
  "?=10 $ categorical[10, 10, \"dUX1\"]]",
  "?[10 $ categorical[10, 10, \"dUX1\"]$",
  "?[10 $ categorical[10, 10, \"dUX1\"]$",
  "10 = categorical[10, 10, \"dUX1\"]",
  "?10 = categorical[10, 10, \"dUX1\"]",
  "?[10 = categorical[10, 10, \"dUX1\"]]",
  "$10 = categorical[10, 10, \"dUX1\"]",
  "=10 = categorical[10, 10, \"dUX1\"]",
  "$[10 = categorical[10, 10, \"dUX1\"]]",
  "xxx[10 = categorical[10, 10, \"dUX1\"]]",
  "?$10 = categorical[10, 10, \"dUX1\"]]",
  "?=10 = categorical[10, 10, \"dUX1\"]]",
  "?[10 = categorical[10, 10, \"dUX1\"]$",
  "?[10 = categorical[10, 10, \"dUX1\"]$",
  "$10 * categorical[127]",
  "=10 * categorical[127]",
  "$[10 * categorical[127]]",
  "xxx[10 * categorical[127]]",
  "?$10 * categorical[127]]",
  "?=10 * categorical[127]]",
  "?[10 * categorical[127]$",
  "?[10 * categorical[127]$",
  "$categorical[127]",
  "=categorical[127]",
  "$[categorical[127]]",
  "xxx[categorical[127]]",
  "?$categorical[127]]",
  "?=categorical[127]]",
  "?[categorical[127]$",
  "?[categorical[127]$",
  "10 $ categorical[127]",
  "?10 $ categorical[127]",
  "?[10 $ categorical[127]]",
  "$10 $ categorical[127]",
  "=10 $ categorical[127]",
  "$[10 $ categorical[127]]",
  "xxx[10 $ categorical[127]]",
  "?$10 $ categorical[127]]",
  "?=10 $ categorical[127]]",
  "?[10 $ categorical[127]$",
  "?[10 $ categorical[127]$",
  "10 = categorical[127]",
  "?10 = categorical[127]",
  "?[10 = categorical[127]]",
  "$10 = categorical[127]",
  "=10 = categorical[127]",
  "$[10 = categorical[127]]",
  "xxx[10 = categorical[127]]",
  "?$10 = categorical[127]]",
  "?=10 = categorical[127]]",
  "?[10 = categorical[127]$",
  "?[10 = categorical[127]$",
  "$10 * categorical[10, 127]",
  "=10 * categorical[10, 127]",
  "$[10 * categorical[10, 127]]",
  "xxx[10 * categorical[10, 127]]",
  "?$10 * categorical[10, 127]]",
  "?=10 * categorical[10, 127]]",
  "?[10 * categorical[10, 127]$",
  "?[10 * categorical[10, 127]$",
  "$categorical[10, 127]",
  "=categorical[10, 127]",
  "$[categorical[10, 127]]",
  "xxx[categorical[10, 127]]",
  "?$categorical[10, 127]]",
  "?=categorical[10, 127]]",
  "?[categorical[10, 127]$",
  "?[categorical[10, 127]$",
  "10 $ categorical[10, 127]",
  "?10 $ categorical[10, 127]",
  "?[10 $ categorical[10, 127]]",
  "$10 $ categorical[10, 127]",
  "=10 $ categorical[10, 127]",
  "$[10 $ categorical[10, 127]]",
  "xxx[10 $ categorical[10, 127]]",
  "?$10 $ categorical[10, 127]]",
  "?=10 $ categorical[10, 127]]",
  "?[10 $ categorical[10, 127]$",
  "?[10 $ categorical[10, 127]$",
  "10 = categorical[10, 127]",
  "?10 = categorical[10, 127]",
  "?[10 = categorical[10, 127]]",
  "$10 = categorical[10, 127]",
  "=10 = categorical[10, 127]",
  "$[10 = categorical[10, 127]]",
  "xxx[10 = categorical[10, 127]]",
  "?$10 = categorical[10, 127]]",
  "?=10 = categorical[10, 127]]",
  "?[10 = categorical[10, 127]$",
  "?[10 = categorical[10, 127]$",
  "$10 * categorical[10, 10, 127]",
  "=10 * categorical[10, 10, 127]",
  "$[10 * categorical[10, 10, 127]]",
  "xxx[10 * categorical[10, 10, 127]]",
  "?$10 * categorical[10, 10, 127]]",
  "?=10 * categorical[10, 10, 127]]",
  "?[10 * categorical[10, 10, 127]$",
  "?[10 * categorical[10, 10, 127]$",
  "$categorical[10, 10, 127]",
  "=categorical[10, 10, 127]",
  "$[categorical[10, 10, 127]]",
  "xxx[categorical[10, 10, 127]]",
  "?$categorical[10, 10, 127]]",
  "?=categorical[10, 10, 127]]",
  "?[categorical[10, 10, 127]$",
  "?[categorical[10, 10, 127]$",
  "10 $ categorical[10, 10, 127]",
  "?10 $ categorical[10, 10, 127]",
  "?[10 $ categorical[10, 10, 127]]",
  "$10 $ categorical[10, 10, 127]",
  "=10 $ categorical[10, 10, 127]",
  "$[10 $ categorical[10, 10, 127]]",
  "xxx[10 $ categorical[10, 10, 127]]",
  "?$10 $ categorical[10, 10, 127]]",
  "?=10 $ categorical[10, 10, 127]]",
  "?[10 $ categorical[10, 10, 127]$",
  "?[10 $ categorical[10, 10, 127]$",
  "10 = categorical[10, 10, 127]",
  "?10 = categorical[10, 10, 127]",
  "?[10 = categorical[10, 10, 127]]",
  "$10 = categorical[10, 10, 127]",
  "=10 = categorical[10, 10, 127]",
  "$[10 = categorical[10, 10, 127]]",
  "xxx[10 = categorical[10, 10, 127]]",
  "?$10 = categorical[10, 10, 127]]",
  "?=10 = categorical[10, 10, 127]]",
  "?[10 = categorical[10, 10, 127]$",
  "?[10 = categorical[10, 10, 127]$",
  "$10 * categorical[-128]",
  "=10 * categorical[-128]",
  "$[10 * categorical[-128]]",
  "xxx[10 * categorical[-128]]",
  "?$10 * categorical[-128]]",
  "?=10 * categorical[-128]]",
  "?[10 * categorical[-128]$",
  "?[10 * categorical[-128]$",
  "$categorical[-128]",
  "=categorical[-128]",
  "$[categorical[-128]]",
  "xxx[categorical[-128]]",
  "?$categorical[-128]]",
  "?=categorical[-128]]",
  "?[categorical[-128]$",
  "?[categorical[-128]$",
  "10 $ categorical[-128]",
  "?10 $ categorical[-128]",
  "?[10 $ categorical[-128]]",
  "$10 $ categorical[-128]",
  "=10 $ categorical[-128]",
  "$[10 $ categorical[-128]]",
  "xxx[10 $ categorical[-128]]",
  "?$10 $ categorical[-128]]",
  "?=10 $ categorical[-128]]",
  "?[10 $ categorical[-128]$",
  "?[10 $ categorical[-128]$",
  "10 = categorical[-128]",
  "?10 = categorical[-128]",
  "?[10 = categorical[-128]]",
  "$10 = categorical[-128]",
  "=10 = categorical[-128]",
  "$[10 = categorical[-128]]",
  "xxx[10 = categorical[-128]]",
  "?$10 = categorical[-128]]",
  "?=10 = categorical[-128]]",
  "?[10 = categorical[-128]$",
  "?[10 = categorical[-128]$",
  "$10 * categorical[10, -128]",
  "=10 * categorical[10, -128]",
  "$[10 * categorical[10, -128]]",
  "xxx[10 * categorical[10, -128]]",
  "?$10 * categorical[10, -128]]",
  "?=10 * categorical[10, -128]]",
  "?[10 * categorical[10, -128]$",
  "?[10 * categorical[10, -128]$",
  "$categorical[10, -128]",
  "=categorical[10, -128]",
  "$[categorical[10, -128]]",
  "xxx[categorical[10, -128]]",
  "?$categorical[10, -128]]",
  "?=categorical[10, -128]]",
  "?[categorical[10, -128]$",
  "?[categorical[10, -128]$",
  "10 $ categorical[10, -128]",
  "?10 $ categorical[10, -128]",
  "?[10 $ categorical[10, -128]]",
  "$10 $ categorical[10, -128]",
  "=10 $ categorical[10, -128]",
  "$[10 $ categorical[10, -128]]",
  "xxx[10 $ categorical[10, -128]]",
  "?$10 $ categorical[10, -128]]",
  "?=10 $ categorical[10, -128]]",
  "?[10 $ categorical[10, -128]$",
  "?[10 $ categorical[10, -128]$",
  "10 = categorical[10, -128]",
  "?10 = categorical[10, -128]",
  "?[10 = categorical[10, -128]]",
  "$10 = categorical[10, -128]",
  "=10 = categorical[10, -128]",
  "$[10 = categorical[10, -128]]",
  "xxx[10 = categorical[10, -128]]",
  "?$10 = categorical[10, -128]]",
  "?=10 = categorical[10, -128]]",
  "?[10 = categorical[10, -128]$",
  "?[10 = categorical[10, -128]$",
  "$10 * categorical[10, 10, -128]",
  "=10 * categorical[10, 10, -128]",
  "$[10 * categorical[10, 10, -128]]",
  "xxx[10 * categorical[10, 10, -128]]",
  "?$10 * categorical[10, 10, -128]]",
  "?=10 * categorical[10, 10, -128]]",
  "?[10 * categorical[10, 10, -128]$",
  "?[10 * categorical[10, 10, -128]$",
  "$categorical[10, 10, -128]",
  "=categorical[10, 10, -128]",
  "$[categorical[10, 10, -128]]",
  "xxx[categorical[10, 10, -128]]",
  "?$categorical[10, 10, -128]]",
  "?=categorical[10, 10, -128]]",
  "?[categorical[10, 10, -128]$",
  "?[categorical[10, 10, -128]$",
  "10 $ categorical[10, 10, -128]",
  "?10 $ categorical[10, 10, -128]",
  "?[10 $ categorical[10, 10, -128]]",
  "$10 $ categorical[10, 10, -128]",
  "=10 $ categorical[10, 10, -128]",
  "$[10 $ categorical[10, 10, -128]]",
  "xxx[10 $ categorical[10, 10, -128]]",
  "?$10 $ categorical[10, 10, -128]]",
  "?=10 $ categorical[10, 10, -128]]",
  "?[10 $ categorical[10, 10, -128]$",
  "?[10 $ categorical[10, 10, -128]$",
  "10 = categorical[10, 10, -128]",
  "?10 = categorical[10, 10, -128]",
  "?[10 = categorical[10, 10, -128]]",
  "$10 = categorical[10, 10, -128]",
  "=10 = categorical[10, 10, -128]",
  "$[10 = categorical[10, 10, -128]]",
  "xxx[10 = categorical[10, 10, -128]]",
  "?$10 = categorical[10, 10, -128]]",
  "?=10 = categorical[10, 10, -128]]",
  "?[10 = categorical[10, 10, -128]$",
  "?[10 = categorical[10, 10, -128]$",
  "$10 * categorical[32767]",
  "=10 * categorical[32767]",
  "$[10 * categorical[32767]]",
  "xxx[10 * categorical[32767]]",
  "?$10 * categorical[32767]]",
  "?=10 * categorical[32767]]",
  "?[10 * categorical[32767]$",
  "?[10 * categorical[32767]$",
  "$categorical[32767]",
  "=categorical[32767]",
  "$[categorical[32767]]",
  "xxx[categorical[32767]]",
  "?$categorical[32767]]",
  "?=categorical[32767]]",
  "?[categorical[32767]$",
  "?[categorical[32767]$",
  "10 $ categorical[32767]",
  "?10 $ categorical[32767]",
  "?[10 $ categorical[32767]]",
  "$10 $ categorical[32767]",
  "=10 $ categorical[32767]",
  "$[10 $ categorical[32767]]",
  "xxx[10 $ categorical[32767]]",
  "?$10 $ categorical[32767]]",
  "?=10 $ categorical[32767]]",
  "?[10 $ categorical[32767]$",
  "?[10 $ categorical[32767]$",
  "10 = categorical[32767]",
  "?10 = categorical[32767]",
  "?[10 = categorical[32767]]",
  "$10 = categorical[32767]",
  "=10 = categorical[32767]",
  "$[10 = categorical[32767]]",
  "xxx[10 = categorical[32767]]",
  "?$10 = categorical[32767]]",
  "?=10 = categorical[32767]]",
  "?[10 = categorical[32767]$",
  "?[10 = categorical[32767]$",
  "$10 * categorical[10, 32767]",
  "=10 * categorical[10, 32767]",
  "$[10 * categorical[10, 32767]]",
  "xxx[10 * categorical[10, 32767]]",
  "?$10 * categorical[10, 32767]]",
  "?=10 * categorical[10, 32767]]",
  "?[10 * categorical[10, 32767]$",
  "?[10 * categorical[10, 32767]$",
  "$categorical[10, 32767]",
  "=categorical[10, 32767]",
  "$[categorical[10, 32767]]",
  "xxx[categorical[10, 32767]]",
  "?$categorical[10, 32767]]",
  "?=categorical[10, 32767]]",
  "?[categorical[10, 32767]$",
  "?[categorical[10, 32767]$",
  "10 $ categorical[10, 32767]",
  "?10 $ categorical[10, 32767]",
  "?[10 $ categorical[10, 32767]]",
  "$10 $ categorical[10, 32767]",
  "=10 $ categorical[10, 32767]",
  "$[10 $ categorical[10, 32767]]",
  "xxx[10 $ categorical[10, 32767]]",
  "?$10 $ categorical[10, 32767]]",
  "?=10 $ categorical[10, 32767]]",
  "?[10 $ categorical[10, 32767]$",
  "?[10 $ categorical[10, 32767]$",
  "10 = categorical[10, 32767]",
  "?10 = categorical[10, 32767]",
  "?[10 = categorical[10, 32767]]",
  "$10 = categorical[10, 32767]",
  "=10 = categorical[10, 32767]",
  "$[10 = categorical[10, 32767]]",
  "xxx[10 = categorical[10, 32767]]",
  "?$10 = categorical[10, 32767]]",
  "?=10 = categorical[10, 32767]]",
  "?[10 = categorical[10, 32767]$",
  "?[10 = categorical[10, 32767]$",
  "$10 * categorical[10, 10, 32767]",
  "=10 * categorical[10, 10, 32767]",
  "$[10 * categorical[10, 10, 32767]]",
  "xxx[10 * categorical[10, 10, 32767]]",
  "?$10 * categorical[10, 10, 32767]]",
  "?=10 * categorical[10, 10, 32767]]",
  "?[10 * categorical[10, 10, 32767]$",
  "?[10 * categorical[10, 10, 32767]$",
  "$categorical[10, 10, 32767]",
  "=categorical[10, 10, 32767]",
  "$[categorical[10, 10, 32767]]",
  "xxx[categorical[10, 10, 32767]]",
  "?$categorical[10, 10, 32767]]",
  "?=categorical[10, 10, 32767]]",
  "?[categorical[10, 10, 32767]$",
  "?[categorical[10, 10, 32767]$",
  "10 $ categorical[10, 10, 32767]",
  "?10 $ categorical[10, 10, 32767]",
  "?[10 $ categorical[10, 10, 32767]]",
  "$10 $ categorical[10, 10, 32767]",
  "=10 $ categorical[10, 10, 32767]",
  "$[10 $ categorical[10, 10, 32767]]",
  "xxx[10 $ categorical[10, 10, 32767]]",
  "?$10 $ categorical[10, 10, 32767]]",
  "?=10 $ categorical[10, 10, 32767]]",
  "?[10 $ categorical[10, 10, 32767]$",
  "?[10 $ categorical[10, 10, 32767]$",
  "10 = categorical[10, 10, 32767]",
  "?10 = categorical[10, 10, 32767]",
  "?[10 = categorical[10, 10, 32767]]",
  "$10 = categorical[10, 10, 32767]",
  "=10 = categorical[10, 10, 32767]",
  "$[10 = categorical[10, 10, 32767]]",
  "xxx[10 = categorical[10, 10, 32767]]",
  "?$10 = categorical[10, 10, 32767]]",
  "?=10 = categorical[10, 10, 32767]]",
  "?[10 = categorical[10, 10, 32767]$",
  "?[10 = categorical[10, 10, 32767]$",
  "$10 * categorical[-32768]",
  "=10 * categorical[-32768]",
  "$[10 * categorical[-32768]]",
  "xxx[10 * categorical[-32768]]",
  "?$10 * categorical[-32768]]",
  "?=10 * categorical[-32768]]",
  "?[10 * categorical[-32768]$",
  "?[10 * categorical[-32768]$",
  "$categorical[-32768]",
  "=categorical[-32768]",
  "$[categorical[-32768]]",
  "xxx[categorical[-32768]]",
  "?$categorical[-32768]]",
  "?=categorical[-32768]]",
  "?[categorical[-32768]$",
  "?[categorical[-32768]$",
  "10 $ categorical[-32768]",
  "?10 $ categorical[-32768]",
  "?[10 $ categorical[-32768]]",
  "$10 $ categorical[-32768]",
  "=10 $ categorical[-32768]",
  "$[10 $ categorical[-32768]]",
  "xxx[10 $ categorical[-32768]]",
  "?$10 $ categorical[-32768]]",
  "?=10 $ categorical[-32768]]",
  "?[10 $ categorical[-32768]$",
  "?[10 $ categorical[-32768]$",
  "10 = categorical[-32768]",
  "?10 = categorical[-32768]",
  "?[10 = categorical[-32768]]",
  "$10 = categorical[-32768]",
  "=10 = categorical[-32768]",
  "$[10 = categorical[-32768]]",
  "xxx[10 = categorical[-32768]]",
  "?$10 = categorical[-32768]]",
  "?=10 = categorical[-32768]]",
  "?[10 = categorical[-32768]$",
  "?[10 = categorical[-32768]$",
  "$10 * categorical[10, -32768]",
  "=10 * categorical[10, -32768]",
  "$[10 * categorical[10, -32768]]",
  "xxx[10 * categorical[10, -32768]]",
  "?$10 * categorical[10, -32768]]",
  "?=10 * categorical[10, -32768]]",
  "?[10 * categorical[10, -32768]$",
  "?[10 * categorical[10, -32768]$",
  "$categorical[10, -32768]",
  "=categorical[10, -32768]",
  "$[categorical[10, -32768]]",
  "xxx[categorical[10, -32768]]",
  "?$categorical[10, -32768]]",
  "?=categorical[10, -32768]]",
  "?[categorical[10, -32768]$",
  "?[categorical[10, -32768]$",
  "10 $ categorical[10, -32768]",
  "?10 $ categorical[10, -32768]",
  "?[10 $ categorical[10, -32768]]",
  "$10 $ categorical[10, -32768]",
  "=10 $ categorical[10, -32768]",
  "$[10 $ categorical[10, -32768]]",
  "xxx[10 $ categorical[10, -32768]]",
  "?$10 $ categorical[10, -32768]]",
  "?=10 $ categorical[10, -32768]]",
  "?[10 $ categorical[10, -32768]$",
  "?[10 $ categorical[10, -32768]$",
  "10 = categorical[10, -32768]",
  "?10 = categorical[10, -32768]",
  "?[10 = categorical[10, -32768]]",
  "$10 = categorical[10, -32768]",
  "=10 = categorical[10, -32768]",
  "$[10 = categorical[10, -32768]]",
  "xxx[10 = categorical[10, -32768]]",
  "?$10 = categorical[10, -32768]]",
  "?=10 = categorical[10, -32768]]",
  "?[10 = categorical[10, -32768]$",
  "?[10 = categorical[10, -32768]$",
  "$10 * categorical[10, 10, -32768]",
  "=10 * categorical[10, 10, -32768]",
  "$[10 * categorical[10, 10, -32768]]",
  "xxx[10 * categorical[10, 10, -32768]]",
  "?$10 * categorical[10, 10, -32768]]",
  "?=10 * categorical[10, 10, -32768]]",
  "?[10 * categorical[10, 10, -32768]$",
  "?[10 * categorical[10, 10, -32768]$",
  "$categorical[10, 10, -32768]",
  "=categorical[10, 10, -32768]",
  "$[categorical[10, 10, -32768]]",
  "xxx[categorical[10, 10, -32768]]",
  "?$categorical[10, 10, -32768]]",
  "?=categorical[10, 10, -32768]]",
  "?[categorical[10, 10, -32768]$",
  "?[categorical[10, 10, -32768]$",
  "10 $ categorical[10, 10, -32768]",
  "?10 $ categorical[10, 10, -32768]",
  "?[10 $ categorical[10, 10, -32768]]",
  "$10 $ categorical[10, 10, -32768]",
  "=10 $ categorical[10, 10, -32768]",
  "$[10 $ categorical[10, 10, -32768]]",
  "xxx[10 $ categorical[10, 10, -32768]]",
  "?$10 $ categorical[10, 10, -32768]]",
  "?=10 $ categorical[10, 10, -32768]]",
  "?[10 $ categorical[10, 10, -32768]$",
  "?[10 $ categorical[10, 10, -32768]$",
  "10 = categorical[10, 10, -32768]",
  "?10 = categorical[10, 10, -32768]",
  "?[10 = categorical[10, 10, -32768]]",
  "$10 = categorical[10, 10, -32768]",
  "=10 = categorical[10, 10, -32768]",
  "$[10 = categorical[10, 10, -32768]]",
  "xxx[10 = categorical[10, 10, -32768]]",
  "?$10 = categorical[10, 10, -32768]]",
  "?=10 = categorical[10, 10, -32768]]",
  "?[10 = categorical[10, 10, -32768]$",
  "?[10 = categorical[10, 10, -32768]$",
  "$10 * categorical[2147483647]",
  "=10 * categorical[2147483647]",
  "$[10 * categorical[2147483647]]",
  "xxx[10 * categorical[2147483647]]",
  "?$10 * categorical[2147483647]]",
  "?=10 * categorical[2147483647]]",
  "?[10 * categorical[2147483647]$",
  "?[10 * categorical[2147483647]$",
  "$categorical[2147483647]",
  "=categorical[2147483647]",
  "$[categorical[2147483647]]",
  "xxx[categorical[2147483647]]",
  "?$categorical[2147483647]]",
  "?=categorical[2147483647]]",
  "?[categorical[2147483647]$",
  "?[categorical[2147483647]$",
  "10 $ categorical[2147483647]",
  "?10 $ categorical[2147483647]",
  "?[10 $ categorical[2147483647]]",
  "$10 $ categorical[2147483647]",
  "=10 $ categorical[2147483647]",
  "$[10 $ categorical[2147483647]]",
  "xxx[10 $ categorical[2147483647]]",
  "?$10 $ categorical[2147483647]]",
  "?=10 $ categorical[2147483647]]",
  "?[10 $ categorical[2147483647]$",
  "?[10 $ categorical[2147483647]$",
  "10 = categorical[2147483647]",
  "?10 = categorical[2147483647]",
  "?[10 = categorical[2147483647]]",
  "$10 = categorical[2147483647]",
  "=10 = categorical[2147483647]",
  "$[10 = categorical[2147483647]]",
  "xxx[10 = categorical[2147483647]]",
  "?$10 = categorical[2147483647]]",
  "?=10 = categorical[2147483647]]",
  "?[10 = categorical[2147483647]$",
  "?[10 = categorical[2147483647]$",
  "$10 * categorical[10, 2147483647]",
  "=10 * categorical[10, 2147483647]",
  "$[10 * categorical[10, 2147483647]]",
  "xxx[10 * categorical[10, 2147483647]]",
  "?$10 * categorical[10, 2147483647]]",
  "?=10 * categorical[10, 2147483647]]",
  "?[10 * categorical[10, 2147483647]$",
  "?[10 * categorical[10, 2147483647]$",
  "$categorical[10, 2147483647]",
  "=categorical[10, 2147483647]",
  "$[categorical[10, 2147483647]]",
  "xxx[categorical[10, 2147483647]]",
  "?$categorical[10, 2147483647]]",
  "?=categorical[10, 2147483647]]",
  "?[categorical[10, 2147483647]$",
  "?[categorical[10, 2147483647]$",
  "10 $ categorical[10, 2147483647]",
  "?10 $ categorical[10, 2147483647]",
  "?[10 $ categorical[10, 2147483647]]",
  "$10 $ categorical[10, 2147483647]",
  "=10 $ categorical[10, 2147483647]",
  "$[10 $ categorical[10, 2147483647]]",
  "xxx[10 $ categorical[10, 2147483647]]",
  "?$10 $ categorical[10, 2147483647]]",
  "?=10 $ categorical[10, 2147483647]]",
  "?[10 $ categorical[10, 2147483647]$",
  "?[10 $ categorical[10, 2147483647]$",
  "10 = categorical[10, 2147483647]",
  "?10 = categorical[10, 2147483647]",
  "?[10 = categorical[10, 2147483647]]",
  "$10 = categorical[10, 2147483647]",
  "=10 = categorical[10, 2147483647]",
  "$[10 = categorical[10, 2147483647]]",
  "xxx[10 = categorical[10, 2147483647]]",
  "?$10 = categorical[10, 2147483647]]",
  "?=10 = categorical[10, 2147483647]]",
  "?[10 = categorical[10, 2147483647]$",
  "?[10 = categorical[10, 2147483647]$",
  "$10 * categorical[10, 10, 2147483647]",
  "=10 * categorical[10, 10, 2147483647]",
  "$[10 * categorical[10, 10, 2147483647]]",
  "xxx[10 * categorical[10, 10, 2147483647]]",
  "?$10 * categorical[10, 10, 2147483647]]",
  "?=10 * categorical[10, 10, 2147483647]]",
  "?[10 * categorical[10, 10, 2147483647]$",
  "?[10 * categorical[10, 10, 2147483647]$",
  "$categorical[10, 10, 2147483647]",
  "=categorical[10, 10, 2147483647]",
  "$[categorical[10, 10, 2147483647]]",
  "xxx[categorical[10, 10, 2147483647]]",
  "?$categorical[10, 10, 2147483647]]",
  "?=categorical[10, 10, 2147483647]]",
  "?[categorical[10, 10, 2147483647]$",
  "?[categorical[10, 10, 2147483647]$",
  "10 $ categorical[10, 10, 2147483647]",
  "?10 $ categorical[10, 10, 2147483647]",
  "?[10 $ categorical[10, 10, 2147483647]]",
  "$10 $ categorical[10, 10, 2147483647]",
  "=10 $ categorical[10, 10, 2147483647]",
  "$[10 $ categorical[10, 10, 2147483647]]",
  "xxx[10 $ categorical[10, 10, 2147483647]]",
  "?$10 $ categorical[10, 10, 2147483647]]",
  "?=10 $ categorical[10, 10, 2147483647]]",
  "?[10 $ categorical[10, 10, 2147483647]$",
  "?[10 $ categorical[10, 10, 2147483647]$",
  "10 = categorical[10, 10, 2147483647]",
  "?10 = categorical[10, 10, 2147483647]",
  "?[10 = categorical[10, 10, 2147483647]]",
  "$10 = categorical[10, 10, 2147483647]",
  "=10 = categorical[10, 10, 2147483647]",
  "$[10 = categorical[10, 10, 2147483647]]",
  "xxx[10 = categorical[10, 10, 2147483647]]",
  "?$10 = categorical[10, 10, 2147483647]]",
  "?=10 = categorical[10, 10, 2147483647]]",
  "?[10 = categorical[10, 10, 2147483647]$",
  "?[10 = categorical[10, 10, 2147483647]$",
  "$10 * categorical[-2147483648]",
  "=10 * categorical[-2147483648]",
  "$[10 * categorical[-2147483648]]",
  "xxx[10 * categorical[-2147483648]]",
  "?$10 * categorical[-2147483648]]",
  "?=10 * categorical[-2147483648]]",
  "?[10 * categorical[-2147483648]$",
  "?[10 * categorical[-2147483648]$",
  "$categorical[-2147483648]",
  "=categorical[-2147483648]",
  "$[categorical[-2147483648]]",
  "xxx[categorical[-2147483648]]",
  "?$categorical[-2147483648]]",
  "?=categorical[-2147483648]]",
  "?[categorical[-2147483648]$",
  "?[categorical[-2147483648]$",
  "10 $ categorical[-2147483648]",
  "?10 $ categorical[-2147483648]",
  "?[10 $ categorical[-2147483648]]",
  "$10 $ categorical[-2147483648]",
  "=10 $ categorical[-2147483648]",
  "$[10 $ categorical[-2147483648]]",
  "xxx[10 $ categorical[-2147483648]]",
  "?$10 $ categorical[-2147483648]]",
  "?=10 $ categorical[-2147483648]]",
  "?[10 $ categorical[-2147483648]$",
  "?[10 $ categorical[-2147483648]$",
  "10 = categorical[-2147483648]",
  "?10 = categorical[-2147483648]",
  "?[10 = categorical[-2147483648]]",
  "$10 = categorical[-2147483648]",
  "=10 = categorical[-2147483648]",
  "$[10 = categorical[-2147483648]]",
  "xxx[10 = categorical[-2147483648]]",
  "?$10 = categorical[-2147483648]]",
  "?=10 = categorical[-2147483648]]",
  "?[10 = categorical[-2147483648]$",
  "?[10 = categorical[-2147483648]$",
  "$10 * categorical[10, -2147483648]",
  "=10 * categorical[10, -2147483648]",
  "$[10 * categorical[10, -2147483648]]",
  "xxx[10 * categorical[10, -2147483648]]",
  "?$10 * categorical[10, -2147483648]]",
  "?=10 * categorical[10, -2147483648]]",
  "?[10 * categorical[10, -2147483648]$",
  "?[10 * categorical[10, -2147483648]$",
  "$categorical[10, -2147483648]",
  "=categorical[10, -2147483648]",
  "$[categorical[10, -2147483648]]",
  "xxx[categorical[10, -2147483648]]",
  "?$categorical[10, -2147483648]]",
  "?=categorical[10, -2147483648]]",
  "?[categorical[10, -2147483648]$",
  "?[categorical[10, -2147483648]$",
  "10 $ categorical[10, -2147483648]",
  "?10 $ categorical[10, -2147483648]",
  "?[10 $ categorical[10, -2147483648]]",
  "$10 $ categorical[10, -2147483648]",
  "=10 $ categorical[10, -2147483648]",
  "$[10 $ categorical[10, -2147483648]]",
  "xxx[10 $ categorical[10, -2147483648]]",
  "?$10 $ categorical[10, -2147483648]]",
  "?=10 $ categorical[10, -2147483648]]",
  "?[10 $ categorical[10, -2147483648]$",
  "?[10 $ categorical[10, -2147483648]$",
  "10 = categorical[10, -2147483648]",
  "?10 = categorical[10, -2147483648]",
  "?[10 = categorical[10, -2147483648]]",
  "$10 = categorical[10, -2147483648]",
  "=10 = categorical[10, -2147483648]",
  "$[10 = categorical[10, -2147483648]]",
  "xxx[10 = categorical[10, -2147483648]]",
  "?$10 = categorical[10, -2147483648]]",
  "?=10 = categorical[10, -2147483648]]",
  "?[10 = categorical[10, -2147483648]$",
  "?[10 = categorical[10, -2147483648]$",
  "$10 * categorical[10, 10, -2147483648]",
  "=10 * categorical[10, 10, -2147483648]",
  "$[10 * categorical[10, 10, -2147483648]]",
  "xxx[10 * categorical[10, 10, -2147483648]]",
  "?$10 * categorical[10, 10, -2147483648]]",
  "?=10 * categorical[10, 10, -2147483648]]",
  "?[10 * categorical[10, 10, -2147483648]$",
  "?[10 * categorical[10, 10, -2147483648]$",
  "$categorical[10, 10, -2147483648]",
  "=categorical[10, 10, -2147483648]",
  "$[categorical[10, 10, -2147483648]]",
  "xxx[categorical[10, 10, -2147483648]]",
  "?$categorical[10, 10, -2147483648]]",
  "?=categorical[10, 10, -2147483648]]",
  "?[categorical[10, 10, -2147483648]$",
  "?[categorical[10, 10, -2147483648]$",
  "10 $ categorical[10, 10, -2147483648]",
  "?10 $ categorical[10, 10, -2147483648]",
  "?[10 $ categorical[10, 10, -2147483648]]",
  "$10 $ categorical[10, 10, -2147483648]",
  "=10 $ categorical[10, 10, -2147483648]",
  "$[10 $ categorical[10, 10, -2147483648]]",
  "xxx[10 $ categorical[10, 10, -2147483648]]",
  "?$10 $ categorical[10, 10, -2147483648]]",
  "?=10 $ categorical[10, 10, -2147483648]]",
  "?[10 $ categorical[10, 10, -2147483648]$",
  "?[10 $ categorical[10, 10, -2147483648]$",
  "10 = categorical[10, 10, -2147483648]",
  "?10 = categorical[10, 10, -2147483648]",
  "?[10 = categorical[10, 10, -2147483648]]",
  "$10 = categorical[10, 10, -2147483648]",
  "=10 = categorical[10, 10, -2147483648]",
  "$[10 = categorical[10, 10, -2147483648]]",
  "xxx[10 = categorical[10, 10, -2147483648]]",
  "?$10 = categorical[10, 10, -2147483648]]",
  "?=10 = categorical[10, 10, -2147483648]]",
  "?[10 = categorical[10, 10, -2147483648]$",
  "?[10 = categorical[10, 10, -2147483648]$",
  "$10 * categorical[9223372036854775807]",
  "=10 * categorical[9223372036854775807]",
  "$[10 * categorical[9223372036854775807]]",
  "xxx[10 * categorical[9223372036854775807]]",
  "?$10 * categorical[9223372036854775807]]",
  "?=10 * categorical[9223372036854775807]]",
  "?[10 * categorical[9223372036854775807]$",
  "?[10 * categorical[9223372036854775807]$",
  "$categorical[9223372036854775807]",
  "=categorical[9223372036854775807]",
  "$[categorical[9223372036854775807]]",
  "xxx[categorical[9223372036854775807]]",
  "?$categorical[9223372036854775807]]",
  "?=categorical[9223372036854775807]]",
  "?[categorical[9223372036854775807]$",
  "?[categorical[9223372036854775807]$",
  "10 $ categorical[9223372036854775807]",
  "?10 $ categorical[9223372036854775807]",
  "?[10 $ categorical[9223372036854775807]]",
  "$10 $ categorical[9223372036854775807]",
  "=10 $ categorical[9223372036854775807]",
  "$[10 $ categorical[9223372036854775807]]",
  "xxx[10 $ categorical[9223372036854775807]]",
  "?$10 $ categorical[9223372036854775807]]",
  "?=10 $ categorical[9223372036854775807]]",
  "?[10 $ categorical[9223372036854775807]$",
  "?[10 $ categorical[9223372036854775807]$",
  "10 = categorical[9223372036854775807]",
  "?10 = categorical[9223372036854775807]",
  "?[10 = categorical[9223372036854775807]]",
  "$10 = categorical[9223372036854775807]",
  "=10 = categorical[9223372036854775807]",
  "$[10 = categorical[9223372036854775807]]",
  "xxx[10 = categorical[9223372036854775807]]",
  "?$10 = categorical[9223372036854775807]]",
  "?=10 = categorical[9223372036854775807]]",
  "?[10 = categorical[9223372036854775807]$",
  "?[10 = categorical[9223372036854775807]$",
  "$10 * categorical[10, 9223372036854775807]",
  "=10 * categorical[10, 9223372036854775807]",
  "$[10 * categorical[10, 9223372036854775807]]",
  "xxx[10 * categorical[10, 9223372036854775807]]",
  "?$10 * categorical[10, 9223372036854775807]]",
  "?=10 * categorical[10, 9223372036854775807]]",
  "?[10 * categorical[10, 9223372036854775807]$",
  "?[10 * categorical[10, 9223372036854775807]$",
  "$categorical[10, 9223372036854775807]",
  "=categorical[10, 9223372036854775807]",
  "$[categorical[10, 9223372036854775807]]",
  "xxx[categorical[10, 9223372036854775807]]",
  "?$categorical[10, 9223372036854775807]]",
  "?=categorical[10, 9223372036854775807]]",
  "?[categorical[10, 9223372036854775807]$",
  "?[categorical[10, 9223372036854775807]$",
  "10 $ categorical[10, 9223372036854775807]",
  "?10 $ categorical[10, 9223372036854775807]",
  "?[10 $ categorical[10, 9223372036854775807]]",
  "$10 $ categorical[10, 9223372036854775807]",
  "=10 $ categorical[10, 9223372036854775807]",
  "$[10 $ categorical[10, 9223372036854775807]]",
  "xxx[10 $ categorical[10, 9223372036854775807]]",
  "?$10 $ categorical[10, 9223372036854775807]]",
  "?=10 $ categorical[10, 9223372036854775807]]",
  "?[10 $ categorical[10, 9223372036854775807]$",
  "?[10 $ categorical[10, 9223372036854775807]$",
  "10 = categorical[10, 9223372036854775807]",
  "?10 = categorical[10, 9223372036854775807]",
  "?[10 = categorical[10, 9223372036854775807]]",
  "$10 = categorical[10, 9223372036854775807]",
  "=10 = categorical[10, 9223372036854775807]",
  "$[10 = categorical[10, 9223372036854775807]]",
  "xxx[10 = categorical[10, 9223372036854775807]]",
  "?$10 = categorical[10, 9223372036854775807]]",
  "?=10 = categorical[10, 9223372036854775807]]",
  "?[10 = categorical[10, 9223372036854775807]$",
  "?[10 = categorical[10, 9223372036854775807]$",
  "$10 * categorical[10, 10, 9223372036854775807]",
  "=10 * categorical[10, 10, 9223372036854775807]",
  "$[10 * categorical[10, 10, 9223372036854775807]]",
  "xxx[10 * categorical[10, 10, 9223372036854775807]]",
  "?$10 * categorical[10, 10, 9223372036854775807]]",
  "?=10 * categorical[10, 10, 9223372036854775807]]",
  "?[10 * categorical[10, 10, 9223372036854775807]$",
  "?[10 * categorical[10, 10, 9223372036854775807]$",
  "$categorical[10, 10, 9223372036854775807]",
  "=categorical[10, 10, 9223372036854775807]",
  "$[categorical[10, 10, 9223372036854775807]]",
  "xxx[categorical[10, 10, 9223372036854775807]]",
  "?$categorical[10, 10, 9223372036854775807]]",
  "?=categorical[10, 10, 9223372036854775807]]",
  "?[categorical[10, 10, 9223372036854775807]$",
  "?[categorical[10, 10, 9223372036854775807]$",
  "10 $ categorical[10, 10, 9223372036854775807]",
  "?10 $ categorical[10, 10, 9223372036854775807]",
  "?[10 $ categorical[10, 10, 9223372036854775807]]",
  "$10 $ categorical[10, 10, 9223372036854775807]",
  "=10 $ categorical[10, 10, 9223372036854775807]",
  "$[10 $ categorical[10, 10, 9223372036854775807]]",
  "xxx[10 $ categorical[10, 10, 9223372036854775807]]",
  "?$10 $ categorical[10, 10, 9223372036854775807]]",
  "?=10 $ categorical[10, 10, 9223372036854775807]]",
  "?[10 $ categorical[10, 10, 9223372036854775807]$",
  "?[10 $ categorical[10, 10, 9223372036854775807]$",
  "10 = categorical[10, 10, 9223372036854775807]",
  "?10 = categorical[10, 10, 9223372036854775807]",
  "?[10 = categorical[10, 10, 9223372036854775807]]",
  "$10 = categorical[10, 10, 9223372036854775807]",
  "=10 = categorical[10, 10, 9223372036854775807]",
  "$[10 = categorical[10, 10, 9223372036854775807]]",
  "xxx[10 = categorical[10, 10, 9223372036854775807]]",
  "?$10 = categorical[10, 10, 9223372036854775807]]",
  "?=10 = categorical[10, 10, 9223372036854775807]]",
  "?[10 = categorical[10, 10, 9223372036854775807]$",
  "?[10 = categorical[10, 10, 9223372036854775807]$",
  "$10 * categorical[-9223372036854775808]",
  "=10 * categorical[-9223372036854775808]",
  "$[10 * categorical[-9223372036854775808]]",
  "xxx[10 * categorical[-9223372036854775808]]",
  "?$10 * categorical[-9223372036854775808]]",
  "?=10 * categorical[-9223372036854775808]]",
  "?[10 * categorical[-9223372036854775808]$",
  "?[10 * categorical[-9223372036854775808]$",
  "$categorical[-9223372036854775808]",
  "=categorical[-9223372036854775808]",
  "$[categorical[-9223372036854775808]]",
  "xxx[categorical[-9223372036854775808]]",
  "?$categorical[-9223372036854775808]]",
  "?=categorical[-9223372036854775808]]",
  "?[categorical[-9223372036854775808]$",
  "?[categorical[-9223372036854775808]$",
  "10 $ categorical[-9223372036854775808]",
  "?10 $ categorical[-9223372036854775808]",
  "?[10 $ categorical[-9223372036854775808]]",
  "$10 $ categorical[-9223372036854775808]",
  "=10 $ categorical[-9223372036854775808]",
  "$[10 $ categorical[-9223372036854775808]]",
  "xxx[10 $ categorical[-9223372036854775808]]",
  "?$10 $ categorical[-9223372036854775808]]",
  "?=10 $ categorical[-9223372036854775808]]",
  "?[10 $ categorical[-9223372036854775808]$",
  "?[10 $ categorical[-9223372036854775808]$",
  "10 = categorical[-9223372036854775808]",
  "?10 = categorical[-9223372036854775808]",
  "?[10 = categorical[-9223372036854775808]]",
  "$10 = categorical[-9223372036854775808]",
  "=10 = categorical[-9223372036854775808]",
  "$[10 = categorical[-9223372036854775808]]",
  "xxx[10 = categorical[-9223372036854775808]]",
  "?$10 = categorical[-9223372036854775808]]",
  "?=10 = categorical[-9223372036854775808]]",
  "?[10 = categorical[-9223372036854775808]$",
  "?[10 = categorical[-9223372036854775808]$",
  "$10 * categorical[10, -9223372036854775808]",
  "=10 * categorical[10, -9223372036854775808]",
  "$[10 * categorical[10, -9223372036854775808]]",
  "xxx[10 * categorical[10, -9223372036854775808]]",
  "?$10 * categorical[10, -9223372036854775808]]",
  "?=10 * categorical[10, -9223372036854775808]]",
  "?[10 * categorical[10, -9223372036854775808]$",
  "?[10 * categorical[10, -9223372036854775808]$",
  "$categorical[10, -9223372036854775808]",
  "=categorical[10, -9223372036854775808]",
  "$[categorical[10, -9223372036854775808]]",
  "xxx[categorical[10, -9223372036854775808]]",
  "?$categorical[10, -9223372036854775808]]",
  "?=categorical[10, -9223372036854775808]]",
  "?[categorical[10, -9223372036854775808]$",
  "?[categorical[10, -9223372036854775808]$",
  "10 $ categorical[10, -9223372036854775808]",
  "?10 $ categorical[10, -9223372036854775808]",
  "?[10 $ categorical[10, -9223372036854775808]]",
  "$10 $ categorical[10, -9223372036854775808]",
  "=10 $ categorical[10, -9223372036854775808]",
  "$[10 $ categorical[10, -9223372036854775808]]",
  "xxx[10 $ categorical[10, -9223372036854775808]]",
  "?$10 $ categorical[10, -9223372036854775808]]",
  "?=10 $ categorical[10, -9223372036854775808]]",
  "?[10 $ categorical[10, -9223372036854775808]$",
  "?[10 $ categorical[10, -9223372036854775808]$",
  "10 = categorical[10, -9223372036854775808]",
  "?10 = categorical[10, -9223372036854775808]",
  "?[10 = categorical[10, -9223372036854775808]]",
  "$10 = categorical[10, -9223372036854775808]",
  "=10 = categorical[10, -9223372036854775808]",
  "$[10 = categorical[10, -9223372036854775808]]",
  "xxx[10 = categorical[10, -9223372036854775808]]",
  "?$10 = categorical[10, -9223372036854775808]]",
  "?=10 = categorical[10, -9223372036854775808]]",
  "?[10 = categorical[10, -9223372036854775808]$",
  "?[10 = categorical[10, -9223372036854775808]$",
  "$10 * categorical[10, 10, -9223372036854775808]",
  "=10 * categorical[10, 10, -9223372036854775808]",
  "$[10 * categorical[10, 10, -9223372036854775808]]",
  "xxx[10 * categorical[10, 10, -9223372036854775808]]",
  "?$10 * categorical[10, 10, -9223372036854775808]]",
  "?=10 * categorical[10, 10, -9223372036854775808]]",
  "?[10 * categorical[10, 10, -9223372036854775808]$",
  "?[10 * categorical[10, 10, -9223372036854775808]$",
  "$categorical[10, 10, -9223372036854775808]",
  "=categorical[10, 10, -9223372036854775808]",
  "$[categorical[10, 10, -9223372036854775808]]",
  "xxx[categorical[10, 10, -9223372036854775808]]",
  "?$categorical[10, 10, -9223372036854775808]]",
  "?=categorical[10, 10, -9223372036854775808]]",
  "?[categorical[10, 10, -9223372036854775808]$",
  "?[categorical[10, 10, -9223372036854775808]$",
  "10 $ categorical[10, 10, -9223372036854775808]",
  "?10 $ categorical[10, 10, -9223372036854775808]",
  "?[10 $ categorical[10, 10, -9223372036854775808]]",
  "$10 $ categorical[10, 10, -9223372036854775808]",
  "=10 $ categorical[10, 10, -9223372036854775808]",
  "$[10 $ categorical[10, 10, -9223372036854775808]]",
  "xxx[10 $ categorical[10, 10, -9223372036854775808]]",
  "?$10 $ categorical[10, 10, -9223372036854775808]]",
  "?=10 $ categorical[10, 10, -9223372036854775808]]",
  "?[10 $ categorical[10, 10, -9223372036854775808]$",
  "?[10 $ categorical[10, 10, -9223372036854775808]$",
  "10 = categorical[10, 10, -9223372036854775808]",
  "?10 = categorical[10, 10, -9223372036854775808]",
  "?[10 = categorical[10, 10, -9223372036854775808]]",
  "$10 = categorical[10, 10, -9223372036854775808]",
  "=10 = categorical[10, 10, -9223372036854775808]",
  "$[10 = categorical[10, 10, -9223372036854775808]]",
  "xxx[10 = categorical[10, 10, -9223372036854775808]]",
  "?$10 = categorical[10, 10, -9223372036854775808]]",
  "?=10 = categorical[10, 10, -9223372036854775808]]",
  "?[10 = categorical[10, 10, -9223372036854775808]$",
  "?[10 = categorical[10, 10, -9223372036854775808]$",
  "$10 * categorical[255]",
  "=10 * categorical[255]",
  "$[10 * categorical[255]]",
  "xxx[10 * categorical[255]]",
  "?$10 * categorical[255]]",
  "?=10 * categorical[255]]",
  "?[10 * categorical[255]$",
  "?[10 * categorical[255]$",
  "$categorical[255]",
  "=categorical[255]",
  "$[categorical[255]]",
  "xxx[categorical[255]]",
  "?$categorical[255]]",
  "?=categorical[255]]",
  "?[categorical[255]$",
  "?[categorical[255]$",
  "10 $ categorical[255]",
  "?10 $ categorical[255]",
  "?[10 $ categorical[255]]",
  "$10 $ categorical[255]",
  "=10 $ categorical[255]",
  "$[10 $ categorical[255]]",
  "xxx[10 $ categorical[255]]",
  "?$10 $ categorical[255]]",
  "?=10 $ categorical[255]]",
  "?[10 $ categorical[255]$",
  "?[10 $ categorical[255]$",
  "10 = categorical[255]",
  "?10 = categorical[255]",
  "?[10 = categorical[255]]",
  "$10 = categorical[255]",
  "=10 = categorical[255]",
  "$[10 = categorical[255]]",
  "xxx[10 = categorical[255]]",
  "?$10 = categorical[255]]",
  "?=10 = categorical[255]]",
  "?[10 = categorical[255]$",
  "?[10 = categorical[255]$",
  "$10 * categorical[10, 255]",
  "=10 * categorical[10, 255]",
  "$[10 * categorical[10, 255]]",
  "xxx[10 * categorical[10, 255]]",
  "?$10 * categorical[10, 255]]",
  "?=10 * categorical[10, 255]]",
  "?[10 * categorical[10, 255]$",
  "?[10 * categorical[10, 255]$",
  "$categorical[10, 255]",
  "=categorical[10, 255]",
  "$[categorical[10, 255]]",
  "xxx[categorical[10, 255]]",
  "?$categorical[10, 255]]",
  "?=categorical[10, 255]]",
  "?[categorical[10, 255]$",
  "?[categorical[10, 255]$",
  "10 $ categorical[10, 255]",
  "?10 $ categorical[10, 255]",
  "?[10 $ categorical[10, 255]]",
  "$10 $ categorical[10, 255]",
  "=10 $ categorical[10, 255]",
  "$[10 $ categorical[10, 255]]",
  "xxx[10 $ categorical[10, 255]]",
  "?$10 $ categorical[10, 255]]",
  "?=10 $ categorical[10, 255]]",
  "?[10 $ categorical[10, 255]$",
  "?[10 $ categorical[10, 255]$",
  "10 = categorical[10, 255]",
  "?10 = categorical[10, 255]",
  "?[10 = categorical[10, 255]]",
  "$10 = categorical[10, 255]",
  "=10 = categorical[10, 255]",
  "$[10 = categorical[10, 255]]",
  "xxx[10 = categorical[10, 255]]",
  "?$10 = categorical[10, 255]]",
  "?=10 = categorical[10, 255]]",
  "?[10 = categorical[10, 255]$",
  "?[10 = categorical[10, 255]$",
  "$10 * categorical[10, 10, 255]",
  "=10 * categorical[10, 10, 255]",
  "$[10 * categorical[10, 10, 255]]",
  "xxx[10 * categorical[10, 10, 255]]",
  "?$10 * categorical[10, 10, 255]]",
  "?=10 * categorical[10, 10, 255]]",
  "?[10 * categorical[10, 10, 255]$",
  "?[10 * categorical[10, 10, 255]$",
  "$categorical[10, 10, 255]",
  "=categorical[10, 10, 255]",
  "$[categorical[10, 10, 255]]",
  "xxx[categorical[10, 10, 255]]",
  "?$categorical[10, 10, 255]]",
  "?=categorical[10, 10, 255]]",
  "?[categorical[10, 10, 255]$",
  "?[categorical[10, 10, 255]$",
  "10 $ categorical[10, 10, 255]",
  "?10 $ categorical[10, 10, 255]",
  "?[10 $ categorical[10, 10, 255]]",
  "$10 $ categorical[10, 10, 255]",
  "=10 $ categorical[10, 10, 255]",
  "$[10 $ categorical[10, 10, 255]]",
  "xxx[10 $ categorical[10, 10, 255]]",
  "?$10 $ categorical[10, 10, 255]]",
  "?=10 $ categorical[10, 10, 255]]",
  "?[10 $ categorical[10, 10, 255]$",
  "?[10 $ categorical[10, 10, 255]$",
  "10 = categorical[10, 10, 255]",
  "?10 = categorical[10, 10, 255]",
  "?[10 = categorical[10, 10, 255]]",
  "$10 = categorical[10, 10, 255]",
  "=10 = categorical[10, 10, 255]",
  "$[10 = categorical[10, 10, 255]]",
  "xxx[10 = categorical[10, 10, 255]]",
  "?$10 = categorical[10, 10, 255]]",
  "?=10 = categorical[10, 10, 255]]",
  "?[10 = categorical[10, 10, 255]$",
  "?[10 = categorical[10, 10, 255]$",
  "$10 * categorical[65535]",
  "=10 * categorical[65535]",
  "$[10 * categorical[65535]]",
  "xxx[10 * categorical[65535]]",
  "?$10 * categorical[65535]]",
  "?=10 * categorical[65535]]",
  "?[10 * categorical[65535]$",
  "?[10 * categorical[65535]$",
  "$categorical[65535]",
  "=categorical[65535]",
  "$[categorical[65535]]",
  "xxx[categorical[65535]]",
  "?$categorical[65535]]",
  "?=categorical[65535]]",
  "?[categorical[65535]$",
  "?[categorical[65535]$",
  "10 $ categorical[65535]",
  "?10 $ categorical[65535]",
  "?[10 $ categorical[65535]]",
  "$10 $ categorical[65535]",
  "=10 $ categorical[65535]",
  "$[10 $ categorical[65535]]",
  "xxx[10 $ categorical[65535]]",
  "?$10 $ categorical[65535]]",
  "?=10 $ categorical[65535]]",
  "?[10 $ categorical[65535]$",
  "?[10 $ categorical[65535]$",
  "10 = categorical[65535]",
  "?10 = categorical[65535]",
  "?[10 = categorical[65535]]",
  "$10 = categorical[65535]",
  "=10 = categorical[65535]",
  "$[10 = categorical[65535]]",
  "xxx[10 = categorical[65535]]",
  "?$10 = categorical[65535]]",
  "?=10 = categorical[65535]]",
  "?[10 = categorical[65535]$",
  "?[10 = categorical[65535]$",
  "$10 * categorical[10, 65535]",
  "=10 * categorical[10, 65535]",
  "$[10 * categorical[10, 65535]]",
  "xxx[10 * categorical[10, 65535]]",
  "?$10 * categorical[10, 65535]]",
  "?=10 * categorical[10, 65535]]",
  "?[10 * categorical[10, 65535]$",
  "?[10 * categorical[10, 65535]$",
  "$categorical[10, 65535]",
  "=categorical[10, 65535]",
  "$[categorical[10, 65535]]",
  "xxx[categorical[10, 65535]]",
  "?$categorical[10, 65535]]",
  "?=categorical[10, 65535]]",
  "?[categorical[10, 65535]$",
  "?[categorical[10, 65535]$",
  "10 $ categorical[10, 65535]",
  "?10 $ categorical[10, 65535]",
  "?[10 $ categorical[10, 65535]]",
  "$10 $ categorical[10, 65535]",
  "=10 $ categorical[10, 65535]",
  "$[10 $ categorical[10, 65535]]",
  "xxx[10 $ categorical[10, 65535]]",
  "?$10 $ categorical[10, 65535]]",
  "?=10 $ categorical[10, 65535]]",
  "?[10 $ categorical[10, 65535]$",
  "?[10 $ categorical[10, 65535]$",
  "10 = categorical[10, 65535]",
  "?10 = categorical[10, 65535]",
  "?[10 = categorical[10, 65535]]",
  "$10 = categorical[10, 65535]",
  "=10 = categorical[10, 65535]",
  "$[10 = categorical[10, 65535]]",
  "xxx[10 = categorical[10, 65535]]",
  "?$10 = categorical[10, 65535]]",
  "?=10 = categorical[10, 65535]]",
  "?[10 = categorical[10, 65535]$",
  "?[10 = categorical[10, 65535]$",
  "$10 * categorical[10, 10, 65535]",
  "=10 * categorical[10, 10, 65535]",
  "$[10 * categorical[10, 10, 65535]]",
  "xxx[10 * categorical[10, 10, 65535]]",
  "?$10 * categorical[10, 10, 65535]]",
  "?=10 * categorical[10, 10, 65535]]",
  "?[10 * categorical[10, 10, 65535]$",
  "?[10 * categorical[10, 10, 65535]$",
  "$categorical[10, 10, 65535]",
  "=categorical[10, 10, 65535]",
  "$[categorical[10, 10, 65535]]",
  "xxx[categorical[10, 10, 65535]]",
  "?$categorical[10, 10, 65535]]",
  "?=categorical[10, 10, 65535]]",
  "?[categorical[10, 10, 65535]$",
  "?[categorical[10, 10, 65535]$",
  "10 $ categorical[10, 10, 65535]",
  "?10 $ categorical[10, 10, 65535]",
  "?[10 $ categorical[10, 10, 65535]]",
  "$10 $ categorical[10, 10, 65535]",
  "=10 $ categorical[10, 10, 65535]",
  "$[10 $ categorical[10, 10, 65535]]",
  "xxx[10 $ categorical[10, 10, 65535]]",
  "?$10 $ categorical[10, 10, 65535]]",
  "?=10 $ categorical[10, 10, 65535]]",
  "?[10 $ categorical[10, 10, 65535]$",
  "?[10 $ categorical[10, 10, 65535]$",
  "10 = categorical[10, 10, 65535]",
  "?10 = categorical[10, 10, 65535]",
  "?[10 = categorical[10, 10, 65535]]",
  "$10 = categorical[10, 10, 65535]",
  "=10 = categorical[10, 10, 65535]",
  "$[10 = categorical[10, 10, 65535]]",
  "xxx[10 = categorical[10, 10, 65535]]",
  "?$10 = categorical[10, 10, 65535]]",
  "?=10 = categorical[10, 10, 65535]]",
  "?[10 = categorical[10, 10, 65535]$",
  "?[10 = categorical[10, 10, 65535]$",
  "$10 * categorical[4294967295]",
  "=10 * categorical[4294967295]",
  "$[10 * categorical[4294967295]]",
  "xxx[10 * categorical[4294967295]]",
  "?$10 * categorical[4294967295]]",
  "?=10 * categorical[4294967295]]",
  "?[10 * categorical[4294967295]$",
  "?[10 * categorical[4294967295]$",
  "$categorical[4294967295]",
  "=categorical[4294967295]",
  "$[categorical[4294967295]]",
  "xxx[categorical[4294967295]]",
  "?$categorical[4294967295]]",
  "?=categorical[4294967295]]",
  "?[categorical[4294967295]$",
  "?[categorical[4294967295]$",
  "10 $ categorical[4294967295]",
  "?10 $ categorical[4294967295]",
  "?[10 $ categorical[4294967295]]",
  "$10 $ categorical[4294967295]",
  "=10 $ categorical[4294967295]",
  "$[10 $ categorical[4294967295]]",
  "xxx[10 $ categorical[4294967295]]",
  "?$10 $ categorical[4294967295]]",
  "?=10 $ categorical[4294967295]]",
  "?[10 $ categorical[4294967295]$",
  "?[10 $ categorical[4294967295]$",
  "10 = categorical[4294967295]",
  "?10 = categorical[4294967295]",
  "?[10 = categorical[4294967295]]",
  "$10 = categorical[4294967295]",
  "=10 = categorical[4294967295]",
  "$[10 = categorical[4294967295]]",
  "xxx[10 = categorical[4294967295]]",
  "?$10 = categorical[4294967295]]",
  "?=10 = categorical[4294967295]]",
  "?[10 = categorical[4294967295]$",
  "?[10 = categorical[4294967295]$",
  "$10 * categorical[10, 4294967295]",
  "=10 * categorical[10, 4294967295]",
  "$[10 * categorical[10, 4294967295]]",
  "xxx[10 * categorical[10, 4294967295]]",
  "?$10 * categorical[10, 4294967295]]",
  "?=10 * categorical[10, 4294967295]]",
  "?[10 * categorical[10, 4294967295]$",
  "?[10 * categorical[10, 4294967295]$",
  "$categorical[10, 4294967295]",
  "=categorical[10, 4294967295]",
  "$[categorical[10, 4294967295]]",
  "xxx[categorical[10, 4294967295]]",
  "?$categorical[10, 4294967295]]",
  "?=categorical[10, 4294967295]]",
  "?[categorical[10, 4294967295]$",
  "?[categorical[10, 4294967295]$",
  "10 $ categorical[10, 4294967295]",
  "?10 $ categorical[10, 4294967295]",
  "?[10 $ categorical[10, 4294967295]]",
  "$10 $ categorical[10, 4294967295]",
  "=10 $ categorical[10, 4294967295]",
  "$[10 $ categorical[10, 4294967295]]",
  "xxx[10 $ categorical[10, 4294967295]]",
  "?$10 $ categorical[10, 4294967295]]",
  "?=10 $ categorical[10, 4294967295]]",
  "?[10 $ categorical[10, 4294967295]$",
  "?[10 $ categorical[10, 4294967295]$",
  "10 = categorical[10, 4294967295]",
  "?10 = categorical[10, 4294967295]",
  "?[10 = categorical[10, 4294967295]]",
  "$10 = categorical[10, 4294967295]",
  "=10 = categorical[10, 4294967295]",
  "$[10 = categorical[10, 4294967295]]",
  "xxx[10 = categorical[10, 4294967295]]",
  "?$10 = categorical[10, 4294967295]]",
  "?=10 = categorical[10, 4294967295]]",
  "?[10 = categorical[10, 4294967295]$",
  "?[10 = categorical[10, 4294967295]$",
  "$10 * categorical[10, 10, 4294967295]",
  "=10 * categorical[10, 10, 4294967295]",
  "$[10 * categorical[10, 10, 4294967295]]",
  "xxx[10 * categorical[10, 10, 4294967295]]",
  "?$10 * categorical[10, 10, 4294967295]]",
  "?=10 * categorical[10, 10, 4294967295]]",
  "?[10 * categorical[10, 10, 4294967295]$",
  "?[10 * categorical[10, 10, 4294967295]$",
  "$categorical[10, 10, 4294967295]",
  "=categorical[10, 10, 4294967295]",
  "$[categorical[10, 10, 4294967295]]",
  "xxx[categorical[10, 10, 4294967295]]",
  "?$categorical[10, 10, 4294967295]]",
  "?=categorical[10, 10, 4294967295]]",
  "?[categorical[10, 10, 4294967295]$",
  "?[categorical[10, 10, 4294967295]$",
  "10 $ categorical[10, 10, 4294967295]",
  "?10 $ categorical[10, 10, 4294967295]",
  "?[10 $ categorical[10, 10, 4294967295]]",
  "$10 $ categorical[10, 10, 4294967295]",
  "=10 $ categorical[10, 10, 4294967295]",
  "$[10 $ categorical[10, 10, 4294967295]]",
  "xxx[10 $ categorical[10, 10, 4294967295]]",
  "?$10 $ categorical[10, 10, 4294967295]]",
  "?=10 $ categorical[10, 10, 4294967295]]",
  "?[10 $ categorical[10, 10, 4294967295]$",
  "?[10 $ categorical[10, 10, 4294967295]$",
  "10 = categorical[10, 10, 4294967295]",
  "?10 = categorical[10, 10, 4294967295]",
  "?[10 = categorical[10, 10, 4294967295]]",
  "$10 = categorical[10, 10, 4294967295]",
  "=10 = categorical[10, 10, 4294967295]",
  "$[10 = categorical[10, 10, 4294967295]]",
  "xxx[10 = categorical[10, 10, 4294967295]]",
  "?$10 = categorical[10, 10, 4294967295]]",
  "?=10 = categorical[10, 10, 4294967295]]",
  "?[10 = categorical[10, 10, 4294967295]$",
  "?[10 = categorical[10, 10, 4294967295]$",
  "$10 * categorical[18446744073709551615]",
  "=10 * categorical[18446744073709551615]",
  "$[10 * categorical[18446744073709551615]]",
  "xxx[10 * categorical[18446744073709551615]]",
  "?$10 * categorical[18446744073709551615]]",
  "?=10 * categorical[18446744073709551615]]",
  "?[10 * categorical[18446744073709551615]$",
  "?[10 * categorical[18446744073709551615]$",
  "$categorical[18446744073709551615]",
  "=categorical[18446744073709551615]",
  "$[categorical[18446744073709551615]]",
  "xxx[categorical[18446744073709551615]]",
  "?$categorical[18446744073709551615]]",
  "?=categorical[18446744073709551615]]",
  "?[categorical[18446744073709551615]$",
  "?[categorical[18446744073709551615]$",
  "10 $ categorical[18446744073709551615]",
  "?10 $ categorical[18446744073709551615]",
  "?[10 $ categorical[18446744073709551615]]",
  "$10 $ categorical[18446744073709551615]",
  "=10 $ categorical[18446744073709551615]",
  "$[10 $ categorical[18446744073709551615]]",
  "xxx[10 $ categorical[18446744073709551615]]",
  "?$10 $ categorical[18446744073709551615]]",
  "?=10 $ categorical[18446744073709551615]]",
  "?[10 $ categorical[18446744073709551615]$",
  "?[10 $ categorical[18446744073709551615]$",
  "10 = categorical[18446744073709551615]",
  "?10 = categorical[18446744073709551615]",
  "?[10 = categorical[18446744073709551615]]",
  "$10 = categorical[18446744073709551615]",
  "=10 = categorical[18446744073709551615]",
  "$[10 = categorical[18446744073709551615]]",
  "xxx[10 = categorical[18446744073709551615]]",
  "?$10 = categorical[18446744073709551615]]",
  "?=10 = categorical[18446744073709551615]]",
  "?[10 = categorical[18446744073709551615]$",
  "?[10 = categorical[18446744073709551615]$",
  "$10 * categorical[10, 18446744073709551615]",
  "=10 * categorical[10, 18446744073709551615]",
  "$[10 * categorical[10, 18446744073709551615]]",
  "xxx[10 * categorical[10, 18446744073709551615]]",
  "?$10 * categorical[10, 18446744073709551615]]",
  "?=10 * categorical[10, 18446744073709551615]]",
  "?[10 * categorical[10, 18446744073709551615]$",
  "?[10 * categorical[10, 18446744073709551615]$",
  "$categorical[10, 18446744073709551615]",
  "=categorical[10, 18446744073709551615]",
  "$[categorical[10, 18446744073709551615]]",
  "xxx[categorical[10, 18446744073709551615]]",
  "?$categorical[10, 18446744073709551615]]",
  "?=categorical[10, 18446744073709551615]]",
  "?[categorical[10, 18446744073709551615]$",
  "?[categorical[10, 18446744073709551615]$",
  "10 $ categorical[10, 18446744073709551615]",
  "?10 $ categorical[10, 18446744073709551615]",
  "?[10 $ categorical[10, 18446744073709551615]]",
  "$10 $ categorical[10, 18446744073709551615]",
  "=10 $ categorical[10, 18446744073709551615]",
  "$[10 $ categorical[10, 18446744073709551615]]",
  "xxx[10 $ categorical[10, 18446744073709551615]]",
  "?$10 $ categorical[10, 18446744073709551615]]",
  "?=10 $ categorical[10, 18446744073709551615]]",
  "?[10 $ categorical[10, 18446744073709551615]$",
  "?[10 $ categorical[10, 18446744073709551615]$",
  "10 = categorical[10, 18446744073709551615]",
  "?10 = categorical[10, 18446744073709551615]",
  "?[10 = categorical[10, 18446744073709551615]]",
  "$10 = categorical[10, 18446744073709551615]",
  "=10 = categorical[10, 18446744073709551615]",
  "$[10 = categorical[10, 18446744073709551615]]",
  "xxx[10 = categorical[10, 18446744073709551615]]",
  "?$10 = categorical[10, 18446744073709551615]]",
  "?=10 = categorical[10, 18446744073709551615]]",
  "?[10 = categorical[10, 18446744073709551615]$",
  "?[10 = categorical[10, 18446744073709551615]$",
  "$10 * categorical[10, 10, 18446744073709551615]",
  "=10 * categorical[10, 10, 18446744073709551615]",
  "$[10 * categorical[10, 10, 18446744073709551615]]",
  "xxx[10 * categorical[10, 10, 18446744073709551615]]",
  "?$10 * categorical[10, 10, 18446744073709551615]]",
  "?=10 * categorical[10, 10, 18446744073709551615]]",
  "?[10 * categorical[10, 10, 18446744073709551615]$",
  "?[10 * categorical[10, 10, 18446744073709551615]$",
  "$categorical[10, 10, 18446744073709551615]",
  "=categorical[10, 10, 18446744073709551615]",
  "$[categorical[10, 10, 18446744073709551615]]",
  "xxx[categorical[10, 10, 18446744073709551615]]",
  "?$categorical[10, 10, 18446744073709551615]]",
  "?=categorical[10, 10, 18446744073709551615]]",
  "?[categorical[10, 10, 18446744073709551615]$",
  "?[categorical[10, 10, 18446744073709551615]$",
  "10 $ categorical[10, 10, 18446744073709551615]",
  "?10 $ categorical[10, 10, 18446744073709551615]",
  "?[10 $ categorical[10, 10, 18446744073709551615]]",
  "$10 $ categorical[10, 10, 18446744073709551615]",
  "=10 $ categorical[10, 10, 18446744073709551615]",
  "$[10 $ categorical[10, 10, 18446744073709551615]]",
  "xxx[10 $ categorical[10, 10, 18446744073709551615]]",
  "?$10 $ categorical[10, 10, 18446744073709551615]]",
  "?=10 $ categorical[10, 10, 18446744073709551615]]",
  "?[10 $ categorical[10, 10, 18446744073709551615]$",
  "?[10 $ categorical[10, 10, 18446744073709551615]$",
  "10 = categorical[10, 10, 18446744073709551615]",
  "?10 = categorical[10, 10, 18446744073709551615]",
  "?[10 = categorical[10, 10, 18446744073709551615]]",
  "$10 = categorical[10, 10, 18446744073709551615]",
  "=10 = categorical[10, 10, 18446744073709551615]",
  "$[10 = categorical[10, 10, 18446744073709551615]]",
  "xxx[10 = categorical[10, 10, 18446744073709551615]]",
  "?$10 = categorical[10, 10, 18446744073709551615]]",
  "?=10 = categorical[10, 10, 18446744073709551615]]",
  "?[10 = categorical[10, 10, 18446744073709551615]$",
  "?[10 = categorical[10, 10, 18446744073709551615]$",
  "$10 * categorical[1.17550e-38]",
  "=10 * categorical[1.17550e-38]",
  "$[10 * categorical[1.17550e-38]]",
  "xxx[10 * categorical[1.17550e-38]]",
  "?$10 * categorical[1.17550e-38]]",
  "?=10 * categorical[1.17550e-38]]",
  "?[10 * categorical[1.17550e-38]$",
  "?[10 * categorical[1.17550e-38]$",
  "$categorical[1.17550e-38]",
  "=categorical[1.17550e-38]",
  "$[categorical[1.17550e-38]]",
  "xxx[categorical[1.17550e-38]]",
  "?$categorical[1.17550e-38]]",
  "?=categorical[1.17550e-38]]",
  "?[categorical[1.17550e-38]$",
  "?[categorical[1.17550e-38]$",
  "10 $ categorical[1.17550e-38]",
  "?10 $ categorical[1.17550e-38]",
  "?[10 $ categorical[1.17550e-38]]",
  "$10 $ categorical[1.17550e-38]",
  "=10 $ categorical[1.17550e-38]",
  "$[10 $ categorical[1.17550e-38]]",
  "xxx[10 $ categorical[1.17550e-38]]",
  "?$10 $ categorical[1.17550e-38]]",
  "?=10 $ categorical[1.17550e-38]]",
  "?[10 $ categorical[1.17550e-38]$",
  "?[10 $ categorical[1.17550e-38]$",
  "10 = categorical[1.17550e-38]",
  "?10 = categorical[1.17550e-38]",
  "?[10 = categorical[1.17550e-38]]",
  "$10 = categorical[1.17550e-38]",
  "=10 = categorical[1.17550e-38]",
  "$[10 = categorical[1.17550e-38]]",
  "xxx[10 = categorical[1.17550e-38]]",
  "?$10 = categorical[1.17550e-38]]",
  "?=10 = categorical[1.17550e-38]]",
  "?[10 = categorical[1.17550e-38]$",
  "?[10 = categorical[1.17550e-38]$",
  "$10 * categorical[10, 1.17550e-38]",
  "=10 * categorical[10, 1.17550e-38]",
  "$[10 * categorical[10, 1.17550e-38]]",
  "xxx[10 * categorical[10, 1.17550e-38]]",
  "?$10 * categorical[10, 1.17550e-38]]",
  "?=10 * categorical[10, 1.17550e-38]]",
  "?[10 * categorical[10, 1.17550e-38]$",
  "?[10 * categorical[10, 1.17550e-38]$",
  "$categorical[10, 1.17550e-38]",
  "=categorical[10, 1.17550e-38]",
  "$[categorical[10, 1.17550e-38]]",
  "xxx[categorical[10, 1.17550e-38]]",
  "?$categorical[10, 1.17550e-38]]",
  "?=categorical[10, 1.17550e-38]]",
  "?[categorical[10, 1.17550e-38]$",
  "?[categorical[10, 1.17550e-38]$",
  "10 $ categorical[10, 1.17550e-38]",
  "?10 $ categorical[10, 1.17550e-38]",
  "?[10 $ categorical[10, 1.17550e-38]]",
  "$10 $ categorical[10, 1.17550e-38]",
  "=10 $ categorical[10, 1.17550e-38]",
  "$[10 $ categorical[10, 1.17550e-38]]",
  "xxx[10 $ categorical[10, 1.17550e-38]]",
  "?$10 $ categorical[10, 1.17550e-38]]",
  "?=10 $ categorical[10, 1.17550e-38]]",
  "?[10 $ categorical[10, 1.17550e-38]$",
  "?[10 $ categorical[10, 1.17550e-38]$",
  "10 = categorical[10, 1.17550e-38]",
  "?10 = categorical[10, 1.17550e-38]",
  "?[10 = categorical[10, 1.17550e-38]]",
  "$10 = categorical[10, 1.17550e-38]",
  "=10 = categorical[10, 1.17550e-38]",
  "$[10 = categorical[10, 1.17550e-38]]",
  "xxx[10 = categorical[10, 1.17550e-38]]",
  "?$10 = categorical[10, 1.17550e-38]]",
  "?=10 = categorical[10, 1.17550e-38]]",
  "?[10 = categorical[10, 1.17550e-38]$",
  "?[10 = categorical[10, 1.17550e-38]$",
  "$10 * categorical[10, 10, 1.17550e-38]",
  "=10 * categorical[10, 10, 1.17550e-38]",
  "$[10 * categorical[10, 10, 1.17550e-38]]",
  "xxx[10 * categorical[10, 10, 1.17550e-38]]",
  "?$10 * categorical[10, 10, 1.17550e-38]]",
  "?=10 * categorical[10, 10, 1.17550e-38]]",
  "?[10 * categorical[10, 10, 1.17550e-38]$",
  "?[10 * categorical[10, 10, 1.17550e-38]$",
  "$categorical[10, 10, 1.17550e-38]",
  "=categorical[10, 10, 1.17550e-38]",
  "$[categorical[10, 10, 1.17550e-38]]",
  "xxx[categorical[10, 10, 1.17550e-38]]",
  "?$categorical[10, 10, 1.17550e-38]]",
  "?=categorical[10, 10, 1.17550e-38]]",
  "?[categorical[10, 10, 1.17550e-38]$",
  "?[categorical[10, 10, 1.17550e-38]$",
  "10 $ categorical[10, 10, 1.17550e-38]",
  "?10 $ categorical[10, 10, 1.17550e-38]",
  "?[10 $ categorical[10, 10, 1.17550e-38]]",
  "$10 $ categorical[10, 10, 1.17550e-38]",
  "=10 $ categorical[10, 10, 1.17550e-38]",
  "$[10 $ categorical[10, 10, 1.17550e-38]]",
  "xxx[10 $ categorical[10, 10, 1.17550e-38]]",
  "?$10 $ categorical[10, 10, 1.17550e-38]]",
  "?=10 $ categorical[10, 10, 1.17550e-38]]",
  "?[10 $ categorical[10, 10, 1.17550e-38]$",
  "?[10 $ categorical[10, 10, 1.17550e-38]$",
  "10 = categorical[10, 10, 1.17550e-38]",
  "?10 = categorical[10, 10, 1.17550e-38]",
  "?[10 = categorical[10, 10, 1.17550e-38]]",
  "$10 = categorical[10, 10, 1.17550e-38]",
  "=10 = categorical[10, 10, 1.17550e-38]",
  "$[10 = categorical[10, 10, 1.17550e-38]]",
  "xxx[10 = categorical[10, 10, 1.17550e-38]]",
  "?$10 = categorical[10, 10, 1.17550e-38]]",
  "?=10 = categorical[10, 10, 1.17550e-38]]",
  "?[10 = categorical[10, 10, 1.17550e-38]$",
  "?[10 = categorical[10, 10, 1.17550e-38]$",
  "$10 * categorical[2.22508e-308]",
  "=10 * categorical[2.22508e-308]",
  "$[10 * categorical[2.22508e-308]]",
  "xxx[10 * categorical[2.22508e-308]]",
  "?$10 * categorical[2.22508e-308]]",
  "?=10 * categorical[2.22508e-308]]",
  "?[10 * categorical[2.22508e-308]$",
  "?[10 * categorical[2.22508e-308]$",
  "$categorical[2.22508e-308]",
  "=categorical[2.22508e-308]",
  "$[categorical[2.22508e-308]]",
  "xxx[categorical[2.22508e-308]]",
  "?$categorical[2.22508e-308]]",
  "?=categorical[2.22508e-308]]",
  "?[categorical[2.22508e-308]$",
  "?[categorical[2.22508e-308]$",
  "10 $ categorical[2.22508e-308]",
  "?10 $ categorical[2.22508e-308]",
  "?[10 $ categorical[2.22508e-308]]",
  "$10 $ categorical[2.22508e-308]",
  "=10 $ categorical[2.22508e-308]",
  "$[10 $ categorical[2.22508e-308]]",
  "xxx[10 $ categorical[2.22508e-308]]",
  "?$10 $ categorical[2.22508e-308]]",
  "?=10 $ categorical[2.22508e-308]]",
  "?[10 $ categorical[2.22508e-308]$",
  "?[10 $ categorical[2.22508e-308]$",
  "10 = categorical[2.22508e-308]",
  "?10 = categorical[2.22508e-308]",
  "?[10 = categorical[2.22508e-308]]",
  "$10 = categorical[2.22508e-308]",
  "=10 = categorical[2.22508e-308]",
  "$[10 = categorical[2.22508e-308]]",
  "xxx[10 = categorical[2.22508e-308]]",
  "?$10 = categorical[2.22508e-308]]",
  "?=10 = categorical[2.22508e-308]]",
  "?[10 = categorical[2.22508e-308]$",
  "?[10 = categorical[2.22508e-308]$",
  "$10 * categorical[10, 2.22508e-308]",
  "=10 * categorical[10, 2.22508e-308]",
  "$[10 * categorical[10, 2.22508e-308]]",
  "xxx[10 * categorical[10, 2.22508e-308]]",
  "?$10 * categorical[10, 2.22508e-308]]",
  "?=10 * categorical[10, 2.22508e-308]]",
  "?[10 * categorical[10, 2.22508e-308]$",
  "?[10 * categorical[10, 2.22508e-308]$",
  "$categorical[10, 2.22508e-308]",
  "=categorical[10, 2.22508e-308]",
  "$[categorical[10, 2.22508e-308]]",
  "xxx[categorical[10, 2.22508e-308]]",
  "?$categorical[10, 2.22508e-308]]",
  "?=categorical[10, 2.22508e-308]]",
  "?[categorical[10, 2.22508e-308]$",
  "?[categorical[10, 2.22508e-308]$",
  "10 $ categorical[10, 2.22508e-308]",
  "?10 $ categorical[10, 2.22508e-308]",
  "?[10 $ categorical[10, 2.22508e-308]]",
  "$10 $ categorical[10, 2.22508e-308]",
  "=10 $ categorical[10, 2.22508e-308]",
  "$[10 $ categorical[10, 2.22508e-308]]",
  "xxx[10 $ categorical[10, 2.22508e-308]]",
  "?$10 $ categorical[10, 2.22508e-308]]",
  "?=10 $ categorical[10, 2.22508e-308]]",
  "?[10 $ categorical[10, 2.22508e-308]$",
  "?[10 $ categorical[10, 2.22508e-308]$",
  "10 = categorical[10, 2.22508e-308]",
  "?10 = categorical[10, 2.22508e-308]",
  "?[10 = categorical[10, 2.22508e-308]]",
  "$10 = categorical[10, 2.22508e-308]",
  "=10 = categorical[10, 2.22508e-308]",
  "$[10 = categorical[10, 2.22508e-308]]",
  "xxx[10 = categorical[10, 2.22508e-308]]",
  "?$10 = categorical[10, 2.22508e-308]]",
  "?=10 = categorical[10, 2.22508e-308]]",
  "?[10 = categorical[10, 2.22508e-308]$",
  "?[10 = categorical[10, 2.22508e-308]$",
  "$10 * categorical[10, 10, 2.22508e-308]",
  "=10 * categorical[10, 10, 2.22508e-308]",
  "$[10 * categorical[10, 10, 2.22508e-308]]",
  "xxx[10 * categorical[10, 10, 2.22508e-308]]",
  "?$10 * categorical[10, 10, 2.22508e-308]]",
  "?=10 * categorical[10, 10, 2.22508e-308]]",
  "?[10 * categorical[10, 10, 2.22508e-308]$",
  "?[10 * categorical[10, 10, 2.22508e-308]$",
  "$categorical[10, 10, 2.22508e-308]",
  "=categorical[10, 10, 2.22508e-308]",
  "$[categorical[10, 10, 2.22508e-308]]",
  "xxx[categorical[10, 10, 2.22508e-308]]",
  "?$categorical[10, 10, 2.22508e-308]]",
  "?=categorical[10, 10, 2.22508e-308]]",
  "?[categorical[10, 10, 2.22508e-308]$",
  "?[categorical[10, 10, 2.22508e-308]$",
  "10 $ categorical[10, 10, 2.22508e-308]",
  "?10 $ categorical[10, 10, 2.22508e-308]",
  "?[10 $ categorical[10, 10, 2.22508e-308]]",
  "$10 $ categorical[10, 10, 2.22508e-308]",
  "=10 $ categorical[10, 10, 2.22508e-308]",
  "$[10 $ categorical[10, 10, 2.22508e-308]]",
  "xxx[10 $ categorical[10, 10, 2.22508e-308]]",
  "?$10 $ categorical[10, 10, 2.22508e-308]]",
  "?=10 $ categorical[10, 10, 2.22508e-308]]",
  "?[10 $ categorical[10, 10, 2.22508e-308]$",
  "?[10 $ categorical[10, 10, 2.22508e-308]$",
  "10 = categorical[10, 10, 2.22508e-308]",
  "?10 = categorical[10, 10, 2.22508e-308]",
  "?[10 = categorical[10, 10, 2.22508e-308]]",
  "$10 = categorical[10, 10, 2.22508e-308]",
  "=10 = categorical[10, 10, 2.22508e-308]",
  "$[10 = categorical[10, 10, 2.22508e-308]]",
  "xxx[10 = categorical[10, 10, 2.22508e-308]]",
  "?$10 = categorical[10, 10, 2.22508e-308]]",
  "?=10 = categorical[10, 10, 2.22508e-308]]",
  "?[10 = categorical[10, 10, 2.22508e-308]$",
  "?[10 = categorical[10, 10, 2.22508e-308]$",
  "$10 * categorical[3.40282e+38]",
  "=10 * categorical[3.40282e+38]",
  "$[10 * categorical[3.40282e+38]]",
  "xxx[10 * categorical[3.40282e+38]]",
  "?$10 * categorical[3.40282e+38]]",
  "?=10 * categorical[3.40282e+38]]",
  "?[10 * categorical[3.40282e+38]$",
  "?[10 * categorical[3.40282e+38]$",
  "$categorical[3.40282e+38]",
  "=categorical[3.40282e+38]",
  "$[categorical[3.40282e+38]]",
  "xxx[categorical[3.40282e+38]]",
  "?$categorical[3.40282e+38]]",
  "?=categorical[3.40282e+38]]",
  "?[categorical[3.40282e+38]$",
  "?[categorical[3.40282e+38]$",
  "10 $ categorical[3.40282e+38]",
  "?10 $ categorical[3.40282e+38]",
  "?[10 $ categorical[3.40282e+38]]",
  "$10 $ categorical[3.40282e+38]",
  "=10 $ categorical[3.40282e+38]",
  "$[10 $ categorical[3.40282e+38]]",
  "xxx[10 $ categorical[3.40282e+38]]",
  "?$10 $ categorical[3.40282e+38]]",
  "?=10 $ categorical[3.40282e+38]]",
  "?[10 $ categorical[3.40282e+38]$",
  "?[10 $ categorical[3.40282e+38]$",
  "10 = categorical[3.40282e+38]",
  "?10 = categorical[3.40282e+38]",
  "?[10 = categorical[3.40282e+38]]",
  "$10 = categorical[3.40282e+38]",
  "=10 = categorical[3.40282e+38]",
  "$[10 = categorical[3.40282e+38]]",
  "xxx[10 = categorical[3.40282e+38]]",
  "?$10 = categorical[3.40282e+38]]",
  "?=10 = categorical[3.40282e+38]]",
  "?[10 = categorical[3.40282e+38]$",
  "?[10 = categorical[3.40282e+38]$",
  "$10 * categorical[10, 3.40282e+38]",
  "=10 * categorical[10, 3.40282e+38]",
  "$[10 * categorical[10, 3.40282e+38]]",
  "xxx[10 * categorical[10, 3.40282e+38]]",
  "?$10 * categorical[10, 3.40282e+38]]",
  "?=10 * categorical[10, 3.40282e+38]]",
  "?[10 * categorical[10, 3.40282e+38]$",
  "?[10 * categorical[10, 3.40282e+38]$",
  "$categorical[10, 3.40282e+38]",
  "=categorical[10, 3.40282e+38]",
  "$[categorical[10, 3.40282e+38]]",
  "xxx[categorical[10, 3.40282e+38]]",
  "?$categorical[10, 3.40282e+38]]",
  "?=categorical[10, 3.40282e+38]]",
  "?[categorical[10, 3.40282e+38]$",
  "?[categorical[10, 3.40282e+38]$",
  "10 $ categorical[10, 3.40282e+38]",
  "?10 $ categorical[10, 3.40282e+38]",
  "?[10 $ categorical[10, 3.40282e+38]]",
  "$10 $ categorical[10, 3.40282e+38]",
  "=10 $ categorical[10, 3.40282e+38]",
  "$[10 $ categorical[10, 3.40282e+38]]",
  "xxx[10 $ categorical[10, 3.40282e+38]]",
  "?$10 $ categorical[10, 3.40282e+38]]",
  "?=10 $ categorical[10, 3.40282e+38]]",
  "?[10 $ categorical[10, 3.40282e+38]$",
  "?[10 $ categorical[10, 3.40282e+38]$",
  "10 = categorical[10, 3.40282e+38]",
  "?10 = categorical[10, 3.40282e+38]",
  "?[10 = categorical[10, 3.40282e+38]]",
  "$10 = categorical[10, 3.40282e+38]",
  "=10 = categorical[10, 3.40282e+38]",
  "$[10 = categorical[10, 3.40282e+38]]",
  "xxx[10 = categorical[10, 3.40282e+38]]",
  "?$10 = categorical[10, 3.40282e+38]]",
  "?=10 = categorical[10, 3.40282e+38]]",
  "?[10 = categorical[10, 3.40282e+38]$",
  "?[10 = categorical[10, 3.40282e+38]$",
  "$10 * categorical[10, 10, 3.40282e+38]",
  "=10 * categorical[10, 10, 3.40282e+38]",
  "$[10 * categorical[10, 10, 3.40282e+38]]",
  "xxx[10 * categorical[10, 10, 3.40282e+38]]",
  "?$10 * categorical[10, 10, 3.40282e+38]]",
  "?=10 * categorical[10, 10, 3.40282e+38]]",
  "?[10 * categorical[10, 10, 3.40282e+38]$",
  "?[10 * categorical[10, 10, 3.40282e+38]$",
  "$categorical[10, 10, 3.40282e+38]",
  "=categorical[10, 10, 3.40282e+38]",
  "$[categorical[10, 10, 3.40282e+38]]",
  "xxx[categorical[10, 10, 3.40282e+38]]",
  "?$categorical[10, 10, 3.40282e+38]]",
  "?=categorical[10, 10, 3.40282e+38]]",
  "?[categorical[10, 10, 3.40282e+38]$",
  "?[categorical[10, 10, 3.40282e+38]$",
  "10 $ categorical[10, 10, 3.40282e+38]",
  "?10 $ categorical[10, 10, 3.40282e+38]",
  "?[10 $ categorical[10, 10, 3.40282e+38]]",
  "$10 $ categorical[10, 10, 3.40282e+38]",
  "=10 $ categorical[10, 10, 3.40282e+38]",
  "$[10 $ categorical[10, 10, 3.40282e+38]]",
  "xxx[10 $ categorical[10, 10, 3.40282e+38]]",
  "?$10 $ categorical[10, 10, 3.40282e+38]]",
  "?=10 $ categorical[10, 10, 3.40282e+38]]",
  "?[10 $ categorical[10, 10, 3.40282e+38]$",
  "?[10 $ categorical[10, 10, 3.40282e+38]$",
  "10 = categorical[10, 10, 3.40282e+38]",
  "?10 = categorical[10, 10, 3.40282e+38]",
  "?[10 = categorical[10, 10, 3.40282e+38]]",
  "$10 = categorical[10, 10, 3.40282e+38]",
  "=10 = categorical[10, 10, 3.40282e+38]",
  "$[10 = categorical[10, 10, 3.40282e+38]]",
  "xxx[10 = categorical[10, 10, 3.40282e+38]]",
  "?$10 = categorical[10, 10, 3.40282e+38]]",
  "?=10 = categorical[10, 10, 3.40282e+38]]",
  "?[10 = categorical[10, 10, 3.40282e+38]$",
  "?[10 = categorical[10, 10, 3.40282e+38]$",
  "$10 * categorical[1.79769e+308]",
  "=10 * categorical[1.79769e+308]",
  "$[10 * categorical[1.79769e+308]]",
  "xxx[10 * categorical[1.79769e+308]]",
  "?$10 * categorical[1.79769e+308]]",
  "?=10 * categorical[1.79769e+308]]",
  "?[10 * categorical[1.79769e+308]$",
  "?[10 * categorical[1.79769e+308]$",
  "$categorical[1.79769e+308]",
  "=categorical[1.79769e+308]",
  "$[categorical[1.79769e+308]]",
  "xxx[categorical[1.79769e+308]]",
  "?$categorical[1.79769e+308]]",
  "?=categorical[1.79769e+308]]",
  "?[categorical[1.79769e+308]$",
  "?[categorical[1.79769e+308]$",
  "10 $ categorical[1.79769e+308]",
  "?10 $ categorical[1.79769e+308]",
  "?[10 $ categorical[1.79769e+308]]",
  "$10 $ categorical[1.79769e+308]",
  "=10 $ categorical[1.79769e+308]",
  "$[10 $ categorical[1.79769e+308]]",
  "xxx[10 $ categorical[1.79769e+308]]",
  "?$10 $ categorical[1.79769e+308]]",
  "?=10 $ categorical[1.79769e+308]]",
  "?[10 $ categorical[1.79769e+308]$",
  "?[10 $ categorical[1.79769e+308]$",
  "10 = categorical[1.79769e+308]",
  "?10 = categorical[1.79769e+308]",
  "?[10 = categorical[1.79769e+308]]",
  "$10 = categorical[1.79769e+308]",
  "=10 = categorical[1.79769e+308]",
  "$[10 = categorical[1.79769e+308]]",
  "xxx[10 = categorical[1.79769e+308]]",
  "?$10 = categorical[1.79769e+308]]",
  "?=10 = categorical[1.79769e+308]]",
  "?[10 = categorical[1.79769e+308]$",
  "?[10 = categorical[1.79769e+308]$",
  "$10 * categorical[10, 1.79769e+308]",
  "=10 * categorical[10, 1.79769e+308]",
  "$[10 * categorical[10, 1.79769e+308]]",
  "xxx[10 * categorical[10, 1.79769e+308]]",
  "?$10 * categorical[10, 1.79769e+308]]",
  "?=10 * categorical[10, 1.79769e+308]]",
  "?[10 * categorical[10, 1.79769e+308]$",
  "?[10 * categorical[10, 1.79769e+308]$",
  "$categorical[10, 1.79769e+308]",
  "=categorical[10, 1.79769e+308]",
  "$[categorical[10, 1.79769e+308]]",
  "xxx[categorical[10, 1.79769e+308]]",
  "?$categorical[10, 1.79769e+308]]",
  "?=categorical[10, 1.79769e+308]]",
  "?[categorical[10, 1.79769e+308]$",
  "?[categorical[10, 1.79769e+308]$",
  "10 $ categorical[10, 1.79769e+308]",
  "?10 $ categorical[10, 1.79769e+308]",
  "?[10 $ categorical[10, 1.79769e+308]]",
  "$10 $ categorical[10, 1.79769e+308]",
  "=10 $ categorical[10, 1.79769e+308]",
  "$[10 $ categorical[10, 1.79769e+308]]",
  "xxx[10 $ categorical[10, 1.79769e+308]]",
  "?$10 $ categorical[10, 1.79769e+308]]",
  "?=10 $ categorical[10, 1.79769e+308]]",
  "?[10 $ categorical[10, 1.79769e+308]$",
  "?[10 $ categorical[10, 1.79769e+308]$",
  "10 = categorical[10, 1.79769e+308]",
  "?10 = categorical[10, 1.79769e+308]",
  "?[10 = categorical[10, 1.79769e+308]]",
  "$10 = categorical[10, 1.79769e+308]",
  "=10 = categorical[10, 1.79769e+308]",
  "$[10 = categorical[10, 1.79769e+308]]",
  "xxx[10 = categorical[10, 1.79769e+308]]",
  "?$10 = categorical[10, 1.79769e+308]]",
  "?=10 = categorical[10, 1.79769e+308]]",
  "?[10 = categorical[10, 1.79769e+308]$",
  "?[10 = categorical[10, 1.79769e+308]$",
  "$10 * categorical[10, 10, 1.79769e+308]",
  "=10 * categorical[10, 10, 1.79769e+308]",
  "$[10 * categorical[10, 10, 1.79769e+308]]",
  "xxx[10 * categorical[10, 10, 1.79769e+308]]",
  "?$10 * categorical[10, 10, 1.79769e+308]]",
  "?=10 * categorical[10, 10, 1.79769e+308]]",
  "?[10 * categorical[10, 10, 1.79769e+308]$",
  "?[10 * categorical[10, 10, 1.79769e+308]$",
  "$categorical[10, 10, 1.79769e+308]",
  "=categorical[10, 10, 1.79769e+308]",
  "$[categorical[10, 10, 1.79769e+308]]",
  "xxx[categorical[10, 10, 1.79769e+308]]",
  "?$categorical[10, 10, 1.79769e+308]]",
  "?=categorical[10, 10, 1.79769e+308]]",
  "?[categorical[10, 10, 1.79769e+308]$",
  "?[categorical[10, 10, 1.79769e+308]$",
  "10 $ categorical[10, 10, 1.79769e+308]",
  "?10 $ categorical[10, 10, 1.79769e+308]",
  "?[10 $ categorical[10, 10, 1.79769e+308]]",
  "$10 $ categorical[10, 10, 1.79769e+308]",
  "=10 $ categorical[10, 10, 1.79769e+308]",
  "$[10 $ categorical[10, 10, 1.79769e+308]]",
  "xxx[10 $ categorical[10, 10, 1.79769e+308]]",
  "?$10 $ categorical[10, 10, 1.79769e+308]]",
  "?=10 $ categorical[10, 10, 1.79769e+308]]",
  "?[10 $ categorical[10, 10, 1.79769e+308]$",
  "?[10 $ categorical[10, 10, 1.79769e+308]$",
  "10 = categorical[10, 10, 1.79769e+308]",
  "?10 = categorical[10, 10, 1.79769e+308]",
  "?[10 = categorical[10, 10, 1.79769e+308]]",
  "$10 = categorical[10, 10, 1.79769e+308]",
  "=10 = categorical[10, 10, 1.79769e+308]",
  "$[10 = categorical[10, 10, 1.79769e+308]]",
  "xxx[10 = categorical[10, 10, 1.79769e+308]]",
  "?$10 = categorical[10, 10, 1.79769e+308]]",
  "?=10 = categorical[10, 10, 1.79769e+308]]",
  "?[10 = categorical[10, 10, 1.79769e+308]$",
  "?[10 = categorical[10, 10, 1.79769e+308]$",
  "$10 * categorical[\"\"]",
  "=10 * categorical[\"\"]",
  "$[10 * categorical[\"\"]]",
  "xxx[10 * categorical[\"\"]]",
  "?$10 * categorical[\"\"]]",
  "?=10 * categorical[\"\"]]",
  "?[10 * categorical[\"\"]$",
  "?[10 * categorical[\"\"]$",
  "$categorical[\"\"]",
  "=categorical[\"\"]",
  "$[categorical[\"\"]]",
  "xxx[categorical[\"\"]]",
  "?$categorical[\"\"]]",
  "?=categorical[\"\"]]",
  "?[categorical[\"\"]$",
  "?[categorical[\"\"]$",
  "10 $ categorical[\"\"]",
  "?10 $ categorical[\"\"]",
  "?[10 $ categorical[\"\"]]",
  "$10 $ categorical[\"\"]",
  "=10 $ categorical[\"\"]",
  "$[10 $ categorical[\"\"]]",
  "xxx[10 $ categorical[\"\"]]",
  "?$10 $ categorical[\"\"]]",
  "?=10 $ categorical[\"\"]]",
  "?[10 $ categorical[\"\"]$",
  "?[10 $ categorical[\"\"]$",
  "10 = categorical[\"\"]",
  "?10 = categorical[\"\"]",
  "?[10 = categorical[\"\"]]",
  "$10 = categorical[\"\"]",
  "=10 = categorical[\"\"]",
  "$[10 = categorical[\"\"]]",
  "xxx[10 = categorical[\"\"]]",
  "?$10 = categorical[\"\"]]",
  "?=10 = categorical[\"\"]]",
  "?[10 = categorical[\"\"]$",
  "?[10 = categorical[\"\"]$",
  "$10 * categorical[10, \"\"]",
  "=10 * categorical[10, \"\"]",
  "$[10 * categorical[10, \"\"]]",
  "xxx[10 * categorical[10, \"\"]]",
  "?$10 * categorical[10, \"\"]]",
  "?=10 * categorical[10, \"\"]]",
  "?[10 * categorical[10, \"\"]$",
  "?[10 * categorical[10, \"\"]$",
  "$categorical[10, \"\"]",
  "=categorical[10, \"\"]",
  "$[categorical[10, \"\"]]",
  "xxx[categorical[10, \"\"]]",
  "?$categorical[10, \"\"]]",
  "?=categorical[10, \"\"]]",
  "?[categorical[10, \"\"]$",
  "?[categorical[10, \"\"]$",
  "10 $ categorical[10, \"\"]",
  "?10 $ categorical[10, \"\"]",
  "?[10 $ categorical[10, \"\"]]",
  "$10 $ categorical[10, \"\"]",
  "=10 $ categorical[10, \"\"]",
  "$[10 $ categorical[10, \"\"]]",
  "xxx[10 $ categorical[10, \"\"]]",
  "?$10 $ categorical[10, \"\"]]",
  "?=10 $ categorical[10, \"\"]]",
  "?[10 $ categorical[10, \"\"]$",
  "?[10 $ categorical[10, \"\"]$",
  "10 = categorical[10, \"\"]",
  "?10 = categorical[10, \"\"]",
  "?[10 = categorical[10, \"\"]]",
  "$10 = categorical[10, \"\"]",
  "=10 = categorical[10, \"\"]",
  "$[10 = categorical[10, \"\"]]",
  "xxx[10 = categorical[10, \"\"]]",
  "?$10 = categorical[10, \"\"]]",
  "?=10 = categorical[10, \"\"]]",
  "?[10 = categorical[10, \"\"]$",
  "?[10 = categorical[10, \"\"]$",
  "$10 * categorical[10, 10, \"\"]",
  "=10 * categorical[10, 10, \"\"]",
  "$[10 * categorical[10, 10, \"\"]]",
  "xxx[10 * categorical[10, 10, \"\"]]",
  "?$10 * categorical[10, 10, \"\"]]",
  "?=10 * categorical[10, 10, \"\"]]",
  "?[10 * categorical[10, 10, \"\"]$",
  "?[10 * categorical[10, 10, \"\"]$",
  "$categorical[10, 10, \"\"]",
  "=categorical[10, 10, \"\"]",
  "$[categorical[10, 10, \"\"]]",
  "xxx[categorical[10, 10, \"\"]]",
  "?$categorical[10, 10, \"\"]]",
  "?=categorical[10, 10, \"\"]]",
  "?[categorical[10, 10, \"\"]$",
  "?[categorical[10, 10, \"\"]$",
  "10 $ categorical[10, 10, \"\"]",
  "?10 $ categorical[10, 10, \"\"]",
  "?[10 $ categorical[10, 10, \"\"]]",
  "$10 $ categorical[10, 10, \"\"]",
  "=10 $ categorical[10, 10, \"\"]",
  "$[10 $ categorical[10, 10, \"\"]]",
  "xxx[10 $ categorical[10, 10, \"\"]]",
  "?$10 $ categorical[10, 10, \"\"]]",
  "?=10 $ categorical[10, 10, \"\"]]",
  "?[10 $ categorical[10, 10, \"\"]$",
  "?[10 $ categorical[10, 10, \"\"]$",
  "10 = categorical[10, 10, \"\"]",
  "?10 = categorical[10, 10, \"\"]",
  "?[10 = categorical[10, 10, \"\"]]",
  "$10 = categorical[10, 10, \"\"]",
  "=10 = categorical[10, 10, \"\"]",
  "$[10 = categorical[10, 10, \"\"]]",
  "xxx[10 = categorical[10, 10, \"\"]]",
  "?$10 = categorical[10, 10, \"\"]]",
  "?=10 = categorical[10, 10, \"\"]]",
  "?[10 = categorical[10, 10, \"\"]$",
  "?[10 = categorical[10, 10, \"\"]$",
  "$10 * categorical['']",
  "=10 * categorical['']",
  "$[10 * categorical['']]",
  "xxx[10 * categorical['']]",
  "?$10 * categorical['']]",
  "?=10 * categorical['']]",
  "?[10 * categorical['']$",
  "?[10 * categorical['']$",
  "$categorical['']",
  "=categorical['']",
  "$[categorical['']]",
  "xxx[categorical['']]",
  "?$categorical['']]",
  "?=categorical['']]",
  "?[categorical['']$",
  "?[categorical['']$",
  "10 $ categorical['']",
  "?10 $ categorical['']",
  "?[10 $ categorical['']]",
  "$10 $ categorical['']",
  "=10 $ categorical['']",
  "$[10 $ categorical['']]",
  "xxx[10 $ categorical['']]",
  "?$10 $ categorical['']]",
  "?=10 $ categorical['']]",
  "?[10 $ categorical['']$",
  "?[10 $ categorical['']$",
  "10 = categorical['']",
  "?10 = categorical['']",
  "?[10 = categorical['']]",
  "$10 = categorical['']",
  "=10 = categorical['']",
  "$[10 = categorical['']]",
  "xxx[10 = categorical['']]",
  "?$10 = categorical['']]",
  "?=10 = categorical['']]",
  "?[10 = categorical['']$",
  "?[10 = categorical['']$",
  "$10 * categorical[10, '']",
  "=10 * categorical[10, '']",
  "$[10 * categorical[10, '']]",
  "xxx[10 * categorical[10, '']]",
  "?$10 * categorical[10, '']]",
  "?=10 * categorical[10, '']]",
  "?[10 * categorical[10, '']$",
  "?[10 * categorical[10, '']$",
  "$categorical[10, '']",
  "=categorical[10, '']",
  "$[categorical[10, '']]",
  "xxx[categorical[10, '']]",
  "?$categorical[10, '']]",
  "?=categorical[10, '']]",
  "?[categorical[10, '']$",
  "?[categorical[10, '']$",
  "10 $ categorical[10, '']",
  "?10 $ categorical[10, '']",
  "?[10 $ categorical[10, '']]",
  "$10 $ categorical[10, '']",
  "=10 $ categorical[10, '']",
  "$[10 $ categorical[10, '']]",
  "xxx[10 $ categorical[10, '']]",
  "?$10 $ categorical[10, '']]",
  "?=10 $ categorical[10, '']]",
  "?[10 $ categorical[10, '']$",
  "?[10 $ categorical[10, '']$",
  "10 = categorical[10, '']",
  "?10 = categorical[10, '']",
  "?[10 = categorical[10, '']]",
  "$10 = categorical[10, '']",
  "=10 = categorical[10, '']",
  "$[10 = categorical[10, '']]",
  "xxx[10 = categorical[10, '']]",
  "?$10 = categorical[10, '']]",
  "?=10 = categorical[10, '']]",
  "?[10 = categorical[10, '']$",
  "?[10 = categorical[10, '']$",
  "$10 * categorical[10, 10, '']",
  "=10 * categorical[10, 10, '']",
  "$[10 * categorical[10, 10, '']]",
  "xxx[10 * categorical[10, 10, '']]",
  "?$10 * categorical[10, 10, '']]",
  "?=10 * categorical[10, 10, '']]",
  "?[10 * categorical[10, 10, '']$",
  "?[10 * categorical[10, 10, '']$",
  "$categorical[10, 10, '']",
  "=categorical[10, 10, '']",
  "$[categorical[10, 10, '']]",
  "xxx[categorical[10, 10, '']]",
  "?$categorical[10, 10, '']]",
  "?=categorical[10, 10, '']]",
  "?[categorical[10, 10, '']$",
  "?[categorical[10, 10, '']$",
  "10 $ categorical[10, 10, '']",
  "?10 $ categorical[10, 10, '']",
  "?[10 $ categorical[10, 10, '']]",
  "$10 $ categorical[10, 10, '']",
  "=10 $ categorical[10, 10, '']",
  "$[10 $ categorical[10, 10, '']]",
  "xxx[10 $ categorical[10, 10, '']]",
  "?$10 $ categorical[10, 10, '']]",
  "?=10 $ categorical[10, 10, '']]",
  "?[10 $ categorical[10, 10, '']$",
  "?[10 $ categorical[10, 10, '']$",
  "10 = categorical[10, 10, '']",
  "?10 = categorical[10, 10, '']",
  "?[10 = categorical[10, 10, '']]",
  "$10 = categorical[10, 10, '']",
  "=10 = categorical[10, 10, '']",
  "$[10 = categorical[10, 10, '']]",
  "xxx[10 = categorical[10, 10, '']]",
  "?$10 = categorical[10, 10, '']]",
  "?=10 = categorical[10, 10, '']]",
  "?[10 = categorical[10, 10, '']$",
  "?[10 = categorical[10, 10, '']$",
  "$10 * categorical[\"xyz\"]",
  "=10 * categorical[\"xyz\"]",
  "$[10 * categorical[\"xyz\"]]",
  "xxx[10 * categorical[\"xyz\"]]",
  "?$10 * categorical[\"xyz\"]]",
  "?=10 * categorical[\"xyz\"]]",
  "?[10 * categorical[\"xyz\"]$",
  "?[10 * categorical[\"xyz\"]$",
  "$categorical[\"xyz\"]",
  "=categorical[\"xyz\"]",
  "$[categorical[\"xyz\"]]",
  "xxx[categorical[\"xyz\"]]",
  "?$categorical[\"xyz\"]]",
  "?=categorical[\"xyz\"]]",
  "?[categorical[\"xyz\"]$",
  "?[categorical[\"xyz\"]$",
  "10 $ categorical[\"xyz\"]",
  "?10 $ categorical[\"xyz\"]",
  "?[10 $ categorical[\"xyz\"]]",
  "$10 $ categorical[\"xyz\"]",
  "=10 $ categorical[\"xyz\"]",
  "$[10 $ categorical[\"xyz\"]]",
  "xxx[10 $ categorical[\"xyz\"]]",
  "?$10 $ categorical[\"xyz\"]]",
  "?=10 $ categorical[\"xyz\"]]",
  "?[10 $ categorical[\"xyz\"]$",
  "?[10 $ categorical[\"xyz\"]$",
  "10 = categorical[\"xyz\"]",
  "?10 = categorical[\"xyz\"]",
  "?[10 = categorical[\"xyz\"]]",
  "$10 = categorical[\"xyz\"]",
  "=10 = categorical[\"xyz\"]",
  "$[10 = categorical[\"xyz\"]]",
  "xxx[10 = categorical[\"xyz\"]]",
  "?$10 = categorical[\"xyz\"]]",
  "?=10 = categorical[\"xyz\"]]",
  "?[10 = categorical[\"xyz\"]$",
  "?[10 = categorical[\"xyz\"]$",
  "$10 * categorical[10, \"xyz\"]",
  "=10 * categorical[10, \"xyz\"]",
  "$[10 * categorical[10, \"xyz\"]]",
  "xxx[10 * categorical[10, \"xyz\"]]",
  "?$10 * categorical[10, \"xyz\"]]",
  "?=10 * categorical[10, \"xyz\"]]",
  "?[10 * categorical[10, \"xyz\"]$",
  "?[10 * categorical[10, \"xyz\"]$",
  "$categorical[10, \"xyz\"]",
  "=categorical[10, \"xyz\"]",
  "$[categorical[10, \"xyz\"]]",
  "xxx[categorical[10, \"xyz\"]]",
  "?$categorical[10, \"xyz\"]]",
  "?=categorical[10, \"xyz\"]]",
  "?[categorical[10, \"xyz\"]$",
  "?[categorical[10, \"xyz\"]$",
  "10 $ categorical[10, \"xyz\"]",
  "?10 $ categorical[10, \"xyz\"]",
  "?[10 $ categorical[10, \"xyz\"]]",
  "$10 $ categorical[10, \"xyz\"]",
  "=10 $ categorical[10, \"xyz\"]",
  "$[10 $ categorical[10, \"xyz\"]]",
  "xxx[10 $ categorical[10, \"xyz\"]]",
  "?$10 $ categorical[10, \"xyz\"]]",
  "?=10 $ categorical[10, \"xyz\"]]",
  "?[10 $ categorical[10, \"xyz\"]$",
  "?[10 $ categorical[10, \"xyz\"]$",
  "10 = categorical[10, \"xyz\"]",
  "?10 = categorical[10, \"xyz\"]",
  "?[10 = categorical[10, \"xyz\"]]",
  "$10 = categorical[10, \"xyz\"]",
  "=10 = categorical[10, \"xyz\"]",
  "$[10 = categorical[10, \"xyz\"]]",
  "xxx[10 = categorical[10, \"xyz\"]]",
  "?$10 = categorical[10, \"xyz\"]]",
  "?=10 = categorical[10, \"xyz\"]]",
  "?[10 = categorical[10, \"xyz\"]$",
  "?[10 = categorical[10, \"xyz\"]$",
  "$10 * categorical[10, 10, \"xyz\"]",
  "=10 * categorical[10, 10, \"xyz\"]",
  "$[10 * categorical[10, 10, \"xyz\"]]",
  "xxx[10 * categorical[10, 10, \"xyz\"]]",
  "?$10 * categorical[10, 10, \"xyz\"]]",
  "?=10 * categorical[10, 10, \"xyz\"]]",
  "?[10 * categorical[10, 10, \"xyz\"]$",
  "?[10 * categorical[10, 10, \"xyz\"]$",
  "$categorical[10, 10, \"xyz\"]",
  "=categorical[10, 10, \"xyz\"]",
  "$[categorical[10, 10, \"xyz\"]]",
  "xxx[categorical[10, 10, \"xyz\"]]",
  "?$categorical[10, 10, \"xyz\"]]",
  "?=categorical[10, 10, \"xyz\"]]",
  "?[categorical[10, 10, \"xyz\"]$",
  "?[categorical[10, 10, \"xyz\"]$",
  "10 $ categorical[10, 10, \"xyz\"]",
  "?10 $ categorical[10, 10, \"xyz\"]",
  "?[10 $ categorical[10, 10, \"xyz\"]]",
  "$10 $ categorical[10, 10, \"xyz\"]",
  "=10 $ categorical[10, 10, \"xyz\"]",
  "$[10 $ categorical[10, 10, \"xyz\"]]",
  "xxx[10 $ categorical[10, 10, \"xyz\"]]",
  "?$10 $ categorical[10, 10, \"xyz\"]]",
  "?=10 $ categorical[10, 10, \"xyz\"]]",
  "?[10 $ categorical[10, 10, \"xyz\"]$",
  "?[10 $ categorical[10, 10, \"xyz\"]$",
  "10 = categorical[10, 10, \"xyz\"]",
  "?10 = categorical[10, 10, \"xyz\"]",
  "?[10 = categorical[10, 10, \"xyz\"]]",
  "$10 = categorical[10, 10, \"xyz\"]",
  "=10 = categorical[10, 10, \"xyz\"]",
  "$[10 = categorical[10, 10, \"xyz\"]]",
  "xxx[10 = categorical[10, 10, \"xyz\"]]",
  "?$10 = categorical[10, 10, \"xyz\"]]",
  "?=10 = categorical[10, 10, \"xyz\"]]",
  "?[10 = categorical[10, 10, \"xyz\"]$",
  "?[10 = categorical[10, 10, \"xyz\"]$",
  "$10 * categorical['xyz']",
  "=10 * categorical['xyz']",
  "$[10 * categorical['xyz']]",
  "xxx[10 * categorical['xyz']]",
  "?$10 * categorical['xyz']]",
  "?=10 * categorical['xyz']]",
  "?[10 * categorical['xyz']$",
  "?[10 * categorical['xyz']$",
  "$categorical['xyz']",
  "=categorical['xyz']",
  "$[categorical['xyz']]",
  "xxx[categorical['xyz']]",
  "?$categorical['xyz']]",
  "?=categorical['xyz']]",
  "?[categorical['xyz']$",
  "?[categorical['xyz']$",
  "10 $ categorical['xyz']",
  "?10 $ categorical['xyz']",
  "?[10 $ categorical['xyz']]",
  "$10 $ categorical['xyz']",
  "=10 $ categorical['xyz']",
  "$[10 $ categorical['xyz']]",
  "xxx[10 $ categorical['xyz']]",
  "?$10 $ categorical['xyz']]",
  "?=10 $ categorical['xyz']]",
  "?[10 $ categorical['xyz']$",
  "?[10 $ categorical['xyz']$",
  "10 = categorical['xyz']",
  "?10 = categorical['xyz']",
  "?[10 = categorical['xyz']]",
  "$10 = categorical['xyz']",
  "=10 = categorical['xyz']",
  "$[10 = categorical['xyz']]",
  "xxx[10 = categorical['xyz']]",
  "?$10 = categorical['xyz']]",
  "?=10 = categorical['xyz']]",
  "?[10 = categorical['xyz']$",
  "?[10 = categorical['xyz']$",
  "$10 * categorical[10, 'xyz']",
  "=10 * categorical[10, 'xyz']",
  "$[10 * categorical[10, 'xyz']]",
  "xxx[10 * categorical[10, 'xyz']]",
  "?$10 * categorical[10, 'xyz']]",
  "?=10 * categorical[10, 'xyz']]",
  "?[10 * categorical[10, 'xyz']$",
  "?[10 * categorical[10, 'xyz']$",
  "$categorical[10, 'xyz']",
  "=categorical[10, 'xyz']",
  "$[categorical[10, 'xyz']]",
  "xxx[categorical[10, 'xyz']]",
  "?$categorical[10, 'xyz']]",
  "?=categorical[10, 'xyz']]",
  "?[categorical[10, 'xyz']$",
  "?[categorical[10, 'xyz']$",
  "10 $ categorical[10, 'xyz']",
  "?10 $ categorical[10, 'xyz']",
  "?[10 $ categorical[10, 'xyz']]",
  "$10 $ categorical[10, 'xyz']",
  "=10 $ categorical[10, 'xyz']",
  "$[10 $ categorical[10, 'xyz']]",
  "xxx[10 $ categorical[10, 'xyz']]",
  "?$10 $ categorical[10, 'xyz']]",
  "?=10 $ categorical[10, 'xyz']]",
  "?[10 $ categorical[10, 'xyz']$",
  "?[10 $ categorical[10, 'xyz']$",
  "10 = categorical[10, 'xyz']",
  "?10 = categorical[10, 'xyz']",
  "?[10 = categorical[10, 'xyz']]",
  "$10 = categorical[10, 'xyz']",
  "=10 = categorical[10, 'xyz']",
  "$[10 = categorical[10, 'xyz']]",
  "xxx[10 = categorical[10, 'xyz']]",
  "?$10 = categorical[10, 'xyz']]",
  "?=10 = categorical[10, 'xyz']]",
  "?[10 = categorical[10, 'xyz']$",
  "?[10 = categorical[10, 'xyz']$",
  "$10 * categorical[10, 10, 'xyz']",
  "=10 * categorical[10, 10, 'xyz']",
  "$[10 * categorical[10, 10, 'xyz']]",
  "xxx[10 * categorical[10, 10, 'xyz']]",
  "?$10 * categorical[10, 10, 'xyz']]",
  "?=10 * categorical[10, 10, 'xyz']]",
  "?[10 * categorical[10, 10, 'xyz']$",
  "?[10 * categorical[10, 10, 'xyz']$",
  "$categorical[10, 10, 'xyz']",
  "=categorical[10, 10, 'xyz']",
  "$[categorical[10, 10, 'xyz']]",
  "xxx[categorical[10, 10, 'xyz']]",
  "?$categorical[10, 10, 'xyz']]",
  "?=categorical[10, 10, 'xyz']]",
  "?[categorical[10, 10, 'xyz']$",
  "?[categorical[10, 10, 'xyz']$",
  "10 $ categorical[10, 10, 'xyz']",
  "?10 $ categorical[10, 10, 'xyz']",
  "?[10 $ categorical[10, 10, 'xyz']]",
  "$10 $ categorical[10, 10, 'xyz']",
  "=10 $ categorical[10, 10, 'xyz']",
  "$[10 $ categorical[10, 10, 'xyz']]",
  "xxx[10 $ categorical[10, 10, 'xyz']]",
  "?$10 $ categorical[10, 10, 'xyz']]",
  "?=10 $ categorical[10, 10, 'xyz']]",
  "?[10 $ categorical[10, 10, 'xyz']$",
  "?[10 $ categorical[10, 10, 'xyz']$",
  "10 = categorical[10, 10, 'xyz']",
  "?10 = categorical[10, 10, 'xyz']",
  "?[10 = categorical[10, 10, 'xyz']]",
  "$10 = categorical[10, 10, 'xyz']",
  "=10 = categorical[10, 10, 'xyz']",
  "$[10 = categorical[10, 10, 'xyz']]",
  "xxx[10 = categorical[10, 10, 'xyz']]",
  "?$10 = categorical[10, 10, 'xyz']]",
  "?=10 = categorical[10, 10, 'xyz']]",
  "?[10 = categorical[10, 10, 'xyz']$",
  "?[10 = categorical[10, 10, 'xyz']$",
  "10 * categorical[128]",
  "?10 * categorical[128]",
  "?[10 * categorical[128]]",
  "$10 * categorical[128]",
  "=10 * categorical[128]",
  "$[10 * categorical[128]]",
  "xxx[10 * categorical[128]]",
  "?$10 * categorical[128]]",
  "?=10 * categorical[128]]",
  "?[10 * categorical[128]$",
  "?[10 * categorical[128]$",
  "categorical[128]",
  "?categorical[128]",
  "?[categorical[128]]",
  "$categorical[128]",
  "=categorical[128]",
  "$[categorical[128]]",
  "xxx[categorical[128]]",
  "?$categorical[128]]",
  "?=categorical[128]]",
  "?[categorical[128]$",
  "?[categorical[128]$",
  "10 $ categorical[128]",
  "?10 $ categorical[128]",
  "?[10 $ categorical[128]]",
  "$10 $ categorical[128]",
  "=10 $ categorical[128]",
  "$[10 $ categorical[128]]",
  "xxx[10 $ categorical[128]]",
  "?$10 $ categorical[128]]",
  "?=10 $ categorical[128]]",
  "?[10 $ categorical[128]$",
  "?[10 $ categorical[128]$",
  "10 = categorical[128]",
  "?10 = categorical[128]",
  "?[10 = categorical[128]]",
  "$10 = categorical[128]",
  "=10 = categorical[128]",
  "$[10 = categorical[128]]",
  "xxx[10 = categorical[128]]",
  "?$10 = categorical[128]]",
  "?=10 = categorical[128]]",
  "?[10 = categorical[128]$",
  "?[10 = categorical[128]$",
  "10 * categorical[10, 128]",
  "?10 * categorical[10, 128]",
  "?[10 * categorical[10, 128]]",
  "$10 * categorical[10, 128]",
  "=10 * categorical[10, 128]",
  "$[10 * categorical[10, 128]]",
  "xxx[10 * categorical[10, 128]]",
  "?$10 * categorical[10, 128]]",
  "?=10 * categorical[10, 128]]",
  "?[10 * categorical[10, 128]$",
  "?[10 * categorical[10, 128]$",
  "categorical[10, 128]",
  "?categorical[10, 128]",
  "?[categorical[10, 128]]",
  "$categorical[10, 128]",
  "=categorical[10, 128]",
  "$[categorical[10, 128]]",
  "xxx[categorical[10, 128]]",
  "?$categorical[10, 128]]",
  "?=categorical[10, 128]]",
  "?[categorical[10, 128]$",
  "?[categorical[10, 128]$",
  "10 $ categorical[10, 128]",
  "?10 $ categorical[10, 128]",
  "?[10 $ categorical[10, 128]]",
  "$10 $ categorical[10, 128]",
  "=10 $ categorical[10, 128]",
  "$[10 $ categorical[10, 128]]",
  "xxx[10 $ categorical[10, 128]]",
  "?$10 $ categorical[10, 128]]",
  "?=10 $ categorical[10, 128]]",
  "?[10 $ categorical[10, 128]$",
  "?[10 $ categorical[10, 128]$",
  "10 = categorical[10, 128]",
  "?10 = categorical[10, 128]",
  "?[10 = categorical[10, 128]]",
  "$10 = categorical[10, 128]",
  "=10 = categorical[10, 128]",
  "$[10 = categorical[10, 128]]",
  "xxx[10 = categorical[10, 128]]",
  "?$10 = categorical[10, 128]]",
  "?=10 = categorical[10, 128]]",
  "?[10 = categorical[10, 128]$",
  "?[10 = categorical[10, 128]$",
  "10 * categorical[10, 10, 128]",
  "?10 * categorical[10, 10, 128]",
  "?[10 * categorical[10, 10, 128]]",
  "$10 * categorical[10, 10, 128]",
  "=10 * categorical[10, 10, 128]",
  "$[10 * categorical[10, 10, 128]]",
  "xxx[10 * categorical[10, 10, 128]]",
  "?$10 * categorical[10, 10, 128]]",
  "?=10 * categorical[10, 10, 128]]",
  "?[10 * categorical[10, 10, 128]$",
  "?[10 * categorical[10, 10, 128]$",
  "categorical[10, 10, 128]",
  "?categorical[10, 10, 128]",
  "?[categorical[10, 10, 128]]",
  "$categorical[10, 10, 128]",
  "=categorical[10, 10, 128]",
  "$[categorical[10, 10, 128]]",
  "xxx[categorical[10, 10, 128]]",
  "?$categorical[10, 10, 128]]",
  "?=categorical[10, 10, 128]]",
  "?[categorical[10, 10, 128]$",
  "?[categorical[10, 10, 128]$",
  "10 $ categorical[10, 10, 128]",
  "?10 $ categorical[10, 10, 128]",
  "?[10 $ categorical[10, 10, 128]]",
  "$10 $ categorical[10, 10, 128]",
  "=10 $ categorical[10, 10, 128]",
  "$[10 $ categorical[10, 10, 128]]",
  "xxx[10 $ categorical[10, 10, 128]]",
  "?$10 $ categorical[10, 10, 128]]",
  "?=10 $ categorical[10, 10, 128]]",
  "?[10 $ categorical[10, 10, 128]$",
  "?[10 $ categorical[10, 10, 128]$",
  "10 = categorical[10, 10, 128]",
  "?10 = categorical[10, 10, 128]",
  "?[10 = categorical[10, 10, 128]]",
  "$10 = categorical[10, 10, 128]",
  "=10 = categorical[10, 10, 128]",
  "$[10 = categorical[10, 10, 128]]",
  "xxx[10 = categorical[10, 10, 128]]",
  "?$10 = categorical[10, 10, 128]]",
  "?=10 = categorical[10, 10, 128]]",
  "?[10 = categorical[10, 10, 128]$",
  "?[10 = categorical[10, 10, 128]$",
  "10 * categorical[-129]",
  "?10 * categorical[-129]",
  "?[10 * categorical[-129]]",
  "$10 * categorical[-129]",
  "=10 * categorical[-129]",
  "$[10 * categorical[-129]]",
  "xxx[10 * categorical[-129]]",
  "?$10 * categorical[-129]]",
  "?=10 * categorical[-129]]",
  "?[10 * categorical[-129]$",
  "?[10 * categorical[-129]$",
  "categorical[-129]",
  "?categorical[-129]",
  "?[categorical[-129]]",
  "$categorical[-129]",
  "=categorical[-129]",
  "$[categorical[-129]]",
  "xxx[categorical[-129]]",
  "?$categorical[-129]]",
  "?=categorical[-129]]",
  "?[categorical[-129]$",
  "?[categorical[-129]$",
  "10 $ categorical[-129]",
  "?10 $ categorical[-129]",
  "?[10 $ categorical[-129]]",
  "$10 $ categorical[-129]",
  "=10 $ categorical[-129]",
  "$[10 $ categorical[-129]]",
  "xxx[10 $ categorical[-129]]",
  "?$10 $ categorical[-129]]",
  "?=10 $ categorical[-129]]",
  "?[10 $ categorical[-129]$",
  "?[10 $ categorical[-129]$",
  "10 = categorical[-129]",
  "?10 = categorical[-129]",
  "?[10 = categorical[-129]]",
  "$10 = categorical[-129]",
  "=10 = categorical[-129]",
  "$[10 = categorical[-129]]",
  "xxx[10 = categorical[-129]]",
  "?$10 = categorical[-129]]",
  "?=10 = categorical[-129]]",
  "?[10 = categorical[-129]$",
  "?[10 = categorical[-129]$",
  "10 * categorical[10, -129]",
  "?10 * categorical[10, -129]",
  "?[10 * categorical[10, -129]]",
  "$10 * categorical[10, -129]",
  "=10 * categorical[10, -129]",
  "$[10 * categorical[10, -129]]",
  "xxx[10 * categorical[10, -129]]",
  "?$10 * categorical[10, -129]]",
  "?=10 * categorical[10, -129]]",
  "?[10 * categorical[10, -129]$",
  "?[10 * categorical[10, -129]$",
  "categorical[10, -129]",
  "?categorical[10, -129]",
  "?[categorical[10, -129]]",
  "$categorical[10, -129]",
  "=categorical[10, -129]",
  "$[categorical[10, -129]]",
  "xxx[categorical[10, -129]]",
  "?$categorical[10, -129]]",
  "?=categorical[10, -129]]",
  "?[categorical[10, -129]$",
  "?[categorical[10, -129]$",
  "10 $ categorical[10, -129]",
  "?10 $ categorical[10, -129]",
  "?[10 $ categorical[10, -129]]",
  "$10 $ categorical[10, -129]",
  "=10 $ categorical[10, -129]",
  "$[10 $ categorical[10, -129]]",
  "xxx[10 $ categorical[10, -129]]",
  "?$10 $ categorical[10, -129]]",
  "?=10 $ categorical[10, -129]]",
  "?[10 $ categorical[10, -129]$",
  "?[10 $ categorical[10, -129]$",
  "10 = categorical[10, -129]",
  "?10 = categorical[10, -129]",
  "?[10 = categorical[10, -129]]",
  "$10 = categorical[10, -129]",
  "=10 = categorical[10, -129]",
  "$[10 = categorical[10, -129]]",
  "xxx[10 = categorical[10, -129]]",
  "?$10 = categorical[10, -129]]",
  "?=10 = categorical[10, -129]]",
  "?[10 = categorical[10, -129]$",
  "?[10 = categorical[10, -129]$",
  "10 * categorical[10, 10, -129]",
  "?10 * categorical[10, 10, -129]",
  "?[10 * categorical[10, 10, -129]]",
  "$10 * categorical[10, 10, -129]",
  "=10 * categorical[10, 10, -129]",
  "$[10 * categorical[10, 10, -129]]",
  "xxx[10 * categorical[10, 10, -129]]",
  "?$10 * categorical[10, 10, -129]]",
  "?=10 * categorical[10, 10, -129]]",
  "?[10 * categorical[10, 10, -129]$",
  "?[10 * categorical[10, 10, -129]$",
  "categorical[10, 10, -129]",
  "?categorical[10, 10, -129]",
  "?[categorical[10, 10, -129]]",
  "$categorical[10, 10, -129]",
  "=categorical[10, 10, -129]",
  "$[categorical[10, 10, -129]]",
  "xxx[categorical[10, 10, -129]]",
  "?$categorical[10, 10, -129]]",
  "?=categorical[10, 10, -129]]",
  "?[categorical[10, 10, -129]$",
  "?[categorical[10, 10, -129]$",
  "10 $ categorical[10, 10, -129]",
  "?10 $ categorical[10, 10, -129]",
  "?[10 $ categorical[10, 10, -129]]",
  "$10 $ categorical[10, 10, -129]",
  "=10 $ categorical[10, 10, -129]",
  "$[10 $ categorical[10, 10, -129]]",
  "xxx[10 $ categorical[10, 10, -129]]",
  "?$10 $ categorical[10, 10, -129]]",
  "?=10 $ categorical[10, 10, -129]]",
  "?[10 $ categorical[10, 10, -129]$",
  "?[10 $ categorical[10, 10, -129]$",
  "10 = categorical[10, 10, -129]",
  "?10 = categorical[10, 10, -129]",
  "?[10 = categorical[10, 10, -129]]",
  "$10 = categorical[10, 10, -129]",
  "=10 = categorical[10, 10, -129]",
  "$[10 = categorical[10, 10, -129]]",
  "xxx[10 = categorical[10, 10, -129]]",
  "?$10 = categorical[10, 10, -129]]",
  "?=10 = categorical[10, 10, -129]]",
  "?[10 = categorical[10, 10, -129]$",
  "?[10 = categorical[10, 10, -129]$",
  "10 * categorical[32768]",
  "?10 * categorical[32768]",
  "?[10 * categorical[32768]]",
  "$10 * categorical[32768]",
  "=10 * categorical[32768]",
  "$[10 * categorical[32768]]",
  "xxx[10 * categorical[32768]]",
  "?$10 * categorical[32768]]",
  "?=10 * categorical[32768]]",
  "?[10 * categorical[32768]$",
  "?[10 * categorical[32768]$",
  "categorical[32768]",
  "?categorical[32768]",
  "?[categorical[32768]]",
  "$categorical[32768]",
  "=categorical[32768]",
  "$[categorical[32768]]",
  "xxx[categorical[32768]]",
  "?$categorical[32768]]",
  "?=categorical[32768]]",
  "?[categorical[32768]$",
  "?[categorical[32768]$",
  "10 $ categorical[32768]",
  "?10 $ categorical[32768]",
  "?[10 $ categorical[32768]]",
  "$10 $ categorical[32768]",
  "=10 $ categorical[32768]",
  "$[10 $ categorical[32768]]",
  "xxx[10 $ categorical[32768]]",
  "?$10 $ categorical[32768]]",
  "?=10 $ categorical[32768]]",
  "?[10 $ categorical[32768]$",
  "?[10 $ categorical[32768]$",
  "10 = categorical[32768]",
  "?10 = categorical[32768]",
  "?[10 = categorical[32768]]",
  "$10 = categorical[32768]",
  "=10 = categorical[32768]",
  "$[10 = categorical[32768]]",
  "xxx[10 = categorical[32768]]",
  "?$10 = categorical[32768]]",
  "?=10 = categorical[32768]]",
  "?[10 = categorical[32768]$",
  "?[10 = categorical[32768]$",
  "10 * categorical[10, 32768]",
  "?10 * categorical[10, 32768]",
  "?[10 * categorical[10, 32768]]",
  "$10 * categorical[10, 32768]",
  "=10 * categorical[10, 32768]",
  "$[10 * categorical[10, 32768]]",
  "xxx[10 * categorical[10, 32768]]",
  "?$10 * categorical[10, 32768]]",
  "?=10 * categorical[10, 32768]]",
  "?[10 * categorical[10, 32768]$",
  "?[10 * categorical[10, 32768]$",
  "categorical[10, 32768]",
  "?categorical[10, 32768]",
  "?[categorical[10, 32768]]",
  "$categorical[10, 32768]",
  "=categorical[10, 32768]",
  "$[categorical[10, 32768]]",
  "xxx[categorical[10, 32768]]",
  "?$categorical[10, 32768]]",
  "?=categorical[10, 32768]]",
  "?[categorical[10, 32768]$",
  "?[categorical[10, 32768]$",
  "10 $ categorical[10, 32768]",
  "?10 $ categorical[10, 32768]",
  "?[10 $ categorical[10, 32768]]",
  "$10 $ categorical[10, 32768]",
  "=10 $ categorical[10, 32768]",
  "$[10 $ categorical[10, 32768]]",
  "xxx[10 $ categorical[10, 32768]]",
  "?$10 $ categorical[10, 32768]]",
  "?=10 $ categorical[10, 32768]]",
  "?[10 $ categorical[10, 32768]$",
  "?[10 $ categorical[10, 32768]$",
  "10 = categorical[10, 32768]",
  "?10 = categorical[10, 32768]",
  "?[10 = categorical[10, 32768]]",
  "$10 = categorical[10, 32768]",
  "=10 = categorical[10, 32768]",
  "$[10 = categorical[10, 32768]]",
  "xxx[10 = categorical[10, 32768]]",
  "?$10 = categorical[10, 32768]]",
  "?=10 = categorical[10, 32768]]",
  "?[10 = categorical[10, 32768]$",
  "?[10 = categorical[10, 32768]$",
  "10 * categorical[10, 10, 32768]",
  "?10 * categorical[10, 10, 32768]",
  "?[10 * categorical[10, 10, 32768]]",
  "$10 * categorical[10, 10, 32768]",
  "=10 * categorical[10, 10, 32768]",
  "$[10 * categorical[10, 10, 32768]]",
  "xxx[10 * categorical[10, 10, 32768]]",
  "?$10 * categorical[10, 10, 32768]]",
  "?=10 * categorical[10, 10, 32768]]",
  "?[10 * categorical[10, 10, 32768]$",
  "?[10 * categorical[10, 10, 32768]$",
  "categorical[10, 10, 32768]",
  "?categorical[10, 10, 32768]",
  "?[categorical[10, 10, 32768]]",
  "$categorical[10, 10, 32768]",
  "=categorical[10, 10, 32768]",
  "$[categorical[10, 10, 32768]]",
  "xxx[categorical[10, 10, 32768]]",
  "?$categorical[10, 10, 32768]]",
  "?=categorical[10, 10, 32768]]",
  "?[categorical[10, 10, 32768]$",
  "?[categorical[10, 10, 32768]$",
  "10 $ categorical[10, 10, 32768]",
  "?10 $ categorical[10, 10, 32768]",
  "?[10 $ categorical[10, 10, 32768]]",
  "$10 $ categorical[10, 10, 32768]",
  "=10 $ categorical[10, 10, 32768]",
  "$[10 $ categorical[10, 10, 32768]]",
  "xxx[10 $ categorical[10, 10, 32768]]",
  "?$10 $ categorical[10, 10, 32768]]",
  "?=10 $ categorical[10, 10, 32768]]",
  "?[10 $ categorical[10, 10, 32768]$",
  "?[10 $ categorical[10, 10, 32768]$",
  "10 = categorical[10, 10, 32768]",
  "?10 = categorical[10, 10, 32768]",
  "?[10 = categorical[10, 10, 32768]]",
  "$10 = categorical[10, 10, 32768]",
  "=10 = categorical[10, 10, 32768]",
  "$[10 = categorical[10, 10, 32768]]",
  "xxx[10 = categorical[10, 10, 32768]]",
  "?$10 = categorical[10, 10, 32768]]",
  "?=10 = categorical[10, 10, 32768]]",
  "?[10 = categorical[10, 10, 32768]$",
  "?[10 = categorical[10, 10, 32768]$",
  "10 * categorical[-32769]",
  "?10 * categorical[-32769]",
  "?[10 * categorical[-32769]]",
  "$10 * categorical[-32769]",
  "=10 * categorical[-32769]",
  "$[10 * categorical[-32769]]",
  "xxx[10 * categorical[-32769]]",
  "?$10 * categorical[-32769]]",
  "?=10 * categorical[-32769]]",
  "?[10 * categorical[-32769]$",
  "?[10 * categorical[-32769]$",
  "categorical[-32769]",
  "?categorical[-32769]",
  "?[categorical[-32769]]",
  "$categorical[-32769]",
  "=categorical[-32769]",
  "$[categorical[-32769]]",
  "xxx[categorical[-32769]]",
  "?$categorical[-32769]]",
  "?=categorical[-32769]]",
  "?[categorical[-32769]$",
  "?[categorical[-32769]$",
  "10 $ categorical[-32769]",
  "?10 $ categorical[-32769]",
  "?[10 $ categorical[-32769]]",
  "$10 $ categorical[-32769]",
  "=10 $ categorical[-32769]",
  "$[10 $ categorical[-32769]]",
  "xxx[10 $ categorical[-32769]]",
  "?$10 $ categorical[-32769]]",
  "?=10 $ categorical[-32769]]",
  "?[10 $ categorical[-32769]$",
  "?[10 $ categorical[-32769]$",
  "10 = categorical[-32769]",
  "?10 = categorical[-32769]",
  "?[10 = categorical[-32769]]",
  "$10 = categorical[-32769]",
  "=10 = categorical[-32769]",
  "$[10 = categorical[-32769]]",
  "xxx[10 = categorical[-32769]]",
  "?$10 = categorical[-32769]]",
  "?=10 = categorical[-32769]]",
  "?[10 = categorical[-32769]$",
  "?[10 = categorical[-32769]$",
  "10 * categorical[10, -32769]",
  "?10 * categorical[10, -32769]",
  "?[10 * categorical[10, -32769]]",
  "$10 * categorical[10, -32769]",
  "=10 * categorical[10, -32769]",
  "$[10 * categorical[10, -32769]]",
  "xxx[10 * categorical[10, -32769]]",
  "?$10 * categorical[10, -32769]]",
  "?=10 * categorical[10, -32769]]",
  "?[10 * categorical[10, -32769]$",
  "?[10 * categorical[10, -32769]$",
  "categorical[10, -32769]",
  "?categorical[10, -32769]",
  "?[categorical[10, -32769]]",
  "$categorical[10, -32769]",
  "=categorical[10, -32769]",
  "$[categorical[10, -32769]]",
  "xxx[categorical[10, -32769]]",
  "?$categorical[10, -32769]]",
  "?=categorical[10, -32769]]",
  "?[categorical[10, -32769]$",
  "?[categorical[10, -32769]$",
  "10 $ categorical[10, -32769]",
  "?10 $ categorical[10, -32769]",
  "?[10 $ categorical[10, -32769]]",
  "$10 $ categorical[10, -32769]",
  "=10 $ categorical[10, -32769]",
  "$[10 $ categorical[10, -32769]]",
  "xxx[10 $ categorical[10, -32769]]",
  "?$10 $ categorical[10, -32769]]",
  "?=10 $ categorical[10, -32769]]",
  "?[10 $ categorical[10, -32769]$",
  "?[10 $ categorical[10, -32769]$",
  "10 = categorical[10, -32769]",
  "?10 = categorical[10, -32769]",
  "?[10 = categorical[10, -32769]]",
  "$10 = categorical[10, -32769]",
  "=10 = categorical[10, -32769]",
  "$[10 = categorical[10, -32769]]",
  "xxx[10 = categorical[10, -32769]]",
  "?$10 = categorical[10, -32769]]",
  "?=10 = categorical[10, -32769]]",
  "?[10 = categorical[10, -32769]$",
  "?[10 = categorical[10, -32769]$",
  "10 * categorical[10, 10, -32769]",
  "?10 * categorical[10, 10, -32769]",
  "?[10 * categorical[10, 10, -32769]]",
  "$10 * categorical[10, 10, -32769]",
  "=10 * categorical[10, 10, -32769]",
  "$[10 * categorical[10, 10, -32769]]",
  "xxx[10 * categorical[10, 10, -32769]]",
  "?$10 * categorical[10, 10, -32769]]",
  "?=10 * categorical[10, 10, -32769]]",
  "?[10 * categorical[10, 10, -32769]$",
  "?[10 * categorical[10, 10, -32769]$",
  "categorical[10, 10, -32769]",
  "?categorical[10, 10, -32769]",
  "?[categorical[10, 10, -32769]]",
  "$categorical[10, 10, -32769]",
  "=categorical[10, 10, -32769]",
  "$[categorical[10, 10, -32769]]",
  "xxx[categorical[10, 10, -32769]]",
  "?$categorical[10, 10, -32769]]",
  "?=categorical[10, 10, -32769]]",
  "?[categorical[10, 10, -32769]$",
  "?[categorical[10, 10, -32769]$",
  "10 $ categorical[10, 10, -32769]",
  "?10 $ categorical[10, 10, -32769]",
  "?[10 $ categorical[10, 10, -32769]]",
  "$10 $ categorical[10, 10, -32769]",
  "=10 $ categorical[10, 10, -32769]",
  "$[10 $ categorical[10, 10, -32769]]",
  "xxx[10 $ categorical[10, 10, -32769]]",
  "?$10 $ categorical[10, 10, -32769]]",
  "?=10 $ categorical[10, 10, -32769]]",
  "?[10 $ categorical[10, 10, -32769]$",
  "?[10 $ categorical[10, 10, -32769]$",
  "10 = categorical[10, 10, -32769]",
  "?10 = categorical[10, 10, -32769]",
  "?[10 = categorical[10, 10, -32769]]",
  "$10 = categorical[10, 10, -32769]",
  "=10 = categorical[10, 10, -32769]",
  "$[10 = categorical[10, 10, -32769]]",
  "xxx[10 = categorical[10, 10, -32769]]",
  "?$10 = categorical[10, 10, -32769]]",
  "?=10 = categorical[10, 10, -32769]]",
  "?[10 = categorical[10, 10, -32769]$",
  "?[10 = categorical[10, 10, -32769]$",
  "10 * categorical[2147483648]",
  "?10 * categorical[2147483648]",
  "?[10 * categorical[2147483648]]",
  "$10 * categorical[2147483648]",
  "=10 * categorical[2147483648]",
  "$[10 * categorical[2147483648]]",
  "xxx[10 * categorical[2147483648]]",
  "?$10 * categorical[2147483648]]",
  "?=10 * categorical[2147483648]]",
  "?[10 * categorical[2147483648]$",
  "?[10 * categorical[2147483648]$",
  "categorical[2147483648]",
  "?categorical[2147483648]",
  "?[categorical[2147483648]]",
  "$categorical[2147483648]",
  "=categorical[2147483648]",
  "$[categorical[2147483648]]",
  "xxx[categorical[2147483648]]",
  "?$categorical[2147483648]]",
  "?=categorical[2147483648]]",
  "?[categorical[2147483648]$",
  "?[categorical[2147483648]$",
  "10 $ categorical[2147483648]",
  "?10 $ categorical[2147483648]",
  "?[10 $ categorical[2147483648]]",
  "$10 $ categorical[2147483648]",
  "=10 $ categorical[2147483648]",
  "$[10 $ categorical[2147483648]]",
  "xxx[10 $ categorical[2147483648]]",
  "?$10 $ categorical[2147483648]]",
  "?=10 $ categorical[2147483648]]",
  "?[10 $ categorical[2147483648]$",
  "?[10 $ categorical[2147483648]$",
  "10 = categorical[2147483648]",
  "?10 = categorical[2147483648]",
  "?[10 = categorical[2147483648]]",
  "$10 = categorical[2147483648]",
  "=10 = categorical[2147483648]",
  "$[10 = categorical[2147483648]]",
  "xxx[10 = categorical[2147483648]]",
  "?$10 = categorical[2147483648]]",
  "?=10 = categorical[2147483648]]",
  "?[10 = categorical[2147483648]$",
  "?[10 = categorical[2147483648]$",
  "10 * categorical[10, 2147483648]",
  "?10 * categorical[10, 2147483648]",
  "?[10 * categorical[10, 2147483648]]",
  "$10 * categorical[10, 2147483648]",
  "=10 * categorical[10, 2147483648]",
  "$[10 * categorical[10, 2147483648]]",
  "xxx[10 * categorical[10, 2147483648]]",
  "?$10 * categorical[10, 2147483648]]",
  "?=10 * categorical[10, 2147483648]]",
  "?[10 * categorical[10, 2147483648]$",
  "?[10 * categorical[10, 2147483648]$",
  "categorical[10, 2147483648]",
  "?categorical[10, 2147483648]",
  "?[categorical[10, 2147483648]]",
  "$categorical[10, 2147483648]",
  "=categorical[10, 2147483648]",
  "$[categorical[10, 2147483648]]",
  "xxx[categorical[10, 2147483648]]",
  "?$categorical[10, 2147483648]]",
  "?=categorical[10, 2147483648]]",
  "?[categorical[10, 2147483648]$",
  "?[categorical[10, 2147483648]$",
  "10 $ categorical[10, 2147483648]",
  "?10 $ categorical[10, 2147483648]",
  "?[10 $ categorical[10, 2147483648]]",
  "$10 $ categorical[10, 2147483648]",
  "=10 $ categorical[10, 2147483648]",
  "$[10 $ categorical[10, 2147483648]]",
  "xxx[10 $ categorical[10, 2147483648]]",
  "?$10 $ categorical[10, 2147483648]]",
  "?=10 $ categorical[10, 2147483648]]",
  "?[10 $ categorical[10, 2147483648]$",
  "?[10 $ categorical[10, 2147483648]$",
  "10 = categorical[10, 2147483648]",
  "?10 = categorical[10, 2147483648]",
  "?[10 = categorical[10, 2147483648]]",
  "$10 = categorical[10, 2147483648]",
  "=10 = categorical[10, 2147483648]",
  "$[10 = categorical[10, 2147483648]]",
  "xxx[10 = categorical[10, 2147483648]]",
  "?$10 = categorical[10, 2147483648]]",
  "?=10 = categorical[10, 2147483648]]",
  "?[10 = categorical[10, 2147483648]$",
  "?[10 = categorical[10, 2147483648]$",
  "10 * categorical[10, 10, 2147483648]",
  "?10 * categorical[10, 10, 2147483648]",
  "?[10 * categorical[10, 10, 2147483648]]",
  "$10 * categorical[10, 10, 2147483648]",
  "=10 * categorical[10, 10, 2147483648]",
  "$[10 * categorical[10, 10, 2147483648]]",
  "xxx[10 * categorical[10, 10, 2147483648]]",
  "?$10 * categorical[10, 10, 2147483648]]",
  "?=10 * categorical[10, 10, 2147483648]]",
  "?[10 * categorical[10, 10, 2147483648]$",
  "?[10 * categorical[10, 10, 2147483648]$",
  "categorical[10, 10, 2147483648]",
  "?categorical[10, 10, 2147483648]",
  "?[categorical[10, 10, 2147483648]]",
  "$categorical[10, 10, 2147483648]",
  "=categorical[10, 10, 2147483648]",
  "$[categorical[10, 10, 2147483648]]",
  "xxx[categorical[10, 10, 2147483648]]",
  "?$categorical[10, 10, 2147483648]]",
  "?=categorical[10, 10, 2147483648]]",
  "?[categorical[10, 10, 2147483648]$",
  "?[categorical[10, 10, 2147483648]$",
  "10 $ categorical[10, 10, 2147483648]",
  "?10 $ categorical[10, 10, 2147483648]",
  "?[10 $ categorical[10, 10, 2147483648]]",
  "$10 $ categorical[10, 10, 2147483648]",
  "=10 $ categorical[10, 10, 2147483648]",
  "$[10 $ categorical[10, 10, 2147483648]]",
  "xxx[10 $ categorical[10, 10, 2147483648]]",
  "?$10 $ categorical[10, 10, 2147483648]]",
  "?=10 $ categorical[10, 10, 2147483648]]",
  "?[10 $ categorical[10, 10, 2147483648]$",
  "?[10 $ categorical[10, 10, 2147483648]$",
  "10 = categorical[10, 10, 2147483648]",
  "?10 = categorical[10, 10, 2147483648]",
  "?[10 = categorical[10, 10, 2147483648]]",
  "$10 = categorical[10, 10, 2147483648]",
  "=10 = categorical[10, 10, 2147483648]",
  "$[10 = categorical[10, 10, 2147483648]]",
  "xxx[10 = categorical[10, 10, 2147483648]]",
  "?$10 = categorical[10, 10, 2147483648]]",
  "?=10 = categorical[10, 10, 2147483648]]",
  "?[10 = categorical[10, 10, 2147483648]$",
  "?[10 = categorical[10, 10, 2147483648]$",
  "10 * categorical[-2147483649]",
  "?10 * categorical[-2147483649]",
  "?[10 * categorical[-2147483649]]",
  "$10 * categorical[-2147483649]",
  "=10 * categorical[-2147483649]",
  "$[10 * categorical[-2147483649]]",
  "xxx[10 * categorical[-2147483649]]",
  "?$10 * categorical[-2147483649]]",
  "?=10 * categorical[-2147483649]]",
  "?[10 * categorical[-2147483649]$",
  "?[10 * categorical[-2147483649]$",
  "categorical[-2147483649]",
  "?categorical[-2147483649]",
  "?[categorical[-2147483649]]",
  "$categorical[-2147483649]",
  "=categorical[-2147483649]",
  "$[categorical[-2147483649]]",
  "xxx[categorical[-2147483649]]",
  "?$categorical[-2147483649]]",
  "?=categorical[-2147483649]]",
  "?[categorical[-2147483649]$",
  "?[categorical[-2147483649]$",
  "10 $ categorical[-2147483649]",
  "?10 $ categorical[-2147483649]",
  "?[10 $ categorical[-2147483649]]",
  "$10 $ categorical[-2147483649]",
  "=10 $ categorical[-2147483649]",
  "$[10 $ categorical[-2147483649]]",
  "xxx[10 $ categorical[-2147483649]]",
  "?$10 $ categorical[-2147483649]]",
  "?=10 $ categorical[-2147483649]]",
  "?[10 $ categorical[-2147483649]$",
  "?[10 $ categorical[-2147483649]$",
  "10 = categorical[-2147483649]",
  "?10 = categorical[-2147483649]",
  "?[10 = categorical[-2147483649]]",
  "$10 = categorical[-2147483649]",
  "=10 = categorical[-2147483649]",
  "$[10 = categorical[-2147483649]]",
  "xxx[10 = categorical[-2147483649]]",
  "?$10 = categorical[-2147483649]]",
  "?=10 = categorical[-2147483649]]",
  "?[10 = categorical[-2147483649]$",
  "?[10 = categorical[-2147483649]$",
  "10 * categorical[10, -2147483649]",
  "?10 * categorical[10, -2147483649]",
  "?[10 * categorical[10, -2147483649]]",
  "$10 * categorical[10, -2147483649]",
  "=10 * categorical[10, -2147483649]",
  "$[10 * categorical[10, -2147483649]]",
  "xxx[10 * categorical[10, -2147483649]]",
  "?$10 * categorical[10, -2147483649]]",
  "?=10 * categorical[10, -2147483649]]",
  "?[10 * categorical[10, -2147483649]$",
  "?[10 * categorical[10, -2147483649]$",
  "categorical[10, -2147483649]",
  "?categorical[10, -2147483649]",
  "?[categorical[10, -2147483649]]",
  "$categorical[10, -2147483649]",
  "=categorical[10, -2147483649]",
  "$[categorical[10, -2147483649]]",
  "xxx[categorical[10, -2147483649]]",
  "?$categorical[10, -2147483649]]",
  "?=categorical[10, -2147483649]]",
  "?[categorical[10, -2147483649]$",
  "?[categorical[10, -2147483649]$",
  "10 $ categorical[10, -2147483649]",
  "?10 $ categorical[10, -2147483649]",
  "?[10 $ categorical[10, -2147483649]]",
  "$10 $ categorical[10, -2147483649]",
  "=10 $ categorical[10, -2147483649]",
  "$[10 $ categorical[10, -2147483649]]",
  "xxx[10 $ categorical[10, -2147483649]]",
  "?$10 $ categorical[10, -2147483649]]",
  "?=10 $ categorical[10, -2147483649]]",
  "?[10 $ categorical[10, -2147483649]$",
  "?[10 $ categorical[10, -2147483649]$",
  "10 = categorical[10, -2147483649]",
  "?10 = categorical[10, -2147483649]",
  "?[10 = categorical[10, -2147483649]]",
  "$10 = categorical[10, -2147483649]",
  "=10 = categorical[10, -2147483649]",
  "$[10 = categorical[10, -2147483649]]",
  "xxx[10 = categorical[10, -2147483649]]",
  "?$10 = categorical[10, -2147483649]]",
  "?=10 = categorical[10, -2147483649]]",
  "?[10 = categorical[10, -2147483649]$",
  "?[10 = categorical[10, -2147483649]$",
  "10 * categorical[10, 10, -2147483649]",
  "?10 * categorical[10, 10, -2147483649]",
  "?[10 * categorical[10, 10, -2147483649]]",
  "$10 * categorical[10, 10, -2147483649]",
  "=10 * categorical[10, 10, -2147483649]",
  "$[10 * categorical[10, 10, -2147483649]]",
  "xxx[10 * categorical[10, 10, -2147483649]]",
  "?$10 * categorical[10, 10, -2147483649]]",
  "?=10 * categorical[10, 10, -2147483649]]",
  "?[10 * categorical[10, 10, -2147483649]$",
  "?[10 * categorical[10, 10, -2147483649]$",
  "categorical[10, 10, -2147483649]",
  "?categorical[10, 10, -2147483649]",
  "?[categorical[10, 10, -2147483649]]",
  "$categorical[10, 10, -2147483649]",
  "=categorical[10, 10, -2147483649]",
  "$[categorical[10, 10, -2147483649]]",
  "xxx[categorical[10, 10, -2147483649]]",
  "?$categorical[10, 10, -2147483649]]",
  "?=categorical[10, 10, -2147483649]]",
  "?[categorical[10, 10, -2147483649]$",
  "?[categorical[10, 10, -2147483649]$",
  "10 $ categorical[10, 10, -2147483649]",
  "?10 $ categorical[10, 10, -2147483649]",
  "?[10 $ categorical[10, 10, -2147483649]]",
  "$10 $ categorical[10, 10, -2147483649]",
  "=10 $ categorical[10, 10, -2147483649]",
  "$[10 $ categorical[10, 10, -2147483649]]",
  "xxx[10 $ categorical[10, 10, -2147483649]]",
  "?$10 $ categorical[10, 10, -2147483649]]",
  "?=10 $ categorical[10, 10, -2147483649]]",
  "?[10 $ categorical[10, 10, -2147483649]$",
  "?[10 $ categorical[10, 10, -2147483649]$",
  "10 = categorical[10, 10, -2147483649]",
  "?10 = categorical[10, 10, -2147483649]",
  "?[10 = categorical[10, 10, -2147483649]]",
  "$10 = categorical[10, 10, -2147483649]",
  "=10 = categorical[10, 10, -2147483649]",
  "$[10 = categorical[10, 10, -2147483649]]",
  "xxx[10 = categorical[10, 10, -2147483649]]",
  "?$10 = categorical[10, 10, -2147483649]]",
  "?=10 = categorical[10, 10, -2147483649]]",
  "?[10 = categorical[10, 10, -2147483649]$",
  "?[10 = categorical[10, 10, -2147483649]$",
  "10 * categorical[9223372036854775808]",
  "?10 * categorical[9223372036854775808]",
  "?[10 * categorical[9223372036854775808]]",
  "$10 * categorical[9223372036854775808]",
  "=10 * categorical[9223372036854775808]",
  "$[10 * categorical[9223372036854775808]]",
  "xxx[10 * categorical[9223372036854775808]]",
  "?$10 * categorical[9223372036854775808]]",
  "?=10 * categorical[9223372036854775808]]",
  "?[10 * categorical[9223372036854775808]$",
  "?[10 * categorical[9223372036854775808]$",
  "categorical[9223372036854775808]",
  "?categorical[9223372036854775808]",
  "?[categorical[9223372036854775808]]",
  "$categorical[9223372036854775808]",
  "=categorical[9223372036854775808]",
  "$[categorical[9223372036854775808]]",
  "xxx[categorical[9223372036854775808]]",
  "?$categorical[9223372036854775808]]",
  "?=categorical[9223372036854775808]]",
  "?[categorical[9223372036854775808]$",
  "?[categorical[9223372036854775808]$",
  "10 $ categorical[9223372036854775808]",
  "?10 $ categorical[9223372036854775808]",
  "?[10 $ categorical[9223372036854775808]]",
  "$10 $ categorical[9223372036854775808]",
  "=10 $ categorical[9223372036854775808]",
  "$[10 $ categorical[9223372036854775808]]",
  "xxx[10 $ categorical[9223372036854775808]]",
  "?$10 $ categorical[9223372036854775808]]",
  "?=10 $ categorical[9223372036854775808]]",
  "?[10 $ categorical[9223372036854775808]$",
  "?[10 $ categorical[9223372036854775808]$",
  "10 = categorical[9223372036854775808]",
  "?10 = categorical[9223372036854775808]",
  "?[10 = categorical[9223372036854775808]]",
  "$10 = categorical[9223372036854775808]",
  "=10 = categorical[9223372036854775808]",
  "$[10 = categorical[9223372036854775808]]",
  "xxx[10 = categorical[9223372036854775808]]",
  "?$10 = categorical[9223372036854775808]]",
  "?=10 = categorical[9223372036854775808]]",
  "?[10 = categorical[9223372036854775808]$",
  "?[10 = categorical[9223372036854775808]$",
  "10 * categorical[10, 9223372036854775808]",
  "?10 * categorical[10, 9223372036854775808]",
  "?[10 * categorical[10, 9223372036854775808]]",
  "$10 * categorical[10, 9223372036854775808]",
  "=10 * categorical[10, 9223372036854775808]",
  "$[10 * categorical[10, 9223372036854775808]]",
  "xxx[10 * categorical[10, 9223372036854775808]]",
  "?$10 * categorical[10, 9223372036854775808]]",
  "?=10 * categorical[10, 9223372036854775808]]",
  "?[10 * categorical[10, 9223372036854775808]$",
  "?[10 * categorical[10, 9223372036854775808]$",
  "categorical[10, 9223372036854775808]",
  "?categorical[10, 9223372036854775808]",
  "?[categorical[10, 9223372036854775808]]",
  "$categorical[10, 9223372036854775808]",
  "=categorical[10, 9223372036854775808]",
  "$[categorical[10, 9223372036854775808]]",
  "xxx[categorical[10, 9223372036854775808]]",
  "?$categorical[10, 9223372036854775808]]",
  "?=categorical[10, 9223372036854775808]]",
  "?[categorical[10, 9223372036854775808]$",
  "?[categorical[10, 9223372036854775808]$",
  "10 $ categorical[10, 9223372036854775808]",
  "?10 $ categorical[10, 9223372036854775808]",
  "?[10 $ categorical[10, 9223372036854775808]]",
  "$10 $ categorical[10, 9223372036854775808]",
  "=10 $ categorical[10, 9223372036854775808]",
  "$[10 $ categorical[10, 9223372036854775808]]",
  "xxx[10 $ categorical[10, 9223372036854775808]]",
  "?$10 $ categorical[10, 9223372036854775808]]",
  "?=10 $ categorical[10, 9223372036854775808]]",
  "?[10 $ categorical[10, 9223372036854775808]$",
  "?[10 $ categorical[10, 9223372036854775808]$",
  "10 = categorical[10, 9223372036854775808]",
  "?10 = categorical[10, 9223372036854775808]",
  "?[10 = categorical[10, 9223372036854775808]]",
  "$10 = categorical[10, 9223372036854775808]",
  "=10 = categorical[10, 9223372036854775808]",
  "$[10 = categorical[10, 9223372036854775808]]",
  "xxx[10 = categorical[10, 9223372036854775808]]",
  "?$10 = categorical[10, 9223372036854775808]]",
  "?=10 = categorical[10, 9223372036854775808]]",
  "?[10 = categorical[10, 9223372036854775808]$",
  "?[10 = categorical[10, 9223372036854775808]$",
  "10 * categorical[10, 10, 9223372036854775808]",
  "?10 * categorical[10, 10, 9223372036854775808]",
  "?[10 * categorical[10, 10, 9223372036854775808]]",
  "$10 * categorical[10, 10, 9223372036854775808]",
  "=10 * categorical[10, 10, 9223372036854775808]",
  "$[10 * categorical[10, 10, 9223372036854775808]]",
  "xxx[10 * categorical[10, 10, 9223372036854775808]]",
  "?$10 * categorical[10, 10, 9223372036854775808]]",
  "?=10 * categorical[10, 10, 9223372036854775808]]",
  "?[10 * categorical[10, 10, 9223372036854775808]$",
  "?[10 * categorical[10, 10, 9223372036854775808]$",
  "categorical[10, 10, 9223372036854775808]",
  "?categorical[10, 10, 9223372036854775808]",
  "?[categorical[10, 10, 9223372036854775808]]",
  "$categorical[10, 10, 9223372036854775808]",
  "=categorical[10, 10, 9223372036854775808]",
  "$[categorical[10, 10, 9223372036854775808]]",
  "xxx[categorical[10, 10, 9223372036854775808]]",
  "?$categorical[10, 10, 9223372036854775808]]",
  "?=categorical[10, 10, 9223372036854775808]]",
  "?[categorical[10, 10, 9223372036854775808]$",
  "?[categorical[10, 10, 9223372036854775808]$",
  "10 $ categorical[10, 10, 9223372036854775808]",
  "?10 $ categorical[10, 10, 9223372036854775808]",
  "?[10 $ categorical[10, 10, 9223372036854775808]]",
  "$10 $ categorical[10, 10, 9223372036854775808]",
  "=10 $ categorical[10, 10, 9223372036854775808]",
  "$[10 $ categorical[10, 10, 9223372036854775808]]",
  "xxx[10 $ categorical[10, 10, 9223372036854775808]]",
  "?$10 $ categorical[10, 10, 9223372036854775808]]",
  "?=10 $ categorical[10, 10, 9223372036854775808]]",
  "?[10 $ categorical[10, 10, 9223372036854775808]$",
  "?[10 $ categorical[10, 10, 9223372036854775808]$",
  "10 = categorical[10, 10, 9223372036854775808]",
  "?10 = categorical[10, 10, 9223372036854775808]",
  "?[10 = categorical[10, 10, 9223372036854775808]]",
  "$10 = categorical[10, 10, 9223372036854775808]",
  "=10 = categorical[10, 10, 9223372036854775808]",
  "$[10 = categorical[10, 10, 9223372036854775808]]",
  "xxx[10 = categorical[10, 10, 9223372036854775808]]",
  "?$10 = categorical[10, 10, 9223372036854775808]]",
  "?=10 = categorical[10, 10, 9223372036854775808]]",
  "?[10 = categorical[10, 10, 9223372036854775808]$",
  "?[10 = categorical[10, 10, 9223372036854775808]$",
  "10 * categorical[-9223372036854775809]",
  "?10 * categorical[-9223372036854775809]",
  "?[10 * categorical[-9223372036854775809]]",
  "$10 * categorical[-9223372036854775809]",
  "=10 * categorical[-9223372036854775809]",
  "$[10 * categorical[-9223372036854775809]]",
  "xxx[10 * categorical[-9223372036854775809]]",
  "?$10 * categorical[-9223372036854775809]]",
  "?=10 * categorical[-9223372036854775809]]",
  "?[10 * categorical[-9223372036854775809]$",
  "?[10 * categorical[-9223372036854775809]$",
  "categorical[-9223372036854775809]",
  "?categorical[-9223372036854775809]",
  "?[categorical[-9223372036854775809]]",
  "$categorical[-9223372036854775809]",
  "=categorical[-9223372036854775809]",
  "$[categorical[-9223372036854775809]]",
  "xxx[categorical[-9223372036854775809]]",
  "?$categorical[-9223372036854775809]]",
  "?=categorical[-9223372036854775809]]",
  "?[categorical[-9223372036854775809]$",
  "?[categorical[-9223372036854775809]$",
  "10 $ categorical[-9223372036854775809]",
  "?10 $ categorical[-9223372036854775809]",
  "?[10 $ categorical[-9223372036854775809]]",
  "$10 $ categorical[-9223372036854775809]",
  "=10 $ categorical[-9223372036854775809]",
  "$[10 $ categorical[-9223372036854775809]]",
  "xxx[10 $ categorical[-9223372036854775809]]",
  "?$10 $ categorical[-9223372036854775809]]",
  "?=10 $ categorical[-9223372036854775809]]",
  "?[10 $ categorical[-9223372036854775809]$",
  "?[10 $ categorical[-9223372036854775809]$",
  "10 = categorical[-9223372036854775809]",
  "?10 = categorical[-9223372036854775809]",
  "?[10 = categorical[-9223372036854775809]]",
  "$10 = categorical[-9223372036854775809]",
  "=10 = categorical[-9223372036854775809]",
  "$[10 = categorical[-9223372036854775809]]",
  "xxx[10 = categorical[-9223372036854775809]]",
  "?$10 = categorical[-9223372036854775809]]",
  "?=10 = categorical[-9223372036854775809]]",
  "?[10 = categorical[-9223372036854775809]$",
  "?[10 = categorical[-9223372036854775809]$",
  "10 * categorical[10, -9223372036854775809]",
  "?10 * categorical[10, -9223372036854775809]",
  "?[10 * categorical[10, -9223372036854775809]]",
  "$10 * categorical[10, -9223372036854775809]",
  "=10 * categorical[10, -9223372036854775809]",
  "$[10 * categorical[10, -9223372036854775809]]",
  "xxx[10 * categorical[10, -9223372036854775809]]",
  "?$10 * categorical[10, -9223372036854775809]]",
  "?=10 * categorical[10, -9223372036854775809]]",
  "?[10 * categorical[10, -9223372036854775809]$",
  "?[10 * categorical[10, -9223372036854775809]$",
  "categorical[10, -9223372036854775809]",
  "?categorical[10, -9223372036854775809]",
  "?[categorical[10, -9223372036854775809]]",
  "$categorical[10, -9223372036854775809]",
  "=categorical[10, -9223372036854775809]",
  "$[categorical[10, -9223372036854775809]]",
  "xxx[categorical[10, -9223372036854775809]]",
  "?$categorical[10, -9223372036854775809]]",
  "?=categorical[10, -9223372036854775809]]",
  "?[categorical[10, -9223372036854775809]$",
  "?[categorical[10, -9223372036854775809]$",
  "10 $ categorical[10, -9223372036854775809]",
  "?10 $ categorical[10, -9223372036854775809]",
  "?[10 $ categorical[10, -9223372036854775809]]",
  "$10 $ categorical[10, -9223372036854775809]",
  "=10 $ categorical[10, -9223372036854775809]",
  "$[10 $ categorical[10, -9223372036854775809]]",
  "xxx[10 $ categorical[10, -9223372036854775809]]",
  "?$10 $ categorical[10, -9223372036854775809]]",
  "?=10 $ categorical[10, -9223372036854775809]]",
  "?[10 $ categorical[10, -9223372036854775809]$",
  "?[10 $ categorical[10, -9223372036854775809]$",
  "10 = categorical[10, -9223372036854775809]",
  "?10 = categorical[10, -9223372036854775809]",
  "?[10 = categorical[10, -9223372036854775809]]",
  "$10 = categorical[10, -9223372036854775809]",
  "=10 = categorical[10, -9223372036854775809]",
  "$[10 = categorical[10, -9223372036854775809]]",
  "xxx[10 = categorical[10, -9223372036854775809]]",
  "?$10 = categorical[10, -9223372036854775809]]",
  "?=10 = categorical[10, -9223372036854775809]]",
  "?[10 = categorical[10, -9223372036854775809]$",
  "?[10 = categorical[10, -9223372036854775809]$",
  "10 * categorical[10, 10, -9223372036854775809]",
  "?10 * categorical[10, 10, -9223372036854775809]",
  "?[10 * categorical[10, 10, -9223372036854775809]]",
  "$10 * categorical[10, 10, -9223372036854775809]",
  "=10 * categorical[10, 10, -9223372036854775809]",
  "$[10 * categorical[10, 10, -9223372036854775809]]",
  "xxx[10 * categorical[10, 10, -9223372036854775809]]",
  "?$10 * categorical[10, 10, -9223372036854775809]]",
  "?=10 * categorical[10, 10, -9223372036854775809]]",
  "?[10 * categorical[10, 10, -9223372036854775809]$",
  "?[10 * categorical[10, 10, -9223372036854775809]$",
  "categorical[10, 10, -9223372036854775809]",
  "?categorical[10, 10, -9223372036854775809]",
  "?[categorical[10, 10, -9223372036854775809]]",
  "$categorical[10, 10, -9223372036854775809]",
  "=categorical[10, 10, -9223372036854775809]",
  "$[categorical[10, 10, -9223372036854775809]]",
  "xxx[categorical[10, 10, -9223372036854775809]]",
  "?$categorical[10, 10, -9223372036854775809]]",
  "?=categorical[10, 10, -9223372036854775809]]",
  "?[categorical[10, 10, -9223372036854775809]$",
  "?[categorical[10, 10, -9223372036854775809]$",
  "10 $ categorical[10, 10, -9223372036854775809]",
  "?10 $ categorical[10, 10, -9223372036854775809]",
  "?[10 $ categorical[10, 10, -9223372036854775809]]",
  "$10 $ categorical[10, 10, -9223372036854775809]",
  "=10 $ categorical[10, 10, -9223372036854775809]",
  "$[10 $ categorical[10, 10, -9223372036854775809]]",
  "xxx[10 $ categorical[10, 10, -9223372036854775809]]",
  "?$10 $ categorical[10, 10, -9223372036854775809]]",
  "?=10 $ categorical[10, 10, -9223372036854775809]]",
  "?[10 $ categorical[10, 10, -9223372036854775809]$",
  "?[10 $ categorical[10, 10, -9223372036854775809]$",
  "10 = categorical[10, 10, -9223372036854775809]",
  "?10 = categorical[10, 10, -9223372036854775809]",
  "?[10 = categorical[10, 10, -9223372036854775809]]",
  "$10 = categorical[10, 10, -9223372036854775809]",
  "=10 = categorical[10, 10, -9223372036854775809]",
  "$[10 = categorical[10, 10, -9223372036854775809]]",
  "xxx[10 = categorical[10, 10, -9223372036854775809]]",
  "?$10 = categorical[10, 10, -9223372036854775809]]",
  "?=10 = categorical[10, 10, -9223372036854775809]]",
  "?[10 = categorical[10, 10, -9223372036854775809]$",
  "?[10 = categorical[10, 10, -9223372036854775809]$",
  "10 * categorical[256]",
  "?10 * categorical[256]",
  "?[10 * categorical[256]]",
  "$10 * categorical[256]",
  "=10 * categorical[256]",
  "$[10 * categorical[256]]",
  "xxx[10 * categorical[256]]",
  "?$10 * categorical[256]]",
  "?=10 * categorical[256]]",
  "?[10 * categorical[256]$",
  "?[10 * categorical[256]$",
  "categorical[256]",
  "?categorical[256]",
  "?[categorical[256]]",
  "$categorical[256]",
  "=categorical[256]",
  "$[categorical[256]]",
  "xxx[categorical[256]]",
  "?$categorical[256]]",
  "?=categorical[256]]",
  "?[categorical[256]$",
  "?[categorical[256]$",
  "10 $ categorical[256]",
  "?10 $ categorical[256]",
  "?[10 $ categorical[256]]",
  "$10 $ categorical[256]",
  "=10 $ categorical[256]",
  "$[10 $ categorical[256]]",
  "xxx[10 $ categorical[256]]",
  "?$10 $ categorical[256]]",
  "?=10 $ categorical[256]]",
  "?[10 $ categorical[256]$",
  "?[10 $ categorical[256]$",
  "10 = categorical[256]",
  "?10 = categorical[256]",
  "?[10 = categorical[256]]",
  "$10 = categorical[256]",
  "=10 = categorical[256]",
  "$[10 = categorical[256]]",
  "xxx[10 = categorical[256]]",
  "?$10 = categorical[256]]",
  "?=10 = categorical[256]]",
  "?[10 = categorical[256]$",
  "?[10 = categorical[256]$",
  "10 * categorical[10, 256]",
  "?10 * categorical[10, 256]",
  "?[10 * categorical[10, 256]]",
  "$10 * categorical[10, 256]",
  "=10 * categorical[10, 256]",
  "$[10 * categorical[10, 256]]",
  "xxx[10 * categorical[10, 256]]",
  "?$10 * categorical[10, 256]]",
  "?=10 * categorical[10, 256]]",
  "?[10 * categorical[10, 256]$",
  "?[10 * categorical[10, 256]$",
  "categorical[10, 256]",
  "?categorical[10, 256]",
  "?[categorical[10, 256]]",
  "$categorical[10, 256]",
  "=categorical[10, 256]",
  "$[categorical[10, 256]]",
  "xxx[categorical[10, 256]]",
  "?$categorical[10, 256]]",
  "?=categorical[10, 256]]",
  "?[categorical[10, 256]$",
  "?[categorical[10, 256]$",
  "10 $ categorical[10, 256]",
  "?10 $ categorical[10, 256]",
  "?[10 $ categorical[10, 256]]",
  "$10 $ categorical[10, 256]",
  "=10 $ categorical[10, 256]",
  "$[10 $ categorical[10, 256]]",
  "xxx[10 $ categorical[10, 256]]",
  "?$10 $ categorical[10, 256]]",
  "?=10 $ categorical[10, 256]]",
  "?[10 $ categorical[10, 256]$",
  "?[10 $ categorical[10, 256]$",
  "10 = categorical[10, 256]",
  "?10 = categorical[10, 256]",
  "?[10 = categorical[10, 256]]",
  "$10 = categorical[10, 256]",
  "=10 = categorical[10, 256]",
  "$[10 = categorical[10, 256]]",
  "xxx[10 = categorical[10, 256]]",
  "?$10 = categorical[10, 256]]",
  "?=10 = categorical[10, 256]]",
  "?[10 = categorical[10, 256]$",
  "?[10 = categorical[10, 256]$",
  "10 * categorical[10, 10, 256]",
  "?10 * categorical[10, 10, 256]",
  "?[10 * categorical[10, 10, 256]]",
  "$10 * categorical[10, 10, 256]",
  "=10 * categorical[10, 10, 256]",
  "$[10 * categorical[10, 10, 256]]",
  "xxx[10 * categorical[10, 10, 256]]",
  "?$10 * categorical[10, 10, 256]]",
  "?=10 * categorical[10, 10, 256]]",
  "?[10 * categorical[10, 10, 256]$",
  "?[10 * categorical[10, 10, 256]$",
  "categorical[10, 10, 256]",
  "?categorical[10, 10, 256]",
  "?[categorical[10, 10, 256]]",
  "$categorical[10, 10, 256]",
  "=categorical[10, 10, 256]",
  "$[categorical[10, 10, 256]]",
  "xxx[categorical[10, 10, 256]]",
  "?$categorical[10, 10, 256]]",
  "?=categorical[10, 10, 256]]",
  "?[categorical[10, 10, 256]$",
  "?[categorical[10, 10, 256]$",
  "10 $ categorical[10, 10, 256]",
  "?10 $ categorical[10, 10, 256]",
  "?[10 $ categorical[10, 10, 256]]",
  "$10 $ categorical[10, 10, 256]",
  "=10 $ categorical[10, 10, 256]",
  "$[10 $ categorical[10, 10, 256]]",
  "xxx[10 $ categorical[10, 10, 256]]",
  "?$10 $ categorical[10, 10, 256]]",
  "?=10 $ categorical[10, 10, 256]]",
  "?[10 $ categorical[10, 10, 256]$",
  "?[10 $ categorical[10, 10, 256]$",
  "10 = categorical[10, 10, 256]",
  "?10 = categorical[10, 10, 256]",
  "?[10 = categorical[10, 10, 256]]",
  "$10 = categorical[10, 10, 256]",
  "=10 = categorical[10, 10, 256]",
  "$[10 = categorical[10, 10, 256]]",
  "xxx[10 = categorical[10, 10, 256]]",
  "?$10 = categorical[10, 10, 256]]",
  "?=10 = categorical[10, 10, 256]]",
  "?[10 = categorical[10, 10, 256]$",
  "?[10 = categorical[10, 10, 256]$",
  "10 * categorical[-1]",
  "?10 * categorical[-1]",
  "?[10 * categorical[-1]]",
  "$10 * categorical[-1]",
  "=10 * categorical[-1]",
  "$[10 * categorical[-1]]",
  "xxx[10 * categorical[-1]]",
  "?$10 * categorical[-1]]",
  "?=10 * categorical[-1]]",
  "?[10 * categorical[-1]$",
  "?[10 * categorical[-1]$",
  "categorical[-1]",
  "?categorical[-1]",
  "?[categorical[-1]]",
  "$categorical[-1]",
  "=categorical[-1]",
  "$[categorical[-1]]",
  "xxx[categorical[-1]]",
  "?$categorical[-1]]",
  "?=categorical[-1]]",
  "?[categorical[-1]$",
  "?[categorical[-1]$",
  "10 $ categorical[-1]",
  "?10 $ categorical[-1]",
  "?[10 $ categorical[-1]]",
  "$10 $ categorical[-1]",
  "=10 $ categorical[-1]",
  "$[10 $ categorical[-1]]",
  "xxx[10 $ categorical[-1]]",
  "?$10 $ categorical[-1]]",
  "?=10 $ categorical[-1]]",
  "?[10 $ categorical[-1]$",
  "?[10 $ categorical[-1]$",
  "10 = categorical[-1]",
  "?10 = categorical[-1]",
  "?[10 = categorical[-1]]",
  "$10 = categorical[-1]",
  "=10 = categorical[-1]",
  "$[10 = categorical[-1]]",
  "xxx[10 = categorical[-1]]",
  "?$10 = categorical[-1]]",
  "?=10 = categorical[-1]]",
  "?[10 = categorical[-1]$",
  "?[10 = categorical[-1]$",
  "10 * categorical[10, -1]",
  "?10 * categorical[10, -1]",
  "?[10 * categorical[10, -1]]",
  "$10 * categorical[10, -1]",
  "=10 * categorical[10, -1]",
  "$[10 * categorical[10, -1]]",
  "xxx[10 * categorical[10, -1]]",
  "?$10 * categorical[10, -1]]",
  "?=10 * categorical[10, -1]]",
  "?[10 * categorical[10, -1]$",
  "?[10 * categorical[10, -1]$",
  "categorical[10, -1]",
  "?categorical[10, -1]",
  "?[categorical[10, -1]]",
  "$categorical[10, -1]",
  "=categorical[10, -1]",
  "$[categorical[10, -1]]",
  "xxx[categorical[10, -1]]",
  "?$categorical[10, -1]]",
  "?=categorical[10, -1]]",
  "?[categorical[10, -1]$",
  "?[categorical[10, -1]$",
  "10 $ categorical[10, -1]",
  "?10 $ categorical[10, -1]",
  "?[10 $ categorical[10, -1]]",
  "$10 $ categorical[10, -1]",
  "=10 $ categorical[10, -1]",
  "$[10 $ categorical[10, -1]]",
  "xxx[10 $ categorical[10, -1]]",
  "?$10 $ categorical[10, -1]]",
  "?=10 $ categorical[10, -1]]",
  "?[10 $ categorical[10, -1]$",
  "?[10 $ categorical[10, -1]$",
  "10 = categorical[10, -1]",
  "?10 = categorical[10, -1]",
  "?[10 = categorical[10, -1]]",
  "$10 = categorical[10, -1]",
  "=10 = categorical[10, -1]",
  "$[10 = categorical[10, -1]]",
  "xxx[10 = categorical[10, -1]]",
  "?$10 = categorical[10, -1]]",
  "?=10 = categorical[10, -1]]",
  "?[10 = categorical[10, -1]$",
  "?[10 = categorical[10, -1]$",
  "10 * categorical[10, 10, -1]",
  "?10 * categorical[10, 10, -1]",
  "?[10 * categorical[10, 10, -1]]",
  "$10 * categorical[10, 10, -1]",
  "=10 * categorical[10, 10, -1]",
  "$[10 * categorical[10, 10, -1]]",
  "xxx[10 * categorical[10, 10, -1]]",
  "?$10 * categorical[10, 10, -1]]",
  "?=10 * categorical[10, 10, -1]]",
  "?[10 * categorical[10, 10, -1]$",
  "?[10 * categorical[10, 10, -1]$",
  "categorical[10, 10, -1]",
  "?categorical[10, 10, -1]",
  "?[categorical[10, 10, -1]]",
  "$categorical[10, 10, -1]",
  "=categorical[10, 10, -1]",
  "$[categorical[10, 10, -1]]",
  "xxx[categorical[10, 10, -1]]",
  "?$categorical[10, 10, -1]]",
  "?=categorical[10, 10, -1]]",
  "?[categorical[10, 10, -1]$",
  "?[categorical[10, 10, -1]$",
  "10 $ categorical[10, 10, -1]",
  "?10 $ categorical[10, 10, -1]",
  "?[10 $ categorical[10, 10, -1]]",
  "$10 $ categorical[10, 10, -1]",
  "=10 $ categorical[10, 10, -1]",
  "$[10 $ categorical[10, 10, -1]]",
  "xxx[10 $ categorical[10, 10, -1]]",
  "?$10 $ categorical[10, 10, -1]]",
  "?=10 $ categorical[10, 10, -1]]",
  "?[10 $ categorical[10, 10, -1]$",
  "?[10 $ categorical[10, 10, -1]$",
  "10 = categorical[10, 10, -1]",
  "?10 = categorical[10, 10, -1]",
  "?[10 = categorical[10, 10, -1]]",
  "$10 = categorical[10, 10, -1]",
  "=10 = categorical[10, 10, -1]",
  "$[10 = categorical[10, 10, -1]]",
  "xxx[10 = categorical[10, 10, -1]]",
  "?$10 = categorical[10, 10, -1]]",
  "?=10 = categorical[10, 10, -1]]",
  "?[10 = categorical[10, 10, -1]$",
  "?[10 = categorical[10, 10, -1]$",
  "10 * categorical[65536]",
  "?10 * categorical[65536]",
  "?[10 * categorical[65536]]",
  "$10 * categorical[65536]",
  "=10 * categorical[65536]",
  "$[10 * categorical[65536]]",
  "xxx[10 * categorical[65536]]",
  "?$10 * categorical[65536]]",
  "?=10 * categorical[65536]]",
  "?[10 * categorical[65536]$",
  "?[10 * categorical[65536]$",
  "categorical[65536]",
  "?categorical[65536]",
  "?[categorical[65536]]",
  "$categorical[65536]",
  "=categorical[65536]",
  "$[categorical[65536]]",
  "xxx[categorical[65536]]",
  "?$categorical[65536]]",
  "?=categorical[65536]]",
  "?[categorical[65536]$",
  "?[categorical[65536]$",
  "10 $ categorical[65536]",
  "?10 $ categorical[65536]",
  "?[10 $ categorical[65536]]",
  "$10 $ categorical[65536]",
  "=10 $ categorical[65536]",
  "$[10 $ categorical[65536]]",
  "xxx[10 $ categorical[65536]]",
  "?$10 $ categorical[65536]]",
  "?=10 $ categorical[65536]]",
  "?[10 $ categorical[65536]$",
  "?[10 $ categorical[65536]$",
  "10 = categorical[65536]",
  "?10 = categorical[65536]",
  "?[10 = categorical[65536]]",
  "$10 = categorical[65536]",
  "=10 = categorical[65536]",
  "$[10 = categorical[65536]]",
  "xxx[10 = categorical[65536]]",
  "?$10 = categorical[65536]]",
  "?=10 = categorical[65536]]",
  "?[10 = categorical[65536]$",
  "?[10 = categorical[65536]$",
  "10 * categorical[10, 65536]",
  "?10 * categorical[10, 65536]",
  "?[10 * categorical[10, 65536]]",
  "$10 * categorical[10, 65536]",
  "=10 * categorical[10, 65536]",
  "$[10 * categorical[10, 65536]]",
  "xxx[10 * categorical[10, 65536]]",
  "?$10 * categorical[10, 65536]]",
  "?=10 * categorical[10, 65536]]",
  "?[10 * categorical[10, 65536]$",
  "?[10 * categorical[10, 65536]$",
  "categorical[10, 65536]",
  "?categorical[10, 65536]",
  "?[categorical[10, 65536]]",
  "$categorical[10, 65536]",
  "=categorical[10, 65536]",
  "$[categorical[10, 65536]]",
  "xxx[categorical[10, 65536]]",
  "?$categorical[10, 65536]]",
  "?=categorical[10, 65536]]",
  "?[categorical[10, 65536]$",
  "?[categorical[10, 65536]$",
  "10 $ categorical[10, 65536]",
  "?10 $ categorical[10, 65536]",
  "?[10 $ categorical[10, 65536]]",
  "$10 $ categorical[10, 65536]",
  "=10 $ categorical[10, 65536]",
  "$[10 $ categorical[10, 65536]]",
  "xxx[10 $ categorical[10, 65536]]",
  "?$10 $ categorical[10, 65536]]",
  "?=10 $ categorical[10, 65536]]",
  "?[10 $ categorical[10, 65536]$",
  "?[10 $ categorical[10, 65536]$",
  "10 = categorical[10, 65536]",
  "?10 = categorical[10, 65536]",
  "?[10 = categorical[10, 65536]]",
  "$10 = categorical[10, 65536]",
  "=10 = categorical[10, 65536]",
  "$[10 = categorical[10, 65536]]",
  "xxx[10 = categorical[10, 65536]]",
  "?$10 = categorical[10, 65536]]",
  "?=10 = categorical[10, 65536]]",
  "?[10 = categorical[10, 65536]$",
  "?[10 = categorical[10, 65536]$",
  "10 * categorical[10, 10, 65536]",
  "?10 * categorical[10, 10, 65536]",
  "?[10 * categorical[10, 10, 65536]]",
  "$10 * categorical[10, 10, 65536]",
  "=10 * categorical[10, 10, 65536]",
  "$[10 * categorical[10, 10, 65536]]",
  "xxx[10 * categorical[10, 10, 65536]]",
  "?$10 * categorical[10, 10, 65536]]",
  "?=10 * categorical[10, 10, 65536]]",
  "?[10 * categorical[10, 10, 65536]$",
  "?[10 * categorical[10, 10, 65536]$",
  "categorical[10, 10, 65536]",
  "?categorical[10, 10, 65536]",
  "?[categorical[10, 10, 65536]]",
  "$categorical[10, 10, 65536]",
  "=categorical[10, 10, 65536]",
  "$[categorical[10, 10, 65536]]",
  "xxx[categorical[10, 10, 65536]]",
  "?$categorical[10, 10, 65536]]",
  "?=categorical[10, 10, 65536]]",
  "?[categorical[10, 10, 65536]$",
  "?[categorical[10, 10, 65536]$",
  "10 $ categorical[10, 10, 65536]",
  "?10 $ categorical[10, 10, 65536]",
  "?[10 $ categorical[10, 10, 65536]]",
  "$10 $ categorical[10, 10, 65536]",
  "=10 $ categorical[10, 10, 65536]",
  "$[10 $ categorical[10, 10, 65536]]",
  "xxx[10 $ categorical[10, 10, 65536]]",
  "?$10 $ categorical[10, 10, 65536]]",
  "?=10 $ categorical[10, 10, 65536]]",
  "?[10 $ categorical[10, 10, 65536]$",
  "?[10 $ categorical[10, 10, 65536]$",
  "10 = categorical[10, 10, 65536]",
  "?10 = categorical[10, 10, 65536]",
  "?[10 = categorical[10, 10, 65536]]",
  "$10 = categorical[10, 10, 65536]",
  "=10 = categorical[10, 10, 65536]",
  "$[10 = categorical[10, 10, 65536]]",
  "xxx[10 = categorical[10, 10, 65536]]",
  "?$10 = categorical[10, 10, 65536]]",
  "?=10 = categorical[10, 10, 65536]]",
  "?[10 = categorical[10, 10, 65536]$",
  "?[10 = categorical[10, 10, 65536]$",
  "10 * categorical[-1]",
  "?10 * categorical[-1]",
  "?[10 * categorical[-1]]",
  "$10 * categorical[-1]",
  "=10 * categorical[-1]",
  "$[10 * categorical[-1]]",
  "xxx[10 * categorical[-1]]",
  "?$10 * categorical[-1]]",
  "?=10 * categorical[-1]]",
  "?[10 * categorical[-1]$",
  "?[10 * categorical[-1]$",
  "categorical[-1]",
  "?categorical[-1]",
  "?[categorical[-1]]",
  "$categorical[-1]",
  "=categorical[-1]",
  "$[categorical[-1]]",
  "xxx[categorical[-1]]",
  "?$categorical[-1]]",
  "?=categorical[-1]]",
  "?[categorical[-1]$",
  "?[categorical[-1]$",
  "10 $ categorical[-1]",
  "?10 $ categorical[-1]",
  "?[10 $ categorical[-1]]",
  "$10 $ categorical[-1]",
  "=10 $ categorical[-1]",
  "$[10 $ categorical[-1]]",
  "xxx[10 $ categorical[-1]]",
  "?$10 $ categorical[-1]]",
  "?=10 $ categorical[-1]]",
  "?[10 $ categorical[-1]$",
  "?[10 $ categorical[-1]$",
  "10 = categorical[-1]",
  "?10 = categorical[-1]",
  "?[10 = categorical[-1]]",
  "$10 = categorical[-1]",
  "=10 = categorical[-1]",
  "$[10 = categorical[-1]]",
  "xxx[10 = categorical[-1]]",
  "?$10 = categorical[-1]]",
  "?=10 = categorical[-1]]",
  "?[10 = categorical[-1]$",
  "?[10 = categorical[-1]$",
  "10 * categorical[10, -1]",
  "?10 * categorical[10, -1]",
  "?[10 * categorical[10, -1]]",
  "$10 * categorical[10, -1]",
  "=10 * categorical[10, -1]",
  "$[10 * categorical[10, -1]]",
  "xxx[10 * categorical[10, -1]]",
  "?$10 * categorical[10, -1]]",
  "?=10 * categorical[10, -1]]",
  "?[10 * categorical[10, -1]$",
  "?[10 * categorical[10, -1]$",
  "categorical[10, -1]",
  "?categorical[10, -1]",
  "?[categorical[10, -1]]",
  "$categorical[10, -1]",
  "=categorical[10, -1]",
  "$[categorical[10, -1]]",
  "xxx[categorical[10, -1]]",
  "?$categorical[10, -1]]",
  "?=categorical[10, -1]]",
  "?[categorical[10, -1]$",
  "?[categorical[10, -1]$",
  "10 $ categorical[10, -1]",
  "?10 $ categorical[10, -1]",
  "?[10 $ categorical[10, -1]]",
  "$10 $ categorical[10, -1]",
  "=10 $ categorical[10, -1]",
  "$[10 $ categorical[10, -1]]",
  "xxx[10 $ categorical[10, -1]]",
  "?$10 $ categorical[10, -1]]",
  "?=10 $ categorical[10, -1]]",
  "?[10 $ categorical[10, -1]$",
  "?[10 $ categorical[10, -1]$",
  "10 = categorical[10, -1]",
  "?10 = categorical[10, -1]",
  "?[10 = categorical[10, -1]]",
  "$10 = categorical[10, -1]",
  "=10 = categorical[10, -1]",
  "$[10 = categorical[10, -1]]",
  "xxx[10 = categorical[10, -1]]",
  "?$10 = categorical[10, -1]]",
  "?=10 = categorical[10, -1]]",
  "?[10 = categorical[10, -1]$",
  "?[10 = categorical[10, -1]$",
  "10 * categorical[10, 10, -1]",
  "?10 * categorical[10, 10, -1]",
  "?[10 * categorical[10, 10, -1]]",
  "$10 * categorical[10, 10, -1]",
  "=10 * categorical[10, 10, -1]",
  "$[10 * categorical[10, 10, -1]]",
  "xxx[10 * categorical[10, 10, -1]]",
  "?$10 * categorical[10, 10, -1]]",
  "?=10 * categorical[10, 10, -1]]",
  "?[10 * categorical[10, 10, -1]$",
  "?[10 * categorical[10, 10, -1]$",
  "categorical[10, 10, -1]",
  "?categorical[10, 10, -1]",
  "?[categorical[10, 10, -1]]",
  "$categorical[10, 10, -1]",
  "=categorical[10, 10, -1]",
  "$[categorical[10, 10, -1]]",
  "xxx[categorical[10, 10, -1]]",
  "?$categorical[10, 10, -1]]",
  "?=categorical[10, 10, -1]]",
  "?[categorical[10, 10, -1]$",
  "?[categorical[10, 10, -1]$",
  "10 $ categorical[10, 10, -1]",
  "?10 $ categorical[10, 10, -1]",
  "?[10 $ categorical[10, 10, -1]]",
  "$10 $ categorical[10, 10, -1]",
  "=10 $ categorical[10, 10, -1]",
  "$[10 $ categorical[10, 10, -1]]",
  "xxx[10 $ categorical[10, 10, -1]]",
  "?$10 $ categorical[10, 10, -1]]",
  "?=10 $ categorical[10, 10, -1]]",
  "?[10 $ categorical[10, 10, -1]$",
  "?[10 $ categorical[10, 10, -1]$",
  "10 = categorical[10, 10, -1]",
  "?10 = categorical[10, 10, -1]",
  "?[10 = categorical[10, 10, -1]]",
  "$10 = categorical[10, 10, -1]",
  "=10 = categorical[10, 10, -1]",
  "$[10 = categorical[10, 10, -1]]",
  "xxx[10 = categorical[10, 10, -1]]",
  "?$10 = categorical[10, 10, -1]]",
  "?=10 = categorical[10, 10, -1]]",
  "?[10 = categorical[10, 10, -1]$",
  "?[10 = categorical[10, 10, -1]$",
  "10 * categorical[4294967296]",
  "?10 * categorical[4294967296]",
  "?[10 * categorical[4294967296]]",
  "$10 * categorical[4294967296]",
  "=10 * categorical[4294967296]",
  "$[10 * categorical[4294967296]]",
  "xxx[10 * categorical[4294967296]]",
  "?$10 * categorical[4294967296]]",
  "?=10 * categorical[4294967296]]",
  "?[10 * categorical[4294967296]$",
  "?[10 * categorical[4294967296]$",
  "categorical[4294967296]",
  "?categorical[4294967296]",
  "?[categorical[4294967296]]",
  "$categorical[4294967296]",
  "=categorical[4294967296]",
  "$[categorical[4294967296]]",
  "xxx[categorical[4294967296]]",
  "?$categorical[4294967296]]",
  "?=categorical[4294967296]]",
  "?[categorical[4294967296]$",
  "?[categorical[4294967296]$",
  "10 $ categorical[4294967296]",
  "?10 $ categorical[4294967296]",
  "?[10 $ categorical[4294967296]]",
  "$10 $ categorical[4294967296]",
  "=10 $ categorical[4294967296]",
  "$[10 $ categorical[4294967296]]",
  "xxx[10 $ categorical[4294967296]]",
  "?$10 $ categorical[4294967296]]",
  "?=10 $ categorical[4294967296]]",
  "?[10 $ categorical[4294967296]$",
  "?[10 $ categorical[4294967296]$",
  "10 = categorical[4294967296]",
  "?10 = categorical[4294967296]",
  "?[10 = categorical[4294967296]]",
  "$10 = categorical[4294967296]",
  "=10 = categorical[4294967296]",
  "$[10 = categorical[4294967296]]",
  "xxx[10 = categorical[4294967296]]",
  "?$10 = categorical[4294967296]]",
  "?=10 = categorical[4294967296]]",
  "?[10 = categorical[4294967296]$",
  "?[10 = categorical[4294967296]$",
  "10 * categorical[10, 4294967296]",
  "?10 * categorical[10, 4294967296]",
  "?[10 * categorical[10, 4294967296]]",
  "$10 * categorical[10, 4294967296]",
  "=10 * categorical[10, 4294967296]",
  "$[10 * categorical[10, 4294967296]]",
  "xxx[10 * categorical[10, 4294967296]]",
  "?$10 * categorical[10, 4294967296]]",
  "?=10 * categorical[10, 4294967296]]",
  "?[10 * categorical[10, 4294967296]$",
  "?[10 * categorical[10, 4294967296]$",
  "categorical[10, 4294967296]",
  "?categorical[10, 4294967296]",
  "?[categorical[10, 4294967296]]",
  "$categorical[10, 4294967296]",
  "=categorical[10, 4294967296]",
  "$[categorical[10, 4294967296]]",
  "xxx[categorical[10, 4294967296]]",
  "?$categorical[10, 4294967296]]",
  "?=categorical[10, 4294967296]]",
  "?[categorical[10, 4294967296]$",
  "?[categorical[10, 4294967296]$",
  "10 $ categorical[10, 4294967296]",
  "?10 $ categorical[10, 4294967296]",
  "?[10 $ categorical[10, 4294967296]]",
  "$10 $ categorical[10, 4294967296]",
  "=10 $ categorical[10, 4294967296]",
  "$[10 $ categorical[10, 4294967296]]",
  "xxx[10 $ categorical[10, 4294967296]]",
  "?$10 $ categorical[10, 4294967296]]",
  "?=10 $ categorical[10, 4294967296]]",
  "?[10 $ categorical[10, 4294967296]$",
  "?[10 $ categorical[10, 4294967296]$",
  "10 = categorical[10, 4294967296]",
  "?10 = categorical[10, 4294967296]",
  "?[10 = categorical[10, 4294967296]]",
  "$10 = categorical[10, 4294967296]",
  "=10 = categorical[10, 4294967296]",
  "$[10 = categorical[10, 4294967296]]",
  "xxx[10 = categorical[10, 4294967296]]",
  "?$10 = categorical[10, 4294967296]]",
  "?=10 = categorical[10, 4294967296]]",
  "?[10 = categorical[10, 4294967296]$",
  "?[10 = categorical[10, 4294967296]$",
  "10 * categorical[10, 10, 4294967296]",
  "?10 * categorical[10, 10, 4294967296]",
  "?[10 * categorical[10, 10, 4294967296]]",
  "$10 * categorical[10, 10, 4294967296]",
  "=10 * categorical[10, 10, 4294967296]",
  "$[10 * categorical[10, 10, 4294967296]]",
  "xxx[10 * categorical[10, 10, 4294967296]]",
  "?$10 * categorical[10, 10, 4294967296]]",
  "?=10 * categorical[10, 10, 4294967296]]",
  "?[10 * categorical[10, 10, 4294967296]$",
  "?[10 * categorical[10, 10, 4294967296]$",
  "categorical[10, 10, 4294967296]",
  "?categorical[10, 10, 4294967296]",
  "?[categorical[10, 10, 4294967296]]",
  "$categorical[10, 10, 4294967296]",
  "=categorical[10, 10, 4294967296]",
  "$[categorical[10, 10, 4294967296]]",
  "xxx[categorical[10, 10, 4294967296]]",
  "?$categorical[10, 10, 4294967296]]",
  "?=categorical[10, 10, 4294967296]]",
  "?[categorical[10, 10, 4294967296]$",
  "?[categorical[10, 10, 4294967296]$",
  "10 $ categorical[10, 10, 4294967296]",
  "?10 $ categorical[10, 10, 4294967296]",
  "?[10 $ categorical[10, 10, 4294967296]]",
  "$10 $ categorical[10, 10, 4294967296]",
  "=10 $ categorical[10, 10, 4294967296]",
  "$[10 $ categorical[10, 10, 4294967296]]",
  "xxx[10 $ categorical[10, 10, 4294967296]]",
  "?$10 $ categorical[10, 10, 4294967296]]",
  "?=10 $ categorical[10, 10, 4294967296]]",
  "?[10 $ categorical[10, 10, 4294967296]$",
  "?[10 $ categorical[10, 10, 4294967296]$",
  "10 = categorical[10, 10, 4294967296]",
  "?10 = categorical[10, 10, 4294967296]",
  "?[10 = categorical[10, 10, 4294967296]]",
  "$10 = categorical[10, 10, 4294967296]",
  "=10 = categorical[10, 10, 4294967296]",
  "$[10 = categorical[10, 10, 4294967296]]",
  "xxx[10 = categorical[10, 10, 4294967296]]",
  "?$10 = categorical[10, 10, 4294967296]]",
  "?=10 = categorical[10, 10, 4294967296]]",
  "?[10 = categorical[10, 10, 4294967296]$",
  "?[10 = categorical[10, 10, 4294967296]$",
  "10 * categorical[-1]",
  "?10 * categorical[-1]",
  "?[10 * categorical[-1]]",
  "$10 * categorical[-1]",
  "=10 * categorical[-1]",
  "$[10 * categorical[-1]]",
  "xxx[10 * categorical[-1]]",
  "?$10 * categorical[-1]]",
  "?=10 * categorical[-1]]",
  "?[10 * categorical[-1]$",
  "?[10 * categorical[-1]$",
  "categorical[-1]",
  "?categorical[-1]",
  "?[categorical[-1]]",
  "$categorical[-1]",
  "=categorical[-1]",
  "$[categorical[-1]]",
  "xxx[categorical[-1]]",
  "?$categorical[-1]]",
  "?=categorical[-1]]",
  "?[categorical[-1]$",
  "?[categorical[-1]$",
  "10 $ categorical[-1]",
  "?10 $ categorical[-1]",
  "?[10 $ categorical[-1]]",
  "$10 $ categorical[-1]",
  "=10 $ categorical[-1]",
  "$[10 $ categorical[-1]]",
  "xxx[10 $ categorical[-1]]",
  "?$10 $ categorical[-1]]",
  "?=10 $ categorical[-1]]",
  "?[10 $ categorical[-1]$",
  "?[10 $ categorical[-1]$",
  "10 = categorical[-1]",
  "?10 = categorical[-1]",
  "?[10 = categorical[-1]]",
  "$10 = categorical[-1]",
  "=10 = categorical[-1]",
  "$[10 = categorical[-1]]",
  "xxx[10 = categorical[-1]]",
  "?$10 = categorical[-1]]",
  "?=10 = categorical[-1]]",
  "?[10 = categorical[-1]$",
  "?[10 = categorical[-1]$",
  "10 * categorical[10, -1]",
  "?10 * categorical[10, -1]",
  "?[10 * categorical[10, -1]]",
  "$10 * categorical[10, -1]",
  "=10 * categorical[10, -1]",
  "$[10 * categorical[10, -1]]",
  "xxx[10 * categorical[10, -1]]",
  "?$10 * categorical[10, -1]]",
  "?=10 * categorical[10, -1]]",
  "?[10 * categorical[10, -1]$",
  "?[10 * categorical[10, -1]$",
  "categorical[10, -1]",
  "?categorical[10, -1]",
  "?[categorical[10, -1]]",
  "$categorical[10, -1]",
  "=categorical[10, -1]",
  "$[categorical[10, -1]]",
  "xxx[categorical[10, -1]]",
  "?$categorical[10, -1]]",
  "?=categorical[10, -1]]",
  "?[categorical[10, -1]$",
  "?[categorical[10, -1]$",
  "10 $ categorical[10, -1]",
  "?10 $ categorical[10, -1]",
  "?[10 $ categorical[10, -1]]",
  "$10 $ categorical[10, -1]",
  "=10 $ categorical[10, -1]",
  "$[10 $ categorical[10, -1]]",
  "xxx[10 $ categorical[10, -1]]",
  "?$10 $ categorical[10, -1]]",
  "?=10 $ categorical[10, -1]]",
  "?[10 $ categorical[10, -1]$",
  "?[10 $ categorical[10, -1]$",
  "10 = categorical[10, -1]",
  "?10 = categorical[10, -1]",
  "?[10 = categorical[10, -1]]",
  "$10 = categorical[10, -1]",
  "=10 = categorical[10, -1]",
  "$[10 = categorical[10, -1]]",
  "xxx[10 = categorical[10, -1]]",
  "?$10 = categorical[10, -1]]",
  "?=10 = categorical[10, -1]]",
  "?[10 = categorical[10, -1]$",
  "?[10 = categorical[10, -1]$",
  "10 * categorical[10, 10, -1]",
  "?10 * categorical[10, 10, -1]",
  "?[10 * categorical[10, 10, -1]]",
  "$10 * categorical[10, 10, -1]",
  "=10 * categorical[10, 10, -1]",
  "$[10 * categorical[10, 10, -1]]",
  "xxx[10 * categorical[10, 10, -1]]",
  "?$10 * categorical[10, 10, -1]]",
  "?=10 * categorical[10, 10, -1]]",
  "?[10 * categorical[10, 10, -1]$",
  "?[10 * categorical[10, 10, -1]$",
  "categorical[10, 10, -1]",
  "?categorical[10, 10, -1]",
  "?[categorical[10, 10, -1]]",
  "$categorical[10, 10, -1]",
  "=categorical[10, 10, -1]",
  "$[categorical[10, 10, -1]]",
  "xxx[categorical[10, 10, -1]]",
  "?$categorical[10, 10, -1]]",
  "?=categorical[10, 10, -1]]",
  "?[categorical[10, 10, -1]$",
  "?[categorical[10, 10, -1]$",
  "10 $ categorical[10, 10, -1]",
  "?10 $ categorical[10, 10, -1]",
  "?[10 $ categorical[10, 10, -1]]",
  "$10 $ categorical[10, 10, -1]",
  "=10 $ categorical[10, 10, -1]",
  "$[10 $ categorical[10, 10, -1]]",
  "xxx[10 $ categorical[10, 10, -1]]",
  "?$10 $ categorical[10, 10, -1]]",
  "?=10 $ categorical[10, 10, -1]]",
  "?[10 $ categorical[10, 10, -1]$",
  "?[10 $ categorical[10, 10, -1]$",
  "10 = categorical[10, 10, -1]",
  "?10 = categorical[10, 10, -1]",
  "?[10 = categorical[10, 10, -1]]",
  "$10 = categorical[10, 10, -1]",
  "=10 = categorical[10, 10, -1]",
  "$[10 = categorical[10, 10, -1]]",
  "xxx[10 = categorical[10, 10, -1]]",
  "?$10 = categorical[10, 10, -1]]",
  "?=10 = categorical[10, 10, -1]]",
  "?[10 = categorical[10, 10, -1]$",
  "?[10 = categorical[10, 10, -1]$",
  "10 * categorical[18446744073709551616]",
  "?10 * categorical[18446744073709551616]",
  "?[10 * categorical[18446744073709551616]]",
  "$10 * categorical[18446744073709551616]",
  "=10 * categorical[18446744073709551616]",
  "$[10 * categorical[18446744073709551616]]",
  "xxx[10 * categorical[18446744073709551616]]",
  "?$10 * categorical[18446744073709551616]]",
  "?=10 * categorical[18446744073709551616]]",
  "?[10 * categorical[18446744073709551616]$",
  "?[10 * categorical[18446744073709551616]$",
  "categorical[18446744073709551616]",
  "?categorical[18446744073709551616]",
  "?[categorical[18446744073709551616]]",
  "$categorical[18446744073709551616]",
  "=categorical[18446744073709551616]",
  "$[categorical[18446744073709551616]]",
  "xxx[categorical[18446744073709551616]]",
  "?$categorical[18446744073709551616]]",
  "?=categorical[18446744073709551616]]",
  "?[categorical[18446744073709551616]$",
  "?[categorical[18446744073709551616]$",
  "10 $ categorical[18446744073709551616]",
  "?10 $ categorical[18446744073709551616]",
  "?[10 $ categorical[18446744073709551616]]",
  "$10 $ categorical[18446744073709551616]",
  "=10 $ categorical[18446744073709551616]",
  "$[10 $ categorical[18446744073709551616]]",
  "xxx[10 $ categorical[18446744073709551616]]",
  "?$10 $ categorical[18446744073709551616]]",
  "?=10 $ categorical[18446744073709551616]]",
  "?[10 $ categorical[18446744073709551616]$",
  "?[10 $ categorical[18446744073709551616]$",
  "10 = categorical[18446744073709551616]",
  "?10 = categorical[18446744073709551616]",
  "?[10 = categorical[18446744073709551616]]",
  "$10 = categorical[18446744073709551616]",
  "=10 = categorical[18446744073709551616]",
  "$[10 = categorical[18446744073709551616]]",
  "xxx[10 = categorical[18446744073709551616]]",
  "?$10 = categorical[18446744073709551616]]",
  "?=10 = categorical[18446744073709551616]]",
  "?[10 = categorical[18446744073709551616]$",
  "?[10 = categorical[18446744073709551616]$",
  "10 * categorical[10, 18446744073709551616]",
  "?10 * categorical[10, 18446744073709551616]",
  "?[10 * categorical[10, 18446744073709551616]]",
  "$10 * categorical[10, 18446744073709551616]",
  "=10 * categorical[10, 18446744073709551616]",
  "$[10 * categorical[10, 18446744073709551616]]",
  "xxx[10 * categorical[10, 18446744073709551616]]",
  "?$10 * categorical[10, 18446744073709551616]]",
  "?=10 * categorical[10, 18446744073709551616]]",
  "?[10 * categorical[10, 18446744073709551616]$",
  "?[10 * categorical[10, 18446744073709551616]$",
  "categorical[10, 18446744073709551616]",
  "?categorical[10, 18446744073709551616]",
  "?[categorical[10, 18446744073709551616]]",
  "$categorical[10, 18446744073709551616]",
  "=categorical[10, 18446744073709551616]",
  "$[categorical[10, 18446744073709551616]]",
  "xxx[categorical[10, 18446744073709551616]]",
  "?$categorical[10, 18446744073709551616]]",
  "?=categorical[10, 18446744073709551616]]",
  "?[categorical[10, 18446744073709551616]$",
  "?[categorical[10, 18446744073709551616]$",
  "10 $ categorical[10, 18446744073709551616]",
  "?10 $ categorical[10, 18446744073709551616]",
  "?[10 $ categorical[10, 18446744073709551616]]",
  "$10 $ categorical[10, 18446744073709551616]",
  "=10 $ categorical[10, 18446744073709551616]",
  "$[10 $ categorical[10, 18446744073709551616]]",
  "xxx[10 $ categorical[10, 18446744073709551616]]",
  "?$10 $ categorical[10, 18446744073709551616]]",
  "?=10 $ categorical[10, 18446744073709551616]]",
  "?[10 $ categorical[10, 18446744073709551616]$",
  "?[10 $ categorical[10, 18446744073709551616]$",
  "10 = categorical[10, 18446744073709551616]",
  "?10 = categorical[10, 18446744073709551616]",
  "?[10 = categorical[10, 18446744073709551616]]",
  "$10 = categorical[10, 18446744073709551616]",
  "=10 = categorical[10, 18446744073709551616]",
  "$[10 = categorical[10, 18446744073709551616]]",
  "xxx[10 = categorical[10, 18446744073709551616]]",
  "?$10 = categorical[10, 18446744073709551616]]",
  "?=10 = categorical[10, 18446744073709551616]]",
  "?[10 = categorical[10, 18446744073709551616]$",
  "?[10 = categorical[10, 18446744073709551616]$",
  "10 * categorical[10, 10, 18446744073709551616]",
  "?10 * categorical[10, 10, 18446744073709551616]",
  "?[10 * categorical[10, 10, 18446744073709551616]]",
  "$10 * categorical[10, 10, 18446744073709551616]",
  "=10 * categorical[10, 10, 18446744073709551616]",
  "$[10 * categorical[10, 10, 18446744073709551616]]",
  "xxx[10 * categorical[10, 10, 18446744073709551616]]",
  "?$10 * categorical[10, 10, 18446744073709551616]]",
  "?=10 * categorical[10, 10, 18446744073709551616]]",
  "?[10 * categorical[10, 10, 18446744073709551616]$",
  "?[10 * categorical[10, 10, 18446744073709551616]$",
  "categorical[10, 10, 18446744073709551616]",
  "?categorical[10, 10, 18446744073709551616]",
  "?[categorical[10, 10, 18446744073709551616]]",
  "$categorical[10, 10, 18446744073709551616]",
  "=categorical[10, 10, 18446744073709551616]",
  "$[categorical[10, 10, 18446744073709551616]]",
  "xxx[categorical[10, 10, 18446744073709551616]]",
  "?$categorical[10, 10, 18446744073709551616]]",
  "?=categorical[10, 10, 18446744073709551616]]",
  "?[categorical[10, 10, 18446744073709551616]$",
  "?[categorical[10, 10, 18446744073709551616]$",
  "10 $ categorical[10, 10, 18446744073709551616]",
  "?10 $ categorical[10, 10, 18446744073709551616]",
  "?[10 $ categorical[10, 10, 18446744073709551616]]",
  "$10 $ categorical[10, 10, 18446744073709551616]",
  "=10 $ categorical[10, 10, 18446744073709551616]",
  "$[10 $ categorical[10, 10, 18446744073709551616]]",
  "xxx[10 $ categorical[10, 10, 18446744073709551616]]",
  "?$10 $ categorical[10, 10, 18446744073709551616]]",
  "?=10 $ categorical[10, 10, 18446744073709551616]]",
  "?[10 $ categorical[10, 10, 18446744073709551616]$",
  "?[10 $ categorical[10, 10, 18446744073709551616]$",
  "10 = categorical[10, 10, 18446744073709551616]",
  "?10 = categorical[10, 10, 18446744073709551616]",
  "?[10 = categorical[10, 10, 18446744073709551616]]",
  "$10 = categorical[10, 10, 18446744073709551616]",
  "=10 = categorical[10, 10, 18446744073709551616]",
  "$[10 = categorical[10, 10, 18446744073709551616]]",
  "xxx[10 = categorical[10, 10, 18446744073709551616]]",
  "?$10 = categorical[10, 10, 18446744073709551616]]",
  "?=10 = categorical[10, 10, 18446744073709551616]]",
  "?[10 = categorical[10, 10, 18446744073709551616]$",
  "?[10 = categorical[10, 10, 18446744073709551616]$",
  "10 * categorical[-1]",
  "?10 * categorical[-1]",
  "?[10 * categorical[-1]]",
  "$10 * categorical[-1]",
  "=10 * categorical[-1]",
  "$[10 * categorical[-1]]",
  "xxx[10 * categorical[-1]]",
  "?$10 * categorical[-1]]",
  "?=10 * categorical[-1]]",
  "?[10 * categorical[-1]$",
  "?[10 * categorical[-1]$",
  "categorical[-1]",
  "?categorical[-1]",
  "?[categorical[-1]]",
  "$categorical[-1]",
  "=categorical[-1]",
  "$[categorical[-1]]",
  "xxx[categorical[-1]]",
  "?$categorical[-1]]",
  "?=categorical[-1]]",
  "?[categorical[-1]$",
  "?[categorical[-1]$",
  "10 $ categorical[-1]",
  "?10 $ categorical[-1]",
  "?[10 $ categorical[-1]]",
  "$10 $ categorical[-1]",
  "=10 $ categorical[-1]",
  "$[10 $ categorical[-1]]",
  "xxx[10 $ categorical[-1]]",
  "?$10 $ categorical[-1]]",
  "?=10 $ categorical[-1]]",
  "?[10 $ categorical[-1]$",
  "?[10 $ categorical[-1]$",
  "10 = categorical[-1]",
  "?10 = categorical[-1]",
  "?[10 = categorical[-1]]",
  "$10 = categorical[-1]",
  "=10 = categorical[-1]",
  "$[10 = categorical[-1]]",
  "xxx[10 = categorical[-1]]",
  "?$10 = categorical[-1]]",
  "?=10 = categorical[-1]]",
  "?[10 = categorical[-1]$",
  "?[10 = categorical[-1]$",
  "10 * categorical[10, -1]",
  "?10 * categorical[10, -1]",
  "?[10 * categorical[10, -1]]",
  "$10 * categorical[10, -1]",
  "=10 * categorical[10, -1]",
  "$[10 * categorical[10, -1]]",
  "xxx[10 * categorical[10, -1]]",
  "?$10 * categorical[10, -1]]",
  "?=10 * categorical[10, -1]]",
  "?[10 * categorical[10, -1]$",
  "?[10 * categorical[10, -1]$",
  "categorical[10, -1]",
  "?categorical[10, -1]",
  "?[categorical[10, -1]]",
  "$categorical[10, -1]",
  "=categorical[10, -1]",
  "$[categorical[10, -1]]",
  "xxx[categorical[10, -1]]",
  "?$categorical[10, -1]]",
  "?=categorical[10, -1]]",
  "?[categorical[10, -1]$",
  "?[categorical[10, -1]$",
  "10 $ categorical[10, -1]",
  "?10 $ categorical[10, -1]",
  "?[10 $ categorical[10, -1]]",
  "$10 $ categorical[10, -1]",
  "=10 $ categorical[10, -1]",
  "$[10 $ categorical[10, -1]]",
  "xxx[10 $ categorical[10, -1]]",
  "?$10 $ categorical[10, -1]]",
  "?=10 $ categorical[10, -1]]",
  "?[10 $ categorical[10, -1]$",
  "?[10 $ categorical[10, -1]$",
  "10 = categorical[10, -1]",
  "?10 = categorical[10, -1]",
  "?[10 = categorical[10, -1]]",
  "$10 = categorical[10, -1]",
  "=10 = categorical[10, -1]",
  "$[10 = categorical[10, -1]]",
  "xxx[10 = categorical[10, -1]]",
  "?$10 = categorical[10, -1]]",
  "?=10 = categorical[10, -1]]",
  "?[10 = categorical[10, -1]$",
  "?[10 = categorical[10, -1]$",
  "10 * categorical[10, 10, -1]",
  "?10 * categorical[10, 10, -1]",
  "?[10 * categorical[10, 10, -1]]",
  "$10 * categorical[10, 10, -1]",
  "=10 * categorical[10, 10, -1]",
  "$[10 * categorical[10, 10, -1]]",
  "xxx[10 * categorical[10, 10, -1]]",
  "?$10 * categorical[10, 10, -1]]",
  "?=10 * categorical[10, 10, -1]]",
  "?[10 * categorical[10, 10, -1]$",
  "?[10 * categorical[10, 10, -1]$",
  "categorical[10, 10, -1]",
  "?categorical[10, 10, -1]",
  "?[categorical[10, 10, -1]]",
  "$categorical[10, 10, -1]",
  "=categorical[10, 10, -1]",
  "$[categorical[10, 10, -1]]",
  "xxx[categorical[10, 10, -1]]",
  "?$categorical[10, 10, -1]]",
  "?=categorical[10, 10, -1]]",
  "?[categorical[10, 10, -1]$",
  "?[categorical[10, 10, -1]$",
  "10 $ categorical[10, 10, -1]",
  "?10 $ categorical[10, 10, -1]",
  "?[10 $ categorical[10, 10, -1]]",
  "$10 $ categorical[10, 10, -1]",
  "=10 $ categorical[10, 10, -1]",
  "$[10 $ categorical[10, 10, -1]]",
  "xxx[10 $ categorical[10, 10, -1]]",
  "?$10 $ categorical[10, 10, -1]]",
  "?=10 $ categorical[10, 10, -1]]",
  "?[10 $ categorical[10, 10, -1]$",
  "?[10 $ categorical[10, 10, -1]$",
  "10 = categorical[10, 10, -1]",
  "?10 = categorical[10, 10, -1]",
  "?[10 = categorical[10, 10, -1]]",
  "$10 = categorical[10, 10, -1]",
  "=10 = categorical[10, 10, -1]",
  "$[10 = categorical[10, 10, -1]]",
  "xxx[10 = categorical[10, 10, -1]]",
  "?$10 = categorical[10, 10, -1]]",
  "?=10 = categorical[10, 10, -1]]",
  "?[10 = categorical[10, 10, -1]$",
  "?[10 = categorical[10, 10, -1]$",
  "10 * categorical[inf]",
  "?10 * categorical[inf]",
  "?[10 * categorical[inf]]",
  "$10 * categorical[inf]",
  "=10 * categorical[inf]",
  "$[10 * categorical[inf]]",
  "xxx[10 * categorical[inf]]",
  "?$10 * categorical[inf]]",
  "?=10 * categorical[inf]]",
  "?[10 * categorical[inf]$",
  "?[10 * categorical[inf]$",
  "categorical[inf]",
  "?categorical[inf]",
  "?[categorical[inf]]",
  "$categorical[inf]",
  "=categorical[inf]",
  "$[categorical[inf]]",
  "xxx[categorical[inf]]",
  "?$categorical[inf]]",
  "?=categorical[inf]]",
  "?[categorical[inf]$",
  "?[categorical[inf]$",
  "10 $ categorical[inf]",
  "?10 $ categorical[inf]",
  "?[10 $ categorical[inf]]",
  "$10 $ categorical[inf]",
  "=10 $ categorical[inf]",
  "$[10 $ categorical[inf]]",
  "xxx[10 $ categorical[inf]]",
  "?$10 $ categorical[inf]]",
  "?=10 $ categorical[inf]]",
  "?[10 $ categorical[inf]$",
  "?[10 $ categorical[inf]$",
  "10 = categorical[inf]",
  "?10 = categorical[inf]",
  "?[10 = categorical[inf]]",
  "$10 = categorical[inf]",
  "=10 = categorical[inf]",
  "$[10 = categorical[inf]]",
  "xxx[10 = categorical[inf]]",
  "?$10 = categorical[inf]]",
  "?=10 = categorical[inf]]",
  "?[10 = categorical[inf]$",
  "?[10 = categorical[inf]$",
  "10 * categorical[10, inf]",
  "?10 * categorical[10, inf]",
  "?[10 * categorical[10, inf]]",
  "$10 * categorical[10, inf]",
  "=10 * categorical[10, inf]",
  "$[10 * categorical[10, inf]]",
  "xxx[10 * categorical[10, inf]]",
  "?$10 * categorical[10, inf]]",
  "?=10 * categorical[10, inf]]",
  "?[10 * categorical[10, inf]$",
  "?[10 * categorical[10, inf]$",
  "categorical[10, inf]",
  "?categorical[10, inf]",
  "?[categorical[10, inf]]",
  "$categorical[10, inf]",
  "=categorical[10, inf]",
  "$[categorical[10, inf]]",
  "xxx[categorical[10, inf]]",
  "?$categorical[10, inf]]",
  "?=categorical[10, inf]]",
  "?[categorical[10, inf]$",
  "?[categorical[10, inf]$",
  "10 $ categorical[10, inf]",
  "?10 $ categorical[10, inf]",
  "?[10 $ categorical[10, inf]]",
  "$10 $ categorical[10, inf]",
  "=10 $ categorical[10, inf]",
  "$[10 $ categorical[10, inf]]",
  "xxx[10 $ categorical[10, inf]]",
  "?$10 $ categorical[10, inf]]",
  "?=10 $ categorical[10, inf]]",
  "?[10 $ categorical[10, inf]$",
  "?[10 $ categorical[10, inf]$",
  "10 = categorical[10, inf]",
  "?10 = categorical[10, inf]",
  "?[10 = categorical[10, inf]]",
  "$10 = categorical[10, inf]",
  "=10 = categorical[10, inf]",
  "$[10 = categorical[10, inf]]",
  "xxx[10 = categorical[10, inf]]",
  "?$10 = categorical[10, inf]]",
  "?=10 = categorical[10, inf]]",
  "?[10 = categorical[10, inf]$",
  "?[10 = categorical[10, inf]$",
  "10 * categorical[10, 10, inf]",
  "?10 * categorical[10, 10, inf]",
  "?[10 * categorical[10, 10, inf]]",
  "$10 * categorical[10, 10, inf]",
  "=10 * categorical[10, 10, inf]",
  "$[10 * categorical[10, 10, inf]]",
  "xxx[10 * categorical[10, 10, inf]]",
  "?$10 * categorical[10, 10, inf]]",
  "?=10 * categorical[10, 10, inf]]",
  "?[10 * categorical[10, 10, inf]$",
  "?[10 * categorical[10, 10, inf]$",
  "categorical[10, 10, inf]",
  "?categorical[10, 10, inf]",
  "?[categorical[10, 10, inf]]",
  "$categorical[10, 10, inf]",
  "=categorical[10, 10, inf]",
  "$[categorical[10, 10, inf]]",
  "xxx[categorical[10, 10, inf]]",
  "?$categorical[10, 10, inf]]",
  "?=categorical[10, 10, inf]]",
  "?[categorical[10, 10, inf]$",
  "?[categorical[10, 10, inf]$",
  "10 $ categorical[10, 10, inf]",
  "?10 $ categorical[10, 10, inf]",
  "?[10 $ categorical[10, 10, inf]]",
  "$10 $ categorical[10, 10, inf]",
  "=10 $ categorical[10, 10, inf]",
  "$[10 $ categorical[10, 10, inf]]",
  "xxx[10 $ categorical[10, 10, inf]]",
  "?$10 $ categorical[10, 10, inf]]",
  "?=10 $ categorical[10, 10, inf]]",
  "?[10 $ categorical[10, 10, inf]$",
  "?[10 $ categorical[10, 10, inf]$",
  "10 = categorical[10, 10, inf]",
  "?10 = categorical[10, 10, inf]",
  "?[10 = categorical[10, 10, inf]]",
  "$10 = categorical[10, 10, inf]",
  "=10 = categorical[10, 10, inf]",
  "$[10 = categorical[10, 10, inf]]",
  "xxx[10 = categorical[10, 10, inf]]",
  "?$10 = categorical[10, 10, inf]]",
  "?=10 = categorical[10, 10, inf]]",
  "?[10 = categorical[10, 10, inf]$",
  "?[10 = categorical[10, 10, inf]$",
  "10 * categorical[inf]",
  "?10 * categorical[inf]",
  "?[10 * categorical[inf]]",
  "$10 * categorical[inf]",
  "=10 * categorical[inf]",
  "$[10 * categorical[inf]]",
  "xxx[10 * categorical[inf]]",
  "?$10 * categorical[inf]]",
  "?=10 * categorical[inf]]",
  "?[10 * categorical[inf]$",
  "?[10 * categorical[inf]$",
  "categorical[inf]",
  "?categorical[inf]",
  "?[categorical[inf]]",
  "$categorical[inf]",
  "=categorical[inf]",
  "$[categorical[inf]]",
  "xxx[categorical[inf]]",
  "?$categorical[inf]]",
  "?=categorical[inf]]",
  "?[categorical[inf]$",
  "?[categorical[inf]$",
  "10 $ categorical[inf]",
  "?10 $ categorical[inf]",
  "?[10 $ categorical[inf]]",
  "$10 $ categorical[inf]",
  "=10 $ categorical[inf]",
  "$[10 $ categorical[inf]]",
  "xxx[10 $ categorical[inf]]",
  "?$10 $ categorical[inf]]",
  "?=10 $ categorical[inf]]",
  "?[10 $ categorical[inf]$",
  "?[10 $ categorical[inf]$",
  "10 = categorical[inf]",
  "?10 = categorical[inf]",
  "?[10 = categorical[inf]]",
  "$10 = categorical[inf]",
  "=10 = categorical[inf]",
  "$[10 = categorical[inf]]",
  "xxx[10 = categorical[inf]]",
  "?$10 = categorical[inf]]",
  "?=10 = categorical[inf]]",
  "?[10 = categorical[inf]$",
  "?[10 = categorical[inf]$",
  "10 * categorical[10, inf]",
  "?10 * categorical[10, inf]",
  "?[10 * categorical[10, inf]]",
  "$10 * categorical[10, inf]",
  "=10 * categorical[10, inf]",
  "$[10 * categorical[10, inf]]",
  "xxx[10 * categorical[10, inf]]",
  "?$10 * categorical[10, inf]]",
  "?=10 * categorical[10, inf]]",
  "?[10 * categorical[10, inf]$",
  "?[10 * categorical[10, inf]$",
  "categorical[10, inf]",
  "?categorical[10, inf]",
  "?[categorical[10, inf]]",
  "$categorical[10, inf]",
  "=categorical[10, inf]",
  "$[categorical[10, inf]]",
  "xxx[categorical[10, inf]]",
  "?$categorical[10, inf]]",
  "?=categorical[10, inf]]",
  "?[categorical[10, inf]$",
  "?[categorical[10, inf]$",
  "10 $ categorical[10, inf]",
  "?10 $ categorical[10, inf]",
  "?[10 $ categorical[10, inf]]",
  "$10 $ categorical[10, inf]",
  "=10 $ categorical[10, inf]",
  "$[10 $ categorical[10, inf]]",
  "xxx[10 $ categorical[10, inf]]",
  "?$10 $ categorical[10, inf]]",
  "?=10 $ categorical[10, inf]]",
  "?[10 $ categorical[10, inf]$",
  "?[10 $ categorical[10, inf]$",
  "10 = categorical[10, inf]",
  "?10 = categorical[10, inf]",
  "?[10 = categorical[10, inf]]",
  "$10 = categorical[10, inf]",
  "=10 = categorical[10, inf]",
  "$[10 = categorical[10, inf]]",
  "xxx[10 = categorical[10, inf]]",
  "?$10 = categorical[10, inf]]",
  "?=10 = categorical[10, inf]]",
  "?[10 = categorical[10, inf]$",
  "?[10 = categorical[10, inf]$",
  "10 * categorical[10, 10, inf]",
  "?10 * categorical[10, 10, inf]",
  "?[10 * categorical[10, 10, inf]]",
  "$10 * categorical[10, 10, inf]",
  "=10 * categorical[10, 10, inf]",
  "$[10 * categorical[10, 10, inf]]",
  "xxx[10 * categorical[10, 10, inf]]",
  "?$10 * categorical[10, 10, inf]]",
  "?=10 * categorical[10, 10, inf]]",
  "?[10 * categorical[10, 10, inf]$",
  "?[10 * categorical[10, 10, inf]$",
  "categorical[10, 10, inf]",
  "?categorical[10, 10, inf]",
  "?[categorical[10, 10, inf]]",
  "$categorical[10, 10, inf]",
  "=categorical[10, 10, inf]",
  "$[categorical[10, 10, inf]]",
  "xxx[categorical[10, 10, inf]]",
  "?$categorical[10, 10, inf]]",
  "?=categorical[10, 10, inf]]",
  "?[categorical[10, 10, inf]$",
  "?[categorical[10, 10, inf]$",
  "10 $ categorical[10, 10, inf]",
  "?10 $ categorical[10, 10, inf]",
  "?[10 $ categorical[10, 10, inf]]",
  "$10 $ categorical[10, 10, inf]",
  "=10 $ categorical[10, 10, inf]",
  "$[10 $ categorical[10, 10, inf]]",
  "xxx[10 $ categorical[10, 10, inf]]",
  "?$10 $ categorical[10, 10, inf]]",
  "?=10 $ categorical[10, 10, inf]]",
  "?[10 $ categorical[10, 10, inf]$",
  "?[10 $ categorical[10, 10, inf]$",
  "10 = categorical[10, 10, inf]",
  "?10 = categorical[10, 10, inf]",
  "?[10 = categorical[10, 10, inf]]",
  "$10 = categorical[10, 10, inf]",
  "=10 = categorical[10, 10, inf]",
  "$[10 = categorical[10, 10, inf]]",
  "xxx[10 = categorical[10, 10, inf]]",
  "?$10 = categorical[10, 10, inf]]",
  "?=10 = categorical[10, 10, inf]]",
  "?[10 = categorical[10, 10, inf]$",
  "?[10 = categorical[10, 10, inf]$",
  "10 * categorical['xxx']",
  "?10 * categorical['xxx']",
  "?[10 * categorical['xxx']]",
  "$10 * categorical['xxx']",
  "=10 * categorical['xxx']",
  "$[10 * categorical['xxx']]",
  "xxx[10 * categorical['xxx']]",
  "?$10 * categorical['xxx']]",
  "?=10 * categorical['xxx']]",
  "?[10 * categorical['xxx']$",
  "?[10 * categorical['xxx']$",
  "categorical['xxx']",
  "?categorical['xxx']",
  "?[categorical['xxx']]",
  "$categorical['xxx']",
  "=categorical['xxx']",
  "$[categorical['xxx']]",
  "xxx[categorical['xxx']]",
  "?$categorical['xxx']]",
  "?=categorical['xxx']]",
  "?[categorical['xxx']$",
  "?[categorical['xxx']$",
  "10 $ categorical['xxx']",
  "?10 $ categorical['xxx']",
  "?[10 $ categorical['xxx']]",
  "$10 $ categorical['xxx']",
  "=10 $ categorical['xxx']",
  "$[10 $ categorical['xxx']]",
  "xxx[10 $ categorical['xxx']]",
  "?$10 $ categorical['xxx']]",
  "?=10 $ categorical['xxx']]",
  "?[10 $ categorical['xxx']$",
  "?[10 $ categorical['xxx']$",
  "10 = categorical['xxx']",
  "?10 = categorical['xxx']",
  "?[10 = categorical['xxx']]",
  "$10 = categorical['xxx']",
  "=10 = categorical['xxx']",
  "$[10 = categorical['xxx']]",
  "xxx[10 = categorical['xxx']]",
  "?$10 = categorical['xxx']]",
  "?=10 = categorical['xxx']]",
  "?[10 = categorical['xxx']$",
  "?[10 = categorical['xxx']$",
  "10 * categorical[10, 'xxx']",
  "?10 * categorical[10, 'xxx']",
  "?[10 * categorical[10, 'xxx']]",
  "$10 * categorical[10, 'xxx']",
  "=10 * categorical[10, 'xxx']",
  "$[10 * categorical[10, 'xxx']]",
  "xxx[10 * categorical[10, 'xxx']]",
  "?$10 * categorical[10, 'xxx']]",
  "?=10 * categorical[10, 'xxx']]",
  "?[10 * categorical[10, 'xxx']$",
  "?[10 * categorical[10, 'xxx']$",
  "categorical[10, 'xxx']",
  "?categorical[10, 'xxx']",
  "?[categorical[10, 'xxx']]",
  "$categorical[10, 'xxx']",
  "=categorical[10, 'xxx']",
  "$[categorical[10, 'xxx']]",
  "xxx[categorical[10, 'xxx']]",
  "?$categorical[10, 'xxx']]",
  "?=categorical[10, 'xxx']]",
  "?[categorical[10, 'xxx']$",
  "?[categorical[10, 'xxx']$",
  "10 $ categorical[10, 'xxx']",
  "?10 $ categorical[10, 'xxx']",
  "?[10 $ categorical[10, 'xxx']]",
  "$10 $ categorical[10, 'xxx']",
  "=10 $ categorical[10, 'xxx']",
  "$[10 $ categorical[10, 'xxx']]",
  "xxx[10 $ categorical[10, 'xxx']]",
  "?$10 $ categorical[10, 'xxx']]",
  "?=10 $ categorical[10, 'xxx']]",
  "?[10 $ categorical[10, 'xxx']$",
  "?[10 $ categorical[10, 'xxx']$",
  "10 = categorical[10, 'xxx']",
  "?10 = categorical[10, 'xxx']",
  "?[10 = categorical[10, 'xxx']]",
  "$10 = categorical[10, 'xxx']",
  "=10 = categorical[10, 'xxx']",
  "$[10 = categorical[10, 'xxx']]",
  "xxx[10 = categorical[10, 'xxx']]",
  "?$10 = categorical[10, 'xxx']]",
  "?=10 = categorical[10, 'xxx']]",
  "?[10 = categorical[10, 'xxx']$",
  "?[10 = categorical[10, 'xxx']$",
  "10 * categorical[10, 10, 'xxx']",
  "?10 * categorical[10, 10, 'xxx']",
  "?[10 * categorical[10, 10, 'xxx']]",
  "$10 * categorical[10, 10, 'xxx']",
  "=10 * categorical[10, 10, 'xxx']",
  "$[10 * categorical[10, 10, 'xxx']]",
  "xxx[10 * categorical[10, 10, 'xxx']]",
  "?$10 * categorical[10, 10, 'xxx']]",
  "?=10 * categorical[10, 10, 'xxx']]",
  "?[10 * categorical[10, 10, 'xxx']$",
  "?[10 * categorical[10, 10, 'xxx']$",
  "categorical[10, 10, 'xxx']",
  "?categorical[10, 10, 'xxx']",
  "?[categorical[10, 10, 'xxx']]",
  "$categorical[10, 10, 'xxx']",
  "=categorical[10, 10, 'xxx']",
  "$[categorical[10, 10, 'xxx']]",
  "xxx[categorical[10, 10, 'xxx']]",
  "?$categorical[10, 10, 'xxx']]",
  "?=categorical[10, 10, 'xxx']]",
  "?[categorical[10, 10, 'xxx']$",
  "?[categorical[10, 10, 'xxx']$",
  "10 $ categorical[10, 10, 'xxx']",
  "?10 $ categorical[10, 10, 'xxx']",
  "?[10 $ categorical[10, 10, 'xxx']]",
  "$10 $ categorical[10, 10, 'xxx']",
  "=10 $ categorical[10, 10, 'xxx']",
  "$[10 $ categorical[10, 10, 'xxx']]",
  "xxx[10 $ categorical[10, 10, 'xxx']]",
  "?$10 $ categorical[10, 10, 'xxx']]",
  "?=10 $ categorical[10, 10, 'xxx']]",
  "?[10 $ categorical[10, 10, 'xxx']$",
  "?[10 $ categorical[10, 10, 'xxx']$",
  "10 = categorical[10, 10, 'xxx']",
  "?10 = categorical[10, 10, 'xxx']",
  "?[10 = categorical[10, 10, 'xxx']]",
  "$10 = categorical[10, 10, 'xxx']",
  "=10 = categorical[10, 10, 'xxx']",
  "$[10 = categorical[10, 10, 'xxx']]",
  "xxx[10 = categorical[10, 10, 'xxx']]",
  "?$10 = categorical[10, 10, 'xxx']]",
  "?=10 = categorical[10, 10, 'xxx']]",
  "?[10 = categorical[10, 10, 'xxx']$",
  "?[10 = categorical[10, 10, 'xxx']$",
  "10 * categorical[1.2]",
  "?10 * categorical[1.2]",
  "?[10 * categorical[1.2]]",
  "$10 * categorical[1.2]",
  "=10 * categorical[1.2]",
  "$[10 * categorical[1.2]]",
  "xxx[10 * categorical[1.2]]",
  "?$10 * categorical[1.2]]",
  "?=10 * categorical[1.2]]",
  "?[10 * categorical[1.2]$",
  "?[10 * categorical[1.2]$",
  "categorical[1.2]",
  "?categorical[1.2]",
  "?[categorical[1.2]]",
  "$categorical[1.2]",
  "=categorical[1.2]",
  "$[categorical[1.2]]",
  "xxx[categorical[1.2]]",
  "?$categorical[1.2]]",
  "?=categorical[1.2]]",
  "?[categorical[1.2]$",
  "?[categorical[1.2]$",
  "10 $ categorical[1.2]",
  "?10 $ categorical[1.2]",
  "?[10 $ categorical[1.2]]",
  "$10 $ categorical[1.2]",
  "=10 $ categorical[1.2]",
  "$[10 $ categorical[1.2]]",
  "xxx[10 $ categorical[1.2]]",
  "?$10 $ categorical[1.2]]",
  "?=10 $ categorical[1.2]]",
  "?[10 $ categorical[1.2]$",
  "?[10 $ categorical[1.2]$",
  "10 = categorical[1.2]",
  "?10 = categorical[1.2]",
  "?[10 = categorical[1.2]]",
  "$10 = categorical[1.2]",
  "=10 = categorical[1.2]",
  "$[10 = categorical[1.2]]",
  "xxx[10 = categorical[1.2]]",
  "?$10 = categorical[1.2]]",
  "?=10 = categorical[1.2]]",
  "?[10 = categorical[1.2]$",
  "?[10 = categorical[1.2]$",
  "10 * categorical[10, 1.2]",
  "?10 * categorical[10, 1.2]",
  "?[10 * categorical[10, 1.2]]",
  "$10 * categorical[10, 1.2]",
  "=10 * categorical[10, 1.2]",
  "$[10 * categorical[10, 1.2]]",
  "xxx[10 * categorical[10, 1.2]]",
  "?$10 * categorical[10, 1.2]]",
  "?=10 * categorical[10, 1.2]]",
  "?[10 * categorical[10, 1.2]$",
  "?[10 * categorical[10, 1.2]$",
  "categorical[10, 1.2]",
  "?categorical[10, 1.2]",
  "?[categorical[10, 1.2]]",
  "$categorical[10, 1.2]",
  "=categorical[10, 1.2]",
  "$[categorical[10, 1.2]]",
  "xxx[categorical[10, 1.2]]",
  "?$categorical[10, 1.2]]",
  "?=categorical[10, 1.2]]",
  "?[categorical[10, 1.2]$",
  "?[categorical[10, 1.2]$",
  "10 $ categorical[10, 1.2]",
  "?10 $ categorical[10, 1.2]",
  "?[10 $ categorical[10, 1.2]]",
  "$10 $ categorical[10, 1.2]",
  "=10 $ categorical[10, 1.2]",
  "$[10 $ categorical[10, 1.2]]",
  "xxx[10 $ categorical[10, 1.2]]",
  "?$10 $ categorical[10, 1.2]]",
  "?=10 $ categorical[10, 1.2]]",
  "?[10 $ categorical[10, 1.2]$",
  "?[10 $ categorical[10, 1.2]$",
  "10 = categorical[10, 1.2]",
  "?10 = categorical[10, 1.2]",
  "?[10 = categorical[10, 1.2]]",
  "$10 = categorical[10, 1.2]",
  "=10 = categorical[10, 1.2]",
  "$[10 = categorical[10, 1.2]]",
  "xxx[10 = categorical[10, 1.2]]",
  "?$10 = categorical[10, 1.2]]",
  "?=10 = categorical[10, 1.2]]",
  "?[10 = categorical[10, 1.2]$",
  "?[10 = categorical[10, 1.2]$",
  "10 * categorical[10, 10, 1.2]",
  "?10 * categorical[10, 10, 1.2]",
  "?[10 * categorical[10, 10, 1.2]]",
  "$10 * categorical[10, 10, 1.2]",
  "=10 * categorical[10, 10, 1.2]",
  "$[10 * categorical[10, 10, 1.2]]",
  "xxx[10 * categorical[10, 10, 1.2]]",
  "?$10 * categorical[10, 10, 1.2]]",
  "?=10 * categorical[10, 10, 1.2]]",
  "?[10 * categorical[10, 10, 1.2]$",
  "?[10 * categorical[10, 10, 1.2]$",
  "categorical[10, 10, 1.2]",
  "?categorical[10, 10, 1.2]",
  "?[categorical[10, 10, 1.2]]",
  "$categorical[10, 10, 1.2]",
  "=categorical[10, 10, 1.2]",
  "$[categorical[10, 10, 1.2]]",
  "xxx[categorical[10, 10, 1.2]]",
  "?$categorical[10, 10, 1.2]]",
  "?=categorical[10, 10, 1.2]]",
  "?[categorical[10, 10, 1.2]$",
  "?[categorical[10, 10, 1.2]$",
  "10 $ categorical[10, 10, 1.2]",
  "?10 $ categorical[10, 10, 1.2]",
  "?[10 $ categorical[10, 10, 1.2]]",
  "$10 $ categorical[10, 10, 1.2]",
  "=10 $ categorical[10, 10, 1.2]",
  "$[10 $ categorical[10, 10, 1.2]]",
  "xxx[10 $ categorical[10, 10, 1.2]]",
  "?$10 $ categorical[10, 10, 1.2]]",
  "?=10 $ categorical[10, 10, 1.2]]",
  "?[10 $ categorical[10, 10, 1.2]$",
  "?[10 $ categorical[10, 10, 1.2]$",
  "10 = categorical[10, 10, 1.2]",
  "?10 = categorical[10, 10, 1.2]",
  "?[10 = categorical[10, 10, 1.2]]",
  "$10 = categorical[10, 10, 1.2]",
  "=10 = categorical[10, 10, 1.2]",
  "$[10 = categorical[10, 10, 1.2]]",
  "xxx[10 = categorical[10, 10, 1.2]]",
  "?$10 = categorical[10, 10, 1.2]]",
  "?=10 = categorical[10, 10, 1.2]]",
  "?[10 = categorical[10, 10, 1.2]$",
  "?[10 = categorical[10, 10, 1.2]$",
  "10 * categorical[9223372036854xxx]",
  "?10 * categorical[9223372036854xxx]",
  "?[10 * categorical[9223372036854xxx]]",
  "$10 * categorical[9223372036854xxx]",
  "=10 * categorical[9223372036854xxx]",
  "$[10 * categorical[9223372036854xxx]]",
  "xxx[10 * categorical[9223372036854xxx]]",
  "?$10 * categorical[9223372036854xxx]]",
  "?=10 * categorical[9223372036854xxx]]",
  "?[10 * categorical[9223372036854xxx]$",
  "?[10 * categorical[9223372036854xxx]$",
  "categorical[9223372036854xxx]",
  "?categorical[9223372036854xxx]",
  "?[categorical[9223372036854xxx]]",
  "$categorical[9223372036854xxx]",
  "=categorical[9223372036854xxx]",
  "$[categorical[9223372036854xxx]]",
  "xxx[categorical[9223372036854xxx]]",
  "?$categorical[9223372036854xxx]]",
  "?=categorical[9223372036854xxx]]",
  "?[categorical[9223372036854xxx]$",
  "?[categorical[9223372036854xxx]$",
  "10 $ categorical[9223372036854xxx]",
  "?10 $ categorical[9223372036854xxx]",
  "?[10 $ categorical[9223372036854xxx]]",
  "$10 $ categorical[9223372036854xxx]",
  "=10 $ categorical[9223372036854xxx]",
  "$[10 $ categorical[9223372036854xxx]]",
  "xxx[10 $ categorical[9223372036854xxx]]",
  "?$10 $ categorical[9223372036854xxx]]",
  "?=10 $ categorical[9223372036854xxx]]",
  "?[10 $ categorical[9223372036854xxx]$",
  "?[10 $ categorical[9223372036854xxx]$",
  "10 = categorical[9223372036854xxx]",
  "?10 = categorical[9223372036854xxx]",
  "?[10 = categorical[9223372036854xxx]]",
  "$10 = categorical[9223372036854xxx]",
  "=10 = categorical[9223372036854xxx]",
  "$[10 = categorical[9223372036854xxx]]",
  "xxx[10 = categorical[9223372036854xxx]]",
  "?$10 = categorical[9223372036854xxx]]",
  "?=10 = categorical[9223372036854xxx]]",
  "?[10 = categorical[9223372036854xxx]$",
  "?[10 = categorical[9223372036854xxx]$",
  "10 * categorical[10, 9223372036854xxx]",
  "?10 * categorical[10, 9223372036854xxx]",
  "?[10 * categorical[10, 9223372036854xxx]]",
  "$10 * categorical[10, 9223372036854xxx]",
  "=10 * categorical[10, 9223372036854xxx]",
  "$[10 * categorical[10, 9223372036854xxx]]",
  "xxx[10 * categorical[10, 9223372036854xxx]]",
  "?$10 * categorical[10, 9223372036854xxx]]",
  "?=10 * categorical[10, 9223372036854xxx]]",
  "?[10 * categorical[10, 9223372036854xxx]$",
  "?[10 * categorical[10, 9223372036854xxx]$",
  "categorical[10, 9223372036854xxx]",
  "?categorical[10, 9223372036854xxx]",
  "?[categorical[10, 9223372036854xxx]]",
  "$categorical[10, 9223372036854xxx]",
  "=categorical[10, 9223372036854xxx]",
  "$[categorical[10, 9223372036854xxx]]",
  "xxx[categorical[10, 9223372036854xxx]]",
  "?$categorical[10, 9223372036854xxx]]",
  "?=categorical[10, 9223372036854xxx]]",
  "?[categorical[10, 9223372036854xxx]$",
  "?[categorical[10, 9223372036854xxx]$",
  "10 $ categorical[10, 9223372036854xxx]",
  "?10 $ categorical[10, 9223372036854xxx]",
  "?[10 $ categorical[10, 9223372036854xxx]]",
  "$10 $ categorical[10, 9223372036854xxx]",
  "=10 $ categorical[10, 9223372036854xxx]",
  "$[10 $ categorical[10, 9223372036854xxx]]",
  "xxx[10 $ categorical[10, 9223372036854xxx]]",
  "?$10 $ categorical[10, 9223372036854xxx]]",
  "?=10 $ categorical[10, 9223372036854xxx]]",
  "?[10 $ categorical[10, 9223372036854xxx]$",
  "?[10 $ categorical[10, 9223372036854xxx]$",
  "10 = categorical[10, 9223372036854xxx]",
  "?10 = categorical[10, 9223372036854xxx]",
  "?[10 = categorical[10, 9223372036854xxx]]",
  "$10 = categorical[10, 9223372036854xxx]",
  "=10 = categorical[10, 9223372036854xxx]",
  "$[10 = categorical[10, 9223372036854xxx]]",
  "xxx[10 = categorical[10, 9223372036854xxx]]",
  "?$10 = categorical[10, 9223372036854xxx]]",
  "?=10 = categorical[10, 9223372036854xxx]]",
  "?[10 = categorical[10, 9223372036854xxx]$",
  "?[10 = categorical[10, 9223372036854xxx]$",
  "10 * categorical[10, 10, 9223372036854xxx]",
  "?10 * categorical[10, 10, 9223372036854xxx]",
  "?[10 * categorical[10, 10, 9223372036854xxx]]",
  "$10 * categorical[10, 10, 9223372036854xxx]",
  "=10 * categorical[10, 10, 9223372036854xxx]",
  "$[10 * categorical[10, 10, 9223372036854xxx]]",
  "xxx[10 * categorical[10, 10, 9223372036854xxx]]",
  "?$10 * categorical[10, 10, 9223372036854xxx]]",
  "?=10 * categorical[10, 10, 9223372036854xxx]]",
  "?[10 * categorical[10, 10, 9223372036854xxx]$",
  "?[10 * categorical[10, 10, 9223372036854xxx]$",
  "categorical[10, 10, 9223372036854xxx]",
  "?categorical[10, 10, 9223372036854xxx]",
  "?[categorical[10, 10, 9223372036854xxx]]",
  "$categorical[10, 10, 9223372036854xxx]",
  "=categorical[10, 10, 9223372036854xxx]",
  "$[categorical[10, 10, 9223372036854xxx]]",
  "xxx[categorical[10, 10, 9223372036854xxx]]",
  "?$categorical[10, 10, 9223372036854xxx]]",
  "?=categorical[10, 10, 9223372036854xxx]]",
  "?[categorical[10, 10, 9223372036854xxx]$",
  "?[categorical[10, 10, 9223372036854xxx]$",
  "10 $ categorical[10, 10, 9223372036854xxx]",
  "?10 $ categorical[10, 10, 9223372036854xxx]",
  "?[10 $ categorical[10, 10, 9223372036854xxx]]",
  "$10 $ categorical[10, 10, 9223372036854xxx]",
  "=10 $ categorical[10, 10, 9223372036854xxx]",
  "$[10 $ categorical[10, 10, 9223372036854xxx]]",
  "xxx[10 $ categorical[10, 10, 9223372036854xxx]]",
  "?$10 $ categorical[10, 10, 9223372036854xxx]]",
  "?=10 $ categorical[10, 10, 9223372036854xxx]]",
  "?[10 $ categorical[10, 10, 9223372036854xxx]$",
  "?[10 $ categorical[10, 10, 9223372036854xxx]$",
  "10 = categorical[10, 10, 9223372036854xxx]",
  "?10 = categorical[10, 10, 9223372036854xxx]",
  "?[10 = categorical[10, 10, 9223372036854xxx]]",
  "$10 = categorical[10, 10, 9223372036854xxx]",
  "=10 = categorical[10, 10, 9223372036854xxx]",
  "$[10 = categorical[10, 10, 9223372036854xxx]]",
  "xxx[10 = categorical[10, 10, 9223372036854xxx]]",
  "?$10 = categorical[10, 10, 9223372036854xxx]]",
  "?=10 = categorical[10, 10, 9223372036854xxx]]",
  "?[10 = categorical[10, 10, 9223372036854xxx]$",
  "?[10 = categorical[10, 10, 9223372036854xxx]$",
  "10 * categorical[-922337203685xxx]",
  "?10 * categorical[-922337203685xxx]",
  "?[10 * categorical[-922337203685xxx]]",
  "$10 * categorical[-922337203685xxx]",
  "=10 * categorical[-922337203685xxx]",
  "$[10 * categorical[-922337203685xxx]]",
  "xxx[10 * categorical[-922337203685xxx]]",
  "?$10 * categorical[-922337203685xxx]]",
  "?=10 * categorical[-922337203685xxx]]",
  "?[10 * categorical[-922337203685xxx]$",
  "?[10 * categorical[-922337203685xxx]$",
  "categorical[-922337203685xxx]",
  "?categorical[-922337203685xxx]",
  "?[categorical[-922337203685xxx]]",
  "$categorical[-922337203685xxx]",
  "=categorical[-922337203685xxx]",
  "$[categorical[-922337203685xxx]]",
  "xxx[categorical[-922337203685xxx]]",
  "?$categorical[-922337203685xxx]]",
  "?=categorical[-922337203685xxx]]",
  "?[categorical[-922337203685xxx]$",
  "?[categorical[-922337203685xxx]$",
  "10 $ categorical[-922337203685xxx]",
  "?10 $ categorical[-922337203685xxx]",
  "?[10 $ categorical[-922337203685xxx]]",
  "$10 $ categorical[-922337203685xxx]",
  "=10 $ categorical[-922337203685xxx]",
  "$[10 $ categorical[-922337203685xxx]]",
  "xxx[10 $ categorical[-922337203685xxx]]",
  "?$10 $ categorical[-922337203685xxx]]",
  "?=10 $ categorical[-922337203685xxx]]",
  "?[10 $ categorical[-922337203685xxx]$",
  "?[10 $ categorical[-922337203685xxx]$",
  "10 = categorical[-922337203685xxx]",
  "?10 = categorical[-922337203685xxx]",
  "?[10 = categorical[-922337203685xxx]]",
  "$10 = categorical[-922337203685xxx]",
  "=10 = categorical[-922337203685xxx]",
  "$[10 = categorical[-922337203685xxx]]",
  "xxx[10 = categorical[-922337203685xxx]]",
  "?$10 = categorical[-922337203685xxx]]",
  "?=10 = categorical[-922337203685xxx]]",
  "?[10 = categorical[-922337203685xxx]$",
  "?[10 = categorical[-922337203685xxx]$",
  "10 * categorical[10, -922337203685xxx]",
  "?10 * categorical[10, -922337203685xxx]",
  "?[10 * categorical[10, -922337203685xxx]]",
  "$10 * categorical[10, -922337203685xxx]",
  "=10 * categorical[10, -922337203685xxx]",
  "$[10 * categorical[10, -922337203685xxx]]",
  "xxx[10 * categorical[10, -922337203685xxx]]",
  "?$10 * categorical[10, -922337203685xxx]]",
  "?=10 * categorical[10, -922337203685xxx]]",
  "?[10 * categorical[10, -922337203685xxx]$",
  "?[10 * categorical[10, -922337203685xxx]$",
  "categorical[10, -922337203685xxx]",
  "?categorical[10, -922337203685xxx]",
  "?[categorical[10, -922337203685xxx]]",
  "$categorical[10, -922337203685xxx]",
  "=categorical[10, -922337203685xxx]",
  "$[categorical[10, -922337203685xxx]]",
  "xxx[categorical[10, -922337203685xxx]]",
  "?$categorical[10, -922337203685xxx]]",
  "?=categorical[10, -922337203685xxx]]",
  "?[categorical[10, -922337203685xxx]$",
  "?[categorical[10, -922337203685xxx]$",
  "10 $ categorical[10, -922337203685xxx]",
  "?10 $ categorical[10, -922337203685xxx]",
  "?[10 $ categorical[10, -922337203685xxx]]",
  "$10 $ categorical[10, -922337203685xxx]",
  "=10 $ categorical[10, -922337203685xxx]",
  "$[10 $ categorical[10, -922337203685xxx]]",
  "xxx[10 $ categorical[10, -922337203685xxx]]",
  "?$10 $ categorical[10, -922337203685xxx]]",
  "?=10 $ categorical[10, -922337203685xxx]]",
  "?[10 $ categorical[10, -922337203685xxx]$",
  "?[10 $ categorical[10, -922337203685xxx]$",
  "10 = categorical[10, -922337203685xxx]",
  "?10 = categorical[10, -922337203685xxx]",
  "?[10 = categorical[10, -922337203685xxx]]",
  "$10 = categorical[10, -922337203685xxx]",
  "=10 = categorical[10, -922337203685xxx]",
  "$[10 = categorical[10, -922337203685xxx]]",
  "xxx[10 = categorical[10, -922337203685xxx]]",
  "?$10 = categorical[10, -922337203685xxx]]",
  "?=10 = categorical[10, -922337203685xxx]]",
  "?[10 = categorical[10, -922337203685xxx]$",
  "?[10 = categorical[10, -922337203685xxx]$",
  "10 * categorical[10, 10, -922337203685xxx]",
  "?10 * categorical[10, 10, -922337203685xxx]",
  "?[10 * categorical[10, 10, -922337203685xxx]]",
  "$10 * categorical[10, 10, -922337203685xxx]",
  "=10 * categorical[10, 10, -922337203685xxx]",
  "$[10 * categorical[10, 10, -922337203685xxx]]",
  "xxx[10 * categorical[10, 10, -922337203685xxx]]",
  "?$10 * categorical[10, 10, -922337203685xxx]]",
  "?=10 * categorical[10, 10, -922337203685xxx]]",
  "?[10 * categorical[10, 10, -922337203685xxx]$",
  "?[10 * categorical[10, 10, -922337203685xxx]$",
  "categorical[10, 10, -922337203685xxx]",
  "?categorical[10, 10, -922337203685xxx]",
  "?[categorical[10, 10, -922337203685xxx]]",
  "$categorical[10, 10, -922337203685xxx]",
  "=categorical[10, 10, -922337203685xxx]",
  "$[categorical[10, 10, -922337203685xxx]]",
  "xxx[categorical[10, 10, -922337203685xxx]]",
  "?$categorical[10, 10, -922337203685xxx]]",
  "?=categorical[10, 10, -922337203685xxx]]",
  "?[categorical[10, 10, -922337203685xxx]$",
  "?[categorical[10, 10, -922337203685xxx]$",
  "10 $ categorical[10, 10, -922337203685xxx]",
  "?10 $ categorical[10, 10, -922337203685xxx]",
  "?[10 $ categorical[10, 10, -922337203685xxx]]",
  "$10 $ categorical[10, 10, -922337203685xxx]",
  "=10 $ categorical[10, 10, -922337203685xxx]",
  "$[10 $ categorical[10, 10, -922337203685xxx]]",
  "xxx[10 $ categorical[10, 10, -922337203685xxx]]",
  "?$10 $ categorical[10, 10, -922337203685xxx]]",
  "?=10 $ categorical[10, 10, -922337203685xxx]]",
  "?[10 $ categorical[10, 10, -922337203685xxx]$",
  "?[10 $ categorical[10, 10, -922337203685xxx]$",
  "10 = categorical[10, 10, -922337203685xxx]",
  "?10 = categorical[10, 10, -922337203685xxx]",
  "?[10 = categorical[10, 10, -922337203685xxx]]",
  "$10 = categorical[10, 10, -922337203685xxx]",
  "=10 = categorical[10, 10, -922337203685xxx]",
  "$[10 = categorical[10, 10, -922337203685xxx]]",
  "xxx[10 = categorical[10, 10, -922337203685xxx]]",
  "?$10 = categorical[10, 10, -922337203685xxx]]",
  "?=10 = categorical[10, 10, -922337203685xxx]]",
  "?[10 = categorical[10, 10, -922337203685xxx]$",
  "?[10 = categorical[10, 10, -922337203685xxx]$",
  "10 * categorical[--922337203]",
  "?10 * categorical[--922337203]",
  "?[10 * categorical[--922337203]]",
  "$10 * categorical[--922337203]",
  "=10 * categorical[--922337203]",
  "$[10 * categorical[--922337203]]",
  "xxx[10 * categorical[--922337203]]",
  "?$10 * categorical[--922337203]]",
  "?=10 * categorical[--922337203]]",
  "?[10 * categorical[--922337203]$",
  "?[10 * categorical[--922337203]$",
  "categorical[--922337203]",
  "?categorical[--922337203]",
  "?[categorical[--922337203]]",
  "$categorical[--922337203]",
  "=categorical[--922337203]",
  "$[categorical[--922337203]]",
  "xxx[categorical[--922337203]]",
  "?$categorical[--922337203]]",
  "?=categorical[--922337203]]",
  "?[categorical[--922337203]$",
  "?[categorical[--922337203]$",
  "10 $ categorical[--922337203]",
  "?10 $ categorical[--922337203]",
  "?[10 $ categorical[--922337203]]",
  "$10 $ categorical[--922337203]",
  "=10 $ categorical[--922337203]",
  "$[10 $ categorical[--922337203]]",
  "xxx[10 $ categorical[--922337203]]",
  "?$10 $ categorical[--922337203]]",
  "?=10 $ categorical[--922337203]]",
  "?[10 $ categorical[--922337203]$",
  "?[10 $ categorical[--922337203]$",
  "10 = categorical[--922337203]",
  "?10 = categorical[--922337203]",
  "?[10 = categorical[--922337203]]",
  "$10 = categorical[--922337203]",
  "=10 = categorical[--922337203]",
  "$[10 = categorical[--922337203]]",
  "xxx[10 = categorical[--922337203]]",
  "?$10 = categorical[--922337203]]",
  "?=10 = categorical[--922337203]]",
  "?[10 = categorical[--922337203]$",
  "?[10 = categorical[--922337203]$",
  "10 * categorical[10, --922337203]",
  "?10 * categorical[10, --922337203]",
  "?[10 * categorical[10, --922337203]]",
  "$10 * categorical[10, --922337203]",
  "=10 * categorical[10, --922337203]",
  "$[10 * categorical[10, --922337203]]",
  "xxx[10 * categorical[10, --922337203]]",
  "?$10 * categorical[10, --922337203]]",
  "?=10 * categorical[10, --922337203]]",
  "?[10 * categorical[10, --922337203]$",
  "?[10 * categorical[10, --922337203]$",
  "categorical[10, --922337203]",
  "?categorical[10, --922337203]",
  "?[categorical[10, --922337203]]",
  "$categorical[10, --922337203]",
  "=categorical[10, --922337203]",
  "$[categorical[10, --922337203]]",
  "xxx[categorical[10, --922337203]]",
  "?$categorical[10, --922337203]]",
  "?=categorical[10, --922337203]]",
  "?[categorical[10, --922337203]$",
  "?[categorical[10, --922337203]$",
  "10 $ categorical[10, --922337203]",
  "?10 $ categorical[10, --922337203]",
  "?[10 $ categorical[10, --922337203]]",
  "$10 $ categorical[10, --922337203]",
  "=10 $ categorical[10, --922337203]",
  "$[10 $ categorical[10, --922337203]]",
  "xxx[10 $ categorical[10, --922337203]]",
  "?$10 $ categorical[10, --922337203]]",
  "?=10 $ categorical[10, --922337203]]",
  "?[10 $ categorical[10, --922337203]$",
  "?[10 $ categorical[10, --922337203]$",
  "10 = categorical[10, --922337203]",
  "?10 = categorical[10, --922337203]",
  "?[10 = categorical[10, --922337203]]",
  "$10 = categorical[10, --922337203]",
  "=10 = categorical[10, --922337203]",
  "$[10 = categorical[10, --922337203]]",
  "xxx[10 = categorical[10, --922337203]]",
  "?$10 = categorical[10, --922337203]]",
  "?=10 = categorical[10, --922337203]]",
  "?[10 = categorical[10, --922337203]$",
  "?[10 = categorical[10, --922337203]$",
  "10 * categorical[10, 10, --922337203]",
  "?10 * categorical[10, 10, --922337203]",
  "?[10 * categorical[10, 10, --922337203]]",
  "$10 * categorical[10, 10, --922337203]",
  "=10 * categorical[10, 10, --922337203]",
  "$[10 * categorical[10, 10, --922337203]]",
  "xxx[10 * categorical[10, 10, --922337203]]",
  "?$10 * categorical[10, 10, --922337203]]",
  "?=10 * categorical[10, 10, --922337203]]",
  "?[10 * categorical[10, 10, --922337203]$",
  "?[10 * categorical[10, 10, --922337203]$",
  "categorical[10, 10, --922337203]",
  "?categorical[10, 10, --922337203]",
  "?[categorical[10, 10, --922337203]]",
  "$categorical[10, 10, --922337203]",
  "=categorical[10, 10, --922337203]",
  "$[categorical[10, 10, --922337203]]",
  "xxx[categorical[10, 10, --922337203]]",
  "?$categorical[10, 10, --922337203]]",
  "?=categorical[10, 10, --922337203]]",
  "?[categorical[10, 10, --922337203]$",
  "?[categorical[10, 10, --922337203]$",
  "10 $ categorical[10, 10, --922337203]",
  "?10 $ categorical[10, 10, --922337203]",
  "?[10 $ categorical[10, 10, --922337203]]",
  "$10 $ categorical[10, 10, --922337203]",
  "=10 $ categorical[10, 10, --922337203]",
  "$[10 $ categorical[10, 10, --922337203]]",
  "xxx[10 $ categorical[10, 10, --922337203]]",
  "?$10 $ categorical[10, 10, --922337203]]",
  "?=10 $ categorical[10, 10, --922337203]]",
  "?[10 $ categorical[10, 10, --922337203]$",
  "?[10 $ categorical[10, 10, --922337203]$",
  "10 = categorical[10, 10, --922337203]",
  "?10 = categorical[10, 10, --922337203]",
  "?[10 = categorical[10, 10, --922337203]]",
  "$10 = categorical[10, 10, --922337203]",
  "=10 = categorical[10, 10, --922337203]",
  "$[10 = categorical[10, 10, --922337203]]",
  "xxx[10 = categorical[10, 10, --922337203]]",
  "?$10 = categorical[10, 10, --922337203]]",
  "?=10 = categorical[10, 10, --922337203]]",
  "?[10 = categorical[10, 10, --922337203]$",
  "?[10 = categorical[10, 10, --922337203]$",
  "10 * categorical['xxx']",
  "?10 * categorical['xxx']",
  "?[10 * categorical['xxx']]",
  "$10 * categorical['xxx']",
  "=10 * categorical['xxx']",
  "$[10 * categorical['xxx']]",
  "xxx[10 * categorical['xxx']]",
  "?$10 * categorical['xxx']]",
  "?=10 * categorical['xxx']]",
  "?[10 * categorical['xxx']$",
  "?[10 * categorical['xxx']$",
  "categorical['xxx']",
  "?categorical['xxx']",
  "?[categorical['xxx']]",
  "$categorical['xxx']",
  "=categorical['xxx']",
  "$[categorical['xxx']]",
  "xxx[categorical['xxx']]",
  "?$categorical['xxx']]",
  "?=categorical['xxx']]",
  "?[categorical['xxx']$",
  "?[categorical['xxx']$",
  "10 $ categorical['xxx']",
  "?10 $ categorical['xxx']",
  "?[10 $ categorical['xxx']]",
  "$10 $ categorical['xxx']",
  "=10 $ categorical['xxx']",
  "$[10 $ categorical['xxx']]",
  "xxx[10 $ categorical['xxx']]",
  "?$10 $ categorical['xxx']]",
  "?=10 $ categorical['xxx']]",
  "?[10 $ categorical['xxx']$",
  "?[10 $ categorical['xxx']$",
  "10 = categorical['xxx']",
  "?10 = categorical['xxx']",
  "?[10 = categorical['xxx']]",
  "$10 = categorical['xxx']",
  "=10 = categorical['xxx']",
  "$[10 = categorical['xxx']]",
  "xxx[10 = categorical['xxx']]",
  "?$10 = categorical['xxx']]",
  "?=10 = categorical['xxx']]",
  "?[10 = categorical['xxx']$",
  "?[10 = categorical['xxx']$",
  "10 * categorical[10, 'xxx']",
  "?10 * categorical[10, 'xxx']",
  "?[10 * categorical[10, 'xxx']]",
  "$10 * categorical[10, 'xxx']",
  "=10 * categorical[10, 'xxx']",
  "$[10 * categorical[10, 'xxx']]",
  "xxx[10 * categorical[10, 'xxx']]",
  "?$10 * categorical[10, 'xxx']]",
  "?=10 * categorical[10, 'xxx']]",
  "?[10 * categorical[10, 'xxx']$",
  "?[10 * categorical[10, 'xxx']$",
  "categorical[10, 'xxx']",
  "?categorical[10, 'xxx']",
  "?[categorical[10, 'xxx']]",
  "$categorical[10, 'xxx']",
  "=categorical[10, 'xxx']",
  "$[categorical[10, 'xxx']]",
  "xxx[categorical[10, 'xxx']]",
  "?$categorical[10, 'xxx']]",
  "?=categorical[10, 'xxx']]",
  "?[categorical[10, 'xxx']$",
  "?[categorical[10, 'xxx']$",
  "10 $ categorical[10, 'xxx']",
  "?10 $ categorical[10, 'xxx']",
  "?[10 $ categorical[10, 'xxx']]",
  "$10 $ categorical[10, 'xxx']",
  "=10 $ categorical[10, 'xxx']",
  "$[10 $ categorical[10, 'xxx']]",
  "xxx[10 $ categorical[10, 'xxx']]",
  "?$10 $ categorical[10, 'xxx']]",
  "?=10 $ categorical[10, 'xxx']]",
  "?[10 $ categorical[10, 'xxx']$",
  "?[10 $ categorical[10, 'xxx']$",
  "10 = categorical[10, 'xxx']",
  "?10 = categorical[10, 'xxx']",
  "?[10 = categorical[10, 'xxx']]",
  "$10 = categorical[10, 'xxx']",
  "=10 = categorical[10, 'xxx']",
  "$[10 = categorical[10, 'xxx']]",
  "xxx[10 = categorical[10, 'xxx']]",
  "?$10 = categorical[10, 'xxx']]",
  "?=10 = categorical[10, 'xxx']]",
  "?[10 = categorical[10, 'xxx']$",
  "?[10 = categorical[10, 'xxx']$",
  "10 * categorical[10, 10, 'xxx']",
  "?10 * categorical[10, 10, 'xxx']",
  "?[10 * categorical[10, 10, 'xxx']]",
  "$10 * categorical[10, 10, 'xxx']",
  "=10 * categorical[10, 10, 'xxx']",
  "$[10 * categorical[10, 10, 'xxx']]",
  "xxx[10 * categorical[10, 10, 'xxx']]",
  "?$10 * categorical[10, 10, 'xxx']]",
  "?=10 * categorical[10, 10, 'xxx']]",
  "?[10 * categorical[10, 10, 'xxx']$",
  "?[10 * categorical[10, 10, 'xxx']$",
  "categorical[10, 10, 'xxx']",
  "?categorical[10, 10, 'xxx']",
  "?[categorical[10, 10, 'xxx']]",
  "$categorical[10, 10, 'xxx']",
  "=categorical[10, 10, 'xxx']",
  "$[categorical[10, 10, 'xxx']]",
  "xxx[categorical[10, 10, 'xxx']]",
  "?$categorical[10, 10, 'xxx']]",
  "?=categorical[10, 10, 'xxx']]",
  "?[categorical[10, 10, 'xxx']$",
  "?[categorical[10, 10, 'xxx']$",
  "10 $ categorical[10, 10, 'xxx']",
  "?10 $ categorical[10, 10, 'xxx']",
  "?[10 $ categorical[10, 10, 'xxx']]",
  "$10 $ categorical[10, 10, 'xxx']",
  "=10 $ categorical[10, 10, 'xxx']",
  "$[10 $ categorical[10, 10, 'xxx']]",
  "xxx[10 $ categorical[10, 10, 'xxx']]",
  "?$10 $ categorical[10, 10, 'xxx']]",
  "?=10 $ categorical[10, 10, 'xxx']]",
  "?[10 $ categorical[10, 10, 'xxx']$",
  "?[10 $ categorical[10, 10, 'xxx']$",
  "10 = categorical[10, 10, 'xxx']",
  "?10 = categorical[10, 10, 'xxx']",
  "?[10 = categorical[10, 10, 'xxx']]",
  "$10 = categorical[10, 10, 'xxx']",
  "=10 = categorical[10, 10, 'xxx']",
  "$[10 = categorical[10, 10, 'xxx']]",
  "xxx[10 = categorical[10, 10, 'xxx']]",
  "?$10 = categorical[10, 10, 'xxx']]",
  "?=10 = categorical[10, 10, 'xxx']]",
  "?[10 = categorical[10, 10, 'xxx']$",
  "?[10 = categorical[10, 10, 'xxx']$",
  "10 * categorical[1.2]",
  "?10 * categorical[1.2]",
  "?[10 * categorical[1.2]]",
  "$10 * categorical[1.2]",
  "=10 * categorical[1.2]",
  "$[10 * categorical[1.2]]",
  "xxx[10 * categorical[1.2]]",
  "?$10 * categorical[1.2]]",
  "?=10 * categorical[1.2]]",
  "?[10 * categorical[1.2]$",
  "?[10 * categorical[1.2]$",
  "categorical[1.2]",
  "?categorical[1.2]",
  "?[categorical[1.2]]",
  "$categorical[1.2]",
  "=categorical[1.2]",
  "$[categorical[1.2]]",
  "xxx[categorical[1.2]]",
  "?$categorical[1.2]]",
  "?=categorical[1.2]]",
  "?[categorical[1.2]$",
  "?[categorical[1.2]$",
  "10 $ categorical[1.2]",
  "?10 $ categorical[1.2]",
  "?[10 $ categorical[1.2]]",
  "$10 $ categorical[1.2]",
  "=10 $ categorical[1.2]",
  "$[10 $ categorical[1.2]]",
  "xxx[10 $ categorical[1.2]]",
  "?$10 $ categorical[1.2]]",
  "?=10 $ categorical[1.2]]",
  "?[10 $ categorical[1.2]$",
  "?[10 $ categorical[1.2]$",
  "10 = categorical[1.2]",
  "?10 = categorical[1.2]",
  "?[10 = categorical[1.2]]",
  "$10 = categorical[1.2]",
  "=10 = categorical[1.2]",
  "$[10 = categorical[1.2]]",
  "xxx[10 = categorical[1.2]]",
  "?$10 = categorical[1.2]]",
  "?=10 = categorical[1.2]]",
  "?[10 = categorical[1.2]$",
  "?[10 = categorical[1.2]$",
  "10 * categorical[10, 1.2]",
  "?10 * categorical[10, 1.2]",
  "?[10 * categorical[10, 1.2]]",
  "$10 * categorical[10, 1.2]",
  "=10 * categorical[10, 1.2]",
  "$[10 * categorical[10, 1.2]]",
  "xxx[10 * categorical[10, 1.2]]",
  "?$10 * categorical[10, 1.2]]",
  "?=10 * categorical[10, 1.2]]",
  "?[10 * categorical[10, 1.2]$",
  "?[10 * categorical[10, 1.2]$",
  "categorical[10, 1.2]",
  "?categorical[10, 1.2]",
  "?[categorical[10, 1.2]]",
  "$categorical[10, 1.2]",
  "=categorical[10, 1.2]",
  "$[categorical[10, 1.2]]",
  "xxx[categorical[10, 1.2]]",
  "?$categorical[10, 1.2]]",
  "?=categorical[10, 1.2]]",
  "?[categorical[10, 1.2]$",
  "?[categorical[10, 1.2]$",
  "10 $ categorical[10, 1.2]",
  "?10 $ categorical[10, 1.2]",
  "?[10 $ categorical[10, 1.2]]",
  "$10 $ categorical[10, 1.2]",
  "=10 $ categorical[10, 1.2]",
  "$[10 $ categorical[10, 1.2]]",
  "xxx[10 $ categorical[10, 1.2]]",
  "?$10 $ categorical[10, 1.2]]",
  "?=10 $ categorical[10, 1.2]]",
  "?[10 $ categorical[10, 1.2]$",
  "?[10 $ categorical[10, 1.2]$",
  "10 = categorical[10, 1.2]",
  "?10 = categorical[10, 1.2]",
  "?[10 = categorical[10, 1.2]]",
  "$10 = categorical[10, 1.2]",
  "=10 = categorical[10, 1.2]",
  "$[10 = categorical[10, 1.2]]",
  "xxx[10 = categorical[10, 1.2]]",
  "?$10 = categorical[10, 1.2]]",
  "?=10 = categorical[10, 1.2]]",
  "?[10 = categorical[10, 1.2]$",
  "?[10 = categorical[10, 1.2]$",
  "10 * categorical[10, 10, 1.2]",
  "?10 * categorical[10, 10, 1.2]",
  "?[10 * categorical[10, 10, 1.2]]",
  "$10 * categorical[10, 10, 1.2]",
  "=10 * categorical[10, 10, 1.2]",
  "$[10 * categorical[10, 10, 1.2]]",
  "xxx[10 * categorical[10, 10, 1.2]]",
  "?$10 * categorical[10, 10, 1.2]]",
  "?=10 * categorical[10, 10, 1.2]]",
  "?[10 * categorical[10, 10, 1.2]$",
  "?[10 * categorical[10, 10, 1.2]$",
  "categorical[10, 10, 1.2]",
  "?categorical[10, 10, 1.2]",
  "?[categorical[10, 10, 1.2]]",
  "$categorical[10, 10, 1.2]",
  "=categorical[10, 10, 1.2]",
  "$[categorical[10, 10, 1.2]]",
  "xxx[categorical[10, 10, 1.2]]",
  "?$categorical[10, 10, 1.2]]",
  "?=categorical[10, 10, 1.2]]",
  "?[categorical[10, 10, 1.2]$",
  "?[categorical[10, 10, 1.2]$",
  "10 $ categorical[10, 10, 1.2]",
  "?10 $ categorical[10, 10, 1.2]",
  "?[10 $ categorical[10, 10, 1.2]]",
  "$10 $ categorical[10, 10, 1.2]",
  "=10 $ categorical[10, 10, 1.2]",
  "$[10 $ categorical[10, 10, 1.2]]",
  "xxx[10 $ categorical[10, 10, 1.2]]",
  "?$10 $ categorical[10, 10, 1.2]]",
  "?=10 $ categorical[10, 10, 1.2]]",
  "?[10 $ categorical[10, 10, 1.2]$",
  "?[10 $ categorical[10, 10, 1.2]$",
  "10 = categorical[10, 10, 1.2]",
  "?10 = categorical[10, 10, 1.2]",
  "?[10 = categorical[10, 10, 1.2]]",
  "$10 = categorical[10, 10, 1.2]",
  "=10 = categorical[10, 10, 1.2]",
  "$[10 = categorical[10, 10, 1.2]]",
  "xxx[10 = categorical[10, 10, 1.2]]",
  "?$10 = categorical[10, 10, 1.2]]",
  "?=10 = categorical[10, 10, 1.2]]",
  "?[10 = categorical[10, 10, 1.2]$",
  "?[10 = categorical[10, 10, 1.2]$",
  "10 * categorical[10xxx]",
  "?10 * categorical[10xxx]",
  "?[10 * categorical[10xxx]]",
  "$10 * categorical[10xxx]",
  "=10 * categorical[10xxx]",
  "$[10 * categorical[10xxx]]",
  "xxx[10 * categorical[10xxx]]",
  "?$10 * categorical[10xxx]]",
  "?=10 * categorical[10xxx]]",
  "?[10 * categorical[10xxx]$",
  "?[10 * categorical[10xxx]$",
  "categorical[10xxx]",
  "?categorical[10xxx]",
  "?[categorical[10xxx]]",
  "$categorical[10xxx]",
  "=categorical[10xxx]",
  "$[categorical[10xxx]]",
  "xxx[categorical[10xxx]]",
  "?$categorical[10xxx]]",
  "?=categorical[10xxx]]",
  "?[categorical[10xxx]$",
  "?[categorical[10xxx]$",
  "10 $ categorical[10xxx]",
  "?10 $ categorical[10xxx]",
  "?[10 $ categorical[10xxx]]",
  "$10 $ categorical[10xxx]",
  "=10 $ categorical[10xxx]",
  "$[10 $ categorical[10xxx]]",
  "xxx[10 $ categorical[10xxx]]",
  "?$10 $ categorical[10xxx]]",
  "?=10 $ categorical[10xxx]]",
  "?[10 $ categorical[10xxx]$",
  "?[10 $ categorical[10xxx]$",
  "10 = categorical[10xxx]",
  "?10 = categorical[10xxx]",
  "?[10 = categorical[10xxx]]",
  "$10 = categorical[10xxx]",
  "=10 = categorical[10xxx]",
  "$[10 = categorical[10xxx]]",
  "xxx[10 = categorical[10xxx]]",
  "?$10 = categorical[10xxx]]",
  "?=10 = categorical[10xxx]]",
  "?[10 = categorical[10xxx]$",
  "?[10 = categorical[10xxx]$",
  "10 * categorical[10, 10xxx]",
  "?10 * categorical[10, 10xxx]",
  "?[10 * categorical[10, 10xxx]]",
  "$10 * categorical[10, 10xxx]",
  "=10 * categorical[10, 10xxx]",
  "$[10 * categorical[10, 10xxx]]",
  "xxx[10 * categorical[10, 10xxx]]",
  "?$10 * categorical[10, 10xxx]]",
  "?=10 * categorical[10, 10xxx]]",
  "?[10 * categorical[10, 10xxx]$",
  "?[10 * categorical[10, 10xxx]$",
  "categorical[10, 10xxx]",
  "?categorical[10, 10xxx]",
  "?[categorical[10, 10xxx]]",
  "$categorical[10, 10xxx]",
  "=categorical[10, 10xxx]",
  "$[categorical[10, 10xxx]]",
  "xxx[categorical[10, 10xxx]]",
  "?$categorical[10, 10xxx]]",
  "?=categorical[10, 10xxx]]",
  "?[categorical[10, 10xxx]$",
  "?[categorical[10, 10xxx]$",
  "10 $ categorical[10, 10xxx]",
  "?10 $ categorical[10, 10xxx]",
  "?[10 $ categorical[10, 10xxx]]",
  "$10 $ categorical[10, 10xxx]",
  "=10 $ categorical[10, 10xxx]",
  "$[10 $ categorical[10, 10xxx]]",
  "xxx[10 $ categorical[10, 10xxx]]",
  "?$10 $ categorical[10, 10xxx]]",
  "?=10 $ categorical[10, 10xxx]]",
  "?[10 $ categorical[10, 10xxx]$",
  "?[10 $ categorical[10, 10xxx]$",
  "10 = categorical[10, 10xxx]",
  "?10 = categorical[10, 10xxx]",
  "?[10 = categorical[10, 10xxx]]",
  "$10 = categorical[10, 10xxx]",
  "=10 = categorical[10, 10xxx]",
  "$[10 = categorical[10, 10xxx]]",
  "xxx[10 = categorical[10, 10xxx]]",
  "?$10 = categorical[10, 10xxx]]",
  "?=10 = categorical[10, 10xxx]]",
  "?[10 = categorical[10, 10xxx]$",
  "?[10 = categorical[10, 10xxx]$",
  "10 * categorical[10, 10, 10xxx]",
  "?10 * categorical[10, 10, 10xxx]",
  "?[10 * categorical[10, 10, 10xxx]]",
  "$10 * categorical[10, 10, 10xxx]",
  "=10 * categorical[10, 10, 10xxx]",
  "$[10 * categorical[10, 10, 10xxx]]",
  "xxx[10 * categorical[10, 10, 10xxx]]",
  "?$10 * categorical[10, 10, 10xxx]]",
  "?=10 * categorical[10, 10, 10xxx]]",
  "?[10 * categorical[10, 10, 10xxx]$",
  "?[10 * categorical[10, 10, 10xxx]$",
  "categorical[10, 10, 10xxx]",
  "?categorical[10, 10, 10xxx]",
  "?[categorical[10, 10, 10xxx]]",
  "$categorical[10, 10, 10xxx]",
  "=categorical[10, 10, 10xxx]",
  "$[categorical[10, 10, 10xxx]]",
  "xxx[categorical[10, 10, 10xxx]]",
  "?$categorical[10, 10, 10xxx]]",
  "?=categorical[10, 10, 10xxx]]",
  "?[categorical[10, 10, 10xxx]$",
  "?[categorical[10, 10, 10xxx]$",
  "10 $ categorical[10, 10, 10xxx]",
  "?10 $ categorical[10, 10, 10xxx]",
  "?[10 $ categorical[10, 10, 10xxx]]",
  "$10 $ categorical[10, 10, 10xxx]",
  "=10 $ categorical[10, 10, 10xxx]",
  "$[10 $ categorical[10, 10, 10xxx]]",
  "xxx[10 $ categorical[10, 10, 10xxx]]",
  "?$10 $ categorical[10, 10, 10xxx]]",
  "?=10 $ categorical[10, 10, 10xxx]]",
  "?[10 $ categorical[10, 10, 10xxx]$",
  "?[10 $ categorical[10, 10, 10xxx]$",
  "10 = categorical[10, 10, 10xxx]",
  "?10 = categorical[10, 10, 10xxx]",
  "?[10 = categorical[10, 10, 10xxx]]",
  "$10 = categorical[10, 10, 10xxx]",
  "=10 = categorical[10, 10, 10xxx]",
  "$[10 = categorical[10, 10, 10xxx]]",
  "xxx[10 = categorical[10, 10, 10xxx]]",
  "?$10 = categorical[10, 10, 10xxx]]",
  "?=10 = categorical[10, 10, 10xxx]]",
  "?[10 = categorical[10, 10, 10xxx]$",
  "?[10 = categorical[10, 10, 10xxx]$",
  "10 * categorical[-1844674407]",
  "?10 * categorical[-1844674407]",
  "?[10 * categorical[-1844674407]]",
  "$10 * categorical[-1844674407]",
  "=10 * categorical[-1844674407]",
  "$[10 * categorical[-1844674407]]",
  "xxx[10 * categorical[-1844674407]]",
  "?$10 * categorical[-1844674407]]",
  "?=10 * categorical[-1844674407]]",
  "?[10 * categorical[-1844674407]$",
  "?[10 * categorical[-1844674407]$",
  "categorical[-1844674407]",
  "?categorical[-1844674407]",
  "?[categorical[-1844674407]]",
  "$categorical[-1844674407]",
  "=categorical[-1844674407]",
  "$[categorical[-1844674407]]",
  "xxx[categorical[-1844674407]]",
  "?$categorical[-1844674407]]",
  "?=categorical[-1844674407]]",
  "?[categorical[-1844674407]$",
  "?[categorical[-1844674407]$",
  "10 $ categorical[-1844674407]",
  "?10 $ categorical[-1844674407]",
  "?[10 $ categorical[-1844674407]]",
  "$10 $ categorical[-1844674407]",
  "=10 $ categorical[-1844674407]",
  "$[10 $ categorical[-1844674407]]",
  "xxx[10 $ categorical[-1844674407]]",
  "?$10 $ categorical[-1844674407]]",
  "?=10 $ categorical[-1844674407]]",
  "?[10 $ categorical[-1844674407]$",
  "?[10 $ categorical[-1844674407]$",
  "10 = categorical[-1844674407]",
  "?10 = categorical[-1844674407]",
  "?[10 = categorical[-1844674407]]",
  "$10 = categorical[-1844674407]",
  "=10 = categorical[-1844674407]",
  "$[10 = categorical[-1844674407]]",
  "xxx[10 = categorical[-1844674407]]",
  "?$10 = categorical[-1844674407]]",
  "?=10 = categorical[-1844674407]]",
  "?[10 = categorical[-1844674407]$",
  "?[10 = categorical[-1844674407]$",
  "10 * categorical[10, -1844674407]",
  "?10 * categorical[10, -1844674407]",
  "?[10 * categorical[10, -1844674407]]",
  "$10 * categorical[10, -1844674407]",
  "=10 * categorical[10, -1844674407]",
  "$[10 * categorical[10, -1844674407]]",
  "xxx[10 * categorical[10, -1844674407]]",
  "?$10 * categorical[10, -1844674407]]",
  "?=10 * categorical[10, -1844674407]]",
  "?[10 * categorical[10, -1844674407]$",
  "?[10 * categorical[10, -1844674407]$",
  "categorical[10, -1844674407]",
  "?categorical[10, -1844674407]",
  "?[categorical[10, -1844674407]]",
  "$categorical[10, -1844674407]",
  "=categorical[10, -1844674407]",
  "$[categorical[10, -1844674407]]",
  "xxx[categorical[10, -1844674407]]",
  "?$categorical[10, -1844674407]]",
  "?=categorical[10, -1844674407]]",
  "?[categorical[10, -1844674407]$",
  "?[categorical[10, -1844674407]$",
  "10 $ categorical[10, -1844674407]",
  "?10 $ categorical[10, -1844674407]",
  "?[10 $ categorical[10, -1844674407]]",
  "$10 $ categorical[10, -1844674407]",
  "=10 $ categorical[10, -1844674407]",
  "$[10 $ categorical[10, -1844674407]]",
  "xxx[10 $ categorical[10, -1844674407]]",
  "?$10 $ categorical[10, -1844674407]]",
  "?=10 $ categorical[10, -1844674407]]",
  "?[10 $ categorical[10, -1844674407]$",
  "?[10 $ categorical[10, -1844674407]$",
  "10 = categorical[10, -1844674407]",
  "?10 = categorical[10, -1844674407]",
  "?[10 = categorical[10, -1844674407]]",
  "$10 = categorical[10, -1844674407]",
  "=10 = categorical[10, -1844674407]",
  "$[10 = categorical[10, -1844674407]]",
  "xxx[10 = categorical[10, -1844674407]]",
  "?$10 = categorical[10, -1844674407]]",
  "?=10 = categorical[10, -1844674407]]",
  "?[10 = categorical[10, -1844674407]$",
  "?[10 = categorical[10, -1844674407]$",
  "10 * categorical[10, 10, -1844674407]",
  "?10 * categorical[10, 10, -1844674407]",
  "?[10 * categorical[10, 10, -1844674407]]",
  "$10 * categorical[10, 10, -1844674407]",
  "=10 * categorical[10, 10, -1844674407]",
  "$[10 * categorical[10, 10, -1844674407]]",
  "xxx[10 * categorical[10, 10, -1844674407]]",
  "?$10 * categorical[10, 10, -1844674407]]",
  "?=10 * categorical[10, 10, -1844674407]]",
  "?[10 * categorical[10, 10, -1844674407]$",
  "?[10 * categorical[10, 10, -1844674407]$",
  "categorical[10, 10, -1844674407]",
  "?categorical[10, 10, -1844674407]",
  "?[categorical[10, 10, -1844674407]]",
  "$categorical[10, 10, -1844674407]",
  "=categorical[10, 10, -1844674407]",
  "$[categorical[10, 10, -1844674407]]",
  "xxx[categorical[10, 10, -1844674407]]",
  "?$categorical[10, 10, -1844674407]]",
  "?=categorical[10, 10, -1844674407]]",
  "?[categorical[10, 10, -1844674407]$",
  "?[categorical[10, 10, -1844674407]$",
  "10 $ categorical[10, 10, -1844674407]",
  "?10 $ categorical[10, 10, -1844674407]",
  "?[10 $ categorical[10, 10, -1844674407]]",
  "$10 $ categorical[10, 10, -1844674407]",
  "=10 $ categorical[10, 10, -1844674407]",
  "$[10 $ categorical[10, 10, -1844674407]]",
  "xxx[10 $ categorical[10, 10, -1844674407]]",
  "?$10 $ categorical[10, 10, -1844674407]]",
  "?=10 $ categorical[10, 10, -1844674407]]",
  "?[10 $ categorical[10, 10, -1844674407]$",
  "?[10 $ categorical[10, 10, -1844674407]$",
  "10 = categorical[10, 10, -1844674407]",
  "?10 = categorical[10, 10, -1844674407]",
  "?[10 = categorical[10, 10, -1844674407]]",
  "$10 = categorical[10, 10, -1844674407]",
  "=10 = categorical[10, 10, -1844674407]",
  "$[10 = categorical[10, 10, -1844674407]]",
  "xxx[10 = categorical[10, 10, -1844674407]]",
  "?$10 = categorical[10, 10, -1844674407]]",
  "?=10 = categorical[10, 10, -1844674407]]",
  "?[10 = categorical[10, 10, -1844674407]$",
  "?[10 = categorical[10, 10, -1844674407]$",
  "10 * categorical[--1844674407]",
  "?10 * categorical[--1844674407]",
  "?[10 * categorical[--1844674407]]",
  "$10 * categorical[--1844674407]",
  "=10 * categorical[--1844674407]",
  "$[10 * categorical[--1844674407]]",
  "xxx[10 * categorical[--1844674407]]",
  "?$10 * categorical[--1844674407]]",
  "?=10 * categorical[--1844674407]]",
  "?[10 * categorical[--1844674407]$",
  "?[10 * categorical[--1844674407]$",
  "categorical[--1844674407]",
  "?categorical[--1844674407]",
  "?[categorical[--1844674407]]",
  "$categorical[--1844674407]",
  "=categorical[--1844674407]",
  "$[categorical[--1844674407]]",
  "xxx[categorical[--1844674407]]",
  "?$categorical[--1844674407]]",
  "?=categorical[--1844674407]]",
  "?[categorical[--1844674407]$",
  "?[categorical[--1844674407]$",
  "10 $ categorical[--1844674407]",
  "?10 $ categorical[--1844674407]",
  "?[10 $ categorical[--1844674407]]",
  "$10 $ categorical[--1844674407]",
  "=10 $ categorical[--1844674407]",
  "$[10 $ categorical[--1844674407]]",
  "xxx[10 $ categorical[--1844674407]]",
  "?$10 $ categorical[--1844674407]]",
  "?=10 $ categorical[--1844674407]]",
  "?[10 $ categorical[--1844674407]$",
  "?[10 $ categorical[--1844674407]$",
  "10 = categorical[--1844674407]",
  "?10 = categorical[--1844674407]",
  "?[10 = categorical[--1844674407]]",
  "$10 = categorical[--1844674407]",
  "=10 = categorical[--1844674407]",
  "$[10 = categorical[--1844674407]]",
  "xxx[10 = categorical[--1844674407]]",
  "?$10 = categorical[--1844674407]]",
  "?=10 = categorical[--1844674407]]",
  "?[10 = categorical[--1844674407]$",
  "?[10 = categorical[--1844674407]$",
  "10 * categorical[10, --1844674407]",
  "?10 * categorical[10, --1844674407]",
  "?[10 * categorical[10, --1844674407]]",
  "$10 * categorical[10, --1844674407]",
  "=10 * categorical[10, --1844674407]",
  "$[10 * categorical[10, --1844674407]]",
  "xxx[10 * categorical[10, --1844674407]]",
  "?$10 * categorical[10, --1844674407]]",
  "?=10 * categorical[10, --1844674407]]",
  "?[10 * categorical[10, --1844674407]$",
  "?[10 * categorical[10, --1844674407]$",
  "categorical[10, --1844674407]",
  "?categorical[10, --1844674407]",
  "?[categorical[10, --1844674407]]",
  "$categorical[10, --1844674407]",
  "=categorical[10, --1844674407]",
  "$[categorical[10, --1844674407]]",
  "xxx[categorical[10, --1844674407]]",
  "?$categorical[10, --1844674407]]",
  "?=categorical[10, --1844674407]]",
  "?[categorical[10, --1844674407]$",
  "?[categorical[10, --1844674407]$",
  "10 $ categorical[10, --1844674407]",
  "?10 $ categorical[10, --1844674407]",
  "?[10 $ categorical[10, --1844674407]]",
  "$10 $ categorical[10, --1844674407]",
  "=10 $ categorical[10, --1844674407]",
  "$[10 $ categorical[10, --1844674407]]",
  "xxx[10 $ categorical[10, --1844674407]]",
  "?$10 $ categorical[10, --1844674407]]",
  "?=10 $ categorical[10, --1844674407]]",
  "?[10 $ categorical[10, --1844674407]$",
  "?[10 $ categorical[10, --1844674407]$",
  "10 = categorical[10, --1844674407]",
  "?10 = categorical[10, --1844674407]",
  "?[10 = categorical[10, --1844674407]]",
  "$10 = categorical[10, --1844674407]",
  "=10 = categorical[10, --1844674407]",
  "$[10 = categorical[10, --1844674407]]",
  "xxx[10 = categorical[10, --1844674407]]",
  "?$10 = categorical[10, --1844674407]]",
  "?=10 = categorical[10, --1844674407]]",
  "?[10 = categorical[10, --1844674407]$",
  "?[10 = categorical[10, --1844674407]$",
  "10 * categorical[10, 10, --1844674407]",
  "?10 * categorical[10, 10, --1844674407]",
  "?[10 * categorical[10, 10, --1844674407]]",
  "$10 * categorical[10, 10, --1844674407]",
  "=10 * categorical[10, 10, --1844674407]",
  "$[10 * categorical[10, 10, --1844674407]]",
  "xxx[10 * categorical[10, 10, --1844674407]]",
  "?$10 * categorical[10, 10, --1844674407]]",
  "?=10 * categorical[10, 10, --1844674407]]",
  "?[10 * categorical[10, 10, --1844674407]$",
  "?[10 * categorical[10, 10, --1844674407]$",
  "categorical[10, 10, --1844674407]",
  "?categorical[10, 10, --1844674407]",
  "?[categorical[10, 10, --1844674407]]",
  "$categorical[10, 10, --1844674407]",
  "=categorical[10, 10, --1844674407]",
  "$[categorical[10, 10, --1844674407]]",
  "xxx[categorical[10, 10, --1844674407]]",
  "?$categorical[10, 10, --1844674407]]",
  "?=categorical[10, 10, --1844674407]]",
  "?[categorical[10, 10, --1844674407]$",
  "?[categorical[10, 10, --1844674407]$",
  "10 $ categorical[10, 10, --1844674407]",
  "?10 $ categorical[10, 10, --1844674407]",
  "?[10 $ categorical[10, 10, --1844674407]]",
  "$10 $ categorical[10, 10, --1844674407]",
  "=10 $ categorical[10, 10, --1844674407]",
  "$[10 $ categorical[10, 10, --1844674407]]",
  "xxx[10 $ categorical[10, 10, --1844674407]]",
  "?$10 $ categorical[10, 10, --1844674407]]",
  "?=10 $ categorical[10, 10, --1844674407]]",
  "?[10 $ categorical[10, 10, --1844674407]$",
  "?[10 $ categorical[10, 10, --1844674407]$",
  "10 = categorical[10, 10, --1844674407]",
  "?10 = categorical[10, 10, --1844674407]",
  "?[10 = categorical[10, 10, --1844674407]]",
  "$10 = categorical[10, 10, --1844674407]",
  "=10 = categorical[10, 10, --1844674407]",
  "$[10 = categorical[10, 10, --1844674407]]",
  "xxx[10 = categorical[10, 10, --1844674407]]",
  "?$10 = categorical[10, 10, --1844674407]]",
  "?=10 = categorical[10, 10, --1844674407]]",
  "?[10 = categorical[10, 10, --1844674407]$",
  "?[10 = categorical[10, 10, --1844674407]$",
  "10 * categorical['']",
  "?10 * categorical['']",
  "?[10 * categorical['']]",
  "$10 * categorical['']",
  "=10 * categorical['']",
  "$[10 * categorical['']]",
  "xxx[10 * categorical['']]",
  "?$10 * categorical['']]",
  "?=10 * categorical['']]",
  "?[10 * categorical['']$",
  "?[10 * categorical['']$",
  "categorical['']",
  "?categorical['']",
  "?[categorical['']]",
  "$categorical['']",
  "=categorical['']",
  "$[categorical['']]",
  "xxx[categorical['']]",
  "?$categorical['']]",
  "?=categorical['']]",
  "?[categorical['']$",
  "?[categorical['']$",
  "10 $ categorical['']",
  "?10 $ categorical['']",
  "?[10 $ categorical['']]",
  "$10 $ categorical['']",
  "=10 $ categorical['']",
  "$[10 $ categorical['']]",
  "xxx[10 $ categorical['']]",
  "?$10 $ categorical['']]",
  "?=10 $ categorical['']]",
  "?[10 $ categorical['']$",
  "?[10 $ categorical['']$",
  "10 = categorical['']",
  "?10 = categorical['']",
  "?[10 = categorical['']]",
  "$10 = categorical['']",
  "=10 = categorical['']",
  "$[10 = categorical['']]",
  "xxx[10 = categorical['']]",
  "?$10 = categorical['']]",
  "?=10 = categorical['']]",
  "?[10 = categorical['']$",
  "?[10 = categorical['']$",
  "10 * categorical[10, '']",
  "?10 * categorical[10, '']",
  "?[10 * categorical[10, '']]",
  "$10 * categorical[10, '']",
  "=10 * categorical[10, '']",
  "$[10 * categorical[10, '']]",
  "xxx[10 * categorical[10, '']]",
  "?$10 * categorical[10, '']]",
  "?=10 * categorical[10, '']]",
  "?[10 * categorical[10, '']$",
  "?[10 * categorical[10, '']$",
  "categorical[10, '']",
  "?categorical[10, '']",
  "?[categorical[10, '']]",
  "$categorical[10, '']",
  "=categorical[10, '']",
  "$[categorical[10, '']]",
  "xxx[categorical[10, '']]",
  "?$categorical[10, '']]",
  "?=categorical[10, '']]",
  "?[categorical[10, '']$",
  "?[categorical[10, '']$",
  "10 $ categorical[10, '']",
  "?10 $ categorical[10, '']",
  "?[10 $ categorical[10, '']]",
  "$10 $ categorical[10, '']",
  "=10 $ categorical[10, '']",
  "$[10 $ categorical[10, '']]",
  "xxx[10 $ categorical[10, '']]",
  "?$10 $ categorical[10, '']]",
  "?=10 $ categorical[10, '']]",
  "?[10 $ categorical[10, '']$",
  "?[10 $ categorical[10, '']$",
  "10 = categorical[10, '']",
  "?10 = categorical[10, '']",
  "?[10 = categorical[10, '']]",
  "$10 = categorical[10, '']",
  "=10 = categorical[10, '']",
  "$[10 = categorical[10, '']]",
  "xxx[10 = categorical[10, '']]",
  "?$10 = categorical[10, '']]",
  "?=10 = categorical[10, '']]",
  "?[10 = categorical[10, '']$",
  "?[10 = categorical[10, '']$",
  "10 * categorical[10, 10, '']",
  "?10 * categorical[10, 10, '']",
  "?[10 * categorical[10, 10, '']]",
  "$10 * categorical[10, 10, '']",
  "=10 * categorical[10, 10, '']",
  "$[10 * categorical[10, 10, '']]",
  "xxx[10 * categorical[10, 10, '']]",
  "?$10 * categorical[10, 10, '']]",
  "?=10 * categorical[10, 10, '']]",
  "?[10 * categorical[10, 10, '']$",
  "?[10 * categorical[10, 10, '']$",
  "categorical[10, 10, '']",
  "?categorical[10, 10, '']",
  "?[categorical[10, 10, '']]",
  "$categorical[10, 10, '']",
  "=categorical[10, 10, '']",
  "$[categorical[10, 10, '']]",
  "xxx[categorical[10, 10, '']]",
  "?$categorical[10, 10, '']]",
  "?=categorical[10, 10, '']]",
  "?[categorical[10, 10, '']$",
  "?[categorical[10, 10, '']$",
  "10 $ categorical[10, 10, '']",
  "?10 $ categorical[10, 10, '']",
  "?[10 $ categorical[10, 10, '']]",
  "$10 $ categorical[10, 10, '']",
  "=10 $ categorical[10, 10, '']",
  "$[10 $ categorical[10, 10, '']]",
  "xxx[10 $ categorical[10, 10, '']]",
  "?$10 $ categorical[10, 10, '']]",
  "?=10 $ categorical[10, 10, '']]",
  "?[10 $ categorical[10, 10, '']$",
  "?[10 $ categorical[10, 10, '']$",
  "10 = categorical[10, 10, '']",
  "?10 = categorical[10, 10, '']",
  "?[10 = categorical[10, 10, '']]",
  "$10 = categorical[10, 10, '']",
  "=10 = categorical[10, 10, '']",
  "$[10 = categorical[10, 10, '']]",
  "xxx[10 = categorical[10, 10, '']]",
  "?$10 = categorical[10, 10, '']]",
  "?=10 = categorical[10, 10, '']]",
  "?[10 = categorical[10, 10, '']$",
  "?[10 = categorical[10, 10, '']$",
  "10 * categorical['xxx']",
  "?10 * categorical['xxx']",
  "?[10 * categorical['xxx']]",
  "$10 * categorical['xxx']",
  "=10 * categorical['xxx']",
  "$[10 * categorical['xxx']]",
  "xxx[10 * categorical['xxx']]",
  "?$10 * categorical['xxx']]",
  "?=10 * categorical['xxx']]",
  "?[10 * categorical['xxx']$",
  "?[10 * categorical['xxx']$",
  "categorical['xxx']",
  "?categorical['xxx']",
  "?[categorical['xxx']]",
  "$categorical['xxx']",
  "=categorical['xxx']",
  "$[categorical['xxx']]",
  "xxx[categorical['xxx']]",
  "?$categorical['xxx']]",
  "?=categorical['xxx']]",
  "?[categorical['xxx']$",
  "?[categorical['xxx']$",
  "10 $ categorical['xxx']",
  "?10 $ categorical['xxx']",
  "?[10 $ categorical['xxx']]",
  "$10 $ categorical['xxx']",
  "=10 $ categorical['xxx']",
  "$[10 $ categorical['xxx']]",
  "xxx[10 $ categorical['xxx']]",
  "?$10 $ categorical['xxx']]",
  "?=10 $ categorical['xxx']]",
  "?[10 $ categorical['xxx']$",
  "?[10 $ categorical['xxx']$",
  "10 = categorical['xxx']",
  "?10 = categorical['xxx']",
  "?[10 = categorical['xxx']]",
  "$10 = categorical['xxx']",
  "=10 = categorical['xxx']",
  "$[10 = categorical['xxx']]",
  "xxx[10 = categorical['xxx']]",
  "?$10 = categorical['xxx']]",
  "?=10 = categorical['xxx']]",
  "?[10 = categorical['xxx']$",
  "?[10 = categorical['xxx']$",
  "10 * categorical[10, 'xxx']",
  "?10 * categorical[10, 'xxx']",
  "?[10 * categorical[10, 'xxx']]",
  "$10 * categorical[10, 'xxx']",
  "=10 * categorical[10, 'xxx']",
  "$[10 * categorical[10, 'xxx']]",
  "xxx[10 * categorical[10, 'xxx']]",
  "?$10 * categorical[10, 'xxx']]",
  "?=10 * categorical[10, 'xxx']]",
  "?[10 * categorical[10, 'xxx']$",
  "?[10 * categorical[10, 'xxx']$",
  "categorical[10, 'xxx']",
  "?categorical[10, 'xxx']",
  "?[categorical[10, 'xxx']]",
  "$categorical[10, 'xxx']",
  "=categorical[10, 'xxx']",
  "$[categorical[10, 'xxx']]",
  "xxx[categorical[10, 'xxx']]",
  "?$categorical[10, 'xxx']]",
  "?=categorical[10, 'xxx']]",
  "?[categorical[10, 'xxx']$",
  "?[categorical[10, 'xxx']$",
  "10 $ categorical[10, 'xxx']",
  "?10 $ categorical[10, 'xxx']",
  "?[10 $ categorical[10, 'xxx']]",
  "$10 $ categorical[10, 'xxx']",
  "=10 $ categorical[10, 'xxx']",
  "$[10 $ categorical[10, 'xxx']]",
  "xxx[10 $ categorical[10, 'xxx']]",
  "?$10 $ categorical[10, 'xxx']]",
  "?=10 $ categorical[10, 'xxx']]",
  "?[10 $ categorical[10, 'xxx']$",
  "?[10 $ categorical[10, 'xxx']$",
  "10 = categorical[10, 'xxx']",
  "?10 = categorical[10, 'xxx']",
  "?[10 = categorical[10, 'xxx']]",
  "$10 = categorical[10, 'xxx']",
  "=10 = categorical[10, 'xxx']",
  "$[10 = categorical[10, 'xxx']]",
  "xxx[10 = categorical[10, 'xxx']]",
  "?$10 = categorical[10, 'xxx']]",
  "?=10 = categorical[10, 'xxx']]",
  "?[10 = categorical[10, 'xxx']$",
  "?[10 = categorical[10, 'xxx']$",
  "10 * categorical[10, 10, 'xxx']",
  "?10 * categorical[10, 10, 'xxx']",
  "?[10 * categorical[10, 10, 'xxx']]",
  "$10 * categorical[10, 10, 'xxx']",
  "=10 * categorical[10, 10, 'xxx']",
  "$[10 * categorical[10, 10, 'xxx']]",
  "xxx[10 * categorical[10, 10, 'xxx']]",
  "?$10 * categorical[10, 10, 'xxx']]",
  "?=10 * categorical[10, 10, 'xxx']]",
  "?[10 * categorical[10, 10, 'xxx']$",
  "?[10 * categorical[10, 10, 'xxx']$",
  "categorical[10, 10, 'xxx']",
  "?categorical[10, 10, 'xxx']",
  "?[categorical[10, 10, 'xxx']]",
  "$categorical[10, 10, 'xxx']",
  "=categorical[10, 10, 'xxx']",
  "$[categorical[10, 10, 'xxx']]",
  "xxx[categorical[10, 10, 'xxx']]",
  "?$categorical[10, 10, 'xxx']]",
  "?=categorical[10, 10, 'xxx']]",
  "?[categorical[10, 10, 'xxx']$",
  "?[categorical[10, 10, 'xxx']$",
  "10 $ categorical[10, 10, 'xxx']",
  "?10 $ categorical[10, 10, 'xxx']",
  "?[10 $ categorical[10, 10, 'xxx']]",
  "$10 $ categorical[10, 10, 'xxx']",
  "=10 $ categorical[10, 10, 'xxx']",
  "$[10 $ categorical[10, 10, 'xxx']]",
  "xxx[10 $ categorical[10, 10, 'xxx']]",
  "?$10 $ categorical[10, 10, 'xxx']]",
  "?=10 $ categorical[10, 10, 'xxx']]",
  "?[10 $ categorical[10, 10, 'xxx']$",
  "?[10 $ categorical[10, 10, 'xxx']$",
  "10 = categorical[10, 10, 'xxx']",
  "?10 = categorical[10, 10, 'xxx']",
  "?[10 = categorical[10, 10, 'xxx']]",
  "$10 = categorical[10, 10, 'xxx']",
  "=10 = categorical[10, 10, 'xxx']",
  "$[10 = categorical[10, 10, 'xxx']]",
  "xxx[10 = categorical[10, 10, 'xxx']]",
  "?$10 = categorical[10, 10, 'xxx']]",
  "?=10 = categorical[10, 10, 'xxx']]",
  "?[10 = categorical[10, 10, 'xxx']$",
  "?[10 = categorical[10, 10, 'xxx']$",
  "10 * categorical[2.22x508e-308]",
  "?10 * categorical[2.22x508e-308]",
  "?[10 * categorical[2.22x508e-308]]",
  "$10 * categorical[2.22x508e-308]",
  "=10 * categorical[2.22x508e-308]",
  "$[10 * categorical[2.22x508e-308]]",
  "xxx[10 * categorical[2.22x508e-308]]",
  "?$10 * categorical[2.22x508e-308]]",
  "?=10 * categorical[2.22x508e-308]]",
  "?[10 * categorical[2.22x508e-308]$",
  "?[10 * categorical[2.22x508e-308]$",
  "categorical[2.22x508e-308]",
  "?categorical[2.22x508e-308]",
  "?[categorical[2.22x508e-308]]",
  "$categorical[2.22x508e-308]",
  "=categorical[2.22x508e-308]",
  "$[categorical[2.22x508e-308]]",
  "xxx[categorical[2.22x508e-308]]",
  "?$categorical[2.22x508e-308]]",
  "?=categorical[2.22x508e-308]]",
  "?[categorical[2.22x508e-308]$",
  "?[categorical[2.22x508e-308]$",
  "10 $ categorical[2.22x508e-308]",
  "?10 $ categorical[2.22x508e-308]",
  "?[10 $ categorical[2.22x508e-308]]",
  "$10 $ categorical[2.22x508e-308]",
  "=10 $ categorical[2.22x508e-308]",
  "$[10 $ categorical[2.22x508e-308]]",
  "xxx[10 $ categorical[2.22x508e-308]]",
  "?$10 $ categorical[2.22x508e-308]]",
  "?=10 $ categorical[2.22x508e-308]]",
  "?[10 $ categorical[2.22x508e-308]$",
  "?[10 $ categorical[2.22x508e-308]$",
  "10 = categorical[2.22x508e-308]",
  "?10 = categorical[2.22x508e-308]",
  "?[10 = categorical[2.22x508e-308]]",
  "$10 = categorical[2.22x508e-308]",
  "=10 = categorical[2.22x508e-308]",
  "$[10 = categorical[2.22x508e-308]]",
  "xxx[10 = categorical[2.22x508e-308]]",
  "?$10 = categorical[2.22x508e-308]]",
  "?=10 = categorical[2.22x508e-308]]",
  "?[10 = categorical[2.22x508e-308]$",
  "?[10 = categorical[2.22x508e-308]$",
  "10 * categorical[10, 2.22x508e-308]",
  "?10 * categorical[10, 2.22x508e-308]",
  "?[10 * categorical[10, 2.22x508e-308]]",
  "$10 * categorical[10, 2.22x508e-308]",
  "=10 * categorical[10, 2.22x508e-308]",
  "$[10 * categorical[10, 2.22x508e-308]]",
  "xxx[10 * categorical[10, 2.22x508e-308]]",
  "?$10 * categorical[10, 2.22x508e-308]]",
  "?=10 * categorical[10, 2.22x508e-308]]",
  "?[10 * categorical[10, 2.22x508e-308]$",
  "?[10 * categorical[10, 2.22x508e-308]$",
  "categorical[10, 2.22x508e-308]",
  "?categorical[10, 2.22x508e-308]",
  "?[categorical[10, 2.22x508e-308]]",
  "$categorical[10, 2.22x508e-308]",
  "=categorical[10, 2.22x508e-308]",
  "$[categorical[10, 2.22x508e-308]]",
  "xxx[categorical[10, 2.22x508e-308]]",
  "?$categorical[10, 2.22x508e-308]]",
  "?=categorical[10, 2.22x508e-308]]",
  "?[categorical[10, 2.22x508e-308]$",
  "?[categorical[10, 2.22x508e-308]$",
  "10 $ categorical[10, 2.22x508e-308]",
  "?10 $ categorical[10, 2.22x508e-308]",
  "?[10 $ categorical[10, 2.22x508e-308]]",
  "$10 $ categorical[10, 2.22x508e-308]",
  "=10 $ categorical[10, 2.22x508e-308]",
  "$[10 $ categorical[10, 2.22x508e-308]]",
  "xxx[10 $ categorical[10, 2.22x508e-308]]",
  "?$10 $ categorical[10, 2.22x508e-308]]",
  "?=10 $ categorical[10, 2.22x508e-308]]",
  "?[10 $ categorical[10, 2.22x508e-308]$",
  "?[10 $ categorical[10, 2.22x508e-308]$",
  "10 = categorical[10, 2.22x508e-308]",
  "?10 = categorical[10, 2.22x508e-308]",
  "?[10 = categorical[10, 2.22x508e-308]]",
  "$10 = categorical[10, 2.22x508e-308]",
  "=10 = categorical[10, 2.22x508e-308]",
  "$[10 = categorical[10, 2.22x508e-308]]",
  "xxx[10 = categorical[10, 2.22x508e-308]]",
  "?$10 = categorical[10, 2.22x508e-308]]",
  "?=10 = categorical[10, 2.22x508e-308]]",
  "?[10 = categorical[10, 2.22x508e-308]$",
  "?[10 = categorical[10, 2.22x508e-308]$",
  "10 * categorical[10, 10, 2.22x508e-308]",
  "?10 * categorical[10, 10, 2.22x508e-308]",
  "?[10 * categorical[10, 10, 2.22x508e-308]]",
  "$10 * categorical[10, 10, 2.22x508e-308]",
  "=10 * categorical[10, 10, 2.22x508e-308]",
  "$[10 * categorical[10, 10, 2.22x508e-308]]",
  "xxx[10 * categorical[10, 10, 2.22x508e-308]]",
  "?$10 * categorical[10, 10, 2.22x508e-308]]",
  "?=10 * categorical[10, 10, 2.22x508e-308]]",
  "?[10 * categorical[10, 10, 2.22x508e-308]$",
  "?[10 * categorical[10, 10, 2.22x508e-308]$",
  "categorical[10, 10, 2.22x508e-308]",
  "?categorical[10, 10, 2.22x508e-308]",
  "?[categorical[10, 10, 2.22x508e-308]]",
  "$categorical[10, 10, 2.22x508e-308]",
  "=categorical[10, 10, 2.22x508e-308]",
  "$[categorical[10, 10, 2.22x508e-308]]",
  "xxx[categorical[10, 10, 2.22x508e-308]]",
  "?$categorical[10, 10, 2.22x508e-308]]",
  "?=categorical[10, 10, 2.22x508e-308]]",
  "?[categorical[10, 10, 2.22x508e-308]$",
  "?[categorical[10, 10, 2.22x508e-308]$",
  "10 $ categorical[10, 10, 2.22x508e-308]",
  "?10 $ categorical[10, 10, 2.22x508e-308]",
  "?[10 $ categorical[10, 10, 2.22x508e-308]]",
  "$10 $ categorical[10, 10, 2.22x508e-308]",
  "=10 $ categorical[10, 10, 2.22x508e-308]",
  "$[10 $ categorical[10, 10, 2.22x508e-308]]",
  "xxx[10 $ categorical[10, 10, 2.22x508e-308]]",
  "?$10 $ categorical[10, 10, 2.22x508e-308]]",
  "?=10 $ categorical[10, 10, 2.22x508e-308]]",
  "?[10 $ categorical[10, 10, 2.22x508e-308]$",
  "?[10 $ categorical[10, 10, 2.22x508e-308]$",
  "10 = categorical[10, 10, 2.22x508e-308]",
  "?10 = categorical[10, 10, 2.22x508e-308]",
  "?[10 = categorical[10, 10, 2.22x508e-308]]",
  "$10 = categorical[10, 10, 2.22x508e-308]",
  "=10 = categorical[10, 10, 2.22x508e-308]",
  "$[10 = categorical[10, 10, 2.22x508e-308]]",
  "xxx[10 = categorical[10, 10, 2.22x508e-308]]",
  "?$10 = categorical[10, 10, 2.22x508e-308]]",
  "?=10 = categorical[10, 10, 2.22x508e-308]]",
  "?[10 = categorical[10, 10, 2.22x508e-308]$",
  "?[10 = categorical[10, 10, 2.22x508e-308]$",
  "10 * categorical[--1.79770e+308]",
  "?10 * categorical[--1.79770e+308]",
  "?[10 * categorical[--1.79770e+308]]",
  "$10 * categorical[--1.79770e+308]",
  "=10 * categorical[--1.79770e+308]",
  "$[10 * categorical[--1.79770e+308]]",
  "xxx[10 * categorical[--1.79770e+308]]",
  "?$10 * categorical[--1.79770e+308]]",
  "?=10 * categorical[--1.79770e+308]]",
  "?[10 * categorical[--1.79770e+308]$",
  "?[10 * categorical[--1.79770e+308]$",
  "categorical[--1.79770e+308]",
  "?categorical[--1.79770e+308]",
  "?[categorical[--1.79770e+308]]",
  "$categorical[--1.79770e+308]",
  "=categorical[--1.79770e+308]",
  "$[categorical[--1.79770e+308]]",
  "xxx[categorical[--1.79770e+308]]",
  "?$categorical[--1.79770e+308]]",
  "?=categorical[--1.79770e+308]]",
  "?[categorical[--1.79770e+308]$",
  "?[categorical[--1.79770e+308]$",
  "10 $ categorical[--1.79770e+308]",
  "?10 $ categorical[--1.79770e+308]",
  "?[10 $ categorical[--1.79770e+308]]",
  "$10 $ categorical[--1.79770e+308]",
  "=10 $ categorical[--1.79770e+308]",
  "$[10 $ categorical[--1.79770e+308]]",
  "xxx[10 $ categorical[--1.79770e+308]]",
  "?$10 $ categorical[--1.79770e+308]]",
  "?=10 $ categorical[--1.79770e+308]]",
  "?[10 $ categorical[--1.79770e+308]$",
  "?[10 $ categorical[--1.79770e+308]$",
  "10 = categorical[--1.79770e+308]",
  "?10 = categorical[--1.79770e+308]",
  "?[10 = categorical[--1.79770e+308]]",
  "$10 = categorical[--1.79770e+308]",
  "=10 = categorical[--1.79770e+308]",
  "$[10 = categorical[--1.79770e+308]]",
  "xxx[10 = categorical[--1.79770e+308]]",
  "?$10 = categorical[--1.79770e+308]]",
  "?=10 = categorical[--1.79770e+308]]",
  "?[10 = categorical[--1.79770e+308]$",
  "?[10 = categorical[--1.79770e+308]$",
  "10 * categorical[10, --1.79770e+308]",
  "?10 * categorical[10, --1.79770e+308]",
  "?[10 * categorical[10, --1.79770e+308]]",
  "$10 * categorical[10, --1.79770e+308]",
  "=10 * categorical[10, --1.79770e+308]",
  "$[10 * categorical[10, --1.79770e+308]]",
  "xxx[10 * categorical[10, --1.79770e+308]]",
  "?$10 * categorical[10, --1.79770e+308]]",
  "?=10 * categorical[10, --1.79770e+308]]",
  "?[10 * categorical[10, --1.79770e+308]$",
  "?[10 * categorical[10, --1.79770e+308]$",
  "categorical[10, --1.79770e+308]",
  "?categorical[10, --1.79770e+308]",
  "?[categorical[10, --1.79770e+308]]",
  "$categorical[10, --1.79770e+308]",
  "=categorical[10, --1.79770e+308]",
  "$[categorical[10, --1.79770e+308]]",
  "xxx[categorical[10, --1.79770e+308]]",
  "?$categorical[10, --1.79770e+308]]",
  "?=categorical[10, --1.79770e+308]]",
  "?[categorical[10, --1.79770e+308]$",
  "?[categorical[10, --1.79770e+308]$",
  "10 $ categorical[10, --1.79770e+308]",
  "?10 $ categorical[10, --1.79770e+308]",
  "?[10 $ categorical[10, --1.79770e+308]]",
  "$10 $ categorical[10, --1.79770e+308]",
  "=10 $ categorical[10, --1.79770e+308]",
  "$[10 $ categorical[10, --1.79770e+308]]",
  "xxx[10 $ categorical[10, --1.79770e+308]]",
  "?$10 $ categorical[10, --1.79770e+308]]",
  "?=10 $ categorical[10, --1.79770e+308]]",
  "?[10 $ categorical[10, --1.79770e+308]$",
  "?[10 $ categorical[10, --1.79770e+308]$",
  "10 = categorical[10, --1.79770e+308]",
  "?10 = categorical[10, --1.79770e+308]",
  "?[10 = categorical[10, --1.79770e+308]]",
  "$10 = categorical[10, --1.79770e+308]",
  "=10 = categorical[10, --1.79770e+308]",
  "$[10 = categorical[10, --1.79770e+308]]",
  "xxx[10 = categorical[10, --1.79770e+308]]",
  "?$10 = categorical[10, --1.79770e+308]]",
  "?=10 = categorical[10, --1.79770e+308]]",
  "?[10 = categorical[10, --1.79770e+308]$",
  "?[10 = categorical[10, --1.79770e+308]$",
  "10 * categorical[10, 10, --1.79770e+308]",
  "?10 * categorical[10, 10, --1.79770e+308]",
  "?[10 * categorical[10, 10, --1.79770e+308]]",
  "$10 * categorical[10, 10, --1.79770e+308]",
  "=10 * categorical[10, 10, --1.79770e+308]",
  "$[10 * categorical[10, 10, --1.79770e+308]]",
  "xxx[10 * categorical[10, 10, --1.79770e+308]]",
  "?$10 * categorical[10, 10, --1.79770e+308]]",
  "?=10 * categorical[10, 10, --1.79770e+308]]",
  "?[10 * categorical[10, 10, --1.79770e+308]$",
  "?[10 * categorical[10, 10, --1.79770e+308]$",
  "categorical[10, 10, --1.79770e+308]",
  "?categorical[10, 10, --1.79770e+308]",
  "?[categorical[10, 10, --1.79770e+308]]",
  "$categorical[10, 10, --1.79770e+308]",
  "=categorical[10, 10, --1.79770e+308]",
  "$[categorical[10, 10, --1.79770e+308]]",
  "xxx[categorical[10, 10, --1.79770e+308]]",
  "?$categorical[10, 10, --1.79770e+308]]",
  "?=categorical[10, 10, --1.79770e+308]]",
  "?[categorical[10, 10, --1.79770e+308]$",
  "?[categorical[10, 10, --1.79770e+308]$",
  "10 $ categorical[10, 10, --1.79770e+308]",
  "?10 $ categorical[10, 10, --1.79770e+308]",
  "?[10 $ categorical[10, 10, --1.79770e+308]]",
  "$10 $ categorical[10, 10, --1.79770e+308]",
  "=10 $ categorical[10, 10, --1.79770e+308]",
  "$[10 $ categorical[10, 10, --1.79770e+308]]",
  "xxx[10 $ categorical[10, 10, --1.79770e+308]]",
  "?$10 $ categorical[10, 10, --1.79770e+308]]",
  "?=10 $ categorical[10, 10, --1.79770e+308]]",
  "?[10 $ categorical[10, 10, --1.79770e+308]$",
  "?[10 $ categorical[10, 10, --1.79770e+308]$",
  "10 = categorical[10, 10, --1.79770e+308]",
  "?10 = categorical[10, 10, --1.79770e+308]",
  "?[10 = categorical[10, 10, --1.79770e+308]]",
  "$10 = categorical[10, 10, --1.79770e+308]",
  "=10 = categorical[10, 10, --1.79770e+308]",
  "$[10 = categorical[10, 10, --1.79770e+308]]",
  "xxx[10 = categorical[10, 10, --1.79770e+308]]",
  "?$10 = categorical[10, 10, --1.79770e+308]]",
  "?=10 = categorical[10, 10, --1.79770e+308]]",
  "?[10 = categorical[10, 10, --1.79770e+308]$",
  "?[10 = categorical[10, 10, --1.79770e+308]$",
  "10 * $[1 : int64]",
  "?10 * $[1 : int64]",
  "?[10 * $[1 : int64]]",
  "$10 * $[1 : int64]",
  "=10 * $[1 : int64]",
  "$[10 * $[1 : int64]]",
  "xxx[10 * $[1 : int64]]",
  "?$10 * $[1 : int64]]",
  "?=10 * $[1 : int64]]",
  "?[10 * $[1 : int64]$",
  "?[10 * $[1 : int64]$",
  "$[1 : int64]",
  "?$[1 : int64]",
  "?[$[1 : int64]]",
  "$$[1 : int64]",
  "=$[1 : int64]",
  "$[$[1 : int64]]",
  "xxx[$[1 : int64]]",
  "?$$[1 : int64]]",
  "?=$[1 : int64]]",
  "?[$[1 : int64]$",
  "?[$[1 : int64]$",
  "10 $ $[1 : int64]",
  "?10 $ $[1 : int64]",
  "?[10 $ $[1 : int64]]",
  "$10 $ $[1 : int64]",
  "=10 $ $[1 : int64]",
  "$[10 $ $[1 : int64]]",
  "xxx[10 $ $[1 : int64]]",
  "?$10 $ $[1 : int64]]",
  "?=10 $ $[1 : int64]]",
  "?[10 $ $[1 : int64]$",
  "?[10 $ $[1 : int64]$",
  "10 = $[1 : int64]",
  "?10 = $[1 : int64]",
  "?[10 = $[1 : int64]]",
  "$10 = $[1 : int64]",
  "=10 = $[1 : int64]",
  "$[10 = $[1 : int64]]",
  "xxx[10 = $[1 : int64]]",
  "?$10 = $[1 : int64]]",
  "?=10 = $[1 : int64]]",
  "?[10 = $[1 : int64]$",
  "?[10 = $[1 : int64]$",
  "10 * =[1 : int64]",
  "?10 * =[1 : int64]",
  "?[10 * =[1 : int64]]",
  "$10 * =[1 : int64]",
  "=10 * =[1 : int64]",
  "$[10 * =[1 : int64]]",
  "xxx[10 * =[1 : int64]]",
  "?$10 * =[1 : int64]]",
  "?=10 * =[1 : int64]]",
  "?[10 * =[1 : int64]$",
  "?[10 * =[1 : int64]$",
  "=[1 : int64]",
  "?=[1 : int64]",
  "?[=[1 : int64]]",
  "$=[1 : int64]",
  "==[1 : int64]",
  "$[=[1 : int64]]",
  "xxx[=[1 : int64]]",
  "?$=[1 : int64]]",
  "?==[1 : int64]]",
  "?[=[1 : int64]$",
  "?[=[1 : int64]$",
  "10 $ =[1 : int64]",
  "?10 $ =[1 : int64]",
  "?[10 $ =[1 : int64]]",
  "$10 $ =[1 : int64]",
  "=10 $ =[1 : int64]",
  "$[10 $ =[1 : int64]]",
  "xxx[10 $ =[1 : int64]]",
  "?$10 $ =[1 : int64]]",
  "?=10 $ =[1 : int64]]",
  "?[10 $ =[1 : int64]$",
  "?[10 $ =[1 : int64]$",
  "10 = =[1 : int64]",
  "?10 = =[1 : int64]",
  "?[10 = =[1 : int64]]",
  "$10 = =[1 : int64]",
  "=10 = =[1 : int64]",
  "$[10 = =[1 : int64]]",
  "xxx[10 = =[1 : int64]]",
  "?$10 = =[1 : int64]]",
  "?=10 = =[1 : int64]]",
  "?[10 = =[1 : int64]$",
  "?[10 = =[1 : int64]$",
  "10 * xxx[1 : int64]",
  "?10 * xxx[1 : int64]",
  "?[10 * xxx[1 : int64]]",
  "$10 * xxx[1 : int64]",
  "=10 * xxx[1 : int64]",
  "$[10 * xxx[1 : int64]]",
  "xxx[10 * xxx[1 : int64]]",
  "?$10 * xxx[1 : int64]]",
  "?=10 * xxx[1 : int64]]",
  "?[10 * xxx[1 : int64]$",
  "?[10 * xxx[1 : int64]$",
  "xxx[1 : int64]",
  "?xxx[1 : int64]",
  "?[xxx[1 : int64]]",
  "$xxx[1 : int64]",
  "=xxx[1 : int64]",
  "$[xxx[1 : int64]]",
  "xxx[xxx[1 : int64]]",
  "?$xxx[1 : int64]]",
  "?=xxx[1 : int64]]",
  "?[xxx[1 : int64]$",
  "?[xxx[1 : int64]$",
  "10 $ xxx[1 : int64]",
  "?10 $ xxx[1 : int64]",
  "?[10 $ xxx[1 : int64]]",
  "$10 $ xxx[1 : int64]",
  "=10 $ xxx[1 : int64]",
  "$[10 $ xxx[1 : int64]]",
  "xxx[10 $ xxx[1 : int64]]",
  "?$10 $ xxx[1 : int64]]",
  "?=10 $ xxx[1 : int64]]",
  "?[10 $ xxx[1 : int64]$",
  "?[10 $ xxx[1 : int64]$",
  "10 = xxx[1 : int64]",
  "?10 = xxx[1 : int64]",
  "?[10 = xxx[1 : int64]]",
  "$10 = xxx[1 : int64]",
  "=10 = xxx[1 : int64]",
  "$[10 = xxx[1 : int64]]",
  "xxx[10 = xxx[1 : int64]]",
  "?$10 = xxx[1 : int64]]",
  "?=10 = xxx[1 : int64]]",
  "?[10 = xxx[1 : int64]$",
  "?[10 = xxx[1 : int64]$",
  "10 * categorical$1]",
  "?10 * categorical$1]",
  "?(10 * categorical$1])",
  "$10 * categorical$1]",
  "=10 * categorical$1]",
  "$[10 * categorical$1]]",
  "xxx[10 * categorical$1]]",
  "?$10 * categorical$1]]",
  "?=10 * categorical$1]]",
  "?(10 * categorical$1)$",
  "?(10 * categorical$1)$",
  "categorical$1]",
  "?categorical$1]",
  "?(categorical$1])",
  "$categorical$1]",
  "=categorical$1]",
  "$[categorical$1]]",
  "xxx[categorical$1]]",
  "?$categorical$1]]",
  "?=categorical$1]]",
  "?(categorical$1)$",
  "?(categorical$1)$",
  "10 $ categorical$1]",
  "?10 $ categorical$1]",
  "?(10 $ categorical$1])",
  "$10 $ categorical$1]",
  "=10 $ categorical$1]",
  "$[10 $ categorical$1]]",
  "xxx[10 $ categorical$1]]",
  "?$10 $ categorical$1]]",
  "?=10 $ categorical$1]]",
  "?(10 $ categorical$1)$",
  "?(10 $ categorical$1)$",
  "10 = categorical$1]",
  "?10 = categorical$1]",
  "?(10 = categorical$1])",
  "$10 = categorical$1]",
  "=10 = categorical$1]",
  "$[10 = categorical$1]]",
  "xxx[10 = categorical$1]]",
  "?$10 = categorical$1]]",
  "?=10 = categorical$1]]",
  "?(10 = categorical$1)$",
  "?(10 = categorical$1)$",
  "10 * categorical=1]",
  "?10 * categorical=1]",
  "?(10 * categorical=1])",
  "$10 * categorical=1]",
  "=10 * categorical=1]",
  "$[10 * categorical=1]]",
  "xxx[10 * categorical=1]]",
  "?$10 * categorical=1]]",
  "?=10 * categorical=1]]",
  "?(10 * categorical=1)$",
  "?(10 * categorical=1)$",
  "categorical=1]",
  "?categorical=1]",
  "?(categorical=1])",
  "$categorical=1]",
  "=categorical=1]",
  "$[categorical=1]]",
  "xxx[categorical=1]]",
  "?$categorical=1]]",
  "?=categorical=1]]",
  "?(categorical=1)$",
  "?(categorical=1)$",
  "10 $ categorical=1]",
  "?10 $ categorical=1]",
  "?(10 $ categorical=1])",
  "$10 $ categorical=1]",
  "=10 $ categorical=1]",
  "$[10 $ categorical=1]]",
  "xxx[10 $ categorical=1]]",
  "?$10 $ categorical=1]]",
  "?=10 $ categorical=1]]",
  "?(10 $ categorical=1)$",
  "?(10 $ categorical=1)$",
  "10 = categorical=1]",
  "?10 = categorical=1]",
  "?(10 = categorical=1])",
  "$10 = categorical=1]",
  "=10 = categorical=1]",
  "$[10 = categorical=1]]",
  "xxx[10 = categorical=1]]",
  "?$10 = categorical=1]]",
  "?=10 = categorical=1]]",
  "?(10 = categorical=1)$",
  "?(10 = categorical=1)$",
  "10 * categorical[$]",
  "?10 * categorical[$]",
  "?[10 * categorical[$]]",
  "$10 * categorical[$]",
  "=10 * categorical[$]",
  "$[10 * categorical[$]]",
  "xxx[10 * categorical[$]]",
  "?$10 * categorical[$]]",
  "?=10 * categorical[$]]",
  "?[10 * categorical[$]$",
  "?[10 * categorical[$]$",
  "categorical[$]",
  "?categorical[$]",
  "?[categorical[$]]",
  "$categorical[$]",
  "=categorical[$]",
  "$[categorical[$]]",
  "xxx[categorical[$]]",
  "?$categorical[$]]",
  "?=categorical[$]]",
  "?[categorical[$]$",
  "?[categorical[$]$",
  "10 $ categorical[$]",
  "?10 $ categorical[$]",
  "?[10 $ categorical[$]]",
  "$10 $ categorical[$]",
  "=10 $ categorical[$]",
  "$[10 $ categorical[$]]",
  "xxx[10 $ categorical[$]]",
  "?$10 $ categorical[$]]",
  "?=10 $ categorical[$]]",
  "?[10 $ categorical[$]$",
  "?[10 $ categorical[$]$",
  "10 = categorical[$]",
  "?10 = categorical[$]",
  "?[10 = categorical[$]]",
  "$10 = categorical[$]",
  "=10 = categorical[$]",
  "$[10 = categorical[$]]",
  "xxx[10 = categorical[$]]",
  "?$10 = categorical[$]]",
  "?=10 = categorical[$]]",
  "?[10 = categorical[$]$",
  "?[10 = categorical[$]$",
  "10 * categorical[=]",
  "?10 * categorical[=]",
  "?[10 * categorical[=]]",
  "$10 * categorical[=]",
  "=10 * categorical[=]",
  "$[10 * categorical[=]]",
  "xxx[10 * categorical[=]]",
  "?$10 * categorical[=]]",
  "?=10 * categorical[=]]",
  "?[10 * categorical[=]$",
  "?[10 * categorical[=]$",
  "categorical[=]",
  "?categorical[=]",
  "?[categorical[=]]",
  "$categorical[=]",
  "=categorical[=]",
  "$[categorical[=]]",
  "xxx[categorical[=]]",
  "?$categorical[=]]",
  "?=categorical[=]]",
  "?[categorical[=]$",
  "?[categorical[=]$",
  "10 $ categorical[=]",
  "?10 $ categorical[=]",
  "?[10 $ categorical[=]]",
  "$10 $ categorical[=]",
  "=10 $ categorical[=]",
  "$[10 $ categorical[=]]",
  "xxx[10 $ categorical[=]]",
  "?$10 $ categorical[=]]",
  "?=10 $ categorical[=]]",
  "?[10 $ categorical[=]$",
  "?[10 $ categorical[=]$",
  "10 = categorical[=]",
  "?10 = categorical[=]",
  "?[10 = categorical[=]]",
  "$10 = categorical[=]",
  "=10 = categorical[=]",
  "$[10 = categorical[=]]",
  "xxx[10 = categorical[=]]",
  "?$10 = categorical[=]]",
  "?=10 = categorical[=]]",
  "?[10 = categorical[=]$",
  "?[10 = categorical[=]$",
  "10 * categorical['xxx']",
  "?10 * categorical['xxx']",
  "?[10 * categorical['xxx']]",
  "$10 * categorical['xxx']",
  "=10 * categorical['xxx']",
  "$[10 * categorical['xxx']]",
  "xxx[10 * categorical['xxx']]",
  "?$10 * categorical['xxx']]",
  "?=10 * categorical['xxx']]",
  "?[10 * categorical['xxx']$",
  "?[10 * categorical['xxx']$",
  "categorical['xxx']",
  "?categorical['xxx']",
  "?[categorical['xxx']]",
  "$categorical['xxx']",
  "=categorical['xxx']",
  "$[categorical['xxx']]",
  "xxx[categorical['xxx']]",
  "?$categorical['xxx']]",
  "?=categorical['xxx']]",
  "?[categorical['xxx']$",
  "?[categorical['xxx']$",
  "10 $ categorical['xxx']",
  "?10 $ categorical['xxx']",
  "?[10 $ categorical['xxx']]",
  "$10 $ categorical['xxx']",
  "=10 $ categorical['xxx']",
  "$[10 $ categorical['xxx']]",
  "xxx[10 $ categorical['xxx']]",
  "?$10 $ categorical['xxx']]",
  "?=10 $ categorical['xxx']]",
  "?[10 $ categorical['xxx']$",
  "?[10 $ categorical['xxx']$",
  "10 = categorical['xxx']",
  "?10 = categorical['xxx']",
  "?[10 = categorical['xxx']]",
  "$10 = categorical['xxx']",
  "=10 = categorical['xxx']",
  "$[10 = categorical['xxx']]",
  "xxx[10 = categorical['xxx']]",
  "?$10 = categorical['xxx']]",
  "?=10 = categorical['xxx']]",
  "?[10 = categorical['xxx']$",
  "?[10 = categorical['xxx']$",
  "10 * categorical[10000000000000000000000000000000000000000000]",
  "?10 * categorical[10000000000000000000000000000000000000000000]",
  "?[10 * categorical[10000000000000000000000000000000000000000000]]",
  "$10 * categorical[10000000000000000000000000000000000000000000]",
  "=10 * categorical[10000000000000000000000000000000000000000000]",
  "$[10 * categorical[10000000000000000000000000000000000000000000]]",
  "xxx[10 * categorical[10000000000000000000000000000000000000000000]]",
  "?$10 * categorical[10000000000000000000000000000000000000000000]]",
  "?=10 * categorical[10000000000000000000000000000000000000000000]]",
  "?[10 * categorical[10000000000000000000000000000000000000000000]$",
  "?[10 * categorical[10000000000000000000000000000000000000000000]$",
  "categorical[10000000000000000000000000000000000000000000]",
  "?categorical[10000000000000000000000000000000000000000000]",
  "?[categorical[10000000000000000000000000000000000000000000]]",
  "$categorical[10000000000000000000000000000000000000000000]",
  "=categorical[10000000000000000000000000000000000000000000]",
  "$[categorical[10000000000000000000000000000000000000000000]]",
  "xxx[categorical[10000000000000000000000000000000000000000000]]",
  "?$categorical[10000000000000000000000000000000000000000000]]",
  "?=categorical[10000000000000000000000000000000000000000000]]",
  "?[categorical[10000000000000000000000000000000000000000000]$",
  "?[categorical[10000000000000000000000000000000000000000000]$",
  "10 $ categorical[10000000000000000000000000000000000000000000]",
  "?10 $ categorical[10000000000000000000000000000000000000000000]",
  "?[10 $ categorical[10000000000000000000000000000000000000000000]]",
  "$10 $ categorical[10000000000000000000000000000000000000000000]",
  "=10 $ categorical[10000000000000000000000000000000000000000000]",
  "$[10 $ categorical[10000000000000000000000000000000000000000000]]",
  "xxx[10 $ categorical[10000000000000000000000000000000000000000000]]",
  "?$10 $ categorical[10000000000000000000000000000000000000000000]]",
  "?=10 $ categorical[10000000000000000000000000000000000000000000]]",
  "?[10 $ categorical[10000000000000000000000000000000000000000000]$",
  "?[10 $ categorical[10000000000000000000000000000000000000000000]$",
  "10 = categorical[10000000000000000000000000000000000000000000]",
  "?10 = categorical[10000000000000000000000000000000000000000000]",
  "?[10 = categorical[10000000000000000000000000000000000000000000]]",
  "$10 = categorical[10000000000000000000000000000000000000000000]",
  "=10 = categorical[10000000000000000000000000000000000000000000]",
  "$[10 = categorical[10000000000000000000000000000000000000000000]]",
  "xxx[10 = categorical[10000000000000000000000000000000000000000000]]",
  "?$10 = categorical[10000000000000000000000000000000000000000000]]",
  "?=10 = categorical[10000000000000000000000000000000000000000000]]",
  "?[10 = categorical[10000000000000000000000000000000000000000000]$",
  "?[10 = categorical[10000000000000000000000000000000000000000000]$",
  "10 * categorical[1 $ int64]",
  "?10 * categorical[1 $ int64]",
  "?[10 * categorical[1 $ int64]]",
  "$10 * categorical[1 $ int64]",
  "=10 * categorical[1 $ int64]",
  "$[10 * categorical[1 $ int64]]",
  "xxx[10 * categorical[1 $ int64]]",
  "?$10 * categorical[1 $ int64]]",
  "?=10 * categorical[1 $ int64]]",
  "?[10 * categorical[1 $ int64]$",
  "?[10 * categorical[1 $ int64]$",
  "categorical[1 $ int64]",
  "?categorical[1 $ int64]",
  "?[categorical[1 $ int64]]",
  "$categorical[1 $ int64]",
  "=categorical[1 $ int64]",
  "$[categorical[1 $ int64]]",
  "xxx[categorical[1 $ int64]]",
  "?$categorical[1 $ int64]]",
  "?=categorical[1 $ int64]]",
  "?[categorical[1 $ int64]$",
  "?[categorical[1 $ int64]$",
  "10 $ categorical[1 $ int64]",
  "?10 $ categorical[1 $ int64]",
  "?[10 $ categorical[1 $ int64]]",
  "$10 $ categorical[1 $ int64]",
  "=10 $ categorical[1 $ int64]",
  "$[10 $ categorical[1 $ int64]]",
  "xxx[10 $ categorical[1 $ int64]]",
  "?$10 $ categorical[1 $ int64]]",
  "?=10 $ categorical[1 $ int64]]",
  "?[10 $ categorical[1 $ int64]$",
  "?[10 $ categorical[1 $ int64]$",
  "10 = categorical[1 $ int64]",
  "?10 = categorical[1 $ int64]",
  "?[10 = categorical[1 $ int64]]",
  "$10 = categorical[1 $ int64]",
  "=10 = categorical[1 $ int64]",
  "$[10 = categorical[1 $ int64]]",
  "xxx[10 = categorical[1 $ int64]]",
  "?$10 = categorical[1 $ int64]]",
  "?=10 = categorical[1 $ int64]]",
  "?[10 = categorical[1 $ int64]$",
  "?[10 = categorical[1 $ int64]$",
  "10 * categorical[1 = int64]",
  "?10 * categorical[1 = int64]",
  "?[10 * categorical[1 = int64]]",
  "$10 * categorical[1 = int64]",
  "=10 * categorical[1 = int64]",
  "$[10 * categorical[1 = int64]]",
  "xxx[10 * categorical[1 = int64]]",
  "?$10 * categorical[1 = int64]]",
  "?=10 * categorical[1 = int64]]",
  "?[10 * categorical[1 = int64]$",
  "?[10 * categorical[1 = int64]$",
  "categorical[1 = int64]",
  "?categorical[1 = int64]",
  "?[categorical[1 = int64]]",
  "$categorical[1 = int64]",
  "=categorical[1 = int64]",
  "$[categorical[1 = int64]]",
  "xxx[categorical[1 = int64]]",
  "?$categorical[1 = int64]]",
  "?=categorical[1 = int64]]",
  "?[categorical[1 = int64]$",
  "?[categorical[1 = int64]$",
  "10 $ categorical[1 = int64]",
  "?10 $ categorical[1 = int64]",
  "?[10 $ categorical[1 = int64]]",
  "$10 $ categorical[1 = int64]",
  "=10 $ categorical[1 = int64]",
  "$[10 $ categorical[1 = int64]]",
  "xxx[10 $ categorical[1 = int64]]",
  "?$10 $ categorical[1 = int64]]",
  "?=10 $ categorical[1 = int64]]",
  "?[10 $ categorical[1 = int64]$",
  "?[10 $ categorical[1 = int64]$",
  "10 = categorical[1 = int64]",
  "?10 = categorical[1 = int64]",
  "?[10 = categorical[1 = int64]]",
  "$10 = categorical[1 = int64]",
  "=10 = categorical[1 = int64]",
  "$[10 = categorical[1 = int64]]",
  "xxx[10 = categorical[1 = int64]]",
  "?$10 = categorical[1 = int64]]",
  "?=10 = categorical[1 = int64]]",
  "?[10 = categorical[1 = int64]$",
  "?[10 = categorical[1 = int64]$",
  "10 * categorical[1 'xxx' int64]",
  "?10 * categorical[1 'xxx' int64]",
  "?[10 * categorical[1 'xxx' int64]]",
  "$10 * categorical[1 'xxx' int64]",
  "=10 * categorical[1 'xxx' int64]",
  "$[10 * categorical[1 'xxx' int64]]",
  "xxx[10 * categorical[1 'xxx' int64]]",
  "?$10 * categorical[1 'xxx' int64]]",
  "?=10 * categorical[1 'xxx' int64]]",
  "?[10 * categorical[1 'xxx' int64]$",
  "?[10 * categorical[1 'xxx' int64]$",
  "categorical[1 'xxx' int64]",
  "?categorical[1 'xxx' int64]",
  "?[categorical[1 'xxx' int64]]",
  "$categorical[1 'xxx' int64]",
  "=categorical[1 'xxx' int64]",
  "$[categorical[1 'xxx' int64]]",
  "xxx[categorical[1 'xxx' int64]]",
  "?$categorical[1 'xxx' int64]]",
  "?=categorical[1 'xxx' int64]]",
  "?[categorical[1 'xxx' int64]$",
  "?[categorical[1 'xxx' int64]$",
  "10 $ categorical[1 'xxx' int64]",
  "?10 $ categorical[1 'xxx' int64]",
  "?[10 $ categorical[1 'xxx' int64]]",
  "$10 $ categorical[1 'xxx' int64]",
  "=10 $ categorical[1 'xxx' int64]",
  "$[10 $ categorical[1 'xxx' int64]]",
  "xxx[10 $ categorical[1 'xxx' int64]]",
  "?$10 $ categorical[1 'xxx' int64]]",
  "?=10 $ categorical[1 'xxx' int64]]",
  "?[10 $ categorical[1 'xxx' int64]$",
  "?[10 $ categorical[1 'xxx' int64]$",
  "10 = categorical[1 'xxx' int64]",
  "?10 = categorical[1 'xxx' int64]",
  "?[10 = categorical[1 'xxx' int64]]",
  "$10 = categorical[1 'xxx' int64]",
  "=10 = categorical[1 'xxx' int64]",
  "$[10 = categorical[1 'xxx' int64]]",
  "xxx[10 = categorical[1 'xxx' int64]]",
  "?$10 = categorical[1 'xxx' int64]]",
  "?=10 = categorical[1 'xxx' int64]]",
  "?[10 = categorical[1 'xxx' int64]$",
  "?[10 = categorical[1 'xxx' int64]$",
  "10 * categorical[1 : $]",
  "?10 * categorical[1 : $]",
  "?[10 * categorical[1 : $]]",
  "$10 * categorical[1 : $]",
  "=10 * categorical[1 : $]",
  "$[10 * categorical[1 : $]]",
  "xxx[10 * categorical[1 : $]]",
  "?$10 * categorical[1 : $]]",
  "?=10 * categorical[1 : $]]",
  "?[10 * categorical[1 : $]$",
  "?[10 * categorical[1 : $]$",
  "categorical[1 : $]",
  "?categorical[1 : $]",
  "?[categorical[1 : $]]",
  "$categorical[1 : $]",
  "=categorical[1 : $]",
  "$[categorical[1 : $]]",
  "xxx[categorical[1 : $]]",
  "?$categorical[1 : $]]",
  "?=categorical[1 : $]]",
  "?[categorical[1 : $]$",
  "?[categorical[1 : $]$",
  "10 $ categorical[1 : $]",
  "?10 $ categorical[1 : $]",
  "?[10 $ categorical[1 : $]]",
  "$10 $ categorical[1 : $]",
  "=10 $ categorical[1 : $]",
  "$[10 $ categorical[1 : $]]",
  "xxx[10 $ categorical[1 : $]]",
  "?$10 $ categorical[1 : $]]",
  "?=10 $ categorical[1 : $]]",
  "?[10 $ categorical[1 : $]$",
  "?[10 $ categorical[1 : $]$",
  "10 = categorical[1 : $]",
  "?10 = categorical[1 : $]",
  "?[10 = categorical[1 : $]]",
  "$10 = categorical[1 : $]",
  "=10 = categorical[1 : $]",
  "$[10 = categorical[1 : $]]",
  "xxx[10 = categorical[1 : $]]",
  "?$10 = categorical[1 : $]]",
  "?=10 = categorical[1 : $]]",
  "?[10 = categorical[1 : $]$",
  "?[10 = categorical[1 : $]$",
  "10 * categorical[1 : $]",
  "?10 * categorical[1 : $]",
  "?[10 * categorical[1 : $]]",
  "$10 * categorical[1 : $]",
  "=10 * categorical[1 : $]",
  "$[10 * categorical[1 : $]]",
  "xxx[10 * categorical[1 : $]]",
  "?$10 * categorical[1 : $]]",
  "?=10 * categorical[1 : $]]",
  "?[10 * categorical[1 : $]$",
  "?[10 * categorical[1 : $]$",
  "categorical[1 : $]",
  "?categorical[1 : $]",
  "?[categorical[1 : $]]",
  "$categorical[1 : $]",
  "=categorical[1 : $]",
  "$[categorical[1 : $]]",
  "xxx[categorical[1 : $]]",
  "?$categorical[1 : $]]",
  "?=categorical[1 : $]]",
  "?[categorical[1 : $]$",
  "?[categorical[1 : $]$",
  "10 $ categorical[1 : $]",
  "?10 $ categorical[1 : $]",
  "?[10 $ categorical[1 : $]]",
  "$10 $ categorical[1 : $]",
  "=10 $ categorical[1 : $]",
  "$[10 $ categorical[1 : $]]",
  "xxx[10 $ categorical[1 : $]]",
  "?$10 $ categorical[1 : $]]",
  "?=10 $ categorical[1 : $]]",
  "?[10 $ categorical[1 : $]$",
  "?[10 $ categorical[1 : $]$",
  "10 = categorical[1 : $]",
  "?10 = categorical[1 : $]",
  "?[10 = categorical[1 : $]]",
  "$10 = categorical[1 : $]",
  "=10 = categorical[1 : $]",
  "$[10 = categorical[1 : $]]",
  "xxx[10 = categorical[1 : $]]",
  "?$10 = categorical[1 : $]]",
  "?=10 = categorical[1 : $]]",
  "?[10 = categorical[1 : $]$",
  "?[10 = categorical[1 : $]$",
  "10 * categorical[1$",
  "?10 * categorical[1$",
  "?[10 * categorical[1$]",
  "$10 * categorical[1$",
  "=10 * categorical[1$",
  "$[10 * categorical[1$]",
  "xxx[10 * categorical[1$]",
  "?$10 * categorical[1$]",
  "?=10 * categorical[1$]",
  "?[10 * categorical[1$$",
  "?[10 * categorical[1$$",
  "categorical[1$",
  "?categorical[1$",
  "?[categorical[1$]",
  "$categorical[1$",
  "=categorical[1$",
  "$[categorical[1$]",
  "xxx[categorical[1$]",
  "?$categorical[1$]",
  "?=categorical[1$]",
  "?[categorical[1$$",
  "?[categorical[1$$",
  "10 $ categorical[1$",
  "?10 $ categorical[1$",
  "?[10 $ categorical[1$]",
  "$10 $ categorical[1$",
  "=10 $ categorical[1$",
  "$[10 $ categorical[1$]",
  "xxx[10 $ categorical[1$]",
  "?$10 $ categorical[1$]",
  "?=10 $ categorical[1$]",
  "?[10 $ categorical[1$$",
  "?[10 $ categorical[1$$",
  "10 = categorical[1$",
  "?10 = categorical[1$",
  "?[10 = categorical[1$]",
  "$10 = categorical[1$",
  "=10 = categorical[1$",
  "$[10 = categorical[1$]",
  "xxx[10 = categorical[1$]",
  "?$10 = categorical[1$]",
  "?=10 = categorical[1$]",
  "?[10 = categorical[1$$",
  "?[10 = categorical[1$$",
  "10 * categorical[1$",
  "?10 * categorical[1$",
  "?[10 * categorical[1$]",
  "$10 * categorical[1$",
  "=10 * categorical[1$",
  "$[10 * categorical[1$]",
  "xxx[10 * categorical[1$]",
  "?$10 * categorical[1$]",
  "?=10 * categorical[1$]",
  "?[10 * categorical[1$$",
  "?[10 * categorical[1$$",
  "categorical[1$",
  "?categorical[1$",
  "?[categorical[1$]",
  "$categorical[1$",
  "=categorical[1$",
  "$[categorical[1$]",
  "xxx[categorical[1$]",
  "?$categorical[1$]",
  "?=categorical[1$]",
  "?[categorical[1$$",
  "?[categorical[1$$",
  "10 $ categorical[1$",
  "?10 $ categorical[1$",
  "?[10 $ categorical[1$]",
  "$10 $ categorical[1$",
  "=10 $ categorical[1$",
  "$[10 $ categorical[1$]",
  "xxx[10 $ categorical[1$]",
  "?$10 $ categorical[1$]",
  "?=10 $ categorical[1$]",
  "?[10 $ categorical[1$$",
  "?[10 $ categorical[1$$",
  "10 = categorical[1$",
  "?10 = categorical[1$",
  "?[10 = categorical[1$]",
  "$10 = categorical[1$",
  "=10 = categorical[1$",
  "$[10 = categorical[1$]",
  "xxx[10 = categorical[1$]",
  "?$10 = categorical[1$]",
  "?=10 = categorical[1$]",
  "?[10 = categorical[1$$",
  "?[10 = categorical[1$$",
  "10 * categorical[1 'xxx'",
  "?10 * categorical[1 'xxx'",
  "?[10 * categorical[1 'xxx']",
  "$10 * categorical[1 'xxx'",
  "=10 * categorical[1 'xxx'",
  "$[10 * categorical[1 'xxx']",
  "xxx[10 * categorical[1 'xxx']",
  "?$10 * categorical[1 'xxx']",
  "?=10 * categorical[1 'xxx']",
  "?[10 * categorical[1 'xxx'$",
  "?[10 * categorical[1 'xxx'$",
  "categorical[1 'xxx'",
  "?categorical[1 'xxx'",
  "?[categorical[1 'xxx']",
  "$categorical[1 'xxx'",
  "=categorical[1 'xxx'",
  "$[categorical[1 'xxx']",
  "xxx[categorical[1 'xxx']",
  "?$categorical[1 'xxx']",
  "?=categorical[1 'xxx']",
  "?[categorical[1 'xxx'$",
  "?[categorical[1 'xxx'$",
  "10 $ categorical[1 'xxx'",
  "?10 $ categorical[1 'xxx'",
  "?[10 $ categorical[1 'xxx']",
  "$10 $ categorical[1 'xxx'",
  "=10 $ categorical[1 'xxx'",
  "$[10 $ categorical[1 'xxx']",
  "xxx[10 $ categorical[1 'xxx']",
  "?$10 $ categorical[1 'xxx']",
  "?=10 $ categorical[1 'xxx']",
  "?[10 $ categorical[1 'xxx'$",
  "?[10 $ categorical[1 'xxx'$",
  "10 = categorical[1 'xxx'",
  "?10 = categorical[1 'xxx'",
  "?[10 = categorical[1 'xxx']",
  "$10 = categorical[1 'xxx'",
  "=10 = categorical[1 'xxx'",
  "$[10 = categorical[1 'xxx']",
  "xxx[10 = categorical[1 'xxx']",
  "?$10 = categorical[1 'xxx']",
  "?=10 = categorical[1 'xxx']",
  "?[10 = categorical[1 'xxx'$",
  "?[10 = categorical[1 'xxx'$",
  "$10 * ref(Any)",
  "=10 * ref(Any)",
  "$[10 * ref(Any])",
  "xxx[10 * ref(Any])",
  "?$10 * ref(Any])",
  "?=10 * ref(Any])",
  "?[10 * ref(Any)$",
  "?[10 * ref(Any)$",
  "$ref(Any)",
  "=ref(Any)",
  "$[ref(Any])",
  "xxx[ref(Any])",
  "?$ref(Any])",
  "?=ref(Any])",
  "?[ref(Any)$",
  "?[ref(Any)$",
  "10 $ ref(Any)",
  "?10 $ ref(Any)",
  "?(10 $ ref(Any))",
  "$10 $ ref(Any)",
  "=10 $ ref(Any)",
  "$[10 $ ref(Any])",
  "xxx[10 $ ref(Any])",
  "?$10 $ ref(Any])",
  "?=10 $ ref(Any])",
  "?[10 $ ref(Any)$",
  "?[10 $ ref(Any)$",
  "10 = ref(Any)",
  "?10 = ref(Any)",
  "?(10 = ref(Any))",
  "$10 = ref(Any)",
  "=10 = ref(Any)",
  "$[10 = ref(Any])",
  "xxx[10 = ref(Any])",
  "?$10 = ref(Any])",
  "?=10 = ref(Any])",
  "?[10 = ref(Any)$",
  "?[10 = ref(Any)$",
  "10 * $[int64]",
  "?10 * $[int64]",
  "?[10 * $[int64]]",
  "$10 * $[int64]",
  "=10 * $[int64]",
  "$[10 * $[int64]]",
  "xxx[10 * $[int64]]",
  "?$10 * $[int64]]",
  "?=10 * $[int64]]",
  "?[10 * $[int64]$",
  "?[10 * $[int64]$",
  "$[int64]",
  "?$[int64]",
  "?[$[int64]]",
  "$$[int64]",
  "=$[int64]",
  "$[$[int64]]",
  "xxx[$[int64]]",
  "?$$[int64]]",
  "?=$[int64]]",
  "?[$[int64]$",
  "?[$[int64]$",
  "10 $ $[int64]",
  "?10 $ $[int64]",
  "?[10 $ $[int64]]",
  "$10 $ $[int64]",
  "=10 $ $[int64]",
  "$[10 $ $[int64]]",
  "xxx[10 $ $[int64]]",
  "?$10 $ $[int64]]",
  "?=10 $ $[int64]]",
  "?[10 $ $[int64]$",
  "?[10 $ $[int64]$",
  "10 = $[int64]",
  "?10 = $[int64]",
  "?[10 = $[int64]]",
  "$10 = $[int64]",
  "=10 = $[int64]",
  "$[10 = $[int64]]",
  "xxx[10 = $[int64]]",
  "?$10 = $[int64]]",
  "?=10 = $[int64]]",
  "?[10 = $[int64]$",
  "?[10 = $[int64]$",
  "10 * =[int64]",
  "?10 * =[int64]",
  "?[10 * =[int64]]",
  "$10 * =[int64]",
  "=10 * =[int64]",
  "$[10 * =[int64]]",
  "xxx[10 * =[int64]]",
  "?$10 * =[int64]]",
  "?=10 * =[int64]]",
  "?[10 * =[int64]$",
  "?[10 * =[int64]$",
  "=[int64]",
  "?=[int64]",
  "?[=[int64]]",
  "$=[int64]",
  "==[int64]",
  "$[=[int64]]",
  "xxx[=[int64]]",
  "?$=[int64]]",
  "?==[int64]]",
  "?[=[int64]$",
  "?[=[int64]$",
  "10 $ =[int64]",
  "?10 $ =[int64]",
  "?[10 $ =[int64]]",
  "$10 $ =[int64]",
  "=10 $ =[int64]",
  "$[10 $ =[int64]]",
  "xxx[10 $ =[int64]]",
  "?$10 $ =[int64]]",
  "?=10 $ =[int64]]",
  "?[10 $ =[int64]$",
  "?[10 $ =[int64]$",
  "10 = =[int64]",
  "?10 = =[int64]",
  "?[10 = =[int64]]",
  "$10 = =[int64]",
  "=10 = =[int64]",
  "$[10 = =[int64]]",
  "xxx[10 = =[int64]]",
  "?$10 = =[int64]]",
  "?=10 = =[int64]]",
  "?[10 = =[int64]$",
  "?[10 = =[int64]$",
  "10 * xxx[int64]",
  "?10 * xxx[int64]",
  "?[10 * xxx[int64]]",
  "$10 * xxx[int64]",
  "=10 * xxx[int64]",
  "$[10 * xxx[int64]]",
  "xxx[10 * xxx[int64]]",
  "?$10 * xxx[int64]]",
  "?=10 * xxx[int64]]",
  "?[10 * xxx[int64]$",
  "?[10 * xxx[int64]$",
  "xxx[int64]",
  "?xxx[int64]",
  "?[xxx[int64]]",
  "$xxx[int64]",
  "=xxx[int64]",
  "$[xxx[int64]]",
  "xxx[xxx[int64]]",
  "?$xxx[int64]]",
  "?=xxx[int64]]",
  "?[xxx[int64]$",
  "?[xxx[int64]$",
  "10 $ xxx[int64]",
  "?10 $ xxx[int64]",
  "?[10 $ xxx[int64]]",
  "$10 $ xxx[int64]",
  "=10 $ xxx[int64]",
  "$[10 $ xxx[int64]]",
  "xxx[10 $ xxx[int64]]",
  "?$10 $ xxx[int64]]",
  "?=10 $ xxx[int64]]",
  "?[10 $ xxx[int64]$",
  "?[10 $ xxx[int64]$",
  "10 = xxx[int64]",
  "?10 = xxx[int64]",
  "?[10 = xxx[int64]]",
  "$10 = xxx[int64]",
  "=10 = xxx[int64]",
  "$[10 = xxx[int64]]",
  "xxx[10 = xxx[int64]]",
  "?$10 = xxx[int64]]",
  "?=10 = xxx[int64]]",
  "?[10 = xxx[int64]$",
  "?[10 = xxx[int64]$",
  "10 * ref$int64]",
  "?10 * ref$int64]",
  "?(10 * ref$int64])",
  "$10 * ref$int64]",
  "=10 * ref$int64]",
  "$[10 * ref$int64]]",
  "xxx[10 * ref$int64]]",
  "?$10 * ref$int64]]",
  "?=10 * ref$int64]]",
  "?(10 * ref$int64)$",
  "?(10 * ref$int64)$",
  "ref$int64]",
  "?ref$int64]",
  "?(ref$int64])",
  "$ref$int64]",
  "=ref$int64]",
  "$[ref$int64]]",
  "xxx[ref$int64]]",
  "?$ref$int64]]",
  "?=ref$int64]]",
  "?(ref$int64)$",
  "?(ref$int64)$",
  "10 $ ref$int64]",
  "?10 $ ref$int64]",
  "?(10 $ ref$int64])",
  "$10 $ ref$int64]",
  "=10 $ ref$int64]",
  "$[10 $ ref$int64]]",
  "xxx[10 $ ref$int64]]",
  "?$10 $ ref$int64]]",
  "?=10 $ ref$int64]]",
  "?(10 $ ref$int64)$",
  "?(10 $ ref$int64)$",
  "10 = ref$int64]",
  "?10 = ref$int64]",
  "?(10 = ref$int64])",
  "$10 = ref$int64]",
  "=10 = ref$int64]",
  "$[10 = ref$int64]]",
  "xxx[10 = ref$int64]]",
  "?$10 = ref$int64]]",
  "?=10 = ref$int64]]",
  "?(10 = ref$int64)$",
  "?(10 = ref$int64)$",
  "10 * ref^int64]",
  "?10 * ref^int64]",
  "?(10 * ref^int64])",
  "$10 * ref^int64]",
  "=10 * ref^int64]",
  "$[10 * ref^int64]]",
  "xxx[10 * ref^int64]]",
  "?$10 * ref^int64]]",
  "?=10 * ref^int64]]",
  "?(10 * ref^int64)$",
  "?(10 * ref^int64)$",
  "ref^int64]",
  "?ref^int64]",
  "?(ref^int64])",
  "$ref^int64]",
  "=ref^int64]",
  "$[ref^int64]]",
  "xxx[ref^int64]]",
  "?$ref^int64]]",
  "?=ref^int64]]",
  "?(ref^int64)$",
  "?(ref^int64)$",
  "10 $ ref^int64]",
  "?10 $ ref^int64]",
  "?(10 $ ref^int64])",
  "$10 $ ref^int64]",
  "=10 $ ref^int64]",
  "$[10 $ ref^int64]]",
  "xxx[10 $ ref^int64]]",
  "?$10 $ ref^int64]]",
  "?=10 $ ref^int64]]",
  "?(10 $ ref^int64)$",
  "?(10 $ ref^int64)$",
  "10 = ref^int64]",
  "?10 = ref^int64]",
  "?(10 = ref^int64])",
  "$10 = ref^int64]",
  "=10 = ref^int64]",
  "$[10 = ref^int64]]",
  "xxx[10 = ref^int64]]",
  "?$10 = ref^int64]]",
  "?=10 = ref^int64]]",
  "?(10 = ref^int64)$",
  "?(10 = ref^int64)$",
  "10 * ref($)",
  "?10 * ref($)",
  "?(10 * ref($))",
  "$10 * ref($)",
  "=10 * ref($)",
  "$[10 * ref($])",
  "xxx[10 * ref($])",
  "?$10 * ref($])",
  "?=10 * ref($])",
  "?[10 * ref($)$",
  "?[10 * ref($)$",
  "ref($)",
  "?ref($)",
  "?(ref($))",
  "$ref($)",
  "=ref($)",
  "$[ref($])",
  "xxx[ref($])",
  "?$ref($])",
  "?=ref($])",
  "?[ref($)$",
  "?[ref($)$",
  "10 $ ref($)",
  "?10 $ ref($)",
  "?(10 $ ref($))",
  "$10 $ ref($)",
  "=10 $ ref($)",
  "$[10 $ ref($])",
  "xxx[10 $ ref($])",
  "?$10 $ ref($])",
  "?=10 $ ref($])",
  "?[10 $ ref($)$",
  "?[10 $ ref($)$",
  "10 = ref($)",
  "?10 = ref($)",
  "?(10 = ref($))",
  "$10 = ref($)",
  "=10 = ref($)",
  "$[10 = ref($])",
  "xxx[10 = ref($])",
  "?$10 = ref($])",
  "?=10 = ref($])",
  "?[10 = ref($)$",
  "?[10 = ref($)$",
  "10 * ref(xxx)",
  "?10 * ref(xxx)",
  "?(10 * ref(xxx))",
  "$10 * ref(xxx)",
  "=10 * ref(xxx)",
  "$[10 * ref(xxx])",
  "xxx[10 * ref(xxx])",
  "?$10 * ref(xxx])",
  "?=10 * ref(xxx])",
  "?[10 * ref(xxx)$",
  "?[10 * ref(xxx)$",
  "ref(xxx)",
  "?ref(xxx)",
  "?(ref(xxx))",
  "$ref(xxx)",
  "=ref(xxx)",
  "$[ref(xxx])",
  "xxx[ref(xxx])",
  "?$ref(xxx])",
  "?=ref(xxx])",
  "?[ref(xxx)$",
  "?[ref(xxx)$",
  "10 $ ref(xxx)",
  "?10 $ ref(xxx)",
  "?(10 $ ref(xxx))",
  "$10 $ ref(xxx)",
  "=10 $ ref(xxx)",
  "$[10 $ ref(xxx])",
  "xxx[10 $ ref(xxx])",
  "?$10 $ ref(xxx])",
  "?=10 $ ref(xxx])",
  "?[10 $ ref(xxx)$",
  "?[10 $ ref(xxx)$",
  "10 = ref(xxx)",
  "?10 = ref(xxx)",
  "?(10 = ref(xxx))",
  "$10 = ref(xxx)",
  "=10 = ref(xxx)",
  "$[10 = ref(xxx])",
  "xxx[10 = ref(xxx])",
  "?$10 = ref(xxx])",
  "?=10 = ref(xxx])",
  "?[10 = ref(xxx)$",
  "?[10 = ref(xxx)$",
  "10 * ref[int64$",
  "?10 * ref[int64$",
  "?[10 * ref(int64$)",
  "$10 * ref[int64$",
  "=10 * ref[int64$",
  "$[10 * ref(int64$)",
  "xxx[10 * ref(int64$)",
  "?$10 * ref(int64$)",
  "?=10 * ref(int64$)",
  "?[10 * ref[int64$$",
  "?[10 * ref[int64$$",
  "ref[int64$",
  "?ref[int64$",
  "?[ref(int64$)",
  "$ref[int64$",
  "=ref[int64$",
  "$[ref(int64$)",
  "xxx[ref(int64$)",
  "?$ref(int64$)",
  "?=ref(int64$)",
  "?[ref[int64$$",
  "?[ref[int64$$",
  "10 $ ref[int64$",
  "?10 $ ref[int64$",
  "?[10 $ ref(int64$)",
  "$10 $ ref[int64$",
  "=10 $ ref[int64$",
  "$[10 $ ref(int64$)",
  "xxx[10 $ ref(int64$)",
  "?$10 $ ref(int64$)",
  "?=10 $ ref(int64$)",
  "?[10 $ ref[int64$$",
  "?[10 $ ref[int64$$",
  "10 = ref[int64$",
  "?10 = ref[int64$",
  "?[10 = ref(int64$)",
  "$10 = ref[int64$",
  "=10 = ref[int64$",
  "$[10 = ref(int64$)",
  "xxx[10 = ref(int64$)",
  "?$10 = ref(int64$)",
  "?=10 = ref(int64$)",
  "?[10 = ref[int64$$",
  "?[10 = ref[int64$$",
  "10 * ref[int64$",
  "?10 * ref[int64$",
  "?[10 * ref(int64$)",
  "$10 * ref[int64$",
  "=10 * ref[int64$",
  "$[10 * ref(int64$)",
  "xxx[10 * ref(int64$)",
  "?$10 * ref(int64$)",
  "?=10 * ref(int64$)",
  "?[10 * ref[int64$$",
  "?[10 * ref[int64$$",
  "ref[int64$",
  "?ref[int64$",
  "?[ref(int64$)",
  "$ref[int64$",
  "=ref[int64$",
  "$[ref(int64$)",
  "xxx[ref(int64$)",
  "?$ref(int64$)",
  "?=ref(int64$)",
  "?[ref[int64$$",
  "?[ref[int64$$",
  "10 $ ref[int64$",
  "?10 $ ref[int64$",
  "?[10 $ ref(int64$)",
  "$10 $ ref[int64$",
  "=10 $ ref[int64$",
  "$[10 $ ref(int64$)",
  "xxx[10 $ ref(int64$)",
  "?$10 $ ref(int64$)",
  "?=10 $ ref(int64$)",
  "?[10 $ ref[int64$$",
  "?[10 $ ref[int64$$",
  "10 = ref[int64$",
  "?10 = ref[int64$",
  "?[10 = ref(int64$)",
  "$10 = ref[int64$",
  "=10 = ref[int64$",
  "$[10 = ref(int64$)",
  "xxx[10 = ref(int64$)",
  "?$10 = ref(int64$)",
  "?=10 = ref(int64$)",
  "?[10 = ref[int64$$",
  "?[10 = ref[int64$$",
  "10 * $",
  "?10 * $",
  "?(10 * $)",
  "$10 * $",
  "=10 * $",
  "$[10 * $]",
  "xxx[10 * $]",
  "?$10 * $]",
  "?=10 * $]",
  "?[10 * $$",
  "?[10 * $$",
  "$",
  "?$",
  "?($)",
  "$$",
  "=$",
  "$[$]",
  "xxx[$]",
  "?$$]",
  "?=$]",
  "?[$$",
  "?[$$",
  "10 $ $",
  "?10 $ $",
  "?(10 $ $)",
  "$10 $ $",
  "=10 $ $",
  "$[10 $ $]",
  "xxx[10 $ $]",
  "?$10 $ $]",
  "?=10 $ $]",
  "?[10 $ $$",
  "?[10 $ $$",
  "10 = $",
  "?10 = $",
  "?(10 = $)",
  "$10 = $",
  "=10 = $",
  "$[10 = $]",
  "xxx[10 = $]",
  "?$10 = $]",
  "?=10 = $]",
  "?[10 = $$",
  "?[10 = $$",
  "10 * xxx",
  "?10 * xxx",
  "?(10 * xxx)",
  "$10 * xxx",
  "=10 * xxx",
  "$[10 * xxx]",
  "xxx[10 * xxx]",
  "?$10 * xxx]",
  "?=10 * xxx]",
  "?[10 * xxx$",
  "?[10 * xxx$",
  "xxx",
  "?xxx",
  "?(xxx)",
  "$xxx",
  "=xxx",
  "$[xxx]",
  "xxx[xxx]",
  "?$xxx]",
  "?=xxx]",
  "?[xxx$",
  "?[xxx$",
  "10 $ xxx",
  "?10 $ xxx",
  "?(10 $ xxx)",
  "$10 $ xxx",
  "=10 $ xxx",
  "$[10 $ xxx]",
  "xxx[10 $ xxx]",
  "?$10 $ xxx]",
  "?=10 $ xxx]",
  "?[10 $ xxx$",
  "?[10 $ xxx$",
  "10 = xxx",
  "?10 = xxx",
  "?(10 = xxx)",
  "$10 = xxx",
  "=10 = xxx",
  "$[10 = xxx]",
  "xxx[10 = xxx]",
  "?$10 = xxx]",
  "?=10 = xxx]",
  "?[10 = xxx$",
  "?[10 = xxx$",
  "$10 * (...)",
  "=10 * (...)",
  "$[10 * (...)]",
  "xxx[10 * (...)]",
  "?$10 * (...)]",
  "?=10 * (...)]",
  "?[10 * (...)$",
  "?[10 * (...)$",
  "$(...)",
  "=(...)",
  "$[(...)]",
  "xxx[(...)]",
  "?$(...)]",
  "?=(...)]",
  "?[(...)$",
  "?[(...)$",
  "10 $ (...)",
  "?10 $ (...)",
  "?(10 $ (...))",
  "$10 $ (...)",
  "=10 $ (...)",
  "$[10 $ (...)]",
  "xxx[10 $ (...)]",
  "?$10 $ (...)]",
  "?=10 $ (...)]",
  "?[10 $ (...)$",
  "?[10 $ (...)$",
  "10 = (...)",
  "?10 = (...)",
  "?(10 = (...))",
  "$10 = (...)",
  "=10 = (...)",
  "$[10 = (...)]",
  "xxx[10 = (...)]",
  "?$10 = (...)]",
  "?=10 = (...)]",
  "?[10 = (...)$",
  "?[10 = (...)$",
  "$10 * (Any)",
  "=10 * (Any)",
  "$[10 * (Any)]",
  "xxx[10 * (Any)]",
  "?$10 * (Any)]",
  "?=10 * (Any)]",
  "?[10 * (Any)$",
  "?[10 * (Any)$",
  "$(Any)",
  "=(Any)",
  "$[(Any)]",
  "xxx[(Any)]",
  "?$(Any)]",
  "?=(Any)]",
  "?[(Any)$",
  "?[(Any)$",
  "10 $ (Any)",
  "?10 $ (Any)",
  "?(10 $ (Any))",
  "$10 $ (Any)",
  "=10 $ (Any)",
  "$[10 $ (Any)]",
  "xxx[10 $ (Any)]",
  "?$10 $ (Any)]",
  "?=10 $ (Any)]",
  "?[10 $ (Any)$",
  "?[10 $ (Any)$",
  "10 = (Any)",
  "?10 = (Any)",
  "?(10 = (Any))",
  "$10 = (Any)",
  "=10 = (Any)",
  "$[10 = (Any)]",
  "xxx[10 = (Any)]",
  "?$10 = (Any)]",
  "?=10 = (Any)]",
  "?[10 = (Any)$",
  "?[10 = (Any)$",
  "$10 * (int64, Any)",
  "=10 * (int64, Any)",
  "$[10 * (int64, Any)]",
  "xxx[10 * (int64, Any)]",
  "?$10 * (int64, Any)]",
  "?=10 * (int64, Any)]",
  "?[10 * (int64, Any)$",
  "?[10 * (int64, Any)$",
  "$(int64, Any)",
  "=(int64, Any)",
  "$[(int64, Any)]",
  "xxx[(int64, Any)]",
  "?$(int64, Any)]",
  "?=(int64, Any)]",
  "?[(int64, Any)$",
  "?[(int64, Any)$",
  "10 $ (int64, Any)",
  "?10 $ (int64, Any)",
  "?(10 $ (int64, Any))",
  "$10 $ (int64, Any)",
  "=10 $ (int64, Any)",
  "$[10 $ (int64, Any)]",
  "xxx[10 $ (int64, Any)]",
  "?$10 $ (int64, Any)]",
  "?=10 $ (int64, Any)]",
  "?[10 $ (int64, Any)$",
  "?[10 $ (int64, Any)$",
  "10 = (int64, Any)",
  "?10 = (int64, Any)",
  "?(10 = (int64, Any))",
  "$10 = (int64, Any)",
  "=10 = (int64, Any)",
  "$[10 = (int64, Any)]",
  "xxx[10 = (int64, Any)]",
  "?$10 = (int64, Any)]",
  "?=10 = (int64, Any)]",
  "?[10 = (int64, Any)$",
  "?[10 = (int64, Any)$",
  "10 * (int64$ Any)",
  "?10 * (int64$ Any)",
  "?(10 * (int64$ Any))",
  "$10 * (int64$ Any)",
  "=10 * (int64$ Any)",
  "$[10 * (int64$ Any)]",
  "xxx[10 * (int64$ Any)]",
  "?$10 * (int64$ Any)]",
  "?=10 * (int64$ Any)]",
  "?[10 * (int64$ Any)$",
  "?[10 * (int64$ Any)$",
  "(int64$ Any)",
  "?(int64$ Any)",
  "?((int64$ Any))",
  "$(int64$ Any)",
  "=(int64$ Any)",
  "$[(int64$ Any)]",
  "xxx[(int64$ Any)]",
  "?$(int64$ Any)]",
  "?=(int64$ Any)]",
  "?[(int64$ Any)$",
  "?[(int64$ Any)$",
  "10 $ (int64$ Any)",
  "?10 $ (int64$ Any)",
  "?(10 $ (int64$ Any))",
  "$10 $ (int64$ Any)",
  "=10 $ (int64$ Any)",
  "$[10 $ (int64$ Any)]",
  "xxx[10 $ (int64$ Any)]",
  "?$10 $ (int64$ Any)]",
  "?=10 $ (int64$ Any)]",
  "?[10 $ (int64$ Any)$",
  "?[10 $ (int64$ Any)$",
  "10 = (int64$ Any)",
  "?10 = (int64$ Any)",
  "?(10 = (int64$ Any))",
  "$10 = (int64$ Any)",
  "=10 = (int64$ Any)",
  "$[10 = (int64$ Any)]",
  "xxx[10 = (int64$ Any)]",
  "?$10 = (int64$ Any)]",
  "?=10 = (int64$ Any)]",
  "?[10 = (int64$ Any)$",
  "?[10 = (int64$ Any)$",
  "10 * (int64= Any)",
  "?10 * (int64= Any)",
  "?(10 * (int64= Any))",
  "$10 * (int64= Any)",
  "=10 * (int64= Any)",
  "$[10 * (int64= Any)]",
  "xxx[10 * (int64= Any)]",
  "?$10 * (int64= Any)]",
  "?=10 * (int64= Any)]",
  "?[10 * (int64= Any)$",
  "?[10 * (int64= Any)$",
  "(int64= Any)",
  "?(int64= Any)",
  "?((int64= Any))",
  "$(int64= Any)",
  "=(int64= Any)",
  "$[(int64= Any)]",
  "xxx[(int64= Any)]",
  "?$(int64= Any)]",
  "?=(int64= Any)]",
  "?[(int64= Any)$",
  "?[(int64= Any)$",
  "10 $ (int64= Any)",
  "?10 $ (int64= Any)",
  "?(10 $ (int64= Any))",
  "$10 $ (int64= Any)",
  "=10 $ (int64= Any)",
  "$[10 $ (int64= Any)]",
  "xxx[10 $ (int64= Any)]",
  "?$10 $ (int64= Any)]",
  "?=10 $ (int64= Any)]",
  "?[10 $ (int64= Any)$",
  "?[10 $ (int64= Any)$",
  "10 = (int64= Any)",
  "?10 = (int64= Any)",
  "?(10 = (int64= Any))",
  "$10 = (int64= Any)",
  "=10 = (int64= Any)",
  "$[10 = (int64= Any)]",
  "xxx[10 = (int64= Any)]",
  "?$10 = (int64= Any)]",
  "?=10 = (int64= Any)]",
  "?[10 = (int64= Any)$",
  "?[10 = (int64= Any)$",
  "$10 * (int64, int64, Any)",
  "=10 * (int64, int64, Any)",
  "$[10 * (int64, int64, Any)]",
  "xxx[10 * (int64, int64, Any)]",
  "?$10 * (int64, int64, Any)]",
  "?=10 * (int64, int64, Any)]",
  "?[10 * (int64, int64, Any)$",
  "?[10 * (int64, int64, Any)$",
  "$(int64, int64, Any)",
  "=(int64, int64, Any)",
  "$[(int64, int64, Any)]",
  "xxx[(int64, int64, Any)]",
  "?$(int64, int64, Any)]",
  "?=(int64, int64, Any)]",
  "?[(int64, int64, Any)$",
  "?[(int64, int64, Any)$",
  "10 $ (int64, int64, Any)",
  "?10 $ (int64, int64, Any)",
  "?(10 $ (int64, int64, Any))",
  "$10 $ (int64, int64, Any)",
  "=10 $ (int64, int64, Any)",
  "$[10 $ (int64, int64, Any)]",
  "xxx[10 $ (int64, int64, Any)]",
  "?$10 $ (int64, int64, Any)]",
  "?=10 $ (int64, int64, Any)]",
  "?[10 $ (int64, int64, Any)$",
  "?[10 $ (int64, int64, Any)$",
  "10 = (int64, int64, Any)",
  "?10 = (int64, int64, Any)",
  "?(10 = (int64, int64, Any))",
  "$10 = (int64, int64, Any)",
  "=10 = (int64, int64, Any)",
  "$[10 = (int64, int64, Any)]",
  "xxx[10 = (int64, int64, Any)]",
  "?$10 = (int64, int64, Any)]",
  "?=10 = (int64, int64, Any)]",
  "?[10 = (int64, int64, Any)$",
  "?[10 = (int64, int64, Any)$",
  "10 * (int64, int64$ Any)",
  "?10 * (int64, int64$ Any)",
  "?(10 * (int64, int64$ Any))",
  "$10 * (int64, int64$ Any)",
  "=10 * (int64, int64$ Any)",
  "$[10 * (int64, int64$ Any)]",
  "xxx[10 * (int64, int64$ Any)]",
  "?$10 * (int64, int64$ Any)]",
  "?=10 * (int64, int64$ Any)]",
  "?[10 * (int64, int64$ Any)$",
  "?[10 * (int64, int64$ Any)$",
  "(int64, int64$ Any)",
  "?(int64, int64$ Any)",
  "?((int64, int64$ Any))",
  "$(int64, int64$ Any)",
  "=(int64, int64$ Any)",
  "$[(int64, int64$ Any)]",
  "xxx[(int64, int64$ Any)]",
  "?$(int64, int64$ Any)]",
  "?=(int64, int64$ Any)]",
  "?[(int64, int64$ Any)$",
  "?[(int64, int64$ Any)$",
  "10 $ (int64, int64$ Any)",
  "?10 $ (int64, int64$ Any)",
  "?(10 $ (int64, int64$ Any))",
  "$10 $ (int64, int64$ Any)",
  "=10 $ (int64, int64$ Any)",
  "$[10 $ (int64, int64$ Any)]",
  "xxx[10 $ (int64, int64$ Any)]",
  "?$10 $ (int64, int64$ Any)]",
  "?=10 $ (int64, int64$ Any)]",
  "?[10 $ (int64, int64$ Any)$",
  "?[10 $ (int64, int64$ Any)$",
  "10 = (int64, int64$ Any)",
  "?10 = (int64, int64$ Any)",
  "?(10 = (int64, int64$ Any))",
  "$10 = (int64, int64$ Any)",
  "=10 = (int64, int64$ Any)",
  "$[10 = (int64, int64$ Any)]",
  "xxx[10 = (int64, int64$ Any)]",
  "?$10 = (int64, int64$ Any)]",
  "?=10 = (int64, int64$ Any)]",
  "?[10 = (int64, int64$ Any)$",
  "?[10 = (int64, int64$ Any)$",
  "10 * (int64, int64= Any)",
  "?10 * (int64, int64= Any)",
  "?(10 * (int64, int64= Any))",
  "$10 * (int64, int64= Any)",
  "=10 * (int64, int64= Any)",
  "$[10 * (int64, int64= Any)]",
  "xxx[10 * (int64, int64= Any)]",
  "?$10 * (int64, int64= Any)]",
  "?=10 * (int64, int64= Any)]",
  "?[10 * (int64, int64= Any)$",
  "?[10 * (int64, int64= Any)$",
  "(int64, int64= Any)",
  "?(int64, int64= Any)",
  "?((int64, int64= Any))",
  "$(int64, int64= Any)",
  "=(int64, int64= Any)",
  "$[(int64, int64= Any)]",
  "xxx[(int64, int64= Any)]",
  "?$(int64, int64= Any)]",
  "?=(int64, int64= Any)]",
  "?[(int64, int64= Any)$",
  "?[(int64, int64= Any)$",
  "10 $ (int64, int64= Any)",
  "?10 $ (int64, int64= Any)",
  "?(10 $ (int64, int64= Any))",
  "$10 $ (int64, int64= Any)",
  "=10 $ (int64, int64= Any)",
  "$[10 $ (int64, int64= Any)]",
  "xxx[10 $ (int64, int64= Any)]",
  "?$10 $ (int64, int64= Any)]",
  "?=10 $ (int64, int64= Any)]",
  "?[10 $ (int64, int64= Any)$",
  "?[10 $ (int64, int64= Any)$",
  "10 = (int64, int64= Any)",
  "?10 = (int64, int64= Any)",
  "?(10 = (int64, int64= Any))",
  "$10 = (int64, int64= Any)",
  "=10 = (int64, int64= Any)",
  "$[10 = (int64, int64= Any)]",
  "xxx[10 = (int64, int64= Any)]",
  "?$10 = (int64, int64= Any)]",
  "?=10 = (int64, int64= Any)]",
  "?[10 = (int64, int64= Any)$",
  "?[10 = (int64, int64= Any)$",
  "10 * (xxx)",
  "?10 * (xxx)",
  "?(10 * (xxx))",
  "$10 * (xxx)",
  "=10 * (xxx)",
  "$[10 * (xxx)]",
  "xxx[10 * (xxx)]",
  "?$10 * (xxx)]",
  "?=10 * (xxx)]",
  "?[10 * (xxx)$",
  "?[10 * (xxx)$",
  "(xxx)",
  "?(xxx)",
  "?((xxx))",
  "$(xxx)",
  "=(xxx)",
  "$[(xxx)]",
  "xxx[(xxx)]",
  "?$(xxx)]",
  "?=(xxx)]",
  "?[(xxx)$",
  "?[(xxx)$",
  "10 $ (xxx)",
  "?10 $ (xxx)",
  "?(10 $ (xxx))",
  "$10 $ (xxx)",
  "=10 $ (xxx)",
  "$[10 $ (xxx)]",
  "xxx[10 $ (xxx)]",
  "?$10 $ (xxx)]",
  "?=10 $ (xxx)]",
  "?[10 $ (xxx)$",
  "?[10 $ (xxx)$",
  "10 = (xxx)",
  "?10 = (xxx)",
  "?(10 = (xxx))",
  "$10 = (xxx)",
  "=10 = (xxx)",
  "$[10 = (xxx)]",
  "xxx[10 = (xxx)]",
  "?$10 = (xxx)]",
  "?=10 = (xxx)]",
  "?[10 = (xxx)$",
  "?[10 = (xxx)$",
  "10 * (int64, xxx)",
  "?10 * (int64, xxx)",
  "?(10 * (int64, xxx))",
  "$10 * (int64, xxx)",
  "=10 * (int64, xxx)",
  "$[10 * (int64, xxx)]",
  "xxx[10 * (int64, xxx)]",
  "?$10 * (int64, xxx)]",
  "?=10 * (int64, xxx)]",
  "?[10 * (int64, xxx)$",
  "?[10 * (int64, xxx)$",
  "(int64, xxx)",
  "?(int64, xxx)",
  "?((int64, xxx))",
  "$(int64, xxx)",
  "=(int64, xxx)",
  "$[(int64, xxx)]",
  "xxx[(int64, xxx)]",
  "?$(int64, xxx)]",
  "?=(int64, xxx)]",
  "?[(int64, xxx)$",
  "?[(int64, xxx)$",
  "10 $ (int64, xxx)",
  "?10 $ (int64, xxx)",
  "?(10 $ (int64, xxx))",
  "$10 $ (int64, xxx)",
  "=10 $ (int64, xxx)",
  "$[10 $ (int64, xxx)]",
  "xxx[10 $ (int64, xxx)]",
  "?$10 $ (int64, xxx)]",
  "?=10 $ (int64, xxx)]",
  "?[10 $ (int64, xxx)$",
  "?[10 $ (int64, xxx)$",
  "10 = (int64, xxx)",
  "?10 = (int64, xxx)",
  "?(10 = (int64, xxx))",
  "$10 = (int64, xxx)",
  "=10 = (int64, xxx)",
  "$[10 = (int64, xxx)]",
  "xxx[10 = (int64, xxx)]",
  "?$10 = (int64, xxx)]",
  "?=10 = (int64, xxx)]",
  "?[10 = (int64, xxx)$",
  "?[10 = (int64, xxx)$",
  "10 * (int64$ xxx)",
  "?10 * (int64$ xxx)",
  "?(10 * (int64$ xxx))",
  "$10 * (int64$ xxx)",
  "=10 * (int64$ xxx)",
  "$[10 * (int64$ xxx)]",
  "xxx[10 * (int64$ xxx)]",
  "?$10 * (int64$ xxx)]",
  "?=10 * (int64$ xxx)]",
  "?[10 * (int64$ xxx)$",
  "?[10 * (int64$ xxx)$",
  "(int64$ xxx)",
  "?(int64$ xxx)",
  "?((int64$ xxx))",
  "$(int64$ xxx)",
  "=(int64$ xxx)",
  "$[(int64$ xxx)]",
  "xxx[(int64$ xxx)]",
  "?$(int64$ xxx)]",
  "?=(int64$ xxx)]",
  "?[(int64$ xxx)$",
  "?[(int64$ xxx)$",
  "10 $ (int64$ xxx)",
  "?10 $ (int64$ xxx)",
  "?(10 $ (int64$ xxx))",
  "$10 $ (int64$ xxx)",
  "=10 $ (int64$ xxx)",
  "$[10 $ (int64$ xxx)]",
  "xxx[10 $ (int64$ xxx)]",
  "?$10 $ (int64$ xxx)]",
  "?=10 $ (int64$ xxx)]",
  "?[10 $ (int64$ xxx)$",
  "?[10 $ (int64$ xxx)$",
  "10 = (int64$ xxx)",
  "?10 = (int64$ xxx)",
  "?(10 = (int64$ xxx))",
  "$10 = (int64$ xxx)",
  "=10 = (int64$ xxx)",
  "$[10 = (int64$ xxx)]",
  "xxx[10 = (int64$ xxx)]",
  "?$10 = (int64$ xxx)]",
  "?=10 = (int64$ xxx)]",
  "?[10 = (int64$ xxx)$",
  "?[10 = (int64$ xxx)$",
  "10 * (int64= xxx)",
  "?10 * (int64= xxx)",
  "?(10 * (int64= xxx))",
  "$10 * (int64= xxx)",
  "=10 * (int64= xxx)",
  "$[10 * (int64= xxx)]",
  "xxx[10 * (int64= xxx)]",
  "?$10 * (int64= xxx)]",
  "?=10 * (int64= xxx)]",
  "?[10 * (int64= xxx)$",
  "?[10 * (int64= xxx)$",
  "(int64= xxx)",
  "?(int64= xxx)",
  "?((int64= xxx))",
  "$(int64= xxx)",
  "=(int64= xxx)",
  "$[(int64= xxx)]",
  "xxx[(int64= xxx)]",
  "?$(int64= xxx)]",
  "?=(int64= xxx)]",
  "?[(int64= xxx)$",
  "?[(int64= xxx)$",
  "10 $ (int64= xxx)",
  "?10 $ (int64= xxx)",
  "?(10 $ (int64= xxx))",
  "$10 $ (int64= xxx)",
  "=10 $ (int64= xxx)",
  "$[10 $ (int64= xxx)]",
  "xxx[10 $ (int64= xxx)]",
  "?$10 $ (int64= xxx)]",
  "?=10 $ (int64= xxx)]",
  "?[10 $ (int64= xxx)$",
  "?[10 $ (int64= xxx)$",
  "10 = (int64= xxx)",
  "?10 = (int64= xxx)",
  "?(10 = (int64= xxx))",
  "$10 = (int64= xxx)",
  "=10 = (int64= xxx)",
  "$[10 = (int64= xxx)]",
  "xxx[10 = (int64= xxx)]",
  "?$10 = (int64= xxx)]",
  "?=10 = (int64= xxx)]",
  "?[10 = (int64= xxx)$",
  "?[10 = (int64= xxx)$",
  "10 * (int64, int64, xxx)",
  "?10 * (int64, int64, xxx)",
  "?(10 * (int64, int64, xxx))",
  "$10 * (int64, int64, xxx)",
  "=10 * (int64, int64, xxx)",
  "$[10 * (int64, int64, xxx)]",
  "xxx[10 * (int64, int64, xxx)]",
  "?$10 * (int64, int64, xxx)]",
  "?=10 * (int64, int64, xxx)]",
  "?[10 * (int64, int64, xxx)$",
  "?[10 * (int64, int64, xxx)$",
  "(int64, int64, xxx)",
  "?(int64, int64, xxx)",
  "?((int64, int64, xxx))",
  "$(int64, int64, xxx)",
  "=(int64, int64, xxx)",
  "$[(int64, int64, xxx)]",
  "xxx[(int64, int64, xxx)]",
  "?$(int64, int64, xxx)]",
  "?=(int64, int64, xxx)]",
  "?[(int64, int64, xxx)$",
  "?[(int64, int64, xxx)$",
  "10 $ (int64, int64, xxx)",
  "?10 $ (int64, int64, xxx)",
  "?(10 $ (int64, int64, xxx))",
  "$10 $ (int64, int64, xxx)",
  "=10 $ (int64, int64, xxx)",
  "$[10 $ (int64, int64, xxx)]",
  "xxx[10 $ (int64, int64, xxx)]",
  "?$10 $ (int64, int64, xxx)]",
  "?=10 $ (int64, int64, xxx)]",
  "?[10 $ (int64, int64, xxx)$",
  "?[10 $ (int64, int64, xxx)$",
  "10 = (int64, int64, xxx)",
  "?10 = (int64, int64, xxx)",
  "?(10 = (int64, int64, xxx))",
  "$10 = (int64, int64, xxx)",
  "=10 = (int64, int64, xxx)",
  "$[10 = (int64, int64, xxx)]",
  "xxx[10 = (int64, int64, xxx)]",
  "?$10 = (int64, int64, xxx)]",
  "?=10 = (int64, int64, xxx)]",
  "?[10 = (int64, int64, xxx)$",
  "?[10 = (int64, int64, xxx)$",
  "10 * (int64, int64$ xxx)",
  "?10 * (int64, int64$ xxx)",
  "?(10 * (int64, int64$ xxx))",
  "$10 * (int64, int64$ xxx)",
  "=10 * (int64, int64$ xxx)",
  "$[10 * (int64, int64$ xxx)]",
  "xxx[10 * (int64, int64$ xxx)]",
  "?$10 * (int64, int64$ xxx)]",
  "?=10 * (int64, int64$ xxx)]",
  "?[10 * (int64, int64$ xxx)$",
  "?[10 * (int64, int64$ xxx)$",
  "(int64, int64$ xxx)",
  "?(int64, int64$ xxx)",
  "?((int64, int64$ xxx))",
  "$(int64, int64$ xxx)",
  "=(int64, int64$ xxx)",
  "$[(int64, int64$ xxx)]",
  "xxx[(int64, int64$ xxx)]",
  "?$(int64, int64$ xxx)]",
  "?=(int64, int64$ xxx)]",
  "?[(int64, int64$ xxx)$",
  "?[(int64, int64$ xxx)$",
  "10 $ (int64, int64$ xxx)",
  "?10 $ (int64, int64$ xxx)",
  "?(10 $ (int64, int64$ xxx))",
  "$10 $ (int64, int64$ xxx)",
  "=10 $ (int64, int64$ xxx)",
  "$[10 $ (int64, int64$ xxx)]",
  "xxx[10 $ (int64, int64$ xxx)]",
  "?$10 $ (int64, int64$ xxx)]",
  "?=10 $ (int64, int64$ xxx)]",
  "?[10 $ (int64, int64$ xxx)$",
  "?[10 $ (int64, int64$ xxx)$",
  "10 = (int64, int64$ xxx)",
  "?10 = (int64, int64$ xxx)",
  "?(10 = (int64, int64$ xxx))",
  "$10 = (int64, int64$ xxx)",
  "=10 = (int64, int64$ xxx)",
  "$[10 = (int64, int64$ xxx)]",
  "xxx[10 = (int64, int64$ xxx)]",
  "?$10 = (int64, int64$ xxx)]",
  "?=10 = (int64, int64$ xxx)]",
  "?[10 = (int64, int64$ xxx)$",
  "?[10 = (int64, int64$ xxx)$",
  "10 * (int64, int64= xxx)",
  "?10 * (int64, int64= xxx)",
  "?(10 * (int64, int64= xxx))",
  "$10 * (int64, int64= xxx)",
  "=10 * (int64, int64= xxx)",
  "$[10 * (int64, int64= xxx)]",
  "xxx[10 * (int64, int64= xxx)]",
  "?$10 * (int64, int64= xxx)]",
  "?=10 * (int64, int64= xxx)]",
  "?[10 * (int64, int64= xxx)$",
  "?[10 * (int64, int64= xxx)$",
  "(int64, int64= xxx)",
  "?(int64, int64= xxx)",
  "?((int64, int64= xxx))",
  "$(int64, int64= xxx)",
  "=(int64, int64= xxx)",
  "$[(int64, int64= xxx)]",
  "xxx[(int64, int64= xxx)]",
  "?$(int64, int64= xxx)]",
  "?=(int64, int64= xxx)]",
  "?[(int64, int64= xxx)$",
  "?[(int64, int64= xxx)$",
  "10 $ (int64, int64= xxx)",
  "?10 $ (int64, int64= xxx)",
  "?(10 $ (int64, int64= xxx))",
  "$10 $ (int64, int64= xxx)",
  "=10 $ (int64, int64= xxx)",
  "$[10 $ (int64, int64= xxx)]",
  "xxx[10 $ (int64, int64= xxx)]",
  "?$10 $ (int64, int64= xxx)]",
  "?=10 $ (int64, int64= xxx)]",
  "?[10 $ (int64, int64= xxx)$",
  "?[10 $ (int64, int64= xxx)$",
  "10 = (int64, int64= xxx)",
  "?10 = (int64, int64= xxx)",
  "?(10 = (int64, int64= xxx))",
  "$10 = (int64, int64= xxx)",
  "=10 = (int64, int64= xxx)",
  "$[10 = (int64, int64= xxx)]",
  "xxx[10 = (int64, int64= xxx)]",
  "?$10 = (int64, int64= xxx)]",
  "?=10 = (int64, int64= xxx)]",
  "?[10 = (int64, int64= xxx)$",
  "?[10 = (int64, int64= xxx)$",
  "10 * ($)",
  "?10 * ($)",
  "?(10 * ($))",
  "$10 * ($)",
  "=10 * ($)",
  "$[10 * ($)]",
  "xxx[10 * ($)]",
  "?$10 * ($)]",
  "?=10 * ($)]",
  "?[10 * ($)$",
  "?[10 * ($)$",
  "($)",
  "?($)",
  "?(($))",
  "$($)",
  "=($)",
  "$[($)]",
  "xxx[($)]",
  "?$($)]",
  "?=($)]",
  "?[($)$",
  "?[($)$",
  "10 $ ($)",
  "?10 $ ($)",
  "?(10 $ ($))",
  "$10 $ ($)",
  "=10 $ ($)",
  "$[10 $ ($)]",
  "xxx[10 $ ($)]",
  "?$10 $ ($)]",
  "?=10 $ ($)]",
  "?[10 $ ($)$",
  "?[10 $ ($)$",
  "10 = ($)",
  "?10 = ($)",
  "?(10 = ($))",
  "$10 = ($)",
  "=10 = ($)",
  "$[10 = ($)]",
  "xxx[10 = ($)]",
  "?$10 = ($)]",
  "?=10 = ($)]",
  "?[10 = ($)$",
  "?[10 = ($)$",
  "10 * (int64, $)",
  "?10 * (int64, $)",
  "?(10 * (int64, $))",
  "$10 * (int64, $)",
  "=10 * (int64, $)",
  "$[10 * (int64, $)]",
  "xxx[10 * (int64, $)]",
  "?$10 * (int64, $)]",
  "?=10 * (int64, $)]",
  "?[10 * (int64, $)$",
  "?[10 * (int64, $)$",
  "(int64, $)",
  "?(int64, $)",
  "?((int64, $))",
  "$(int64, $)",
  "=(int64, $)",
  "$[(int64, $)]",
  "xxx[(int64, $)]",
  "?$(int64, $)]",
  "?=(int64, $)]",
  "?[(int64, $)$",
  "?[(int64, $)$",
  "10 $ (int64, $)",
  "?10 $ (int64, $)",
  "?(10 $ (int64, $))",
  "$10 $ (int64, $)",
  "=10 $ (int64, $)",
  "$[10 $ (int64, $)]",
  "xxx[10 $ (int64, $)]",
  "?$10 $ (int64, $)]",
  "?=10 $ (int64, $)]",
  "?[10 $ (int64, $)$",
  "?[10 $ (int64, $)$",
  "10 = (int64, $)",
  "?10 = (int64, $)",
  "?(10 = (int64, $))",
  "$10 = (int64, $)",
  "=10 = (int64, $)",
  "$[10 = (int64, $)]",
  "xxx[10 = (int64, $)]",
  "?$10 = (int64, $)]",
  "?=10 = (int64, $)]",
  "?[10 = (int64, $)$",
  "?[10 = (int64, $)$",
  "10 * (int64$ $)",
  "?10 * (int64$ $)",
  "?(10 * (int64$ $))",
  "$10 * (int64$ $)",
  "=10 * (int64$ $)",
  "$[10 * (int64$ $)]",
  "xxx[10 * (int64$ $)]",
  "?$10 * (int64$ $)]",
  "?=10 * (int64$ $)]",
  "?[10 * (int64$ $)$",
  "?[10 * (int64$ $)$",
  "(int64$ $)",
  "?(int64$ $)",
  "?((int64$ $))",
  "$(int64$ $)",
  "=(int64$ $)",
  "$[(int64$ $)]",
  "xxx[(int64$ $)]",
  "?$(int64$ $)]",
  "?=(int64$ $)]",
  "?[(int64$ $)$",
  "?[(int64$ $)$",
  "10 $ (int64$ $)",
  "?10 $ (int64$ $)",
  "?(10 $ (int64$ $))",
  "$10 $ (int64$ $)",
  "=10 $ (int64$ $)",
  "$[10 $ (int64$ $)]",
  "xxx[10 $ (int64$ $)]",
  "?$10 $ (int64$ $)]",
  "?=10 $ (int64$ $)]",
  "?[10 $ (int64$ $)$",
  "?[10 $ (int64$ $)$",
  "10 = (int64$ $)",
  "?10 = (int64$ $)",
  "?(10 = (int64$ $))",
  "$10 = (int64$ $)",
  "=10 = (int64$ $)",
  "$[10 = (int64$ $)]",
  "xxx[10 = (int64$ $)]",
  "?$10 = (int64$ $)]",
  "?=10 = (int64$ $)]",
  "?[10 = (int64$ $)$",
  "?[10 = (int64$ $)$",
  "10 * (int64= $)",
  "?10 * (int64= $)",
  "?(10 * (int64= $))",
  "$10 * (int64= $)",
  "=10 * (int64= $)",
  "$[10 * (int64= $)]",
  "xxx[10 * (int64= $)]",
  "?$10 * (int64= $)]",
  "?=10 * (int64= $)]",
  "?[10 * (int64= $)$",
  "?[10 * (int64= $)$",
  "(int64= $)",
  "?(int64= $)",
  "?((int64= $))",
  "$(int64= $)",
  "=(int64= $)",
  "$[(int64= $)]",
  "xxx[(int64= $)]",
  "?$(int64= $)]",
  "?=(int64= $)]",
  "?[(int64= $)$",
  "?[(int64= $)$",
  "10 $ (int64= $)",
  "?10 $ (int64= $)",
  "?(10 $ (int64= $))",
  "$10 $ (int64= $)",
  "=10 $ (int64= $)",
  "$[10 $ (int64= $)]",
  "xxx[10 $ (int64= $)]",
  "?$10 $ (int64= $)]",
  "?=10 $ (int64= $)]",
  "?[10 $ (int64= $)$",
  "?[10 $ (int64= $)$",
  "10 = (int64= $)",
  "?10 = (int64= $)",
  "?(10 = (int64= $))",
  "$10 = (int64= $)",
  "=10 = (int64= $)",
  "$[10 = (int64= $)]",
  "xxx[10 = (int64= $)]",
  "?$10 = (int64= $)]",
  "?=10 = (int64= $)]",
  "?[10 = (int64= $)$",
  "?[10 = (int64= $)$",
  "10 * (int64, int64, $)",
  "?10 * (int64, int64, $)",
  "?(10 * (int64, int64, $))",
  "$10 * (int64, int64, $)",
  "=10 * (int64, int64, $)",
  "$[10 * (int64, int64, $)]",
  "xxx[10 * (int64, int64, $)]",
  "?$10 * (int64, int64, $)]",
  "?=10 * (int64, int64, $)]",
  "?[10 * (int64, int64, $)$",
  "?[10 * (int64, int64, $)$",
  "(int64, int64, $)",
  "?(int64, int64, $)",
  "?((int64, int64, $))",
  "$(int64, int64, $)",
  "=(int64, int64, $)",
  "$[(int64, int64, $)]",
  "xxx[(int64, int64, $)]",
  "?$(int64, int64, $)]",
  "?=(int64, int64, $)]",
  "?[(int64, int64, $)$",
  "?[(int64, int64, $)$",
  "10 $ (int64, int64, $)",
  "?10 $ (int64, int64, $)",
  "?(10 $ (int64, int64, $))",
  "$10 $ (int64, int64, $)",
  "=10 $ (int64, int64, $)",
  "$[10 $ (int64, int64, $)]",
  "xxx[10 $ (int64, int64, $)]",
  "?$10 $ (int64, int64, $)]",
  "?=10 $ (int64, int64, $)]",
  "?[10 $ (int64, int64, $)$",
  "?[10 $ (int64, int64, $)$",
  "10 = (int64, int64, $)",
  "?10 = (int64, int64, $)",
  "?(10 = (int64, int64, $))",
  "$10 = (int64, int64, $)",
  "=10 = (int64, int64, $)",
  "$[10 = (int64, int64, $)]",
  "xxx[10 = (int64, int64, $)]",
  "?$10 = (int64, int64, $)]",
  "?=10 = (int64, int64, $)]",
  "?[10 = (int64, int64, $)$",
  "?[10 = (int64, int64, $)$",
  "10 * (int64, int64$ $)",
  "?10 * (int64, int64$ $)",
  "?(10 * (int64, int64$ $))",
  "$10 * (int64, int64$ $)",
  "=10 * (int64, int64$ $)",
  "$[10 * (int64, int64$ $)]",
  "xxx[10 * (int64, int64$ $)]",
  "?$10 * (int64, int64$ $)]",
  "?=10 * (int64, int64$ $)]",
  "?[10 * (int64, int64$ $)$",
  "?[10 * (int64, int64$ $)$",
  "(int64, int64$ $)",
  "?(int64, int64$ $)",
  "?((int64, int64$ $))",
  "$(int64, int64$ $)",
  "=(int64, int64$ $)",
  "$[(int64, int64$ $)]",
  "xxx[(int64, int64$ $)]",
  "?$(int64, int64$ $)]",
  "?=(int64, int64$ $)]",
  "?[(int64, int64$ $)$",
  "?[(int64, int64$ $)$",
  "10 $ (int64, int64$ $)",
  "?10 $ (int64, int64$ $)",
  "?(10 $ (int64, int64$ $))",
  "$10 $ (int64, int64$ $)",
  "=10 $ (int64, int64$ $)",
  "$[10 $ (int64, int64$ $)]",
  "xxx[10 $ (int64, int64$ $)]",
  "?$10 $ (int64, int64$ $)]",
  "?=10 $ (int64, int64$ $)]",
  "?[10 $ (int64, int64$ $)$",
  "?[10 $ (int64, int64$ $)$",
  "10 = (int64, int64$ $)",
  "?10 = (int64, int64$ $)",
  "?(10 = (int64, int64$ $))",
  "$10 = (int64, int64$ $)",
  "=10 = (int64, int64$ $)",
  "$[10 = (int64, int64$ $)]",
  "xxx[10 = (int64, int64$ $)]",
  "?$10 = (int64, int64$ $)]",
  "?=10 = (int64, int64$ $)]",
  "?[10 = (int64, int64$ $)$",
  "?[10 = (int64, int64$ $)$",
  "10 * (int64, int64= $)",
  "?10 * (int64, int64= $)",
  "?(10 * (int64, int64= $))",
  "$10 * (int64, int64= $)",
  "=10 * (int64, int64= $)",
  "$[10 * (int64, int64= $)]",
  "xxx[10 * (int64, int64= $)]",
  "?$10 * (int64, int64= $)]",
  "?=10 * (int64, int64= $)]",
  "?[10 * (int64, int64= $)$",
  "?[10 * (int64, int64= $)$",
  "(int64, int64= $)",
  "?(int64, int64= $)",
  "?((int64, int64= $))",
  "$(int64, int64= $)",
  "=(int64, int64= $)",
  "$[(int64, int64= $)]",
  "xxx[(int64, int64= $)]",
  "?$(int64, int64= $)]",
  "?=(int64, int64= $)]",
  "?[(int64, int64= $)$",
  "?[(int64, int64= $)$",
  "10 $ (int64, int64= $)",
  "?10 $ (int64, int64= $)",
  "?(10 $ (int64, int64= $))",
  "$10 $ (int64, int64= $)",
  "=10 $ (int64, int64= $)",
  "$[10 $ (int64, int64= $)]",
  "xxx[10 $ (int64, int64= $)]",
  "?$10 $ (int64, int64= $)]",
  "?=10 $ (int64, int64= $)]",
  "?[10 $ (int64, int64= $)$",
  "?[10 $ (int64, int64= $)$",
  "10 = (int64, int64= $)",
  "?10 = (int64, int64= $)",
  "?(10 = (int64, int64= $))",
  "$10 = (int64, int64= $)",
  "=10 = (int64, int64= $)",
  "$[10 = (int64, int64= $)]",
  "xxx[10 = (int64, int64= $)]",
  "?$10 = (int64, int64= $)]",
  "?=10 = (int64, int64= $)]",
  "?[10 = (int64, int64= $)$",
  "?[10 = (int64, int64= $)$",
  "10 * (=)",
  "?10 * (=)",
  "?(10 * (=))",
  "$10 * (=)",
  "=10 * (=)",
  "$[10 * (=)]",
  "xxx[10 * (=)]",
  "?$10 * (=)]",
  "?=10 * (=)]",
  "?[10 * (=)$",
  "?[10 * (=)$",
  "(=)",
  "?(=)",
  "?((=))",
  "$(=)",
  "=(=)",
  "$[(=)]",
  "xxx[(=)]",
  "?$(=)]",
  "?=(=)]",
  "?[(=)$",
  "?[(=)$",
  "10 $ (=)",
  "?10 $ (=)",
  "?(10 $ (=))",
  "$10 $ (=)",
  "=10 $ (=)",
  "$[10 $ (=)]",
  "xxx[10 $ (=)]",
  "?$10 $ (=)]",
  "?=10 $ (=)]",
  "?[10 $ (=)$",
  "?[10 $ (=)$",
  "10 = (=)",
  "?10 = (=)",
  "?(10 = (=))",
  "$10 = (=)",
  "=10 = (=)",
  "$[10 = (=)]",
  "xxx[10 = (=)]",
  "?$10 = (=)]",
  "?=10 = (=)]",
  "?[10 = (=)$",
  "?[10 = (=)$",
  "10 * (int64, =)",
  "?10 * (int64, =)",
  "?(10 * (int64, =))",
  "$10 * (int64, =)",
  "=10 * (int64, =)",
  "$[10 * (int64, =)]",
  "xxx[10 * (int64, =)]",
  "?$10 * (int64, =)]",
  "?=10 * (int64, =)]",
  "?[10 * (int64, =)$",
  "?[10 * (int64, =)$",
  "(int64, =)",
  "?(int64, =)",
  "?((int64, =))",
  "$(int64, =)",
  "=(int64, =)",
  "$[(int64, =)]",
  "xxx[(int64, =)]",
  "?$(int64, =)]",
  "?=(int64, =)]",
  "?[(int64, =)$",
  "?[(int64, =)$",
  "10 $ (int64, =)",
  "?10 $ (int64, =)",
  "?(10 $ (int64, =))",
  "$10 $ (int64, =)",
  "=10 $ (int64, =)",
  "$[10 $ (int64, =)]",
  "xxx[10 $ (int64, =)]",
  "?$10 $ (int64, =)]",
  "?=10 $ (int64, =)]",
  "?[10 $ (int64, =)$",
  "?[10 $ (int64, =)$",
  "10 = (int64, =)",
  "?10 = (int64, =)",
  "?(10 = (int64, =))",
  "$10 = (int64, =)",
  "=10 = (int64, =)",
  "$[10 = (int64, =)]",
  "xxx[10 = (int64, =)]",
  "?$10 = (int64, =)]",
  "?=10 = (int64, =)]",
  "?[10 = (int64, =)$",
  "?[10 = (int64, =)$",
  "10 * (int64$ =)",
  "?10 * (int64$ =)",
  "?(10 * (int64$ =))",
  "$10 * (int64$ =)",
  "=10 * (int64$ =)",
  "$[10 * (int64$ =)]",
  "xxx[10 * (int64$ =)]",
  "?$10 * (int64$ =)]",
  "?=10 * (int64$ =)]",
  "?[10 * (int64$ =)$",
  "?[10 * (int64$ =)$",
  "(int64$ =)",
  "?(int64$ =)",
  "?((int64$ =))",
  "$(int64$ =)",
  "=(int64$ =)",
  "$[(int64$ =)]",
  "xxx[(int64$ =)]",
  "?$(int64$ =)]",
  "?=(int64$ =)]",
  "?[(int64$ =)$",
  "?[(int64$ =)$",
  "10 $ (int64$ =)",
  "?10 $ (int64$ =)",
  "?(10 $ (int64$ =))",
  "$10 $ (int64$ =)",
  "=10 $ (int64$ =)",
  "$[10 $ (int64$ =)]",
  "xxx[10 $ (int64$ =)]",
  "?$10 $ (int64$ =)]",
  "?=10 $ (int64$ =)]",
  "?[10 $ (int64$ =)$",
  "?[10 $ (int64$ =)$",
  "10 = (int64$ =)",
  "?10 = (int64$ =)",
  "?(10 = (int64$ =))",
  "$10 = (int64$ =)",
  "=10 = (int64$ =)",
  "$[10 = (int64$ =)]",
  "xxx[10 = (int64$ =)]",
  "?$10 = (int64$ =)]",
  "?=10 = (int64$ =)]",
  "?[10 = (int64$ =)$",
  "?[10 = (int64$ =)$",
  "10 * (int64= =)",
  "?10 * (int64= =)",
  "?(10 * (int64= =))",
  "$10 * (int64= =)",
  "=10 * (int64= =)",
  "$[10 * (int64= =)]",
  "xxx[10 * (int64= =)]",
  "?$10 * (int64= =)]",
  "?=10 * (int64= =)]",
  "?[10 * (int64= =)$",
  "?[10 * (int64= =)$",
  "(int64= =)",
  "?(int64= =)",
  "?((int64= =))",
  "$(int64= =)",
  "=(int64= =)",
  "$[(int64= =)]",
  "xxx[(int64= =)]",
  "?$(int64= =)]",
  "?=(int64= =)]",
  "?[(int64= =)$",
  "?[(int64= =)$",
  "10 $ (int64= =)",
  "?10 $ (int64= =)",
  "?(10 $ (int64= =))",
  "$10 $ (int64= =)",
  "=10 $ (int64= =)",
  "$[10 $ (int64= =)]",
  "xxx[10 $ (int64= =)]",
  "?$10 $ (int64= =)]",
  "?=10 $ (int64= =)]",
  "?[10 $ (int64= =)$",
  "?[10 $ (int64= =)$",
  "10 = (int64= =)",
  "?10 = (int64= =)",
  "?(10 = (int64= =))",
  "$10 = (int64= =)",
  "=10 = (int64= =)",
  "$[10 = (int64= =)]",
  "xxx[10 = (int64= =)]",
  "?$10 = (int64= =)]",
  "?=10 = (int64= =)]",
  "?[10 = (int64= =)$",
  "?[10 = (int64= =)$",
  "10 * (int64, int64, =)",
  "?10 * (int64, int64, =)",
  "?(10 * (int64, int64, =))",
  "$10 * (int64, int64, =)",
  "=10 * (int64, int64, =)",
  "$[10 * (int64, int64, =)]",
  "xxx[10 * (int64, int64, =)]",
  "?$10 * (int64, int64, =)]",
  "?=10 * (int64, int64, =)]",
  "?[10 * (int64, int64, =)$",
  "?[10 * (int64, int64, =)$",
  "(int64, int64, =)",
  "?(int64, int64, =)",
  "?((int64, int64, =))",
  "$(int64, int64, =)",
  "=(int64, int64, =)",
  "$[(int64, int64, =)]",
  "xxx[(int64, int64, =)]",
  "?$(int64, int64, =)]",
  "?=(int64, int64, =)]",
  "?[(int64, int64, =)$",
  "?[(int64, int64, =)$",
  "10 $ (int64, int64, =)",
  "?10 $ (int64, int64, =)",
  "?(10 $ (int64, int64, =))",
  "$10 $ (int64, int64, =)",
  "=10 $ (int64, int64, =)",
  "$[10 $ (int64, int64, =)]",
  "xxx[10 $ (int64, int64, =)]",
  "?$10 $ (int64, int64, =)]",
  "?=10 $ (int64, int64, =)]",
  "?[10 $ (int64, int64, =)$",
  "?[10 $ (int64, int64, =)$",
  "10 = (int64, int64, =)",
  "?10 = (int64, int64, =)",
  "?(10 = (int64, int64, =))",
  "$10 = (int64, int64, =)",
  "=10 = (int64, int64, =)",
  "$[10 = (int64, int64, =)]",
  "xxx[10 = (int64, int64, =)]",
  "?$10 = (int64, int64, =)]",
  "?=10 = (int64, int64, =)]",
  "?[10 = (int64, int64, =)$",
  "?[10 = (int64, int64, =)$",
  "10 * (int64, int64$ =)",
  "?10 * (int64, int64$ =)",
  "?(10 * (int64, int64$ =))",
  "$10 * (int64, int64$ =)",
  "=10 * (int64, int64$ =)",
  "$[10 * (int64, int64$ =)]",
  "xxx[10 * (int64, int64$ =)]",
  "?$10 * (int64, int64$ =)]",
  "?=10 * (int64, int64$ =)]",
  "?[10 * (int64, int64$ =)$",
  "?[10 * (int64, int64$ =)$",
  "(int64, int64$ =)",
  "?(int64, int64$ =)",
  "?((int64, int64$ =))",
  "$(int64, int64$ =)",
  "=(int64, int64$ =)",
  "$[(int64, int64$ =)]",
  "xxx[(int64, int64$ =)]",
  "?$(int64, int64$ =)]",
  "?=(int64, int64$ =)]",
  "?[(int64, int64$ =)$",
  "?[(int64, int64$ =)$",
  "10 $ (int64, int64$ =)",
  "?10 $ (int64, int64$ =)",
  "?(10 $ (int64, int64$ =))",
  "$10 $ (int64, int64$ =)",
  "=10 $ (int64, int64$ =)",
  "$[10 $ (int64, int64$ =)]",
  "xxx[10 $ (int64, int64$ =)]",
  "?$10 $ (int64, int64$ =)]",
  "?=10 $ (int64, int64$ =)]",
  "?[10 $ (int64, int64$ =)$",
  "?[10 $ (int64, int64$ =)$",
  "10 = (int64, int64$ =)",
  "?10 = (int64, int64$ =)",
  "?(10 = (int64, int64$ =))",
  "$10 = (int64, int64$ =)",
  "=10 = (int64, int64$ =)",
  "$[10 = (int64, int64$ =)]",
  "xxx[10 = (int64, int64$ =)]",
  "?$10 = (int64, int64$ =)]",
  "?=10 = (int64, int64$ =)]",
  "?[10 = (int64, int64$ =)$",
  "?[10 = (int64, int64$ =)$",
  "10 * (int64, int64= =)",
  "?10 * (int64, int64= =)",
  "?(10 * (int64, int64= =))",
  "$10 * (int64, int64= =)",
  "=10 * (int64, int64= =)",
  "$[10 * (int64, int64= =)]",
  "xxx[10 * (int64, int64= =)]",
  "?$10 * (int64, int64= =)]",
  "?=10 * (int64, int64= =)]",
  "?[10 * (int64, int64= =)$",
  "?[10 * (int64, int64= =)$",
  "(int64, int64= =)",
  "?(int64, int64= =)",
  "?((int64, int64= =))",
  "$(int64, int64= =)",
  "=(int64, int64= =)",
  "$[(int64, int64= =)]",
  "xxx[(int64, int64= =)]",
  "?$(int64, int64= =)]",
  "?=(int64, int64= =)]",
  "?[(int64, int64= =)$",
  "?[(int64, int64= =)$",
  "10 $ (int64, int64= =)",
  "?10 $ (int64, int64= =)",
  "?(10 $ (int64, int64= =))",
  "$10 $ (int64, int64= =)",
  "=10 $ (int64, int64= =)",
  "$[10 $ (int64, int64= =)]",
  "xxx[10 $ (int64, int64= =)]",
  "?$10 $ (int64, int64= =)]",
  "?=10 $ (int64, int64= =)]",
  "?[10 $ (int64, int64= =)$",
  "?[10 $ (int64, int64= =)$",
  "10 = (int64, int64= =)",
  "?10 = (int64, int64= =)",
  "?(10 = (int64, int64= =))",
  "$10 = (int64, int64= =)",
  "=10 = (int64, int64= =)",
  "$[10 = (int64, int64= =)]",
  "xxx[10 = (int64, int64= =)]",
  "?$10 = (int64, int64= =)]",
  "?=10 = (int64, int64= =)]",
  "?[10 = (int64, int64= =)$",
  "?[10 = (int64, int64= =)$",
  "$10 * (Any)",
  "=10 * (Any)",
  "$[10 * (Any)]",
  "xxx[10 * (Any)]",
  "?$10 * (Any)]",
  "?=10 * (Any)]",
  "?[10 * (Any)$",
  "?[10 * (Any)$",
  "$(Any)",
  "=(Any)",
  "$[(Any)]",
  "xxx[(Any)]",
  "?$(Any)]",
  "?=(Any)]",
  "?[(Any)$",
  "?[(Any)$",
  "10 $ (Any)",
  "?10 $ (Any)",
  "?(10 $ (Any))",
  "$10 $ (Any)",
  "=10 $ (Any)",
  "$[10 $ (Any)]",
  "xxx[10 $ (Any)]",
  "?$10 $ (Any)]",
  "?=10 $ (Any)]",
  "?[10 $ (Any)$",
  "?[10 $ (Any)$",
  "10 = (Any)",
  "?10 = (Any)",
  "?(10 = (Any))",
  "$10 = (Any)",
  "=10 = (Any)",
  "$[10 = (Any)]",
  "xxx[10 = (Any)]",
  "?$10 = (Any)]",
  "?=10 = (Any)]",
  "?[10 = (Any)$",
  "?[10 = (Any)$",
  "$10 * (Any, )",
  "=10 * (Any, )",
  "$[10 * (Any, )]",
  "xxx[10 * (Any, )]",
  "?$10 * (Any, )]",
  "?=10 * (Any, )]",
  "?[10 * (Any, )$",
  "?[10 * (Any, )$",
  "$(Any, )",
  "=(Any, )",
  "$[(Any, )]",
  "xxx[(Any, )]",
  "?$(Any, )]",
  "?=(Any, )]",
  "?[(Any, )$",
  "?[(Any, )$",
  "10 $ (Any, )",
  "?10 $ (Any, )",
  "?(10 $ (Any, ))",
  "$10 $ (Any, )",
  "=10 $ (Any, )",
  "$[10 $ (Any, )]",
  "xxx[10 $ (Any, )]",
  "?$10 $ (Any, )]",
  "?=10 $ (Any, )]",
  "?[10 $ (Any, )$",
  "?[10 $ (Any, )$",
  "10 = (Any, )",
  "?10 = (Any, )",
  "?(10 = (Any, ))",
  "$10 = (Any, )",
  "=10 = (Any, )",
  "$[10 = (Any, )]",
  "xxx[10 = (Any, )]",
  "?$10 = (Any, )]",
  "?=10 = (Any, )]",
  "?[10 = (Any, )$",
  "?[10 = (Any, )$",
  "$10 * (Any, ...)",
  "=10 * (Any, ...)",
  "$[10 * (Any, ...)]",
  "xxx[10 * (Any, ...)]",
  "?$10 * (Any, ...)]",
  "?=10 * (Any, ...)]",
  "?[10 * (Any, ...)$",
  "?[10 * (Any, ...)$",
  "$(Any, ...)",
  "=(Any, ...)",
  "$[(Any, ...)]",
  "xxx[(Any, ...)]",
  "?$(Any, ...)]",
  "?=(Any, ...)]",
  "?[(Any, ...)$",
  "?[(Any, ...)$",
  "10 $ (Any, ...)",
  "?10 $ (Any, ...)",
  "?(10 $ (Any, ...))",
  "$10 $ (Any, ...)",
  "=10 $ (Any, ...)",
  "$[10 $ (Any, ...)]",
  "xxx[10 $ (Any, ...)]",
  "?$10 $ (Any, ...)]",
  "?=10 $ (Any, ...)]",
  "?[10 $ (Any, ...)$",
  "?[10 $ (Any, ...)$",
  "10 = (Any, ...)",
  "?10 = (Any, ...)",
  "?(10 = (Any, ...))",
  "$10 = (Any, ...)",
  "=10 = (Any, ...)",
  "$[10 = (Any, ...)]",
  "xxx[10 = (Any, ...)]",
  "?$10 = (Any, ...)]",
  "?=10 = (Any, ...)]",
  "?[10 = (Any, ...)$",
  "?[10 = (Any, ...)$",
  "10 * (Any, $)",
  "?10 * (Any, $)",
  "?(10 * (Any, $))",
  "$10 * (Any, $)",
  "=10 * (Any, $)",
  "$[10 * (Any, $)]",
  "xxx[10 * (Any, $)]",
  "?$10 * (Any, $)]",
  "?=10 * (Any, $)]",
  "?[10 * (Any, $)$",
  "?[10 * (Any, $)$",
  "(Any, $)",
  "?(Any, $)",
  "?((Any, $))",
  "$(Any, $)",
  "=(Any, $)",
  "$[(Any, $)]",
  "xxx[(Any, $)]",
  "?$(Any, $)]",
  "?=(Any, $)]",
  "?[(Any, $)$",
  "?[(Any, $)$",
  "10 $ (Any, $)",
  "?10 $ (Any, $)",
  "?(10 $ (Any, $))",
  "$10 $ (Any, $)",
  "=10 $ (Any, $)",
  "$[10 $ (Any, $)]",
  "xxx[10 $ (Any, $)]",
  "?$10 $ (Any, $)]",
  "?=10 $ (Any, $)]",
  "?[10 $ (Any, $)$",
  "?[10 $ (Any, $)$",
  "10 = (Any, $)",
  "?10 = (Any, $)",
  "?(10 = (Any, $))",
  "$10 = (Any, $)",
  "=10 = (Any, $)",
  "$[10 = (Any, $)]",
  "xxx[10 = (Any, $)]",
  "?$10 = (Any, $)]",
  "?=10 = (Any, $)]",
  "?[10 = (Any, $)$",
  "?[10 = (Any, $)$",
  "$10 * {...}",
  "=10 * {...}",
  "$[10 * {...}]",
  "xxx[10 * {...}]",
  "?$10 * {...}]",
  "?=10 * {...}]",
  "?[10 * {...}$",
  "?[10 * {...}$",
  "${...}",
  "={...}",
  "$[{...}]",
  "xxx[{...}]",
  "?${...}]",
  "?={...}]",
  "?[{...}$",
  "?[{...}$",
  "10 $ {...}",
  "?10 $ {...}",
  "?(10 $ {...})",
  "$10 $ {...}",
  "=10 $ {...}",
  "$[10 $ {...}]",
  "xxx[10 $ {...}]",
  "?$10 $ {...}]",
  "?=10 $ {...}]",
  "?[10 $ {...}$",
  "?[10 $ {...}$",
  "10 = {...}",
  "?10 = {...}",
  "?(10 = {...})",
  "$10 = {...}",
  "=10 = {...}",
  "$[10 = {...}]",
  "xxx[10 = {...}]",
  "?$10 = {...}]",
  "?=10 = {...}]",
  "?[10 = {...}$",
  "?[10 = {...}$",
  "$10 * {a: Any}",
  "=10 * {a: Any}",
  "$[10 * {a: Any}]",
  "xxx[10 * {a: Any}]",
  "?$10 * {a: Any}]",
  "?=10 * {a: Any}]",
  "?[10 * {a: Any}$",
  "?[10 * {a: Any}$",
  "${a: Any}",
  "={a: Any}",
  "$[{a: Any}]",
  "xxx[{a: Any}]",
  "?${a: Any}]",
  "?={a: Any}]",
  "?[{a: Any}$",
  "?[{a: Any}$",
  "10 $ {a: Any}",
  "?10 $ {a: Any}",
  "?(10 $ {a: Any})",
  "$10 $ {a: Any}",
  "=10 $ {a: Any}",
  "$[10 $ {a: Any}]",
  "xxx[10 $ {a: Any}]",
  "?$10 $ {a: Any}]",
  "?=10 $ {a: Any}]",
  "?[10 $ {a: Any}$",
  "?[10 $ {a: Any}$",
  "10 = {a: Any}",
  "?10 = {a: Any}",
  "?(10 = {a: Any})",
  "$10 = {a: Any}",
  "=10 = {a: Any}",
  "$[10 = {a: Any}]",
  "xxx[10 = {a: Any}]",
  "?$10 = {a: Any}]",
  "?=10 = {a: Any}]",
  "?[10 = {a: Any}$",
  "?[10 = {a: Any}$",
  "$10 * {x: int64, a: Any}",
  "=10 * {x: int64, a: Any}",
  "$[10 * {x: int64, a: Any}]",
  "xxx[10 * {x: int64, a: Any}]",
  "?$10 * {x: int64, a: Any}]",
  "?=10 * {x: int64, a: Any}]",
  "?[10 * {x: int64, a: Any}$",
  "?[10 * {x: int64, a: Any}$",
  "${x: int64, a: Any}",
  "={x: int64, a: Any}",
  "$[{x: int64, a: Any}]",
  "xxx[{x: int64, a: Any}]",
  "?${x: int64, a: Any}]",
  "?={x: int64, a: Any}]",
  "?[{x: int64, a: Any}$",
  "?[{x: int64, a: Any}$",
  "10 $ {x: int64, a: Any}",
  "?10 $ {x: int64, a: Any}",
  "?(10 $ {x: int64, a: Any})",
  "$10 $ {x: int64, a: Any}",
  "=10 $ {x: int64, a: Any}",
  "$[10 $ {x: int64, a: Any}]",
  "xxx[10 $ {x: int64, a: Any}]",
  "?$10 $ {x: int64, a: Any}]",
  "?=10 $ {x: int64, a: Any}]",
  "?[10 $ {x: int64, a: Any}$",
  "?[10 $ {x: int64, a: Any}$",
  "10 = {x: int64, a: Any}",
  "?10 = {x: int64, a: Any}",
  "?(10 = {x: int64, a: Any})",
  "$10 = {x: int64, a: Any}",
  "=10 = {x: int64, a: Any}",
  "$[10 = {x: int64, a: Any}]",
  "xxx[10 = {x: int64, a: Any}]",
  "?$10 = {x: int64, a: Any}]",
  "?=10 = {x: int64, a: Any}]",
  "?[10 = {x: int64, a: Any}$",
  "?[10 = {x: int64, a: Any}$",
  "10 * {x: int64$ a: Any}",
  "?10 * {x: int64$ a: Any}",
  "?(10 * {x: int64$ a: Any})",
  "$10 * {x: int64$ a: Any}",
  "=10 * {x: int64$ a: Any}",
  "$[10 * {x: int64$ a: Any}]",
  "xxx[10 * {x: int64$ a: Any}]",
  "?$10 * {x: int64$ a: Any}]",
  "?=10 * {x: int64$ a: Any}]",
  "?[10 * {x: int64$ a: Any}$",
  "?[10 * {x: int64$ a: Any}$",
  "{x: int64$ a: Any}",
  "?{x: int64$ a: Any}",
  "?({x: int64$ a: Any})",
  "${x: int64$ a: Any}",
  "={x: int64$ a: Any}",
  "$[{x: int64$ a: Any}]",
  "xxx[{x: int64$ a: Any}]",
  "?${x: int64$ a: Any}]",
  "?={x: int64$ a: Any}]",
  "?[{x: int64$ a: Any}$",
  "?[{x: int64$ a: Any}$",
  "10 $ {x: int64$ a: Any}",
  "?10 $ {x: int64$ a: Any}",
  "?(10 $ {x: int64$ a: Any})",
  "$10 $ {x: int64$ a: Any}",
  "=10 $ {x: int64$ a: Any}",
  "$[10 $ {x: int64$ a: Any}]",
  "xxx[10 $ {x: int64$ a: Any}]",
  "?$10 $ {x: int64$ a: Any}]",
  "?=10 $ {x: int64$ a: Any}]",
  "?[10 $ {x: int64$ a: Any}$",
  "?[10 $ {x: int64$ a: Any}$",
  "10 = {x: int64$ a: Any}",
  "?10 = {x: int64$ a: Any}",
  "?(10 = {x: int64$ a: Any})",
  "$10 = {x: int64$ a: Any}",
  "=10 = {x: int64$ a: Any}",
  "$[10 = {x: int64$ a: Any}]",
  "xxx[10 = {x: int64$ a: Any}]",
  "?$10 = {x: int64$ a: Any}]",
  "?=10 = {x: int64$ a: Any}]",
  "?[10 = {x: int64$ a: Any}$",
  "?[10 = {x: int64$ a: Any}$",
  "10 * {x: int64= a: Any}",
  "?10 * {x: int64= a: Any}",
  "?(10 * {x: int64= a: Any})",
  "$10 * {x: int64= a: Any}",
  "=10 * {x: int64= a: Any}",
  "$[10 * {x: int64= a: Any}]",
  "xxx[10 * {x: int64= a: Any}]",
  "?$10 * {x: int64= a: Any}]",
  "?=10 * {x: int64= a: Any}]",
  "?[10 * {x: int64= a: Any}$",
  "?[10 * {x: int64= a: Any}$",
  "{x: int64= a: Any}",
  "?{x: int64= a: Any}",
  "?({x: int64= a: Any})",
  "${x: int64= a: Any}",
  "={x: int64= a: Any}",
  "$[{x: int64= a: Any}]",
  "xxx[{x: int64= a: Any}]",
  "?${x: int64= a: Any}]",
  "?={x: int64= a: Any}]",
  "?[{x: int64= a: Any}$",
  "?[{x: int64= a: Any}$",
  "10 $ {x: int64= a: Any}",
  "?10 $ {x: int64= a: Any}",
  "?(10 $ {x: int64= a: Any})",
  "$10 $ {x: int64= a: Any}",
  "=10 $ {x: int64= a: Any}",
  "$[10 $ {x: int64= a: Any}]",
  "xxx[10 $ {x: int64= a: Any}]",
  "?$10 $ {x: int64= a: Any}]",
  "?=10 $ {x: int64= a: Any}]",
  "?[10 $ {x: int64= a: Any}$",
  "?[10 $ {x: int64= a: Any}$",
  "10 = {x: int64= a: Any}",
  "?10 = {x: int64= a: Any}",
  "?(10 = {x: int64= a: Any})",
  "$10 = {x: int64= a: Any}",
  "=10 = {x: int64= a: Any}",
  "$[10 = {x: int64= a: Any}]",
  "xxx[10 = {x: int64= a: Any}]",
  "?$10 = {x: int64= a: Any}]",
  "?=10 = {x: int64= a: Any}]",
  "?[10 = {x: int64= a: Any}$",
  "?[10 = {x: int64= a: Any}$",
  "$10 * {x: int64, x: int64, a: Any}",
  "=10 * {x: int64, x: int64, a: Any}",
  "$[10 * {x: int64, x: int64, a: Any}]",
  "xxx[10 * {x: int64, x: int64, a: Any}]",
  "?$10 * {x: int64, x: int64, a: Any}]",
  "?=10 * {x: int64, x: int64, a: Any}]",
  "?[10 * {x: int64, x: int64, a: Any}$",
  "?[10 * {x: int64, x: int64, a: Any}$",
  "${x: int64, x: int64, a: Any}",
  "={x: int64, x: int64, a: Any}",
  "$[{x: int64, x: int64, a: Any}]",
  "xxx[{x: int64, x: int64, a: Any}]",
  "?${x: int64, x: int64, a: Any}]",
  "?={x: int64, x: int64, a: Any}]",
  "?[{x: int64, x: int64, a: Any}$",
  "?[{x: int64, x: int64, a: Any}$",
  "10 $ {x: int64, x: int64, a: Any}",
  "?10 $ {x: int64, x: int64, a: Any}",
  "?(10 $ {x: int64, x: int64, a: Any})",
  "$10 $ {x: int64, x: int64, a: Any}",
  "=10 $ {x: int64, x: int64, a: Any}",
  "$[10 $ {x: int64, x: int64, a: Any}]",
  "xxx[10 $ {x: int64, x: int64, a: Any}]",
  "?$10 $ {x: int64, x: int64, a: Any}]",
  "?=10 $ {x: int64, x: int64, a: Any}]",
  "?[10 $ {x: int64, x: int64, a: Any}$",
  "?[10 $ {x: int64, x: int64, a: Any}$",
  "10 = {x: int64, x: int64, a: Any}",
  "?10 = {x: int64, x: int64, a: Any}",
  "?(10 = {x: int64, x: int64, a: Any})",
  "$10 = {x: int64, x: int64, a: Any}",
  "=10 = {x: int64, x: int64, a: Any}",
  "$[10 = {x: int64, x: int64, a: Any}]",
  "xxx[10 = {x: int64, x: int64, a: Any}]",
  "?$10 = {x: int64, x: int64, a: Any}]",
  "?=10 = {x: int64, x: int64, a: Any}]",
  "?[10 = {x: int64, x: int64, a: Any}$",
  "?[10 = {x: int64, x: int64, a: Any}$",
  "10 * {x: int64, x: int64$ a: Any}",
  "?10 * {x: int64, x: int64$ a: Any}",
  "?(10 * {x: int64, x: int64$ a: Any})",
  "$10 * {x: int64, x: int64$ a: Any}",
  "=10 * {x: int64, x: int64$ a: Any}",
  "$[10 * {x: int64, x: int64$ a: Any}]",
  "xxx[10 * {x: int64, x: int64$ a: Any}]",
  "?$10 * {x: int64, x: int64$ a: Any}]",
  "?=10 * {x: int64, x: int64$ a: Any}]",
  "?[10 * {x: int64, x: int64$ a: Any}$",
  "?[10 * {x: int64, x: int64$ a: Any}$",
  "{x: int64, x: int64$ a: Any}",
  "?{x: int64, x: int64$ a: Any}",
  "?({x: int64, x: int64$ a: Any})",
  "${x: int64, x: int64$ a: Any}",
  "={x: int64, x: int64$ a: Any}",
  "$[{x: int64, x: int64$ a: Any}]",
  "xxx[{x: int64, x: int64$ a: Any}]",
  "?${x: int64, x: int64$ a: Any}]",
  "?={x: int64, x: int64$ a: Any}]",
  "?[{x: int64, x: int64$ a: Any}$",
  "?[{x: int64, x: int64$ a: Any}$",
  "10 $ {x: int64, x: int64$ a: Any}",
  "?10 $ {x: int64, x: int64$ a: Any}",
  "?(10 $ {x: int64, x: int64$ a: Any})",
  "$10 $ {x: int64, x: int64$ a: Any}",
  "=10 $ {x: int64, x: int64$ a: Any}",
  "$[10 $ {x: int64, x: int64$ a: Any}]",
  "xxx[10 $ {x: int64, x: int64$ a: Any}]",
  "?$10 $ {x: int64, x: int64$ a: Any}]",
  "?=10 $ {x: int64, x: int64$ a: Any}]",
  "?[10 $ {x: int64, x: int64$ a: Any}$",
  "?[10 $ {x: int64, x: int64$ a: Any}$",
  "10 = {x: int64, x: int64$ a: Any}",
  "?10 = {x: int64, x: int64$ a: Any}",
  "?(10 = {x: int64, x: int64$ a: Any})",
  "$10 = {x: int64, x: int64$ a: Any}",
  "=10 = {x: int64, x: int64$ a: Any}",
  "$[10 = {x: int64, x: int64$ a: Any}]",
  "xxx[10 = {x: int64, x: int64$ a: Any}]",
  "?$10 = {x: int64, x: int64$ a: Any}]",
  "?=10 = {x: int64, x: int64$ a: Any}]",
  "?[10 = {x: int64, x: int64$ a: Any}$",
  "?[10 = {x: int64, x: int64$ a: Any}$",
  "10 * {x: int64, x: int64= a: Any}",
  "?10 * {x: int64, x: int64= a: Any}",
  "?(10 * {x: int64, x: int64= a: Any})",
  "$10 * {x: int64, x: int64= a: Any}",
  "=10 * {x: int64, x: int64= a: Any}",
  "$[10 * {x: int64, x: int64= a: Any}]",
  "xxx[10 * {x: int64, x: int64= a: Any}]",
  "?$10 * {x: int64, x: int64= a: Any}]",
  "?=10 * {x: int64, x: int64= a: Any}]",
  "?[10 * {x: int64, x: int64= a: Any}$",
  "?[10 * {x: int64, x: int64= a: Any}$",
  "{x: int64, x: int64= a: Any}",
  "?{x: int64, x: int64= a: Any}",
  "?({x: int64, x: int64= a: Any})",
  "${x: int64, x: int64= a: Any}",
  "={x: int64, x: int64= a: Any}",
  "$[{x: int64, x: int64= a: Any}]",
  "xxx[{x: int64, x: int64= a: Any}]",
  "?${x: int64, x: int64= a: Any}]",
  "?={x: int64, x: int64= a: Any}]",
  "?[{x: int64, x: int64= a: Any}$",
  "?[{x: int64, x: int64= a: Any}$",
  "10 $ {x: int64, x: int64= a: Any}",
  "?10 $ {x: int64, x: int64= a: Any}",
  "?(10 $ {x: int64, x: int64= a: Any})",
  "$10 $ {x: int64, x: int64= a: Any}",
  "=10 $ {x: int64, x: int64= a: Any}",
  "$[10 $ {x: int64, x: int64= a: Any}]",
  "xxx[10 $ {x: int64, x: int64= a: Any}]",
  "?$10 $ {x: int64, x: int64= a: Any}]",
  "?=10 $ {x: int64, x: int64= a: Any}]",
  "?[10 $ {x: int64, x: int64= a: Any}$",
  "?[10 $ {x: int64, x: int64= a: Any}$",
  "10 = {x: int64, x: int64= a: Any}",
  "?10 = {x: int64, x: int64= a: Any}",
  "?(10 = {x: int64, x: int64= a: Any})",
  "$10 = {x: int64, x: int64= a: Any}",
  "=10 = {x: int64, x: int64= a: Any}",
  "$[10 = {x: int64, x: int64= a: Any}]",
  "xxx[10 = {x: int64, x: int64= a: Any}]",
  "?$10 = {x: int64, x: int64= a: Any}]",
  "?=10 = {x: int64, x: int64= a: Any}]",
  "?[10 = {x: int64, x: int64= a: Any}$",
  "?[10 = {x: int64, x: int64= a: Any}$",
  "10 * {$: Any}",
  "?10 * {$: Any}",
  "?(10 * {$: Any})",
  "$10 * {$: Any}",
  "=10 * {$: Any}",
  "$[10 * {$: Any}]",
  "xxx[10 * {$: Any}]",
  "?$10 * {$: Any}]",
  "?=10 * {$: Any}]",
  "?[10 * {$: Any}$",
  "?[10 * {$: Any}$",
  "{$: Any}",
  "?{$: Any}",
  "?({$: Any})",
  "${$: Any}",
  "={$: Any}",
  "$[{$: Any}]",
  "xxx[{$: Any}]",
  "?${$: Any}]",
  "?={$: Any}]",
  "?[{$: Any}$",
  "?[{$: Any}$",
  "10 $ {$: Any}",
  "?10 $ {$: Any}",
  "?(10 $ {$: Any})",
  "$10 $ {$: Any}",
  "=10 $ {$: Any}",
  "$[10 $ {$: Any}]",
  "xxx[10 $ {$: Any}]",
  "?$10 $ {$: Any}]",
  "?=10 $ {$: Any}]",
  "?[10 $ {$: Any}$",
  "?[10 $ {$: Any}$",
  "10 = {$: Any}",
  "?10 = {$: Any}",
  "?(10 = {$: Any})",
  "$10 = {$: Any}",
  "=10 = {$: Any}",
  "$[10 = {$: Any}]",
  "xxx[10 = {$: Any}]",
  "?$10 = {$: Any}]",
  "?=10 = {$: Any}]",
  "?[10 = {$: Any}$",
  "?[10 = {$: Any}$",
  "10 * {x: int64, $: Any}",
  "?10 * {x: int64, $: Any}",
  "?(10 * {x: int64, $: Any})",
  "$10 * {x: int64, $: Any}",
  "=10 * {x: int64, $: Any}",
  "$[10 * {x: int64, $: Any}]",
  "xxx[10 * {x: int64, $: Any}]",
  "?$10 * {x: int64, $: Any}]",
  "?=10 * {x: int64, $: Any}]",
  "?[10 * {x: int64, $: Any}$",
  "?[10 * {x: int64, $: Any}$",
  "{x: int64, $: Any}",
  "?{x: int64, $: Any}",
  "?({x: int64, $: Any})",
  "${x: int64, $: Any}",
  "={x: int64, $: Any}",
  "$[{x: int64, $: Any}]",
  "xxx[{x: int64, $: Any}]",
  "?${x: int64, $: Any}]",
  "?={x: int64, $: Any}]",
  "?[{x: int64, $: Any}$",
  "?[{x: int64, $: Any}$",
  "10 $ {x: int64, $: Any}",
  "?10 $ {x: int64, $: Any}",
  "?(10 $ {x: int64, $: Any})",
  "$10 $ {x: int64, $: Any}",
  "=10 $ {x: int64, $: Any}",
  "$[10 $ {x: int64, $: Any}]",
  "xxx[10 $ {x: int64, $: Any}]",
  "?$10 $ {x: int64, $: Any}]",
  "?=10 $ {x: int64, $: Any}]",
  "?[10 $ {x: int64, $: Any}$",
  "?[10 $ {x: int64, $: Any}$",
  "10 = {x: int64, $: Any}",
  "?10 = {x: int64, $: Any}",
  "?(10 = {x: int64, $: Any})",
  "$10 = {x: int64, $: Any}",
  "=10 = {x: int64, $: Any}",
  "$[10 = {x: int64, $: Any}]",
  "xxx[10 = {x: int64, $: Any}]",
  "?$10 = {x: int64, $: Any}]",
  "?=10 = {x: int64, $: Any}]",
  "?[10 = {x: int64, $: Any}$",
  "?[10 = {x: int64, $: Any}$",
  "10 * {x: int64$ $: Any}",
  "?10 * {x: int64$ $: Any}",
  "?(10 * {x: int64$ $: Any})",
  "$10 * {x: int64$ $: Any}",
  "=10 * {x: int64$ $: Any}",
  "$[10 * {x: int64$ $: Any}]",
  "xxx[10 * {x: int64$ $: Any}]",
  "?$10 * {x: int64$ $: Any}]",
  "?=10 * {x: int64$ $: Any}]",
  "?[10 * {x: int64$ $: Any}$",
  "?[10 * {x: int64$ $: Any}$",
  "{x: int64$ $: Any}",
  "?{x: int64$ $: Any}",
  "?({x: int64$ $: Any})",
  "${x: int64$ $: Any}",
  "={x: int64$ $: Any}",
  "$[{x: int64$ $: Any}]",
  "xxx[{x: int64$ $: Any}]",
  "?${x: int64$ $: Any}]",
  "?={x: int64$ $: Any}]",
  "?[{x: int64$ $: Any}$",
  "?[{x: int64$ $: Any}$",
  "10 $ {x: int64$ $: Any}",
  "?10 $ {x: int64$ $: Any}",
  "?(10 $ {x: int64$ $: Any})",
  "$10 $ {x: int64$ $: Any}",
  "=10 $ {x: int64$ $: Any}",
  "$[10 $ {x: int64$ $: Any}]",
  "xxx[10 $ {x: int64$ $: Any}]",
  "?$10 $ {x: int64$ $: Any}]",
  "?=10 $ {x: int64$ $: Any}]",
  "?[10 $ {x: int64$ $: Any}$",
  "?[10 $ {x: int64$ $: Any}$",
  "10 = {x: int64$ $: Any}",
  "?10 = {x: int64$ $: Any}",
  "?(10 = {x: int64$ $: Any})",
  "$10 = {x: int64$ $: Any}",
  "=10 = {x: int64$ $: Any}",
  "$[10 = {x: int64$ $: Any}]",
  "xxx[10 = {x: int64$ $: Any}]",
  "?$10 = {x: int64$ $: Any}]",
  "?=10 = {x: int64$ $: Any}]",
  "?[10 = {x: int64$ $: Any}$",
  "?[10 = {x: int64$ $: Any}$",
  "10 * {x: int64= $: Any}",
  "?10 * {x: int64= $: Any}",
  "?(10 * {x: int64= $: Any})",
  "$10 * {x: int64= $: Any}",
  "=10 * {x: int64= $: Any}",
  "$[10 * {x: int64= $: Any}]",
  "xxx[10 * {x: int64= $: Any}]",
  "?$10 * {x: int64= $: Any}]",
  "?=10 * {x: int64= $: Any}]",
  "?[10 * {x: int64= $: Any}$",
  "?[10 * {x: int64= $: Any}$",
  "{x: int64= $: Any}",
  "?{x: int64= $: Any}",
  "?({x: int64= $: Any})",
  "${x: int64= $: Any}",
  "={x: int64= $: Any}",
  "$[{x: int64= $: Any}]",
  "xxx[{x: int64= $: Any}]",
  "?${x: int64= $: Any}]",
  "?={x: int64= $: Any}]",
  "?[{x: int64= $: Any}$",
  "?[{x: int64= $: Any}$",
  "10 $ {x: int64= $: Any}",
  "?10 $ {x: int64= $: Any}",
  "?(10 $ {x: int64= $: Any})",
  "$10 $ {x: int64= $: Any}",
  "=10 $ {x: int64= $: Any}",
  "$[10 $ {x: int64= $: Any}]",
  "xxx[10 $ {x: int64= $: Any}]",
  "?$10 $ {x: int64= $: Any}]",
  "?=10 $ {x: int64= $: Any}]",
  "?[10 $ {x: int64= $: Any}$",
  "?[10 $ {x: int64= $: Any}$",
  "10 = {x: int64= $: Any}",
  "?10 = {x: int64= $: Any}",
  "?(10 = {x: int64= $: Any})",
  "$10 = {x: int64= $: Any}",
  "=10 = {x: int64= $: Any}",
  "$[10 = {x: int64= $: Any}]",
  "xxx[10 = {x: int64= $: Any}]",
  "?$10 = {x: int64= $: Any}]",
  "?=10 = {x: int64= $: Any}]",
  "?[10 = {x: int64= $: Any}$",
  "?[10 = {x: int64= $: Any}$",
  "10 * {x: int64, x: int64, $: Any}",
  "?10 * {x: int64, x: int64, $: Any}",
  "?(10 * {x: int64, x: int64, $: Any})",
  "$10 * {x: int64, x: int64, $: Any}",
  "=10 * {x: int64, x: int64, $: Any}",
  "$[10 * {x: int64, x: int64, $: Any}]",
  "xxx[10 * {x: int64, x: int64, $: Any}]",
  "?$10 * {x: int64, x: int64, $: Any}]",
  "?=10 * {x: int64, x: int64, $: Any}]",
  "?[10 * {x: int64, x: int64, $: Any}$",
  "?[10 * {x: int64, x: int64, $: Any}$",
  "{x: int64, x: int64, $: Any}",
  "?{x: int64, x: int64, $: Any}",
  "?({x: int64, x: int64, $: Any})",
  "${x: int64, x: int64, $: Any}",
  "={x: int64, x: int64, $: Any}",
  "$[{x: int64, x: int64, $: Any}]",
  "xxx[{x: int64, x: int64, $: Any}]",
  "?${x: int64, x: int64, $: Any}]",
  "?={x: int64, x: int64, $: Any}]",
  "?[{x: int64, x: int64, $: Any}$",
  "?[{x: int64, x: int64, $: Any}$",
  "10 $ {x: int64, x: int64, $: Any}",
  "?10 $ {x: int64, x: int64, $: Any}",
  "?(10 $ {x: int64, x: int64, $: Any})",
  "$10 $ {x: int64, x: int64, $: Any}",
  "=10 $ {x: int64, x: int64, $: Any}",
  "$[10 $ {x: int64, x: int64, $: Any}]",
  "xxx[10 $ {x: int64, x: int64, $: Any}]",
  "?$10 $ {x: int64, x: int64, $: Any}]",
  "?=10 $ {x: int64, x: int64, $: Any}]",
  "?[10 $ {x: int64, x: int64, $: Any}$",
  "?[10 $ {x: int64, x: int64, $: Any}$",
  "10 = {x: int64, x: int64, $: Any}",
  "?10 = {x: int64, x: int64, $: Any}",
  "?(10 = {x: int64, x: int64, $: Any})",
  "$10 = {x: int64, x: int64, $: Any}",
  "=10 = {x: int64, x: int64, $: Any}",
  "$[10 = {x: int64, x: int64, $: Any}]",
  "xxx[10 = {x: int64, x: int64, $: Any}]",
  "?$10 = {x: int64, x: int64, $: Any}]",
  "?=10 = {x: int64, x: int64, $: Any}]",
  "?[10 = {x: int64, x: int64, $: Any}$",
  "?[10 = {x: int64, x: int64, $: Any}$",
  "10 * {x: int64, x: int64$ $: Any}",
  "?10 * {x: int64, x: int64$ $: Any}",
  "?(10 * {x: int64, x: int64$ $: Any})",
  "$10 * {x: int64, x: int64$ $: Any}",
  "=10 * {x: int64, x: int64$ $: Any}",
  "$[10 * {x: int64, x: int64$ $: Any}]",
  "xxx[10 * {x: int64, x: int64$ $: Any}]",
  "?$10 * {x: int64, x: int64$ $: Any}]",
  "?=10 * {x: int64, x: int64$ $: Any}]",
  "?[10 * {x: int64, x: int64$ $: Any}$",
  "?[10 * {x: int64, x: int64$ $: Any}$",
  "{x: int64, x: int64$ $: Any}",
  "?{x: int64, x: int64$ $: Any}",
  "?({x: int64, x: int64$ $: Any})",
  "${x: int64, x: int64$ $: Any}",
  "={x: int64, x: int64$ $: Any}",
  "$[{x: int64, x: int64$ $: Any}]",
  "xxx[{x: int64, x: int64$ $: Any}]",
  "?${x: int64, x: int64$ $: Any}]",
  "?={x: int64, x: int64$ $: Any}]",
  "?[{x: int64, x: int64$ $: Any}$",
  "?[{x: int64, x: int64$ $: Any}$",
  "10 $ {x: int64, x: int64$ $: Any}",
  "?10 $ {x: int64, x: int64$ $: Any}",
  "?(10 $ {x: int64, x: int64$ $: Any})",
  "$10 $ {x: int64, x: int64$ $: Any}",
  "=10 $ {x: int64, x: int64$ $: Any}",
  "$[10 $ {x: int64, x: int64$ $: Any}]",
  "xxx[10 $ {x: int64, x: int64$ $: Any}]",
  "?$10 $ {x: int64, x: int64$ $: Any}]",
  "?=10 $ {x: int64, x: int64$ $: Any}]",
  "?[10 $ {x: int64, x: int64$ $: Any}$",
  "?[10 $ {x: int64, x: int64$ $: Any}$",
  "10 = {x: int64, x: int64$ $: Any}",
  "?10 = {x: int64, x: int64$ $: Any}",
  "?(10 = {x: int64, x: int64$ $: Any})",
  "$10 = {x: int64, x: int64$ $: Any}",
  "=10 = {x: int64, x: int64$ $: Any}",
  "$[10 = {x: int64, x: int64$ $: Any}]",
  "xxx[10 = {x: int64, x: int64$ $: Any}]",
  "?$10 = {x: int64, x: int64$ $: Any}]",
  "?=10 = {x: int64, x: int64$ $: Any}]",
  "?[10 = {x: int64, x: int64$ $: Any}$",
  "?[10 = {x: int64, x: int64$ $: Any}$",
  "10 * {x: int64, x: int64= $: Any}",
  "?10 * {x: int64, x: int64= $: Any}",
  "?(10 * {x: int64, x: int64= $: Any})",
  "$10 * {x: int64, x: int64= $: Any}",
  "=10 * {x: int64, x: int64= $: Any}",
  "$[10 * {x: int64, x: int64= $: Any}]",
  "xxx[10 * {x: int64, x: int64= $: Any}]",
  "?$10 * {x: int64, x: int64= $: Any}]",
  "?=10 * {x: int64, x: int64= $: Any}]",
  "?[10 * {x: int64, x: int64= $: Any}$",
  "?[10 * {x: int64, x: int64= $: Any}$",
  "{x: int64, x: int64= $: Any}",
  "?{x: int64, x: int64= $: Any}",
  "?({x: int64, x: int64= $: Any})",
  "${x: int64, x: int64= $: Any}",
  "={x: int64, x: int64= $: Any}",
  "$[{x: int64, x: int64= $: Any}]",
  "xxx[{x: int64, x: int64= $: Any}]",
  "?${x: int64, x: int64= $: Any}]",
  "?={x: int64, x: int64= $: Any}]",
  "?[{x: int64, x: int64= $: Any}$",
  "?[{x: int64, x: int64= $: Any}$",
  "10 $ {x: int64, x: int64= $: Any}",
  "?10 $ {x: int64, x: int64= $: Any}",
  "?(10 $ {x: int64, x: int64= $: Any})",
  "$10 $ {x: int64, x: int64= $: Any}",
  "=10 $ {x: int64, x: int64= $: Any}",
  "$[10 $ {x: int64, x: int64= $: Any}]",
  "xxx[10 $ {x: int64, x: int64= $: Any}]",
  "?$10 $ {x: int64, x: int64= $: Any}]",
  "?=10 $ {x: int64, x: int64= $: Any}]",
  "?[10 $ {x: int64, x: int64= $: Any}$",
  "?[10 $ {x: int64, x: int64= $: Any}$",
  "10 = {x: int64, x: int64= $: Any}",
  "?10 = {x: int64, x: int64= $: Any}",
  "?(10 = {x: int64, x: int64= $: Any})",
  "$10 = {x: int64, x: int64= $: Any}",
  "=10 = {x: int64, x: int64= $: Any}",
  "$[10 = {x: int64, x: int64= $: Any}]",
  "xxx[10 = {x: int64, x: int64= $: Any}]",
  "?$10 = {x: int64, x: int64= $: Any}]",
  "?=10 = {x: int64, x: int64= $: Any}]",
  "?[10 = {x: int64, x: int64= $: Any}$",
  "?[10 = {x: int64, x: int64= $: Any}$",
  "10 * {=: Any}",
  "?10 * {=: Any}",
  "?(10 * {=: Any})",
  "$10 * {=: Any}",
  "=10 * {=: Any}",
  "$[10 * {=: Any}]",
  "xxx[10 * {=: Any}]",
  "?$10 * {=: Any}]",
  "?=10 * {=: Any}]",
  "?[10 * {=: Any}$",
  "?[10 * {=: Any}$",
  "{=: Any}",
  "?{=: Any}",
  "?({=: Any})",
  "${=: Any}",
  "={=: Any}",
  "$[{=: Any}]",
  "xxx[{=: Any}]",
  "?${=: Any}]",
  "?={=: Any}]",
  "?[{=: Any}$",
  "?[{=: Any}$",
  "10 $ {=: Any}",
  "?10 $ {=: Any}",
  "?(10 $ {=: Any})",
  "$10 $ {=: Any}",
  "=10 $ {=: Any}",
  "$[10 $ {=: Any}]",
  "xxx[10 $ {=: Any}]",
  "?$10 $ {=: Any}]",
  "?=10 $ {=: Any}]",
  "?[10 $ {=: Any}$",
  "?[10 $ {=: Any}$",
  "10 = {=: Any}",
  "?10 = {=: Any}",
  "?(10 = {=: Any})",
  "$10 = {=: Any}",
  "=10 = {=: Any}",
  "$[10 = {=: Any}]",
  "xxx[10 = {=: Any}]",
  "?$10 = {=: Any}]",
  "?=10 = {=: Any}]",
  "?[10 = {=: Any}$",
  "?[10 = {=: Any}$",
  "10 * {x: int64, =: Any}",
  "?10 * {x: int64, =: Any}",
  "?(10 * {x: int64, =: Any})",
  "$10 * {x: int64, =: Any}",
  "=10 * {x: int64, =: Any}",
  "$[10 * {x: int64, =: Any}]",
  "xxx[10 * {x: int64, =: Any}]",
  "?$10 * {x: int64, =: Any}]",
  "?=10 * {x: int64, =: Any}]",
  "?[10 * {x: int64, =: Any}$",
  "?[10 * {x: int64, =: Any}$",
  "{x: int64, =: Any}",
  "?{x: int64, =: Any}",
  "?({x: int64, =: Any})",
  "${x: int64, =: Any}",
  "={x: int64, =: Any}",
  "$[{x: int64, =: Any}]",
  "xxx[{x: int64, =: Any}]",
  "?${x: int64, =: Any}]",
  "?={x: int64, =: Any}]",
  "?[{x: int64, =: Any}$",
  "?[{x: int64, =: Any}$",
  "10 $ {x: int64, =: Any}",
  "?10 $ {x: int64, =: Any}",
  "?(10 $ {x: int64, =: Any})",
  "$10 $ {x: int64, =: Any}",
  "=10 $ {x: int64, =: Any}",
  "$[10 $ {x: int64, =: Any}]",
  "xxx[10 $ {x: int64, =: Any}]",
  "?$10 $ {x: int64, =: Any}]",
  "?=10 $ {x: int64, =: Any}]",
  "?[10 $ {x: int64, =: Any}$",
  "?[10 $ {x: int64, =: Any}$",
  "10 = {x: int64, =: Any}",
  "?10 = {x: int64, =: Any}",
  "?(10 = {x: int64, =: Any})",
  "$10 = {x: int64, =: Any}",
  "=10 = {x: int64, =: Any}",
  "$[10 = {x: int64, =: Any}]",
  "xxx[10 = {x: int64, =: Any}]",
  "?$10 = {x: int64, =: Any}]",
  "?=10 = {x: int64, =: Any}]",
  "?[10 = {x: int64, =: Any}$",
  "?[10 = {x: int64, =: Any}$",
  "10 * {x: int64$ =: Any}",
  "?10 * {x: int64$ =: Any}",
  "?(10 * {x: int64$ =: Any})",
  "$10 * {x: int64$ =: Any}",
  "=10 * {x: int64$ =: Any}",
  "$[10 * {x: int64$ =: Any}]",
  "xxx[10 * {x: int64$ =: Any}]",
  "?$10 * {x: int64$ =: Any}]",
  "?=10 * {x: int64$ =: Any}]",
  "?[10 * {x: int64$ =: Any}$",
  "?[10 * {x: int64$ =: Any}$",
  "{x: int64$ =: Any}",
  "?{x: int64$ =: Any}",
  "?({x: int64$ =: Any})",
  "${x: int64$ =: Any}",
  "={x: int64$ =: Any}",
  "$[{x: int64$ =: Any}]",
  "xxx[{x: int64$ =: Any}]",
  "?${x: int64$ =: Any}]",
  "?={x: int64$ =: Any}]",
  "?[{x: int64$ =: Any}$",
  "?[{x: int64$ =: Any}$",
  "10 $ {x: int64$ =: Any}",
  "?10 $ {x: int64$ =: Any}",
  "?(10 $ {x: int64$ =: Any})",
  "$10 $ {x: int64$ =: Any}",
  "=10 $ {x: int64$ =: Any}",
  "$[10 $ {x: int64$ =: Any}]",
  "xxx[10 $ {x: int64$ =: Any}]",
  "?$10 $ {x: int64$ =: Any}]",
  "?=10 $ {x: int64$ =: Any}]",
  "?[10 $ {x: int64$ =: Any}$",
  "?[10 $ {x: int64$ =: Any}$",
  "10 = {x: int64$ =: Any}",
  "?10 = {x: int64$ =: Any}",
  "?(10 = {x: int64$ =: Any})",
  "$10 = {x: int64$ =: Any}",
  "=10 = {x: int64$ =: Any}",
  "$[10 = {x: int64$ =: Any}]",
  "xxx[10 = {x: int64$ =: Any}]",
  "?$10 = {x: int64$ =: Any}]",
  "?=10 = {x: int64$ =: Any}]",
  "?[10 = {x: int64$ =: Any}$",
  "?[10 = {x: int64$ =: Any}$",
  "10 * {x: int64= =: Any}",
  "?10 * {x: int64= =: Any}",
  "?(10 * {x: int64= =: Any})",
  "$10 * {x: int64= =: Any}",
  "=10 * {x: int64= =: Any}",
  "$[10 * {x: int64= =: Any}]",
  "xxx[10 * {x: int64= =: Any}]",
  "?$10 * {x: int64= =: Any}]",
  "?=10 * {x: int64= =: Any}]",
  "?[10 * {x: int64= =: Any}$",
  "?[10 * {x: int64= =: Any}$",
  "{x: int64= =: Any}",
  "?{x: int64= =: Any}",
  "?({x: int64= =: Any})",
  "${x: int64= =: Any}",
  "={x: int64= =: Any}",
  "$[{x: int64= =: Any}]",
  "xxx[{x: int64= =: Any}]",
  "?${x: int64= =: Any}]",
  "?={x: int64= =: Any}]",
  "?[{x: int64= =: Any}$",
  "?[{x: int64= =: Any}$",
  "10 $ {x: int64= =: Any}",
  "?10 $ {x: int64= =: Any}",
  "?(10 $ {x: int64= =: Any})",
  "$10 $ {x: int64= =: Any}",
  "=10 $ {x: int64= =: Any}",
  "$[10 $ {x: int64= =: Any}]",
  "xxx[10 $ {x: int64= =: Any}]",
  "?$10 $ {x: int64= =: Any}]",
  "?=10 $ {x: int64= =: Any}]",
  "?[10 $ {x: int64= =: Any}$",
  "?[10 $ {x: int64= =: Any}$",
  "10 = {x: int64= =: Any}",
  "?10 = {x: int64= =: Any}",
  "?(10 = {x: int64= =: Any})",
  "$10 = {x: int64= =: Any}",
  "=10 = {x: int64= =: Any}",
  "$[10 = {x: int64= =: Any}]",
  "xxx[10 = {x: int64= =: Any}]",
  "?$10 = {x: int64= =: Any}]",
  "?=10 = {x: int64= =: Any}]",
  "?[10 = {x: int64= =: Any}$",
  "?[10 = {x: int64= =: Any}$",
  "10 * {x: int64, x: int64, =: Any}",
  "?10 * {x: int64, x: int64, =: Any}",
  "?(10 * {x: int64, x: int64, =: Any})",
  "$10 * {x: int64, x: int64, =: Any}",
  "=10 * {x: int64, x: int64, =: Any}",
  "$[10 * {x: int64, x: int64, =: Any}]",
  "xxx[10 * {x: int64, x: int64, =: Any}]",
  "?$10 * {x: int64, x: int64, =: Any}]",
  "?=10 * {x: int64, x: int64, =: Any}]",
  "?[10 * {x: int64, x: int64, =: Any}$",
  "?[10 * {x: int64, x: int64, =: Any}$",
  "{x: int64, x: int64, =: Any}",
  "?{x: int64, x: int64, =: Any}",
  "?({x: int64, x: int64, =: Any})",
  "${x: int64, x: int64, =: Any}",
  "={x: int64, x: int64, =: Any}",
  "$[{x: int64, x: int64, =: Any}]",
  "xxx[{x: int64, x: int64, =: Any}]",
  "?${x: int64, x: int64, =: Any}]",
  "?={x: int64, x: int64, =: Any}]",
  "?[{x: int64, x: int64, =: Any}$",
  "?[{x: int64, x: int64, =: Any}$",
  "10 $ {x: int64, x: int64, =: Any}",
  "?10 $ {x: int64, x: int64, =: Any}",
  "?(10 $ {x: int64, x: int64, =: Any})",
  "$10 $ {x: int64, x: int64, =: Any}",
  "=10 $ {x: int64, x: int64, =: Any}",
  "$[10 $ {x: int64, x: int64, =: Any}]",
  "xxx[10 $ {x: int64, x: int64, =: Any}]",
  "?$10 $ {x: int64, x: int64, =: Any}]",
  "?=10 $ {x: int64, x: int64, =: Any}]",
  "?[10 $ {x: int64, x: int64, =: Any}$",
  "?[10 $ {x: int64, x: int64, =: Any}$",
  "10 = {x: int64, x: int64, =: Any}",
  "?10 = {x: int64, x: int64, =: Any}",
  "?(10 = {x: int64, x: int64, =: Any})",
  "$10 = {x: int64, x: int64, =: Any}",
  "=10 = {x: int64, x: int64, =: Any}",
  "$[10 = {x: int64, x: int64, =: Any}]",
  "xxx[10 = {x: int64, x: int64, =: Any}]",
  "?$10 = {x: int64, x: int64, =: Any}]",
  "?=10 = {x: int64, x: int64, =: Any}]",
  "?[10 = {x: int64, x: int64, =: Any}$",
  "?[10 = {x: int64, x: int64, =: Any}$",
  "10 * {x: int64, x: int64$ =: Any}",
  "?10 * {x: int64, x: int64$ =: Any}",
  "?(10 * {x: int64, x: int64$ =: Any})",
  "$10 * {x: int64, x: int64$ =: Any}",
  "=10 * {x: int64, x: int64$ =: Any}",
  "$[10 * {x: int64, x: int64$ =: Any}]",
  "xxx[10 * {x: int64, x: int64$ =: Any}]",
  "?$10 * {x: int64, x: int64$ =: Any}]",
  "?=10 * {x: int64, x: int64$ =: Any}]",
  "?[10 * {x: int64, x: int64$ =: Any}$",
  "?[10 * {x: int64, x: int64$ =: Any}$",
  "{x: int64, x: int64$ =: Any}",
  "?{x: int64, x: int64$ =: Any}",
  "?({x: int64, x: int64$ =: Any})",
  "${x: int64, x: int64$ =: Any}",
  "={x: int64, x: int64$ =: Any}",
  "$[{x: int64, x: int64$ =: Any}]",
  "xxx[{x: int64, x: int64$ =: Any}]",
  "?${x: int64, x: int64$ =: Any}]",
  "?={x: int64, x: int64$ =: Any}]",
  "?[{x: int64, x: int64$ =: Any}$",
  "?[{x: int64, x: int64$ =: Any}$",
  "10 $ {x: int64, x: int64$ =: Any}",
  "?10 $ {x: int64, x: int64$ =: Any}",
  "?(10 $ {x: int64, x: int64$ =: Any})",
  "$10 $ {x: int64, x: int64$ =: Any}",
  "=10 $ {x: int64, x: int64$ =: Any}",
  "$[10 $ {x: int64, x: int64$ =: Any}]",
  "xxx[10 $ {x: int64, x: int64$ =: Any}]",
  "?$10 $ {x: int64, x: int64$ =: Any}]",
  "?=10 $ {x: int64, x: int64$ =: Any}]",
  "?[10 $ {x: int64, x: int64$ =: Any}$",
  "?[10 $ {x: int64, x: int64$ =: Any}$",
  "10 = {x: int64, x: int64$ =: Any}",
  "?10 = {x: int64, x: int64$ =: Any}",
  "?(10 = {x: int64, x: int64$ =: Any})",
  "$10 = {x: int64, x: int64$ =: Any}",
  "=10 = {x: int64, x: int64$ =: Any}",
  "$[10 = {x: int64, x: int64$ =: Any}]",
  "xxx[10 = {x: int64, x: int64$ =: Any}]",
  "?$10 = {x: int64, x: int64$ =: Any}]",
  "?=10 = {x: int64, x: int64$ =: Any}]",
  "?[10 = {x: int64, x: int64$ =: Any}$",
  "?[10 = {x: int64, x: int64$ =: Any}$",
  "10 * {x: int64, x: int64= =: Any}",
  "?10 * {x: int64, x: int64= =: Any}",
  "?(10 * {x: int64, x: int64= =: Any})",
  "$10 * {x: int64, x: int64= =: Any}",
  "=10 * {x: int64, x: int64= =: Any}",
  "$[10 * {x: int64, x: int64= =: Any}]",
  "xxx[10 * {x: int64, x: int64= =: Any}]",
  "?$10 * {x: int64, x: int64= =: Any}]",
  "?=10 * {x: int64, x: int64= =: Any}]",
  "?[10 * {x: int64, x: int64= =: Any}$",
  "?[10 * {x: int64, x: int64= =: Any}$",
  "{x: int64, x: int64= =: Any}",
  "?{x: int64, x: int64= =: Any}",
  "?({x: int64, x: int64= =: Any})",
  "${x: int64, x: int64= =: Any}",
  "={x: int64, x: int64= =: Any}",
  "$[{x: int64, x: int64= =: Any}]",
  "xxx[{x: int64, x: int64= =: Any}]",
  "?${x: int64, x: int64= =: Any}]",
  "?={x: int64, x: int64= =: Any}]",
  "?[{x: int64, x: int64= =: Any}$",
  "?[{x: int64, x: int64= =: Any}$",
  "10 $ {x: int64, x: int64= =: Any}",
  "?10 $ {x: int64, x: int64= =: Any}",
  "?(10 $ {x: int64, x: int64= =: Any})",
  "$10 $ {x: int64, x: int64= =: Any}",
  "=10 $ {x: int64, x: int64= =: Any}",
  "$[10 $ {x: int64, x: int64= =: Any}]",
  "xxx[10 $ {x: int64, x: int64= =: Any}]",
  "?$10 $ {x: int64, x: int64= =: Any}]",
  "?=10 $ {x: int64, x: int64= =: Any}]",
  "?[10 $ {x: int64, x: int64= =: Any}$",
  "?[10 $ {x: int64, x: int64= =: Any}$",
  "10 = {x: int64, x: int64= =: Any}",
  "?10 = {x: int64, x: int64= =: Any}",
  "?(10 = {x: int64, x: int64= =: Any})",
  "$10 = {x: int64, x: int64= =: Any}",
  "=10 = {x: int64, x: int64= =: Any}",
  "$[10 = {x: int64, x: int64= =: Any}]",
  "xxx[10 = {x: int64, x: int64= =: Any}]",
  "?$10 = {x: int64, x: int64= =: Any}]",
  "?=10 = {x: int64, x: int64= =: Any}]",
  "?[10 = {x: int64, x: int64= =: Any}$",
  "?[10 = {x: int64, x: int64= =: Any}$",
  "10 * {a$ Any}",
  "?10 * {a$ Any}",
  "?(10 * {a$ Any})",
  "$10 * {a$ Any}",
  "=10 * {a$ Any}",
  "$[10 * {a$ Any}]",
  "xxx[10 * {a$ Any}]",
  "?$10 * {a$ Any}]",
  "?=10 * {a$ Any}]",
  "?[10 * {a$ Any}$",
  "?[10 * {a$ Any}$",
  "{a$ Any}",
  "?{a$ Any}",
  "?({a$ Any})",
  "${a$ Any}",
  "={a$ Any}",
  "$[{a$ Any}]",
  "xxx[{a$ Any}]",
  "?${a$ Any}]",
  "?={a$ Any}]",
  "?[{a$ Any}$",
  "?[{a$ Any}$",
  "10 $ {a$ Any}",
  "?10 $ {a$ Any}",
  "?(10 $ {a$ Any})",
  "$10 $ {a$ Any}",
  "=10 $ {a$ Any}",
  "$[10 $ {a$ Any}]",
  "xxx[10 $ {a$ Any}]",
  "?$10 $ {a$ Any}]",
  "?=10 $ {a$ Any}]",
  "?[10 $ {a$ Any}$",
  "?[10 $ {a$ Any}$",
  "10 = {a$ Any}",
  "?10 = {a$ Any}",
  "?(10 = {a$ Any})",
  "$10 = {a$ Any}",
  "=10 = {a$ Any}",
  "$[10 = {a$ Any}]",
  "xxx[10 = {a$ Any}]",
  "?$10 = {a$ Any}]",
  "?=10 = {a$ Any}]",
  "?[10 = {a$ Any}$",
  "?[10 = {a$ Any}$",
  "10 * {x: int64, a$ Any}",
  "?10 * {x: int64, a$ Any}",
  "?(10 * {x: int64, a$ Any})",
  "$10 * {x: int64, a$ Any}",
  "=10 * {x: int64, a$ Any}",
  "$[10 * {x: int64, a$ Any}]",
  "xxx[10 * {x: int64, a$ Any}]",
  "?$10 * {x: int64, a$ Any}]",
  "?=10 * {x: int64, a$ Any}]",
  "?[10 * {x: int64, a$ Any}$",
  "?[10 * {x: int64, a$ Any}$",
  "{x: int64, a$ Any}",
  "?{x: int64, a$ Any}",
  "?({x: int64, a$ Any})",
  "${x: int64, a$ Any}",
  "={x: int64, a$ Any}",
  "$[{x: int64, a$ Any}]",
  "xxx[{x: int64, a$ Any}]",
  "?${x: int64, a$ Any}]",
  "?={x: int64, a$ Any}]",
  "?[{x: int64, a$ Any}$",
  "?[{x: int64, a$ Any}$",
  "10 $ {x: int64, a$ Any}",
  "?10 $ {x: int64, a$ Any}",
  "?(10 $ {x: int64, a$ Any})",
  "$10 $ {x: int64, a$ Any}",
  "=10 $ {x: int64, a$ Any}",
  "$[10 $ {x: int64, a$ Any}]",
  "xxx[10 $ {x: int64, a$ Any}]",
  "?$10 $ {x: int64, a$ Any}]",
  "?=10 $ {x: int64, a$ Any}]",
  "?[10 $ {x: int64, a$ Any}$",
  "?[10 $ {x: int64, a$ Any}$",
  "10 = {x: int64, a$ Any}",
  "?10 = {x: int64, a$ Any}",
  "?(10 = {x: int64, a$ Any})",
  "$10 = {x: int64, a$ Any}",
  "=10 = {x: int64, a$ Any}",
  "$[10 = {x: int64, a$ Any}]",
  "xxx[10 = {x: int64, a$ Any}]",
  "?$10 = {x: int64, a$ Any}]",
  "?=10 = {x: int64, a$ Any}]",
  "?[10 = {x: int64, a$ Any}$",
  "?[10 = {x: int64, a$ Any}$",
  "10 * {x: int64$ a$ Any}",
  "?10 * {x: int64$ a$ Any}",
  "?(10 * {x: int64$ a$ Any})",
  "$10 * {x: int64$ a$ Any}",
  "=10 * {x: int64$ a$ Any}",
  "$[10 * {x: int64$ a$ Any}]",
  "xxx[10 * {x: int64$ a$ Any}]",
  "?$10 * {x: int64$ a$ Any}]",
  "?=10 * {x: int64$ a$ Any}]",
  "?[10 * {x: int64$ a$ Any}$",
  "?[10 * {x: int64$ a$ Any}$",
  "{x: int64$ a$ Any}",
  "?{x: int64$ a$ Any}",
  "?({x: int64$ a$ Any})",
  "${x: int64$ a$ Any}",
  "={x: int64$ a$ Any}",
  "$[{x: int64$ a$ Any}]",
  "xxx[{x: int64$ a$ Any}]",
  "?${x: int64$ a$ Any}]",
  "?={x: int64$ a$ Any}]",
  "?[{x: int64$ a$ Any}$",
  "?[{x: int64$ a$ Any}$",
  "10 $ {x: int64$ a$ Any}",
  "?10 $ {x: int64$ a$ Any}",
  "?(10 $ {x: int64$ a$ Any})",
  "$10 $ {x: int64$ a$ Any}",
  "=10 $ {x: int64$ a$ Any}",
  "$[10 $ {x: int64$ a$ Any}]",
  "xxx[10 $ {x: int64$ a$ Any}]",
  "?$10 $ {x: int64$ a$ Any}]",
  "?=10 $ {x: int64$ a$ Any}]",
  "?[10 $ {x: int64$ a$ Any}$",
  "?[10 $ {x: int64$ a$ Any}$",
  "10 = {x: int64$ a$ Any}",
  "?10 = {x: int64$ a$ Any}",
  "?(10 = {x: int64$ a$ Any})",
  "$10 = {x: int64$ a$ Any}",
  "=10 = {x: int64$ a$ Any}",
  "$[10 = {x: int64$ a$ Any}]",
  "xxx[10 = {x: int64$ a$ Any}]",
  "?$10 = {x: int64$ a$ Any}]",
  "?=10 = {x: int64$ a$ Any}]",
  "?[10 = {x: int64$ a$ Any}$",
  "?[10 = {x: int64$ a$ Any}$",
  "10 * {x: int64= a$ Any}",
  "?10 * {x: int64= a$ Any}",
  "?(10 * {x: int64= a$ Any})",
  "$10 * {x: int64= a$ Any}",
  "=10 * {x: int64= a$ Any}",
  "$[10 * {x: int64= a$ Any}]",
  "xxx[10 * {x: int64= a$ Any}]",
  "?$10 * {x: int64= a$ Any}]",
  "?=10 * {x: int64= a$ Any}]",
  "?[10 * {x: int64= a$ Any}$",
  "?[10 * {x: int64= a$ Any}$",
  "{x: int64= a$ Any}",
  "?{x: int64= a$ Any}",
  "?({x: int64= a$ Any})",
  "${x: int64= a$ Any}",
  "={x: int64= a$ Any}",
  "$[{x: int64= a$ Any}]",
  "xxx[{x: int64= a$ Any}]",
  "?${x: int64= a$ Any}]",
  "?={x: int64= a$ Any}]",
  "?[{x: int64= a$ Any}$",
  "?[{x: int64= a$ Any}$",
  "10 $ {x: int64= a$ Any}",
  "?10 $ {x: int64= a$ Any}",
  "?(10 $ {x: int64= a$ Any})",
  "$10 $ {x: int64= a$ Any}",
  "=10 $ {x: int64= a$ Any}",
  "$[10 $ {x: int64= a$ Any}]",
  "xxx[10 $ {x: int64= a$ Any}]",
  "?$10 $ {x: int64= a$ Any}]",
  "?=10 $ {x: int64= a$ Any}]",
  "?[10 $ {x: int64= a$ Any}$",
  "?[10 $ {x: int64= a$ Any}$",
  "10 = {x: int64= a$ Any}",
  "?10 = {x: int64= a$ Any}",
  "?(10 = {x: int64= a$ Any})",
  "$10 = {x: int64= a$ Any}",
  "=10 = {x: int64= a$ Any}",
  "$[10 = {x: int64= a$ Any}]",
  "xxx[10 = {x: int64= a$ Any}]",
  "?$10 = {x: int64= a$ Any}]",
  "?=10 = {x: int64= a$ Any}]",
  "?[10 = {x: int64= a$ Any}$",
  "?[10 = {x: int64= a$ Any}$",
  "10 * {x: int64, x: int64, a$ Any}",
  "?10 * {x: int64, x: int64, a$ Any}",
  "?(10 * {x: int64, x: int64, a$ Any})",
  "$10 * {x: int64, x: int64, a$ Any}",
  "=10 * {x: int64, x: int64, a$ Any}",
  "$[10 * {x: int64, x: int64, a$ Any}]",
  "xxx[10 * {x: int64, x: int64, a$ Any}]",
  "?$10 * {x: int64, x: int64, a$ Any}]",
  "?=10 * {x: int64, x: int64, a$ Any}]",
  "?[10 * {x: int64, x: int64, a$ Any}$",
  "?[10 * {x: int64, x: int64, a$ Any}$",
  "{x: int64, x: int64, a$ Any}",
  "?{x: int64, x: int64, a$ Any}",
  "?({x: int64, x: int64, a$ Any})",
  "${x: int64, x: int64, a$ Any}",
  "={x: int64, x: int64, a$ Any}",
  "$[{x: int64, x: int64, a$ Any}]",
  "xxx[{x: int64, x: int64, a$ Any}]",
  "?${x: int64, x: int64, a$ Any}]",
  "?={x: int64, x: int64, a$ Any}]",
  "?[{x: int64, x: int64, a$ Any}$",
  "?[{x: int64, x: int64, a$ Any}$",
  "10 $ {x: int64, x: int64, a$ Any}",
  "?10 $ {x: int64, x: int64, a$ Any}",
  "?(10 $ {x: int64, x: int64, a$ Any})",
  "$10 $ {x: int64, x: int64, a$ Any}",
  "=10 $ {x: int64, x: int64, a$ Any}",
  "$[10 $ {x: int64, x: int64, a$ Any}]",
  "xxx[10 $ {x: int64, x: int64, a$ Any}]",
  "?$10 $ {x: int64, x: int64, a$ Any}]",
  "?=10 $ {x: int64, x: int64, a$ Any}]",
  "?[10 $ {x: int64, x: int64, a$ Any}$",
  "?[10 $ {x: int64, x: int64, a$ Any}$",
  "10 = {x: int64, x: int64, a$ Any}",
  "?10 = {x: int64, x: int64, a$ Any}",
  "?(10 = {x: int64, x: int64, a$ Any})",
  "$10 = {x: int64, x: int64, a$ Any}",
  "=10 = {x: int64, x: int64, a$ Any}",
  "$[10 = {x: int64, x: int64, a$ Any}]",
  "xxx[10 = {x: int64, x: int64, a$ Any}]",
  "?$10 = {x: int64, x: int64, a$ Any}]",
  "?=10 = {x: int64, x: int64, a$ Any}]",
  "?[10 = {x: int64, x: int64, a$ Any}$",
  "?[10 = {x: int64, x: int64, a$ Any}$",
  "10 * {x: int64, x: int64$ a$ Any}",
  "?10 * {x: int64, x: int64$ a$ Any}",
  "?(10 * {x: int64, x: int64$ a$ Any})",
  "$10 * {x: int64, x: int64$ a$ Any}",
  "=10 * {x: int64, x: int64$ a$ Any}",
  "$[10 * {x: int64, x: int64$ a$ Any}]",
  "xxx[10 * {x: int64, x: int64$ a$ Any}]",
  "?$10 * {x: int64, x: int64$ a$ Any}]",
  "?=10 * {x: int64, x: int64$ a$ Any}]",
  "?[10 * {x: int64, x: int64$ a$ Any}$",
  "?[10 * {x: int64, x: int64$ a$ Any}$",
  "{x: int64, x: int64$ a$ Any}",
  "?{x: int64, x: int64$ a$ Any}",
  "?({x: int64, x: int64$ a$ Any})",
  "${x: int64, x: int64$ a$ Any}",
  "={x: int64, x: int64$ a$ Any}",
  "$[{x: int64, x: int64$ a$ Any}]",
  "xxx[{x: int64, x: int64$ a$ Any}]",
  "?${x: int64, x: int64$ a$ Any}]",
  "?={x: int64, x: int64$ a$ Any}]",
  "?[{x: int64, x: int64$ a$ Any}$",
  "?[{x: int64, x: int64$ a$ Any}$",
  "10 $ {x: int64, x: int64$ a$ Any}",
  "?10 $ {x: int64, x: int64$ a$ Any}",
  "?(10 $ {x: int64, x: int64$ a$ Any})",
  "$10 $ {x: int64, x: int64$ a$ Any}",
  "=10 $ {x: int64, x: int64$ a$ Any}",
  "$[10 $ {x: int64, x: int64$ a$ Any}]",
  "xxx[10 $ {x: int64, x: int64$ a$ Any}]",
  "?$10 $ {x: int64, x: int64$ a$ Any}]",
  "?=10 $ {x: int64, x: int64$ a$ Any}]",
  "?[10 $ {x: int64, x: int64$ a$ Any}$",
  "?[10 $ {x: int64, x: int64$ a$ Any}$",
  "10 = {x: int64, x: int64$ a$ Any}",
  "?10 = {x: int64, x: int64$ a$ Any}",
  "?(10 = {x: int64, x: int64$ a$ Any})",
  "$10 = {x: int64, x: int64$ a$ Any}",
  "=10 = {x: int64, x: int64$ a$ Any}",
  "$[10 = {x: int64, x: int64$ a$ Any}]",
  "xxx[10 = {x: int64, x: int64$ a$ Any}]",
  "?$10 = {x: int64, x: int64$ a$ Any}]",
  "?=10 = {x: int64, x: int64$ a$ Any}]",
  "?[10 = {x: int64, x: int64$ a$ Any}$",
  "?[10 = {x: int64, x: int64$ a$ Any}$",
  "10 * {x: int64, x: int64= a$ Any}",
  "?10 * {x: int64, x: int64= a$ Any}",
  "?(10 * {x: int64, x: int64= a$ Any})",
  "$10 * {x: int64, x: int64= a$ Any}",
  "=10 * {x: int64, x: int64= a$ Any}",
  "$[10 * {x: int64, x: int64= a$ Any}]",
  "xxx[10 * {x: int64, x: int64= a$ Any}]",
  "?$10 * {x: int64, x: int64= a$ Any}]",
  "?=10 * {x: int64, x: int64= a$ Any}]",
  "?[10 * {x: int64, x: int64= a$ Any}$",
  "?[10 * {x: int64, x: int64= a$ Any}$",
  "{x: int64, x: int64= a$ Any}",
  "?{x: int64, x: int64= a$ Any}",
  "?({x: int64, x: int64= a$ Any})",
  "${x: int64, x: int64= a$ Any}",
  "={x: int64, x: int64= a$ Any}",
  "$[{x: int64, x: int64= a$ Any}]",
  "xxx[{x: int64, x: int64= a$ Any}]",
  "?${x: int64, x: int64= a$ Any}]",
  "?={x: int64, x: int64= a$ Any}]",
  "?[{x: int64, x: int64= a$ Any}$",
  "?[{x: int64, x: int64= a$ Any}$",
  "10 $ {x: int64, x: int64= a$ Any}",
  "?10 $ {x: int64, x: int64= a$ Any}",
  "?(10 $ {x: int64, x: int64= a$ Any})",
  "$10 $ {x: int64, x: int64= a$ Any}",
  "=10 $ {x: int64, x: int64= a$ Any}",
  "$[10 $ {x: int64, x: int64= a$ Any}]",
  "xxx[10 $ {x: int64, x: int64= a$ Any}]",
  "?$10 $ {x: int64, x: int64= a$ Any}]",
  "?=10 $ {x: int64, x: int64= a$ Any}]",
  "?[10 $ {x: int64, x: int64= a$ Any}$",
  "?[10 $ {x: int64, x: int64= a$ Any}$",
  "10 = {x: int64, x: int64= a$ Any}",
  "?10 = {x: int64, x: int64= a$ Any}",
  "?(10 = {x: int64, x: int64= a$ Any})",
  "$10 = {x: int64, x: int64= a$ Any}",
  "=10 = {x: int64, x: int64= a$ Any}",
  "$[10 = {x: int64, x: int64= a$ Any}]",
  "xxx[10 = {x: int64, x: int64= a$ Any}]",
  "?$10 = {x: int64, x: int64= a$ Any}]",
  "?=10 = {x: int64, x: int64= a$ Any}]",
  "?[10 = {x: int64, x: int64= a$ Any}$",
  "?[10 = {x: int64, x: int64= a$ Any}$",
  "10 * {a: $}",
  "?10 * {a: $}",
  "?(10 * {a: $})",
  "$10 * {a: $}",
  "=10 * {a: $}",
  "$[10 * {a: $}]",
  "xxx[10 * {a: $}]",
  "?$10 * {a: $}]",
  "?=10 * {a: $}]",
  "?[10 * {a: $}$",
  "?[10 * {a: $}$",
  "{a: $}",
  "?{a: $}",
  "?({a: $})",
  "${a: $}",
  "={a: $}",
  "$[{a: $}]",
  "xxx[{a: $}]",
  "?${a: $}]",
  "?={a: $}]",
  "?[{a: $}$",
  "?[{a: $}$",
  "10 $ {a: $}",
  "?10 $ {a: $}",
  "?(10 $ {a: $})",
  "$10 $ {a: $}",
  "=10 $ {a: $}",
  "$[10 $ {a: $}]",
  "xxx[10 $ {a: $}]",
  "?$10 $ {a: $}]",
  "?=10 $ {a: $}]",
  "?[10 $ {a: $}$",
  "?[10 $ {a: $}$",
  "10 = {a: $}",
  "?10 = {a: $}",
  "?(10 = {a: $})",
  "$10 = {a: $}",
  "=10 = {a: $}",
  "$[10 = {a: $}]",
  "xxx[10 = {a: $}]",
  "?$10 = {a: $}]",
  "?=10 = {a: $}]",
  "?[10 = {a: $}$",
  "?[10 = {a: $}$",
  "10 * {x: int64, a: $}",
  "?10 * {x: int64, a: $}",
  "?(10 * {x: int64, a: $})",
  "$10 * {x: int64, a: $}",
  "=10 * {x: int64, a: $}",
  "$[10 * {x: int64, a: $}]",
  "xxx[10 * {x: int64, a: $}]",
  "?$10 * {x: int64, a: $}]",
  "?=10 * {x: int64, a: $}]",
  "?[10 * {x: int64, a: $}$",
  "?[10 * {x: int64, a: $}$",
  "{x: int64, a: $}",
  "?{x: int64, a: $}",
  "?({x: int64, a: $})",
  "${x: int64, a: $}",
  "={x: int64, a: $}",
  "$[{x: int64, a: $}]",
  "xxx[{x: int64, a: $}]",
  "?${x: int64, a: $}]",
  "?={x: int64, a: $}]",
  "?[{x: int64, a: $}$",
  "?[{x: int64, a: $}$",
  "10 $ {x: int64, a: $}",
  "?10 $ {x: int64, a: $}",
  "?(10 $ {x: int64, a: $})",
  "$10 $ {x: int64, a: $}",
  "=10 $ {x: int64, a: $}",
  "$[10 $ {x: int64, a: $}]",
  "xxx[10 $ {x: int64, a: $}]",
  "?$10 $ {x: int64, a: $}]",
  "?=10 $ {x: int64, a: $}]",
  "?[10 $ {x: int64, a: $}$",
  "?[10 $ {x: int64, a: $}$",
  "10 = {x: int64, a: $}",
  "?10 = {x: int64, a: $}",
  "?(10 = {x: int64, a: $})",
  "$10 = {x: int64, a: $}",
  "=10 = {x: int64, a: $}",
  "$[10 = {x: int64, a: $}]",
  "xxx[10 = {x: int64, a: $}]",
  "?$10 = {x: int64, a: $}]",
  "?=10 = {x: int64, a: $}]",
  "?[10 = {x: int64, a: $}$",
  "?[10 = {x: int64, a: $}$",
  "10 * {x: int64$ a: $}",
  "?10 * {x: int64$ a: $}",
  "?(10 * {x: int64$ a: $})",
  "$10 * {x: int64$ a: $}",
  "=10 * {x: int64$ a: $}",
  "$[10 * {x: int64$ a: $}]",
  "xxx[10 * {x: int64$ a: $}]",
  "?$10 * {x: int64$ a: $}]",
  "?=10 * {x: int64$ a: $}]",
  "?[10 * {x: int64$ a: $}$",
  "?[10 * {x: int64$ a: $}$",
  "{x: int64$ a: $}",
  "?{x: int64$ a: $}",
  "?({x: int64$ a: $})",
  "${x: int64$ a: $}",
  "={x: int64$ a: $}",
  "$[{x: int64$ a: $}]",
  "xxx[{x: int64$ a: $}]",
  "?${x: int64$ a: $}]",
  "?={x: int64$ a: $}]",
  "?[{x: int64$ a: $}$",
  "?[{x: int64$ a: $}$",
  "10 $ {x: int64$ a: $}",
  "?10 $ {x: int64$ a: $}",
  "?(10 $ {x: int64$ a: $})",
  "$10 $ {x: int64$ a: $}",
  "=10 $ {x: int64$ a: $}",
  "$[10 $ {x: int64$ a: $}]",
  "xxx[10 $ {x: int64$ a: $}]",
  "?$10 $ {x: int64$ a: $}]",
  "?=10 $ {x: int64$ a: $}]",
  "?[10 $ {x: int64$ a: $}$",
  "?[10 $ {x: int64$ a: $}$",
  "10 = {x: int64$ a: $}",
  "?10 = {x: int64$ a: $}",
  "?(10 = {x: int64$ a: $})",
  "$10 = {x: int64$ a: $}",
  "=10 = {x: int64$ a: $}",
  "$[10 = {x: int64$ a: $}]",
  "xxx[10 = {x: int64$ a: $}]",
  "?$10 = {x: int64$ a: $}]",
  "?=10 = {x: int64$ a: $}]",
  "?[10 = {x: int64$ a: $}$",
  "?[10 = {x: int64$ a: $}$",
  "10 * {x: int64= a: $}",
  "?10 * {x: int64= a: $}",
  "?(10 * {x: int64= a: $})",
  "$10 * {x: int64= a: $}",
  "=10 * {x: int64= a: $}",
  "$[10 * {x: int64= a: $}]",
  "xxx[10 * {x: int64= a: $}]",
  "?$10 * {x: int64= a: $}]",
  "?=10 * {x: int64= a: $}]",
  "?[10 * {x: int64= a: $}$",
  "?[10 * {x: int64= a: $}$",
  "{x: int64= a: $}",
  "?{x: int64= a: $}",
  "?({x: int64= a: $})",
  "${x: int64= a: $}",
  "={x: int64= a: $}",
  "$[{x: int64= a: $}]",
  "xxx[{x: int64= a: $}]",
  "?${x: int64= a: $}]",
  "?={x: int64= a: $}]",
  "?[{x: int64= a: $}$",
  "?[{x: int64= a: $}$",
  "10 $ {x: int64= a: $}",
  "?10 $ {x: int64= a: $}",
  "?(10 $ {x: int64= a: $})",
  "$10 $ {x: int64= a: $}",
  "=10 $ {x: int64= a: $}",
  "$[10 $ {x: int64= a: $}]",
  "xxx[10 $ {x: int64= a: $}]",
  "?$10 $ {x: int64= a: $}]",
  "?=10 $ {x: int64= a: $}]",
  "?[10 $ {x: int64= a: $}$",
  "?[10 $ {x: int64= a: $}$",
  "10 = {x: int64= a: $}",
  "?10 = {x: int64= a: $}",
  "?(10 = {x: int64= a: $})",
  "$10 = {x: int64= a: $}",
  "=10 = {x: int64= a: $}",
  "$[10 = {x: int64= a: $}]",
  "xxx[10 = {x: int64= a: $}]",
  "?$10 = {x: int64= a: $}]",
  "?=10 = {x: int64= a: $}]",
  "?[10 = {x: int64= a: $}$",
  "?[10 = {x: int64= a: $}$",
  "10 * {x: int64, x: int64, a: $}",
  "?10 * {x: int64, x: int64, a: $}",
  "?(10 * {x: int64, x: int64, a: $})",
  "$10 * {x: int64, x: int64, a: $}",
  "=10 * {x: int64, x: int64, a: $}",
  "$[10 * {x: int64, x: int64, a: $}]",
  "xxx[10 * {x: int64, x: int64, a: $}]",
  "?$10 * {x: int64, x: int64, a: $}]",
  "?=10 * {x: int64, x: int64, a: $}]",
  "?[10 * {x: int64, x: int64, a: $}$",
  "?[10 * {x: int64, x: int64, a: $}$",
  "{x: int64, x: int64, a: $}",
  "?{x: int64, x: int64, a: $}",
  "?({x: int64, x: int64, a: $})",
  "${x: int64, x: int64, a: $}",
  "={x: int64, x: int64, a: $}",
  "$[{x: int64, x: int64, a: $}]",
  "xxx[{x: int64, x: int64, a: $}]",
  "?${x: int64, x: int64, a: $}]",
  "?={x: int64, x: int64, a: $}]",
  "?[{x: int64, x: int64, a: $}$",
  "?[{x: int64, x: int64, a: $}$",
  "10 $ {x: int64, x: int64, a: $}",
  "?10 $ {x: int64, x: int64, a: $}",
  "?(10 $ {x: int64, x: int64, a: $})",
  "$10 $ {x: int64, x: int64, a: $}",
  "=10 $ {x: int64, x: int64, a: $}",
  "$[10 $ {x: int64, x: int64, a: $}]",
  "xxx[10 $ {x: int64, x: int64, a: $}]",
  "?$10 $ {x: int64, x: int64, a: $}]",
  "?=10 $ {x: int64, x: int64, a: $}]",
  "?[10 $ {x: int64, x: int64, a: $}$",
  "?[10 $ {x: int64, x: int64, a: $}$",
  "10 = {x: int64, x: int64, a: $}",
  "?10 = {x: int64, x: int64, a: $}",
  "?(10 = {x: int64, x: int64, a: $})",
  "$10 = {x: int64, x: int64, a: $}",
  "=10 = {x: int64, x: int64, a: $}",
  "$[10 = {x: int64, x: int64, a: $}]",
  "xxx[10 = {x: int64, x: int64, a: $}]",
  "?$10 = {x: int64, x: int64, a: $}]",
  "?=10 = {x: int64, x: int64, a: $}]",
  "?[10 = {x: int64, x: int64, a: $}$",
  "?[10 = {x: int64, x: int64, a: $}$",
  "10 * {x: int64, x: int64$ a: $}",
  "?10 * {x: int64, x: int64$ a: $}",
  "?(10 * {x: int64, x: int64$ a: $})",
  "$10 * {x: int64, x: int64$ a: $}",
  "=10 * {x: int64, x: int64$ a: $}",
  "$[10 * {x: int64, x: int64$ a: $}]",
  "xxx[10 * {x: int64, x: int64$ a: $}]",
  "?$10 * {x: int64, x: int64$ a: $}]",
  "?=10 * {x: int64, x: int64$ a: $}]",
  "?[10 * {x: int64, x: int64$ a: $}$",
  "?[10 * {x: int64, x: int64$ a: $}$",
  "{x: int64, x: int64$ a: $}",
  "?{x: int64, x: int64$ a: $}",
  "?({x: int64, x: int64$ a: $})",
  "${x: int64, x: int64$ a: $}",
  "={x: int64, x: int64$ a: $}",
  "$[{x: int64, x: int64$ a: $}]",
  "xxx[{x: int64, x: int64$ a: $}]",
  "?${x: int64, x: int64$ a: $}]",
  "?={x: int64, x: int64$ a: $}]",
  "?[{x: int64, x: int64$ a: $}$",
  "?[{x: int64, x: int64$ a: $}$",
  "10 $ {x: int64, x: int64$ a: $}",
  "?10 $ {x: int64, x: int64$ a: $}",
  "?(10 $ {x: int64, x: int64$ a: $})",
  "$10 $ {x: int64, x: int64$ a: $}",
  "=10 $ {x: int64, x: int64$ a: $}",
  "$[10 $ {x: int64, x: int64$ a: $}]",
  "xxx[10 $ {x: int64, x: int64$ a: $}]",
  "?$10 $ {x: int64, x: int64$ a: $}]",
  "?=10 $ {x: int64, x: int64$ a: $}]",
  "?[10 $ {x: int64, x: int64$ a: $}$",
  "?[10 $ {x: int64, x: int64$ a: $}$",
  "10 = {x: int64, x: int64$ a: $}",
  "?10 = {x: int64, x: int64$ a: $}",
  "?(10 = {x: int64, x: int64$ a: $})",
  "$10 = {x: int64, x: int64$ a: $}",
  "=10 = {x: int64, x: int64$ a: $}",
  "$[10 = {x: int64, x: int64$ a: $}]",
  "xxx[10 = {x: int64, x: int64$ a: $}]",
  "?$10 = {x: int64, x: int64$ a: $}]",
  "?=10 = {x: int64, x: int64$ a: $}]",
  "?[10 = {x: int64, x: int64$ a: $}$",
  "?[10 = {x: int64, x: int64$ a: $}$",
  "10 * {x: int64, x: int64= a: $}",
  "?10 * {x: int64, x: int64= a: $}",
  "?(10 * {x: int64, x: int64= a: $})",
  "$10 * {x: int64, x: int64= a: $}",
  "=10 * {x: int64, x: int64= a: $}",
  "$[10 * {x: int64, x: int64= a: $}]",
  "xxx[10 * {x: int64, x: int64= a: $}]",
  "?$10 * {x: int64, x: int64= a: $}]",
  "?=10 * {x: int64, x: int64= a: $}]",
  "?[10 * {x: int64, x: int64= a: $}$",
  "?[10 * {x: int64, x: int64= a: $}$",
  "{x: int64, x: int64= a: $}",
  "?{x: int64, x: int64= a: $}",
  "?({x: int64, x: int64= a: $})",
  "${x: int64, x: int64= a: $}",
  "={x: int64, x: int64= a: $}",
  "$[{x: int64, x: int64= a: $}]",
  "xxx[{x: int64, x: int64= a: $}]",
  "?${x: int64, x: int64= a: $}]",
  "?={x: int64, x: int64= a: $}]",
  "?[{x: int64, x: int64= a: $}$",
  "?[{x: int64, x: int64= a: $}$",
  "10 $ {x: int64, x: int64= a: $}",
  "?10 $ {x: int64, x: int64= a: $}",
  "?(10 $ {x: int64, x: int64= a: $})",
  "$10 $ {x: int64, x: int64= a: $}",
  "=10 $ {x: int64, x: int64= a: $}",
  "$[10 $ {x: int64, x: int64= a: $}]",
  "xxx[10 $ {x: int64, x: int64= a: $}]",
  "?$10 $ {x: int64, x: int64= a: $}]",
  "?=10 $ {x: int64, x: int64= a: $}]",
  "?[10 $ {x: int64, x: int64= a: $}$",
  "?[10 $ {x: int64, x: int64= a: $}$",
  "10 = {x: int64, x: int64= a: $}",
  "?10 = {x: int64, x: int64= a: $}",
  "?(10 = {x: int64, x: int64= a: $})",
  "$10 = {x: int64, x: int64= a: $}",
  "=10 = {x: int64, x: int64= a: $}",
  "$[10 = {x: int64, x: int64= a: $}]",
  "xxx[10 = {x: int64, x: int64= a: $}]",
  "?$10 = {x: int64, x: int64= a: $}]",
  "?=10 = {x: int64, x: int64= a: $}]",
  "?[10 = {x: int64, x: int64= a: $}$",
  "?[10 = {x: int64, x: int64= a: $}$",
  "10 * {a: xxx}",
  "?10 * {a: xxx}",
  "?(10 * {a: xxx})",
  "$10 * {a: xxx}",
  "=10 * {a: xxx}",
  "$[10 * {a: xxx}]",
  "xxx[10 * {a: xxx}]",
  "?$10 * {a: xxx}]",
  "?=10 * {a: xxx}]",
  "?[10 * {a: xxx}$",
  "?[10 * {a: xxx}$",
  "{a: xxx}",
  "?{a: xxx}",
  "?({a: xxx})",
  "${a: xxx}",
  "={a: xxx}",
  "$[{a: xxx}]",
  "xxx[{a: xxx}]",
  "?${a: xxx}]",
  "?={a: xxx}]",
  "?[{a: xxx}$",
  "?[{a: xxx}$",
  "10 $ {a: xxx}",
  "?10 $ {a: xxx}",
  "?(10 $ {a: xxx})",
  "$10 $ {a: xxx}",
  "=10 $ {a: xxx}",
  "$[10 $ {a: xxx}]",
  "xxx[10 $ {a: xxx}]",
  "?$10 $ {a: xxx}]",
  "?=10 $ {a: xxx}]",
  "?[10 $ {a: xxx}$",
  "?[10 $ {a: xxx}$",
  "10 = {a: xxx}",
  "?10 = {a: xxx}",
  "?(10 = {a: xxx})",
  "$10 = {a: xxx}",
  "=10 = {a: xxx}",
  "$[10 = {a: xxx}]",
  "xxx[10 = {a: xxx}]",
  "?$10 = {a: xxx}]",
  "?=10 = {a: xxx}]",
  "?[10 = {a: xxx}$",
  "?[10 = {a: xxx}$",
  "10 * {x: int64, a: xxx}",
  "?10 * {x: int64, a: xxx}",
  "?(10 * {x: int64, a: xxx})",
  "$10 * {x: int64, a: xxx}",
  "=10 * {x: int64, a: xxx}",
  "$[10 * {x: int64, a: xxx}]",
  "xxx[10 * {x: int64, a: xxx}]",
  "?$10 * {x: int64, a: xxx}]",
  "?=10 * {x: int64, a: xxx}]",
  "?[10 * {x: int64, a: xxx}$",
  "?[10 * {x: int64, a: xxx}$",
  "{x: int64, a: xxx}",
  "?{x: int64, a: xxx}",
  "?({x: int64, a: xxx})",
  "${x: int64, a: xxx}",
  "={x: int64, a: xxx}",
  "$[{x: int64, a: xxx}]",
  "xxx[{x: int64, a: xxx}]",
  "?${x: int64, a: xxx}]",
  "?={x: int64, a: xxx}]",
  "?[{x: int64, a: xxx}$",
  "?[{x: int64, a: xxx}$",
  "10 $ {x: int64, a: xxx}",
  "?10 $ {x: int64, a: xxx}",
  "?(10 $ {x: int64, a: xxx})",
  "$10 $ {x: int64, a: xxx}",
  "=10 $ {x: int64, a: xxx}",
  "$[10 $ {x: int64, a: xxx}]",
  "xxx[10 $ {x: int64, a: xxx}]",
  "?$10 $ {x: int64, a: xxx}]",
  "?=10 $ {x: int64, a: xxx}]",
  "?[10 $ {x: int64, a: xxx}$",
  "?[10 $ {x: int64, a: xxx}$",
  "10 = {x: int64, a: xxx}",
  "?10 = {x: int64, a: xxx}",
  "?(10 = {x: int64, a: xxx})",
  "$10 = {x: int64, a: xxx}",
  "=10 = {x: int64, a: xxx}",
  "$[10 = {x: int64, a: xxx}]",
  "xxx[10 = {x: int64, a: xxx}]",
  "?$10 = {x: int64, a: xxx}]",
  "?=10 = {x: int64, a: xxx}]",
  "?[10 = {x: int64, a: xxx}$",
  "?[10 = {x: int64, a: xxx}$",
  "10 * {x: int64$ a: xxx}",
  "?10 * {x: int64$ a: xxx}",
  "?(10 * {x: int64$ a: xxx})",
  "$10 * {x: int64$ a: xxx}",
  "=10 * {x: int64$ a: xxx}",
  "$[10 * {x: int64$ a: xxx}]",
  "xxx[10 * {x: int64$ a: xxx}]",
  "?$10 * {x: int64$ a: xxx}]",
  "?=10 * {x: int64$ a: xxx}]",
  "?[10 * {x: int64$ a: xxx}$",
  "?[10 * {x: int64$ a: xxx}$",
  "{x: int64$ a: xxx}",
  "?{x: int64$ a: xxx}",
  "?({x: int64$ a: xxx})",
  "${x: int64$ a: xxx}",
  "={x: int64$ a: xxx}",
  "$[{x: int64$ a: xxx}]",
  "xxx[{x: int64$ a: xxx}]",
  "?${x: int64$ a: xxx}]",
  "?={x: int64$ a: xxx}]",
  "?[{x: int64$ a: xxx}$",
  "?[{x: int64$ a: xxx}$",
  "10 $ {x: int64$ a: xxx}",
  "?10 $ {x: int64$ a: xxx}",
  "?(10 $ {x: int64$ a: xxx})",
  "$10 $ {x: int64$ a: xxx}",
  "=10 $ {x: int64$ a: xxx}",
  "$[10 $ {x: int64$ a: xxx}]",
  "xxx[10 $ {x: int64$ a: xxx}]",
  "?$10 $ {x: int64$ a: xxx}]",
  "?=10 $ {x: int64$ a: xxx}]",
  "?[10 $ {x: int64$ a: xxx}$",
  "?[10 $ {x: int64$ a: xxx}$",
  "10 = {x: int64$ a: xxx}",
  "?10 = {x: int64$ a: xxx}",
  "?(10 = {x: int64$ a: xxx})",
  "$10 = {x: int64$ a: xxx}",
  "=10 = {x: int64$ a: xxx}",
  "$[10 = {x: int64$ a: xxx}]",
  "xxx[10 = {x: int64$ a: xxx}]",
  "?$10 = {x: int64$ a: xxx}]",
  "?=10 = {x: int64$ a: xxx}]",
  "?[10 = {x: int64$ a: xxx}$",
  "?[10 = {x: int64$ a: xxx}$",
  "10 * {x: int64= a: xxx}",
  "?10 * {x: int64= a: xxx}",
  "?(10 * {x: int64= a: xxx})",
  "$10 * {x: int64= a: xxx}",
  "=10 * {x: int64= a: xxx}",
  "$[10 * {x: int64= a: xxx}]",
  "xxx[10 * {x: int64= a: xxx}]",
  "?$10 * {x: int64= a: xxx}]",
  "?=10 * {x: int64= a: xxx}]",
  "?[10 * {x: int64= a: xxx}$",
  "?[10 * {x: int64= a: xxx}$",
  "{x: int64= a: xxx}",
  "?{x: int64= a: xxx}",
  "?({x: int64= a: xxx})",
  "${x: int64= a: xxx}",
  "={x: int64= a: xxx}",
  "$[{x: int64= a: xxx}]",
  "xxx[{x: int64= a: xxx}]",
  "?${x: int64= a: xxx}]",
  "?={x: int64= a: xxx}]",
  "?[{x: int64= a: xxx}$",
  "?[{x: int64= a: xxx}$",
  "10 $ {x: int64= a: xxx}",
  "?10 $ {x: int64= a: xxx}",
  "?(10 $ {x: int64= a: xxx})",
  "$10 $ {x: int64= a: xxx}",
  "=10 $ {x: int64= a: xxx}",
  "$[10 $ {x: int64= a: xxx}]",
  "xxx[10 $ {x: int64= a: xxx}]",
  "?$10 $ {x: int64= a: xxx}]",
  "?=10 $ {x: int64= a: xxx}]",
  "?[10 $ {x: int64= a: xxx}$",
  "?[10 $ {x: int64= a: xxx}$",
  "10 = {x: int64= a: xxx}",
  "?10 = {x: int64= a: xxx}",
  "?(10 = {x: int64= a: xxx})",
  "$10 = {x: int64= a: xxx}",
  "=10 = {x: int64= a: xxx}",
  "$[10 = {x: int64= a: xxx}]",
  "xxx[10 = {x: int64= a: xxx}]",
  "?$10 = {x: int64= a: xxx}]",
  "?=10 = {x: int64= a: xxx}]",
  "?[10 = {x: int64= a: xxx}$",
  "?[10 = {x: int64= a: xxx}$",
  "10 * {x: int64, x: int64, a: xxx}",
  "?10 * {x: int64, x: int64, a: xxx}",
  "?(10 * {x: int64, x: int64, a: xxx})",
  "$10 * {x: int64, x: int64, a: xxx}",
  "=10 * {x: int64, x: int64, a: xxx}",
  "$[10 * {x: int64, x: int64, a: xxx}]",
  "xxx[10 * {x: int64, x: int64, a: xxx}]",
  "?$10 * {x: int64, x: int64, a: xxx}]",
  "?=10 * {x: int64, x: int64, a: xxx}]",
  "?[10 * {x: int64, x: int64, a: xxx}$",
  "?[10 * {x: int64, x: int64, a: xxx}$",
  "{x: int64, x: int64, a: xxx}",
  "?{x: int64, x: int64, a: xxx}",
  "?({x: int64, x: int64, a: xxx})",
  "${x: int64, x: int64, a: xxx}",
  "={x: int64, x: int64, a: xxx}",
  "$[{x: int64, x: int64, a: xxx}]",
  "xxx[{x: int64, x: int64, a: xxx}]",
  "?${x: int64, x: int64, a: xxx}]",
  "?={x: int64, x: int64, a: xxx}]",
  "?[{x: int64, x: int64, a: xxx}$",
  "?[{x: int64, x: int64, a: xxx}$",
  "10 $ {x: int64, x: int64, a: xxx}",
  "?10 $ {x: int64, x: int64, a: xxx}",
  "?(10 $ {x: int64, x: int64, a: xxx})",
  "$10 $ {x: int64, x: int64, a: xxx}",
  "=10 $ {x: int64, x: int64, a: xxx}",
  "$[10 $ {x: int64, x: int64, a: xxx}]",
  "xxx[10 $ {x: int64, x: int64, a: xxx}]",
  "?$10 $ {x: int64, x: int64, a: xxx}]",
  "?=10 $ {x: int64, x: int64, a: xxx}]",
  "?[10 $ {x: int64, x: int64, a: xxx}$",
  "?[10 $ {x: int64, x: int64, a: xxx}$",
  "10 = {x: int64, x: int64, a: xxx}",
  "?10 = {x: int64, x: int64, a: xxx}",
  "?(10 = {x: int64, x: int64, a: xxx})",
  "$10 = {x: int64, x: int64, a: xxx}",
  "=10 = {x: int64, x: int64, a: xxx}",
  "$[10 = {x: int64, x: int64, a: xxx}]",
  "xxx[10 = {x: int64, x: int64, a: xxx}]",
  "?$10 = {x: int64, x: int64, a: xxx}]",
  "?=10 = {x: int64, x: int64, a: xxx}]",
  "?[10 = {x: int64, x: int64, a: xxx}$",
  "?[10 = {x: int64, x: int64, a: xxx}$",
  "10 * {x: int64, x: int64$ a: xxx}",
  "?10 * {x: int64, x: int64$ a: xxx}",
  "?(10 * {x: int64, x: int64$ a: xxx})",
  "$10 * {x: int64, x: int64$ a: xxx}",
  "=10 * {x: int64, x: int64$ a: xxx}",
  "$[10 * {x: int64, x: int64$ a: xxx}]",
  "xxx[10 * {x: int64, x: int64$ a: xxx}]",
  "?$10 * {x: int64, x: int64$ a: xxx}]",
  "?=10 * {x: int64, x: int64$ a: xxx}]",
  "?[10 * {x: int64, x: int64$ a: xxx}$",
  "?[10 * {x: int64, x: int64$ a: xxx}$",
  "{x: int64, x: int64$ a: xxx}",
  "?{x: int64, x: int64$ a: xxx}",
  "?({x: int64, x: int64$ a: xxx})",
  "${x: int64, x: int64$ a: xxx}",
  "={x: int64, x: int64$ a: xxx}",
  "$[{x: int64, x: int64$ a: xxx}]",
  "xxx[{x: int64, x: int64$ a: xxx}]",
  "?${x: int64, x: int64$ a: xxx}]",
  "?={x: int64, x: int64$ a: xxx}]",
  "?[{x: int64, x: int64$ a: xxx}$",
  "?[{x: int64, x: int64$ a: xxx}$",
  "10 $ {x: int64, x: int64$ a: xxx}",
  "?10 $ {x: int64, x: int64$ a: xxx}",
  "?(10 $ {x: int64, x: int64$ a: xxx})",
  "$10 $ {x: int64, x: int64$ a: xxx}",
  "=10 $ {x: int64, x: int64$ a: xxx}",
  "$[10 $ {x: int64, x: int64$ a: xxx}]",
  "xxx[10 $ {x: int64, x: int64$ a: xxx}]",
  "?$10 $ {x: int64, x: int64$ a: xxx}]",
  "?=10 $ {x: int64, x: int64$ a: xxx}]",
  "?[10 $ {x: int64, x: int64$ a: xxx}$",
  "?[10 $ {x: int64, x: int64$ a: xxx}$",
  "10 = {x: int64, x: int64$ a: xxx}",
  "?10 = {x: int64, x: int64$ a: xxx}",
  "?(10 = {x: int64, x: int64$ a: xxx})",
  "$10 = {x: int64, x: int64$ a: xxx}",
  "=10 = {x: int64, x: int64$ a: xxx}",
  "$[10 = {x: int64, x: int64$ a: xxx}]",
  "xxx[10 = {x: int64, x: int64$ a: xxx}]",
  "?$10 = {x: int64, x: int64$ a: xxx}]",
  "?=10 = {x: int64, x: int64$ a: xxx}]",
  "?[10 = {x: int64, x: int64$ a: xxx}$",
  "?[10 = {x: int64, x: int64$ a: xxx}$",
  "10 * {x: int64, x: int64= a: xxx}",
  "?10 * {x: int64, x: int64= a: xxx}",
  "?(10 * {x: int64, x: int64= a: xxx})",
  "$10 * {x: int64, x: int64= a: xxx}",
  "=10 * {x: int64, x: int64= a: xxx}",
  "$[10 * {x: int64, x: int64= a: xxx}]",
  "xxx[10 * {x: int64, x: int64= a: xxx}]",
  "?$10 * {x: int64, x: int64= a: xxx}]",
  "?=10 * {x: int64, x: int64= a: xxx}]",
  "?[10 * {x: int64, x: int64= a: xxx}$",
  "?[10 * {x: int64, x: int64= a: xxx}$",
  "{x: int64, x: int64= a: xxx}",
  "?{x: int64, x: int64= a: xxx}",
  "?({x: int64, x: int64= a: xxx})",
  "${x: int64, x: int64= a: xxx}",
  "={x: int64, x: int64= a: xxx}",
  "$[{x: int64, x: int64= a: xxx}]",
  "xxx[{x: int64, x: int64= a: xxx}]",
  "?${x: int64, x: int64= a: xxx}]",
  "?={x: int64, x: int64= a: xxx}]",
  "?[{x: int64, x: int64= a: xxx}$",
  "?[{x: int64, x: int64= a: xxx}$",
  "10 $ {x: int64, x: int64= a: xxx}",
  "?10 $ {x: int64, x: int64= a: xxx}",
  "?(10 $ {x: int64, x: int64= a: xxx})",
  "$10 $ {x: int64, x: int64= a: xxx}",
  "=10 $ {x: int64, x: int64= a: xxx}",
  "$[10 $ {x: int64, x: int64= a: xxx}]",
  "xxx[10 $ {x: int64, x: int64= a: xxx}]",
  "?$10 $ {x: int64, x: int64= a: xxx}]",
  "?=10 $ {x: int64, x: int64= a: xxx}]",
  "?[10 $ {x: int64, x: int64= a: xxx}$",
  "?[10 $ {x: int64, x: int64= a: xxx}$",
  "10 = {x: int64, x: int64= a: xxx}",
  "?10 = {x: int64, x: int64= a: xxx}",
  "?(10 = {x: int64, x: int64= a: xxx})",
  "$10 = {x: int64, x: int64= a: xxx}",
  "=10 = {x: int64, x: int64= a: xxx}",
  "$[10 = {x: int64, x: int64= a: xxx}]",
  "xxx[10 = {x: int64, x: int64= a: xxx}]",
  "?$10 = {x: int64, x: int64= a: xxx}]",
  "?=10 = {x: int64, x: int64= a: xxx}]",
  "?[10 = {x: int64, x: int64= a: xxx}$",
  "?[10 = {x: int64, x: int64= a: xxx}$",
  "$10 * {a : int64}",
  "=10 * {a : int64}",
  "$[10 * {a : int64}]",
  "xxx[10 * {a : int64}]",
  "?$10 * {a : int64}]",
  "?=10 * {a : int64}]",
  "?[10 * {a : int64}$",
  "?[10 * {a : int64}$",
  "${a : int64}",
  "={a : int64}",
  "$[{a : int64}]",
  "xxx[{a : int64}]",
  "?${a : int64}]",
  "?={a : int64}]",
  "?[{a : int64}$",
  "?[{a : int64}$",
  "10 $ {a : int64}",
  "?10 $ {a : int64}",
  "?(10 $ {a : int64})",
  "$10 $ {a : int64}",
  "=10 $ {a : int64}",
  "$[10 $ {a : int64}]",
  "xxx[10 $ {a : int64}]",
  "?$10 $ {a : int64}]",
  "?=10 $ {a : int64}]",
  "?[10 $ {a : int64}$",
  "?[10 $ {a : int64}$",
  "10 = {a : int64}",
  "?10 = {a : int64}",
  "?(10 = {a : int64})",
  "$10 = {a : int64}",
  "=10 = {a : int64}",
  "$[10 = {a : int64}]",
  "xxx[10 = {a : int64}]",
  "?$10 = {a : int64}]",
  "?=10 = {a : int64}]",
  "?[10 = {a : int64}$",
  "?[10 = {a : int64}$",
  "$10 * {a : int64, }",
  "=10 * {a : int64, }",
  "$[10 * {a : int64, }]",
  "xxx[10 * {a : int64, }]",
  "?$10 * {a : int64, }]",
  "?=10 * {a : int64, }]",
  "?[10 * {a : int64, }$",
  "?[10 * {a : int64, }$",
  "${a : int64, }",
  "={a : int64, }",
  "$[{a : int64, }]",
  "xxx[{a : int64, }]",
  "?${a : int64, }]",
  "?={a : int64, }]",
  "?[{a : int64, }$",
  "?[{a : int64, }$",
  "10 $ {a : int64, }",
  "?10 $ {a : int64, }",
  "?(10 $ {a : int64, })",
  "$10 $ {a : int64, }",
  "=10 $ {a : int64, }",
  "$[10 $ {a : int64, }]",
  "xxx[10 $ {a : int64, }]",
  "?$10 $ {a : int64, }]",
  "?=10 $ {a : int64, }]",
  "?[10 $ {a : int64, }$",
  "?[10 $ {a : int64, }$",
  "10 = {a : int64, }",
  "?10 = {a : int64, }",
  "?(10 = {a : int64, })",
  "$10 = {a : int64, }",
  "=10 = {a : int64, }",
  "$[10 = {a : int64, }]",
  "xxx[10 = {a : int64, }]",
  "?$10 = {a : int64, }]",
  "?=10 = {a : int64, }]",
  "?[10 = {a : int64, }$",
  "?[10 = {a : int64, }$",
  "$10 * {a : int64, ...}",
  "=10 * {a : int64, ...}",
  "$[10 * {a : int64, ...}]",
  "xxx[10 * {a : int64, ...}]",
  "?$10 * {a : int64, ...}]",
  "?=10 * {a : int64, ...}]",
  "?[10 * {a : int64, ...}$",
  "?[10 * {a : int64, ...}$",
  "${a : int64, ...}",
  "={a : int64, ...}",
  "$[{a : int64, ...}]",
  "xxx[{a : int64, ...}]",
  "?${a : int64, ...}]",
  "?={a : int64, ...}]",
  "?[{a : int64, ...}$",
  "?[{a : int64, ...}$",
  "10 $ {a : int64, ...}",
  "?10 $ {a : int64, ...}",
  "?(10 $ {a : int64, ...})",
  "$10 $ {a : int64, ...}",
  "=10 $ {a : int64, ...}",
  "$[10 $ {a : int64, ...}]",
  "xxx[10 $ {a : int64, ...}]",
  "?$10 $ {a : int64, ...}]",
  "?=10 $ {a : int64, ...}]",
  "?[10 $ {a : int64, ...}$",
  "?[10 $ {a : int64, ...}$",
  "10 = {a : int64, ...}",
  "?10 = {a : int64, ...}",
  "?(10 = {a : int64, ...})",
  "$10 = {a : int64, ...}",
  "=10 = {a : int64, ...}",
  "$[10 = {a : int64, ...}]",
  "xxx[10 = {a : int64, ...}]",
  "?$10 = {a : int64, ...}]",
  "?=10 = {a : int64, ...}]",
  "?[10 = {a : int64, ...}$",
  "?[10 = {a : int64, ...}$",
  "10 * {a : int64, $}",
  "?10 * {a : int64, $}",
  "?(10 * {a : int64, $})",
  "$10 * {a : int64, $}",
  "=10 * {a : int64, $}",
  "$[10 * {a : int64, $}]",
  "xxx[10 * {a : int64, $}]",
  "?$10 * {a : int64, $}]",
  "?=10 * {a : int64, $}]",
  "?[10 * {a : int64, $}$",
  "?[10 * {a : int64, $}$",
  "{a : int64, $}",
  "?{a : int64, $}",
  "?({a : int64, $})",
  "${a : int64, $}",
  "={a : int64, $}",
  "$[{a : int64, $}]",
  "xxx[{a : int64, $}]",
  "?${a : int64, $}]",
  "?={a : int64, $}]",
  "?[{a : int64, $}$",
  "?[{a : int64, $}$",
  "10 $ {a : int64, $}",
  "?10 $ {a : int64, $}",
  "?(10 $ {a : int64, $})",
  "$10 $ {a : int64, $}",
  "=10 $ {a : int64, $}",
  "$[10 $ {a : int64, $}]",
  "xxx[10 $ {a : int64, $}]",
  "?$10 $ {a : int64, $}]",
  "?=10 $ {a : int64, $}]",
  "?[10 $ {a : int64, $}$",
  "?[10 $ {a : int64, $}$",
  "10 = {a : int64, $}",
  "?10 = {a : int64, $}",
  "?(10 = {a : int64, $})",
  "$10 = {a : int64, $}",
  "=10 = {a : int64, $}",
  "$[10 = {a : int64, $}]",
  "xxx[10 = {a : int64, $}]",
  "?$10 = {a : int64, $}]",
  "?=10 = {a : int64, $}]",
  "?[10 = {a : int64, $}$",
  "?[10 = {a : int64, $}$",
  "$10 * (...)-> Any",
  "=10 * (...)-> Any",
  "$[10 * (...)-> Any]",
  "xxx[10 * (...)-> Any]",
  "?$10 * (...)-> Any]",
  "?=10 * (...)-> Any]",
  "?[10 * (...)-> Any$",
  "?[10 * (...)-> Any$",
  "$(...)-> Any",
  "=(...)-> Any",
  "$[(...)-> Any]",
  "xxx[(...)-> Any]",
  "?$(...)-> Any]",
  "?=(...)-> Any]",
  "?[(...)-> Any$",
  "?[(...)-> Any$",
  "10 $ (...)-> Any",
  "?10 $ (...)-> Any",
  "?(10 $ (...)-> Any)",
  "$10 $ (...)-> Any",
  "=10 $ (...)-> Any",
  "$[10 $ (...)-> Any]",
  "xxx[10 $ (...)-> Any]",
  "?$10 $ (...)-> Any]",
  "?=10 $ (...)-> Any]",
  "?[10 $ (...)-> Any$",
  "?[10 $ (...)-> Any$",
  "10 = (...)-> Any",
  "?10 = (...)-> Any",
  "?(10 = (...)-> Any)",
  "$10 = (...)-> Any",
  "=10 = (...)-> Any",
  "$[10 = (...)-> Any]",
  "xxx[10 = (...)-> Any]",
  "?$10 = (...)-> Any]",
  "?=10 = (...)-> Any]",
  "?[10 = (...)-> Any$",
  "?[10 = (...)-> Any$",
  "$10 * ()->(...)",
  "=10 * ()->(...)",
  "$[10 * ()->(...)]",
  "xxx[10 * ()->(...)]",
  "?$10 * ()->(...)]",
  "?=10 * ()->(...)]",
  "?[10 * ()->(...)$",
  "?[10 * ()->(...)$",
  "$()->(...)",
  "=()->(...)",
  "$[()->(...)]",
  "xxx[()->(...)]",
  "?$()->(...)]",
  "?=()->(...)]",
  "?[()->(...)$",
  "?[()->(...)$",
  "10 $ ()->(...)",
  "?10 $ ()->(...)",
  "?(10 $ ()->(...))",
  "$10 $ ()->(...)",
  "=10 $ ()->(...)",
  "$[10 $ ()->(...)]",
  "xxx[10 $ ()->(...)]",
  "?$10 $ ()->(...)]",
  "?=10 $ ()->(...)]",
  "?[10 $ ()->(...)$",
  "?[10 $ ()->(...)$",
  "10 = ()->(...)",
  "?10 = ()->(...)",
  "?(10 = ()->(...))",
  "$10 = ()->(...)",
  "=10 = ()->(...)",
  "$[10 = ()->(...)]",
  "xxx[10 = ()->(...)]",
  "?$10 = ()->(...)]",
  "?=10 = ()->(...)]",
  "?[10 = ()->(...)$",
  "?[10 = ()->(...)$",
  "$10 * (Any)-> Any",
  "=10 * (Any)-> Any",
  "$[10 * (Any)-> Any]",
  "xxx[10 * (Any)-> Any]",
  "?$10 * (Any)-> Any]",
  "?=10 * (Any)-> Any]",
  "?[10 * (Any)-> Any$",
  "?[10 * (Any)-> Any$",
  "$(Any)-> Any",
  "=(Any)-> Any",
  "$[(Any)-> Any]",
  "xxx[(Any)-> Any]",
  "?$(Any)-> Any]",
  "?=(Any)-> Any]",
  "?[(Any)-> Any$",
  "?[(Any)-> Any$",
  "10 $ (Any)-> Any",
  "?10 $ (Any)-> Any",
  "?(10 $ (Any)-> Any)",
  "$10 $ (Any)-> Any",
  "=10 $ (Any)-> Any",
  "$[10 $ (Any)-> Any]",
  "xxx[10 $ (Any)-> Any]",
  "?$10 $ (Any)-> Any]",
  "?=10 $ (Any)-> Any]",
  "?[10 $ (Any)-> Any$",
  "?[10 $ (Any)-> Any$",
  "10 = (Any)-> Any",
  "?10 = (Any)-> Any",
  "?(10 = (Any)-> Any)",
  "$10 = (Any)-> Any",
  "=10 = (Any)-> Any",
  "$[10 = (Any)-> Any]",
  "xxx[10 = (Any)-> Any]",
  "?$10 = (Any)-> Any]",
  "?=10 = (Any)-> Any]",
  "?[10 = (Any)-> Any$",
  "?[10 = (Any)-> Any$",
  "$10 * ()->(Any)",
  "=10 * ()->(Any)",
  "$[10 * ()->(Any)]",
  "xxx[10 * ()->(Any)]",
  "?$10 * ()->(Any)]",
  "?=10 * ()->(Any)]",
  "?[10 * ()->(Any)$",
  "?[10 * ()->(Any)$",
  "$()->(Any)",
  "=()->(Any)",
  "$[()->(Any)]",
  "xxx[()->(Any)]",
  "?$()->(Any)]",
  "?=()->(Any)]",
  "?[()->(Any)$",
  "?[()->(Any)$",
  "10 $ ()->(Any)",
  "?10 $ ()->(Any)",
  "?(10 $ ()->(Any))",
  "$10 $ ()->(Any)",
  "=10 $ ()->(Any)",
  "$[10 $ ()->(Any)]",
  "xxx[10 $ ()->(Any)]",
  "?$10 $ ()->(Any)]",
  "?=10 $ ()->(Any)]",
  "?[10 $ ()->(Any)$",
  "?[10 $ ()->(Any)$",
  "10 = ()->(Any)",
  "?10 = ()->(Any)",
  "?(10 = ()->(Any))",
  "$10 = ()->(Any)",
  "=10 = ()->(Any)",
  "$[10 = ()->(Any)]",
  "xxx[10 = ()->(Any)]",
  "?$10 = ()->(Any)]",
  "?=10 = ()->(Any)]",
  "?[10 = ()->(Any)$",
  "?[10 = ()->(Any)$",
  "$10 * (int64, Any)-> Any",
  "=10 * (int64, Any)-> Any",
  "$[10 * (int64, Any)-> Any]",
  "xxx[10 * (int64, Any)-> Any]",
  "?$10 * (int64, Any)-> Any]",
  "?=10 * (int64, Any)-> Any]",
  "?[10 * (int64, Any)-> Any$",
  "?[10 * (int64, Any)-> Any$",
  "$(int64, Any)-> Any",
  "=(int64, Any)-> Any",
  "$[(int64, Any)-> Any]",
  "xxx[(int64, Any)-> Any]",
  "?$(int64, Any)-> Any]",
  "?=(int64, Any)-> Any]",
  "?[(int64, Any)-> Any$",
  "?[(int64, Any)-> Any$",
  "10 $ (int64, Any)-> Any",
  "?10 $ (int64, Any)-> Any",
  "?(10 $ (int64, Any)-> Any)",
  "$10 $ (int64, Any)-> Any",
  "=10 $ (int64, Any)-> Any",
  "$[10 $ (int64, Any)-> Any]",
  "xxx[10 $ (int64, Any)-> Any]",
  "?$10 $ (int64, Any)-> Any]",
  "?=10 $ (int64, Any)-> Any]",
  "?[10 $ (int64, Any)-> Any$",
  "?[10 $ (int64, Any)-> Any$",
  "10 = (int64, Any)-> Any",
  "?10 = (int64, Any)-> Any",
  "?(10 = (int64, Any)-> Any)",
  "$10 = (int64, Any)-> Any",
  "=10 = (int64, Any)-> Any",
  "$[10 = (int64, Any)-> Any]",
  "xxx[10 = (int64, Any)-> Any]",
  "?$10 = (int64, Any)-> Any]",
  "?=10 = (int64, Any)-> Any]",
  "?[10 = (int64, Any)-> Any$",
  "?[10 = (int64, Any)-> Any$",
  "$10 * ()->(int64, Any)",
  "=10 * ()->(int64, Any)",
  "$[10 * ()->(int64, Any)]",
  "xxx[10 * ()->(int64, Any)]",
  "?$10 * ()->(int64, Any)]",
  "?=10 * ()->(int64, Any)]",
  "?[10 * ()->(int64, Any)$",
  "?[10 * ()->(int64, Any)$",
  "$()->(int64, Any)",
  "=()->(int64, Any)",
  "$[()->(int64, Any)]",
  "xxx[()->(int64, Any)]",
  "?$()->(int64, Any)]",
  "?=()->(int64, Any)]",
  "?[()->(int64, Any)$",
  "?[()->(int64, Any)$",
  "10 $ ()->(int64, Any)",
  "?10 $ ()->(int64, Any)",
  "?(10 $ ()->(int64, Any))",
  "$10 $ ()->(int64, Any)",
  "=10 $ ()->(int64, Any)",
  "$[10 $ ()->(int64, Any)]",
  "xxx[10 $ ()->(int64, Any)]",
  "?$10 $ ()->(int64, Any)]",
  "?=10 $ ()->(int64, Any)]",
  "?[10 $ ()->(int64, Any)$",
  "?[10 $ ()->(int64, Any)$",
  "10 = ()->(int64, Any)",
  "?10 = ()->(int64, Any)",
  "?(10 = ()->(int64, Any))",
  "$10 = ()->(int64, Any)",
  "=10 = ()->(int64, Any)",
  "$[10 = ()->(int64, Any)]",
  "xxx[10 = ()->(int64, Any)]",
  "?$10 = ()->(int64, Any)]",
  "?=10 = ()->(int64, Any)]",
  "?[10 = ()->(int64, Any)$",
  "?[10 = ()->(int64, Any)$",
  "10 * (int64$ Any)-> Any",
  "?10 * (int64$ Any)-> Any",
  "?(10 * (int64$ Any)-> Any)",
  "$10 * (int64$ Any)-> Any",
  "=10 * (int64$ Any)-> Any",
  "$[10 * (int64$ Any)-> Any]",
  "xxx[10 * (int64$ Any)-> Any]",
  "?$10 * (int64$ Any)-> Any]",
  "?=10 * (int64$ Any)-> Any]",
  "?[10 * (int64$ Any)-> Any$",
  "?[10 * (int64$ Any)-> Any$",
  "(int64$ Any)-> Any",
  "?(int64$ Any)-> Any",
  "?((int64$ Any)-> Any)",
  "$(int64$ Any)-> Any",
  "=(int64$ Any)-> Any",
  "$[(int64$ Any)-> Any]",
  "xxx[(int64$ Any)-> Any]",
  "?$(int64$ Any)-> Any]",
  "?=(int64$ Any)-> Any]",
  "?[(int64$ Any)-> Any$",
  "?[(int64$ Any)-> Any$",
  "10 $ (int64$ Any)-> Any",
  "?10 $ (int64$ Any)-> Any",
  "?(10 $ (int64$ Any)-> Any)",
  "$10 $ (int64$ Any)-> Any",
  "=10 $ (int64$ Any)-> Any",
  "$[10 $ (int64$ Any)-> Any]",
  "xxx[10 $ (int64$ Any)-> Any]",
  "?$10 $ (int64$ Any)-> Any]",
  "?=10 $ (int64$ Any)-> Any]",
  "?[10 $ (int64$ Any)-> Any$",
  "?[10 $ (int64$ Any)-> Any$",
  "10 = (int64$ Any)-> Any",
  "?10 = (int64$ Any)-> Any",
  "?(10 = (int64$ Any)-> Any)",
  "$10 = (int64$ Any)-> Any",
  "=10 = (int64$ Any)-> Any",
  "$[10 = (int64$ Any)-> Any]",
  "xxx[10 = (int64$ Any)-> Any]",
  "?$10 = (int64$ Any)-> Any]",
  "?=10 = (int64$ Any)-> Any]",
  "?[10 = (int64$ Any)-> Any$",
  "?[10 = (int64$ Any)-> Any$",
  "10 * ()->(int64$ Any)",
  "?10 * ()->(int64$ Any)",
  "?(10 * ()->(int64$ Any))",
  "$10 * ()->(int64$ Any)",
  "=10 * ()->(int64$ Any)",
  "$[10 * ()->(int64$ Any)]",
  "xxx[10 * ()->(int64$ Any)]",
  "?$10 * ()->(int64$ Any)]",
  "?=10 * ()->(int64$ Any)]",
  "?[10 * ()->(int64$ Any)$",
  "?[10 * ()->(int64$ Any)$",
  "()->(int64$ Any)",
  "?()->(int64$ Any)",
  "?(()->(int64$ Any))",
  "$()->(int64$ Any)",
  "=()->(int64$ Any)",
  "$[()->(int64$ Any)]",
  "xxx[()->(int64$ Any)]",
  "?$()->(int64$ Any)]",
  "?=()->(int64$ Any)]",
  "?[()->(int64$ Any)$",
  "?[()->(int64$ Any)$",
  "10 $ ()->(int64$ Any)",
  "?10 $ ()->(int64$ Any)",
  "?(10 $ ()->(int64$ Any))",
  "$10 $ ()->(int64$ Any)",
  "=10 $ ()->(int64$ Any)",
  "$[10 $ ()->(int64$ Any)]",
  "xxx[10 $ ()->(int64$ Any)]",
  "?$10 $ ()->(int64$ Any)]",
  "?=10 $ ()->(int64$ Any)]",
  "?[10 $ ()->(int64$ Any)$",
  "?[10 $ ()->(int64$ Any)$",
  "10 = ()->(int64$ Any)",
  "?10 = ()->(int64$ Any)",
  "?(10 = ()->(int64$ Any))",
  "$10 = ()->(int64$ Any)",
  "=10 = ()->(int64$ Any)",
  "$[10 = ()->(int64$ Any)]",
  "xxx[10 = ()->(int64$ Any)]",
  "?$10 = ()->(int64$ Any)]",
  "?=10 = ()->(int64$ Any)]",
  "?[10 = ()->(int64$ Any)$",
  "?[10 = ()->(int64$ Any)$",
  "10 * (int64= Any)-> Any",
  "?10 * (int64= Any)-> Any",
  "?(10 * (int64= Any)-> Any)",
  "$10 * (int64= Any)-> Any",
  "=10 * (int64= Any)-> Any",
  "$[10 * (int64= Any)-> Any]",
  "xxx[10 * (int64= Any)-> Any]",
  "?$10 * (int64= Any)-> Any]",
  "?=10 * (int64= Any)-> Any]",
  "?[10 * (int64= Any)-> Any$",
  "?[10 * (int64= Any)-> Any$",
  "(int64= Any)-> Any",
  "?(int64= Any)-> Any",
  "?((int64= Any)-> Any)",
  "$(int64= Any)-> Any",
  "=(int64= Any)-> Any",
  "$[(int64= Any)-> Any]",
  "xxx[(int64= Any)-> Any]",
  "?$(int64= Any)-> Any]",
  "?=(int64= Any)-> Any]",
  "?[(int64= Any)-> Any$",
  "?[(int64= Any)-> Any$",
  "10 $ (int64= Any)-> Any",
  "?10 $ (int64= Any)-> Any",
  "?(10 $ (int64= Any)-> Any)",
  "$10 $ (int64= Any)-> Any",
  "=10 $ (int64= Any)-> Any",
  "$[10 $ (int64= Any)-> Any]",
  "xxx[10 $ (int64= Any)-> Any]",
  "?$10 $ (int64= Any)-> Any]",
  "?=10 $ (int64= Any)-> Any]",
  "?[10 $ (int64= Any)-> Any$",
  "?[10 $ (int64= Any)-> Any$",
  "10 = (int64= Any)-> Any",
  "?10 = (int64= Any)-> Any",
  "?(10 = (int64= Any)-> Any)",
  "$10 = (int64= Any)-> Any",
  "=10 = (int64= Any)-> Any",
  "$[10 = (int64= Any)-> Any]",
  "xxx[10 = (int64= Any)-> Any]",
  "?$10 = (int64= Any)-> Any]",
  "?=10 = (int64= Any)-> Any]",
  "?[10 = (int64= Any)-> Any$",
  "?[10 = (int64= Any)-> Any$",
  "10 * ()->(int64= Any)",
  "?10 * ()->(int64= Any)",
  "?(10 * ()->(int64= Any))",
  "$10 * ()->(int64= Any)",
  "=10 * ()->(int64= Any)",
  "$[10 * ()->(int64= Any)]",
  "xxx[10 * ()->(int64= Any)]",
  "?$10 * ()->(int64= Any)]",
  "?=10 * ()->(int64= Any)]",
  "?[10 * ()->(int64= Any)$",
  "?[10 * ()->(int64= Any)$",
  "()->(int64= Any)",
  "?()->(int64= Any)",
  "?(()->(int64= Any))",
  "$()->(int64= Any)",
  "=()->(int64= Any)",
  "$[()->(int64= Any)]",
  "xxx[()->(int64= Any)]",
  "?$()->(int64= Any)]",
  "?=()->(int64= Any)]",
  "?[()->(int64= Any)$",
  "?[()->(int64= Any)$",
  "10 $ ()->(int64= Any)",
  "?10 $ ()->(int64= Any)",
  "?(10 $ ()->(int64= Any))",
  "$10 $ ()->(int64= Any)",
  "=10 $ ()->(int64= Any)",
  "$[10 $ ()->(int64= Any)]",
  "xxx[10 $ ()->(int64= Any)]",
  "?$10 $ ()->(int64= Any)]",
  "?=10 $ ()->(int64= Any)]",
  "?[10 $ ()->(int64= Any)$",
  "?[10 $ ()->(int64= Any)$",
  "10 = ()->(int64= Any)",
  "?10 = ()->(int64= Any)",
  "?(10 = ()->(int64= Any))",
  "$10 = ()->(int64= Any)",
  "=10 = ()->(int64= Any)",
  "$[10 = ()->(int64= Any)]",
  "xxx[10 = ()->(int64= Any)]",
  "?$10 = ()->(int64= Any)]",
  "?=10 = ()->(int64= Any)]",
  "?[10 = ()->(int64= Any)$",
  "?[10 = ()->(int64= Any)$",
  "$10 * (int64, int64, Any)-> Any",
  "=10 * (int64, int64, Any)-> Any",
  "$[10 * (int64, int64, Any)-> Any]",
  "xxx[10 * (int64, int64, Any)-> Any]",
  "?$10 * (int64, int64, Any)-> Any]",
  "?=10 * (int64, int64, Any)-> Any]",
  "?[10 * (int64, int64, Any)-> Any$",
  "?[10 * (int64, int64, Any)-> Any$",
  "$(int64, int64, Any)-> Any",
  "=(int64, int64, Any)-> Any",
  "$[(int64, int64, Any)-> Any]",
  "xxx[(int64, int64, Any)-> Any]",
  "?$(int64, int64, Any)-> Any]",
  "?=(int64, int64, Any)-> Any]",
  "?[(int64, int64, Any)-> Any$",
  "?[(int64, int64, Any)-> Any$",
  "10 $ (int64, int64, Any)-> Any",
  "?10 $ (int64, int64, Any)-> Any",
  "?(10 $ (int64, int64, Any)-> Any)",
  "$10 $ (int64, int64, Any)-> Any",
  "=10 $ (int64, int64, Any)-> Any",
  "$[10 $ (int64, int64, Any)-> Any]",
  "xxx[10 $ (int64, int64, Any)-> Any]",
  "?$10 $ (int64, int64, Any)-> Any]",
  "?=10 $ (int64, int64, Any)-> Any]",
  "?[10 $ (int64, int64, Any)-> Any$",
  "?[10 $ (int64, int64, Any)-> Any$",
  "10 = (int64, int64, Any)-> Any",
  "?10 = (int64, int64, Any)-> Any",
  "?(10 = (int64, int64, Any)-> Any)",
  "$10 = (int64, int64, Any)-> Any",
  "=10 = (int64, int64, Any)-> Any",
  "$[10 = (int64, int64, Any)-> Any]",
  "xxx[10 = (int64, int64, Any)-> Any]",
  "?$10 = (int64, int64, Any)-> Any]",
  "?=10 = (int64, int64, Any)-> Any]",
  "?[10 = (int64, int64, Any)-> Any$",
  "?[10 = (int64, int64, Any)-> Any$",
  "$10 * ()->(int64, int64, Any)",
  "=10 * ()->(int64, int64, Any)",
  "$[10 * ()->(int64, int64, Any)]",
  "xxx[10 * ()->(int64, int64, Any)]",
  "?$10 * ()->(int64, int64, Any)]",
  "?=10 * ()->(int64, int64, Any)]",
  "?[10 * ()->(int64, int64, Any)$",
  "?[10 * ()->(int64, int64, Any)$",
  "$()->(int64, int64, Any)",
  "=()->(int64, int64, Any)",
  "$[()->(int64, int64, Any)]",
  "xxx[()->(int64, int64, Any)]",
  "?$()->(int64, int64, Any)]",
  "?=()->(int64, int64, Any)]",
  "?[()->(int64, int64, Any)$",
  "?[()->(int64, int64, Any)$",
  "10 $ ()->(int64, int64, Any)",
  "?10 $ ()->(int64, int64, Any)",
  "?(10 $ ()->(int64, int64, Any))",
  "$10 $ ()->(int64, int64, Any)",
  "=10 $ ()->(int64, int64, Any)",
  "$[10 $ ()->(int64, int64, Any)]",
  "xxx[10 $ ()->(int64, int64, Any)]",
  "?$10 $ ()->(int64, int64, Any)]",
  "?=10 $ ()->(int64, int64, Any)]",
  "?[10 $ ()->(int64, int64, Any)$",
  "?[10 $ ()->(int64, int64, Any)$",
  "10 = ()->(int64, int64, Any)",
  "?10 = ()->(int64, int64, Any)",
  "?(10 = ()->(int64, int64, Any))",
  "$10 = ()->(int64, int64, Any)",
  "=10 = ()->(int64, int64, Any)",
  "$[10 = ()->(int64, int64, Any)]",
  "xxx[10 = ()->(int64, int64, Any)]",
  "?$10 = ()->(int64, int64, Any)]",
  "?=10 = ()->(int64, int64, Any)]",
  "?[10 = ()->(int64, int64, Any)$",
  "?[10 = ()->(int64, int64, Any)$",
  "10 * (int64, int64$ Any)-> Any",
  "?10 * (int64, int64$ Any)-> Any",
  "?(10 * (int64, int64$ Any)-> Any)",
  "$10 * (int64, int64$ Any)-> Any",
  "=10 * (int64, int64$ Any)-> Any",
  "$[10 * (int64, int64$ Any)-> Any]",
  "xxx[10 * (int64, int64$ Any)-> Any]",
  "?$10 * (int64, int64$ Any)-> Any]",
  "?=10 * (int64, int64$ Any)-> Any]",
  "?[10 * (int64, int64$ Any)-> Any$",
  "?[10 * (int64, int64$ Any)-> Any$",
  "(int64, int64$ Any)-> Any",
  "?(int64, int64$ Any)-> Any",
  "?((int64, int64$ Any)-> Any)",
  "$(int64, int64$ Any)-> Any",
  "=(int64, int64$ Any)-> Any",
  "$[(int64, int64$ Any)-> Any]",
  "xxx[(int64, int64$ Any)-> Any]",
  "?$(int64, int64$ Any)-> Any]",
  "?=(int64, int64$ Any)-> Any]",
  "?[(int64, int64$ Any)-> Any$",
  "?[(int64, int64$ Any)-> Any$",
  "10 $ (int64, int64$ Any)-> Any",
  "?10 $ (int64, int64$ Any)-> Any",
  "?(10 $ (int64, int64$ Any)-> Any)",
  "$10 $ (int64, int64$ Any)-> Any",
  "=10 $ (int64, int64$ Any)-> Any",
  "$[10 $ (int64, int64$ Any)-> Any]",
  "xxx[10 $ (int64, int64$ Any)-> Any]",
  "?$10 $ (int64, int64$ Any)-> Any]",
  "?=10 $ (int64, int64$ Any)-> Any]",
  "?[10 $ (int64, int64$ Any)-> Any$",
  "?[10 $ (int64, int64$ Any)-> Any$",
  "10 = (int64, int64$ Any)-> Any",
  "?10 = (int64, int64$ Any)-> Any",
  "?(10 = (int64, int64$ Any)-> Any)",
  "$10 = (int64, int64$ Any)-> Any",
  "=10 = (int64, int64$ Any)-> Any",
  "$[10 = (int64, int64$ Any)-> Any]",
  "xxx[10 = (int64, int64$ Any)-> Any]",
  "?$10 = (int64, int64$ Any)-> Any]",
  "?=10 = (int64, int64$ Any)-> Any]",
  "?[10 = (int64, int64$ Any)-> Any$",
  "?[10 = (int64, int64$ Any)-> Any$",
  "10 * ()->(int64, int64$ Any)",
  "?10 * ()->(int64, int64$ Any)",
  "?(10 * ()->(int64, int64$ Any))",
  "$10 * ()->(int64, int64$ Any)",
  "=10 * ()->(int64, int64$ Any)",
  "$[10 * ()->(int64, int64$ Any)]",
  "xxx[10 * ()->(int64, int64$ Any)]",
  "?$10 * ()->(int64, int64$ Any)]",
  "?=10 * ()->(int64, int64$ Any)]",
  "?[10 * ()->(int64, int64$ Any)$",
  "?[10 * ()->(int64, int64$ Any)$",
  "()->(int64, int64$ Any)",
  "?()->(int64, int64$ Any)",
  "?(()->(int64, int64$ Any))",
  "$()->(int64, int64$ Any)",
  "=()->(int64, int64$ Any)",
  "$[()->(int64, int64$ Any)]",
  "xxx[()->(int64, int64$ Any)]",
  "?$()->(int64, int64$ Any)]",
  "?=()->(int64, int64$ Any)]",
  "?[()->(int64, int64$ Any)$",
  "?[()->(int64, int64$ Any)$",
  "10 $ ()->(int64, int64$ Any)",
  "?10 $ ()->(int64, int64$ Any)",
  "?(10 $ ()->(int64, int64$ Any))",
  "$10 $ ()->(int64, int64$ Any)",
  "=10 $ ()->(int64, int64$ Any)",
  "$[10 $ ()->(int64, int64$ Any)]",
  "xxx[10 $ ()->(int64, int64$ Any)]",
  "?$10 $ ()->(int64, int64$ Any)]",
  "?=10 $ ()->(int64, int64$ Any)]",
  "?[10 $ ()->(int64, int64$ Any)$",
  "?[10 $ ()->(int64, int64$ Any)$",
  "10 = ()->(int64, int64$ Any)",
  "?10 = ()->(int64, int64$ Any)",
  "?(10 = ()->(int64, int64$ Any))",
  "$10 = ()->(int64, int64$ Any)",
  "=10 = ()->(int64, int64$ Any)",
  "$[10 = ()->(int64, int64$ Any)]",
  "xxx[10 = ()->(int64, int64$ Any)]",
  "?$10 = ()->(int64, int64$ Any)]",
  "?=10 = ()->(int64, int64$ Any)]",
  "?[10 = ()->(int64, int64$ Any)$",
  "?[10 = ()->(int64, int64$ Any)$",
  "10 * (int64, int64= Any)-> Any",
  "?10 * (int64, int64= Any)-> Any",
  "?(10 * (int64, int64= Any)-> Any)",
  "$10 * (int64, int64= Any)-> Any",
  "=10 * (int64, int64= Any)-> Any",
  "$[10 * (int64, int64= Any)-> Any]",
  "xxx[10 * (int64, int64= Any)-> Any]",
  "?$10 * (int64, int64= Any)-> Any]",
  "?=10 * (int64, int64= Any)-> Any]",
  "?[10 * (int64, int64= Any)-> Any$",
  "?[10 * (int64, int64= Any)-> Any$",
  "(int64, int64= Any)-> Any",
  "?(int64, int64= Any)-> Any",
  "?((int64, int64= Any)-> Any)",
  "$(int64, int64= Any)-> Any",
  "=(int64, int64= Any)-> Any",
  "$[(int64, int64= Any)-> Any]",
  "xxx[(int64, int64= Any)-> Any]",
  "?$(int64, int64= Any)-> Any]",
  "?=(int64, int64= Any)-> Any]",
  "?[(int64, int64= Any)-> Any$",
  "?[(int64, int64= Any)-> Any$",
  "10 $ (int64, int64= Any)-> Any",
  "?10 $ (int64, int64= Any)-> Any",
  "?(10 $ (int64, int64= Any)-> Any)",
  "$10 $ (int64, int64= Any)-> Any",
  "=10 $ (int64, int64= Any)-> Any",
  "$[10 $ (int64, int64= Any)-> Any]",
  "xxx[10 $ (int64, int64= Any)-> Any]",
  "?$10 $ (int64, int64= Any)-> Any]",
  "?=10 $ (int64, int64= Any)-> Any]",
  "?[10 $ (int64, int64= Any)-> Any$",
  "?[10 $ (int64, int64= Any)-> Any$",
  "10 = (int64, int64= Any)-> Any",
  "?10 = (int64, int64= Any)-> Any",
  "?(10 = (int64, int64= Any)-> Any)",
  "$10 = (int64, int64= Any)-> Any",
  "=10 = (int64, int64= Any)-> Any",
  "$[10 = (int64, int64= Any)-> Any]",
  "xxx[10 = (int64, int64= Any)-> Any]",
  "?$10 = (int64, int64= Any)-> Any]",
  "?=10 = (int64, int64= Any)-> Any]",
  "?[10 = (int64, int64= Any)-> Any$",
  "?[10 = (int64, int64= Any)-> Any$",
  "10 * ()->(int64, int64= Any)",
  "?10 * ()->(int64, int64= Any)",
  "?(10 * ()->(int64, int64= Any))",
  "$10 * ()->(int64, int64= Any)",
  "=10 * ()->(int64, int64= Any)",
  "$[10 * ()->(int64, int64= Any)]",
  "xxx[10 * ()->(int64, int64= Any)]",
  "?$10 * ()->(int64, int64= Any)]",
  "?=10 * ()->(int64, int64= Any)]",
  "?[10 * ()->(int64, int64= Any)$",
  "?[10 * ()->(int64, int64= Any)$",
  "()->(int64, int64= Any)",
  "?()->(int64, int64= Any)",
  "?(()->(int64, int64= Any))",
  "$()->(int64, int64= Any)",
  "=()->(int64, int64= Any)",
  "$[()->(int64, int64= Any)]",
  "xxx[()->(int64, int64= Any)]",
  "?$()->(int64, int64= Any)]",
  "?=()->(int64, int64= Any)]",
  "?[()->(int64, int64= Any)$",
  "?[()->(int64, int64= Any)$",
  "10 $ ()->(int64, int64= Any)",
  "?10 $ ()->(int64, int64= Any)",
  "?(10 $ ()->(int64, int64= Any))",
  "$10 $ ()->(int64, int64= Any)",
  "=10 $ ()->(int64, int64= Any)",
  "$[10 $ ()->(int64, int64= Any)]",
  "xxx[10 $ ()->(int64, int64= Any)]",
  "?$10 $ ()->(int64, int64= Any)]",
  "?=10 $ ()->(int64, int64= Any)]",
  "?[10 $ ()->(int64, int64= Any)$",
  "?[10 $ ()->(int64, int64= Any)$",
  "10 = ()->(int64, int64= Any)",
  "?10 = ()->(int64, int64= Any)",
  "?(10 = ()->(int64, int64= Any))",
  "$10 = ()->(int64, int64= Any)",
  "=10 = ()->(int64, int64= Any)",
  "$[10 = ()->(int64, int64= Any)]",
  "xxx[10 = ()->(int64, int64= Any)]",
  "?$10 = ()->(int64, int64= Any)]",
  "?=10 = ()->(int64, int64= Any)]",
  "?[10 = ()->(int64, int64= Any)$",
  "?[10 = ()->(int64, int64= Any)$",
  "10 * (xxx)-> Any",
  "?10 * (xxx)-> Any",
  "?(10 * (xxx)-> Any)",
  "$10 * (xxx)-> Any",
  "=10 * (xxx)-> Any",
  "$[10 * (xxx)-> Any]",
  "xxx[10 * (xxx)-> Any]",
  "?$10 * (xxx)-> Any]",
  "?=10 * (xxx)-> Any]",
  "?[10 * (xxx)-> Any$",
  "?[10 * (xxx)-> Any$",
  "(xxx)-> Any",
  "?(xxx)-> Any",
  "?((xxx)-> Any)",
  "$(xxx)-> Any",
  "=(xxx)-> Any",
  "$[(xxx)-> Any]",
  "xxx[(xxx)-> Any]",
  "?$(xxx)-> Any]",
  "?=(xxx)-> Any]",
  "?[(xxx)-> Any$",
  "?[(xxx)-> Any$",
  "10 $ (xxx)-> Any",
  "?10 $ (xxx)-> Any",
  "?(10 $ (xxx)-> Any)",
  "$10 $ (xxx)-> Any",
  "=10 $ (xxx)-> Any",
  "$[10 $ (xxx)-> Any]",
  "xxx[10 $ (xxx)-> Any]",
  "?$10 $ (xxx)-> Any]",
  "?=10 $ (xxx)-> Any]",
  "?[10 $ (xxx)-> Any$",
  "?[10 $ (xxx)-> Any$",
  "10 = (xxx)-> Any",
  "?10 = (xxx)-> Any",
  "?(10 = (xxx)-> Any)",
  "$10 = (xxx)-> Any",
  "=10 = (xxx)-> Any",
  "$[10 = (xxx)-> Any]",
  "xxx[10 = (xxx)-> Any]",
  "?$10 = (xxx)-> Any]",
  "?=10 = (xxx)-> Any]",
  "?[10 = (xxx)-> Any$",
  "?[10 = (xxx)-> Any$",
  "10 * ()->(xxx)",
  "?10 * ()->(xxx)",
  "?(10 * ()->(xxx))",
  "$10 * ()->(xxx)",
  "=10 * ()->(xxx)",
  "$[10 * ()->(xxx)]",
  "xxx[10 * ()->(xxx)]",
  "?$10 * ()->(xxx)]",
  "?=10 * ()->(xxx)]",
  "?[10 * ()->(xxx)$",
  "?[10 * ()->(xxx)$",
  "()->(xxx)",
  "?()->(xxx)",
  "?(()->(xxx))",
  "$()->(xxx)",
  "=()->(xxx)",
  "$[()->(xxx)]",
  "xxx[()->(xxx)]",
  "?$()->(xxx)]",
  "?=()->(xxx)]",
  "?[()->(xxx)$",
  "?[()->(xxx)$",
  "10 $ ()->(xxx)",
  "?10 $ ()->(xxx)",
  "?(10 $ ()->(xxx))",
  "$10 $ ()->(xxx)",
  "=10 $ ()->(xxx)",
  "$[10 $ ()->(xxx)]",
  "xxx[10 $ ()->(xxx)]",
  "?$10 $ ()->(xxx)]",
  "?=10 $ ()->(xxx)]",
  "?[10 $ ()->(xxx)$",
  "?[10 $ ()->(xxx)$",
  "10 = ()->(xxx)",
  "?10 = ()->(xxx)",
  "?(10 = ()->(xxx))",
  "$10 = ()->(xxx)",
  "=10 = ()->(xxx)",
  "$[10 = ()->(xxx)]",
  "xxx[10 = ()->(xxx)]",
  "?$10 = ()->(xxx)]",
  "?=10 = ()->(xxx)]",
  "?[10 = ()->(xxx)$",
  "?[10 = ()->(xxx)$",
  "10 * (int64, xxx)-> Any",
  "?10 * (int64, xxx)-> Any",
  "?(10 * (int64, xxx)-> Any)",
  "$10 * (int64, xxx)-> Any",
  "=10 * (int64, xxx)-> Any",
  "$[10 * (int64, xxx)-> Any]",
  "xxx[10 * (int64, xxx)-> Any]",
  "?$10 * (int64, xxx)-> Any]",
  "?=10 * (int64, xxx)-> Any]",
  "?[10 * (int64, xxx)-> Any$",
  "?[10 * (int64, xxx)-> Any$",
  "(int64, xxx)-> Any",
  "?(int64, xxx)-> Any",
  "?((int64, xxx)-> Any)",
  "$(int64, xxx)-> Any",
  "=(int64, xxx)-> Any",
  "$[(int64, xxx)-> Any]",
  "xxx[(int64, xxx)-> Any]",
  "?$(int64, xxx)-> Any]",
  "?=(int64, xxx)-> Any]",
  "?[(int64, xxx)-> Any$",
  "?[(int64, xxx)-> Any$",
  "10 $ (int64, xxx)-> Any",
  "?10 $ (int64, xxx)-> Any",
  "?(10 $ (int64, xxx)-> Any)",
  "$10 $ (int64, xxx)-> Any",
  "=10 $ (int64, xxx)-> Any",
  "$[10 $ (int64, xxx)-> Any]",
  "xxx[10 $ (int64, xxx)-> Any]",
  "?$10 $ (int64, xxx)-> Any]",
  "?=10 $ (int64, xxx)-> Any]",
  "?[10 $ (int64, xxx)-> Any$",
  "?[10 $ (int64, xxx)-> Any$",
  "10 = (int64, xxx)-> Any",
  "?10 = (int64, xxx)-> Any",
  "?(10 = (int64, xxx)-> Any)",
  "$10 = (int64, xxx)-> Any",
  "=10 = (int64, xxx)-> Any",
  "$[10 = (int64, xxx)-> Any]",
  "xxx[10 = (int64, xxx)-> Any]",
  "?$10 = (int64, xxx)-> Any]",
  "?=10 = (int64, xxx)-> Any]",
  "?[10 = (int64, xxx)-> Any$",
  "?[10 = (int64, xxx)-> Any$",
  "10 * ()->(int64, xxx)",
  "?10 * ()->(int64, xxx)",
  "?(10 * ()->(int64, xxx))",
  "$10 * ()->(int64, xxx)",
  "=10 * ()->(int64, xxx)",
  "$[10 * ()->(int64, xxx)]",
  "xxx[10 * ()->(int64, xxx)]",
  "?$10 * ()->(int64, xxx)]",
  "?=10 * ()->(int64, xxx)]",
  "?[10 * ()->(int64, xxx)$",
  "?[10 * ()->(int64, xxx)$",
  "()->(int64, xxx)",
  "?()->(int64, xxx)",
  "?(()->(int64, xxx))",
  "$()->(int64, xxx)",
  "=()->(int64, xxx)",
  "$[()->(int64, xxx)]",
  "xxx[()->(int64, xxx)]",
  "?$()->(int64, xxx)]",
  "?=()->(int64, xxx)]",
  "?[()->(int64, xxx)$",
  "?[()->(int64, xxx)$",
  "10 $ ()->(int64, xxx)",
  "?10 $ ()->(int64, xxx)",
  "?(10 $ ()->(int64, xxx))",
  "$10 $ ()->(int64, xxx)",
  "=10 $ ()->(int64, xxx)",
  "$[10 $ ()->(int64, xxx)]",
  "xxx[10 $ ()->(int64, xxx)]",
  "?$10 $ ()->(int64, xxx)]",
  "?=10 $ ()->(int64, xxx)]",
  "?[10 $ ()->(int64, xxx)$",
  "?[10 $ ()->(int64, xxx)$",
  "10 = ()->(int64, xxx)",
  "?10 = ()->(int64, xxx)",
  "?(10 = ()->(int64, xxx))",
  "$10 = ()->(int64, xxx)",
  "=10 = ()->(int64, xxx)",
  "$[10 = ()->(int64, xxx)]",
  "xxx[10 = ()->(int64, xxx)]",
  "?$10 = ()->(int64, xxx)]",
  "?=10 = ()->(int64, xxx)]",
  "?[10 = ()->(int64, xxx)$",
  "?[10 = ()->(int64, xxx)$",
  "10 * (int64$ xxx)-> Any",
  "?10 * (int64$ xxx)-> Any",
  "?(10 * (int64$ xxx)-> Any)",
  "$10 * (int64$ xxx)-> Any",
  "=10 * (int64$ xxx)-> Any",
  "$[10 * (int64$ xxx)-> Any]",
  "xxx[10 * (int64$ xxx)-> Any]",
  "?$10 * (int64$ xxx)-> Any]",
  "?=10 * (int64$ xxx)-> Any]",
  "?[10 * (int64$ xxx)-> Any$",
  "?[10 * (int64$ xxx)-> Any$",
  "(int64$ xxx)-> Any",
  "?(int64$ xxx)-> Any",
  "?((int64$ xxx)-> Any)",
  "$(int64$ xxx)-> Any",
  "=(int64$ xxx)-> Any",
  "$[(int64$ xxx)-> Any]",
  "xxx[(int64$ xxx)-> Any]",
  "?$(int64$ xxx)-> Any]",
  "?=(int64$ xxx)-> Any]",
  "?[(int64$ xxx)-> Any$",
  "?[(int64$ xxx)-> Any$",
  "10 $ (int64$ xxx)-> Any",
  "?10 $ (int64$ xxx)-> Any",
  "?(10 $ (int64$ xxx)-> Any)",
  "$10 $ (int64$ xxx)-> Any",
  "=10 $ (int64$ xxx)-> Any",
  "$[10 $ (int64$ xxx)-> Any]",
  "xxx[10 $ (int64$ xxx)-> Any]",
  "?$10 $ (int64$ xxx)-> Any]",
  "?=10 $ (int64$ xxx)-> Any]",
  "?[10 $ (int64$ xxx)-> Any$",
  "?[10 $ (int64$ xxx)-> Any$",
  "10 = (int64$ xxx)-> Any",
  "?10 = (int64$ xxx)-> Any",
  "?(10 = (int64$ xxx)-> Any)",
  "$10 = (int64$ xxx)-> Any",
  "=10 = (int64$ xxx)-> Any",
  "$[10 = (int64$ xxx)-> Any]",
  "xxx[10 = (int64$ xxx)-> Any]",
  "?$10 = (int64$ xxx)-> Any]",
  "?=10 = (int64$ xxx)-> Any]",
  "?[10 = (int64$ xxx)-> Any$",
  "?[10 = (int64$ xxx)-> Any$",
  "10 * ()->(int64$ xxx)",
  "?10 * ()->(int64$ xxx)",
  "?(10 * ()->(int64$ xxx))",
  "$10 * ()->(int64$ xxx)",
  "=10 * ()->(int64$ xxx)",
  "$[10 * ()->(int64$ xxx)]",
  "xxx[10 * ()->(int64$ xxx)]",
  "?$10 * ()->(int64$ xxx)]",
  "?=10 * ()->(int64$ xxx)]",
  "?[10 * ()->(int64$ xxx)$",
  "?[10 * ()->(int64$ xxx)$",
  "()->(int64$ xxx)",
  "?()->(int64$ xxx)",
  "?(()->(int64$ xxx))",
  "$()->(int64$ xxx)",
  "=()->(int64$ xxx)",
  "$[()->(int64$ xxx)]",
  "xxx[()->(int64$ xxx)]",
  "?$()->(int64$ xxx)]",
  "?=()->(int64$ xxx)]",
  "?[()->(int64$ xxx)$",
  "?[()->(int64$ xxx)$",
  "10 $ ()->(int64$ xxx)",
  "?10 $ ()->(int64$ xxx)",
  "?(10 $ ()->(int64$ xxx))",
  "$10 $ ()->(int64$ xxx)",
  "=10 $ ()->(int64$ xxx)",
  "$[10 $ ()->(int64$ xxx)]",
  "xxx[10 $ ()->(int64$ xxx)]",
  "?$10 $ ()->(int64$ xxx)]",
  "?=10 $ ()->(int64$ xxx)]",
  "?[10 $ ()->(int64$ xxx)$",
  "?[10 $ ()->(int64$ xxx)$",
  "10 = ()->(int64$ xxx)",
  "?10 = ()->(int64$ xxx)",
  "?(10 = ()->(int64$ xxx))",
  "$10 = ()->(int64$ xxx)",
  "=10 = ()->(int64$ xxx)",
  "$[10 = ()->(int64$ xxx)]",
  "xxx[10 = ()->(int64$ xxx)]",
  "?$10 = ()->(int64$ xxx)]",
  "?=10 = ()->(int64$ xxx)]",
  "?[10 = ()->(int64$ xxx)$",
  "?[10 = ()->(int64$ xxx)$",
  "10 * (int64= xxx)-> Any",
  "?10 * (int64= xxx)-> Any",
  "?(10 * (int64= xxx)-> Any)",
  "$10 * (int64= xxx)-> Any",
  "=10 * (int64= xxx)-> Any",
  "$[10 * (int64= xxx)-> Any]",
  "xxx[10 * (int64= xxx)-> Any]",
  "?$10 * (int64= xxx)-> Any]",
  "?=10 * (int64= xxx)-> Any]",
  "?[10 * (int64= xxx)-> Any$",
  "?[10 * (int64= xxx)-> Any$",
  "(int64= xxx)-> Any",
  "?(int64= xxx)-> Any",
  "?((int64= xxx)-> Any)",
  "$(int64= xxx)-> Any",
  "=(int64= xxx)-> Any",
  "$[(int64= xxx)-> Any]",
  "xxx[(int64= xxx)-> Any]",
  "?$(int64= xxx)-> Any]",
  "?=(int64= xxx)-> Any]",
  "?[(int64= xxx)-> Any$",
  "?[(int64= xxx)-> Any$",
  "10 $ (int64= xxx)-> Any",
  "?10 $ (int64= xxx)-> Any",
  "?(10 $ (int64= xxx)-> Any)",
  "$10 $ (int64= xxx)-> Any",
  "=10 $ (int64= xxx)-> Any",
  "$[10 $ (int64= xxx)-> Any]",
  "xxx[10 $ (int64= xxx)-> Any]",
  "?$10 $ (int64= xxx)-> Any]",
  "?=10 $ (int64= xxx)-> Any]",
  "?[10 $ (int64= xxx)-> Any$",
  "?[10 $ (int64= xxx)-> Any$",
  "10 = (int64= xxx)-> Any",
  "?10 = (int64= xxx)-> Any",
  "?(10 = (int64= xxx)-> Any)",
  "$10 = (int64= xxx)-> Any",
  "=10 = (int64= xxx)-> Any",
  "$[10 = (int64= xxx)-> Any]",
  "xxx[10 = (int64= xxx)-> Any]",
  "?$10 = (int64= xxx)-> Any]",
  "?=10 = (int64= xxx)-> Any]",
  "?[10 = (int64= xxx)-> Any$",
  "?[10 = (int64= xxx)-> Any$",
  "10 * ()->(int64= xxx)",
  "?10 * ()->(int64= xxx)",
  "?(10 * ()->(int64= xxx))",
  "$10 * ()->(int64= xxx)",
  "=10 * ()->(int64= xxx)",
  "$[10 * ()->(int64= xxx)]",
  "xxx[10 * ()->(int64= xxx)]",
  "?$10 * ()->(int64= xxx)]",
  "?=10 * ()->(int64= xxx)]",
  "?[10 * ()->(int64= xxx)$",
  "?[10 * ()->(int64= xxx)$",
  "()->(int64= xxx)",
  "?()->(int64= xxx)",
  "?(()->(int64= xxx))",
  "$()->(int64= xxx)",
  "=()->(int64= xxx)",
  "$[()->(int64= xxx)]",
  "xxx[()->(int64= xxx)]",
  "?$()->(int64= xxx)]",
  "?=()->(int64= xxx)]",
  "?[()->(int64= xxx)$",
  "?[()->(int64= xxx)$",
  "10 $ ()->(int64= xxx)",
  "?10 $ ()->(int64= xxx)",
  "?(10 $ ()->(int64= xxx))",
  "$10 $ ()->(int64= xxx)",
  "=10 $ ()->(int64= xxx)",
  "$[10 $ ()->(int64= xxx)]",
  "xxx[10 $ ()->(int64= xxx)]",
  "?$10 $ ()->(int64= xxx)]",
  "?=10 $ ()->(int64= xxx)]",
  "?[10 $ ()->(int64= xxx)$",
  "?[10 $ ()->(int64= xxx)$",
  "10 = ()->(int64= xxx)",
  "?10 = ()->(int64= xxx)",
  "?(10 = ()->(int64= xxx))",
  "$10 = ()->(int64= xxx)",
  "=10 = ()->(int64= xxx)",
  "$[10 = ()->(int64= xxx)]",
  "xxx[10 = ()->(int64= xxx)]",
  "?$10 = ()->(int64= xxx)]",
  "?=10 = ()->(int64= xxx)]",
  "?[10 = ()->(int64= xxx)$",
  "?[10 = ()->(int64= xxx)$",
  "10 * (int64, int64, xxx)-> Any",
  "?10 * (int64, int64, xxx)-> Any",
  "?(10 * (int64, int64, xxx)-> Any)",
  "$10 * (int64, int64, xxx)-> Any",
  "=10 * (int64, int64, xxx)-> Any",
  "$[10 * (int64, int64, xxx)-> Any]",
  "xxx[10 * (int64, int64, xxx)-> Any]",
  "?$10 * (int64, int64, xxx)-> Any]",
  "?=10 * (int64, int64, xxx)-> Any]",
  "?[10 * (int64, int64, xxx)-> Any$",
  "?[10 * (int64, int64, xxx)-> Any$",
  "(int64, int64, xxx)-> Any",
  "?(int64, int64, xxx)-> Any",
  "?((int64, int64, xxx)-> Any)",
  "$(int64, int64, xxx)-> Any",
  "=(int64, int64, xxx)-> Any",
  "$[(int64, int64, xxx)-> Any]",
  "xxx[(int64, int64, xxx)-> Any]",
  "?$(int64, int64, xxx)-> Any]",
  "?=(int64, int64, xxx)-> Any]",
  "?[(int64, int64, xxx)-> Any$",
  "?[(int64, int64, xxx)-> Any$",
  "10 $ (int64, int64, xxx)-> Any",
  "?10 $ (int64, int64, xxx)-> Any",
  "?(10 $ (int64, int64, xxx)-> Any)",
  "$10 $ (int64, int64, xxx)-> Any",
  "=10 $ (int64, int64, xxx)-> Any",
  "$[10 $ (int64, int64, xxx)-> Any]",
  "xxx[10 $ (int64, int64, xxx)-> Any]",
  "?$10 $ (int64, int64, xxx)-> Any]",
  "?=10 $ (int64, int64, xxx)-> Any]",
  "?[10 $ (int64, int64, xxx)-> Any$",
  "?[10 $ (int64, int64, xxx)-> Any$",
  "10 = (int64, int64, xxx)-> Any",
  "?10 = (int64, int64, xxx)-> Any",
  "?(10 = (int64, int64, xxx)-> Any)",
  "$10 = (int64, int64, xxx)-> Any",
  "=10 = (int64, int64, xxx)-> Any",
  "$[10 = (int64, int64, xxx)-> Any]",
  "xxx[10 = (int64, int64, xxx)-> Any]",
  "?$10 = (int64, int64, xxx)-> Any]",
  "?=10 = (int64, int64, xxx)-> Any]",
  "?[10 = (int64, int64, xxx)-> Any$",
  "?[10 = (int64, int64, xxx)-> Any$",
  "10 * ()->(int64, int64, xxx)",
  "?10 * ()->(int64, int64, xxx)",
  "?(10 * ()->(int64, int64, xxx))",
  "$10 * ()->(int64, int64, xxx)",
  "=10 * ()->(int64, int64, xxx)",
  "$[10 * ()->(int64, int64, xxx)]",
  "xxx[10 * ()->(int64, int64, xxx)]",
  "?$10 * ()->(int64, int64, xxx)]",
  "?=10 * ()->(int64, int64, xxx)]",
  "?[10 * ()->(int64, int64, xxx)$",
  "?[10 * ()->(int64, int64, xxx)$",
  "()->(int64, int64, xxx)",
  "?()->(int64, int64, xxx)",
  "?(()->(int64, int64, xxx))",
  "$()->(int64, int64, xxx)",
  "=()->(int64, int64, xxx)",
  "$[()->(int64, int64, xxx)]",
  "xxx[()->(int64, int64, xxx)]",
  "?$()->(int64, int64, xxx)]",
  "?=()->(int64, int64, xxx)]",
  "?[()->(int64, int64, xxx)$",
  "?[()->(int64, int64, xxx)$",
  "10 $ ()->(int64, int64, xxx)",
  "?10 $ ()->(int64, int64, xxx)",
  "?(10 $ ()->(int64, int64, xxx))",
  "$10 $ ()->(int64, int64, xxx)",
  "=10 $ ()->(int64, int64, xxx)",
  "$[10 $ ()->(int64, int64, xxx)]",
  "xxx[10 $ ()->(int64, int64, xxx)]",
  "?$10 $ ()->(int64, int64, xxx)]",
  "?=10 $ ()->(int64, int64, xxx)]",
  "?[10 $ ()->(int64, int64, xxx)$",
  "?[10 $ ()->(int64, int64, xxx)$",
  "10 = ()->(int64, int64, xxx)",
  "?10 = ()->(int64, int64, xxx)",
  "?(10 = ()->(int64, int64, xxx))",
  "$10 = ()->(int64, int64, xxx)",
  "=10 = ()->(int64, int64, xxx)",
  "$[10 = ()->(int64, int64, xxx)]",
  "xxx[10 = ()->(int64, int64, xxx)]",
  "?$10 = ()->(int64, int64, xxx)]",
  "?=10 = ()->(int64, int64, xxx)]",
  "?[10 = ()->(int64, int64, xxx)$",
  "?[10 = ()->(int64, int64, xxx)$",
  "10 * (int64, int64$ xxx)-> Any",
  "?10 * (int64, int64$ xxx)-> Any",
  "?(10 * (int64, int64$ xxx)-> Any)",
  "$10 * (int64, int64$ xxx)-> Any",
  "=10 * (int64, int64$ xxx)-> Any",
  "$[10 * (int64, int64$ xxx)-> Any]",
  "xxx[10 * (int64, int64$ xxx)-> Any]",
  "?$10 * (int64, int64$ xxx)-> Any]",
  "?=10 * (int64, int64$ xxx)-> Any]",
  "?[10 * (int64, int64$ xxx)-> Any$",
  "?[10 * (int64, int64$ xxx)-> Any$",
  "(int64, int64$ xxx)-> Any",
  "?(int64, int64$ xxx)-> Any",
  "?((int64, int64$ xxx)-> Any)",
  "$(int64, int64$ xxx)-> Any",
  "=(int64, int64$ xxx)-> Any",
  "$[(int64, int64$ xxx)-> Any]",
  "xxx[(int64, int64$ xxx)-> Any]",
  "?$(int64, int64$ xxx)-> Any]",
  "?=(int64, int64$ xxx)-> Any]",
  "?[(int64, int64$ xxx)-> Any$",
  "?[(int64, int64$ xxx)-> Any$",
  "10 $ (int64, int64$ xxx)-> Any",
  "?10 $ (int64, int64$ xxx)-> Any",
  "?(10 $ (int64, int64$ xxx)-> Any)",
  "$10 $ (int64, int64$ xxx)-> Any",
  "=10 $ (int64, int64$ xxx)-> Any",
  "$[10 $ (int64, int64$ xxx)-> Any]",
  "xxx[10 $ (int64, int64$ xxx)-> Any]",
  "?$10 $ (int64, int64$ xxx)-> Any]",
  "?=10 $ (int64, int64$ xxx)-> Any]",
  "?[10 $ (int64, int64$ xxx)-> Any$",
  "?[10 $ (int64, int64$ xxx)-> Any$",
  "10 = (int64, int64$ xxx)-> Any",
  "?10 = (int64, int64$ xxx)-> Any",
  "?(10 = (int64, int64$ xxx)-> Any)",
  "$10 = (int64, int64$ xxx)-> Any",
  "=10 = (int64, int64$ xxx)-> Any",
  "$[10 = (int64, int64$ xxx)-> Any]",
  "xxx[10 = (int64, int64$ xxx)-> Any]",
  "?$10 = (int64, int64$ xxx)-> Any]",
  "?=10 = (int64, int64$ xxx)-> Any]",
  "?[10 = (int64, int64$ xxx)-> Any$",
  "?[10 = (int64, int64$ xxx)-> Any$",
  "10 * ()->(int64, int64$ xxx)",
  "?10 * ()->(int64, int64$ xxx)",
  "?(10 * ()->(int64, int64$ xxx))",
  "$10 * ()->(int64, int64$ xxx)",
  "=10 * ()->(int64, int64$ xxx)",
  "$[10 * ()->(int64, int64$ xxx)]",
  "xxx[10 * ()->(int64, int64$ xxx)]",
  "?$10 * ()->(int64, int64$ xxx)]",
  "?=10 * ()->(int64, int64$ xxx)]",
  "?[10 * ()->(int64, int64$ xxx)$",
  "?[10 * ()->(int64, int64$ xxx)$",
  "()->(int64, int64$ xxx)",
  "?()->(int64, int64$ xxx)",
  "?(()->(int64, int64$ xxx))",
  "$()->(int64, int64$ xxx)",
  "=()->(int64, int64$ xxx)",
  "$[()->(int64, int64$ xxx)]",
  "xxx[()->(int64, int64$ xxx)]",
  "?$()->(int64, int64$ xxx)]",
  "?=()->(int64, int64$ xxx)]",
  "?[()->(int64, int64$ xxx)$",
  "?[()->(int64, int64$ xxx)$",
  "10 $ ()->(int64, int64$ xxx)",
  "?10 $ ()->(int64, int64$ xxx)",
  "?(10 $ ()->(int64, int64$ xxx))",
  "$10 $ ()->(int64, int64$ xxx)",
  "=10 $ ()->(int64, int64$ xxx)",
  "$[10 $ ()->(int64, int64$ xxx)]",
  "xxx[10 $ ()->(int64, int64$ xxx)]",
  "?$10 $ ()->(int64, int64$ xxx)]",
  "?=10 $ ()->(int64, int64$ xxx)]",
  "?[10 $ ()->(int64, int64$ xxx)$",
  "?[10 $ ()->(int64, int64$ xxx)$",
  "10 = ()->(int64, int64$ xxx)",
  "?10 = ()->(int64, int64$ xxx)",
  "?(10 = ()->(int64, int64$ xxx))",
  "$10 = ()->(int64, int64$ xxx)",
  "=10 = ()->(int64, int64$ xxx)",
  "$[10 = ()->(int64, int64$ xxx)]",
  "xxx[10 = ()->(int64, int64$ xxx)]",
  "?$10 = ()->(int64, int64$ xxx)]",
  "?=10 = ()->(int64, int64$ xxx)]",
  "?[10 = ()->(int64, int64$ xxx)$",
  "?[10 = ()->(int64, int64$ xxx)$",
  "10 * (int64, int64= xxx)-> Any",
  "?10 * (int64, int64= xxx)-> Any",
  "?(10 * (int64, int64= xxx)-> Any)",
  "$10 * (int64, int64= xxx)-> Any",
  "=10 * (int64, int64= xxx)-> Any",
  "$[10 * (int64, int64= xxx)-> Any]",
  "xxx[10 * (int64, int64= xxx)-> Any]",
  "?$10 * (int64, int64= xxx)-> Any]",
  "?=10 * (int64, int64= xxx)-> Any]",
  "?[10 * (int64, int64= xxx)-> Any$",
  "?[10 * (int64, int64= xxx)-> Any$",
  "(int64, int64= xxx)-> Any",
  "?(int64, int64= xxx)-> Any",
  "?((int64, int64= xxx)-> Any)",
  "$(int64, int64= xxx)-> Any",
  "=(int64, int64= xxx)-> Any",
  "$[(int64, int64= xxx)-> Any]",
  "xxx[(int64, int64= xxx)-> Any]",
  "?$(int64, int64= xxx)-> Any]",
  "?=(int64, int64= xxx)-> Any]",
  "?[(int64, int64= xxx)-> Any$",
  "?[(int64, int64= xxx)-> Any$",
  "10 $ (int64, int64= xxx)-> Any",
  "?10 $ (int64, int64= xxx)-> Any",
  "?(10 $ (int64, int64= xxx)-> Any)",
  "$10 $ (int64, int64= xxx)-> Any",
  "=10 $ (int64, int64= xxx)-> Any",
  "$[10 $ (int64, int64= xxx)-> Any]",
  "xxx[10 $ (int64, int64= xxx)-> Any]",
  "?$10 $ (int64, int64= xxx)-> Any]",
  "?=10 $ (int64, int64= xxx)-> Any]",
  "?[10 $ (int64, int64= xxx)-> Any$",
  "?[10 $ (int64, int64= xxx)-> Any$",
  "10 = (int64, int64= xxx)-> Any",
  "?10 = (int64, int64= xxx)-> Any",
  "?(10 = (int64, int64= xxx)-> Any)",
  "$10 = (int64, int64= xxx)-> Any",
  "=10 = (int64, int64= xxx)-> Any",
  "$[10 = (int64, int64= xxx)-> Any]",
  "xxx[10 = (int64, int64= xxx)-> Any]",
  "?$10 = (int64, int64= xxx)-> Any]",
  "?=10 = (int64, int64= xxx)-> Any]",
  "?[10 = (int64, int64= xxx)-> Any$",
  "?[10 = (int64, int64= xxx)-> Any$",
  "10 * ()->(int64, int64= xxx)",
  "?10 * ()->(int64, int64= xxx)",
  "?(10 * ()->(int64, int64= xxx))",
  "$10 * ()->(int64, int64= xxx)",
  "=10 * ()->(int64, int64= xxx)",
  "$[10 * ()->(int64, int64= xxx)]",
  "xxx[10 * ()->(int64, int64= xxx)]",
  "?$10 * ()->(int64, int64= xxx)]",
  "?=10 * ()->(int64, int64= xxx)]",
  "?[10 * ()->(int64, int64= xxx)$",
  "?[10 * ()->(int64, int64= xxx)$",
  "()->(int64, int64= xxx)",
  "?()->(int64, int64= xxx)",
  "?(()->(int64, int64= xxx))",
  "$()->(int64, int64= xxx)",
  "=()->(int64, int64= xxx)",
  "$[()->(int64, int64= xxx)]",
  "xxx[()->(int64, int64= xxx)]",
  "?$()->(int64, int64= xxx)]",
  "?=()->(int64, int64= xxx)]",
  "?[()->(int64, int64= xxx)$",
  "?[()->(int64, int64= xxx)$",
  "10 $ ()->(int64, int64= xxx)",
  "?10 $ ()->(int64, int64= xxx)",
  "?(10 $ ()->(int64, int64= xxx))",
  "$10 $ ()->(int64, int64= xxx)",
  "=10 $ ()->(int64, int64= xxx)",
  "$[10 $ ()->(int64, int64= xxx)]",
  "xxx[10 $ ()->(int64, int64= xxx)]",
  "?$10 $ ()->(int64, int64= xxx)]",
  "?=10 $ ()->(int64, int64= xxx)]",
  "?[10 $ ()->(int64, int64= xxx)$",
  "?[10 $ ()->(int64, int64= xxx)$",
  "10 = ()->(int64, int64= xxx)",
  "?10 = ()->(int64, int64= xxx)",
  "?(10 = ()->(int64, int64= xxx))",
  "$10 = ()->(int64, int64= xxx)",
  "=10 = ()->(int64, int64= xxx)",
  "$[10 = ()->(int64, int64= xxx)]",
  "xxx[10 = ()->(int64, int64= xxx)]",
  "?$10 = ()->(int64, int64= xxx)]",
  "?=10 = ()->(int64, int64= xxx)]",
  "?[10 = ()->(int64, int64= xxx)$",
  "?[10 = ()->(int64, int64= xxx)$",
  "10 * ($)-> Any",
  "?10 * ($)-> Any",
  "?(10 * ($)-> Any)",
  "$10 * ($)-> Any",
  "=10 * ($)-> Any",
  "$[10 * ($)-> Any]",
  "xxx[10 * ($)-> Any]",
  "?$10 * ($)-> Any]",
  "?=10 * ($)-> Any]",
  "?[10 * ($)-> Any$",
  "?[10 * ($)-> Any$",
  "($)-> Any",
  "?($)-> Any",
  "?(($)-> Any)",
  "$($)-> Any",
  "=($)-> Any",
  "$[($)-> Any]",
  "xxx[($)-> Any]",
  "?$($)-> Any]",
  "?=($)-> Any]",
  "?[($)-> Any$",
  "?[($)-> Any$",
  "10 $ ($)-> Any",
  "?10 $ ($)-> Any",
  "?(10 $ ($)-> Any)",
  "$10 $ ($)-> Any",
  "=10 $ ($)-> Any",
  "$[10 $ ($)-> Any]",
  "xxx[10 $ ($)-> Any]",
  "?$10 $ ($)-> Any]",
  "?=10 $ ($)-> Any]",
  "?[10 $ ($)-> Any$",
  "?[10 $ ($)-> Any$",
  "10 = ($)-> Any",
  "?10 = ($)-> Any",
  "?(10 = ($)-> Any)",
  "$10 = ($)-> Any",
  "=10 = ($)-> Any",
  "$[10 = ($)-> Any]",
  "xxx[10 = ($)-> Any]",
  "?$10 = ($)-> Any]",
  "?=10 = ($)-> Any]",
  "?[10 = ($)-> Any$",
  "?[10 = ($)-> Any$",
  "10 * ()->($)",
  "?10 * ()->($)",
  "?(10 * ()->($))",
  "$10 * ()->($)",
  "=10 * ()->($)",
  "$[10 * ()->($)]",
  "xxx[10 * ()->($)]",
  "?$10 * ()->($)]",
  "?=10 * ()->($)]",
  "?[10 * ()->($)$",
  "?[10 * ()->($)$",
  "()->($)",
  "?()->($)",
  "?(()->($))",
  "$()->($)",
  "=()->($)",
  "$[()->($)]",
  "xxx[()->($)]",
  "?$()->($)]",
  "?=()->($)]",
  "?[()->($)$",
  "?[()->($)$",
  "10 $ ()->($)",
  "?10 $ ()->($)",
  "?(10 $ ()->($))",
  "$10 $ ()->($)",
  "=10 $ ()->($)",
  "$[10 $ ()->($)]",
  "xxx[10 $ ()->($)]",
  "?$10 $ ()->($)]",
  "?=10 $ ()->($)]",
  "?[10 $ ()->($)$",
  "?[10 $ ()->($)$",
  "10 = ()->($)",
  "?10 = ()->($)",
  "?(10 = ()->($))",
  "$10 = ()->($)",
  "=10 = ()->($)",
  "$[10 = ()->($)]",
  "xxx[10 = ()->($)]",
  "?$10 = ()->($)]",
  "?=10 = ()->($)]",
  "?[10 = ()->($)$",
  "?[10 = ()->($)$",
  "10 * (int64, $)-> Any",
  "?10 * (int64, $)-> Any",
  "?(10 * (int64, $)-> Any)",
  "$10 * (int64, $)-> Any",
  "=10 * (int64, $)-> Any",
  "$[10 * (int64, $)-> Any]",
  "xxx[10 * (int64, $)-> Any]",
  "?$10 * (int64, $)-> Any]",
  "?=10 * (int64, $)-> Any]",
  "?[10 * (int64, $)-> Any$",
  "?[10 * (int64, $)-> Any$",
  "(int64, $)-> Any",
  "?(int64, $)-> Any",
  "?((int64, $)-> Any)",
  "$(int64, $)-> Any",
  "=(int64, $)-> Any",
  "$[(int64, $)-> Any]",
  "xxx[(int64, $)-> Any]",
  "?$(int64, $)-> Any]",
  "?=(int64, $)-> Any]",
  "?[(int64, $)-> Any$",
  "?[(int64, $)-> Any$",
  "10 $ (int64, $)-> Any",
  "?10 $ (int64, $)-> Any",
  "?(10 $ (int64, $)-> Any)",
  "$10 $ (int64, $)-> Any",
  "=10 $ (int64, $)-> Any",
  "$[10 $ (int64, $)-> Any]",
  "xxx[10 $ (int64, $)-> Any]",
  "?$10 $ (int64, $)-> Any]",
  "?=10 $ (int64, $)-> Any]",
  "?[10 $ (int64, $)-> Any$",
  "?[10 $ (int64, $)-> Any$",
  "10 = (int64, $)-> Any",
  "?10 = (int64, $)-> Any",
  "?(10 = (int64, $)-> Any)",
  "$10 = (int64, $)-> Any",
  "=10 = (int64, $)-> Any",
  "$[10 = (int64, $)-> Any]",
  "xxx[10 = (int64, $)-> Any]",
  "?$10 = (int64, $)-> Any]",
  "?=10 = (int64, $)-> Any]",
  "?[10 = (int64, $)-> Any$",
  "?[10 = (int64, $)-> Any$",
  "10 * ()->(int64, $)",
  "?10 * ()->(int64, $)",
  "?(10 * ()->(int64, $))",
  "$10 * ()->(int64, $)",
  "=10 * ()->(int64, $)",
  "$[10 * ()->(int64, $)]",
  "xxx[10 * ()->(int64, $)]",
  "?$10 * ()->(int64, $)]",
  "?=10 * ()->(int64, $)]",
  "?[10 * ()->(int64, $)$",
  "?[10 * ()->(int64, $)$",
  "()->(int64, $)",
  "?()->(int64, $)",
  "?(()->(int64, $))",
  "$()->(int64, $)",
  "=()->(int64, $)",
  "$[()->(int64, $)]",
  "xxx[()->(int64, $)]",
  "?$()->(int64, $)]",
  "?=()->(int64, $)]",
  "?[()->(int64, $)$",
  "?[()->(int64, $)$",
  "10 $ ()->(int64, $)",
  "?10 $ ()->(int64, $)",
  "?(10 $ ()->(int64, $))",
  "$10 $ ()->(int64, $)",
  "=10 $ ()->(int64, $)",
  "$[10 $ ()->(int64, $)]",
  "xxx[10 $ ()->(int64, $)]",
  "?$10 $ ()->(int64, $)]",
  "?=10 $ ()->(int64, $)]",
  "?[10 $ ()->(int64, $)$",
  "?[10 $ ()->(int64, $)$",
  "10 = ()->(int64, $)",
  "?10 = ()->(int64, $)",
  "?(10 = ()->(int64, $))",
  "$10 = ()->(int64, $)",
  "=10 = ()->(int64, $)",
  "$[10 = ()->(int64, $)]",
  "xxx[10 = ()->(int64, $)]",
  "?$10 = ()->(int64, $)]",
  "?=10 = ()->(int64, $)]",
  "?[10 = ()->(int64, $)$",
  "?[10 = ()->(int64, $)$",
  "10 * (int64$ $)-> Any",
  "?10 * (int64$ $)-> Any",
  "?(10 * (int64$ $)-> Any)",
  "$10 * (int64$ $)-> Any",
  "=10 * (int64$ $)-> Any",
  "$[10 * (int64$ $)-> Any]",
  "xxx[10 * (int64$ $)-> Any]",
  "?$10 * (int64$ $)-> Any]",
  "?=10 * (int64$ $)-> Any]",
  "?[10 * (int64$ $)-> Any$",
  "?[10 * (int64$ $)-> Any$",
  "(int64$ $)-> Any",
  "?(int64$ $)-> Any",
  "?((int64$ $)-> Any)",
  "$(int64$ $)-> Any",
  "=(int64$ $)-> Any",
  "$[(int64$ $)-> Any]",
  "xxx[(int64$ $)-> Any]",
  "?$(int64$ $)-> Any]",
  "?=(int64$ $)-> Any]",
  "?[(int64$ $)-> Any$",
  "?[(int64$ $)-> Any$",
  "10 $ (int64$ $)-> Any",
  "?10 $ (int64$ $)-> Any",
  "?(10 $ (int64$ $)-> Any)",
  "$10 $ (int64$ $)-> Any",
  "=10 $ (int64$ $)-> Any",
  "$[10 $ (int64$ $)-> Any]",
  "xxx[10 $ (int64$ $)-> Any]",
  "?$10 $ (int64$ $)-> Any]",
  "?=10 $ (int64$ $)-> Any]",
  "?[10 $ (int64$ $)-> Any$",
  "?[10 $ (int64$ $)-> Any$",
  "10 = (int64$ $)-> Any",
  "?10 = (int64$ $)-> Any",
  "?(10 = (int64$ $)-> Any)",
  "$10 = (int64$ $)-> Any",
  "=10 = (int64$ $)-> Any",
  "$[10 = (int64$ $)-> Any]",
  "xxx[10 = (int64$ $)-> Any]",
  "?$10 = (int64$ $)-> Any]",
  "?=10 = (int64$ $)-> Any]",
  "?[10 = (int64$ $)-> Any$",
  "?[10 = (int64$ $)-> Any$",
  "10 * ()->(int64$ $)",
  "?10 * ()->(int64$ $)",
  "?(10 * ()->(int64$ $))",
  "$10 * ()->(int64$ $)",
  "=10 * ()->(int64$ $)",
  "$[10 * ()->(int64$ $)]",
  "xxx[10 * ()->(int64$ $)]",
  "?$10 * ()->(int64$ $)]",
  "?=10 * ()->(int64$ $)]",
  "?[10 * ()->(int64$ $)$",
  "?[10 * ()->(int64$ $)$",
  "()->(int64$ $)",
  "?()->(int64$ $)",
  "?(()->(int64$ $))",
  "$()->(int64$ $)",
  "=()->(int64$ $)",
  "$[()->(int64$ $)]",
  "xxx[()->(int64$ $)]",
  "?$()->(int64$ $)]",
  "?=()->(int64$ $)]",
  "?[()->(int64$ $)$",
  "?[()->(int64$ $)$",
  "10 $ ()->(int64$ $)",
  "?10 $ ()->(int64$ $)",
  "?(10 $ ()->(int64$ $))",
  "$10 $ ()->(int64$ $)",
  "=10 $ ()->(int64$ $)",
  "$[10 $ ()->(int64$ $)]",
  "xxx[10 $ ()->(int64$ $)]",
  "?$10 $ ()->(int64$ $)]",
  "?=10 $ ()->(int64$ $)]",
  "?[10 $ ()->(int64$ $)$",
  "?[10 $ ()->(int64$ $)$",
  "10 = ()->(int64$ $)",
  "?10 = ()->(int64$ $)",
  "?(10 = ()->(int64$ $))",
  "$10 = ()->(int64$ $)",
  "=10 = ()->(int64$ $)",
  "$[10 = ()->(int64$ $)]",
  "xxx[10 = ()->(int64$ $)]",
  "?$10 = ()->(int64$ $)]",
  "?=10 = ()->(int64$ $)]",
  "?[10 = ()->(int64$ $)$",
  "?[10 = ()->(int64$ $)$",
  "10 * (int64= $)-> Any",
  "?10 * (int64= $)-> Any",
  "?(10 * (int64= $)-> Any)",
  "$10 * (int64= $)-> Any",
  "=10 * (int64= $)-> Any",
  "$[10 * (int64= $)-> Any]",
  "xxx[10 * (int64= $)-> Any]",
  "?$10 * (int64= $)-> Any]",
  "?=10 * (int64= $)-> Any]",
  "?[10 * (int64= $)-> Any$",
  "?[10 * (int64= $)-> Any$",
  "(int64= $)-> Any",
  "?(int64= $)-> Any",
  "?((int64= $)-> Any)",
  "$(int64= $)-> Any",
  "=(int64= $)-> Any",
  "$[(int64= $)-> Any]",
  "xxx[(int64= $)-> Any]",
  "?$(int64= $)-> Any]",
  "?=(int64= $)-> Any]",
  "?[(int64= $)-> Any$",
  "?[(int64= $)-> Any$",
  "10 $ (int64= $)-> Any",
  "?10 $ (int64= $)-> Any",
  "?(10 $ (int64= $)-> Any)",
  "$10 $ (int64= $)-> Any",
  "=10 $ (int64= $)-> Any",
  "$[10 $ (int64= $)-> Any]",
  "xxx[10 $ (int64= $)-> Any]",
  "?$10 $ (int64= $)-> Any]",
  "?=10 $ (int64= $)-> Any]",
  "?[10 $ (int64= $)-> Any$",
  "?[10 $ (int64= $)-> Any$",
  "10 = (int64= $)-> Any",
  "?10 = (int64= $)-> Any",
  "?(10 = (int64= $)-> Any)",
  "$10 = (int64= $)-> Any",
  "=10 = (int64= $)-> Any",
  "$[10 = (int64= $)-> Any]",
  "xxx[10 = (int64= $)-> Any]",
  "?$10 = (int64= $)-> Any]",
  "?=10 = (int64= $)-> Any]",
  "?[10 = (int64= $)-> Any$",
  "?[10 = (int64= $)-> Any$",
  "10 * ()->(int64= $)",
  "?10 * ()->(int64= $)",
  "?(10 * ()->(int64= $))",
  "$10 * ()->(int64= $)",
  "=10 * ()->(int64= $)",
  "$[10 * ()->(int64= $)]",
  "xxx[10 * ()->(int64= $)]",
  "?$10 * ()->(int64= $)]",
  "?=10 * ()->(int64= $)]",
  "?[10 * ()->(int64= $)$",
  "?[10 * ()->(int64= $)$",
  "()->(int64= $)",
  "?()->(int64= $)",
  "?(()->(int64= $))",
  "$()->(int64= $)",
  "=()->(int64= $)",
  "$[()->(int64= $)]",
  "xxx[()->(int64= $)]",
  "?$()->(int64= $)]",
  "?=()->(int64= $)]",
  "?[()->(int64= $)$",
  "?[()->(int64= $)$",
  "10 $ ()->(int64= $)",
  "?10 $ ()->(int64= $)",
  "?(10 $ ()->(int64= $))",
  "$10 $ ()->(int64= $)",
  "=10 $ ()->(int64= $)",
  "$[10 $ ()->(int64= $)]",
  "xxx[10 $ ()->(int64= $)]",
  "?$10 $ ()->(int64= $)]",
  "?=10 $ ()->(int64= $)]",
  "?[10 $ ()->(int64= $)$",
  "?[10 $ ()->(int64= $)$",
  "10 = ()->(int64= $)",
  "?10 = ()->(int64= $)",
  "?(10 = ()->(int64= $))",
  "$10 = ()->(int64= $)",
  "=10 = ()->(int64= $)",
  "$[10 = ()->(int64= $)]",
  "xxx[10 = ()->(int64= $)]",
  "?$10 = ()->(int64= $)]",
  "?=10 = ()->(int64= $)]",
  "?[10 = ()->(int64= $)$",
  "?[10 = ()->(int64= $)$",
  "10 * (int64, int64, $)-> Any",
  "?10 * (int64, int64, $)-> Any",
  "?(10 * (int64, int64, $)-> Any)",
  "$10 * (int64, int64, $)-> Any",
  "=10 * (int64, int64, $)-> Any",
  "$[10 * (int64, int64, $)-> Any]",
  "xxx[10 * (int64, int64, $)-> Any]",
  "?$10 * (int64, int64, $)-> Any]",
  "?=10 * (int64, int64, $)-> Any]",
  "?[10 * (int64, int64, $)-> Any$",
  "?[10 * (int64, int64, $)-> Any$",
  "(int64, int64, $)-> Any",
  "?(int64, int64, $)-> Any",
  "?((int64, int64, $)-> Any)",
  "$(int64, int64, $)-> Any",
  "=(int64, int64, $)-> Any",
  "$[(int64, int64, $)-> Any]",
  "xxx[(int64, int64, $)-> Any]",
  "?$(int64, int64, $)-> Any]",
  "?=(int64, int64, $)-> Any]",
  "?[(int64, int64, $)-> Any$",
  "?[(int64, int64, $)-> Any$",
  "10 $ (int64, int64, $)-> Any",
  "?10 $ (int64, int64, $)-> Any",
  "?(10 $ (int64, int64, $)-> Any)",
  "$10 $ (int64, int64, $)-> Any",
  "=10 $ (int64, int64, $)-> Any",
  "$[10 $ (int64, int64, $)-> Any]",
  "xxx[10 $ (int64, int64, $)-> Any]",
  "?$10 $ (int64, int64, $)-> Any]",
  "?=10 $ (int64, int64, $)-> Any]",
  "?[10 $ (int64, int64, $)-> Any$",
  "?[10 $ (int64, int64, $)-> Any$",
  "10 = (int64, int64, $)-> Any",
  "?10 = (int64, int64, $)-> Any",
  "?(10 = (int64, int64, $)-> Any)",
  "$10 = (int64, int64, $)-> Any",
  "=10 = (int64, int64, $)-> Any",
  "$[10 = (int64, int64, $)-> Any]",
  "xxx[10 = (int64, int64, $)-> Any]",
  "?$10 = (int64, int64, $)-> Any]",
  "?=10 = (int64, int64, $)-> Any]",
  "?[10 = (int64, int64, $)-> Any$",
  "?[10 = (int64, int64, $)-> Any$",
  "10 * ()->(int64, int64, $)",
  "?10 * ()->(int64, int64, $)",
  "?(10 * ()->(int64, int64, $))",
  "$10 * ()->(int64, int64, $)",
  "=10 * ()->(int64, int64, $)",
  "$[10 * ()->(int64, int64, $)]",
  "xxx[10 * ()->(int64, int64, $)]",
  "?$10 * ()->(int64, int64, $)]",
  "?=10 * ()->(int64, int64, $)]",
  "?[10 * ()->(int64, int64, $)$",
  "?[10 * ()->(int64, int64, $)$",
  "()->(int64, int64, $)",
  "?()->(int64, int64, $)",
  "?(()->(int64, int64, $))",
  "$()->(int64, int64, $)",
  "=()->(int64, int64, $)",
  "$[()->(int64, int64, $)]",
  "xxx[()->(int64, int64, $)]",
  "?$()->(int64, int64, $)]",
  "?=()->(int64, int64, $)]",
  "?[()->(int64, int64, $)$",
  "?[()->(int64, int64, $)$",
  "10 $ ()->(int64, int64, $)",
  "?10 $ ()->(int64, int64, $)",
  "?(10 $ ()->(int64, int64, $))",
  "$10 $ ()->(int64, int64, $)",
  "=10 $ ()->(int64, int64, $)",
  "$[10 $ ()->(int64, int64, $)]",
  "xxx[10 $ ()->(int64, int64, $)]",
  "?$10 $ ()->(int64, int64, $)]",
  "?=10 $ ()->(int64, int64, $)]",
  "?[10 $ ()->(int64, int64, $)$",
  "?[10 $ ()->(int64, int64, $)$",
  "10 = ()->(int64, int64, $)",
  "?10 = ()->(int64, int64, $)",
  "?(10 = ()->(int64, int64, $))",
  "$10 = ()->(int64, int64, $)",
  "=10 = ()->(int64, int64, $)",
  "$[10 = ()->(int64, int64, $)]",
  "xxx[10 = ()->(int64, int64, $)]",
  "?$10 = ()->(int64, int64, $)]",
  "?=10 = ()->(int64, int64, $)]",
  "?[10 = ()->(int64, int64, $)$",
  "?[10 = ()->(int64, int64, $)$",
  "10 * (int64, int64$ $)-> Any",
  "?10 * (int64, int64$ $)-> Any",
  "?(10 * (int64, int64$ $)-> Any)",
  "$10 * (int64, int64$ $)-> Any",
  "=10 * (int64, int64$ $)-> Any",
  "$[10 * (int64, int64$ $)-> Any]",
  "xxx[10 * (int64, int64$ $)-> Any]",
  "?$10 * (int64, int64$ $)-> Any]",
  "?=10 * (int64, int64$ $)-> Any]",
  "?[10 * (int64, int64$ $)-> Any$",
  "?[10 * (int64, int64$ $)-> Any$",
  "(int64, int64$ $)-> Any",
  "?(int64, int64$ $)-> Any",
  "?((int64, int64$ $)-> Any)",
  "$(int64, int64$ $)-> Any",
  "=(int64, int64$ $)-> Any",
  "$[(int64, int64$ $)-> Any]",
  "xxx[(int64, int64$ $)-> Any]",
  "?$(int64, int64$ $)-> Any]",
  "?=(int64, int64$ $)-> Any]",
  "?[(int64, int64$ $)-> Any$",
  "?[(int64, int64$ $)-> Any$",
  "10 $ (int64, int64$ $)-> Any",
  "?10 $ (int64, int64$ $)-> Any",
  "?(10 $ (int64, int64$ $)-> Any)",
  "$10 $ (int64, int64$ $)-> Any",
  "=10 $ (int64, int64$ $)-> Any",
  "$[10 $ (int64, int64$ $)-> Any]",
  "xxx[10 $ (int64, int64$ $)-> Any]",
  "?$10 $ (int64, int64$ $)-> Any]",
  "?=10 $ (int64, int64$ $)-> Any]",
  "?[10 $ (int64, int64$ $)-> Any$",
  "?[10 $ (int64, int64$ $)-> Any$",
  "10 = (int64, int64$ $)-> Any",
  "?10 = (int64, int64$ $)-> Any",
  "?(10 = (int64, int64$ $)-> Any)",
  "$10 = (int64, int64$ $)-> Any",
  "=10 = (int64, int64$ $)-> Any",
  "$[10 = (int64, int64$ $)-> Any]",
  "xxx[10 = (int64, int64$ $)-> Any]",
  "?$10 = (int64, int64$ $)-> Any]",
  "?=10 = (int64, int64$ $)-> Any]",
  "?[10 = (int64, int64$ $)-> Any$",
  "?[10 = (int64, int64$ $)-> Any$",
  "10 * ()->(int64, int64$ $)",
  "?10 * ()->(int64, int64$ $)",
  "?(10 * ()->(int64, int64$ $))",
  "$10 * ()->(int64, int64$ $)",
  "=10 * ()->(int64, int64$ $)",
  "$[10 * ()->(int64, int64$ $)]",
  "xxx[10 * ()->(int64, int64$ $)]",
  "?$10 * ()->(int64, int64$ $)]",
  "?=10 * ()->(int64, int64$ $)]",
  "?[10 * ()->(int64, int64$ $)$",
  "?[10 * ()->(int64, int64$ $)$",
  "()->(int64, int64$ $)",
  "?()->(int64, int64$ $)",
  "?(()->(int64, int64$ $))",
  "$()->(int64, int64$ $)",
  "=()->(int64, int64$ $)",
  "$[()->(int64, int64$ $)]",
  "xxx[()->(int64, int64$ $)]",
  "?$()->(int64, int64$ $)]",
  "?=()->(int64, int64$ $)]",
  "?[()->(int64, int64$ $)$",
  "?[()->(int64, int64$ $)$",
  "10 $ ()->(int64, int64$ $)",
  "?10 $ ()->(int64, int64$ $)",
  "?(10 $ ()->(int64, int64$ $))",
  "$10 $ ()->(int64, int64$ $)",
  "=10 $ ()->(int64, int64$ $)",
  "$[10 $ ()->(int64, int64$ $)]",
  "xxx[10 $ ()->(int64, int64$ $)]",
  "?$10 $ ()->(int64, int64$ $)]",
  "?=10 $ ()->(int64, int64$ $)]",
  "?[10 $ ()->(int64, int64$ $)$",
  "?[10 $ ()->(int64, int64$ $)$",
  "10 = ()->(int64, int64$ $)",
  "?10 = ()->(int64, int64$ $)",
  "?(10 = ()->(int64, int64$ $))",
  "$10 = ()->(int64, int64$ $)",
  "=10 = ()->(int64, int64$ $)",
  "$[10 = ()->(int64, int64$ $)]",
  "xxx[10 = ()->(int64, int64$ $)]",
  "?$10 = ()->(int64, int64$ $)]",
  "?=10 = ()->(int64, int64$ $)]",
  "?[10 = ()->(int64, int64$ $)$",
  "?[10 = ()->(int64, int64$ $)$",
  "10 * (int64, int64= $)-> Any",
  "?10 * (int64, int64= $)-> Any",
  "?(10 * (int64, int64= $)-> Any)",
  "$10 * (int64, int64= $)-> Any",
  "=10 * (int64, int64= $)-> Any",
  "$[10 * (int64, int64= $)-> Any]",
  "xxx[10 * (int64, int64= $)-> Any]",
  "?$10 * (int64, int64= $)-> Any]",
  "?=10 * (int64, int64= $)-> Any]",
  "?[10 * (int64, int64= $)-> Any$",
  "?[10 * (int64, int64= $)-> Any$",
  "(int64, int64= $)-> Any",
  "?(int64, int64= $)-> Any",
  "?((int64, int64= $)-> Any)",
  "$(int64, int64= $)-> Any",
  "=(int64, int64= $)-> Any",
  "$[(int64, int64= $)-> Any]",
  "xxx[(int64, int64= $)-> Any]",
  "?$(int64, int64= $)-> Any]",
  "?=(int64, int64= $)-> Any]",
  "?[(int64, int64= $)-> Any$",
  "?[(int64, int64= $)-> Any$",
  "10 $ (int64, int64= $)-> Any",
  "?10 $ (int64, int64= $)-> Any",
  "?(10 $ (int64, int64= $)-> Any)",
  "$10 $ (int64, int64= $)-> Any",
  "=10 $ (int64, int64= $)-> Any",
  "$[10 $ (int64, int64= $)-> Any]",
  "xxx[10 $ (int64, int64= $)-> Any]",
  "?$10 $ (int64, int64= $)-> Any]",
  "?=10 $ (int64, int64= $)-> Any]",
  "?[10 $ (int64, int64= $)-> Any$",
  "?[10 $ (int64, int64= $)-> Any$",
  "10 = (int64, int64= $)-> Any",
  "?10 = (int64, int64= $)-> Any",
  "?(10 = (int64, int64= $)-> Any)",
  "$10 = (int64, int64= $)-> Any",
  "=10 = (int64, int64= $)-> Any",
  "$[10 = (int64, int64= $)-> Any]",
  "xxx[10 = (int64, int64= $)-> Any]",
  "?$10 = (int64, int64= $)-> Any]",
  "?=10 = (int64, int64= $)-> Any]",
  "?[10 = (int64, int64= $)-> Any$",
  "?[10 = (int64, int64= $)-> Any$",
  "10 * ()->(int64, int64= $)",
  "?10 * ()->(int64, int64= $)",
  "?(10 * ()->(int64, int64= $))",
  "$10 * ()->(int64, int64= $)",
  "=10 * ()->(int64, int64= $)",
  "$[10 * ()->(int64, int64= $)]",
  "xxx[10 * ()->(int64, int64= $)]",
  "?$10 * ()->(int64, int64= $)]",
  "?=10 * ()->(int64, int64= $)]",
  "?[10 * ()->(int64, int64= $)$",
  "?[10 * ()->(int64, int64= $)$",
  "()->(int64, int64= $)",
  "?()->(int64, int64= $)",
  "?(()->(int64, int64= $))",
  "$()->(int64, int64= $)",
  "=()->(int64, int64= $)",
  "$[()->(int64, int64= $)]",
  "xxx[()->(int64, int64= $)]",
  "?$()->(int64, int64= $)]",
  "?=()->(int64, int64= $)]",
  "?[()->(int64, int64= $)$",
  "?[()->(int64, int64= $)$",
  "10 $ ()->(int64, int64= $)",
  "?10 $ ()->(int64, int64= $)",
  "?(10 $ ()->(int64, int64= $))",
  "$10 $ ()->(int64, int64= $)",
  "=10 $ ()->(int64, int64= $)",
  "$[10 $ ()->(int64, int64= $)]",
  "xxx[10 $ ()->(int64, int64= $)]",
  "?$10 $ ()->(int64, int64= $)]",
  "?=10 $ ()->(int64, int64= $)]",
  "?[10 $ ()->(int64, int64= $)$",
  "?[10 $ ()->(int64, int64= $)$",
  "10 = ()->(int64, int64= $)",
  "?10 = ()->(int64, int64= $)",
  "?(10 = ()->(int64, int64= $))",
  "$10 = ()->(int64, int64= $)",
  "=10 = ()->(int64, int64= $)",
  "$[10 = ()->(int64, int64= $)]",
  "xxx[10 = ()->(int64, int64= $)]",
  "?$10 = ()->(int64, int64= $)]",
  "?=10 = ()->(int64, int64= $)]",
  "?[10 = ()->(int64, int64= $)$",
  "?[10 = ()->(int64, int64= $)$",
  "10 * (=)-> Any",
  "?10 * (=)-> Any",
  "?(10 * (=)-> Any)",
  "$10 * (=)-> Any",
  "=10 * (=)-> Any",
  "$[10 * (=)-> Any]",
  "xxx[10 * (=)-> Any]",
  "?$10 * (=)-> Any]",
  "?=10 * (=)-> Any]",
  "?[10 * (=)-> Any$",
  "?[10 * (=)-> Any$",
  "(=)-> Any",
  "?(=)-> Any",
  "?((=)-> Any)",
  "$(=)-> Any",
  "=(=)-> Any",
  "$[(=)-> Any]",
  "xxx[(=)-> Any]",
  "?$(=)-> Any]",
  "?=(=)-> Any]",
  "?[(=)-> Any$",
  "?[(=)-> Any$",
  "10 $ (=)-> Any",
  "?10 $ (=)-> Any",
  "?(10 $ (=)-> Any)",
  "$10 $ (=)-> Any",
  "=10 $ (=)-> Any",
  "$[10 $ (=)-> Any]",
  "xxx[10 $ (=)-> Any]",
  "?$10 $ (=)-> Any]",
  "?=10 $ (=)-> Any]",
  "?[10 $ (=)-> Any$",
  "?[10 $ (=)-> Any$",
  "10 = (=)-> Any",
  "?10 = (=)-> Any",
  "?(10 = (=)-> Any)",
  "$10 = (=)-> Any",
  "=10 = (=)-> Any",
  "$[10 = (=)-> Any]",
  "xxx[10 = (=)-> Any]",
  "?$10 = (=)-> Any]",
  "?=10 = (=)-> Any]",
  "?[10 = (=)-> Any$",
  "?[10 = (=)-> Any$",
  "10 * ()->(=)",
  "?10 * ()->(=)",
  "?(10 * ()->(=))",
  "$10 * ()->(=)",
  "=10 * ()->(=)",
  "$[10 * ()->(=)]",
  "xxx[10 * ()->(=)]",
  "?$10 * ()->(=)]",
  "?=10 * ()->(=)]",
  "?[10 * ()->(=)$",
  "?[10 * ()->(=)$",
  "()->(=)",
  "?()->(=)",
  "?(()->(=))",
  "$()->(=)",
  "=()->(=)",
  "$[()->(=)]",
  "xxx[()->(=)]",
  "?$()->(=)]",
  "?=()->(=)]",
  "?[()->(=)$",
  "?[()->(=)$",
  "10 $ ()->(=)",
  "?10 $ ()->(=)",
  "?(10 $ ()->(=))",
  "$10 $ ()->(=)",
  "=10 $ ()->(=)",
  "$[10 $ ()->(=)]",
  "xxx[10 $ ()->(=)]",
  "?$10 $ ()->(=)]",
  "?=10 $ ()->(=)]",
  "?[10 $ ()->(=)$",
  "?[10 $ ()->(=)$",
  "10 = ()->(=)",
  "?10 = ()->(=)",
  "?(10 = ()->(=))",
  "$10 = ()->(=)",
  "=10 = ()->(=)",
  "$[10 = ()->(=)]",
  "xxx[10 = ()->(=)]",
  "?$10 = ()->(=)]",
  "?=10 = ()->(=)]",
  "?[10 = ()->(=)$",
  "?[10 = ()->(=)$",
  "10 * (int64, =)-> Any",
  "?10 * (int64, =)-> Any",
  "?(10 * (int64, =)-> Any)",
  "$10 * (int64, =)-> Any",
  "=10 * (int64, =)-> Any",
  "$[10 * (int64, =)-> Any]",
  "xxx[10 * (int64, =)-> Any]",
  "?$10 * (int64, =)-> Any]",
  "?=10 * (int64, =)-> Any]",
  "?[10 * (int64, =)-> Any$",
  "?[10 * (int64, =)-> Any$",
  "(int64, =)-> Any",
  "?(int64, =)-> Any",
  "?((int64, =)-> Any)",
  "$(int64, =)-> Any",
  "=(int64, =)-> Any",
  "$[(int64, =)-> Any]",
  "xxx[(int64, =)-> Any]",
  "?$(int64, =)-> Any]",
  "?=(int64, =)-> Any]",
  "?[(int64, =)-> Any$",
  "?[(int64, =)-> Any$",
  "10 $ (int64, =)-> Any",
  "?10 $ (int64, =)-> Any",
  "?(10 $ (int64, =)-> Any)",
  "$10 $ (int64, =)-> Any",
  "=10 $ (int64, =)-> Any",
  "$[10 $ (int64, =)-> Any]",
  "xxx[10 $ (int64, =)-> Any]",
  "?$10 $ (int64, =)-> Any]",
  "?=10 $ (int64, =)-> Any]",
  "?[10 $ (int64, =)-> Any$",
  "?[10 $ (int64, =)-> Any$",
  "10 = (int64, =)-> Any",
  "?10 = (int64, =)-> Any",
  "?(10 = (int64, =)-> Any)",
  "$10 = (int64, =)-> Any",
  "=10 = (int64, =)-> Any",
  "$[10 = (int64, =)-> Any]",
  "xxx[10 = (int64, =)-> Any]",
  "?$10 = (int64, =)-> Any]",
  "?=10 = (int64, =)-> Any]",
  "?[10 = (int64, =)-> Any$",
  "?[10 = (int64, =)-> Any$",
  "10 * ()->(int64, =)",
  "?10 * ()->(int64, =)",
  "?(10 * ()->(int64, =))",
  "$10 * ()->(int64, =)",
  "=10 * ()->(int64, =)",
  "$[10 * ()->(int64, =)]",
  "xxx[10 * ()->(int64, =)]",
  "?$10 * ()->(int64, =)]",
  "?=10 * ()->(int64, =)]",
  "?[10 * ()->(int64, =)$",
  "?[10 * ()->(int64, =)$",
  "()->(int64, =)",
  "?()->(int64, =)",
  "?(()->(int64, =))",
  "$()->(int64, =)",
  "=()->(int64, =)",
  "$[()->(int64, =)]",
  "xxx[()->(int64, =)]",
  "?$()->(int64, =)]",
  "?=()->(int64, =)]",
  "?[()->(int64, =)$",
  "?[()->(int64, =)$",
  "10 $ ()->(int64, =)",
  "?10 $ ()->(int64, =)",
  "?(10 $ ()->(int64, =))",
  "$10 $ ()->(int64, =)",
  "=10 $ ()->(int64, =)",
  "$[10 $ ()->(int64, =)]",
  "xxx[10 $ ()->(int64, =)]",
  "?$10 $ ()->(int64, =)]",
  "?=10 $ ()->(int64, =)]",
  "?[10 $ ()->(int64, =)$",
  "?[10 $ ()->(int64, =)$",
  "10 = ()->(int64, =)",
  "?10 = ()->(int64, =)",
  "?(10 = ()->(int64, =))",
  "$10 = ()->(int64, =)",
  "=10 = ()->(int64, =)",
  "$[10 = ()->(int64, =)]",
  "xxx[10 = ()->(int64, =)]",
  "?$10 = ()->(int64, =)]",
  "?=10 = ()->(int64, =)]",
  "?[10 = ()->(int64, =)$",
  "?[10 = ()->(int64, =)$",
  "10 * (int64$ =)-> Any",
  "?10 * (int64$ =)-> Any",
  "?(10 * (int64$ =)-> Any)",
  "$10 * (int64$ =)-> Any",
  "=10 * (int64$ =)-> Any",
  "$[10 * (int64$ =)-> Any]",
  "xxx[10 * (int64$ =)-> Any]",
  "?$10 * (int64$ =)-> Any]",
  "?=10 * (int64$ =)-> Any]",
  "?[10 * (int64$ =)-> Any$",
  "?[10 * (int64$ =)-> Any$",
  "(int64$ =)-> Any",
  "?(int64$ =)-> Any",
  "?((int64$ =)-> Any)",
  "$(int64$ =)-> Any",
  "=(int64$ =)-> Any",
  "$[(int64$ =)-> Any]",
  "xxx[(int64$ =)-> Any]",
  "?$(int64$ =)-> Any]",
  "?=(int64$ =)-> Any]",
  "?[(int64$ =)-> Any$",
  "?[(int64$ =)-> Any$",
  "10 $ (int64$ =)-> Any",
  "?10 $ (int64$ =)-> Any",
  "?(10 $ (int64$ =)-> Any)",
  "$10 $ (int64$ =)-> Any",
  "=10 $ (int64$ =)-> Any",
  "$[10 $ (int64$ =)-> Any]",
  "xxx[10 $ (int64$ =)-> Any]",
  "?$10 $ (int64$ =)-> Any]",
  "?=10 $ (int64$ =)-> Any]",
  "?[10 $ (int64$ =)-> Any$",
  "?[10 $ (int64$ =)-> Any$",
  "10 = (int64$ =)-> Any",
  "?10 = (int64$ =)-> Any",
  "?(10 = (int64$ =)-> Any)",
  "$10 = (int64$ =)-> Any",
  "=10 = (int64$ =)-> Any",
  "$[10 = (int64$ =)-> Any]",
  "xxx[10 = (int64$ =)-> Any]",
  "?$10 = (int64$ =)-> Any]",
  "?=10 = (int64$ =)-> Any]",
  "?[10 = (int64$ =)-> Any$",
  "?[10 = (int64$ =)-> Any$",
  "10 * ()->(int64$ =)",
  "?10 * ()->(int64$ =)",
  "?(10 * ()->(int64$ =))",
  "$10 * ()->(int64$ =)",
  "=10 * ()->(int64$ =)",
  "$[10 * ()->(int64$ =)]",
  "xxx[10 * ()->(int64$ =)]",
  "?$10 * ()->(int64$ =)]",
  "?=10 * ()->(int64$ =)]",
  "?[10 * ()->(int64$ =)$",
  "?[10 * ()->(int64$ =)$",
  "()->(int64$ =)",
  "?()->(int64$ =)",
  "?(()->(int64$ =))",
  "$()->(int64$ =)",
  "=()->(int64$ =)",
  "$[()->(int64$ =)]",
  "xxx[()->(int64$ =)]",
  "?$()->(int64$ =)]",
  "?=()->(int64$ =)]",
  "?[()->(int64$ =)$",
  "?[()->(int64$ =)$",
  "10 $ ()->(int64$ =)",
  "?10 $ ()->(int64$ =)",
  "?(10 $ ()->(int64$ =))",
  "$10 $ ()->(int64$ =)",
  "=10 $ ()->(int64$ =)",
  "$[10 $ ()->(int64$ =)]",
  "xxx[10 $ ()->(int64$ =)]",
  "?$10 $ ()->(int64$ =)]",
  "?=10 $ ()->(int64$ =)]",
  "?[10 $ ()->(int64$ =)$",
  "?[10 $ ()->(int64$ =)$",
  "10 = ()->(int64$ =)",
  "?10 = ()->(int64$ =)",
  "?(10 = ()->(int64$ =))",
  "$10 = ()->(int64$ =)",
  "=10 = ()->(int64$ =)",
  "$[10 = ()->(int64$ =)]",
  "xxx[10 = ()->(int64$ =)]",
  "?$10 = ()->(int64$ =)]",
  "?=10 = ()->(int64$ =)]",
  "?[10 = ()->(int64$ =)$",
  "?[10 = ()->(int64$ =)$",
  "10 * (int64= =)-> Any",
  "?10 * (int64= =)-> Any",
  "?(10 * (int64= =)-> Any)",
  "$10 * (int64= =)-> Any",
  "=10 * (int64= =)-> Any",
  "$[10 * (int64= =)-> Any]",
  "xxx[10 * (int64= =)-> Any]",
  "?$10 * (int64= =)-> Any]",
  "?=10 * (int64= =)-> Any]",
  "?[10 * (int64= =)-> Any$",
  "?[10 * (int64= =)-> Any$",
  "(int64= =)-> Any",
  "?(int64= =)-> Any",
  "?((int64= =)-> Any)",
  "$(int64= =)-> Any",
  "=(int64= =)-> Any",
  "$[(int64= =)-> Any]",
  "xxx[(int64= =)-> Any]",
  "?$(int64= =)-> Any]",
  "?=(int64= =)-> Any]",
  "?[(int64= =)-> Any$",
  "?[(int64= =)-> Any$",
  "10 $ (int64= =)-> Any",
  "?10 $ (int64= =)-> Any",
  "?(10 $ (int64= =)-> Any)",
  "$10 $ (int64= =)-> Any",
  "=10 $ (int64= =)-> Any",
  "$[10 $ (int64= =)-> Any]",
  "xxx[10 $ (int64= =)-> Any]",
  "?$10 $ (int64= =)-> Any]",
  "?=10 $ (int64= =)-> Any]",
  "?[10 $ (int64= =)-> Any$",
  "?[10 $ (int64= =)-> Any$",
  "10 = (int64= =)-> Any",
  "?10 = (int64= =)-> Any",
  "?(10 = (int64= =)-> Any)",
  "$10 = (int64= =)-> Any",
  "=10 = (int64= =)-> Any",
  "$[10 = (int64= =)-> Any]",
  "xxx[10 = (int64= =)-> Any]",
  "?$10 = (int64= =)-> Any]",
  "?=10 = (int64= =)-> Any]",
  "?[10 = (int64= =)-> Any$",
  "?[10 = (int64= =)-> Any$",
  "10 * ()->(int64= =)",
  "?10 * ()->(int64= =)",
  "?(10 * ()->(int64= =))",
  "$10 * ()->(int64= =)",
  "=10 * ()->(int64= =)",
  "$[10 * ()->(int64= =)]",
  "xxx[10 * ()->(int64= =)]",
  "?$10 * ()->(int64= =)]",
  "?=10 * ()->(int64= =)]",
  "?[10 * ()->(int64= =)$",
  "?[10 * ()->(int64= =)$",
  "()->(int64= =)",
  "?()->(int64= =)",
  "?(()->(int64= =))",
  "$()->(int64= =)",
  "=()->(int64= =)",
  "$[()->(int64= =)]",
  "xxx[()->(int64= =)]",
  "?$()->(int64= =)]",
  "?=()->(int64= =)]",
  "?[()->(int64= =)$",
  "?[()->(int64= =)$",
  "10 $ ()->(int64= =)",
  "?10 $ ()->(int64= =)",
  "?(10 $ ()->(int64= =))",
  "$10 $ ()->(int64= =)",
  "=10 $ ()->(int64= =)",
  "$[10 $ ()->(int64= =)]",
  "xxx[10 $ ()->(int64= =)]",
  "?$10 $ ()->(int64= =)]",
  "?=10 $ ()->(int64= =)]",
  "?[10 $ ()->(int64= =)$",
  "?[10 $ ()->(int64= =)$",
  "10 = ()->(int64= =)",
  "?10 = ()->(int64= =)",
  "?(10 = ()->(int64= =))",
  "$10 = ()->(int64= =)",
  "=10 = ()->(int64= =)",
  "$[10 = ()->(int64= =)]",
  "xxx[10 = ()->(int64= =)]",
  "?$10 = ()->(int64= =)]",
  "?=10 = ()->(int64= =)]",
  "?[10 = ()->(int64= =)$",
  "?[10 = ()->(int64= =)$",
  "10 * (int64, int64, =)-> Any",
  "?10 * (int64, int64, =)-> Any",
  "?(10 * (int64, int64, =)-> Any)",
  "$10 * (int64, int64, =)-> Any",
  "=10 * (int64, int64, =)-> Any",
  "$[10 * (int64, int64, =)-> Any]",
  "xxx[10 * (int64, int64, =)-> Any]",
  "?$10 * (int64, int64, =)-> Any]",
  "?=10 * (int64, int64, =)-> Any]",
  "?[10 * (int64, int64, =)-> Any$",
  "?[10 * (int64, int64, =)-> Any$",
  "(int64, int64, =)-> Any",
  "?(int64, int64, =)-> Any",
  "?((int64, int64, =)-> Any)",
  "$(int64, int64, =)-> Any",
  "=(int64, int64, =)-> Any",
  "$[(int64, int64, =)-> Any]",
  "xxx[(int64, int64, =)-> Any]",
  "?$(int64, int64, =)-> Any]",
  "?=(int64, int64, =)-> Any]",
  "?[(int64, int64, =)-> Any$",
  "?[(int64, int64, =)-> Any$",
  "10 $ (int64, int64, =)-> Any",
  "?10 $ (int64, int64, =)-> Any",
  "?(10 $ (int64, int64, =)-> Any)",
  "$10 $ (int64, int64, =)-> Any",
  "=10 $ (int64, int64, =)-> Any",
  "$[10 $ (int64, int64, =)-> Any]",
  "xxx[10 $ (int64, int64, =)-> Any]",
  "?$10 $ (int64, int64, =)-> Any]",
  "?=10 $ (int64, int64, =)-> Any]",
  "?[10 $ (int64, int64, =)-> Any$",
  "?[10 $ (int64, int64, =)-> Any$",
  "10 = (int64, int64, =)-> Any",
  "?10 = (int64, int64, =)-> Any",
  "?(10 = (int64, int64, =)-> Any)",
  "$10 = (int64, int64, =)-> Any",
  "=10 = (int64, int64, =)-> Any",
  "$[10 = (int64, int64, =)-> Any]",
  "xxx[10 = (int64, int64, =)-> Any]",
  "?$10 = (int64, int64, =)-> Any]",
  "?=10 = (int64, int64, =)-> Any]",
  "?[10 = (int64, int64, =)-> Any$",
  "?[10 = (int64, int64, =)-> Any$",
  "10 * ()->(int64, int64, =)",
  "?10 * ()->(int64, int64, =)",
  "?(10 * ()->(int64, int64, =))",
  "$10 * ()->(int64, int64, =)",
  "=10 * ()->(int64, int64, =)",
  "$[10 * ()->(int64, int64, =)]",
  "xxx[10 * ()->(int64, int64, =)]",
  "?$10 * ()->(int64, int64, =)]",
  "?=10 * ()->(int64, int64, =)]",
  "?[10 * ()->(int64, int64, =)$",
  "?[10 * ()->(int64, int64, =)$",
  "()->(int64, int64, =)",
  "?()->(int64, int64, =)",
  "?(()->(int64, int64, =))",
  "$()->(int64, int64, =)",
  "=()->(int64, int64, =)",
  "$[()->(int64, int64, =)]",
  "xxx[()->(int64, int64, =)]",
  "?$()->(int64, int64, =)]",
  "?=()->(int64, int64, =)]",
  "?[()->(int64, int64, =)$",
  "?[()->(int64, int64, =)$",
  "10 $ ()->(int64, int64, =)",
  "?10 $ ()->(int64, int64, =)",
  "?(10 $ ()->(int64, int64, =))",
  "$10 $ ()->(int64, int64, =)",
  "=10 $ ()->(int64, int64, =)",
  "$[10 $ ()->(int64, int64, =)]",
  "xxx[10 $ ()->(int64, int64, =)]",
  "?$10 $ ()->(int64, int64, =)]",
  "?=10 $ ()->(int64, int64, =)]",
  "?[10 $ ()->(int64, int64, =)$",
  "?[10 $ ()->(int64, int64, =)$",
  "10 = ()->(int64, int64, =)",
  "?10 = ()->(int64, int64, =)",
  "?(10 = ()->(int64, int64, =))",
  "$10 = ()->(int64, int64, =)",
  "=10 = ()->(int64, int64, =)",
  "$[10 = ()->(int64, int64, =)]",
  "xxx[10 = ()->(int64, int64, =)]",
  "?$10 = ()->(int64, int64, =)]",
  "?=10 = ()->(int64, int64, =)]",
  "?[10 = ()->(int64, int64, =)$",
  "?[10 = ()->(int64, int64, =)$",
  "10 * (int64, int64$ =)-> Any",
  "?10 * (int64, int64$ =)-> Any",
  "?(10 * (int64, int64$ =)-> Any)",
  "$10 * (int64, int64$ =)-> Any",
  "=10 * (int64, int64$ =)-> Any",
  "$[10 * (int64, int64$ =)-> Any]",
  "xxx[10 * (int64, int64$ =)-> Any]",
  "?$10 * (int64, int64$ =)-> Any]",
  "?=10 * (int64, int64$ =)-> Any]",
  "?[10 * (int64, int64$ =)-> Any$",
  "?[10 * (int64, int64$ =)-> Any$",
  "(int64, int64$ =)-> Any",
  "?(int64, int64$ =)-> Any",
  "?((int64, int64$ =)-> Any)",
  "$(int64, int64$ =)-> Any",
  "=(int64, int64$ =)-> Any",
  "$[(int64, int64$ =)-> Any]",
  "xxx[(int64, int64$ =)-> Any]",
  "?$(int64, int64$ =)-> Any]",
  "?=(int64, int64$ =)-> Any]",
  "?[(int64, int64$ =)-> Any$",
  "?[(int64, int64$ =)-> Any$",
  "10 $ (int64, int64$ =)-> Any",
  "?10 $ (int64, int64$ =)-> Any",
  "?(10 $ (int64, int64$ =)-> Any)",
  "$10 $ (int64, int64$ =)-> Any",
  "=10 $ (int64, int64$ =)-> Any",
  "$[10 $ (int64, int64$ =)-> Any]",
  "xxx[10 $ (int64, int64$ =)-> Any]",
  "?$10 $ (int64, int64$ =)-> Any]",
  "?=10 $ (int64, int64$ =)-> Any]",
  "?[10 $ (int64, int64$ =)-> Any$",
  "?[10 $ (int64, int64$ =)-> Any$",
  "10 = (int64, int64$ =)-> Any",
  "?10 = (int64, int64$ =)-> Any",
  "?(10 = (int64, int64$ =)-> Any)",
  "$10 = (int64, int64$ =)-> Any",
  "=10 = (int64, int64$ =)-> Any",
  "$[10 = (int64, int64$ =)-> Any]",
  "xxx[10 = (int64, int64$ =)-> Any]",
  "?$10 = (int64, int64$ =)-> Any]",
  "?=10 = (int64, int64$ =)-> Any]",
  "?[10 = (int64, int64$ =)-> Any$",
  "?[10 = (int64, int64$ =)-> Any$",
  "10 * ()->(int64, int64$ =)",
  "?10 * ()->(int64, int64$ =)",
  "?(10 * ()->(int64, int64$ =))",
  "$10 * ()->(int64, int64$ =)",
  "=10 * ()->(int64, int64$ =)",
  "$[10 * ()->(int64, int64$ =)]",
  "xxx[10 * ()->(int64, int64$ =)]",
  "?$10 * ()->(int64, int64$ =)]",
  "?=10 * ()->(int64, int64$ =)]",
  "?[10 * ()->(int64, int64$ =)$",
  "?[10 * ()->(int64, int64$ =)$",
  "()->(int64, int64$ =)",
  "?()->(int64, int64$ =)",
  "?(()->(int64, int64$ =))",
  "$()->(int64, int64$ =)",
  "=()->(int64, int64$ =)",
  "$[()->(int64, int64$ =)]",
  "xxx[()->(int64, int64$ =)]",
  "?$()->(int64, int64$ =)]",
  "?=()->(int64, int64$ =)]",
  "?[()->(int64, int64$ =)$",
  "?[()->(int64, int64$ =)$",
  "10 $ ()->(int64, int64$ =)",
  "?10 $ ()->(int64, int64$ =)",
  "?(10 $ ()->(int64, int64$ =))",
  "$10 $ ()->(int64, int64$ =)",
  "=10 $ ()->(int64, int64$ =)",
  "$[10 $ ()->(int64, int64$ =)]",
  "xxx[10 $ ()->(int64, int64$ =)]",
  "?$10 $ ()->(int64, int64$ =)]",
  "?=10 $ ()->(int64, int64$ =)]",
  "?[10 $ ()->(int64, int64$ =)$",
  "?[10 $ ()->(int64, int64$ =)$",
  "10 = ()->(int64, int64$ =)",
  "?10 = ()->(int64, int64$ =)",
  "?(10 = ()->(int64, int64$ =))",
  "$10 = ()->(int64, int64$ =)",
  "=10 = ()->(int64, int64$ =)",
  "$[10 = ()->(int64, int64$ =)]",
  "xxx[10 = ()->(int64, int64$ =)]",
  "?$10 = ()->(int64, int64$ =)]",
  "?=10 = ()->(int64, int64$ =)]",
  "?[10 = ()->(int64, int64$ =)$",
  "?[10 = ()->(int64, int64$ =)$",
  "10 * (int64, int64= =)-> Any",
  "?10 * (int64, int64= =)-> Any",
  "?(10 * (int64, int64= =)-> Any)",
  "$10 * (int64, int64= =)-> Any",
  "=10 * (int64, int64= =)-> Any",
  "$[10 * (int64, int64= =)-> Any]",
  "xxx[10 * (int64, int64= =)-> Any]",
  "?$10 * (int64, int64= =)-> Any]",
  "?=10 * (int64, int64= =)-> Any]",
  "?[10 * (int64, int64= =)-> Any$",
  "?[10 * (int64, int64= =)-> Any$",
  "(int64, int64= =)-> Any",
  "?(int64, int64= =)-> Any",
  "?((int64, int64= =)-> Any)",
  "$(int64, int64= =)-> Any",
  "=(int64, int64= =)-> Any",
  "$[(int64, int64= =)-> Any]",
  "xxx[(int64, int64= =)-> Any]",
  "?$(int64, int64= =)-> Any]",
  "?=(int64, int64= =)-> Any]",
  "?[(int64, int64= =)-> Any$",
  "?[(int64, int64= =)-> Any$",
  "10 $ (int64, int64= =)-> Any",
  "?10 $ (int64, int64= =)-> Any",
  "?(10 $ (int64, int64= =)-> Any)",
  "$10 $ (int64, int64= =)-> Any",
  "=10 $ (int64, int64= =)-> Any",
  "$[10 $ (int64, int64= =)-> Any]",
  "xxx[10 $ (int64, int64= =)-> Any]",
  "?$10 $ (int64, int64= =)-> Any]",
  "?=10 $ (int64, int64= =)-> Any]",
  "?[10 $ (int64, int64= =)-> Any$",
  "?[10 $ (int64, int64= =)-> Any$",
  "10 = (int64, int64= =)-> Any",
  "?10 = (int64, int64= =)-> Any",
  "?(10 = (int64, int64= =)-> Any)",
  "$10 = (int64, int64= =)-> Any",
  "=10 = (int64, int64= =)-> Any",
  "$[10 = (int64, int64= =)-> Any]",
  "xxx[10 = (int64, int64= =)-> Any]",
  "?$10 = (int64, int64= =)-> Any]",
  "?=10 = (int64, int64= =)-> Any]",
  "?[10 = (int64, int64= =)-> Any$",
  "?[10 = (int64, int64= =)-> Any$",
  "10 * ()->(int64, int64= =)",
  "?10 * ()->(int64, int64= =)",
  "?(10 * ()->(int64, int64= =))",
  "$10 * ()->(int64, int64= =)",
  "=10 * ()->(int64, int64= =)",
  "$[10 * ()->(int64, int64= =)]",
  "xxx[10 * ()->(int64, int64= =)]",
  "?$10 * ()->(int64, int64= =)]",
  "?=10 * ()->(int64, int64= =)]",
  "?[10 * ()->(int64, int64= =)$",
  "?[10 * ()->(int64, int64= =)$",
  "()->(int64, int64= =)",
  "?()->(int64, int64= =)",
  "?(()->(int64, int64= =))",
  "$()->(int64, int64= =)",
  "=()->(int64, int64= =)",
  "$[()->(int64, int64= =)]",
  "xxx[()->(int64, int64= =)]",
  "?$()->(int64, int64= =)]",
  "?=()->(int64, int64= =)]",
  "?[()->(int64, int64= =)$",
  "?[()->(int64, int64= =)$",
  "10 $ ()->(int64, int64= =)",
  "?10 $ ()->(int64, int64= =)",
  "?(10 $ ()->(int64, int64= =))",
  "$10 $ ()->(int64, int64= =)",
  "=10 $ ()->(int64, int64= =)",
  "$[10 $ ()->(int64, int64= =)]",
  "xxx[10 $ ()->(int64, int64= =)]",
  "?$10 $ ()->(int64, int64= =)]",
  "?=10 $ ()->(int64, int64= =)]",
  "?[10 $ ()->(int64, int64= =)$",
  "?[10 $ ()->(int64, int64= =)$",
  "10 = ()->(int64, int64= =)",
  "?10 = ()->(int64, int64= =)",
  "?(10 = ()->(int64, int64= =))",
  "$10 = ()->(int64, int64= =)",
  "=10 = ()->(int64, int64= =)",
  "$[10 = ()->(int64, int64= =)]",
  "xxx[10 = ()->(int64, int64= =)]",
  "?$10 = ()->(int64, int64= =)]",
  "?=10 = ()->(int64, int64= =)]",
  "?[10 = ()->(int64, int64= =)$",
  "?[10 = ()->(int64, int64= =)$",
  "$10 * (Any)-> Any",
  "=10 * (Any)-> Any",
  "$[10 * (Any)-> Any]",
  "xxx[10 * (Any)-> Any]",
  "?$10 * (Any)-> Any]",
  "?=10 * (Any)-> Any]",
  "?[10 * (Any)-> Any$",
  "?[10 * (Any)-> Any$",
  "$(Any)-> Any",
  "=(Any)-> Any",
  "$[(Any)-> Any]",
  "xxx[(Any)-> Any]",
  "?$(Any)-> Any]",
  "?=(Any)-> Any]",
  "?[(Any)-> Any$",
  "?[(Any)-> Any$",
  "10 $ (Any)-> Any",
  "?10 $ (Any)-> Any",
  "?(10 $ (Any)-> Any)",
  "$10 $ (Any)-> Any",
  "=10 $ (Any)-> Any",
  "$[10 $ (Any)-> Any]",
  "xxx[10 $ (Any)-> Any]",
  "?$10 $ (Any)-> Any]",
  "?=10 $ (Any)-> Any]",
  "?[10 $ (Any)-> Any$",
  "?[10 $ (Any)-> Any$",
  "10 = (Any)-> Any",
  "?10 = (Any)-> Any",
  "?(10 = (Any)-> Any)",
  "$10 = (Any)-> Any",
  "=10 = (Any)-> Any",
  "$[10 = (Any)-> Any]",
  "xxx[10 = (Any)-> Any]",
  "?$10 = (Any)-> Any]",
  "?=10 = (Any)-> Any]",
  "?[10 = (Any)-> Any$",
  "?[10 = (Any)-> Any$",
  "$10 * ()->(Any)",
  "=10 * ()->(Any)",
  "$[10 * ()->(Any)]",
  "xxx[10 * ()->(Any)]",
  "?$10 * ()->(Any)]",
  "?=10 * ()->(Any)]",
  "?[10 * ()->(Any)$",
  "?[10 * ()->(Any)$",
  "$()->(Any)",
  "=()->(Any)",
  "$[()->(Any)]",
  "xxx[()->(Any)]",
  "?$()->(Any)]",
  "?=()->(Any)]",
  "?[()->(Any)$",
  "?[()->(Any)$",
  "10 $ ()->(Any)",
  "?10 $ ()->(Any)",
  "?(10 $ ()->(Any))",
  "$10 $ ()->(Any)",
  "=10 $ ()->(Any)",
  "$[10 $ ()->(Any)]",
  "xxx[10 $ ()->(Any)]",
  "?$10 $ ()->(Any)]",
  "?=10 $ ()->(Any)]",
  "?[10 $ ()->(Any)$",
  "?[10 $ ()->(Any)$",
  "10 = ()->(Any)",
  "?10 = ()->(Any)",
  "?(10 = ()->(Any))",
  "$10 = ()->(Any)",
  "=10 = ()->(Any)",
  "$[10 = ()->(Any)]",
  "xxx[10 = ()->(Any)]",
  "?$10 = ()->(Any)]",
  "?=10 = ()->(Any)]",
  "?[10 = ()->(Any)$",
  "?[10 = ()->(Any)$",
  "$10 * (Any, )-> Any",
  "=10 * (Any, )-> Any",
  "$[10 * (Any, )-> Any]",
  "xxx[10 * (Any, )-> Any]",
  "?$10 * (Any, )-> Any]",
  "?=10 * (Any, )-> Any]",
  "?[10 * (Any, )-> Any$",
  "?[10 * (Any, )-> Any$",
  "$(Any, )-> Any",
  "=(Any, )-> Any",
  "$[(Any, )-> Any]",
  "xxx[(Any, )-> Any]",
  "?$(Any, )-> Any]",
  "?=(Any, )-> Any]",
  "?[(Any, )-> Any$",
  "?[(Any, )-> Any$",
  "10 $ (Any, )-> Any",
  "?10 $ (Any, )-> Any",
  "?(10 $ (Any, )-> Any)",
  "$10 $ (Any, )-> Any",
  "=10 $ (Any, )-> Any",
  "$[10 $ (Any, )-> Any]",
  "xxx[10 $ (Any, )-> Any]",
  "?$10 $ (Any, )-> Any]",
  "?=10 $ (Any, )-> Any]",
  "?[10 $ (Any, )-> Any$",
  "?[10 $ (Any, )-> Any$",
  "10 = (Any, )-> Any",
  "?10 = (Any, )-> Any",
  "?(10 = (Any, )-> Any)",
  "$10 = (Any, )-> Any",
  "=10 = (Any, )-> Any",
  "$[10 = (Any, )-> Any]",
  "xxx[10 = (Any, )-> Any]",
  "?$10 = (Any, )-> Any]",
  "?=10 = (Any, )-> Any]",
  "?[10 = (Any, )-> Any$",
  "?[10 = (Any, )-> Any$",
  "$10 * ()->(Any, )",
  "=10 * ()->(Any, )",
  "$[10 * ()->(Any, )]",
  "xxx[10 * ()->(Any, )]",
  "?$10 * ()->(Any, )]",
  "?=10 * ()->(Any, )]",
  "?[10 * ()->(Any, )$",
  "?[10 * ()->(Any, )$",
  "$()->(Any, )",
  "=()->(Any, )",
  "$[()->(Any, )]",
  "xxx[()->(Any, )]",
  "?$()->(Any, )]",
  "?=()->(Any, )]",
  "?[()->(Any, )$",
  "?[()->(Any, )$",
  "10 $ ()->(Any, )",
  "?10 $ ()->(Any, )",
  "?(10 $ ()->(Any, ))",
  "$10 $ ()->(Any, )",
  "=10 $ ()->(Any, )",
  "$[10 $ ()->(Any, )]",
  "xxx[10 $ ()->(Any, )]",
  "?$10 $ ()->(Any, )]",
  "?=10 $ ()->(Any, )]",
  "?[10 $ ()->(Any, )$",
  "?[10 $ ()->(Any, )$",
  "10 = ()->(Any, )",
  "?10 = ()->(Any, )",
  "?(10 = ()->(Any, ))",
  "$10 = ()->(Any, )",
  "=10 = ()->(Any, )",
  "$[10 = ()->(Any, )]",
  "xxx[10 = ()->(Any, )]",
  "?$10 = ()->(Any, )]",
  "?=10 = ()->(Any, )]",
  "?[10 = ()->(Any, )$",
  "?[10 = ()->(Any, )$",
  "$10 * (Any, ...)-> Any",
  "=10 * (Any, ...)-> Any",
  "$[10 * (Any, ...)-> Any]",
  "xxx[10 * (Any, ...)-> Any]",
  "?$10 * (Any, ...)-> Any]",
  "?=10 * (Any, ...)-> Any]",
  "?[10 * (Any, ...)-> Any$",
  "?[10 * (Any, ...)-> Any$",
  "$(Any, ...)-> Any",
  "=(Any, ...)-> Any",
  "$[(Any, ...)-> Any]",
  "xxx[(Any, ...)-> Any]",
  "?$(Any, ...)-> Any]",
  "?=(Any, ...)-> Any]",
  "?[(Any, ...)-> Any$",
  "?[(Any, ...)-> Any$",
  "10 $ (Any, ...)-> Any",
  "?10 $ (Any, ...)-> Any",
  "?(10 $ (Any, ...)-> Any)",
  "$10 $ (Any, ...)-> Any",
  "=10 $ (Any, ...)-> Any",
  "$[10 $ (Any, ...)-> Any]",
  "xxx[10 $ (Any, ...)-> Any]",
  "?$10 $ (Any, ...)-> Any]",
  "?=10 $ (Any, ...)-> Any]",
  "?[10 $ (Any, ...)-> Any$",
  "?[10 $ (Any, ...)-> Any$",
  "10 = (Any, ...)-> Any",
  "?10 = (Any, ...)-> Any",
  "?(10 = (Any, ...)-> Any)",
  "$10 = (Any, ...)-> Any",
  "=10 = (Any, ...)-> Any",
  "$[10 = (Any, ...)-> Any]",
  "xxx[10 = (Any, ...)-> Any]",
  "?$10 = (Any, ...)-> Any]",
  "?=10 = (Any, ...)-> Any]",
  "?[10 = (Any, ...)-> Any$",
  "?[10 = (Any, ...)-> Any$",
  "$10 * ()->(Any, ...)",
  "=10 * ()->(Any, ...)",
  "$[10 * ()->(Any, ...)]",
  "xxx[10 * ()->(Any, ...)]",
  "?$10 * ()->(Any, ...)]",
  "?=10 * ()->(Any, ...)]",
  "?[10 * ()->(Any, ...)$",
  "?[10 * ()->(Any, ...)$",
  "$()->(Any, ...)",
  "=()->(Any, ...)",
  "$[()->(Any, ...)]",
  "xxx[()->(Any, ...)]",
  "?$()->(Any, ...)]",
  "?=()->(Any, ...)]",
  "?[()->(Any, ...)$",
  "?[()->(Any, ...)$",
  "10 $ ()->(Any, ...)",
  "?10 $ ()->(Any, ...)",
  "?(10 $ ()->(Any, ...))",
  "$10 $ ()->(Any, ...)",
  "=10 $ ()->(Any, ...)",
  "$[10 $ ()->(Any, ...)]",
  "xxx[10 $ ()->(Any, ...)]",
  "?$10 $ ()->(Any, ...)]",
  "?=10 $ ()->(Any, ...)]",
  "?[10 $ ()->(Any, ...)$",
  "?[10 $ ()->(Any, ...)$",
  "10 = ()->(Any, ...)",
  "?10 = ()->(Any, ...)",
  "?(10 = ()->(Any, ...))",
  "$10 = ()->(Any, ...)",
  "=10 = ()->(Any, ...)",
  "$[10 = ()->(Any, ...)]",
  "xxx[10 = ()->(Any, ...)]",
  "?$10 = ()->(Any, ...)]",
  "?=10 = ()->(Any, ...)]",
  "?[10 = ()->(Any, ...)$",
  "?[10 = ()->(Any, ...)$",
  "10 * (Any, $)-> Any",
  "?10 * (Any, $)-> Any",
  "?(10 * (Any, $)-> Any)",
  "$10 * (Any, $)-> Any",
  "=10 * (Any, $)-> Any",
  "$[10 * (Any, $)-> Any]",
  "xxx[10 * (Any, $)-> Any]",
  "?$10 * (Any, $)-> Any]",
  "?=10 * (Any, $)-> Any]",
  "?[10 * (Any, $)-> Any$",
  "?[10 * (Any, $)-> Any$",
  "(Any, $)-> Any",
  "?(Any, $)-> Any",
  "?((Any, $)-> Any)",
  "$(Any, $)-> Any",
  "=(Any, $)-> Any",
  "$[(Any, $)-> Any]",
  "xxx[(Any, $)-> Any]",
  "?$(Any, $)-> Any]",
  "?=(Any, $)-> Any]",
  "?[(Any, $)-> Any$",
  "?[(Any, $)-> Any$",
  "10 $ (Any, $)-> Any",
  "?10 $ (Any, $)-> Any",
  "?(10 $ (Any, $)-> Any)",
  "$10 $ (Any, $)-> Any",
  "=10 $ (Any, $)-> Any",
  "$[10 $ (Any, $)-> Any]",
  "xxx[10 $ (Any, $)-> Any]",
  "?$10 $ (Any, $)-> Any]",
  "?=10 $ (Any, $)-> Any]",
  "?[10 $ (Any, $)-> Any$",
  "?[10 $ (Any, $)-> Any$",
  "10 = (Any, $)-> Any",
  "?10 = (Any, $)-> Any",
  "?(10 = (Any, $)-> Any)",
  "$10 = (Any, $)-> Any",
  "=10 = (Any, $)-> Any",
  "$[10 = (Any, $)-> Any]",
  "xxx[10 = (Any, $)-> Any]",
  "?$10 = (Any, $)-> Any]",
  "?=10 = (Any, $)-> Any]",
  "?[10 = (Any, $)-> Any$",
  "?[10 = (Any, $)-> Any$",
  "10 * ()->(Any, $)",
  "?10 * ()->(Any, $)",
  "?(10 * ()->(Any, $))",
  "$10 * ()->(Any, $)",
  "=10 * ()->(Any, $)",
  "$[10 * ()->(Any, $)]",
  "xxx[10 * ()->(Any, $)]",
  "?$10 * ()->(Any, $)]",
  "?=10 * ()->(Any, $)]",
  "?[10 * ()->(Any, $)$",
  "?[10 * ()->(Any, $)$",
  "()->(Any, $)",
  "?()->(Any, $)",
  "?(()->(Any, $))",
  "$()->(Any, $)",
  "=()->(Any, $)",
  "$[()->(Any, $)]",
  "xxx[()->(Any, $)]",
  "?$()->(Any, $)]",
  "?=()->(Any, $)]",
  "?[()->(Any, $)$",
  "?[()->(Any, $)$",
  "10 $ ()->(Any, $)",
  "?10 $ ()->(Any, $)",
  "?(10 $ ()->(Any, $))",
  "$10 $ ()->(Any, $)",
  "=10 $ ()->(Any, $)",
  "$[10 $ ()->(Any, $)]",
  "xxx[10 $ ()->(Any, $)]",
  "?$10 $ ()->(Any, $)]",
  "?=10 $ ()->(Any, $)]",
  "?[10 $ ()->(Any, $)$",
  "?[10 $ ()->(Any, $)$",
  "10 = ()->(Any, $)",
  "?10 = ()->(Any, $)",
  "?(10 = ()->(Any, $))",
  "$10 = ()->(Any, $)",
  "=10 = ()->(Any, $)",
  "$[10 = ()->(Any, $)]",
  "xxx[10 = ()->(Any, $)]",
  "?$10 = ()->(Any, $)]",
  "?=10 = ()->(Any, $)]",
  "?[10 = ()->(Any, $)$",
  "?[10 = ()->(Any, $)$",
  "$10 * (a: Any, ...) -> Any",
  "=10 * (a: Any, ...) -> Any",
  "$[10 * (a: Any, ...) -> Any]",
  "xxx[10 * (a: Any, ...) -> Any]",
  "?$10 * (a: Any, ...) -> Any]",
  "?=10 * (a: Any, ...) -> Any]",
  "?[10 * (a: Any, ...) -> Any$",
  "?[10 * (a: Any, ...) -> Any$",
  "$(a: Any, ...) -> Any",
  "=(a: Any, ...) -> Any",
  "$[(a: Any, ...) -> Any]",
  "xxx[(a: Any, ...) -> Any]",
  "?$(a: Any, ...) -> Any]",
  "?=(a: Any, ...) -> Any]",
  "?[(a: Any, ...) -> Any$",
  "?[(a: Any, ...) -> Any$",
  "10 $ (a: Any, ...) -> Any",
  "?10 $ (a: Any, ...) -> Any",
  "?(10 $ (a: Any, ...) -> Any)",
  "$10 $ (a: Any, ...) -> Any",
  "=10 $ (a: Any, ...) -> Any",
  "$[10 $ (a: Any, ...) -> Any]",
  "xxx[10 $ (a: Any, ...) -> Any]",
  "?$10 $ (a: Any, ...) -> Any]",
  "?=10 $ (a: Any, ...) -> Any]",
  "?[10 $ (a: Any, ...) -> Any$",
  "?[10 $ (a: Any, ...) -> Any$",
  "10 = (a: Any, ...) -> Any",
  "?10 = (a: Any, ...) -> Any",
  "?(10 = (a: Any, ...) -> Any)",
  "$10 = (a: Any, ...) -> Any",
  "=10 = (a: Any, ...) -> Any",
  "$[10 = (a: Any, ...) -> Any]",
  "xxx[10 = (a: Any, ...) -> Any]",
  "?$10 = (a: Any, ...) -> Any]",
  "?=10 = (a: Any, ...) -> Any]",
  "?[10 = (a: Any, ...) -> Any$",
  "?[10 = (a: Any, ...) -> Any$",
  "$10 * (x: int64, a: Any, ...) -> Any",
  "=10 * (x: int64, a: Any, ...) -> Any",
  "$[10 * (x: int64, a: Any, ...) -> Any]",
  "xxx[10 * (x: int64, a: Any, ...) -> Any]",
  "?$10 * (x: int64, a: Any, ...) -> Any]",
  "?=10 * (x: int64, a: Any, ...) -> Any]",
  "?[10 * (x: int64, a: Any, ...) -> Any$",
  "?[10 * (x: int64, a: Any, ...) -> Any$",
  "$(x: int64, a: Any, ...) -> Any",
  "=(x: int64, a: Any, ...) -> Any",
  "$[(x: int64, a: Any, ...) -> Any]",
  "xxx[(x: int64, a: Any, ...) -> Any]",
  "?$(x: int64, a: Any, ...) -> Any]",
  "?=(x: int64, a: Any, ...) -> Any]",
  "?[(x: int64, a: Any, ...) -> Any$",
  "?[(x: int64, a: Any, ...) -> Any$",
  "10 $ (x: int64, a: Any, ...) -> Any",
  "?10 $ (x: int64, a: Any, ...) -> Any",
  "?(10 $ (x: int64, a: Any, ...) -> Any)",
  "$10 $ (x: int64, a: Any, ...) -> Any",
  "=10 $ (x: int64, a: Any, ...) -> Any",
  "$[10 $ (x: int64, a: Any, ...) -> Any]",
  "xxx[10 $ (x: int64, a: Any, ...) -> Any]",
  "?$10 $ (x: int64, a: Any, ...) -> Any]",
  "?=10 $ (x: int64, a: Any, ...) -> Any]",
  "?[10 $ (x: int64, a: Any, ...) -> Any$",
  "?[10 $ (x: int64, a: Any, ...) -> Any$",
  "10 = (x: int64, a: Any, ...) -> Any",
  "?10 = (x: int64, a: Any, ...) -> Any",
  "?(10 = (x: int64, a: Any, ...) -> Any)",
  "$10 = (x: int64, a: Any, ...) -> Any",
  "=10 = (x: int64, a: Any, ...) -> Any",
  "$[10 = (x: int64, a: Any, ...) -> Any]",
  "xxx[10 = (x: int64, a: Any, ...) -> Any]",
  "?$10 = (x: int64, a: Any, ...) -> Any]",
  "?=10 = (x: int64, a: Any, ...) -> Any]",
  "?[10 = (x: int64, a: Any, ...) -> Any$",
  "?[10 = (x: int64, a: Any, ...) -> Any$",
  "10 * (x: int64$ a: Any, ...) -> Any",
  "?10 * (x: int64$ a: Any, ...) -> Any",
  "?(10 * (x: int64$ a: Any, ...) -> Any)",
  "$10 * (x: int64$ a: Any, ...) -> Any",
  "=10 * (x: int64$ a: Any, ...) -> Any",
  "$[10 * (x: int64$ a: Any, ...) -> Any]",
  "xxx[10 * (x: int64$ a: Any, ...) -> Any]",
  "?$10 * (x: int64$ a: Any, ...) -> Any]",
  "?=10 * (x: int64$ a: Any, ...) -> Any]",
  "?[10 * (x: int64$ a: Any, ...) -> Any$",
  "?[10 * (x: int64$ a: Any, ...) -> Any$",
  "(x: int64$ a: Any, ...) -> Any",
  "?(x: int64$ a: Any, ...) -> Any",
  "?((x: int64$ a: Any, ...) -> Any)",
  "$(x: int64$ a: Any, ...) -> Any",
  "=(x: int64$ a: Any, ...) -> Any",
  "$[(x: int64$ a: Any, ...) -> Any]",
  "xxx[(x: int64$ a: Any, ...) -> Any]",
  "?$(x: int64$ a: Any, ...) -> Any]",
  "?=(x: int64$ a: Any, ...) -> Any]",
  "?[(x: int64$ a: Any, ...) -> Any$",
  "?[(x: int64$ a: Any, ...) -> Any$",
  "10 $ (x: int64$ a: Any, ...) -> Any",
  "?10 $ (x: int64$ a: Any, ...) -> Any",
  "?(10 $ (x: int64$ a: Any, ...) -> Any)",
  "$10 $ (x: int64$ a: Any, ...) -> Any",
  "=10 $ (x: int64$ a: Any, ...) -> Any",
  "$[10 $ (x: int64$ a: Any, ...) -> Any]",
  "xxx[10 $ (x: int64$ a: Any, ...) -> Any]",
  "?$10 $ (x: int64$ a: Any, ...) -> Any]",
  "?=10 $ (x: int64$ a: Any, ...) -> Any]",
  "?[10 $ (x: int64$ a: Any, ...) -> Any$",
  "?[10 $ (x: int64$ a: Any, ...) -> Any$",
  "10 = (x: int64$ a: Any, ...) -> Any",
  "?10 = (x: int64$ a: Any, ...) -> Any",
  "?(10 = (x: int64$ a: Any, ...) -> Any)",
  "$10 = (x: int64$ a: Any, ...) -> Any",
  "=10 = (x: int64$ a: Any, ...) -> Any",
  "$[10 = (x: int64$ a: Any, ...) -> Any]",
  "xxx[10 = (x: int64$ a: Any, ...) -> Any]",
  "?$10 = (x: int64$ a: Any, ...) -> Any]",
  "?=10 = (x: int64$ a: Any, ...) -> Any]",
  "?[10 = (x: int64$ a: Any, ...) -> Any$",
  "?[10 = (x: int64$ a: Any, ...) -> Any$",
  "10 * (x: int64= a: Any, ...) -> Any",
  "?10 * (x: int64= a: Any, ...) -> Any",
  "?(10 * (x: int64= a: Any, ...) -> Any)",
  "$10 * (x: int64= a: Any, ...) -> Any",
  "=10 * (x: int64= a: Any, ...) -> Any",
  "$[10 * (x: int64= a: Any, ...) -> Any]",
  "xxx[10 * (x: int64= a: Any, ...) -> Any]",
  "?$10 * (x: int64= a: Any, ...) -> Any]",
  "?=10 * (x: int64= a: Any, ...) -> Any]",
  "?[10 * (x: int64= a: Any, ...) -> Any$",
  "?[10 * (x: int64= a: Any, ...) -> Any$",
  "(x: int64= a: Any, ...) -> Any",
  "?(x: int64= a: Any, ...) -> Any",
  "?((x: int64= a: Any, ...) -> Any)",
  "$(x: int64= a: Any, ...) -> Any",
  "=(x: int64= a: Any, ...) -> Any",
  "$[(x: int64= a: Any, ...) -> Any]",
  "xxx[(x: int64= a: Any, ...) -> Any]",
  "?$(x: int64= a: Any, ...) -> Any]",
  "?=(x: int64= a: Any, ...) -> Any]",
  "?[(x: int64= a: Any, ...) -> Any$",
  "?[(x: int64= a: Any, ...) -> Any$",
  "10 $ (x: int64= a: Any, ...) -> Any",
  "?10 $ (x: int64= a: Any, ...) -> Any",
  "?(10 $ (x: int64= a: Any, ...) -> Any)",
  "$10 $ (x: int64= a: Any, ...) -> Any",
  "=10 $ (x: int64= a: Any, ...) -> Any",
  "$[10 $ (x: int64= a: Any, ...) -> Any]",
  "xxx[10 $ (x: int64= a: Any, ...) -> Any]",
  "?$10 $ (x: int64= a: Any, ...) -> Any]",
  "?=10 $ (x: int64= a: Any, ...) -> Any]",
  "?[10 $ (x: int64= a: Any, ...) -> Any$",
  "?[10 $ (x: int64= a: Any, ...) -> Any$",
  "10 = (x: int64= a: Any, ...) -> Any",
  "?10 = (x: int64= a: Any, ...) -> Any",
  "?(10 = (x: int64= a: Any, ...) -> Any)",
  "$10 = (x: int64= a: Any, ...) -> Any",
  "=10 = (x: int64= a: Any, ...) -> Any",
  "$[10 = (x: int64= a: Any, ...) -> Any]",
  "xxx[10 = (x: int64= a: Any, ...) -> Any]",
  "?$10 = (x: int64= a: Any, ...) -> Any]",
  "?=10 = (x: int64= a: Any, ...) -> Any]",
  "?[10 = (x: int64= a: Any, ...) -> Any$",
  "?[10 = (x: int64= a: Any, ...) -> Any$",
  "$10 * (x: int64, x: int64, a: Any, ...) -> Any",
  "=10 * (x: int64, x: int64, a: Any, ...) -> Any",
  "$[10 * (x: int64, x: int64, a: Any, ...) -> Any]",
  "xxx[10 * (x: int64, x: int64, a: Any, ...) -> Any]",
  "?$10 * (x: int64, x: int64, a: Any, ...) -> Any]",
  "?=10 * (x: int64, x: int64, a: Any, ...) -> Any]",
  "?[10 * (x: int64, x: int64, a: Any, ...) -> Any$",
  "?[10 * (x: int64, x: int64, a: Any, ...) -> Any$",
  "$(x: int64, x: int64, a: Any, ...) -> Any",
  "=(x: int64, x: int64, a: Any, ...) -> Any",
  "$[(x: int64, x: int64, a: Any, ...) -> Any]",
  "xxx[(x: int64, x: int64, a: Any, ...) -> Any]",
  "?$(x: int64, x: int64, a: Any, ...) -> Any]",
  "?=(x: int64, x: int64, a: Any, ...) -> Any]",
  "?[(x: int64, x: int64, a: Any, ...) -> Any$",
  "?[(x: int64, x: int64, a: Any, ...) -> Any$",
  "10 $ (x: int64, x: int64, a: Any, ...) -> Any",
  "?10 $ (x: int64, x: int64, a: Any, ...) -> Any",
  "?(10 $ (x: int64, x: int64, a: Any, ...) -> Any)",
  "$10 $ (x: int64, x: int64, a: Any, ...) -> Any",
  "=10 $ (x: int64, x: int64, a: Any, ...) -> Any",
  "$[10 $ (x: int64, x: int64, a: Any, ...) -> Any]",
  "xxx[10 $ (x: int64, x: int64, a: Any, ...) -> Any]",
  "?$10 $ (x: int64, x: int64, a: Any, ...) -> Any]",
  "?=10 $ (x: int64, x: int64, a: Any, ...) -> Any]",
  "?[10 $ (x: int64, x: int64, a: Any, ...) -> Any$",
  "?[10 $ (x: int64, x: int64, a: Any, ...) -> Any$",
  "10 = (x: int64, x: int64, a: Any, ...) -> Any",
  "?10 = (x: int64, x: int64, a: Any, ...) -> Any",
  "?(10 = (x: int64, x: int64, a: Any, ...) -> Any)",
  "$10 = (x: int64, x: int64, a: Any, ...) -> Any",
  "=10 = (x: int64, x: int64, a: Any, ...) -> Any",
  "$[10 = (x: int64, x: int64, a: Any, ...) -> Any]",
  "xxx[10 = (x: int64, x: int64, a: Any, ...) -> Any]",
  "?$10 = (x: int64, x: int64, a: Any, ...) -> Any]",
  "?=10 = (x: int64, x: int64, a: Any, ...) -> Any]",
  "?[10 = (x: int64, x: int64, a: Any, ...) -> Any$",
  "?[10 = (x: int64, x: int64, a: Any, ...) -> Any$",
  "10 * (x: int64, x: int64$ a: Any, ...) -> Any",
  "?10 * (x: int64, x: int64$ a: Any, ...) -> Any",
  "?(10 * (x: int64, x: int64$ a: Any, ...) -> Any)",
  "$10 * (x: int64, x: int64$ a: Any, ...) -> Any",
  "=10 * (x: int64, x: int64$ a: Any, ...) -> Any",
  "$[10 * (x: int64, x: int64$ a: Any, ...) -> Any]",
  "xxx[10 * (x: int64, x: int64$ a: Any, ...) -> Any]",
  "?$10 * (x: int64, x: int64$ a: Any, ...) -> Any]",
  "?=10 * (x: int64, x: int64$ a: Any, ...) -> Any]",
  "?[10 * (x: int64, x: int64$ a: Any, ...) -> Any$",
  "?[10 * (x: int64, x: int64$ a: Any, ...) -> Any$",
  "(x: int64, x: int64$ a: Any, ...) -> Any",
  "?(x: int64, x: int64$ a: Any, ...) -> Any",
  "?((x: int64, x: int64$ a: Any, ...) -> Any)",
  "$(x: int64, x: int64$ a: Any, ...) -> Any",
  "=(x: int64, x: int64$ a: Any, ...) -> Any",
  "$[(x: int64, x: int64$ a: Any, ...) -> Any]",
  "xxx[(x: int64, x: int64$ a: Any, ...) -> Any]",
  "?$(x: int64, x: int64$ a: Any, ...) -> Any]",
  "?=(x: int64, x: int64$ a: Any, ...) -> Any]",
  "?[(x: int64, x: int64$ a: Any, ...) -> Any$",
  "?[(x: int64, x: int64$ a: Any, ...) -> Any$",
  "10 $ (x: int64, x: int64$ a: Any, ...) -> Any",
  "?10 $ (x: int64, x: int64$ a: Any, ...) -> Any",
  "?(10 $ (x: int64, x: int64$ a: Any, ...) -> Any)",
  "$10 $ (x: int64, x: int64$ a: Any, ...) -> Any",
  "=10 $ (x: int64, x: int64$ a: Any, ...) -> Any",
  "$[10 $ (x: int64, x: int64$ a: Any, ...) -> Any]",
  "xxx[10 $ (x: int64, x: int64$ a: Any, ...) -> Any]",
  "?$10 $ (x: int64, x: int64$ a: Any, ...) -> Any]",
  "?=10 $ (x: int64, x: int64$ a: Any, ...) -> Any]",
  "?[10 $ (x: int64, x: int64$ a: Any, ...) -> Any$",
  "?[10 $ (x: int64, x: int64$ a: Any, ...) -> Any$",
  "10 = (x: int64, x: int64$ a: Any, ...) -> Any",
  "?10 = (x: int64, x: int64$ a: Any, ...) -> Any",
  "?(10 = (x: int64, x: int64$ a: Any, ...) -> Any)",
  "$10 = (x: int64, x: int64$ a: Any, ...) -> Any",
  "=10 = (x: int64, x: int64$ a: Any, ...) -> Any",
  "$[10 = (x: int64, x: int64$ a: Any, ...) -> Any]",
  "xxx[10 = (x: int64, x: int64$ a: Any, ...) -> Any]",
  "?$10 = (x: int64, x: int64$ a: Any, ...) -> Any]",
  "?=10 = (x: int64, x: int64$ a: Any, ...) -> Any]",
  "?[10 = (x: int64, x: int64$ a: Any, ...) -> Any$",
  "?[10 = (x: int64, x: int64$ a: Any, ...) -> Any$",
  "10 * (x: int64, x: int64= a: Any, ...) -> Any",
  "?10 * (x: int64, x: int64= a: Any, ...) -> Any",
  "?(10 * (x: int64, x: int64= a: Any, ...) -> Any)",
  "$10 * (x: int64, x: int64= a: Any, ...) -> Any",
  "=10 * (x: int64, x: int64= a: Any, ...) -> Any",
  "$[10 * (x: int64, x: int64= a: Any, ...) -> Any]",
  "xxx[10 * (x: int64, x: int64= a: Any, ...) -> Any]",
  "?$10 * (x: int64, x: int64= a: Any, ...) -> Any]",
  "?=10 * (x: int64, x: int64= a: Any, ...) -> Any]",
  "?[10 * (x: int64, x: int64= a: Any, ...) -> Any$",
  "?[10 * (x: int64, x: int64= a: Any, ...) -> Any$",
  "(x: int64, x: int64= a: Any, ...) -> Any",
  "?(x: int64, x: int64= a: Any, ...) -> Any",
  "?((x: int64, x: int64= a: Any, ...) -> Any)",
  "$(x: int64, x: int64= a: Any, ...) -> Any",
  "=(x: int64, x: int64= a: Any, ...) -> Any",
  "$[(x: int64, x: int64= a: Any, ...) -> Any]",
  "xxx[(x: int64, x: int64= a: Any, ...) -> Any]",
  "?$(x: int64, x: int64= a: Any, ...) -> Any]",
  "?=(x: int64, x: int64= a: Any, ...) -> Any]",
  "?[(x: int64, x: int64= a: Any, ...) -> Any$",
  "?[(x: int64, x: int64= a: Any, ...) -> Any$",
  "10 $ (x: int64, x: int64= a: Any, ...) -> Any",
  "?10 $ (x: int64, x: int64= a: Any, ...) -> Any",
  "?(10 $ (x: int64, x: int64= a: Any, ...) -> Any)",
  "$10 $ (x: int64, x: int64= a: Any, ...) -> Any",
  "=10 $ (x: int64, x: int64= a: Any, ...) -> Any",
  "$[10 $ (x: int64, x: int64= a: Any, ...) -> Any]",
  "xxx[10 $ (x: int64, x: int64= a: Any, ...) -> Any]",
  "?$10 $ (x: int64, x: int64= a: Any, ...) -> Any]",
  "?=10 $ (x: int64, x: int64= a: Any, ...) -> Any]",
  "?[10 $ (x: int64, x: int64= a: Any, ...) -> Any$",
  "?[10 $ (x: int64, x: int64= a: Any, ...) -> Any$",
  "10 = (x: int64, x: int64= a: Any, ...) -> Any",
  "?10 = (x: int64, x: int64= a: Any, ...) -> Any",
  "?(10 = (x: int64, x: int64= a: Any, ...) -> Any)",
  "$10 = (x: int64, x: int64= a: Any, ...) -> Any",
  "=10 = (x: int64, x: int64= a: Any, ...) -> Any",
  "$[10 = (x: int64, x: int64= a: Any, ...) -> Any]",
  "xxx[10 = (x: int64, x: int64= a: Any, ...) -> Any]",
  "?$10 = (x: int64, x: int64= a: Any, ...) -> Any]",
  "?=10 = (x: int64, x: int64= a: Any, ...) -> Any]",
  "?[10 = (x: int64, x: int64= a: Any, ...) -> Any$",
  "?[10 = (x: int64, x: int64= a: Any, ...) -> Any$",
  "10 * ($: Any, ...) -> Any",
  "?10 * ($: Any, ...) -> Any",
  "?(10 * ($: Any, ...) -> Any)",
  "$10 * ($: Any, ...) -> Any",
  "=10 * ($: Any, ...) -> Any",
  "$[10 * ($: Any, ...) -> Any]",
  "xxx[10 * ($: Any, ...) -> Any]",
  "?$10 * ($: Any, ...) -> Any]",
  "?=10 * ($: Any, ...) -> Any]",
  "?[10 * ($: Any, ...) -> Any$",
  "?[10 * ($: Any, ...) -> Any$",
  "($: Any, ...) -> Any",
  "?($: Any, ...) -> Any",
  "?(($: Any, ...) -> Any)",
  "$($: Any, ...) -> Any",
  "=($: Any, ...) -> Any",
  "$[($: Any, ...) -> Any]",
  "xxx[($: Any, ...) -> Any]",
  "?$($: Any, ...) -> Any]",
  "?=($: Any, ...) -> Any]",
  "?[($: Any, ...) -> Any$",
  "?[($: Any, ...) -> Any$",
  "10 $ ($: Any, ...) -> Any",
  "?10 $ ($: Any, ...) -> Any",
  "?(10 $ ($: Any, ...) -> Any)",
  "$10 $ ($: Any, ...) -> Any",
  "=10 $ ($: Any, ...) -> Any",
  "$[10 $ ($: Any, ...) -> Any]",
  "xxx[10 $ ($: Any, ...) -> Any]",
  "?$10 $ ($: Any, ...) -> Any]",
  "?=10 $ ($: Any, ...) -> Any]",
  "?[10 $ ($: Any, ...) -> Any$",
  "?[10 $ ($: Any, ...) -> Any$",
  "10 = ($: Any, ...) -> Any",
  "?10 = ($: Any, ...) -> Any",
  "?(10 = ($: Any, ...) -> Any)",
  "$10 = ($: Any, ...) -> Any",
  "=10 = ($: Any, ...) -> Any",
  "$[10 = ($: Any, ...) -> Any]",
  "xxx[10 = ($: Any, ...) -> Any]",
  "?$10 = ($: Any, ...) -> Any]",
  "?=10 = ($: Any, ...) -> Any]",
  "?[10 = ($: Any, ...) -> Any$",
  "?[10 = ($: Any, ...) -> Any$",
  "10 * (x: int64, $: Any, ...) -> Any",
  "?10 * (x: int64, $: Any, ...) -> Any",
  "?(10 * (x: int64, $: Any, ...) -> Any)",
  "$10 * (x: int64, $: Any, ...) -> Any",
  "=10 * (x: int64, $: Any, ...) -> Any",
  "$[10 * (x: int64, $: Any, ...) -> Any]",
  "xxx[10 * (x: int64, $: Any, ...) -> Any]",
  "?$10 * (x: int64, $: Any, ...) -> Any]",
  "?=10 * (x: int64, $: Any, ...) -> Any]",
  "?[10 * (x: int64, $: Any, ...) -> Any$",
  "?[10 * (x: int64, $: Any, ...) -> Any$",
  "(x: int64, $: Any, ...) -> Any",
  "?(x: int64, $: Any, ...) -> Any",
  "?((x: int64, $: Any, ...) -> Any)",
  "$(x: int64, $: Any, ...) -> Any",
  "=(x: int64, $: Any, ...) -> Any",
  "$[(x: int64, $: Any, ...) -> Any]",
  "xxx[(x: int64, $: Any, ...) -> Any]",
  "?$(x: int64, $: Any, ...) -> Any]",
  "?=(x: int64, $: Any, ...) -> Any]",
  "?[(x: int64, $: Any, ...) -> Any$",
  "?[(x: int64, $: Any, ...) -> Any$",
  "10 $ (x: int64, $: Any, ...) -> Any",
  "?10 $ (x: int64, $: Any, ...) -> Any",
  "?(10 $ (x: int64, $: Any, ...) -> Any)",
  "$10 $ (x: int64, $: Any, ...) -> Any",
  "=10 $ (x: int64, $: Any, ...) -> Any",
  "$[10 $ (x: int64, $: Any, ...) -> Any]",
  "xxx[10 $ (x: int64, $: Any, ...) -> Any]",
  "?$10 $ (x: int64, $: Any, ...) -> Any]",
  "?=10 $ (x: int64, $: Any, ...) -> Any]",
  "?[10 $ (x: int64, $: Any, ...) -> Any$",
  "?[10 $ (x: int64, $: Any, ...) -> Any$",
  "10 = (x: int64, $: Any, ...) -> Any",
  "?10 = (x: int64, $: Any, ...) -> Any",
  "?(10 = (x: int64, $: Any, ...) -> Any)",
  "$10 = (x: int64, $: Any, ...) -> Any",
  "=10 = (x: int64, $: Any, ...) -> Any",
  "$[10 = (x: int64, $: Any, ...) -> Any]",
  "xxx[10 = (x: int64, $: Any, ...) -> Any]",
  "?$10 = (x: int64, $: Any, ...) -> Any]",
  "?=10 = (x: int64, $: Any, ...) -> Any]",
  "?[10 = (x: int64, $: Any, ...) -> Any$",
  "?[10 = (x: int64, $: Any, ...) -> Any$",
  "10 * (x: int64$ $: Any, ...) -> Any",
  "?10 * (x: int64$ $: Any, ...) -> Any",
  "?(10 * (x: int64$ $: Any, ...) -> Any)",
  "$10 * (x: int64$ $: Any, ...) -> Any",
  "=10 * (x: int64$ $: Any, ...) -> Any",
  "$[10 * (x: int64$ $: Any, ...) -> Any]",
  "xxx[10 * (x: int64$ $: Any, ...) -> Any]",
  "?$10 * (x: int64$ $: Any, ...) -> Any]",
  "?=10 * (x: int64$ $: Any, ...) -> Any]",
  "?[10 * (x: int64$ $: Any, ...) -> Any$",
  "?[10 * (x: int64$ $: Any, ...) -> Any$",
  "(x: int64$ $: Any, ...) -> Any",
  "?(x: int64$ $: Any, ...) -> Any",
  "?((x: int64$ $: Any, ...) -> Any)",
  "$(x: int64$ $: Any, ...) -> Any",
  "=(x: int64$ $: Any, ...) -> Any",
  "$[(x: int64$ $: Any, ...) -> Any]",
  "xxx[(x: int64$ $: Any, ...) -> Any]",
  "?$(x: int64$ $: Any, ...) -> Any]",
  "?=(x: int64$ $: Any, ...) -> Any]",
  "?[(x: int64$ $: Any, ...) -> Any$",
  "?[(x: int64$ $: Any, ...) -> Any$",
  "10 $ (x: int64$ $: Any, ...) -> Any",
  "?10 $ (x: int64$ $: Any, ...) -> Any",
  "?(10 $ (x: int64$ $: Any, ...) -> Any)",
  "$10 $ (x: int64$ $: Any, ...) -> Any",
  "=10 $ (x: int64$ $: Any, ...) -> Any",
  "$[10 $ (x: int64$ $: Any, ...) -> Any]",
  "xxx[10 $ (x: int64$ $: Any, ...) -> Any]",
  "?$10 $ (x: int64$ $: Any, ...) -> Any]",
  "?=10 $ (x: int64$ $: Any, ...) -> Any]",
  "?[10 $ (x: int64$ $: Any, ...) -> Any$",
  "?[10 $ (x: int64$ $: Any, ...) -> Any$",
  "10 = (x: int64$ $: Any, ...) -> Any",
  "?10 = (x: int64$ $: Any, ...) -> Any",
  "?(10 = (x: int64$ $: Any, ...) -> Any)",
  "$10 = (x: int64$ $: Any, ...) -> Any",
  "=10 = (x: int64$ $: Any, ...) -> Any",
  "$[10 = (x: int64$ $: Any, ...) -> Any]",
  "xxx[10 = (x: int64$ $: Any, ...) -> Any]",
  "?$10 = (x: int64$ $: Any, ...) -> Any]",
  "?=10 = (x: int64$ $: Any, ...) -> Any]",
  "?[10 = (x: int64$ $: Any, ...) -> Any$",
  "?[10 = (x: int64$ $: Any, ...) -> Any$",
  "10 * (x: int64= $: Any, ...) -> Any",
  "?10 * (x: int64= $: Any, ...) -> Any",
  "?(10 * (x: int64= $: Any, ...) -> Any)",
  "$10 * (x: int64= $: Any, ...) -> Any",
  "=10 * (x: int64= $: Any, ...) -> Any",
  "$[10 * (x: int64= $: Any, ...) -> Any]",
  "xxx[10 * (x: int64= $: Any, ...) -> Any]",
  "?$10 * (x: int64= $: Any, ...) -> Any]",
  "?=10 * (x: int64= $: Any, ...) -> Any]",
  "?[10 * (x: int64= $: Any, ...) -> Any$",
  "?[10 * (x: int64= $: Any, ...) -> Any$",
  "(x: int64= $: Any, ...) -> Any",
  "?(x: int64= $: Any, ...) -> Any",
  "?((x: int64= $: Any, ...) -> Any)",
  "$(x: int64= $: Any, ...) -> Any",
  "=(x: int64= $: Any, ...) -> Any",
  "$[(x: int64= $: Any, ...) -> Any]",
  "xxx[(x: int64= $: Any, ...) -> Any]",
  "?$(x: int64= $: Any, ...) -> Any]",
  "?=(x: int64= $: Any, ...) -> Any]",
  "?[(x: int64= $: Any, ...) -> Any$",
  "?[(x: int64= $: Any, ...) -> Any$",
  "10 $ (x: int64= $: Any, ...) -> Any",
  "?10 $ (x: int64= $: Any, ...) -> Any",
  "?(10 $ (x: int64= $: Any, ...) -> Any)",
  "$10 $ (x: int64= $: Any, ...) -> Any",
  "=10 $ (x: int64= $: Any, ...) -> Any",
  "$[10 $ (x: int64= $: Any, ...) -> Any]",
  "xxx[10 $ (x: int64= $: Any, ...) -> Any]",
  "?$10 $ (x: int64= $: Any, ...) -> Any]",
  "?=10 $ (x: int64= $: Any, ...) -> Any]",
  "?[10 $ (x: int64= $: Any, ...) -> Any$",
  "?[10 $ (x: int64= $: Any, ...) -> Any$",
  "10 = (x: int64= $: Any, ...) -> Any",
  "?10 = (x: int64= $: Any, ...) -> Any",
  "?(10 = (x: int64= $: Any, ...) -> Any)",
  "$10 = (x: int64= $: Any, ...) -> Any",
  "=10 = (x: int64= $: Any, ...) -> Any",
  "$[10 = (x: int64= $: Any, ...) -> Any]",
  "xxx[10 = (x: int64= $: Any, ...) -> Any]",
  "?$10 = (x: int64= $: Any, ...) -> Any]",
  "?=10 = (x: int64= $: Any, ...) -> Any]",
  "?[10 = (x: int64= $: Any, ...) -> Any$",
  "?[10 = (x: int64= $: Any, ...) -> Any$",
  "10 * (x: int64, x: int64, $: Any, ...) -> Any",
  "?10 * (x: int64, x: int64, $: Any, ...) -> Any",
  "?(10 * (x: int64, x: int64, $: Any, ...) -> Any)",
  "$10 * (x: int64, x: int64, $: Any, ...) -> Any",
  "=10 * (x: int64, x: int64, $: Any, ...) -> Any",
  "$[10 * (x: int64, x: int64, $: Any, ...) -> Any]",
  "xxx[10 * (x: int64, x: int64, $: Any, ...) -> Any]",
  "?$10 * (x: int64, x: int64, $: Any, ...) -> Any]",
  "?=10 * (x: int64, x: int64, $: Any, ...) -> Any]",
  "?[10 * (x: int64, x: int64, $: Any, ...) -> Any$",
  "?[10 * (x: int64, x: int64, $: Any, ...) -> Any$",
  "(x: int64, x: int64, $: Any, ...) -> Any",
  "?(x: int64, x: int64, $: Any, ...) -> Any",
  "?((x: int64, x: int64, $: Any, ...) -> Any)",
  "$(x: int64, x: int64, $: Any, ...) -> Any",
  "=(x: int64, x: int64, $: Any, ...) -> Any",
  "$[(x: int64, x: int64, $: Any, ...) -> Any]",
  "xxx[(x: int64, x: int64, $: Any, ...) -> Any]",
  "?$(x: int64, x: int64, $: Any, ...) -> Any]",
  "?=(x: int64, x: int64, $: Any, ...) -> Any]",
  "?[(x: int64, x: int64, $: Any, ...) -> Any$",
  "?[(x: int64, x: int64, $: Any, ...) -> Any$",
  "10 $ (x: int64, x: int64, $: Any, ...) -> Any",
  "?10 $ (x: int64, x: int64, $: Any, ...) -> Any",
  "?(10 $ (x: int64, x: int64, $: Any, ...) -> Any)",
  "$10 $ (x: int64, x: int64, $: Any, ...) -> Any",
  "=10 $ (x: int64, x: int64, $: Any, ...) -> Any",
  "$[10 $ (x: int64, x: int64, $: Any, ...) -> Any]",
  "xxx[10 $ (x: int64, x: int64, $: Any, ...) -> Any]",
  "?$10 $ (x: int64, x: int64, $: Any, ...) -> Any]",
  "?=10 $ (x: int64, x: int64, $: Any, ...) -> Any]",
  "?[10 $ (x: int64, x: int64, $: Any, ...) -> Any$",
  "?[10 $ (x: int64, x: int64, $: Any, ...) -> Any$",
  "10 = (x: int64, x: int64, $: Any, ...) -> Any",
  "?10 = (x: int64, x: int64, $: Any, ...) -> Any",
  "?(10 = (x: int64, x: int64, $: Any, ...) -> Any)",
  "$10 = (x: int64, x: int64, $: Any, ...) -> Any",
  "=10 = (x: int64, x: int64, $: Any, ...) -> Any",
  "$[10 = (x: int64, x: int64, $: Any, ...) -> Any]",
  "xxx[10 = (x: int64, x: int64, $: Any, ...) -> Any]",
  "?$10 = (x: int64, x: int64, $: Any, ...) -> Any]",
  "?=10 = (x: int64, x: int64, $: Any, ...) -> Any]",
  "?[10 = (x: int64, x: int64, $: Any, ...) -> Any$",
  "?[10 = (x: int64, x: int64, $: Any, ...) -> Any$",
  "10 * (x: int64, x: int64$ $: Any, ...) -> Any",
  "?10 * (x: int64, x: int64$ $: Any, ...) -> Any",
  "?(10 * (x: int64, x: int64$ $: Any, ...) -> Any)",
  "$10 * (x: int64, x: int64$ $: Any, ...) -> Any",
  "=10 * (x: int64, x: int64$ $: Any, ...) -> Any",
  "$[10 * (x: int64, x: int64$ $: Any, ...) -> Any]",
  "xxx[10 * (x: int64, x: int64$ $: Any, ...) -> Any]",
  "?$10 * (x: int64, x: int64$ $: Any, ...) -> Any]",
  "?=10 * (x: int64, x: int64$ $: Any, ...) -> Any]",
  "?[10 * (x: int64, x: int64$ $: Any, ...) -> Any$",
  "?[10 * (x: int64, x: int64$ $: Any, ...) -> Any$",
  "(x: int64, x: int64$ $: Any, ...) -> Any",
  "?(x: int64, x: int64$ $: Any, ...) -> Any",
  "?((x: int64, x: int64$ $: Any, ...) -> Any)",
  "$(x: int64, x: int64$ $: Any, ...) -> Any",
  "=(x: int64, x: int64$ $: Any, ...) -> Any",
  "$[(x: int64, x: int64$ $: Any, ...) -> Any]",
  "xxx[(x: int64, x: int64$ $: Any, ...) -> Any]",
  "?$(x: int64, x: int64$ $: Any, ...) -> Any]",
  "?=(x: int64, x: int64$ $: Any, ...) -> Any]",
  "?[(x: int64, x: int64$ $: Any, ...) -> Any$",
  "?[(x: int64, x: int64$ $: Any, ...) -> Any$",
  "10 $ (x: int64, x: int64$ $: Any, ...) -> Any",
  "?10 $ (x: int64, x: int64$ $: Any, ...) -> Any",
  "?(10 $ (x: int64, x: int64$ $: Any, ...) -> Any)",
  "$10 $ (x: int64, x: int64$ $: Any, ...) -> Any",
  "=10 $ (x: int64, x: int64$ $: Any, ...) -> Any",
  "$[10 $ (x: int64, x: int64$ $: Any, ...) -> Any]",
  "xxx[10 $ (x: int64, x: int64$ $: Any, ...) -> Any]",
  "?$10 $ (x: int64, x: int64$ $: Any, ...) -> Any]",
  "?=10 $ (x: int64, x: int64$ $: Any, ...) -> Any]",
  "?[10 $ (x: int64, x: int64$ $: Any, ...) -> Any$",
  "?[10 $ (x: int64, x: int64$ $: Any, ...) -> Any$",
  "10 = (x: int64, x: int64$ $: Any, ...) -> Any",
  "?10 = (x: int64, x: int64$ $: Any, ...) -> Any",
  "?(10 = (x: int64, x: int64$ $: Any, ...) -> Any)",
  "$10 = (x: int64, x: int64$ $: Any, ...) -> Any",
  "=10 = (x: int64, x: int64$ $: Any, ...) -> Any",
  "$[10 = (x: int64, x: int64$ $: Any, ...) -> Any]",
  "xxx[10 = (x: int64, x: int64$ $: Any, ...) -> Any]",
  "?$10 = (x: int64, x: int64$ $: Any, ...) -> Any]",
  "?=10 = (x: int64, x: int64$ $: Any, ...) -> Any]",
  "?[10 = (x: int64, x: int64$ $: Any, ...) -> Any$",
  "?[10 = (x: int64, x: int64$ $: Any, ...) -> Any$",
  "10 * (x: int64, x: int64= $: Any, ...) -> Any",
  "?10 * (x: int64, x: int64= $: Any, ...) -> Any",
  "?(10 * (x: int64, x: int64= $: Any, ...) -> Any)",
  "$10 * (x: int64, x: int64= $: Any, ...) -> Any",
  "=10 * (x: int64, x: int64= $: Any, ...) -> Any",
  "$[10 * (x: int64, x: int64= $: Any, ...) -> Any]",
  "xxx[10 * (x: int64, x: int64= $: Any, ...) -> Any]",
  "?$10 * (x: int64, x: int64= $: Any, ...) -> Any]",
  "?=10 * (x: int64, x: int64= $: Any, ...) -> Any]",
  "?[10 * (x: int64, x: int64= $: Any, ...) -> Any$",
  "?[10 * (x: int64, x: int64= $: Any, ...) -> Any$",
  "(x: int64, x: int64= $: Any, ...) -> Any",
  "?(x: int64, x: int64= $: Any, ...) -> Any",
  "?((x: int64, x: int64= $: Any, ...) -> Any)",
  "$(x: int64, x: int64= $: Any, ...) -> Any",
  "=(x: int64, x: int64= $: Any, ...) -> Any",
  "$[(x: int64, x: int64= $: Any, ...) -> Any]",
  "xxx[(x: int64, x: int64= $: Any, ...) -> Any]",
  "?$(x: int64, x: int64= $: Any, ...) -> Any]",
  "?=(x: int64, x: int64= $: Any, ...) -> Any]",
  "?[(x: int64, x: int64= $: Any, ...) -> Any$",
  "?[(x: int64, x: int64= $: Any, ...) -> Any$",
  "10 $ (x: int64, x: int64= $: Any, ...) -> Any",
  "?10 $ (x: int64, x: int64= $: Any, ...) -> Any",
  "?(10 $ (x: int64, x: int64= $: Any, ...) -> Any)",
  "$10 $ (x: int64, x: int64= $: Any, ...) -> Any",
  "=10 $ (x: int64, x: int64= $: Any, ...) -> Any",
  "$[10 $ (x: int64, x: int64= $: Any, ...) -> Any]",
  "xxx[10 $ (x: int64, x: int64= $: Any, ...) -> Any]",
  "?$10 $ (x: int64, x: int64= $: Any, ...) -> Any]",
  "?=10 $ (x: int64, x: int64= $: Any, ...) -> Any]",
  "?[10 $ (x: int64, x: int64= $: Any, ...) -> Any$",
  "?[10 $ (x: int64, x: int64= $: Any, ...) -> Any$",
  "10 = (x: int64, x: int64= $: Any, ...) -> Any",
  "?10 = (x: int64, x: int64= $: Any, ...) -> Any",
  "?(10 = (x: int64, x: int64= $: Any, ...) -> Any)",
  "$10 = (x: int64, x: int64= $: Any, ...) -> Any",
  "=10 = (x: int64, x: int64= $: Any, ...) -> Any",
  "$[10 = (x: int64, x: int64= $: Any, ...) -> Any]",
  "xxx[10 = (x: int64, x: int64= $: Any, ...) -> Any]",
  "?$10 = (x: int64, x: int64= $: Any, ...) -> Any]",
  "?=10 = (x: int64, x: int64= $: Any, ...) -> Any]",
  "?[10 = (x: int64, x: int64= $: Any, ...) -> Any$",
  "?[10 = (x: int64, x: int64= $: Any, ...) -> Any$",
  "10 * (=: Any, ...) -> Any",
  "?10 * (=: Any, ...) -> Any",
  "?(10 * (=: Any, ...) -> Any)",
  "$10 * (=: Any, ...) -> Any",
  "=10 * (=: Any, ...) -> Any",
  "$[10 * (=: Any, ...) -> Any]",
  "xxx[10 * (=: Any, ...) -> Any]",
  "?$10 * (=: Any, ...) -> Any]",
  "?=10 * (=: Any, ...) -> Any]",
  "?[10 * (=: Any, ...) -> Any$",
  "?[10 * (=: Any, ...) -> Any$",
  "(=: Any, ...) -> Any",
  "?(=: Any, ...) -> Any",
  "?((=: Any, ...) -> Any)",
  "$(=: Any, ...) -> Any",
  "=(=: Any, ...) -> Any",
  "$[(=: Any, ...) -> Any]",
  "xxx[(=: Any, ...) -> Any]",
  "?$(=: Any, ...) -> Any]",
  "?=(=: Any, ...) -> Any]",
  "?[(=: Any, ...) -> Any$",
  "?[(=: Any, ...) -> Any$",
  "10 $ (=: Any, ...) -> Any",
  "?10 $ (=: Any, ...) -> Any",
  "?(10 $ (=: Any, ...) -> Any)",
  "$10 $ (=: Any, ...) -> Any",
  "=10 $ (=: Any, ...) -> Any",
  "$[10 $ (=: Any, ...) -> Any]",
  "xxx[10 $ (=: Any, ...) -> Any]",
  "?$10 $ (=: Any, ...) -> Any]",
  "?=10 $ (=: Any, ...) -> Any]",
  "?[10 $ (=: Any, ...) -> Any$",
  "?[10 $ (=: Any, ...) -> Any$",
  "10 = (=: Any, ...) -> Any",
  "?10 = (=: Any, ...) -> Any",
  "?(10 = (=: Any, ...) -> Any)",
  "$10 = (=: Any, ...) -> Any",
  "=10 = (=: Any, ...) -> Any",
  "$[10 = (=: Any, ...) -> Any]",
  "xxx[10 = (=: Any, ...) -> Any]",
  "?$10 = (=: Any, ...) -> Any]",
  "?=10 = (=: Any, ...) -> Any]",
  "?[10 = (=: Any, ...) -> Any$",
  "?[10 = (=: Any, ...) -> Any$",
  "10 * (x: int64, =: Any, ...) -> Any",
  "?10 * (x: int64, =: Any, ...) -> Any",
  "?(10 * (x: int64, =: Any, ...) -> Any)",
  "$10 * (x: int64, =: Any, ...) -> Any",
  "=10 * (x: int64, =: Any, ...) -> Any",
  "$[10 * (x: int64, =: Any, ...) -> Any]",
  "xxx[10 * (x: int64, =: Any, ...) -> Any]",
  "?$10 * (x: int64, =: Any, ...) -> Any]",
  "?=10 * (x: int64, =: Any, ...) -> Any]",
  "?[10 * (x: int64, =: Any, ...) -> Any$",
  "?[10 * (x: int64, =: Any, ...) -> Any$",
  "(x: int64, =: Any, ...) -> Any",
  "?(x: int64, =: Any, ...) -> Any",
  "?((x: int64, =: Any, ...) -> Any)",
  "$(x: int64, =: Any, ...) -> Any",
  "=(x: int64, =: Any, ...) -> Any",
  "$[(x: int64, =: Any, ...) -> Any]",
  "xxx[(x: int64, =: Any, ...) -> Any]",
  "?$(x: int64, =: Any, ...) -> Any]",
  "?=(x: int64, =: Any, ...) -> Any]",
  "?[(x: int64, =: Any, ...) -> Any$",
  "?[(x: int64, =: Any, ...) -> Any$",
  "10 $ (x: int64, =: Any, ...) -> Any",
  "?10 $ (x: int64, =: Any, ...) -> Any",
  "?(10 $ (x: int64, =: Any, ...) -> Any)",
  "$10 $ (x: int64, =: Any, ...) -> Any",
  "=10 $ (x: int64, =: Any, ...) -> Any",
  "$[10 $ (x: int64, =: Any, ...) -> Any]",
  "xxx[10 $ (x: int64, =: Any, ...) -> Any]",
  "?$10 $ (x: int64, =: Any, ...) -> Any]",
  "?=10 $ (x: int64, =: Any, ...) -> Any]",
  "?[10 $ (x: int64, =: Any, ...) -> Any$",
  "?[10 $ (x: int64, =: Any, ...) -> Any$",
  "10 = (x: int64, =: Any, ...) -> Any",
  "?10 = (x: int64, =: Any, ...) -> Any",
  "?(10 = (x: int64, =: Any, ...) -> Any)",
  "$10 = (x: int64, =: Any, ...) -> Any",
  "=10 = (x: int64, =: Any, ...) -> Any",
  "$[10 = (x: int64, =: Any, ...) -> Any]",
  "xxx[10 = (x: int64, =: Any, ...) -> Any]",
  "?$10 = (x: int64, =: Any, ...) -> Any]",
  "?=10 = (x: int64, =: Any, ...) -> Any]",
  "?[10 = (x: int64, =: Any, ...) -> Any$",
  "?[10 = (x: int64, =: Any, ...) -> Any$",
  "10 * (x: int64$ =: Any, ...) -> Any",
  "?10 * (x: int64$ =: Any, ...) -> Any",
  "?(10 * (x: int64$ =: Any, ...) -> Any)",
  "$10 * (x: int64$ =: Any, ...) -> Any",
  "=10 * (x: int64$ =: Any, ...) -> Any",
  "$[10 * (x: int64$ =: Any, ...) -> Any]",
  "xxx[10 * (x: int64$ =: Any, ...) -> Any]",
  "?$10 * (x: int64$ =: Any, ...) -> Any]",
  "?=10 * (x: int64$ =: Any, ...) -> Any]",
  "?[10 * (x: int64$ =: Any, ...) -> Any$",
  "?[10 * (x: int64$ =: Any, ...) -> Any$",
  "(x: int64$ =: Any, ...) -> Any",
  "?(x: int64$ =: Any, ...) -> Any",
  "?((x: int64$ =: Any, ...) -> Any)",
  "$(x: int64$ =: Any, ...) -> Any",
  "=(x: int64$ =: Any, ...) -> Any",
  "$[(x: int64$ =: Any, ...) -> Any]",
  "xxx[(x: int64$ =: Any, ...) -> Any]",
  "?$(x: int64$ =: Any, ...) -> Any]",
  "?=(x: int64$ =: Any, ...) -> Any]",
  "?[(x: int64$ =: Any, ...) -> Any$",
  "?[(x: int64$ =: Any, ...) -> Any$",
  "10 $ (x: int64$ =: Any, ...) -> Any",
  "?10 $ (x: int64$ =: Any, ...) -> Any",
  "?(10 $ (x: int64$ =: Any, ...) -> Any)",
  "$10 $ (x: int64$ =: Any, ...) -> Any",
  "=10 $ (x: int64$ =: Any, ...) -> Any",
  "$[10 $ (x: int64$ =: Any, ...) -> Any]",
  "xxx[10 $ (x: int64$ =: Any, ...) -> Any]",
  "?$10 $ (x: int64$ =: Any, ...) -> Any]",
  "?=10 $ (x: int64$ =: Any, ...) -> Any]",
  "?[10 $ (x: int64$ =: Any, ...) -> Any$",
  "?[10 $ (x: int64$ =: Any, ...) -> Any$",
  "10 = (x: int64$ =: Any, ...) -> Any",
  "?10 = (x: int64$ =: Any, ...) -> Any",
  "?(10 = (x: int64$ =: Any, ...) -> Any)",
  "$10 = (x: int64$ =: Any, ...) -> Any",
  "=10 = (x: int64$ =: Any, ...) -> Any",
  "$[10 = (x: int64$ =: Any, ...) -> Any]",
  "xxx[10 = (x: int64$ =: Any, ...) -> Any]",
  "?$10 = (x: int64$ =: Any, ...) -> Any]",
  "?=10 = (x: int64$ =: Any, ...) -> Any]",
  "?[10 = (x: int64$ =: Any, ...) -> Any$",
  "?[10 = (x: int64$ =: Any, ...) -> Any$",
  "10 * (x: int64= =: Any, ...) -> Any",
  "?10 * (x: int64= =: Any, ...) -> Any",
  "?(10 * (x: int64= =: Any, ...) -> Any)",
  "$10 * (x: int64= =: Any, ...) -> Any",
  "=10 * (x: int64= =: Any, ...) -> Any",
  "$[10 * (x: int64= =: Any, ...) -> Any]",
  "xxx[10 * (x: int64= =: Any, ...) -> Any]",
  "?$10 * (x: int64= =: Any, ...) -> Any]",
  "?=10 * (x: int64= =: Any, ...) -> Any]",
  "?[10 * (x: int64= =: Any, ...) -> Any$",
  "?[10 * (x: int64= =: Any, ...) -> Any$",
  "(x: int64= =: Any, ...) -> Any",
  "?(x: int64= =: Any, ...) -> Any",
  "?((x: int64= =: Any, ...) -> Any)",
  "$(x: int64= =: Any, ...) -> Any",
  "=(x: int64= =: Any, ...) -> Any",
  "$[(x: int64= =: Any, ...) -> Any]",
  "xxx[(x: int64= =: Any, ...) -> Any]",
  "?$(x: int64= =: Any, ...) -> Any]",
  "?=(x: int64= =: Any, ...) -> Any]",
  "?[(x: int64= =: Any, ...) -> Any$",
  "?[(x: int64= =: Any, ...) -> Any$",
  "10 $ (x: int64= =: Any, ...) -> Any",
  "?10 $ (x: int64= =: Any, ...) -> Any",
  "?(10 $ (x: int64= =: Any, ...) -> Any)",
  "$10 $ (x: int64= =: Any, ...) -> Any",
  "=10 $ (x: int64= =: Any, ...) -> Any",
  "$[10 $ (x: int64= =: Any, ...) -> Any]",
  "xxx[10 $ (x: int64= =: Any, ...) -> Any]",
  "?$10 $ (x: int64= =: Any, ...) -> Any]",
  "?=10 $ (x: int64= =: Any, ...) -> Any]",
  "?[10 $ (x: int64= =: Any, ...) -> Any$",
  "?[10 $ (x: int64= =: Any, ...) -> Any$",
  "10 = (x: int64= =: Any, ...) -> Any",
  "?10 = (x: int64= =: Any, ...) -> Any",
  "?(10 = (x: int64= =: Any, ...) -> Any)",
  "$10 = (x: int64= =: Any, ...) -> Any",
  "=10 = (x: int64= =: Any, ...) -> Any",
  "$[10 = (x: int64= =: Any, ...) -> Any]",
  "xxx[10 = (x: int64= =: Any, ...) -> Any]",
  "?$10 = (x: int64= =: Any, ...) -> Any]",
  "?=10 = (x: int64= =: Any, ...) -> Any]",
  "?[10 = (x: int64= =: Any, ...) -> Any$",
  "?[10 = (x: int64= =: Any, ...) -> Any$",
  "10 * (x: int64, x: int64, =: Any, ...) -> Any",
  "?10 * (x: int64, x: int64, =: Any, ...) -> Any",
  "?(10 * (x: int64, x: int64, =: Any, ...) -> Any)",
  "$10 * (x: int64, x: int64, =: Any, ...) -> Any",
  "=10 * (x: int64, x: int64, =: Any, ...) -> Any",
  "$[10 * (x: int64, x: int64, =: Any, ...) -> Any]",
  "xxx[10 * (x: int64, x: int64, =: Any, ...) -> Any]",
  "?$10 * (x: int64, x: int64, =: Any, ...) -> Any]",
  "?=10 * (x: int64, x: int64, =: Any, ...) -> Any]",
  "?[10 * (x: int64, x: int64, =: Any, ...) -> Any$",
  "?[10 * (x: int64, x: int64, =: Any, ...) -> Any$",
  "(x: int64, x: int64, =: Any, ...) -> Any",
  "?(x: int64, x: int64, =: Any, ...) -> Any",
  "?((x: int64, x: int64, =: Any, ...) -> Any)",
  "$(x: int64, x: int64, =: Any, ...) -> Any",
  "=(x: int64, x: int64, =: Any, ...) -> Any",
  "$[(x: int64, x: int64, =: Any, ...) -> Any]",
  "xxx[(x: int64, x: int64, =: Any, ...) -> Any]",
  "?$(x: int64, x: int64, =: Any, ...) -> Any]",
  "?=(x: int64, x: int64, =: Any, ...) -> Any]",
  "?[(x: int64, x: int64, =: Any, ...) -> Any$",
  "?[(x: int64, x: int64, =: Any, ...) -> Any$",
  "10 $ (x: int64, x: int64, =: Any, ...) -> Any",
  "?10 $ (x: int64, x: int64, =: Any, ...) -> Any",
  "?(10 $ (x: int64, x: int64, =: Any, ...) -> Any)",
  "$10 $ (x: int64, x: int64, =: Any, ...) -> Any",
  "=10 $ (x: int64, x: int64, =: Any, ...) -> Any",
  "$[10 $ (x: int64, x: int64, =: Any, ...) -> Any]",
  "xxx[10 $ (x: int64, x: int64, =: Any, ...) -> Any]",
  "?$10 $ (x: int64, x: int64, =: Any, ...) -> Any]",
  "?=10 $ (x: int64, x: int64, =: Any, ...) -> Any]",
  "?[10 $ (x: int64, x: int64, =: Any, ...) -> Any$",
  "?[10 $ (x: int64, x: int64, =: Any, ...) -> Any$",
  "10 = (x: int64, x: int64, =: Any, ...) -> Any",
  "?10 = (x: int64, x: int64, =: Any, ...) -> Any",
  "?(10 = (x: int64, x: int64, =: Any, ...) -> Any)",
  "$10 = (x: int64, x: int64, =: Any, ...) -> Any",
  "=10 = (x: int64, x: int64, =: Any, ...) -> Any",
  "$[10 = (x: int64, x: int64, =: Any, ...) -> Any]",
  "xxx[10 = (x: int64, x: int64, =: Any, ...) -> Any]",
  "?$10 = (x: int64, x: int64, =: Any, ...) -> Any]",
  "?=10 = (x: int64, x: int64, =: Any, ...) -> Any]",
  "?[10 = (x: int64, x: int64, =: Any, ...) -> Any$",
  "?[10 = (x: int64, x: int64, =: Any, ...) -> Any$",
  "10 * (x: int64, x: int64$ =: Any, ...) -> Any",
  "?10 * (x: int64, x: int64$ =: Any, ...) -> Any",
  "?(10 * (x: int64, x: int64$ =: Any, ...) -> Any)",
  "$10 * (x: int64, x: int64$ =: Any, ...) -> Any",
  "=10 * (x: int64, x: int64$ =: Any, ...) -> Any",
  "$[10 * (x: int64, x: int64$ =: Any, ...) -> Any]",
  "xxx[10 * (x: int64, x: int64$ =: Any, ...) -> Any]",
  "?$10 * (x: int64, x: int64$ =: Any, ...) -> Any]",
  "?=10 * (x: int64, x: int64$ =: Any, ...) -> Any]",
  "?[10 * (x: int64, x: int64$ =: Any, ...) -> Any$",
  "?[10 * (x: int64, x: int64$ =: Any, ...) -> Any$",
  "(x: int64, x: int64$ =: Any, ...) -> Any",
  "?(x: int64, x: int64$ =: Any, ...) -> Any",
  "?((x: int64, x: int64$ =: Any, ...) -> Any)",
  "$(x: int64, x: int64$ =: Any, ...) -> Any",
  "=(x: int64, x: int64$ =: Any, ...) -> Any",
  "$[(x: int64, x: int64$ =: Any, ...) -> Any]",
  "xxx[(x: int64, x: int64$ =: Any, ...) -> Any]",
  "?$(x: int64, x: int64$ =: Any, ...) -> Any]",
  "?=(x: int64, x: int64$ =: Any, ...) -> Any]",
  "?[(x: int64, x: int64$ =: Any, ...) -> Any$",
  "?[(x: int64, x: int64$ =: Any, ...) -> Any$",
  "10 $ (x: int64, x: int64$ =: Any, ...) -> Any",
  "?10 $ (x: int64, x: int64$ =: Any, ...) -> Any",
  "?(10 $ (x: int64, x: int64$ =: Any, ...) -> Any)",
  "$10 $ (x: int64, x: int64$ =: Any, ...) -> Any",
  "=10 $ (x: int64, x: int64$ =: Any, ...) -> Any",
  "$[10 $ (x: int64, x: int64$ =: Any, ...) -> Any]",
  "xxx[10 $ (x: int64, x: int64$ =: Any, ...) -> Any]",
  "?$10 $ (x: int64, x: int64$ =: Any, ...) -> Any]",
  "?=10 $ (x: int64, x: int64$ =: Any, ...) -> Any]",
  "?[10 $ (x: int64, x: int64$ =: Any, ...) -> Any$",
  "?[10 $ (x: int64, x: int64$ =: Any, ...) -> Any$",
  "10 = (x: int64, x: int64$ =: Any, ...) -> Any",
  "?10 = (x: int64, x: int64$ =: Any, ...) -> Any",
  "?(10 = (x: int64, x: int64$ =: Any, ...) -> Any)",
  "$10 = (x: int64, x: int64$ =: Any, ...) -> Any",
  "=10 = (x: int64, x: int64$ =: Any, ...) -> Any",
  "$[10 = (x: int64, x: int64$ =: Any, ...) -> Any]",
  "xxx[10 = (x: int64, x: int64$ =: Any, ...) -> Any]",
  "?$10 = (x: int64, x: int64$ =: Any, ...) -> Any]",
  "?=10 = (x: int64, x: int64$ =: Any, ...) -> Any]",
  "?[10 = (x: int64, x: int64$ =: Any, ...) -> Any$",
  "?[10 = (x: int64, x: int64$ =: Any, ...) -> Any$",
  "10 * (x: int64, x: int64= =: Any, ...) -> Any",
  "?10 * (x: int64, x: int64= =: Any, ...) -> Any",
  "?(10 * (x: int64, x: int64= =: Any, ...) -> Any)",
  "$10 * (x: int64, x: int64= =: Any, ...) -> Any",
  "=10 * (x: int64, x: int64= =: Any, ...) -> Any",
  "$[10 * (x: int64, x: int64= =: Any, ...) -> Any]",
  "xxx[10 * (x: int64, x: int64= =: Any, ...) -> Any]",
  "?$10 * (x: int64, x: int64= =: Any, ...) -> Any]",
  "?=10 * (x: int64, x: int64= =: Any, ...) -> Any]",
  "?[10 * (x: int64, x: int64= =: Any, ...) -> Any$",
  "?[10 * (x: int64, x: int64= =: Any, ...) -> Any$",
  "(x: int64, x: int64= =: Any, ...) -> Any",
  "?(x: int64, x: int64= =: Any, ...) -> Any",
  "?((x: int64, x: int64= =: Any, ...) -> Any)",
  "$(x: int64, x: int64= =: Any, ...) -> Any",
  "=(x: int64, x: int64= =: Any, ...) -> Any",
  "$[(x: int64, x: int64= =: Any, ...) -> Any]",
  "xxx[(x: int64, x: int64= =: Any, ...) -> Any]",
  "?$(x: int64, x: int64= =: Any, ...) -> Any]",
  "?=(x: int64, x: int64= =: Any, ...) -> Any]",
  "?[(x: int64, x: int64= =: Any, ...) -> Any$",
  "?[(x: int64, x: int64= =: Any, ...) -> Any$",
  "10 $ (x: int64, x: int64= =: Any, ...) -> Any",
  "?10 $ (x: int64, x: int64= =: Any, ...) -> Any",
  "?(10 $ (x: int64, x: int64= =: Any, ...) -> Any)",
  "$10 $ (x: int64, x: int64= =: Any, ...) -> Any",
  "=10 $ (x: int64, x: int64= =: Any, ...) -> Any",
  "$[10 $ (x: int64, x: int64= =: Any, ...) -> Any]",
  "xxx[10 $ (x: int64, x: int64= =: Any, ...) -> Any]",
  "?$10 $ (x: int64, x: int64= =: Any, ...) -> Any]",
  "?=10 $ (x: int64, x: int64= =: Any, ...) -> Any]",
  "?[10 $ (x: int64, x: int64= =: Any, ...) -> Any$",
  "?[10 $ (x: int64, x: int64= =: Any, ...) -> Any$",
  "10 = (x: int64, x: int64= =: Any, ...) -> Any",
  "?10 = (x: int64, x: int64= =: Any, ...) -> Any",
  "?(10 = (x: int64, x: int64= =: Any, ...) -> Any)",
  "$10 = (x: int64, x: int64= =: Any, ...) -> Any",
  "=10 = (x: int64, x: int64= =: Any, ...) -> Any",
  "$[10 = (x: int64, x: int64= =: Any, ...) -> Any]",
  "xxx[10 = (x: int64, x: int64= =: Any, ...) -> Any]",
  "?$10 = (x: int64, x: int64= =: Any, ...) -> Any]",
  "?=10 = (x: int64, x: int64= =: Any, ...) -> Any]",
  "?[10 = (x: int64, x: int64= =: Any, ...) -> Any$",
  "?[10 = (x: int64, x: int64= =: Any, ...) -> Any$",
  "10 * (a$ Any, ...) -> Any",
  "?10 * (a$ Any, ...) -> Any",
  "?(10 * (a$ Any, ...) -> Any)",
  "$10 * (a$ Any, ...) -> Any",
  "=10 * (a$ Any, ...) -> Any",
  "$[10 * (a$ Any, ...) -> Any]",
  "xxx[10 * (a$ Any, ...) -> Any]",
  "?$10 * (a$ Any, ...) -> Any]",
  "?=10 * (a$ Any, ...) -> Any]",
  "?[10 * (a$ Any, ...) -> Any$",
  "?[10 * (a$ Any, ...) -> Any$",
  "(a$ Any, ...) -> Any",
  "?(a$ Any, ...) -> Any",
  "?((a$ Any, ...) -> Any)",
  "$(a$ Any, ...) -> Any",
  "=(a$ Any, ...) -> Any",
  "$[(a$ Any, ...) -> Any]",
  "xxx[(a$ Any, ...) -> Any]",
  "?$(a$ Any, ...) -> Any]",
  "?=(a$ Any, ...) -> Any]",
  "?[(a$ Any, ...) -> Any$",
  "?[(a$ Any, ...) -> Any$",
  "10 $ (a$ Any, ...) -> Any",
  "?10 $ (a$ Any, ...) -> Any",
  "?(10 $ (a$ Any, ...) -> Any)",
  "$10 $ (a$ Any, ...) -> Any",
  "=10 $ (a$ Any, ...) -> Any",
  "$[10 $ (a$ Any, ...) -> Any]",
  "xxx[10 $ (a$ Any, ...) -> Any]",
  "?$10 $ (a$ Any, ...) -> Any]",
  "?=10 $ (a$ Any, ...) -> Any]",
  "?[10 $ (a$ Any, ...) -> Any$",
  "?[10 $ (a$ Any, ...) -> Any$",
  "10 = (a$ Any, ...) -> Any",
  "?10 = (a$ Any, ...) -> Any",
  "?(10 = (a$ Any, ...) -> Any)",
  "$10 = (a$ Any, ...) -> Any",
  "=10 = (a$ Any, ...) -> Any",
  "$[10 = (a$ Any, ...) -> Any]",
  "xxx[10 = (a$ Any, ...) -> Any]",
  "?$10 = (a$ Any, ...) -> Any]",
  "?=10 = (a$ Any, ...) -> Any]",
  "?[10 = (a$ Any, ...) -> Any$",
  "?[10 = (a$ Any, ...) -> Any$",
  "10 * (x: int64, a$ Any, ...) -> Any",
  "?10 * (x: int64, a$ Any, ...) -> Any",
  "?(10 * (x: int64, a$ Any, ...) -> Any)",
  "$10 * (x: int64, a$ Any, ...) -> Any",
  "=10 * (x: int64, a$ Any, ...) -> Any",
  "$[10 * (x: int64, a$ Any, ...) -> Any]",
  "xxx[10 * (x: int64, a$ Any, ...) -> Any]",
  "?$10 * (x: int64, a$ Any, ...) -> Any]",
  "?=10 * (x: int64, a$ Any, ...) -> Any]",
  "?[10 * (x: int64, a$ Any, ...) -> Any$",
  "?[10 * (x: int64, a$ Any, ...) -> Any$",
  "(x: int64, a$ Any, ...) -> Any",
  "?(x: int64, a$ Any, ...) -> Any",
  "?((x: int64, a$ Any, ...) -> Any)",
  "$(x: int64, a$ Any, ...) -> Any",
  "=(x: int64, a$ Any, ...) -> Any",
  "$[(x: int64, a$ Any, ...) -> Any]",
  "xxx[(x: int64, a$ Any, ...) -> Any]",
  "?$(x: int64, a$ Any, ...) -> Any]",
  "?=(x: int64, a$ Any, ...) -> Any]",
  "?[(x: int64, a$ Any, ...) -> Any$",
  "?[(x: int64, a$ Any, ...) -> Any$",
  "10 $ (x: int64, a$ Any, ...) -> Any",
  "?10 $ (x: int64, a$ Any, ...) -> Any",
  "?(10 $ (x: int64, a$ Any, ...) -> Any)",
  "$10 $ (x: int64, a$ Any, ...) -> Any",
  "=10 $ (x: int64, a$ Any, ...) -> Any",
  "$[10 $ (x: int64, a$ Any, ...) -> Any]",
  "xxx[10 $ (x: int64, a$ Any, ...) -> Any]",
  "?$10 $ (x: int64, a$ Any, ...) -> Any]",
  "?=10 $ (x: int64, a$ Any, ...) -> Any]",
  "?[10 $ (x: int64, a$ Any, ...) -> Any$",
  "?[10 $ (x: int64, a$ Any, ...) -> Any$",
  "10 = (x: int64, a$ Any, ...) -> Any",
  "?10 = (x: int64, a$ Any, ...) -> Any",
  "?(10 = (x: int64, a$ Any, ...) -> Any)",
  "$10 = (x: int64, a$ Any, ...) -> Any",
  "=10 = (x: int64, a$ Any, ...) -> Any",
  "$[10 = (x: int64, a$ Any, ...) -> Any]",
  "xxx[10 = (x: int64, a$ Any, ...) -> Any]",
  "?$10 = (x: int64, a$ Any, ...) -> Any]",
  "?=10 = (x: int64, a$ Any, ...) -> Any]",
  "?[10 = (x: int64, a$ Any, ...) -> Any$",
  "?[10 = (x: int64, a$ Any, ...) -> Any$",
  "10 * (x: int64$ a$ Any, ...) -> Any",
  "?10 * (x: int64$ a$ Any, ...) -> Any",
  "?(10 * (x: int64$ a$ Any, ...) -> Any)",
  "$10 * (x: int64$ a$ Any, ...) -> Any",
  "=10 * (x: int64$ a$ Any, ...) -> Any",
  "$[10 * (x: int64$ a$ Any, ...) -> Any]",
  "xxx[10 * (x: int64$ a$ Any, ...) -> Any]",
  "?$10 * (x: int64$ a$ Any, ...) -> Any]",
  "?=10 * (x: int64$ a$ Any, ...) -> Any]",
  "?[10 * (x: int64$ a$ Any, ...) -> Any$",
  "?[10 * (x: int64$ a$ Any, ...) -> Any$",
  "(x: int64$ a$ Any, ...) -> Any",
  "?(x: int64$ a$ Any, ...) -> Any",
  "?((x: int64$ a$ Any, ...) -> Any)",
  "$(x: int64$ a$ Any, ...) -> Any",
  "=(x: int64$ a$ Any, ...) -> Any",
  "$[(x: int64$ a$ Any, ...) -> Any]",
  "xxx[(x: int64$ a$ Any, ...) -> Any]",
  "?$(x: int64$ a$ Any, ...) -> Any]",
  "?=(x: int64$ a$ Any, ...) -> Any]",
  "?[(x: int64$ a$ Any, ...) -> Any$",
  "?[(x: int64$ a$ Any, ...) -> Any$",
  "10 $ (x: int64$ a$ Any, ...) -> Any",
  "?10 $ (x: int64$ a$ Any, ...) -> Any",
  "?(10 $ (x: int64$ a$ Any, ...) -> Any)",
  "$10 $ (x: int64$ a$ Any, ...) -> Any",
  "=10 $ (x: int64$ a$ Any, ...) -> Any",
  "$[10 $ (x: int64$ a$ Any, ...) -> Any]",
  "xxx[10 $ (x: int64$ a$ Any, ...) -> Any]",
  "?$10 $ (x: int64$ a$ Any, ...) -> Any]",
  "?=10 $ (x: int64$ a$ Any, ...) -> Any]",
  "?[10 $ (x: int64$ a$ Any, ...) -> Any$",
  "?[10 $ (x: int64$ a$ Any, ...) -> Any$",
  "10 = (x: int64$ a$ Any, ...) -> Any",
  "?10 = (x: int64$ a$ Any, ...) -> Any",
  "?(10 = (x: int64$ a$ Any, ...) -> Any)",
  "$10 = (x: int64$ a$ Any, ...) -> Any",
  "=10 = (x: int64$ a$ Any, ...) -> Any",
  "$[10 = (x: int64$ a$ Any, ...) -> Any]",
  "xxx[10 = (x: int64$ a$ Any, ...) -> Any]",
  "?$10 = (x: int64$ a$ Any, ...) -> Any]",
  "?=10 = (x: int64$ a$ Any, ...) -> Any]",
  "?[10 = (x: int64$ a$ Any, ...) -> Any$",
  "?[10 = (x: int64$ a$ Any, ...) -> Any$",
  "10 * (x: int64= a$ Any, ...) -> Any",
  "?10 * (x: int64= a$ Any, ...) -> Any",
  "?(10 * (x: int64= a$ Any, ...) -> Any)",
  "$10 * (x: int64= a$ Any, ...) -> Any",
  "=10 * (x: int64= a$ Any, ...) -> Any",
  "$[10 * (x: int64= a$ Any, ...) -> Any]",
  "xxx[10 * (x: int64= a$ Any, ...) -> Any]",
  "?$10 * (x: int64= a$ Any, ...) -> Any]",
  "?=10 * (x: int64= a$ Any, ...) -> Any]",
  "?[10 * (x: int64= a$ Any, ...) -> Any$",
  "?[10 * (x: int64= a$ Any, ...) -> Any$",
  "(x: int64= a$ Any, ...) -> Any",
  "?(x: int64= a$ Any, ...) -> Any",
  "?((x: int64= a$ Any, ...) -> Any)",
  "$(x: int64= a$ Any, ...) -> Any",
  "=(x: int64= a$ Any, ...) -> Any",
  "$[(x: int64= a$ Any, ...) -> Any]",
  "xxx[(x: int64= a$ Any, ...) -> Any]",
  "?$(x: int64= a$ Any, ...) -> Any]",
  "?=(x: int64= a$ Any, ...) -> Any]",
  "?[(x: int64= a$ Any, ...) -> Any$",
  "?[(x: int64= a$ Any, ...) -> Any$",
  "10 $ (x: int64= a$ Any, ...) -> Any",
  "?10 $ (x: int64= a$ Any, ...) -> Any",
  "?(10 $ (x: int64= a$ Any, ...) -> Any)",
  "$10 $ (x: int64= a$ Any, ...) -> Any",
  "=10 $ (x: int64= a$ Any, ...) -> Any",
  "$[10 $ (x: int64= a$ Any, ...) -> Any]",
  "xxx[10 $ (x: int64= a$ Any, ...) -> Any]",
  "?$10 $ (x: int64= a$ Any, ...) -> Any]",
  "?=10 $ (x: int64= a$ Any, ...) -> Any]",
  "?[10 $ (x: int64= a$ Any, ...) -> Any$",
  "?[10 $ (x: int64= a$ Any, ...) -> Any$",
  "10 = (x: int64= a$ Any, ...) -> Any",
  "?10 = (x: int64= a$ Any, ...) -> Any",
  "?(10 = (x: int64= a$ Any, ...) -> Any)",
  "$10 = (x: int64= a$ Any, ...) -> Any",
  "=10 = (x: int64= a$ Any, ...) -> Any",
  "$[10 = (x: int64= a$ Any, ...) -> Any]",
  "xxx[10 = (x: int64= a$ Any, ...) -> Any]",
  "?$10 = (x: int64= a$ Any, ...) -> Any]",
  "?=10 = (x: int64= a$ Any, ...) -> Any]",
  "?[10 = (x: int64= a$ Any, ...) -> Any$",
  "?[10 = (x: int64= a$ Any, ...) -> Any$",
  "10 * (x: int64, x: int64, a$ Any, ...) -> Any",
  "?10 * (x: int64, x: int64, a$ Any, ...) -> Any",
  "?(10 * (x: int64, x: int64, a$ Any, ...) -> Any)",
  "$10 * (x: int64, x: int64, a$ Any, ...) -> Any",
  "=10 * (x: int64, x: int64, a$ Any, ...) -> Any",
  "$[10 * (x: int64, x: int64, a$ Any, ...) -> Any]",
  "xxx[10 * (x: int64, x: int64, a$ Any, ...) -> Any]",
  "?$10 * (x: int64, x: int64, a$ Any, ...) -> Any]",
  "?=10 * (x: int64, x: int64, a$ Any, ...) -> Any]",
  "?[10 * (x: int64, x: int64, a$ Any, ...) -> Any$",
  "?[10 * (x: int64, x: int64, a$ Any, ...) -> Any$",
  "(x: int64, x: int64, a$ Any, ...) -> Any",
  "?(x: int64, x: int64, a$ Any, ...) -> Any",
  "?((x: int64, x: int64, a$ Any, ...) -> Any)",
  "$(x: int64, x: int64, a$ Any, ...) -> Any",
  "=(x: int64, x: int64, a$ Any, ...) -> Any",
  "$[(x: int64, x: int64, a$ Any, ...) -> Any]",
  "xxx[(x: int64, x: int64, a$ Any, ...) -> Any]",
  "?$(x: int64, x: int64, a$ Any, ...) -> Any]",
  "?=(x: int64, x: int64, a$ Any, ...) -> Any]",
  "?[(x: int64, x: int64, a$ Any, ...) -> Any$",
  "?[(x: int64, x: int64, a$ Any, ...) -> Any$",
  "10 $ (x: int64, x: int64, a$ Any, ...) -> Any",
  "?10 $ (x: int64, x: int64, a$ Any, ...) -> Any",
  "?(10 $ (x: int64, x: int64, a$ Any, ...) -> Any)",
  "$10 $ (x: int64, x: int64, a$ Any, ...) -> Any",
  "=10 $ (x: int64, x: int64, a$ Any, ...) -> Any",
  "$[10 $ (x: int64, x: int64, a$ Any, ...) -> Any]",
  "xxx[10 $ (x: int64, x: int64, a$ Any, ...) -> Any]",
  "?$10 $ (x: int64, x: int64, a$ Any, ...) -> Any]",
  "?=10 $ (x: int64, x: int64, a$ Any, ...) -> Any]",
  "?[10 $ (x: int64, x: int64, a$ Any, ...) -> Any$",
  "?[10 $ (x: int64, x: int64, a$ Any, ...) -> Any$",
  "10 = (x: int64, x: int64, a$ Any, ...) -> Any",
  "?10 = (x: int64, x: int64, a$ Any, ...) -> Any",
  "?(10 = (x: int64, x: int64, a$ Any, ...) -> Any)",
  "$10 = (x: int64, x: int64, a$ Any, ...) -> Any",
  "=10 = (x: int64, x: int64, a$ Any, ...) -> Any",
  "$[10 = (x: int64, x: int64, a$ Any, ...) -> Any]",
  "xxx[10 = (x: int64, x: int64, a$ Any, ...) -> Any]",
  "?$10 = (x: int64, x: int64, a$ Any, ...) -> Any]",
  "?=10 = (x: int64, x: int64, a$ Any, ...) -> Any]",
  "?[10 = (x: int64, x: int64, a$ Any, ...) -> Any$",
  "?[10 = (x: int64, x: int64, a$ Any, ...) -> Any$",
  "10 * (x: int64, x: int64$ a$ Any, ...) -> Any",
  "?10 * (x: int64, x: int64$ a$ Any, ...) -> Any",
  "?(10 * (x: int64, x: int64$ a$ Any, ...) -> Any)",
  "$10 * (x: int64, x: int64$ a$ Any, ...) -> Any",
  "=10 * (x: int64, x: int64$ a$ Any, ...) -> Any",
  "$[10 * (x: int64, x: int64$ a$ Any, ...) -> Any]",
  "xxx[10 * (x: int64, x: int64$ a$ Any, ...) -> Any]",
  "?$10 * (x: int64, x: int64$ a$ Any, ...) -> Any]",
  "?=10 * (x: int64, x: int64$ a$ Any, ...) -> Any]",
  "?[10 * (x: int64, x: int64$ a$ Any, ...) -> Any$",
  "?[10 * (x: int64, x: int64$ a$ Any, ...) -> Any$",
  "(x: int64, x: int64$ a$ Any, ...) -> Any",
  "?(x: int64, x: int64$ a$ Any, ...) -> Any",
  "?((x: int64, x: int64$ a$ Any, ...) -> Any)",
  "$(x: int64, x: int64$ a$ Any, ...) -> Any",
  "=(x: int64, x: int64$ a$ Any, ...) -> Any",
  "$[(x: int64, x: int64$ a$ Any, ...) -> Any]",
  "xxx[(x: int64, x: int64$ a$ Any, ...) -> Any]",
  "?$(x: int64, x: int64$ a$ Any, ...) -> Any]",
  "?=(x: int64, x: int64$ a$ Any, ...) -> Any]",
  "?[(x: int64, x: int64$ a$ Any, ...) -> Any$",
  "?[(x: int64, x: int64$ a$ Any, ...) -> Any$",
  "10 $ (x: int64, x: int64$ a$ Any, ...) -> Any",
  "?10 $ (x: int64, x: int64$ a$ Any, ...) -> Any",
  "?(10 $ (x: int64, x: int64$ a$ Any, ...) -> Any)",
  "$10 $ (x: int64, x: int64$ a$ Any, ...) -> Any",
  "=10 $ (x: int64, x: int64$ a$ Any, ...) -> Any",
  "$[10 $ (x: int64, x: int64$ a$ Any, ...) -> Any]",
  "xxx[10 $ (x: int64, x: int64$ a$ Any, ...) -> Any]",
  "?$10 $ (x: int64, x: int64$ a$ Any, ...) -> Any]",
  "?=10 $ (x: int64, x: int64$ a$ Any, ...) -> Any]",
  "?[10 $ (x: int64, x: int64$ a$ Any, ...) -> Any$",
  "?[10 $ (x: int64, x: int64$ a$ Any, ...) -> Any$",
  "10 = (x: int64, x: int64$ a$ Any, ...) -> Any",
  "?10 = (x: int64, x: int64$ a$ Any, ...) -> Any",
  "?(10 = (x: int64, x: int64$ a$ Any, ...) -> Any)",
  "$10 = (x: int64, x: int64$ a$ Any, ...) -> Any",
  "=10 = (x: int64, x: int64$ a$ Any, ...) -> Any",
  "$[10 = (x: int64, x: int64$ a$ Any, ...) -> Any]",
  "xxx[10 = (x: int64, x: int64$ a$ Any, ...) -> Any]",
  "?$10 = (x: int64, x: int64$ a$ Any, ...) -> Any]",
  "?=10 = (x: int64, x: int64$ a$ Any, ...) -> Any]",
  "?[10 = (x: int64, x: int64$ a$ Any, ...) -> Any$",
  "?[10 = (x: int64, x: int64$ a$ Any, ...) -> Any$",
  "10 * (x: int64, x: int64= a$ Any, ...) -> Any",
  "?10 * (x: int64, x: int64= a$ Any, ...) -> Any",
  "?(10 * (x: int64, x: int64= a$ Any, ...) -> Any)",
  "$10 * (x: int64, x: int64= a$ Any, ...) -> Any",
  "=10 * (x: int64, x: int64= a$ Any, ...) -> Any",
  "$[10 * (x: int64, x: int64= a$ Any, ...) -> Any]",
  "xxx[10 * (x: int64, x: int64= a$ Any, ...) -> Any]",
  "?$10 * (x: int64, x: int64= a$ Any, ...) -> Any]",
  "?=10 * (x: int64, x: int64= a$ Any, ...) -> Any]",
  "?[10 * (x: int64, x: int64= a$ Any, ...) -> Any$",
  "?[10 * (x: int64, x: int64= a$ Any, ...) -> Any$",
  "(x: int64, x: int64= a$ Any, ...) -> Any",
  "?(x: int64, x: int64= a$ Any, ...) -> Any",
  "?((x: int64, x: int64= a$ Any, ...) -> Any)",
  "$(x: int64, x: int64= a$ Any, ...) -> Any",
  "=(x: int64, x: int64= a$ Any, ...) -> Any",
  "$[(x: int64, x: int64= a$ Any, ...) -> Any]",
  "xxx[(x: int64, x: int64= a$ Any, ...) -> Any]",
  "?$(x: int64, x: int64= a$ Any, ...) -> Any]",
  "?=(x: int64, x: int64= a$ Any, ...) -> Any]",
  "?[(x: int64, x: int64= a$ Any, ...) -> Any$",
  "?[(x: int64, x: int64= a$ Any, ...) -> Any$",
  "10 $ (x: int64, x: int64= a$ Any, ...) -> Any",
  "?10 $ (x: int64, x: int64= a$ Any, ...) -> Any",
  "?(10 $ (x: int64, x: int64= a$ Any, ...) -> Any)",
  "$10 $ (x: int64, x: int64= a$ Any, ...) -> Any",
  "=10 $ (x: int64, x: int64= a$ Any, ...) -> Any",
  "$[10 $ (x: int64, x: int64= a$ Any, ...) -> Any]",
  "xxx[10 $ (x: int64, x: int64= a$ Any, ...) -> Any]",
  "?$10 $ (x: int64, x: int64= a$ Any, ...) -> Any]",
  "?=10 $ (x: int64, x: int64= a$ Any, ...) -> Any]",
  "?[10 $ (x: int64, x: int64= a$ Any, ...) -> Any$",
  "?[10 $ (x: int64, x: int64= a$ Any, ...) -> Any$",
  "10 = (x: int64, x: int64= a$ Any, ...) -> Any",
  "?10 = (x: int64, x: int64= a$ Any, ...) -> Any",
  "?(10 = (x: int64, x: int64= a$ Any, ...) -> Any)",
  "$10 = (x: int64, x: int64= a$ Any, ...) -> Any",
  "=10 = (x: int64, x: int64= a$ Any, ...) -> Any",
  "$[10 = (x: int64, x: int64= a$ Any, ...) -> Any]",
  "xxx[10 = (x: int64, x: int64= a$ Any, ...) -> Any]",
  "?$10 = (x: int64, x: int64= a$ Any, ...) -> Any]",
  "?=10 = (x: int64, x: int64= a$ Any, ...) -> Any]",
  "?[10 = (x: int64, x: int64= a$ Any, ...) -> Any$",
  "?[10 = (x: int64, x: int64= a$ Any, ...) -> Any$",
  "10 * (a: $, ...) -> Any",
  "?10 * (a: $, ...) -> Any",
  "?(10 * (a: $, ...) -> Any)",
  "$10 * (a: $, ...) -> Any",
  "=10 * (a: $, ...) -> Any",
  "$[10 * (a: $, ...) -> Any]",
  "xxx[10 * (a: $, ...) -> Any]",
  "?$10 * (a: $, ...) -> Any]",
  "?=10 * (a: $, ...) -> Any]",
  "?[10 * (a: $, ...) -> Any$",
  "?[10 * (a: $, ...) -> Any$",
  "(a: $, ...) -> Any",
  "?(a: $, ...) -> Any",
  "?((a: $, ...) -> Any)",
  "$(a: $, ...) -> Any",
  "=(a: $, ...) -> Any",
  "$[(a: $, ...) -> Any]",
  "xxx[(a: $, ...) -> Any]",
  "?$(a: $, ...) -> Any]",
  "?=(a: $, ...) -> Any]",
  "?[(a: $, ...) -> Any$",
  "?[(a: $, ...) -> Any$",
  "10 $ (a: $, ...) -> Any",
  "?10 $ (a: $, ...) -> Any",
  "?(10 $ (a: $, ...) -> Any)",
  "$10 $ (a: $, ...) -> Any",
  "=10 $ (a: $, ...) -> Any",
  "$[10 $ (a: $, ...) -> Any]",
  "xxx[10 $ (a: $, ...) -> Any]",
  "?$10 $ (a: $, ...) -> Any]",
  "?=10 $ (a: $, ...) -> Any]",
  "?[10 $ (a: $, ...) -> Any$",
  "?[10 $ (a: $, ...) -> Any$",
  "10 = (a: $, ...) -> Any",
  "?10 = (a: $, ...) -> Any",
  "?(10 = (a: $, ...) -> Any)",
  "$10 = (a: $, ...) -> Any",
  "=10 = (a: $, ...) -> Any",
  "$[10 = (a: $, ...) -> Any]",
  "xxx[10 = (a: $, ...) -> Any]",
  "?$10 = (a: $, ...) -> Any]",
  "?=10 = (a: $, ...) -> Any]",
  "?[10 = (a: $, ...) -> Any$",
  "?[10 = (a: $, ...) -> Any$",
  "10 * (x: int64, a: $, ...) -> Any",
  "?10 * (x: int64, a: $, ...) -> Any",
  "?(10 * (x: int64, a: $, ...) -> Any)",
  "$10 * (x: int64, a: $, ...) -> Any",
  "=10 * (x: int64, a: $, ...) -> Any",
  "$[10 * (x: int64, a: $, ...) -> Any]",
  "xxx[10 * (x: int64, a: $, ...) -> Any]",
  "?$10 * (x: int64, a: $, ...) -> Any]",
  "?=10 * (x: int64, a: $, ...) -> Any]",
  "?[10 * (x: int64, a: $, ...) -> Any$",
  "?[10 * (x: int64, a: $, ...) -> Any$",
  "(x: int64, a: $, ...) -> Any",
  "?(x: int64, a: $, ...) -> Any",
  "?((x: int64, a: $, ...) -> Any)",
  "$(x: int64, a: $, ...) -> Any",
  "=(x: int64, a: $, ...) -> Any",
  "$[(x: int64, a: $, ...) -> Any]",
  "xxx[(x: int64, a: $, ...) -> Any]",
  "?$(x: int64, a: $, ...) -> Any]",
  "?=(x: int64, a: $, ...) -> Any]",
  "?[(x: int64, a: $, ...) -> Any$",
  "?[(x: int64, a: $, ...) -> Any$",
  "10 $ (x: int64, a: $, ...) -> Any",
  "?10 $ (x: int64, a: $, ...) -> Any",
  "?(10 $ (x: int64, a: $, ...) -> Any)",
  "$10 $ (x: int64, a: $, ...) -> Any",
  "=10 $ (x: int64, a: $, ...) -> Any",
  "$[10 $ (x: int64, a: $, ...) -> Any]",
  "xxx[10 $ (x: int64, a: $, ...) -> Any]",
  "?$10 $ (x: int64, a: $, ...) -> Any]",
  "?=10 $ (x: int64, a: $, ...) -> Any]",
  "?[10 $ (x: int64, a: $, ...) -> Any$",
  "?[10 $ (x: int64, a: $, ...) -> Any$",
  "10 = (x: int64, a: $, ...) -> Any",
  "?10 = (x: int64, a: $, ...) -> Any",
  "?(10 = (x: int64, a: $, ...) -> Any)",
  "$10 = (x: int64, a: $, ...) -> Any",
  "=10 = (x: int64, a: $, ...) -> Any",
  "$[10 = (x: int64, a: $, ...) -> Any]",
  "xxx[10 = (x: int64, a: $, ...) -> Any]",
  "?$10 = (x: int64, a: $, ...) -> Any]",
  "?=10 = (x: int64, a: $, ...) -> Any]",
  "?[10 = (x: int64, a: $, ...) -> Any$",
  "?[10 = (x: int64, a: $, ...) -> Any$",
  "10 * (x: int64$ a: $, ...) -> Any",
  "?10 * (x: int64$ a: $, ...) -> Any",
  "?(10 * (x: int64$ a: $, ...) -> Any)",
  "$10 * (x: int64$ a: $, ...) -> Any",
  "=10 * (x: int64$ a: $, ...) -> Any",
  "$[10 * (x: int64$ a: $, ...) -> Any]",
  "xxx[10 * (x: int64$ a: $, ...) -> Any]",
  "?$10 * (x: int64$ a: $, ...) -> Any]",
  "?=10 * (x: int64$ a: $, ...) -> Any]",
  "?[10 * (x: int64$ a: $, ...) -> Any$",
  "?[10 * (x: int64$ a: $, ...) -> Any$",
  "(x: int64$ a: $, ...) -> Any",
  "?(x: int64$ a: $, ...) -> Any",
  "?((x: int64$ a: $, ...) -> Any)",
  "$(x: int64$ a: $, ...) -> Any",
  "=(x: int64$ a: $, ...) -> Any",
  "$[(x: int64$ a: $, ...) -> Any]",
  "xxx[(x: int64$ a: $, ...) -> Any]",
  "?$(x: int64$ a: $, ...) -> Any]",
  "?=(x: int64$ a: $, ...) -> Any]",
  "?[(x: int64$ a: $, ...) -> Any$",
  "?[(x: int64$ a: $, ...) -> Any$",
  "10 $ (x: int64$ a: $, ...) -> Any",
  "?10 $ (x: int64$ a: $, ...) -> Any",
  "?(10 $ (x: int64$ a: $, ...) -> Any)",
  "$10 $ (x: int64$ a: $, ...) -> Any",
  "=10 $ (x: int64$ a: $, ...) -> Any",
  "$[10 $ (x: int64$ a: $, ...) -> Any]",
  "xxx[10 $ (x: int64$ a: $, ...) -> Any]",
  "?$10 $ (x: int64$ a: $, ...) -> Any]",
  "?=10 $ (x: int64$ a: $, ...) -> Any]",
  "?[10 $ (x: int64$ a: $, ...) -> Any$",
  "?[10 $ (x: int64$ a: $, ...) -> Any$",
  "10 = (x: int64$ a: $, ...) -> Any",
  "?10 = (x: int64$ a: $, ...) -> Any",
  "?(10 = (x: int64$ a: $, ...) -> Any)",
  "$10 = (x: int64$ a: $, ...) -> Any",
  "=10 = (x: int64$ a: $, ...) -> Any",
  "$[10 = (x: int64$ a: $, ...) -> Any]",
  "xxx[10 = (x: int64$ a: $, ...) -> Any]",
  "?$10 = (x: int64$ a: $, ...) -> Any]",
  "?=10 = (x: int64$ a: $, ...) -> Any]",
  "?[10 = (x: int64$ a: $, ...) -> Any$",
  "?[10 = (x: int64$ a: $, ...) -> Any$",
  "10 * (x: int64= a: $, ...) -> Any",
  "?10 * (x: int64= a: $, ...) -> Any",
  "?(10 * (x: int64= a: $, ...) -> Any)",
  "$10 * (x: int64= a: $, ...) -> Any",
  "=10 * (x: int64= a: $, ...) -> Any",
  "$[10 * (x: int64= a: $, ...) -> Any]",
  "xxx[10 * (x: int64= a: $, ...) -> Any]",
  "?$10 * (x: int64= a: $, ...) -> Any]",
  "?=10 * (x: int64= a: $, ...) -> Any]",
  "?[10 * (x: int64= a: $, ...) -> Any$",
  "?[10 * (x: int64= a: $, ...) -> Any$",
  "(x: int64= a: $, ...) -> Any",
  "?(x: int64= a: $, ...) -> Any",
  "?((x: int64= a: $, ...) -> Any)",
  "$(x: int64= a: $, ...) -> Any",
  "=(x: int64= a: $, ...) -> Any",
  "$[(x: int64= a: $, ...) -> Any]",
  "xxx[(x: int64= a: $, ...) -> Any]",
  "?$(x: int64= a: $, ...) -> Any]",
  "?=(x: int64= a: $, ...) -> Any]",
  "?[(x: int64= a: $, ...) -> Any$",
  "?[(x: int64= a: $, ...) -> Any$",
  "10 $ (x: int64= a: $, ...) -> Any",
  "?10 $ (x: int64= a: $, ...) -> Any",
  "?(10 $ (x: int64= a: $, ...) -> Any)",
  "$10 $ (x: int64= a: $, ...) -> Any",
  "=10 $ (x: int64= a: $, ...) -> Any",
  "$[10 $ (x: int64= a: $, ...) -> Any]",
  "xxx[10 $ (x: int64= a: $, ...) -> Any]",
  "?$10 $ (x: int64= a: $, ...) -> Any]",
  "?=10 $ (x: int64= a: $, ...) -> Any]",
  "?[10 $ (x: int64= a: $, ...) -> Any$",
  "?[10 $ (x: int64= a: $, ...) -> Any$",
  "10 = (x: int64= a: $, ...) -> Any",
  "?10 = (x: int64= a: $, ...) -> Any",
  "?(10 = (x: int64= a: $, ...) -> Any)",
  "$10 = (x: int64= a: $, ...) -> Any",
  "=10 = (x: int64= a: $, ...) -> Any",
  "$[10 = (x: int64= a: $, ...) -> Any]",
  "xxx[10 = (x: int64= a: $, ...) -> Any]",
  "?$10 = (x: int64= a: $, ...) -> Any]",
  "?=10 = (x: int64= a: $, ...) -> Any]",
  "?[10 = (x: int64= a: $, ...) -> Any$",
  "?[10 = (x: int64= a: $, ...) -> Any$",
  "10 * (x: int64, x: int64, a: $, ...) -> Any",
  "?10 * (x: int64, x: int64, a: $, ...) -> Any",
  "?(10 * (x: int64, x: int64, a: $, ...) -> Any)",
  "$10 * (x: int64, x: int64, a: $, ...) -> Any",
  "=10 * (x: int64, x: int64, a: $, ...) -> Any",
  "$[10 * (x: int64, x: int64, a: $, ...) -> Any]",
  "xxx[10 * (x: int64, x: int64, a: $, ...) -> Any]",
  "?$10 * (x: int64, x: int64, a: $, ...) -> Any]",
  "?=10 * (x: int64, x: int64, a: $, ...) -> Any]",
  "?[10 * (x: int64, x: int64, a: $, ...) -> Any$",
  "?[10 * (x: int64, x: int64, a: $, ...) -> Any$",
  "(x: int64, x: int64, a: $, ...) -> Any",
  "?(x: int64, x: int64, a: $, ...) -> Any",
  "?((x: int64, x: int64, a: $, ...) -> Any)",
  "$(x: int64, x: int64, a: $, ...) -> Any",
  "=(x: int64, x: int64, a: $, ...) -> Any",
  "$[(x: int64, x: int64, a: $, ...) -> Any]",
  "xxx[(x: int64, x: int64, a: $, ...) -> Any]",
  "?$(x: int64, x: int64, a: $, ...) -> Any]",
  "?=(x: int64, x: int64, a: $, ...) -> Any]",
  "?[(x: int64, x: int64, a: $, ...) -> Any$",
  "?[(x: int64, x: int64, a: $, ...) -> Any$",
  "10 $ (x: int64, x: int64, a: $, ...) -> Any",
  "?10 $ (x: int64, x: int64, a: $, ...) -> Any",
  "?(10 $ (x: int64, x: int64, a: $, ...) -> Any)",
  "$10 $ (x: int64, x: int64, a: $, ...) -> Any",
  "=10 $ (x: int64, x: int64, a: $, ...) -> Any",
  "$[10 $ (x: int64, x: int64, a: $, ...) -> Any]",
  "xxx[10 $ (x: int64, x: int64, a: $, ...) -> Any]",
  "?$10 $ (x: int64, x: int64, a: $, ...) -> Any]",
  "?=10 $ (x: int64, x: int64, a: $, ...) -> Any]",
  "?[10 $ (x: int64, x: int64, a: $, ...) -> Any$",
  "?[10 $ (x: int64, x: int64, a: $, ...) -> Any$",
  "10 = (x: int64, x: int64, a: $, ...) -> Any",
  "?10 = (x: int64, x: int64, a: $, ...) -> Any",
  "?(10 = (x: int64, x: int64, a: $, ...) -> Any)",
  "$10 = (x: int64, x: int64, a: $, ...) -> Any",
  "=10 = (x: int64, x: int64, a: $, ...) -> Any",
  "$[10 = (x: int64, x: int64, a: $, ...) -> Any]",
  "xxx[10 = (x: int64, x: int64, a: $, ...) -> Any]",
  "?$10 = (x: int64, x: int64, a: $, ...) -> Any]",
  "?=10 = (x: int64, x: int64, a: $, ...) -> Any]",
  "?[10 = (x: int64, x: int64, a: $, ...) -> Any$",
  "?[10 = (x: int64, x: int64, a: $, ...) -> Any$",
  "10 * (x: int64, x: int64$ a: $, ...) -> Any",
  "?10 * (x: int64, x: int64$ a: $, ...) -> Any",
  "?(10 * (x: int64, x: int64$ a: $, ...) -> Any)",
  "$10 * (x: int64, x: int64$ a: $, ...) -> Any",
  "=10 * (x: int64, x: int64$ a: $, ...) -> Any",
  "$[10 * (x: int64, x: int64$ a: $, ...) -> Any]",
  "xxx[10 * (x: int64, x: int64$ a: $, ...) -> Any]",
  "?$10 * (x: int64, x: int64$ a: $, ...) -> Any]",
  "?=10 * (x: int64, x: int64$ a: $, ...) -> Any]",
  "?[10 * (x: int64, x: int64$ a: $, ...) -> Any$",
  "?[10 * (x: int64, x: int64$ a: $, ...) -> Any$",
  "(x: int64, x: int64$ a: $, ...) -> Any",
  "?(x: int64, x: int64$ a: $, ...) -> Any",
  "?((x: int64, x: int64$ a: $, ...) -> Any)",
  "$(x: int64, x: int64$ a: $, ...) -> Any",
  "=(x: int64, x: int64$ a: $, ...) -> Any",
  "$[(x: int64, x: int64$ a: $, ...) -> Any]",
  "xxx[(x: int64, x: int64$ a: $, ...) -> Any]",
  "?$(x: int64, x: int64$ a: $, ...) -> Any]",
  "?=(x: int64, x: int64$ a: $, ...) -> Any]",
  "?[(x: int64, x: int64$ a: $, ...) -> Any$",
  "?[(x: int64, x: int64$ a: $, ...) -> Any$",
  "10 $ (x: int64, x: int64$ a: $, ...) -> Any",
  "?10 $ (x: int64, x: int64$ a: $, ...) -> Any",
  "?(10 $ (x: int64, x: int64$ a: $, ...) -> Any)",
  "$10 $ (x: int64, x: int64$ a: $, ...) -> Any",
  "=10 $ (x: int64, x: int64$ a: $, ...) -> Any",
  "$[10 $ (x: int64, x: int64$ a: $, ...) -> Any]",
  "xxx[10 $ (x: int64, x: int64$ a: $, ...) -> Any]",
  "?$10 $ (x: int64, x: int64$ a: $, ...) -> Any]",
  "?=10 $ (x: int64, x: int64$ a: $, ...) -> Any]",
  "?[10 $ (x: int64, x: int64$ a: $, ...) -> Any$",
  "?[10 $ (x: int64, x: int64$ a: $, ...) -> Any$",
  "10 = (x: int64, x: int64$ a: $, ...) -> Any",
  "?10 = (x: int64, x: int64$ a: $, ...) -> Any",
  "?(10 = (x: int64, x: int64$ a: $, ...) -> Any)",
  "$10 = (x: int64, x: int64$ a: $, ...) -> Any",
  "=10 = (x: int64, x: int64$ a: $, ...) -> Any",
  "$[10 = (x: int64, x: int64$ a: $, ...) -> Any]",
  "xxx[10 = (x: int64, x: int64$ a: $, ...) -> Any]",
  "?$10 = (x: int64, x: int64$ a: $, ...) -> Any]",
  "?=10 = (x: int64, x: int64$ a: $, ...) -> Any]",
  "?[10 = (x: int64, x: int64$ a: $, ...) -> Any$",
  "?[10 = (x: int64, x: int64$ a: $, ...) -> Any$",
  "10 * (x: int64, x: int64= a: $, ...) -> Any",
  "?10 * (x: int64, x: int64= a: $, ...) -> Any",
  "?(10 * (x: int64, x: int64= a: $, ...) -> Any)",
  "$10 * (x: int64, x: int64= a: $, ...) -> Any",
  "=10 * (x: int64, x: int64= a: $, ...) -> Any",
  "$[10 * (x: int64, x: int64= a: $, ...) -> Any]",
  "xxx[10 * (x: int64, x: int64= a: $, ...) -> Any]",
  "?$10 * (x: int64, x: int64= a: $, ...) -> Any]",
  "?=10 * (x: int64, x: int64= a: $, ...) -> Any]",
  "?[10 * (x: int64, x: int64= a: $, ...) -> Any$",
  "?[10 * (x: int64, x: int64= a: $, ...) -> Any$",
  "(x: int64, x: int64= a: $, ...) -> Any",
  "?(x: int64, x: int64= a: $, ...) -> Any",
  "?((x: int64, x: int64= a: $, ...) -> Any)",
  "$(x: int64, x: int64= a: $, ...) -> Any",
  "=(x: int64, x: int64= a: $, ...) -> Any",
  "$[(x: int64, x: int64= a: $, ...) -> Any]",
  "xxx[(x: int64, x: int64= a: $, ...) -> Any]",
  "?$(x: int64, x: int64= a: $, ...) -> Any]",
  "?=(x: int64, x: int64= a: $, ...) -> Any]",
  "?[(x: int64, x: int64= a: $, ...) -> Any$",
  "?[(x: int64, x: int64= a: $, ...) -> Any$",
  "10 $ (x: int64, x: int64= a: $, ...) -> Any",
  "?10 $ (x: int64, x: int64= a: $, ...) -> Any",
  "?(10 $ (x: int64, x: int64= a: $, ...) -> Any)",
  "$10 $ (x: int64, x: int64= a: $, ...) -> Any",
  "=10 $ (x: int64, x: int64= a: $, ...) -> Any",
  "$[10 $ (x: int64, x: int64= a: $, ...) -> Any]",
  "xxx[10 $ (x: int64, x: int64= a: $, ...) -> Any]",
  "?$10 $ (x: int64, x: int64= a: $, ...) -> Any]",
  "?=10 $ (x: int64, x: int64= a: $, ...) -> Any]",
  "?[10 $ (x: int64, x: int64= a: $, ...) -> Any$",
  "?[10 $ (x: int64, x: int64= a: $, ...) -> Any$",
  "10 = (x: int64, x: int64= a: $, ...) -> Any",
  "?10 = (x: int64, x: int64= a: $, ...) -> Any",
  "?(10 = (x: int64, x: int64= a: $, ...) -> Any)",
  "$10 = (x: int64, x: int64= a: $, ...) -> Any",
  "=10 = (x: int64, x: int64= a: $, ...) -> Any",
  "$[10 = (x: int64, x: int64= a: $, ...) -> Any]",
  "xxx[10 = (x: int64, x: int64= a: $, ...) -> Any]",
  "?$10 = (x: int64, x: int64= a: $, ...) -> Any]",
  "?=10 = (x: int64, x: int64= a: $, ...) -> Any]",
  "?[10 = (x: int64, x: int64= a: $, ...) -> Any$",
  "?[10 = (x: int64, x: int64= a: $, ...) -> Any$",
  "10 * (a: xxx, ...) -> Any",
  "?10 * (a: xxx, ...) -> Any",
  "?(10 * (a: xxx, ...) -> Any)",
  "$10 * (a: xxx, ...) -> Any",
  "=10 * (a: xxx, ...) -> Any",
  "$[10 * (a: xxx, ...) -> Any]",
  "xxx[10 * (a: xxx, ...) -> Any]",
  "?$10 * (a: xxx, ...) -> Any]",
  "?=10 * (a: xxx, ...) -> Any]",
  "?[10 * (a: xxx, ...) -> Any$",
  "?[10 * (a: xxx, ...) -> Any$",
  "(a: xxx, ...) -> Any",
  "?(a: xxx, ...) -> Any",
  "?((a: xxx, ...) -> Any)",
  "$(a: xxx, ...) -> Any",
  "=(a: xxx, ...) -> Any",
  "$[(a: xxx, ...) -> Any]",
  "xxx[(a: xxx, ...) -> Any]",
  "?$(a: xxx, ...) -> Any]",
  "?=(a: xxx, ...) -> Any]",
  "?[(a: xxx, ...) -> Any$",
  "?[(a: xxx, ...) -> Any$",
  "10 $ (a: xxx, ...) -> Any",
  "?10 $ (a: xxx, ...) -> Any",
  "?(10 $ (a: xxx, ...) -> Any)",
  "$10 $ (a: xxx, ...) -> Any",
  "=10 $ (a: xxx, ...) -> Any",
  "$[10 $ (a: xxx, ...) -> Any]",
  "xxx[10 $ (a: xxx, ...) -> Any]",
  "?$10 $ (a: xxx, ...) -> Any]",
  "?=10 $ (a: xxx, ...) -> Any]",
  "?[10 $ (a: xxx, ...) -> Any$",
  "?[10 $ (a: xxx, ...) -> Any$",
  "10 = (a: xxx, ...) -> Any",
  "?10 = (a: xxx, ...) -> Any",
  "?(10 = (a: xxx, ...) -> Any)",
  "$10 = (a: xxx, ...) -> Any",
  "=10 = (a: xxx, ...) -> Any",
  "$[10 = (a: xxx, ...) -> Any]",
  "xxx[10 = (a: xxx, ...) -> Any]",
  "?$10 = (a: xxx, ...) -> Any]",
  "?=10 = (a: xxx, ...) -> Any]",
  "?[10 = (a: xxx, ...) -> Any$",
  "?[10 = (a: xxx, ...) -> Any$",
  "10 * (x: int64, a: xxx, ...) -> Any",
  "?10 * (x: int64, a: xxx, ...) -> Any",
  "?(10 * (x: int64, a: xxx, ...) -> Any)",
  "$10 * (x: int64, a: xxx, ...) -> Any",
  "=10 * (x: int64, a: xxx, ...) -> Any",
  "$[10 * (x: int64, a: xxx, ...) -> Any]",
  "xxx[10 * (x: int64, a: xxx, ...) -> Any]",
  "?$10 * (x: int64, a: xxx, ...) -> Any]",
  "?=10 * (x: int64, a: xxx, ...) -> Any]",
  "?[10 * (x: int64, a: xxx, ...) -> Any$",
  "?[10 * (x: int64, a: xxx, ...) -> Any$",
  "(x: int64, a: xxx, ...) -> Any",
  "?(x: int64, a: xxx, ...) -> Any",
  "?((x: int64, a: xxx, ...) -> Any)",
  "$(x: int64, a: xxx, ...) -> Any",
  "=(x: int64, a: xxx, ...) -> Any",
  "$[(x: int64, a: xxx, ...) -> Any]",
  "xxx[(x: int64, a: xxx, ...) -> Any]",
  "?$(x: int64, a: xxx, ...) -> Any]",
  "?=(x: int64, a: xxx, ...) -> Any]",
  "?[(x: int64, a: xxx, ...) -> Any$",
  "?[(x: int64, a: xxx, ...) -> Any$",
  "10 $ (x: int64, a: xxx, ...) -> Any",
  "?10 $ (x: int64, a: xxx, ...) -> Any",
  "?(10 $ (x: int64, a: xxx, ...) -> Any)",
  "$10 $ (x: int64, a: xxx, ...) -> Any",
  "=10 $ (x: int64, a: xxx, ...) -> Any",
  "$[10 $ (x: int64, a: xxx, ...) -> Any]",
  "xxx[10 $ (x: int64, a: xxx, ...) -> Any]",
  "?$10 $ (x: int64, a: xxx, ...) -> Any]",
  "?=10 $ (x: int64, a: xxx, ...) -> Any]",
  "?[10 $ (x: int64, a: xxx, ...) -> Any$",
  "?[10 $ (x: int64, a: xxx, ...) -> Any$",
  "10 = (x: int64, a: xxx, ...) -> Any",
  "?10 = (x: int64, a: xxx, ...) -> Any",
  "?(10 = (x: int64, a: xxx, ...) -> Any)",
  "$10 = (x: int64, a: xxx, ...) -> Any",
  "=10 = (x: int64, a: xxx, ...) -> Any",
  "$[10 = (x: int64, a: xxx, ...) -> Any]",
  "xxx[10 = (x: int64, a: xxx, ...) -> Any]",
  "?$10 = (x: int64, a: xxx, ...) -> Any]",
  "?=10 = (x: int64, a: xxx, ...) -> Any]",
  "?[10 = (x: int64, a: xxx, ...) -> Any$",
  "?[10 = (x: int64, a: xxx, ...) -> Any$",
  "10 * (x: int64$ a: xxx, ...) -> Any",
  "?10 * (x: int64$ a: xxx, ...) -> Any",
  "?(10 * (x: int64$ a: xxx, ...) -> Any)",
  "$10 * (x: int64$ a: xxx, ...) -> Any",
  "=10 * (x: int64$ a: xxx, ...) -> Any",
  "$[10 * (x: int64$ a: xxx, ...) -> Any]",
  "xxx[10 * (x: int64$ a: xxx, ...) -> Any]",
  "?$10 * (x: int64$ a: xxx, ...) -> Any]",
  "?=10 * (x: int64$ a: xxx, ...) -> Any]",
  "?[10 * (x: int64$ a: xxx, ...) -> Any$",
  "?[10 * (x: int64$ a: xxx, ...) -> Any$",
  "(x: int64$ a: xxx, ...) -> Any",
  "?(x: int64$ a: xxx, ...) -> Any",
  "?((x: int64$ a: xxx, ...) -> Any)",
  "$(x: int64$ a: xxx, ...) -> Any",
  "=(x: int64$ a: xxx, ...) -> Any",
  "$[(x: int64$ a: xxx, ...) -> Any]",
  "xxx[(x: int64$ a: xxx, ...) -> Any]",
  "?$(x: int64$ a: xxx, ...) -> Any]",
  "?=(x: int64$ a: xxx, ...) -> Any]",
  "?[(x: int64$ a: xxx, ...) -> Any$",
  "?[(x: int64$ a: xxx, ...) -> Any$",
  "10 $ (x: int64$ a: xxx, ...) -> Any",
  "?10 $ (x: int64$ a: xxx, ...) -> Any",
  "?(10 $ (x: int64$ a: xxx, ...) -> Any)",
  "$10 $ (x: int64$ a: xxx, ...) -> Any",
  "=10 $ (x: int64$ a: xxx, ...) -> Any",
  "$[10 $ (x: int64$ a: xxx, ...) -> Any]",
  "xxx[10 $ (x: int64$ a: xxx, ...) -> Any]",
  "?$10 $ (x: int64$ a: xxx, ...) -> Any]",
  "?=10 $ (x: int64$ a: xxx, ...) -> Any]",
  "?[10 $ (x: int64$ a: xxx, ...) -> Any$",
  "?[10 $ (x: int64$ a: xxx, ...) -> Any$",
  "10 = (x: int64$ a: xxx, ...) -> Any",
  "?10 = (x: int64$ a: xxx, ...) -> Any",
  "?(10 = (x: int64$ a: xxx, ...) -> Any)",
  "$10 = (x: int64$ a: xxx, ...) -> Any",
  "=10 = (x: int64$ a: xxx, ...) -> Any",
  "$[10 = (x: int64$ a: xxx, ...) -> Any]",
  "xxx[10 = (x: int64$ a: xxx, ...) -> Any]",
  "?$10 = (x: int64$ a: xxx, ...) -> Any]",
  "?=10 = (x: int64$ a: xxx, ...) -> Any]",
  "?[10 = (x: int64$ a: xxx, ...) -> Any$",
  "?[10 = (x: int64$ a: xxx, ...) -> Any$",
  "10 * (x: int64= a: xxx, ...) -> Any",
  "?10 * (x: int64= a: xxx, ...) -> Any",
  "?(10 * (x: int64= a: xxx, ...) -> Any)",
  "$10 * (x: int64= a: xxx, ...) -> Any",
  "=10 * (x: int64= a: xxx, ...) -> Any",
  "$[10 * (x: int64= a: xxx, ...) -> Any]",
  "xxx[10 * (x: int64= a: xxx, ...) -> Any]",
  "?$10 * (x: int64= a: xxx, ...) -> Any]",
  "?=10 * (x: int64= a: xxx, ...) -> Any]",
  "?[10 * (x: int64= a: xxx, ...) -> Any$",
  "?[10 * (x: int64= a: xxx, ...) -> Any$",
  "(x: int64= a: xxx, ...) -> Any",
  "?(x: int64= a: xxx, ...) -> Any",
  "?((x: int64= a: xxx, ...) -> Any)",
  "$(x: int64= a: xxx, ...) -> Any",
  "=(x: int64= a: xxx, ...) -> Any",
  "$[(x: int64= a: xxx, ...) -> Any]",
  "xxx[(x: int64= a: xxx, ...) -> Any]",
  "?$(x: int64= a: xxx, ...) -> Any]",
  "?=(x: int64= a: xxx, ...) -> Any]",
  "?[(x: int64= a: xxx, ...) -> Any$",
  "?[(x: int64= a: xxx, ...) -> Any$",
  "10 $ (x: int64= a: xxx, ...) -> Any",
  "?10 $ (x: int64= a: xxx, ...) -> Any",
  "?(10 $ (x: int64= a: xxx, ...) -> Any)",
  "$10 $ (x: int64= a: xxx, ...) -> Any",
  "=10 $ (x: int64= a: xxx, ...) -> Any",
  "$[10 $ (x: int64= a: xxx, ...) -> Any]",
  "xxx[10 $ (x: int64= a: xxx, ...) -> Any]",
  "?$10 $ (x: int64= a: xxx, ...) -> Any]",
  "?=10 $ (x: int64= a: xxx, ...) -> Any]",
  "?[10 $ (x: int64= a: xxx, ...) -> Any$",
  "?[10 $ (x: int64= a: xxx, ...) -> Any$",
  "10 = (x: int64= a: xxx, ...) -> Any",
  "?10 = (x: int64= a: xxx, ...) -> Any",
  "?(10 = (x: int64= a: xxx, ...) -> Any)",
  "$10 = (x: int64= a: xxx, ...) -> Any",
  "=10 = (x: int64= a: xxx, ...) -> Any",
  "$[10 = (x: int64= a: xxx, ...) -> Any]",
  "xxx[10 = (x: int64= a: xxx, ...) -> Any]",
  "?$10 = (x: int64= a: xxx, ...) -> Any]",
  "?=10 = (x: int64= a: xxx, ...) -> Any]",
  "?[10 = (x: int64= a: xxx, ...) -> Any$",
  "?[10 = (x: int64= a: xxx, ...) -> Any$",
  "10 * (x: int64, x: int64, a: xxx, ...) -> Any",
  "?10 * (x: int64, x: int64, a: xxx, ...) -> Any",
  "?(10 * (x: int64, x: int64, a: xxx, ...) -> Any)",
  "$10 * (x: int64, x: int64, a: xxx, ...) -> Any",
  "=10 * (x: int64, x: int64, a: xxx, ...) -> Any",
  "$[10 * (x: int64, x: int64, a: xxx, ...) -> Any]",
  "xxx[10 * (x: int64, x: int64, a: xxx, ...) -> Any]",
  "?$10 * (x: int64, x: int64, a: xxx, ...) -> Any]",
  "?=10 * (x: int64, x: int64, a: xxx, ...) -> Any]",
  "?[10 * (x: int64, x: int64, a: xxx, ...) -> Any$",
  "?[10 * (x: int64, x: int64, a: xxx, ...) -> Any$",
  "(x: int64, x: int64, a: xxx, ...) -> Any",
  "?(x: int64, x: int64, a: xxx, ...) -> Any",
  "?((x: int64, x: int64, a: xxx, ...) -> Any)",
  "$(x: int64, x: int64, a: xxx, ...) -> Any",
  "=(x: int64, x: int64, a: xxx, ...) -> Any",
  "$[(x: int64, x: int64, a: xxx, ...) -> Any]",
  "xxx[(x: int64, x: int64, a: xxx, ...) -> Any]",
  "?$(x: int64, x: int64, a: xxx, ...) -> Any]",
  "?=(x: int64, x: int64, a: xxx, ...) -> Any]",
  "?[(x: int64, x: int64, a: xxx, ...) -> Any$",
  "?[(x: int64, x: int64, a: xxx, ...) -> Any$",
  "10 $ (x: int64, x: int64, a: xxx, ...) -> Any",
  "?10 $ (x: int64, x: int64, a: xxx, ...) -> Any",
  "?(10 $ (x: int64, x: int64, a: xxx, ...) -> Any)",
  "$10 $ (x: int64, x: int64, a: xxx, ...) -> Any",
  "=10 $ (x: int64, x: int64, a: xxx, ...) -> Any",
  "$[10 $ (x: int64, x: int64, a: xxx, ...) -> Any]",
  "xxx[10 $ (x: int64, x: int64, a: xxx, ...) -> Any]",
  "?$10 $ (x: int64, x: int64, a: xxx, ...) -> Any]",
  "?=10 $ (x: int64, x: int64, a: xxx, ...) -> Any]",
  "?[10 $ (x: int64, x: int64, a: xxx, ...) -> Any$",
  "?[10 $ (x: int64, x: int64, a: xxx, ...) -> Any$",
  "10 = (x: int64, x: int64, a: xxx, ...) -> Any",
  "?10 = (x: int64, x: int64, a: xxx, ...) -> Any",
  "?(10 = (x: int64, x: int64, a: xxx, ...) -> Any)",
  "$10 = (x: int64, x: int64, a: xxx, ...) -> Any",
  "=10 = (x: int64, x: int64, a: xxx, ...) -> Any",
  "$[10 = (x: int64, x: int64, a: xxx, ...) -> Any]",
  "xxx[10 = (x: int64, x: int64, a: xxx, ...) -> Any]",
  "?$10 = (x: int64, x: int64, a: xxx, ...) -> Any]",
  "?=10 = (x: int64, x: int64, a: xxx, ...) -> Any]",
  "?[10 = (x: int64, x: int64, a: xxx, ...) -> Any$",
  "?[10 = (x: int64, x: int64, a: xxx, ...) -> Any$",
  "10 * (x: int64, x: int64$ a: xxx, ...) -> Any",
  "?10 * (x: int64, x: int64$ a: xxx, ...) -> Any",
  "?(10 * (x: int64, x: int64$ a: xxx, ...) -> Any)",
  "$10 * (x: int64, x: int64$ a: xxx, ...) -> Any",
  "=10 * (x: int64, x: int64$ a: xxx, ...) -> Any",
  "$[10 * (x: int64, x: int64$ a: xxx, ...) -> Any]",
  "xxx[10 * (x: int64, x: int64$ a: xxx, ...) -> Any]",
  "?$10 * (x: int64, x: int64$ a: xxx, ...) -> Any]",
  "?=10 * (x: int64, x: int64$ a: xxx, ...) -> Any]",
  "?[10 * (x: int64, x: int64$ a: xxx, ...) -> Any$",
  "?[10 * (x: int64, x: int64$ a: xxx, ...) -> Any$",
  "(x: int64, x: int64$ a: xxx, ...) -> Any",
  "?(x: int64, x: int64$ a: xxx, ...) -> Any",
  "?((x: int64, x: int64$ a: xxx, ...) -> Any)",
  "$(x: int64, x: int64$ a: xxx, ...) -> Any",
  "=(x: int64, x: int64$ a: xxx, ...) -> Any",
  "$[(x: int64, x: int64$ a: xxx, ...) -> Any]",
  "xxx[(x: int64, x: int64$ a: xxx, ...) -> Any]",
  "?$(x: int64, x: int64$ a: xxx, ...) -> Any]",
  "?=(x: int64, x: int64$ a: xxx, ...) -> Any]",
  "?[(x: int64, x: int64$ a: xxx, ...) -> Any$",
  "?[(x: int64, x: int64$ a: xxx, ...) -> Any$",
  "10 $ (x: int64, x: int64$ a: xxx, ...) -> Any",
  "?10 $ (x: int64, x: int64$ a: xxx, ...) -> Any",
  "?(10 $ (x: int64, x: int64$ a: xxx, ...) -> Any)",
  "$10 $ (x: int64, x: int64$ a: xxx, ...) -> Any",
  "=10 $ (x: int64, x: int64$ a: xxx, ...) -> Any",
  "$[10 $ (x: int64, x: int64$ a: xxx, ...) -> Any]",
  "xxx[10 $ (x: int64, x: int64$ a: xxx, ...) -> Any]",
  "?$10 $ (x: int64, x: int64$ a: xxx, ...) -> Any]",
  "?=10 $ (x: int64, x: int64$ a: xxx, ...) -> Any]",
  "?[10 $ (x: int64, x: int64$ a: xxx, ...) -> Any$",
  "?[10 $ (x: int64, x: int64$ a: xxx, ...) -> Any$",
  "10 = (x: int64, x: int64$ a: xxx, ...) -> Any",
  "?10 = (x: int64, x: int64$ a: xxx, ...) -> Any",
  "?(10 = (x: int64, x: int64$ a: xxx, ...) -> Any)",
  "$10 = (x: int64, x: int64$ a: xxx, ...) -> Any",
  "=10 = (x: int64, x: int64$ a: xxx, ...) -> Any",
  "$[10 = (x: int64, x: int64$ a: xxx, ...) -> Any]",
  "xxx[10 = (x: int64, x: int64$ a: xxx, ...) -> Any]",
  "?$10 = (x: int64, x: int64$ a: xxx, ...) -> Any]",
  "?=10 = (x: int64, x: int64$ a: xxx, ...) -> Any]",
  "?[10 = (x: int64, x: int64$ a: xxx, ...) -> Any$",
  "?[10 = (x: int64, x: int64$ a: xxx, ...) -> Any$",
  "10 * (x: int64, x: int64= a: xxx, ...) -> Any",
  "?10 * (x: int64, x: int64= a: xxx, ...) -> Any",
  "?(10 * (x: int64, x: int64= a: xxx, ...) -> Any)",
  "$10 * (x: int64, x: int64= a: xxx, ...) -> Any",
  "=10 * (x: int64, x: int64= a: xxx, ...) -> Any",
  "$[10 * (x: int64, x: int64= a: xxx, ...) -> Any]",
  "xxx[10 * (x: int64, x: int64= a: xxx, ...) -> Any]",
  "?$10 * (x: int64, x: int64= a: xxx, ...) -> Any]",
  "?=10 * (x: int64, x: int64= a: xxx, ...) -> Any]",
  "?[10 * (x: int64, x: int64= a: xxx, ...) -> Any$",
  "?[10 * (x: int64, x: int64= a: xxx, ...) -> Any$",
  "(x: int64, x: int64= a: xxx, ...) -> Any",
  "?(x: int64, x: int64= a: xxx, ...) -> Any",
  "?((x: int64, x: int64= a: xxx, ...) -> Any)",
  "$(x: int64, x: int64= a: xxx, ...) -> Any",
  "=(x: int64, x: int64= a: xxx, ...) -> Any",
  "$[(x: int64, x: int64= a: xxx, ...) -> Any]",
  "xxx[(x: int64, x: int64= a: xxx, ...) -> Any]",
  "?$(x: int64, x: int64= a: xxx, ...) -> Any]",
  "?=(x: int64, x: int64= a: xxx, ...) -> Any]",
  "?[(x: int64, x: int64= a: xxx, ...) -> Any$",
  "?[(x: int64, x: int64= a: xxx, ...) -> Any$",
  "10 $ (x: int64, x: int64= a: xxx, ...) -> Any",
  "?10 $ (x: int64, x: int64= a: xxx, ...) -> Any",
  "?(10 $ (x: int64, x: int64= a: xxx, ...) -> Any)",
  "$10 $ (x: int64, x: int64= a: xxx, ...) -> Any",
  "=10 $ (x: int64, x: int64= a: xxx, ...) -> Any",
  "$[10 $ (x: int64, x: int64= a: xxx, ...) -> Any]",
  "xxx[10 $ (x: int64, x: int64= a: xxx, ...) -> Any]",
  "?$10 $ (x: int64, x: int64= a: xxx, ...) -> Any]",
  "?=10 $ (x: int64, x: int64= a: xxx, ...) -> Any]",
  "?[10 $ (x: int64, x: int64= a: xxx, ...) -> Any$",
  "?[10 $ (x: int64, x: int64= a: xxx, ...) -> Any$",
  "10 = (x: int64, x: int64= a: xxx, ...) -> Any",
  "?10 = (x: int64, x: int64= a: xxx, ...) -> Any",
  "?(10 = (x: int64, x: int64= a: xxx, ...) -> Any)",
  "$10 = (x: int64, x: int64= a: xxx, ...) -> Any",
  "=10 = (x: int64, x: int64= a: xxx, ...) -> Any",
  "$[10 = (x: int64, x: int64= a: xxx, ...) -> Any]",
  "xxx[10 = (x: int64, x: int64= a: xxx, ...) -> Any]",
  "?$10 = (x: int64, x: int64= a: xxx, ...) -> Any]",
  "?=10 = (x: int64, x: int64= a: xxx, ...) -> Any]",
  "?[10 = (x: int64, x: int64= a: xxx, ...) -> Any$",
  "?[10 = (x: int64, x: int64= a: xxx, ...) -> Any$",
  "$10 * (Any) -> Any",
  "=10 * (Any) -> Any",
  "$[10 * (Any) -> Any]",
  "xxx[10 * (Any) -> Any]",
  "?$10 * (Any) -> Any]",
  "?=10 * (Any) -> Any]",
  "?[10 * (Any) -> Any$",
  "?[10 * (Any) -> Any$",
  "$(Any) -> Any",
  "=(Any) -> Any",
  "$[(Any) -> Any]",
  "xxx[(Any) -> Any]",
  "?$(Any) -> Any]",
  "?=(Any) -> Any]",
  "?[(Any) -> Any$",
  "?[(Any) -> Any$",
  "10 $ (Any) -> Any",
  "?10 $ (Any) -> Any",
  "?(10 $ (Any) -> Any)",
  "$10 $ (Any) -> Any",
  "=10 $ (Any) -> Any",
  "$[10 $ (Any) -> Any]",
  "xxx[10 $ (Any) -> Any]",
  "?$10 $ (Any) -> Any]",
  "?=10 $ (Any) -> Any]",
  "?[10 $ (Any) -> Any$",
  "?[10 $ (Any) -> Any$",
  "10 = (Any) -> Any",
  "?10 = (Any) -> Any",
  "?(10 = (Any) -> Any)",
  "$10 = (Any) -> Any",
  "=10 = (Any) -> Any",
  "$[10 = (Any) -> Any]",
  "xxx[10 = (Any) -> Any]",
  "?$10 = (Any) -> Any]",
  "?=10 = (Any) -> Any]",
  "?[10 = (Any) -> Any$",
  "?[10 = (Any) -> Any$",
  "$10 * (Any, ) -> Any",
  "=10 * (Any, ) -> Any",
  "$[10 * (Any, ) -> Any]",
  "xxx[10 * (Any, ) -> Any]",
  "?$10 * (Any, ) -> Any]",
  "?=10 * (Any, ) -> Any]",
  "?[10 * (Any, ) -> Any$",
  "?[10 * (Any, ) -> Any$",
  "$(Any, ) -> Any",
  "=(Any, ) -> Any",
  "$[(Any, ) -> Any]",
  "xxx[(Any, ) -> Any]",
  "?$(Any, ) -> Any]",
  "?=(Any, ) -> Any]",
  "?[(Any, ) -> Any$",
  "?[(Any, ) -> Any$",
  "10 $ (Any, ) -> Any",
  "?10 $ (Any, ) -> Any",
  "?(10 $ (Any, ) -> Any)",
  "$10 $ (Any, ) -> Any",
  "=10 $ (Any, ) -> Any",
  "$[10 $ (Any, ) -> Any]",
  "xxx[10 $ (Any, ) -> Any]",
  "?$10 $ (Any, ) -> Any]",
  "?=10 $ (Any, ) -> Any]",
  "?[10 $ (Any, ) -> Any$",
  "?[10 $ (Any, ) -> Any$",
  "10 = (Any, ) -> Any",
  "?10 = (Any, ) -> Any",
  "?(10 = (Any, ) -> Any)",
  "$10 = (Any, ) -> Any",
  "=10 = (Any, ) -> Any",
  "$[10 = (Any, ) -> Any]",
  "xxx[10 = (Any, ) -> Any]",
  "?$10 = (Any, ) -> Any]",
  "?=10 = (Any, ) -> Any]",
  "?[10 = (Any, ) -> Any$",
  "?[10 = (Any, ) -> Any$",
  "$10 * (Any, ...) -> Any",
  "=10 * (Any, ...) -> Any",
  "$[10 * (Any, ...) -> Any]",
  "xxx[10 * (Any, ...) -> Any]",
  "?$10 * (Any, ...) -> Any]",
  "?=10 * (Any, ...) -> Any]",
  "?[10 * (Any, ...) -> Any$",
  "?[10 * (Any, ...) -> Any$",
  "$(Any, ...) -> Any",
  "=(Any, ...) -> Any",
  "$[(Any, ...) -> Any]",
  "xxx[(Any, ...) -> Any]",
  "?$(Any, ...) -> Any]",
  "?=(Any, ...) -> Any]",
  "?[(Any, ...) -> Any$",
  "?[(Any, ...) -> Any$",
  "10 $ (Any, ...) -> Any",
  "?10 $ (Any, ...) -> Any",
  "?(10 $ (Any, ...) -> Any)",
  "$10 $ (Any, ...) -> Any",
  "=10 $ (Any, ...) -> Any",
  "$[10 $ (Any, ...) -> Any]",
  "xxx[10 $ (Any, ...) -> Any]",
  "?$10 $ (Any, ...) -> Any]",
  "?=10 $ (Any, ...) -> Any]",
  "?[10 $ (Any, ...) -> Any$",
  "?[10 $ (Any, ...) -> Any$",
  "10 = (Any, ...) -> Any",
  "?10 = (Any, ...) -> Any",
  "?(10 = (Any, ...) -> Any)",
  "$10 = (Any, ...) -> Any",
  "=10 = (Any, ...) -> Any",
  "$[10 = (Any, ...) -> Any]",
  "xxx[10 = (Any, ...) -> Any]",
  "?$10 = (Any, ...) -> Any]",
  "?=10 = (Any, ...) -> Any]",
  "?[10 = (Any, ...) -> Any$",
  "?[10 = (Any, ...) -> Any$",
  "10 * (Any, $) -> Any",
  "?10 * (Any, $) -> Any",
  "?(10 * (Any, $) -> Any)",
  "$10 * (Any, $) -> Any",
  "=10 * (Any, $) -> Any",
  "$[10 * (Any, $) -> Any]",
  "xxx[10 * (Any, $) -> Any]",
  "?$10 * (Any, $) -> Any]",
  "?=10 * (Any, $) -> Any]",
  "?[10 * (Any, $) -> Any$",
  "?[10 * (Any, $) -> Any$",
  "(Any, $) -> Any",
  "?(Any, $) -> Any",
  "?((Any, $) -> Any)",
  "$(Any, $) -> Any",
  "=(Any, $) -> Any",
  "$[(Any, $) -> Any]",
  "xxx[(Any, $) -> Any]",
  "?$(Any, $) -> Any]",
  "?=(Any, $) -> Any]",
  "?[(Any, $) -> Any$",
  "?[(Any, $) -> Any$",
  "10 $ (Any, $) -> Any",
  "?10 $ (Any, $) -> Any",
  "?(10 $ (Any, $) -> Any)",
  "$10 $ (Any, $) -> Any",
  "=10 $ (Any, $) -> Any",
  "$[10 $ (Any, $) -> Any]",
  "xxx[10 $ (Any, $) -> Any]",
  "?$10 $ (Any, $) -> Any]",
  "?=10 $ (Any, $) -> Any]",
  "?[10 $ (Any, $) -> Any$",
  "?[10 $ (Any, $) -> Any$",
  "10 = (Any, $) -> Any",
  "?10 = (Any, $) -> Any",
  "?(10 = (Any, $) -> Any)",
  "$10 = (Any, $) -> Any",
  "=10 = (Any, $) -> Any",
  "$[10 = (Any, $) -> Any]",
  "xxx[10 = (Any, $) -> Any]",
  "?$10 = (Any, $) -> Any]",
  "?=10 = (Any, $) -> Any]",
  "?[10 = (Any, $) -> Any$",
  "?[10 = (Any, $) -> Any$",
  "$10 * (Any, a : Any) -> Any",
  "=10 * (Any, a : Any) -> Any",
  "$[10 * (Any, a : Any) -> Any]",
  "xxx[10 * (Any, a : Any) -> Any]",
  "?$10 * (Any, a : Any) -> Any]",
  "?=10 * (Any, a : Any) -> Any]",
  "?[10 * (Any, a : Any) -> Any$",
  "?[10 * (Any, a : Any) -> Any$",
  "$(Any, a : Any) -> Any",
  "=(Any, a : Any) -> Any",
  "$[(Any, a : Any) -> Any]",
  "xxx[(Any, a : Any) -> Any]",
  "?$(Any, a : Any) -> Any]",
  "?=(Any, a : Any) -> Any]",
  "?[(Any, a : Any) -> Any$",
  "?[(Any, a : Any) -> Any$",
  "10 $ (Any, a : Any) -> Any",
  "?10 $ (Any, a : Any) -> Any",
  "?(10 $ (Any, a : Any) -> Any)",
  "$10 $ (Any, a : Any) -> Any",
  "=10 $ (Any, a : Any) -> Any",
  "$[10 $ (Any, a : Any) -> Any]",
  "xxx[10 $ (Any, a : Any) -> Any]",
  "?$10 $ (Any, a : Any) -> Any]",
  "?=10 $ (Any, a : Any) -> Any]",
  "?[10 $ (Any, a : Any) -> Any$",
  "?[10 $ (Any, a : Any) -> Any$",
  "10 = (Any, a : Any) -> Any",
  "?10 = (Any, a : Any) -> Any",
  "?(10 = (Any, a : Any) -> Any)",
  "$10 = (Any, a : Any) -> Any",
  "=10 = (Any, a : Any) -> Any",
  "$[10 = (Any, a : Any) -> Any]",
  "xxx[10 = (Any, a : Any) -> Any]",
  "?$10 = (Any, a : Any) -> Any]",
  "?=10 = (Any, a : Any) -> Any]",
  "?[10 = (Any, a : Any) -> Any$",
  "?[10 = (Any, a : Any) -> Any$",
  "$10 * (Any, ..., a : Any) -> Any",
  "=10 * (Any, ..., a : Any) -> Any",
  "$[10 * (Any, ..., a : Any) -> Any]",
  "xxx[10 * (Any, ..., a : Any) -> Any]",
  "?$10 * (Any, ..., a : Any) -> Any]",
  "?=10 * (Any, ..., a : Any) -> Any]",
  "?[10 * (Any, ..., a : Any) -> Any$",
  "?[10 * (Any, ..., a : Any) -> Any$",
  "$(Any, ..., a : Any) -> Any",
  "=(Any, ..., a : Any) -> Any",
  "$[(Any, ..., a : Any) -> Any]",
  "xxx[(Any, ..., a : Any) -> Any]",
  "?$(Any, ..., a : Any) -> Any]",
  "?=(Any, ..., a : Any) -> Any]",
  "?[(Any, ..., a : Any) -> Any$",
  "?[(Any, ..., a : Any) -> Any$",
  "10 $ (Any, ..., a : Any) -> Any",
  "?10 $ (Any, ..., a : Any) -> Any",
  "?(10 $ (Any, ..., a : Any) -> Any)",
  "$10 $ (Any, ..., a : Any) -> Any",
  "=10 $ (Any, ..., a : Any) -> Any",
  "$[10 $ (Any, ..., a : Any) -> Any]",
  "xxx[10 $ (Any, ..., a : Any) -> Any]",
  "?$10 $ (Any, ..., a : Any) -> Any]",
  "?=10 $ (Any, ..., a : Any) -> Any]",
  "?[10 $ (Any, ..., a : Any) -> Any$",
  "?[10 $ (Any, ..., a : Any) -> Any$",
  "10 = (Any, ..., a : Any) -> Any",
  "?10 = (Any, ..., a : Any) -> Any",
  "?(10 = (Any, ..., a : Any) -> Any)",
  "$10 = (Any, ..., a : Any) -> Any",
  "=10 = (Any, ..., a : Any) -> Any",
  "$[10 = (Any, ..., a : Any) -> Any]",
  "xxx[10 = (Any, ..., a : Any) -> Any]",
  "?$10 = (Any, ..., a : Any) -> Any]",
  "?=10 = (Any, ..., a : Any) -> Any]",
  "?[10 = (Any, ..., a : Any) -> Any$",
  "?[10 = (Any, ..., a : Any) -> Any$",
  "$10 * (int64, Any, a : Any) -> Any",
  "=10 * (int64, Any, a : Any) -> Any",
  "$[10 * (int64, Any, a : Any) -> Any]",
  "xxx[10 * (int64, Any, a : Any) -> Any]",
  "?$10 * (int64, Any, a : Any) -> Any]",
  "?=10 * (int64, Any, a : Any) -> Any]",
  "?[10 * (int64, Any, a : Any) -> Any$",
  "?[10 * (int64, Any, a : Any) -> Any$",
  "$(int64, Any, a : Any) -> Any",
  "=(int64, Any, a : Any) -> Any",
  "$[(int64, Any, a : Any) -> Any]",
  "xxx[(int64, Any, a : Any) -> Any]",
  "?$(int64, Any, a : Any) -> Any]",
  "?=(int64, Any, a : Any) -> Any]",
  "?[(int64, Any, a : Any) -> Any$",
  "?[(int64, Any, a : Any) -> Any$",
  "10 $ (int64, Any, a : Any) -> Any",
  "?10 $ (int64, Any, a : Any) -> Any",
  "?(10 $ (int64, Any, a : Any) -> Any)",
  "$10 $ (int64, Any, a : Any) -> Any",
  "=10 $ (int64, Any, a : Any) -> Any",
  "$[10 $ (int64, Any, a : Any) -> Any]",
  "xxx[10 $ (int64, Any, a : Any) -> Any]",
  "?$10 $ (int64, Any, a : Any) -> Any]",
  "?=10 $ (int64, Any, a : Any) -> Any]",
  "?[10 $ (int64, Any, a : Any) -> Any$",
  "?[10 $ (int64, Any, a : Any) -> Any$",
  "10 = (int64, Any, a : Any) -> Any",
  "?10 = (int64, Any, a : Any) -> Any",
  "?(10 = (int64, Any, a : Any) -> Any)",
  "$10 = (int64, Any, a : Any) -> Any",
  "=10 = (int64, Any, a : Any) -> Any",
  "$[10 = (int64, Any, a : Any) -> Any]",
  "xxx[10 = (int64, Any, a : Any) -> Any]",
  "?$10 = (int64, Any, a : Any) -> Any]",
  "?=10 = (int64, Any, a : Any) -> Any]",
  "?[10 = (int64, Any, a : Any) -> Any$",
  "?[10 = (int64, Any, a : Any) -> Any$",
  "$10 * (int64, Any, ..., a : Any) -> Any",
  "=10 * (int64, Any, ..., a : Any) -> Any",
  "$[10 * (int64, Any, ..., a : Any) -> Any]",
  "xxx[10 * (int64, Any, ..., a : Any) -> Any]",
  "?$10 * (int64, Any, ..., a : Any) -> Any]",
  "?=10 * (int64, Any, ..., a : Any) -> Any]",
  "?[10 * (int64, Any, ..., a : Any) -> Any$",
  "?[10 * (int64, Any, ..., a : Any) -> Any$",
  "$(int64, Any, ..., a : Any) -> Any",
  "=(int64, Any, ..., a : Any) -> Any",
  "$[(int64, Any, ..., a : Any) -> Any]",
  "xxx[(int64, Any, ..., a : Any) -> Any]",
  "?$(int64, Any, ..., a : Any) -> Any]",
  "?=(int64, Any, ..., a : Any) -> Any]",
  "?[(int64, Any, ..., a : Any) -> Any$",
  "?[(int64, Any, ..., a : Any) -> Any$",
  "10 $ (int64, Any, ..., a : Any) -> Any",
  "?10 $ (int64, Any, ..., a : Any) -> Any",
  "?(10 $ (int64, Any, ..., a : Any) -> Any)",
  "$10 $ (int64, Any, ..., a : Any) -> Any",
  "=10 $ (int64, Any, ..., a : Any) -> Any",
  "$[10 $ (int64, Any, ..., a : Any) -> Any]",
  "xxx[10 $ (int64, Any, ..., a : Any) -> Any]",
  "?$10 $ (int64, Any, ..., a : Any) -> Any]",
  "?=10 $ (int64, Any, ..., a : Any) -> Any]",
  "?[10 $ (int64, Any, ..., a : Any) -> Any$",
  "?[10 $ (int64, Any, ..., a : Any) -> Any$",
  "10 = (int64, Any, ..., a : Any) -> Any",
  "?10 = (int64, Any, ..., a : Any) -> Any",
  "?(10 = (int64, Any, ..., a : Any) -> Any)",
  "$10 = (int64, Any, ..., a : Any) -> Any",
  "=10 = (int64, Any, ..., a : Any) -> Any",
  "$[10 = (int64, Any, ..., a : Any) -> Any]",
  "xxx[10 = (int64, Any, ..., a : Any) -> Any]",
  "?$10 = (int64, Any, ..., a : Any) -> Any]",
  "?=10 = (int64, Any, ..., a : Any) -> Any]",
  "?[10 = (int64, Any, ..., a : Any) -> Any$",
  "?[10 = (int64, Any, ..., a : Any) -> Any$",
  "10 * (int64$ Any, a : Any) -> Any",
  "?10 * (int64$ Any, a : Any) -> Any",
  "?(10 * (int64$ Any, a : Any) -> Any)",
  "$10 * (int64$ Any, a : Any) -> Any",
  "=10 * (int64$ Any, a : Any) -> Any",
  "$[10 * (int64$ Any, a : Any) -> Any]",
  "xxx[10 * (int64$ Any, a : Any) -> Any]",
  "?$10 * (int64$ Any, a : Any) -> Any]",
  "?=10 * (int64$ Any, a : Any) -> Any]",
  "?[10 * (int64$ Any, a : Any) -> Any$",
  "?[10 * (int64$ Any, a : Any) -> Any$",
  "(int64$ Any, a : Any) -> Any",
  "?(int64$ Any, a : Any) -> Any",
  "?((int64$ Any, a : Any) -> Any)",
  "$(int64$ Any, a : Any) -> Any",
  "=(int64$ Any, a : Any) -> Any",
  "$[(int64$ Any, a : Any) -> Any]",
  "xxx[(int64$ Any, a : Any) -> Any]",
  "?$(int64$ Any, a : Any) -> Any]",
  "?=(int64$ Any, a : Any) -> Any]",
  "?[(int64$ Any, a : Any) -> Any$",
  "?[(int64$ Any, a : Any) -> Any$",
  "10 $ (int64$ Any, a : Any) -> Any",
  "?10 $ (int64$ Any, a : Any) -> Any",
  "?(10 $ (int64$ Any, a : Any) -> Any)",
  "$10 $ (int64$ Any, a : Any) -> Any",
  "=10 $ (int64$ Any, a : Any) -> Any",
  "$[10 $ (int64$ Any, a : Any) -> Any]",
  "xxx[10 $ (int64$ Any, a : Any) -> Any]",
  "?$10 $ (int64$ Any, a : Any) -> Any]",
  "?=10 $ (int64$ Any, a : Any) -> Any]",
  "?[10 $ (int64$ Any, a : Any) -> Any$",
  "?[10 $ (int64$ Any, a : Any) -> Any$",
  "10 = (int64$ Any, a : Any) -> Any",
  "?10 = (int64$ Any, a : Any) -> Any",
  "?(10 = (int64$ Any, a : Any) -> Any)",
  "$10 = (int64$ Any, a : Any) -> Any",
  "=10 = (int64$ Any, a : Any) -> Any",
  "$[10 = (int64$ Any, a : Any) -> Any]",
  "xxx[10 = (int64$ Any, a : Any) -> Any]",
  "?$10 = (int64$ Any, a : Any) -> Any]",
  "?=10 = (int64$ Any, a : Any) -> Any]",
  "?[10 = (int64$ Any, a : Any) -> Any$",
  "?[10 = (int64$ Any, a : Any) -> Any$",
  "10 * (int64$ Any, ..., a : Any) -> Any",
  "?10 * (int64$ Any, ..., a : Any) -> Any",
  "?(10 * (int64$ Any, ..., a : Any) -> Any)",
  "$10 * (int64$ Any, ..., a : Any) -> Any",
  "=10 * (int64$ Any, ..., a : Any) -> Any",
  "$[10 * (int64$ Any, ..., a : Any) -> Any]",
  "xxx[10 * (int64$ Any, ..., a : Any) -> Any]",
  "?$10 * (int64$ Any, ..., a : Any) -> Any]",
  "?=10 * (int64$ Any, ..., a : Any) -> Any]",
  "?[10 * (int64$ Any, ..., a : Any) -> Any$",
  "?[10 * (int64$ Any, ..., a : Any) -> Any$",
  "(int64$ Any, ..., a : Any) -> Any",
  "?(int64$ Any, ..., a : Any) -> Any",
  "?((int64$ Any, ..., a : Any) -> Any)",
  "$(int64$ Any, ..., a : Any) -> Any",
  "=(int64$ Any, ..., a : Any) -> Any",
  "$[(int64$ Any, ..., a : Any) -> Any]",
  "xxx[(int64$ Any, ..., a : Any) -> Any]",
  "?$(int64$ Any, ..., a : Any) -> Any]",
  "?=(int64$ Any, ..., a : Any) -> Any]",
  "?[(int64$ Any, ..., a : Any) -> Any$",
  "?[(int64$ Any, ..., a : Any) -> Any$",
  "10 $ (int64$ Any, ..., a : Any) -> Any",
  "?10 $ (int64$ Any, ..., a : Any) -> Any",
  "?(10 $ (int64$ Any, ..., a : Any) -> Any)",
  "$10 $ (int64$ Any, ..., a : Any) -> Any",
  "=10 $ (int64$ Any, ..., a : Any) -> Any",
  "$[10 $ (int64$ Any, ..., a : Any) -> Any]",
  "xxx[10 $ (int64$ Any, ..., a : Any) -> Any]",
  "?$10 $ (int64$ Any, ..., a : Any) -> Any]",
  "?=10 $ (int64$ Any, ..., a : Any) -> Any]",
  "?[10 $ (int64$ Any, ..., a : Any) -> Any$",
  "?[10 $ (int64$ Any, ..., a : Any) -> Any$",
  "10 = (int64$ Any, ..., a : Any) -> Any",
  "?10 = (int64$ Any, ..., a : Any) -> Any",
  "?(10 = (int64$ Any, ..., a : Any) -> Any)",
  "$10 = (int64$ Any, ..., a : Any) -> Any",
  "=10 = (int64$ Any, ..., a : Any) -> Any",
  "$[10 = (int64$ Any, ..., a : Any) -> Any]",
  "xxx[10 = (int64$ Any, ..., a : Any) -> Any]",
  "?$10 = (int64$ Any, ..., a : Any) -> Any]",
  "?=10 = (int64$ Any, ..., a : Any) -> Any]",
  "?[10 = (int64$ Any, ..., a : Any) -> Any$",
  "?[10 = (int64$ Any, ..., a : Any) -> Any$",
  "10 * (int64= Any, a : Any) -> Any",
  "?10 * (int64= Any, a : Any) -> Any",
  "?(10 * (int64= Any, a : Any) -> Any)",
  "$10 * (int64= Any, a : Any) -> Any",
  "=10 * (int64= Any, a : Any) -> Any",
  "$[10 * (int64= Any, a : Any) -> Any]",
  "xxx[10 * (int64= Any, a : Any) -> Any]",
  "?$10 * (int64= Any, a : Any) -> Any]",
  "?=10 * (int64= Any, a : Any) -> Any]",
  "?[10 * (int64= Any, a : Any) -> Any$",
  "?[10 * (int64= Any, a : Any) -> Any$",
  "(int64= Any, a : Any) -> Any",
  "?(int64= Any, a : Any) -> Any",
  "?((int64= Any, a : Any) -> Any)",
  "$(int64= Any, a : Any) -> Any",
  "=(int64= Any, a : Any) -> Any",
  "$[(int64= Any, a : Any) -> Any]",
  "xxx[(int64= Any, a : Any) -> Any]",
  "?$(int64= Any, a : Any) -> Any]",
  "?=(int64= Any, a : Any) -> Any]",
  "?[(int64= Any, a : Any) -> Any$",
  "?[(int64= Any, a : Any) -> Any$",
  "10 $ (int64= Any, a : Any) -> Any",
  "?10 $ (int64= Any, a : Any) -> Any",
  "?(10 $ (int64= Any, a : Any) -> Any)",
  "$10 $ (int64= Any, a : Any) -> Any",
  "=10 $ (int64= Any, a : Any) -> Any",
  "$[10 $ (int64= Any, a : Any) -> Any]",
  "xxx[10 $ (int64= Any, a : Any) -> Any]",
  "?$10 $ (int64= Any, a : Any) -> Any]",
  "?=10 $ (int64= Any, a : Any) -> Any]",
  "?[10 $ (int64= Any, a : Any) -> Any$",
  "?[10 $ (int64= Any, a : Any) -> Any$",
  "10 = (int64= Any, a : Any) -> Any",
  "?10 = (int64= Any, a : Any) -> Any",
  "?(10 = (int64= Any, a : Any) -> Any)",
  "$10 = (int64= Any, a : Any) -> Any",
  "=10 = (int64= Any, a : Any) -> Any",
  "$[10 = (int64= Any, a : Any) -> Any]",
  "xxx[10 = (int64= Any, a : Any) -> Any]",
  "?$10 = (int64= Any, a : Any) -> Any]",
  "?=10 = (int64= Any, a : Any) -> Any]",
  "?[10 = (int64= Any, a : Any) -> Any$",
  "?[10 = (int64= Any, a : Any) -> Any$",
  "10 * (int64= Any, ..., a : Any) -> Any",
  "?10 * (int64= Any, ..., a : Any) -> Any",
  "?(10 * (int64= Any, ..., a : Any) -> Any)",
  "$10 * (int64= Any, ..., a : Any) -> Any",
  "=10 * (int64= Any, ..., a : Any) -> Any",
  "$[10 * (int64= Any, ..., a : Any) -> Any]",
  "xxx[10 * (int64= Any, ..., a : Any) -> Any]",
  "?$10 * (int64= Any, ..., a : Any) -> Any]",
  "?=10 * (int64= Any, ..., a : Any) -> Any]",
  "?[10 * (int64= Any, ..., a : Any) -> Any$",
  "?[10 * (int64= Any, ..., a : Any) -> Any$",
  "(int64= Any, ..., a : Any) -> Any",
  "?(int64= Any, ..., a : Any) -> Any",
  "?((int64= Any, ..., a : Any) -> Any)",
  "$(int64= Any, ..., a : Any) -> Any",
  "=(int64= Any, ..., a : Any) -> Any",
  "$[(int64= Any, ..., a : Any) -> Any]",
  "xxx[(int64= Any, ..., a : Any) -> Any]",
  "?$(int64= Any, ..., a : Any) -> Any]",
  "?=(int64= Any, ..., a : Any) -> Any]",
  "?[(int64= Any, ..., a : Any) -> Any$",
  "?[(int64= Any, ..., a : Any) -> Any$",
  "10 $ (int64= Any, ..., a : Any) -> Any",
  "?10 $ (int64= Any, ..., a : Any) -> Any",
  "?(10 $ (int64= Any, ..., a : Any) -> Any)",
  "$10 $ (int64= Any, ..., a : Any) -> Any",
  "=10 $ (int64= Any, ..., a : Any) -> Any",
  "$[10 $ (int64= Any, ..., a : Any) -> Any]",
  "xxx[10 $ (int64= Any, ..., a : Any) -> Any]",
  "?$10 $ (int64= Any, ..., a : Any) -> Any]",
  "?=10 $ (int64= Any, ..., a : Any) -> Any]",
  "?[10 $ (int64= Any, ..., a : Any) -> Any$",
  "?[10 $ (int64= Any, ..., a : Any) -> Any$",
  "10 = (int64= Any, ..., a : Any) -> Any",
  "?10 = (int64= Any, ..., a : Any) -> Any",
  "?(10 = (int64= Any, ..., a : Any) -> Any)",
  "$10 = (int64= Any, ..., a : Any) -> Any",
  "=10 = (int64= Any, ..., a : Any) -> Any",
  "$[10 = (int64= Any, ..., a : Any) -> Any]",
  "xxx[10 = (int64= Any, ..., a : Any) -> Any]",
  "?$10 = (int64= Any, ..., a : Any) -> Any]",
  "?=10 = (int64= Any, ..., a : Any) -> Any]",
  "?[10 = (int64= Any, ..., a : Any) -> Any$",
  "?[10 = (int64= Any, ..., a : Any) -> Any$",
  "$10 * (int64, int64, Any, a : Any) -> Any",
  "=10 * (int64, int64, Any, a : Any) -> Any",
  "$[10 * (int64, int64, Any, a : Any) -> Any]",
  "xxx[10 * (int64, int64, Any, a : Any) -> Any]",
  "?$10 * (int64, int64, Any, a : Any) -> Any]",
  "?=10 * (int64, int64, Any, a : Any) -> Any]",
  "?[10 * (int64, int64, Any, a : Any) -> Any$",
  "?[10 * (int64, int64, Any, a : Any) -> Any$",
  "$(int64, int64, Any, a : Any) -> Any",
  "=(int64, int64, Any, a : Any) -> Any",
  "$[(int64, int64, Any, a : Any) -> Any]",
  "xxx[(int64, int64, Any, a : Any) -> Any]",
  "?$(int64, int64, Any, a : Any) -> Any]",
  "?=(int64, int64, Any, a : Any) -> Any]",
  "?[(int64, int64, Any, a : Any) -> Any$",
  "?[(int64, int64, Any, a : Any) -> Any$",
  "10 $ (int64, int64, Any, a : Any) -> Any",
  "?10 $ (int64, int64, Any, a : Any) -> Any",
  "?(10 $ (int64, int64, Any, a : Any) -> Any)",
  "$10 $ (int64, int64, Any, a : Any) -> Any",
  "=10 $ (int64, int64, Any, a : Any) -> Any",
  "$[10 $ (int64, int64, Any, a : Any) -> Any]",
  "xxx[10 $ (int64, int64, Any, a : Any) -> Any]",
  "?$10 $ (int64, int64, Any, a : Any) -> Any]",
  "?=10 $ (int64, int64, Any, a : Any) -> Any]",
  "?[10 $ (int64, int64, Any, a : Any) -> Any$",
  "?[10 $ (int64, int64, Any, a : Any) -> Any$",
  "10 = (int64, int64, Any, a : Any) -> Any",
  "?10 = (int64, int64, Any, a : Any) -> Any",
  "?(10 = (int64, int64, Any, a : Any) -> Any)",
  "$10 = (int64, int64, Any, a : Any) -> Any",
  "=10 = (int64, int64, Any, a : Any) -> Any",
  "$[10 = (int64, int64, Any, a : Any) -> Any]",
  "xxx[10 = (int64, int64, Any, a : Any) -> Any]",
  "?$10 = (int64, int64, Any, a : Any) -> Any]",
  "?=10 = (int64, int64, Any, a : Any) -> Any]",
  "?[10 = (int64, int64, Any, a : Any) -> Any$",
  "?[10 = (int64, int64, Any, a : Any) -> Any$",
  "$10 * (int64, int64, Any, ..., a : Any) -> Any",
  "=10 * (int64, int64, Any, ..., a : Any) -> Any",
  "$[10 * (int64, int64, Any, ..., a : Any) -> Any]",
  "xxx[10 * (int64, int64, Any, ..., a : Any) -> Any]",
  "?$10 * (int64, int64, Any, ..., a : Any) -> Any]",
  "?=10 * (int64, int64, Any, ..., a : Any) -> Any]",
  "?[10 * (int64, int64, Any, ..., a : Any) -> Any$",
  "?[10 * (int64, int64, Any, ..., a : Any) -> Any$",
  "$(int64, int64, Any, ..., a : Any) -> Any",
  "=(int64, int64, Any, ..., a : Any) -> Any",
  "$[(int64, int64, Any, ..., a : Any) -> Any]",
  "xxx[(int64, int64, Any, ..., a : Any) -> Any]",
  "?$(int64, int64, Any, ..., a : Any) -> Any]",
  "?=(int64, int64, Any, ..., a : Any) -> Any]",
  "?[(int64, int64, Any, ..., a : Any) -> Any$",
  "?[(int64, int64, Any, ..., a : Any) -> Any$",
  "10 $ (int64, int64, Any, ..., a : Any) -> Any",
  "?10 $ (int64, int64, Any, ..., a : Any) -> Any",
  "?(10 $ (int64, int64, Any, ..., a : Any) -> Any)",
  "$10 $ (int64, int64, Any, ..., a : Any) -> Any",
  "=10 $ (int64, int64, Any, ..., a : Any) -> Any",
  "$[10 $ (int64, int64, Any, ..., a : Any) -> Any]",
  "xxx[10 $ (int64, int64, Any, ..., a : Any) -> Any]",
  "?$10 $ (int64, int64, Any, ..., a : Any) -> Any]",
  "?=10 $ (int64, int64, Any, ..., a : Any) -> Any]",
  "?[10 $ (int64, int64, Any, ..., a : Any) -> Any$",
  "?[10 $ (int64, int64, Any, ..., a : Any) -> Any$",
  "10 = (int64, int64, Any, ..., a : Any) -> Any",
  "?10 = (int64, int64, Any, ..., a : Any) -> Any",
  "?(10 = (int64, int64, Any, ..., a : Any) -> Any)",
  "$10 = (int64, int64, Any, ..., a : Any) -> Any",
  "=10 = (int64, int64, Any, ..., a : Any) -> Any",
  "$[10 = (int64, int64, Any, ..., a : Any) -> Any]",
  "xxx[10 = (int64, int64, Any, ..., a : Any) -> Any]",
  "?$10 = (int64, int64, Any, ..., a : Any) -> Any]",
  "?=10 = (int64, int64, Any, ..., a : Any) -> Any]",
  "?[10 = (int64, int64, Any, ..., a : Any) -> Any$",
  "?[10 = (int64, int64, Any, ..., a : Any) -> Any$",
  "10 * (int64, int64$ Any, a : Any) -> Any",
  "?10 * (int64, int64$ Any, a : Any) -> Any",
  "?(10 * (int64, int64$ Any, a : Any) -> Any)",
  "$10 * (int64, int64$ Any, a : Any) -> Any",
  "=10 * (int64, int64$ Any, a : Any) -> Any",
  "$[10 * (int64, int64$ Any, a : Any) -> Any]",
  "xxx[10 * (int64, int64$ Any, a : Any) -> Any]",
  "?$10 * (int64, int64$ Any, a : Any) -> Any]",
  "?=10 * (int64, int64$ Any, a : Any) -> Any]",
  "?[10 * (int64, int64$ Any, a : Any) -> Any$",
  "?[10 * (int64, int64$ Any, a : Any) -> Any$",
  "(int64, int64$ Any, a : Any) -> Any",
  "?(int64, int64$ Any, a : Any) -> Any",
  "?((int64, int64$ Any, a : Any) -> Any)",
  "$(int64, int64$ Any, a : Any) -> Any",
  "=(int64, int64$ Any, a : Any) -> Any",
  "$[(int64, int64$ Any, a : Any) -> Any]",
  "xxx[(int64, int64$ Any, a : Any) -> Any]",
  "?$(int64, int64$ Any, a : Any) -> Any]",
  "?=(int64, int64$ Any, a : Any) -> Any]",
  "?[(int64, int64$ Any, a : Any) -> Any$",
  "?[(int64, int64$ Any, a : Any) -> Any$",
  "10 $ (int64, int64$ Any, a : Any) -> Any",
  "?10 $ (int64, int64$ Any, a : Any) -> Any",
  "?(10 $ (int64, int64$ Any, a : Any) -> Any)",
  "$10 $ (int64, int64$ Any, a : Any) -> Any",
  "=10 $ (int64, int64$ Any, a : Any) -> Any",
  "$[10 $ (int64, int64$ Any, a : Any) -> Any]",
  "xxx[10 $ (int64, int64$ Any, a : Any) -> Any]",
  "?$10 $ (int64, int64$ Any, a : Any) -> Any]",
  "?=10 $ (int64, int64$ Any, a : Any) -> Any]",
  "?[10 $ (int64, int64$ Any, a : Any) -> Any$",
  "?[10 $ (int64, int64$ Any, a : Any) -> Any$",
  "10 = (int64, int64$ Any, a : Any) -> Any",
  "?10 = (int64, int64$ Any, a : Any) -> Any",
  "?(10 = (int64, int64$ Any, a : Any) -> Any)",
  "$10 = (int64, int64$ Any, a : Any) -> Any",
  "=10 = (int64, int64$ Any, a : Any) -> Any",
  "$[10 = (int64, int64$ Any, a : Any) -> Any]",
  "xxx[10 = (int64, int64$ Any, a : Any) -> Any]",
  "?$10 = (int64, int64$ Any, a : Any) -> Any]",
  "?=10 = (int64, int64$ Any, a : Any) -> Any]",
  "?[10 = (int64, int64$ Any, a : Any) -> Any$",
  "?[10 = (int64, int64$ Any, a : Any) -> Any$",
  "10 * (int64, int64$ Any, ..., a : Any) -> Any",
  "?10 * (int64, int64$ Any, ..., a : Any) -> Any",
  "?(10 * (int64, int64$ Any, ..., a : Any) -> Any)",
  "$10 * (int64, int64$ Any, ..., a : Any) -> Any",
  "=10 * (int64, int64$ Any, ..., a : Any) -> Any",
  "$[10 * (int64, int64$ Any, ..., a : Any) -> Any]",
  "xxx[10 * (int64, int64$ Any, ..., a : Any) -> Any]",
  "?$10 * (int64, int64$ Any, ..., a : Any) -> Any]",
  "?=10 * (int64, int64$ Any, ..., a : Any) -> Any]",
  "?[10 * (int64, int64$ Any, ..., a : Any) -> Any$",
  "?[10 * (int64, int64$ Any, ..., a : Any) -> Any$",
  "(int64, int64$ Any, ..., a : Any) -> Any",
  "?(int64, int64$ Any, ..., a : Any) -> Any",
  "?((int64, int64$ Any, ..., a : Any) -> Any)",
  "$(int64, int64$ Any, ..., a : Any) -> Any",
  "=(int64, int64$ Any, ..., a : Any) -> Any",
  "$[(int64, int64$ Any, ..., a : Any) -> Any]",
  "xxx[(int64, int64$ Any, ..., a : Any) -> Any]",
  "?$(int64, int64$ Any, ..., a : Any) -> Any]",
  "?=(int64, int64$ Any, ..., a : Any) -> Any]",
  "?[(int64, int64$ Any, ..., a : Any) -> Any$",
  "?[(int64, int64$ Any, ..., a : Any) -> Any$",
  "10 $ (int64, int64$ Any, ..., a : Any) -> Any",
  "?10 $ (int64, int64$ Any, ..., a : Any) -> Any",
  "?(10 $ (int64, int64$ Any, ..., a : Any) -> Any)",
  "$10 $ (int64, int64$ Any, ..., a : Any) -> Any",
  "=10 $ (int64, int64$ Any, ..., a : Any) -> Any",
  "$[10 $ (int64, int64$ Any, ..., a : Any) -> Any]",
  "xxx[10 $ (int64, int64$ Any, ..., a : Any) -> Any]",
  "?$10 $ (int64, int64$ Any, ..., a : Any) -> Any]",
  "?=10 $ (int64, int64$ Any, ..., a : Any) -> Any]",
  "?[10 $ (int64, int64$ Any, ..., a : Any) -> Any$",
  "?[10 $ (int64, int64$ Any, ..., a : Any) -> Any$",
  "10 = (int64, int64$ Any, ..., a : Any) -> Any",
  "?10 = (int64, int64$ Any, ..., a : Any) -> Any",
  "?(10 = (int64, int64$ Any, ..., a : Any) -> Any)",
  "$10 = (int64, int64$ Any, ..., a : Any) -> Any",
  "=10 = (int64, int64$ Any, ..., a : Any) -> Any",
  "$[10 = (int64, int64$ Any, ..., a : Any) -> Any]",
  "xxx[10 = (int64, int64$ Any, ..., a : Any) -> Any]",
  "?$10 = (int64, int64$ Any, ..., a : Any) -> Any]",
  "?=10 = (int64, int64$ Any, ..., a : Any) -> Any]",
  "?[10 = (int64, int64$ Any, ..., a : Any) -> Any$",
  "?[10 = (int64, int64$ Any, ..., a : Any) -> Any$",
  "10 * (int64, int64= Any, a : Any) -> Any",
  "?10 * (int64, int64= Any, a : Any) -> Any",
  "?(10 * (int64, int64= Any, a : Any) -> Any)",
  "$10 * (int64, int64= Any, a : Any) -> Any",
  "=10 * (int64, int64= Any, a : Any) -> Any",
  "$[10 * (int64, int64= Any, a : Any) -> Any]",
  "xxx[10 * (int64, int64= Any, a : Any) -> Any]",
  "?$10 * (int64, int64= Any, a : Any) -> Any]",
  "?=10 * (int64, int64= Any, a : Any) -> Any]",
  "?[10 * (int64, int64= Any, a : Any) -> Any$",
  "?[10 * (int64, int64= Any, a : Any) -> Any$",
  "(int64, int64= Any, a : Any) -> Any",
  "?(int64, int64= Any, a : Any) -> Any",
  "?((int64, int64= Any, a : Any) -> Any)",
  "$(int64, int64= Any, a : Any) -> Any",
  "=(int64, int64= Any, a : Any) -> Any",
  "$[(int64, int64= Any, a : Any) -> Any]",
  "xxx[(int64, int64= Any, a : Any) -> Any]",
  "?$(int64, int64= Any, a : Any) -> Any]",
  "?=(int64, int64= Any, a : Any) -> Any]",
  "?[(int64, int64= Any, a : Any) -> Any$",
  "?[(int64, int64= Any, a : Any) -> Any$",
  "10 $ (int64, int64= Any, a : Any) -> Any",
  "?10 $ (int64, int64= Any, a : Any) -> Any",
  "?(10 $ (int64, int64= Any, a : Any) -> Any)",
  "$10 $ (int64, int64= Any, a : Any) -> Any",
  "=10 $ (int64, int64= Any, a : Any) -> Any",
  "$[10 $ (int64, int64= Any, a : Any) -> Any]",
  "xxx[10 $ (int64, int64= Any, a : Any) -> Any]",
  "?$10 $ (int64, int64= Any, a : Any) -> Any]",
  "?=10 $ (int64, int64= Any, a : Any) -> Any]",
  "?[10 $ (int64, int64= Any, a : Any) -> Any$",
  "?[10 $ (int64, int64= Any, a : Any) -> Any$",
  "10 = (int64, int64= Any, a : Any) -> Any",
  "?10 = (int64, int64= Any, a : Any) -> Any",
  "?(10 = (int64, int64= Any, a : Any) -> Any)",
  "$10 = (int64, int64= Any, a : Any) -> Any",
  "=10 = (int64, int64= Any, a : Any) -> Any",
  "$[10 = (int64, int64= Any, a : Any) -> Any]",
  "xxx[10 = (int64, int64= Any, a : Any) -> Any]",
  "?$10 = (int64, int64= Any, a : Any) -> Any]",
  "?=10 = (int64, int64= Any, a : Any) -> Any]",
  "?[10 = (int64, int64= Any, a : Any) -> Any$",
  "?[10 = (int64, int64= Any, a : Any) -> Any$",
  "10 * (int64, int64= Any, ..., a : Any) -> Any",
  "?10 * (int64, int64= Any, ..., a : Any) -> Any",
  "?(10 * (int64, int64= Any, ..., a : Any) -> Any)",
  "$10 * (int64, int64= Any, ..., a : Any) -> Any",
  "=10 * (int64, int64= Any, ..., a : Any) -> Any",
  "$[10 * (int64, int64= Any, ..., a : Any) -> Any]",
  "xxx[10 * (int64, int64= Any, ..., a : Any) -> Any]",
  "?$10 * (int64, int64= Any, ..., a : Any) -> Any]",
  "?=10 * (int64, int64= Any, ..., a : Any) -> Any]",
  "?[10 * (int64, int64= Any, ..., a : Any) -> Any$",
  "?[10 * (int64, int64= Any, ..., a : Any) -> Any$",
  "(int64, int64= Any, ..., a : Any) -> Any",
  "?(int64, int64= Any, ..., a : Any) -> Any",
  "?((int64, int64= Any, ..., a : Any) -> Any)",
  "$(int64, int64= Any, ..., a : Any) -> Any",
  "=(int64, int64= Any, ..., a : Any) -> Any",
  "$[(int64, int64= Any, ..., a : Any) -> Any]",
  "xxx[(int64, int64= Any, ..., a : Any) -> Any]",
  "?$(int64, int64= Any, ..., a : Any) -> Any]",
  "?=(int64, int64= Any, ..., a : Any) -> Any]",
  "?[(int64, int64= Any, ..., a : Any) -> Any$",
  "?[(int64, int64= Any, ..., a : Any) -> Any$",
  "10 $ (int64, int64= Any, ..., a : Any) -> Any",
  "?10 $ (int64, int64= Any, ..., a : Any) -> Any",
  "?(10 $ (int64, int64= Any, ..., a : Any) -> Any)",
  "$10 $ (int64, int64= Any, ..., a : Any) -> Any",
  "=10 $ (int64, int64= Any, ..., a : Any) -> Any",
  "$[10 $ (int64, int64= Any, ..., a : Any) -> Any]",
  "xxx[10 $ (int64, int64= Any, ..., a : Any) -> Any]",
  "?$10 $ (int64, int64= Any, ..., a : Any) -> Any]",
  "?=10 $ (int64, int64= Any, ..., a : Any) -> Any]",
  "?[10 $ (int64, int64= Any, ..., a : Any) -> Any$",
  "?[10 $ (int64, int64= Any, ..., a : Any) -> Any$",
  "10 = (int64, int64= Any, ..., a : Any) -> Any",
  "?10 = (int64, int64= Any, ..., a : Any) -> Any",
  "?(10 = (int64, int64= Any, ..., a : Any) -> Any)",
  "$10 = (int64, int64= Any, ..., a : Any) -> Any",
  "=10 = (int64, int64= Any, ..., a : Any) -> Any",
  "$[10 = (int64, int64= Any, ..., a : Any) -> Any]",
  "xxx[10 = (int64, int64= Any, ..., a : Any) -> Any]",
  "?$10 = (int64, int64= Any, ..., a : Any) -> Any]",
  "?=10 = (int64, int64= Any, ..., a : Any) -> Any]",
  "?[10 = (int64, int64= Any, ..., a : Any) -> Any$",
  "?[10 = (int64, int64= Any, ..., a : Any) -> Any$",
  "10 * (xxx, a : Any) -> Any",
  "?10 * (xxx, a : Any) -> Any",
  "?(10 * (xxx, a : Any) -> Any)",
  "$10 * (xxx, a : Any) -> Any",
  "=10 * (xxx, a : Any) -> Any",
  "$[10 * (xxx, a : Any) -> Any]",
  "xxx[10 * (xxx, a : Any) -> Any]",
  "?$10 * (xxx, a : Any) -> Any]",
  "?=10 * (xxx, a : Any) -> Any]",
  "?[10 * (xxx, a : Any) -> Any$",
  "?[10 * (xxx, a : Any) -> Any$",
  "(xxx, a : Any) -> Any",
  "?(xxx, a : Any) -> Any",
  "?((xxx, a : Any) -> Any)",
  "$(xxx, a : Any) -> Any",
  "=(xxx, a : Any) -> Any",
  "$[(xxx, a : Any) -> Any]",
  "xxx[(xxx, a : Any) -> Any]",
  "?$(xxx, a : Any) -> Any]",
  "?=(xxx, a : Any) -> Any]",
  "?[(xxx, a : Any) -> Any$",
  "?[(xxx, a : Any) -> Any$",
  "10 $ (xxx, a : Any) -> Any",
  "?10 $ (xxx, a : Any) -> Any",
  "?(10 $ (xxx, a : Any) -> Any)",
  "$10 $ (xxx, a : Any) -> Any",
  "=10 $ (xxx, a : Any) -> Any",
  "$[10 $ (xxx, a : Any) -> Any]",
  "xxx[10 $ (xxx, a : Any) -> Any]",
  "?$10 $ (xxx, a : Any) -> Any]",
  "?=10 $ (xxx, a : Any) -> Any]",
  "?[10 $ (xxx, a : Any) -> Any$",
  "?[10 $ (xxx, a : Any) -> Any$",
  "10 = (xxx, a : Any) -> Any",
  "?10 = (xxx, a : Any) -> Any",
  "?(10 = (xxx, a : Any) -> Any)",
  "$10 = (xxx, a : Any) -> Any",
  "=10 = (xxx, a : Any) -> Any",
  "$[10 = (xxx, a : Any) -> Any]",
  "xxx[10 = (xxx, a : Any) -> Any]",
  "?$10 = (xxx, a : Any) -> Any]",
  "?=10 = (xxx, a : Any) -> Any]",
  "?[10 = (xxx, a : Any) -> Any$",
  "?[10 = (xxx, a : Any) -> Any$",
  "10 * (xxx, ..., a : Any) -> Any",
  "?10 * (xxx, ..., a : Any) -> Any",
  "?(10 * (xxx, ..., a : Any) -> Any)",
  "$10 * (xxx, ..., a : Any) -> Any",
  "=10 * (xxx, ..., a : Any) -> Any",
  "$[10 * (xxx, ..., a : Any) -> Any]",
  "xxx[10 * (xxx, ..., a : Any) -> Any]",
  "?$10 * (xxx, ..., a : Any) -> Any]",
  "?=10 * (xxx, ..., a : Any) -> Any]",
  "?[10 * (xxx, ..., a : Any) -> Any$",
  "?[10 * (xxx, ..., a : Any) -> Any$",
  "(xxx, ..., a : Any) -> Any",
  "?(xxx, ..., a : Any) -> Any",
  "?((xxx, ..., a : Any) -> Any)",
  "$(xxx, ..., a : Any) -> Any",
  "=(xxx, ..., a : Any) -> Any",
  "$[(xxx, ..., a : Any) -> Any]",
  "xxx[(xxx, ..., a : Any) -> Any]",
  "?$(xxx, ..., a : Any) -> Any]",
  "?=(xxx, ..., a : Any) -> Any]",
  "?[(xxx, ..., a : Any) -> Any$",
  "?[(xxx, ..., a : Any) -> Any$",
  "10 $ (xxx, ..., a : Any) -> Any",
  "?10 $ (xxx, ..., a : Any) -> Any",
  "?(10 $ (xxx, ..., a : Any) -> Any)",
  "$10 $ (xxx, ..., a : Any) -> Any",
  "=10 $ (xxx, ..., a : Any) -> Any",
  "$[10 $ (xxx, ..., a : Any) -> Any]",
  "xxx[10 $ (xxx, ..., a : Any) -> Any]",
  "?$10 $ (xxx, ..., a : Any) -> Any]",
  "?=10 $ (xxx, ..., a : Any) -> Any]",
  "?[10 $ (xxx, ..., a : Any) -> Any$",
  "?[10 $ (xxx, ..., a : Any) -> Any$",
  "10 = (xxx, ..., a : Any) -> Any",
  "?10 = (xxx, ..., a : Any) -> Any",
  "?(10 = (xxx, ..., a : Any) -> Any)",
  "$10 = (xxx, ..., a : Any) -> Any",
  "=10 = (xxx, ..., a : Any) -> Any",
  "$[10 = (xxx, ..., a : Any) -> Any]",
  "xxx[10 = (xxx, ..., a : Any) -> Any]",
  "?$10 = (xxx, ..., a : Any) -> Any]",
  "?=10 = (xxx, ..., a : Any) -> Any]",
  "?[10 = (xxx, ..., a : Any) -> Any$",
  "?[10 = (xxx, ..., a : Any) -> Any$",
  "10 * (int64, xxx, a : Any) -> Any",
  "?10 * (int64, xxx, a : Any) -> Any",
  "?(10 * (int64, xxx, a : Any) -> Any)",
  "$10 * (int64, xxx, a : Any) -> Any",
  "=10 * (int64, xxx, a : Any) -> Any",
  "$[10 * (int64, xxx, a : Any) -> Any]",
  "xxx[10 * (int64, xxx, a : Any) -> Any]",
  "?$10 * (int64, xxx, a : Any) -> Any]",
  "?=10 * (int64, xxx, a : Any) -> Any]",
  "?[10 * (int64, xxx, a : Any) -> Any$",
  "?[10 * (int64, xxx, a : Any) -> Any$",
  "(int64, xxx, a : Any) -> Any",
  "?(int64, xxx, a : Any) -> Any",
  "?((int64, xxx, a : Any) -> Any)",
  "$(int64, xxx, a : Any) -> Any",
  "=(int64, xxx, a : Any) -> Any",
  "$[(int64, xxx, a : Any) -> Any]",
  "xxx[(int64, xxx, a : Any) -> Any]",
  "?$(int64, xxx, a : Any) -> Any]",
  "?=(int64, xxx, a : Any) -> Any]",
  "?[(int64, xxx, a : Any) -> Any$",
  "?[(int64, xxx, a : Any) -> Any$",
  "10 $ (int64, xxx, a : Any) -> Any",
  "?10 $ (int64, xxx, a : Any) -> Any",
  "?(10 $ (int64, xxx, a : Any) -> Any)",
  "$10 $ (int64, xxx, a : Any) -> Any",
  "=10 $ (int64, xxx, a : Any) -> Any",
  "$[10 $ (int64, xxx, a : Any) -> Any]",
  "xxx[10 $ (int64, xxx, a : Any) -> Any]",
  "?$10 $ (int64, xxx, a : Any) -> Any]",
  "?=10 $ (int64, xxx, a : Any) -> Any]",
  "?[10 $ (int64, xxx, a : Any) -> Any$",
  "?[10 $ (int64, xxx, a : Any) -> Any$",
  "10 = (int64, xxx, a : Any) -> Any",
  "?10 = (int64, xxx, a : Any) -> Any",
  "?(10 = (int64, xxx, a : Any) -> Any)",
  "$10 = (int64, xxx, a : Any) -> Any",
  "=10 = (int64, xxx, a : Any) -> Any",
  "$[10 = (int64, xxx, a : Any) -> Any]",
  "xxx[10 = (int64, xxx, a : Any) -> Any]",
  "?$10 = (int64, xxx, a : Any) -> Any]",
  "?=10 = (int64, xxx, a : Any) -> Any]",
  "?[10 = (int64, xxx, a : Any) -> Any$",
  "?[10 = (int64, xxx, a : Any) -> Any$",
  "10 * (int64, xxx, ..., a : Any) -> Any",
  "?10 * (int64, xxx, ..., a : Any) -> Any",
  "?(10 * (int64, xxx, ..., a : Any) -> Any)",
  "$10 * (int64, xxx, ..., a : Any) -> Any",
  "=10 * (int64, xxx, ..., a : Any) -> Any",
  "$[10 * (int64, xxx, ..., a : Any) -> Any]",
  "xxx[10 * (int64, xxx, ..., a : Any) -> Any]",
  "?$10 * (int64, xxx, ..., a : Any) -> Any]",
  "?=10 * (int64, xxx, ..., a : Any) -> Any]",
  "?[10 * (int64, xxx, ..., a : Any) -> Any$",
  "?[10 * (int64, xxx, ..., a : Any) -> Any$",
  "(int64, xxx, ..., a : Any) -> Any",
  "?(int64, xxx, ..., a : Any) -> Any",
  "?((int64, xxx, ..., a : Any) -> Any)",
  "$(int64, xxx, ..., a : Any) -> Any",
  "=(int64, xxx, ..., a : Any) -> Any",
  "$[(int64, xxx, ..., a : Any) -> Any]",
  "xxx[(int64, xxx, ..., a : Any) -> Any]",
  "?$(int64, xxx, ..., a : Any) -> Any]",
  "?=(int64, xxx, ..., a : Any) -> Any]",
  "?[(int64, xxx, ..., a : Any) -> Any$",
  "?[(int64, xxx, ..., a : Any) -> Any$",
  "10 $ (int64, xxx, ..., a : Any) -> Any",
  "?10 $ (int64, xxx, ..., a : Any) -> Any",
  "?(10 $ (int64, xxx, ..., a : Any) -> Any)",
  "$10 $ (int64, xxx, ..., a : Any) -> Any",
  "=10 $ (int64, xxx, ..., a : Any) -> Any",
  "$[10 $ (int64, xxx, ..., a : Any) -> Any]",
  "xxx[10 $ (int64, xxx, ..., a : Any) -> Any]",
  "?$10 $ (int64, xxx, ..., a : Any) -> Any]",
  "?=10 $ (int64, xxx, ..., a : Any) -> Any]",
  "?[10 $ (int64, xxx, ..., a : Any) -> Any$",
  "?[10 $ (int64, xxx, ..., a : Any) -> Any$",
  "10 = (int64, xxx, ..., a : Any) -> Any",
  "?10 = (int64, xxx, ..., a : Any) -> Any",
  "?(10 = (int64, xxx, ..., a : Any) -> Any)",
  "$10 = (int64, xxx, ..., a : Any) -> Any",
  "=10 = (int64, xxx, ..., a : Any) -> Any",
  "$[10 = (int64, xxx, ..., a : Any) -> Any]",
  "xxx[10 = (int64, xxx, ..., a : Any) -> Any]",
  "?$10 = (int64, xxx, ..., a : Any) -> Any]",
  "?=10 = (int64, xxx, ..., a : Any) -> Any]",
  "?[10 = (int64, xxx, ..., a : Any) -> Any$",
  "?[10 = (int64, xxx, ..., a : Any) -> Any$",
  "10 * (int64$ xxx, a : Any) -> Any",
  "?10 * (int64$ xxx, a : Any) -> Any",
  "?(10 * (int64$ xxx, a : Any) -> Any)",
  "$10 * (int64$ xxx, a : Any) -> Any",
  "=10 * (int64$ xxx, a : Any) -> Any",
  "$[10 * (int64$ xxx, a : Any) -> Any]",
  "xxx[10 * (int64$ xxx, a : Any) -> Any]",
  "?$10 * (int64$ xxx, a : Any) -> Any]",
  "?=10 * (int64$ xxx, a : Any) -> Any]",
  "?[10 * (int64$ xxx, a : Any) -> Any$",
  "?[10 * (int64$ xxx, a : Any) -> Any$",
  "(int64$ xxx, a : Any) -> Any",
  "?(int64$ xxx, a : Any) -> Any",
  "?((int64$ xxx, a : Any) -> Any)",
  "$(int64$ xxx, a : Any) -> Any",
  "=(int64$ xxx, a : Any) -> Any",
  "$[(int64$ xxx, a : Any) -> Any]",
  "xxx[(int64$ xxx, a : Any) -> Any]",
  "?$(int64$ xxx, a : Any) -> Any]",
  "?=(int64$ xxx, a : Any) -> Any]",
  "?[(int64$ xxx, a : Any) -> Any$",
  "?[(int64$ xxx, a : Any) -> Any$",
  "10 $ (int64$ xxx, a : Any) -> Any",
  "?10 $ (int64$ xxx, a : Any) -> Any",
  "?(10 $ (int64$ xxx, a : Any) -> Any)",
  "$10 $ (int64$ xxx, a : Any) -> Any",
  "=10 $ (int64$ xxx, a : Any) -> Any",
  "$[10 $ (int64$ xxx, a : Any) -> Any]",
  "xxx[10 $ (int64$ xxx, a : Any) -> Any]",
  "?$10 $ (int64$ xxx, a : Any) -> Any]",
  "?=10 $ (int64$ xxx, a : Any) -> Any]",
  "?[10 $ (int64$ xxx, a : Any) -> Any$",
  "?[10 $ (int64$ xxx, a : Any) -> Any$",
  "10 = (int64$ xxx, a : Any) -> Any",
  "?10 = (int64$ xxx, a : Any) -> Any",
  "?(10 = (int64$ xxx, a : Any) -> Any)",
  "$10 = (int64$ xxx, a : Any) -> Any",
  "=10 = (int64$ xxx, a : Any) -> Any",
  "$[10 = (int64$ xxx, a : Any) -> Any]",
  "xxx[10 = (int64$ xxx, a : Any) -> Any]",
  "?$10 = (int64$ xxx, a : Any) -> Any]",
  "?=10 = (int64$ xxx, a : Any) -> Any]",
  "?[10 = (int64$ xxx, a : Any) -> Any$",
  "?[10 = (int64$ xxx, a : Any) -> Any$",
  "10 * (int64$ xxx, ..., a : Any) -> Any",
  "?10 * (int64$ xxx, ..., a : Any) -> Any",
  "?(10 * (int64$ xxx, ..., a : Any) -> Any)",
  "$10 * (int64$ xxx, ..., a : Any) -> Any",
  "=10 * (int64$ xxx, ..., a : Any) -> Any",
  "$[10 * (int64$ xxx, ..., a : Any) -> Any]",
  "xxx[10 * (int64$ xxx, ..., a : Any) -> Any]",
  "?$10 * (int64$ xxx, ..., a : Any) -> Any]",
  "?=10 * (int64$ xxx, ..., a : Any) -> Any]",
  "?[10 * (int64$ xxx, ..., a : Any) -> Any$",
  "?[10 * (int64$ xxx, ..., a : Any) -> Any$",
  "(int64$ xxx, ..., a : Any) -> Any",
  "?(int64$ xxx, ..., a : Any) -> Any",
  "?((int64$ xxx, ..., a : Any) -> Any)",
  "$(int64$ xxx, ..., a : Any) -> Any",
  "=(int64$ xxx, ..., a : Any) -> Any",
  "$[(int64$ xxx, ..., a : Any) -> Any]",
  "xxx[(int64$ xxx, ..., a : Any) -> Any]",
  "?$(int64$ xxx, ..., a : Any) -> Any]",
  "?=(int64$ xxx, ..., a : Any) -> Any]",
  "?[(int64$ xxx, ..., a : Any) -> Any$",
  "?[(int64$ xxx, ..., a : Any) -> Any$",
  "10 $ (int64$ xxx, ..., a : Any) -> Any",
  "?10 $ (int64$ xxx, ..., a : Any) -> Any",
  "?(10 $ (int64$ xxx, ..., a : Any) -> Any)",
  "$10 $ (int64$ xxx, ..., a : Any) -> Any",
  "=10 $ (int64$ xxx, ..., a : Any) -> Any",
  "$[10 $ (int64$ xxx, ..., a : Any) -> Any]",
  "xxx[10 $ (int64$ xxx, ..., a : Any) -> Any]",
  "?$10 $ (int64$ xxx, ..., a : Any) -> Any]",
  "?=10 $ (int64$ xxx, ..., a : Any) -> Any]",
  "?[10 $ (int64$ xxx, ..., a : Any) -> Any$",
  "?[10 $ (int64$ xxx, ..., a : Any) -> Any$",
  "10 = (int64$ xxx, ..., a : Any) -> Any",
  "?10 = (int64$ xxx, ..., a : Any) -> Any",
  "?(10 = (int64$ xxx, ..., a : Any) -> Any)",
  "$10 = (int64$ xxx, ..., a : Any) -> Any",
  "=10 = (int64$ xxx, ..., a : Any) -> Any",
  "$[10 = (int64$ xxx, ..., a : Any) -> Any]",
  "xxx[10 = (int64$ xxx, ..., a : Any) -> Any]",
  "?$10 = (int64$ xxx, ..., a : Any) -> Any]",
  "?=10 = (int64$ xxx, ..., a : Any) -> Any]",
  "?[10 = (int64$ xxx, ..., a : Any) -> Any$",
  "?[10 = (int64$ xxx, ..., a : Any) -> Any$",
  "10 * (int64= xxx, a : Any) -> Any",
  "?10 * (int64= xxx, a : Any) -> Any",
  "?(10 * (int64= xxx, a : Any) -> Any)",
  "$10 * (int64= xxx, a : Any) -> Any",
  "=10 * (int64= xxx, a : Any) -> Any",
  "$[10 * (int64= xxx, a : Any) -> Any]",
  "xxx[10 * (int64= xxx, a : Any) -> Any]",
  "?$10 * (int64= xxx, a : Any) -> Any]",
  "?=10 * (int64= xxx, a : Any) -> Any]",
  "?[10 * (int64= xxx, a : Any) -> Any$",
  "?[10 * (int64= xxx, a : Any) -> Any$",
  "(int64= xxx, a : Any) -> Any",
  "?(int64= xxx, a : Any) -> Any",
  "?((int64= xxx, a : Any) -> Any)",
  "$(int64= xxx, a : Any) -> Any",
  "=(int64= xxx, a : Any) -> Any",
  "$[(int64= xxx, a : Any) -> Any]",
  "xxx[(int64= xxx, a : Any) -> Any]",
  "?$(int64= xxx, a : Any) -> Any]",
  "?=(int64= xxx, a : Any) -> Any]",
  "?[(int64= xxx, a : Any) -> Any$",
  "?[(int64= xxx, a : Any) -> Any$",
  "10 $ (int64= xxx, a : Any) -> Any",
  "?10 $ (int64= xxx, a : Any) -> Any",
  "?(10 $ (int64= xxx, a : Any) -> Any)",
  "$10 $ (int64= xxx, a : Any) -> Any",
  "=10 $ (int64= xxx, a : Any) -> Any",
  "$[10 $ (int64= xxx, a : Any) -> Any]",
  "xxx[10 $ (int64= xxx, a : Any) -> Any]",
  "?$10 $ (int64= xxx, a : Any) -> Any]",
  "?=10 $ (int64= xxx, a : Any) -> Any]",
  "?[10 $ (int64= xxx, a : Any) -> Any$",
  "?[10 $ (int64= xxx, a : Any) -> Any$",
  "10 = (int64= xxx, a : Any) -> Any",
  "?10 = (int64= xxx, a : Any) -> Any",
  "?(10 = (int64= xxx, a : Any) -> Any)",
  "$10 = (int64= xxx, a : Any) -> Any",
  "=10 = (int64= xxx, a : Any) -> Any",
  "$[10 = (int64= xxx, a : Any) -> Any]",
  "xxx[10 = (int64= xxx, a : Any) -> Any]",
  "?$10 = (int64= xxx, a : Any) -> Any]",
  "?=10 = (int64= xxx, a : Any) -> Any]",
  "?[10 = (int64= xxx, a : Any) -> Any$",
  "?[10 = (int64= xxx, a : Any) -> Any$",
  "10 * (int64= xxx, ..., a : Any) -> Any",
  "?10 * (int64= xxx, ..., a : Any) -> Any",
  "?(10 * (int64= xxx, ..., a : Any) -> Any)",
  "$10 * (int64= xxx, ..., a : Any) -> Any",
  "=10 * (int64= xxx, ..., a : Any) -> Any",
  "$[10 * (int64= xxx, ..., a : Any) -> Any]",
  "xxx[10 * (int64= xxx, ..., a : Any) -> Any]",
  "?$10 * (int64= xxx, ..., a : Any) -> Any]",
  "?=10 * (int64= xxx, ..., a : Any) -> Any]",
  "?[10 * (int64= xxx, ..., a : Any) -> Any$",
  "?[10 * (int64= xxx, ..., a : Any) -> Any$",
  "(int64= xxx, ..., a : Any) -> Any",
  "?(int64= xxx, ..., a : Any) -> Any",
  "?((int64= xxx, ..., a : Any) -> Any)",
  "$(int64= xxx, ..., a : Any) -> Any",
  "=(int64= xxx, ..., a : Any) -> Any",
  "$[(int64= xxx, ..., a : Any) -> Any]",
  "xxx[(int64= xxx, ..., a : Any) -> Any]",
  "?$(int64= xxx, ..., a : Any) -> Any]",
  "?=(int64= xxx, ..., a : Any) -> Any]",
  "?[(int64= xxx, ..., a : Any) -> Any$",
  "?[(int64= xxx, ..., a : Any) -> Any$",
  "10 $ (int64= xxx, ..., a : Any) -> Any",
  "?10 $ (int64= xxx, ..., a : Any) -> Any",
  "?(10 $ (int64= xxx, ..., a : Any) -> Any)",
  "$10 $ (int64= xxx, ..., a : Any) -> Any",
  "=10 $ (int64= xxx, ..., a : Any) -> Any",
  "$[10 $ (int64= xxx, ..., a : Any) -> Any]",
  "xxx[10 $ (int64= xxx, ..., a : Any) -> Any]",
  "?$10 $ (int64= xxx, ..., a : Any) -> Any]",
  "?=10 $ (int64= xxx, ..., a : Any) -> Any]",
  "?[10 $ (int64= xxx, ..., a : Any) -> Any$",
  "?[10 $ (int64= xxx, ..., a : Any) -> Any$",
  "10 = (int64= xxx, ..., a : Any) -> Any",
  "?10 = (int64= xxx, ..., a : Any) -> Any",
  "?(10 = (int64= xxx, ..., a : Any) -> Any)",
  "$10 = (int64= xxx, ..., a : Any) -> Any",
  "=10 = (int64= xxx, ..., a : Any) -> Any",
  "$[10 = (int64= xxx, ..., a : Any) -> Any]",
  "xxx[10 = (int64= xxx, ..., a : Any) -> Any]",
  "?$10 = (int64= xxx, ..., a : Any) -> Any]",
  "?=10 = (int64= xxx, ..., a : Any) -> Any]",
  "?[10 = (int64= xxx, ..., a : Any) -> Any$",
  "?[10 = (int64= xxx, ..., a : Any) -> Any$",
  "10 * (int64, int64, xxx, a : Any) -> Any",
  "?10 * (int64, int64, xxx, a : Any) -> Any",
  "?(10 * (int64, int64, xxx, a : Any) -> Any)",
  "$10 * (int64, int64, xxx, a : Any) -> Any",
  "=10 * (int64, int64, xxx, a : Any) -> Any",
  "$[10 * (int64, int64, xxx, a : Any) -> Any]",
  "xxx[10 * (int64, int64, xxx, a : Any) -> Any]",
  "?$10 * (int64, int64, xxx, a : Any) -> Any]",
  "?=10 * (int64, int64, xxx, a : Any) -> Any]",
  "?[10 * (int64, int64, xxx, a : Any) -> Any$",
  "?[10 * (int64, int64, xxx, a : Any) -> Any$",
  "(int64, int64, xxx, a : Any) -> Any",
  "?(int64, int64, xxx, a : Any) -> Any",
  "?((int64, int64, xxx, a : Any) -> Any)",
  "$(int64, int64, xxx, a : Any) -> Any",
  "=(int64, int64, xxx, a : Any) -> Any",
  "$[(int64, int64, xxx, a : Any) -> Any]",
  "xxx[(int64, int64, xxx, a : Any) -> Any]",
  "?$(int64, int64, xxx, a : Any) -> Any]",
  "?=(int64, int64, xxx, a : Any) -> Any]",
  "?[(int64, int64, xxx, a : Any) -> Any$",
  "?[(int64, int64, xxx, a : Any) -> Any$",
  "10 $ (int64, int64, xxx, a : Any) -> Any",
  "?10 $ (int64, int64, xxx, a : Any) -> Any",
  "?(10 $ (int64, int64, xxx, a : Any) -> Any)",
  "$10 $ (int64, int64, xxx, a : Any) -> Any",
  "=10 $ (int64, int64, xxx, a : Any) -> Any",
  "$[10 $ (int64, int64, xxx, a : Any) -> Any]",
  "xxx[10 $ (int64, int64, xxx, a : Any) -> Any]",
  "?$10 $ (int64, int64, xxx, a : Any) -> Any]",
  "?=10 $ (int64, int64, xxx, a : Any) -> Any]",
  "?[10 $ (int64, int64, xxx, a : Any) -> Any$",
  "?[10 $ (int64, int64, xxx, a : Any) -> Any$",
  "10 = (int64, int64, xxx, a : Any) -> Any",
  "?10 = (int64, int64, xxx, a : Any) -> Any",
  "?(10 = (int64, int64, xxx, a : Any) -> Any)",
  "$10 = (int64, int64, xxx, a : Any) -> Any",
  "=10 = (int64, int64, xxx, a : Any) -> Any",
  "$[10 = (int64, int64, xxx, a : Any) -> Any]",
  "xxx[10 = (int64, int64, xxx, a : Any) -> Any]",
  "?$10 = (int64, int64, xxx, a : Any) -> Any]",
  "?=10 = (int64, int64, xxx, a : Any) -> Any]",
  "?[10 = (int64, int64, xxx, a : Any) -> Any$",
  "?[10 = (int64, int64, xxx, a : Any) -> Any$",
  "10 * (int64, int64, xxx, ..., a : Any) -> Any",
  "?10 * (int64, int64, xxx, ..., a : Any) -> Any",
  "?(10 * (int64, int64, xxx, ..., a : Any) -> Any)",
  "$10 * (int64, int64, xxx, ..., a : Any) -> Any",
  "=10 * (int64, int64, xxx, ..., a : Any) -> Any",
  "$[10 * (int64, int64, xxx, ..., a : Any) -> Any]",
  "xxx[10 * (int64, int64, xxx, ..., a : Any) -> Any]",
  "?$10 * (int64, int64, xxx, ..., a : Any) -> Any]",
  "?=10 * (int64, int64, xxx, ..., a : Any) -> Any]",
  "?[10 * (int64, int64, xxx, ..., a : Any) -> Any$",
  "?[10 * (int64, int64, xxx, ..., a : Any) -> Any$",
  "(int64, int64, xxx, ..., a : Any) -> Any",
  "?(int64, int64, xxx, ..., a : Any) -> Any",
  "?((int64, int64, xxx, ..., a : Any) -> Any)",
  "$(int64, int64, xxx, ..., a : Any) -> Any",
  "=(int64, int64, xxx, ..., a : Any) -> Any",
  "$[(int64, int64, xxx, ..., a : Any) -> Any]",
  "xxx[(int64, int64, xxx, ..., a : Any) -> Any]",
  "?$(int64, int64, xxx, ..., a : Any) -> Any]",
  "?=(int64, int64, xxx, ..., a : Any) -> Any]",
  "?[(int64, int64, xxx, ..., a : Any) -> Any$",
  "?[(int64, int64, xxx, ..., a : Any) -> Any$",
  "10 $ (int64, int64, xxx, ..., a : Any) -> Any",
  "?10 $ (int64, int64, xxx, ..., a : Any) -> Any",
  "?(10 $ (int64, int64, xxx, ..., a : Any) -> Any)",
  "$10 $ (int64, int64, xxx, ..., a : Any) -> Any",
  "=10 $ (int64, int64, xxx, ..., a : Any) -> Any",
  "$[10 $ (int64, int64, xxx, ..., a : Any) -> Any]",
  "xxx[10 $ (int64, int64, xxx, ..., a : Any) -> Any]",
  "?$10 $ (int64, int64, xxx, ..., a : Any) -> Any]",
  "?=10 $ (int64, int64, xxx, ..., a : Any) -> Any]",
  "?[10 $ (int64, int64, xxx, ..., a : Any) -> Any$",
  "?[10 $ (int64, int64, xxx, ..., a : Any) -> Any$",
  "10 = (int64, int64, xxx, ..., a : Any) -> Any",
  "?10 = (int64, int64, xxx, ..., a : Any) -> Any",
  "?(10 = (int64, int64, xxx, ..., a : Any) -> Any)",
  "$10 = (int64, int64, xxx, ..., a : Any) -> Any",
  "=10 = (int64, int64, xxx, ..., a : Any) -> Any",
  "$[10 = (int64, int64, xxx, ..., a : Any) -> Any]",
  "xxx[10 = (int64, int64, xxx, ..., a : Any) -> Any]",
  "?$10 = (int64, int64, xxx, ..., a : Any) -> Any]",
  "?=10 = (int64, int64, xxx, ..., a : Any) -> Any]",
  "?[10 = (int64, int64, xxx, ..., a : Any) -> Any$",
  "?[10 = (int64, int64, xxx, ..., a : Any) -> Any$",
  "10 * (int64, int64$ xxx, a : Any) -> Any",
  "?10 * (int64, int64$ xxx, a : Any) -> Any",
  "?(10 * (int64, int64$ xxx, a : Any) -> Any)",
  "$10 * (int64, int64$ xxx, a : Any) -> Any",
  "=10 * (int64, int64$ xxx, a : Any) -> Any",
  "$[10 * (int64, int64$ xxx, a : Any) -> Any]",
  "xxx[10 * (int64, int64$ xxx, a : Any) -> Any]",
  "?$10 * (int64, int64$ xxx, a : Any) -> Any]",
  "?=10 * (int64, int64$ xxx, a : Any) -> Any]",
  "?[10 * (int64, int64$ xxx, a : Any) -> Any$",
  "?[10 * (int64, int64$ xxx, a : Any) -> Any$",
  "(int64, int64$ xxx, a : Any) -> Any",
  "?(int64, int64$ xxx, a : Any) -> Any",
  "?((int64, int64$ xxx, a : Any) -> Any)",
  "$(int64, int64$ xxx, a : Any) -> Any",
  "=(int64, int64$ xxx, a : Any) -> Any",
  "$[(int64, int64$ xxx, a : Any) -> Any]",
  "xxx[(int64, int64$ xxx, a : Any) -> Any]",
  "?$(int64, int64$ xxx, a : Any) -> Any]",
  "?=(int64, int64$ xxx, a : Any) -> Any]",
  "?[(int64, int64$ xxx, a : Any) -> Any$",
  "?[(int64, int64$ xxx, a : Any) -> Any$",
  "10 $ (int64, int64$ xxx, a : Any) -> Any",
  "?10 $ (int64, int64$ xxx, a : Any) -> Any",
  "?(10 $ (int64, int64$ xxx, a : Any) -> Any)",
  "$10 $ (int64, int64$ xxx, a : Any) -> Any",
  "=10 $ (int64, int64$ xxx, a : Any) -> Any",
  "$[10 $ (int64, int64$ xxx, a : Any) -> Any]",
  "xxx[10 $ (int64, int64$ xxx, a : Any) -> Any]",
  "?$10 $ (int64, int64$ xxx, a : Any) -> Any]",
  "?=10 $ (int64, int64$ xxx, a : Any) -> Any]",
  "?[10 $ (int64, int64$ xxx, a : Any) -> Any$",
  "?[10 $ (int64, int64$ xxx, a : Any) -> Any$",
  "10 = (int64, int64$ xxx, a : Any) -> Any",
  "?10 = (int64, int64$ xxx, a : Any) -> Any",
  "?(10 = (int64, int64$ xxx, a : Any) -> Any)",
  "$10 = (int64, int64$ xxx, a : Any) -> Any",
  "=10 = (int64, int64$ xxx, a : Any) -> Any",
  "$[10 = (int64, int64$ xxx, a : Any) -> Any]",
  "xxx[10 = (int64, int64$ xxx, a : Any) -> Any]",
  "?$10 = (int64, int64$ xxx, a : Any) -> Any]",
  "?=10 = (int64, int64$ xxx, a : Any) -> Any]",
  "?[10 = (int64, int64$ xxx, a : Any) -> Any$",
  "?[10 = (int64, int64$ xxx, a : Any) -> Any$",
  "10 * (int64, int64$ xxx, ..., a : Any) -> Any",
  "?10 * (int64, int64$ xxx, ..., a : Any) -> Any",
  "?(10 * (int64, int64$ xxx, ..., a : Any) -> Any)",
  "$10 * (int64, int64$ xxx, ..., a : Any) -> Any",
  "=10 * (int64, int64$ xxx, ..., a : Any) -> Any",
  "$[10 * (int64, int64$ xxx, ..., a : Any) -> Any]",
  "xxx[10 * (int64, int64$ xxx, ..., a : Any) -> Any]",
  "?$10 * (int64, int64$ xxx, ..., a : Any) -> Any]",
  "?=10 * (int64, int64$ xxx, ..., a : Any) -> Any]",
  "?[10 * (int64, int64$ xxx, ..., a : Any) -> Any$",
  "?[10 * (int64, int64$ xxx, ..., a : Any) -> Any$",
  "(int64, int64$ xxx, ..., a : Any) -> Any",
  "?(int64, int64$ xxx, ..., a : Any) -> Any",
  "?((int64, int64$ xxx, ..., a : Any) -> Any)",
  "$(int64, int64$ xxx, ..., a : Any) -> Any",
  "=(int64, int64$ xxx, ..., a : Any) -> Any",
  "$[(int64, int64$ xxx, ..., a : Any) -> Any]",
  "xxx[(int64, int64$ xxx, ..., a : Any) -> Any]",
  "?$(int64, int64$ xxx, ..., a : Any) -> Any]",
  "?=(int64, int64$ xxx, ..., a : Any) -> Any]",
  "?[(int64, int64$ xxx, ..., a : Any) -> Any$",
  "?[(int64, int64$ xxx, ..., a : Any) -> Any$",
  "10 $ (int64, int64$ xxx, ..., a : Any) -> Any",
  "?10 $ (int64, int64$ xxx, ..., a : Any) -> Any",
  "?(10 $ (int64, int64$ xxx, ..., a : Any) -> Any)",
  "$10 $ (int64, int64$ xxx, ..., a : Any) -> Any",
  "=10 $ (int64, int64$ xxx, ..., a : Any) -> Any",
  "$[10 $ (int64, int64$ xxx, ..., a : Any) -> Any]",
  "xxx[10 $ (int64, int64$ xxx, ..., a : Any) -> Any]",
  "?$10 $ (int64, int64$ xxx, ..., a : Any) -> Any]",
  "?=10 $ (int64, int64$ xxx, ..., a : Any) -> Any]",
  "?[10 $ (int64, int64$ xxx, ..., a : Any) -> Any$",
  "?[10 $ (int64, int64$ xxx, ..., a : Any) -> Any$",
  "10 = (int64, int64$ xxx, ..., a : Any) -> Any",
  "?10 = (int64, int64$ xxx, ..., a : Any) -> Any",
  "?(10 = (int64, int64$ xxx, ..., a : Any) -> Any)",
  "$10 = (int64, int64$ xxx, ..., a : Any) -> Any",
  "=10 = (int64, int64$ xxx, ..., a : Any) -> Any",
  "$[10 = (int64, int64$ xxx, ..., a : Any) -> Any]",
  "xxx[10 = (int64, int64$ xxx, ..., a : Any) -> Any]",
  "?$10 = (int64, int64$ xxx, ..., a : Any) -> Any]",
  "?=10 = (int64, int64$ xxx, ..., a : Any) -> Any]",
  "?[10 = (int64, int64$ xxx, ..., a : Any) -> Any$",
  "?[10 = (int64, int64$ xxx, ..., a : Any) -> Any$",
  "10 * (int64, int64= xxx, a : Any) -> Any",
  "?10 * (int64, int64= xxx, a : Any) -> Any",
  "?(10 * (int64, int64= xxx, a : Any) -> Any)",
  "$10 * (int64, int64= xxx, a : Any) -> Any",
  "=10 * (int64, int64= xxx, a : Any) -> Any",
  "$[10 * (int64, int64= xxx, a : Any) -> Any]",
  "xxx[10 * (int64, int64= xxx, a : Any) -> Any]",
  "?$10 * (int64, int64= xxx, a : Any) -> Any]",
  "?=10 * (int64, int64= xxx, a : Any) -> Any]",
  "?[10 * (int64, int64= xxx, a : Any) -> Any$",
  "?[10 * (int64, int64= xxx, a : Any) -> Any$",
  "(int64, int64= xxx, a : Any) -> Any",
  "?(int64, int64= xxx, a : Any) -> Any",
  "?((int64, int64= xxx, a : Any) -> Any)",
  "$(int64, int64= xxx, a : Any) -> Any",
  "=(int64, int64= xxx, a : Any) -> Any",
  "$[(int64, int64= xxx, a : Any) -> Any]",
  "xxx[(int64, int64= xxx, a : Any) -> Any]",
  "?$(int64, int64= xxx, a : Any) -> Any]",
  "?=(int64, int64= xxx, a : Any) -> Any]",
  "?[(int64, int64= xxx, a : Any) -> Any$",
  "?[(int64, int64= xxx, a : Any) -> Any$",
  "10 $ (int64, int64= xxx, a : Any) -> Any",
  "?10 $ (int64, int64= xxx, a : Any) -> Any",
  "?(10 $ (int64, int64= xxx, a : Any) -> Any)",
  "$10 $ (int64, int64= xxx, a : Any) -> Any",
  "=10 $ (int64, int64= xxx, a : Any) -> Any",
  "$[10 $ (int64, int64= xxx, a : Any) -> Any]",
  "xxx[10 $ (int64, int64= xxx, a : Any) -> Any]",
  "?$10 $ (int64, int64= xxx, a : Any) -> Any]",
  "?=10 $ (int64, int64= xxx, a : Any) -> Any]",
  "?[10 $ (int64, int64= xxx, a : Any) -> Any$",
  "?[10 $ (int64, int64= xxx, a : Any) -> Any$",
  "10 = (int64, int64= xxx, a : Any) -> Any",
  "?10 = (int64, int64= xxx, a : Any) -> Any",
  "?(10 = (int64, int64= xxx, a : Any) -> Any)",
  "$10 = (int64, int64= xxx, a : Any) -> Any",
  "=10 = (int64, int64= xxx, a : Any) -> Any",
  "$[10 = (int64, int64= xxx, a : Any) -> Any]",
  "xxx[10 = (int64, int64= xxx, a : Any) -> Any]",
  "?$10 = (int64, int64= xxx, a : Any) -> Any]",
  "?=10 = (int64, int64= xxx, a : Any) -> Any]",
  "?[10 = (int64, int64= xxx, a : Any) -> Any$",
  "?[10 = (int64, int64= xxx, a : Any) -> Any$",
  "10 * (int64, int64= xxx, ..., a : Any) -> Any",
  "?10 * (int64, int64= xxx, ..., a : Any) -> Any",
  "?(10 * (int64, int64= xxx, ..., a : Any) -> Any)",
  "$10 * (int64, int64= xxx, ..., a : Any) -> Any",
  "=10 * (int64, int64= xxx, ..., a : Any) -> Any",
  "$[10 * (int64, int64= xxx, ..., a : Any) -> Any]",
  "xxx[10 * (int64, int64= xxx, ..., a : Any) -> Any]",
  "?$10 * (int64, int64= xxx, ..., a : Any) -> Any]",
  "?=10 * (int64, int64= xxx, ..., a : Any) -> Any]",
  "?[10 * (int64, int64= xxx, ..., a : Any) -> Any$",
  "?[10 * (int64, int64= xxx, ..., a : Any) -> Any$",
  "(int64, int64= xxx, ..., a : Any) -> Any",
  "?(int64, int64= xxx, ..., a : Any) -> Any",
  "?((int64, int64= xxx, ..., a : Any) -> Any)",
  "$(int64, int64= xxx, ..., a : Any) -> Any",
  "=(int64, int64= xxx, ..., a : Any) -> Any",
  "$[(int64, int64= xxx, ..., a : Any) -> Any]",
  "xxx[(int64, int64= xxx, ..., a : Any) -> Any]",
  "?$(int64, int64= xxx, ..., a : Any) -> Any]",
  "?=(int64, int64= xxx, ..., a : Any) -> Any]",
  "?[(int64, int64= xxx, ..., a : Any) -> Any$",
  "?[(int64, int64= xxx, ..., a : Any) -> Any$",
  "10 $ (int64, int64= xxx, ..., a : Any) -> Any",
  "?10 $ (int64, int64= xxx, ..., a : Any) -> Any",
  "?(10 $ (int64, int64= xxx, ..., a : Any) -> Any)",
  "$10 $ (int64, int64= xxx, ..., a : Any) -> Any",
  "=10 $ (int64, int64= xxx, ..., a : Any) -> Any",
  "$[10 $ (int64, int64= xxx, ..., a : Any) -> Any]",
  "xxx[10 $ (int64, int64= xxx, ..., a : Any) -> Any]",
  "?$10 $ (int64, int64= xxx, ..., a : Any) -> Any]",
  "?=10 $ (int64, int64= xxx, ..., a : Any) -> Any]",
  "?[10 $ (int64, int64= xxx, ..., a : Any) -> Any$",
  "?[10 $ (int64, int64= xxx, ..., a : Any) -> Any$",
  "10 = (int64, int64= xxx, ..., a : Any) -> Any",
  "?10 = (int64, int64= xxx, ..., a : Any) -> Any",
  "?(10 = (int64, int64= xxx, ..., a : Any) -> Any)",
  "$10 = (int64, int64= xxx, ..., a : Any) -> Any",
  "=10 = (int64, int64= xxx, ..., a : Any) -> Any",
  "$[10 = (int64, int64= xxx, ..., a : Any) -> Any]",
  "xxx[10 = (int64, int64= xxx, ..., a : Any) -> Any]",
  "?$10 = (int64, int64= xxx, ..., a : Any) -> Any]",
  "?=10 = (int64, int64= xxx, ..., a : Any) -> Any]",
  "?[10 = (int64, int64= xxx, ..., a : Any) -> Any$",
  "?[10 = (int64, int64= xxx, ..., a : Any) -> Any$",
  "10 * ($, a : Any) -> Any",
  "?10 * ($, a : Any) -> Any",
  "?(10 * ($, a : Any) -> Any)",
  "$10 * ($, a : Any) -> Any",
  "=10 * ($, a : Any) -> Any",
  "$[10 * ($, a : Any) -> Any]",
  "xxx[10 * ($, a : Any) -> Any]",
  "?$10 * ($, a : Any) -> Any]",
  "?=10 * ($, a : Any) -> Any]",
  "?[10 * ($, a : Any) -> Any$",
  "?[10 * ($, a : Any) -> Any$",
  "($, a : Any) -> Any",
  "?($, a : Any) -> Any",
  "?(($, a : Any) -> Any)",
  "$($, a : Any) -> Any",
  "=($, a : Any) -> Any",
  "$[($, a : Any) -> Any]",
  "xxx[($, a : Any) -> Any]",
  "?$($, a : Any) -> Any]",
  "?=($, a : Any) -> Any]",
  "?[($, a : Any) -> Any$",
  "?[($, a : Any) -> Any$",
  "10 $ ($, a : Any) -> Any",
  "?10 $ ($, a : Any) -> Any",
  "?(10 $ ($, a : Any) -> Any)",
  "$10 $ ($, a : Any) -> Any",
  "=10 $ ($, a : Any) -> Any",
  "$[10 $ ($, a : Any) -> Any]",
  "xxx[10 $ ($, a : Any) -> Any]",
  "?$10 $ ($, a : Any) -> Any]",
  "?=10 $ ($, a : Any) -> Any]",
  "?[10 $ ($, a : Any) -> Any$",
  "?[10 $ ($, a : Any) -> Any$",
  "10 = ($, a : Any) -> Any",
  "?10 = ($, a : Any) -> Any",
  "?(10 = ($, a : Any) -> Any)",
  "$10 = ($, a : Any) -> Any",
  "=10 = ($, a : Any) -> Any",
  "$[10 = ($, a : Any) -> Any]",
  "xxx[10 = ($, a : Any) -> Any]",
  "?$10 = ($, a : Any) -> Any]",
  "?=10 = ($, a : Any) -> Any]",
  "?[10 = ($, a : Any) -> Any$",
  "?[10 = ($, a : Any) -> Any$",
  "10 * ($, ..., a : Any) -> Any",
  "?10 * ($, ..., a : Any) -> Any",
  "?(10 * ($, ..., a : Any) -> Any)",
  "$10 * ($, ..., a : Any) -> Any",
  "=10 * ($, ..., a : Any) -> Any",
  "$[10 * ($, ..., a : Any) -> Any]",
  "xxx[10 * ($, ..., a : Any) -> Any]",
  "?$10 * ($, ..., a : Any) -> Any]",
  "?=10 * ($, ..., a : Any) -> Any]",
  "?[10 * ($, ..., a : Any) -> Any$",
  "?[10 * ($, ..., a : Any) -> Any$",
  "($, ..., a : Any) -> Any",
  "?($, ..., a : Any) -> Any",
  "?(($, ..., a : Any) -> Any)",
  "$($, ..., a : Any) -> Any",
  "=($, ..., a : Any) -> Any",
  "$[($, ..., a : Any) -> Any]",
  "xxx[($, ..., a : Any) -> Any]",
  "?$($, ..., a : Any) -> Any]",
  "?=($, ..., a : Any) -> Any]",
  "?[($, ..., a : Any) -> Any$",
  "?[($, ..., a : Any) -> Any$",
  "10 $ ($, ..., a : Any) -> Any",
  "?10 $ ($, ..., a : Any) -> Any",
  "?(10 $ ($, ..., a : Any) -> Any)",
  "$10 $ ($, ..., a : Any) -> Any",
  "=10 $ ($, ..., a : Any) -> Any",
  "$[10 $ ($, ..., a : Any) -> Any]",
  "xxx[10 $ ($, ..., a : Any) -> Any]",
  "?$10 $ ($, ..., a : Any) -> Any]",
  "?=10 $ ($, ..., a : Any) -> Any]",
  "?[10 $ ($, ..., a : Any) -> Any$",
  "?[10 $ ($, ..., a : Any) -> Any$",
  "10 = ($, ..., a : Any) -> Any",
  "?10 = ($, ..., a : Any) -> Any",
  "?(10 = ($, ..., a : Any) -> Any)",
  "$10 = ($, ..., a : Any) -> Any",
  "=10 = ($, ..., a : Any) -> Any",
  "$[10 = ($, ..., a : Any) -> Any]",
  "xxx[10 = ($, ..., a : Any) -> Any]",
  "?$10 = ($, ..., a : Any) -> Any]",
  "?=10 = ($, ..., a : Any) -> Any]",
  "?[10 = ($, ..., a : Any) -> Any$",
  "?[10 = ($, ..., a : Any) -> Any$",
  "10 * (int64, $, a : Any) -> Any",
  "?10 * (int64, $, a : Any) -> Any",
  "?(10 * (int64, $, a : Any) -> Any)",
  "$10 * (int64, $, a : Any) -> Any",
  "=10 * (int64, $, a : Any) -> Any",
  "$[10 * (int64, $, a : Any) -> Any]",
  "xxx[10 * (int64, $, a : Any) -> Any]",
  "?$10 * (int64, $, a : Any) -> Any]",
  "?=10 * (int64, $, a : Any) -> Any]",
  "?[10 * (int64, $, a : Any) -> Any$",
  "?[10 * (int64, $, a : Any) -> Any$",
  "(int64, $, a : Any) -> Any",
  "?(int64, $, a : Any) -> Any",
  "?((int64, $, a : Any) -> Any)",
  "$(int64, $, a : Any) -> Any",
  "=(int64, $, a : Any) -> Any",
  "$[(int64, $, a : Any) -> Any]",
  "xxx[(int64, $, a : Any) -> Any]",
  "?$(int64, $, a : Any) -> Any]",
  "?=(int64, $, a : Any) -> Any]",
  "?[(int64, $, a : Any) -> Any$",
  "?[(int64, $, a : Any) -> Any$",
  "10 $ (int64, $, a : Any) -> Any",
  "?10 $ (int64, $, a : Any) -> Any",
  "?(10 $ (int64, $, a : Any) -> Any)",
  "$10 $ (int64, $, a : Any) -> Any",
  "=10 $ (int64, $, a : Any) -> Any",
  "$[10 $ (int64, $, a : Any) -> Any]",
  "xxx[10 $ (int64, $, a : Any) -> Any]",
  "?$10 $ (int64, $, a : Any) -> Any]",
  "?=10 $ (int64, $, a : Any) -> Any]",
  "?[10 $ (int64, $, a : Any) -> Any$",
  "?[10 $ (int64, $, a : Any) -> Any$",
  "10 = (int64, $, a : Any) -> Any",
  "?10 = (int64, $, a : Any) -> Any",
  "?(10 = (int64, $, a : Any) -> Any)",
  "$10 = (int64, $, a : Any) -> Any",
  "=10 = (int64, $, a : Any) -> Any",
  "$[10 = (int64, $, a : Any) -> Any]",
  "xxx[10 = (int64, $, a : Any) -> Any]",
  "?$10 = (int64, $, a : Any) -> Any]",
  "?=10 = (int64, $, a : Any) -> Any]",
  "?[10 = (int64, $, a : Any) -> Any$",
  "?[10 = (int64, $, a : Any) -> Any$",
  "10 * (int64, $, ..., a : Any) -> Any",
  "?10 * (int64, $, ..., a : Any) -> Any",
  "?(10 * (int64, $, ..., a : Any) -> Any)",
  "$10 * (int64, $, ..., a : Any) -> Any",
  "=10 * (int64, $, ..., a : Any) -> Any",
  "$[10 * (int64, $, ..., a : Any) -> Any]",
  "xxx[10 * (int64, $, ..., a : Any) -> Any]",
  "?$10 * (int64, $, ..., a : Any) -> Any]",
  "?=10 * (int64, $, ..., a : Any) -> Any]",
  "?[10 * (int64, $, ..., a : Any) -> Any$",
  "?[10 * (int64, $, ..., a : Any) -> Any$",
  "(int64, $, ..., a : Any) -> Any",
  "?(int64, $, ..., a : Any) -> Any",
  "?((int64, $, ..., a : Any) -> Any)",
  "$(int64, $, ..., a : Any) -> Any",
  "=(int64, $, ..., a : Any) -> Any",
  "$[(int64, $, ..., a : Any) -> Any]",
  "xxx[(int64, $, ..., a : Any) -> Any]",
  "?$(int64, $, ..., a : Any) -> Any]",
  "?=(int64, $, ..., a : Any) -> Any]",
  "?[(int64, $, ..., a : Any) -> Any$",
  "?[(int64, $, ..., a : Any) -> Any$",
  "10 $ (int64, $, ..., a : Any) -> Any",
  "?10 $ (int64, $, ..., a : Any) -> Any",
  "?(10 $ (int64, $, ..., a : Any) -> Any)",
  "$10 $ (int64, $, ..., a : Any) -> Any",
  "=10 $ (int64, $, ..., a : Any) -> Any",
  "$[10 $ (int64, $, ..., a : Any) -> Any]",
  "xxx[10 $ (int64, $, ..., a : Any) -> Any]",
  "?$10 $ (int64, $, ..., a : Any) -> Any]",
  "?=10 $ (int64, $, ..., a : Any) -> Any]",
  "?[10 $ (int64, $, ..., a : Any) -> Any$",
  "?[10 $ (int64, $, ..., a : Any) -> Any$",
  "10 = (int64, $, ..., a : Any) -> Any",
  "?10 = (int64, $, ..., a : Any) -> Any",
  "?(10 = (int64, $, ..., a : Any) -> Any)",
  "$10 = (int64, $, ..., a : Any) -> Any",
  "=10 = (int64, $, ..., a : Any) -> Any",
  "$[10 = (int64, $, ..., a : Any) -> Any]",
  "xxx[10 = (int64, $, ..., a : Any) -> Any]",
  "?$10 = (int64, $, ..., a : Any) -> Any]",
  "?=10 = (int64, $, ..., a : Any) -> Any]",
  "?[10 = (int64, $, ..., a : Any) -> Any$",
  "?[10 = (int64, $, ..., a : Any) -> Any$",
  "10 * (int64$ $, a : Any) -> Any",
  "?10 * (int64$ $, a : Any) -> Any",
  "?(10 * (int64$ $, a : Any) -> Any)",
  "$10 * (int64$ $, a : Any) -> Any",
  "=10 * (int64$ $, a : Any) -> Any",
  "$[10 * (int64$ $, a : Any) -> Any]",
  "xxx[10 * (int64$ $, a : Any) -> Any]",
  "?$10 * (int64$ $, a : Any) -> Any]",
  "?=10 * (int64$ $, a : Any) -> Any]",
  "?[10 * (int64$ $, a : Any) -> Any$",
  "?[10 * (int64$ $, a : Any) -> Any$",
  "(int64$ $, a : Any) -> Any",
  "?(int64$ $, a : Any) -> Any",
  "?((int64$ $, a : Any) -> Any)",
  "$(int64$ $, a : Any) -> Any",
  "=(int64$ $, a : Any) -> Any",
  "$[(int64$ $, a : Any) -> Any]",
  "xxx[(int64$ $, a : Any) -> Any]",
  "?$(int64$ $, a : Any) -> Any]",
  "?=(int64$ $, a : Any) -> Any]",
  "?[(int64$ $, a : Any) -> Any$",
  "?[(int64$ $, a : Any) -> Any$",
  "10 $ (int64$ $, a : Any) -> Any",
  "?10 $ (int64$ $, a : Any) -> Any",
  "?(10 $ (int64$ $, a : Any) -> Any)",
  "$10 $ (int64$ $, a : Any) -> Any",
  "=10 $ (int64$ $, a : Any) -> Any",
  "$[10 $ (int64$ $, a : Any) -> Any]",
  "xxx[10 $ (int64$ $, a : Any) -> Any]",
  "?$10 $ (int64$ $, a : Any) -> Any]",
  "?=10 $ (int64$ $, a : Any) -> Any]",
  "?[10 $ (int64$ $, a : Any) -> Any$",
  "?[10 $ (int64$ $, a : Any) -> Any$",
  "10 = (int64$ $, a : Any) -> Any",
  "?10 = (int64$ $, a : Any) -> Any",
  "?(10 = (int64$ $, a : Any) -> Any)",
  "$10 = (int64$ $, a : Any) -> Any",
  "=10 = (int64$ $, a : Any) -> Any",
  "$[10 = (int64$ $, a : Any) -> Any]",
  "xxx[10 = (int64$ $, a : Any) -> Any]",
  "?$10 = (int64$ $, a : Any) -> Any]",
  "?=10 = (int64$ $, a : Any) -> Any]",
  "?[10 = (int64$ $, a : Any) -> Any$",
  "?[10 = (int64$ $, a : Any) -> Any$",
  "10 * (int64$ $, ..., a : Any) -> Any",
  "?10 * (int64$ $, ..., a : Any) -> Any",
  "?(10 * (int64$ $, ..., a : Any) -> Any)",
  "$10 * (int64$ $, ..., a : Any) -> Any",
  "=10 * (int64$ $, ..., a : Any) -> Any",
  "$[10 * (int64$ $, ..., a : Any) -> Any]",
  "xxx[10 * (int64$ $, ..., a : Any) -> Any]",
  "?$10 * (int64$ $, ..., a : Any) -> Any]",
  "?=10 * (int64$ $, ..., a : Any) -> Any]",
  "?[10 * (int64$ $, ..., a : Any) -> Any$",
  "?[10 * (int64$ $, ..., a : Any) -> Any$",
  "(int64$ $, ..., a : Any) -> Any",
  "?(int64$ $, ..., a : Any) -> Any",
  "?((int64$ $, ..., a : Any) -> Any)",
  "$(int64$ $, ..., a : Any) -> Any",
  "=(int64$ $, ..., a : Any) -> Any",
  "$[(int64$ $, ..., a : Any) -> Any]",
  "xxx[(int64$ $, ..., a : Any) -> Any]",
  "?$(int64$ $, ..., a : Any) -> Any]",
  "?=(int64$ $, ..., a : Any) -> Any]",
  "?[(int64$ $, ..., a : Any) -> Any$",
  "?[(int64$ $, ..., a : Any) -> Any$",
  "10 $ (int64$ $, ..., a : Any) -> Any",
  "?10 $ (int64$ $, ..., a : Any) -> Any",
  "?(10 $ (int64$ $, ..., a : Any) -> Any)",
  "$10 $ (int64$ $, ..., a : Any) -> Any",
  "=10 $ (int64$ $, ..., a : Any) -> Any",
  "$[10 $ (int64$ $, ..., a : Any) -> Any]",
  "xxx[10 $ (int64$ $, ..., a : Any) -> Any]",
  "?$10 $ (int64$ $, ..., a : Any) -> Any]",
  "?=10 $ (int64$ $, ..., a : Any) -> Any]",
  "?[10 $ (int64$ $, ..., a : Any) -> Any$",
  "?[10 $ (int64$ $, ..., a : Any) -> Any$",
  "10 = (int64$ $, ..., a : Any) -> Any",
  "?10 = (int64$ $, ..., a : Any) -> Any",
  "?(10 = (int64$ $, ..., a : Any) -> Any)",
  "$10 = (int64$ $, ..., a : Any) -> Any",
  "=10 = (int64$ $, ..., a : Any) -> Any",
  "$[10 = (int64$ $, ..., a : Any) -> Any]",
  "xxx[10 = (int64$ $, ..., a : Any) -> Any]",
  "?$10 = (int64$ $, ..., a : Any) -> Any]",
  "?=10 = (int64$ $, ..., a : Any) -> Any]",
  "?[10 = (int64$ $, ..., a : Any) -> Any$",
  "?[10 = (int64$ $, ..., a : Any) -> Any$",
  "10 * (int64= $, a : Any) -> Any",
  "?10 * (int64= $, a : Any) -> Any",
  "?(10 * (int64= $, a : Any) -> Any)",
  "$10 * (int64= $, a : Any) -> Any",
  "=10 * (int64= $, a : Any) -> Any",
  "$[10 * (int64= $, a : Any) -> Any]",
  "xxx[10 * (int64= $, a : Any) -> Any]",
  "?$10 * (int64= $, a : Any) -> Any]",
  "?=10 * (int64= $, a : Any) -> Any]",
  "?[10 * (int64= $, a : Any) -> Any$",
  "?[10 * (int64= $, a : Any) -> Any$",
  "(int64= $, a : Any) -> Any",
  "?(int64= $, a : Any) -> Any",
  "?((int64= $, a : Any) -> Any)",
  "$(int64= $, a : Any) -> Any",
  "=(int64= $, a : Any) -> Any",
  "$[(int64= $, a : Any) -> Any]",
  "xxx[(int64= $, a : Any) -> Any]",
  "?$(int64= $, a : Any) -> Any]",
  "?=(int64= $, a : Any) -> Any]",
  "?[(int64= $, a : Any) -> Any$",
  "?[(int64= $, a : Any) -> Any$",
  "10 $ (int64= $, a : Any) -> Any",
  "?10 $ (int64= $, a : Any) -> Any",
  "?(10 $ (int64= $, a : Any) -> Any)",
  "$10 $ (int64= $, a : Any) -> Any",
  "=10 $ (int64= $, a : Any) -> Any",
  "$[10 $ (int64= $, a : Any) -> Any]",
  "xxx[10 $ (int64= $, a : Any) -> Any]",
  "?$10 $ (int64= $, a : Any) -> Any]",
  "?=10 $ (int64= $, a : Any) -> Any]",
  "?[10 $ (int64= $, a : Any) -> Any$",
  "?[10 $ (int64= $, a : Any) -> Any$",
  "10 = (int64= $, a : Any) -> Any",
  "?10 = (int64= $, a : Any) -> Any",
  "?(10 = (int64= $, a : Any) -> Any)",
  "$10 = (int64= $, a : Any) -> Any",
  "=10 = (int64= $, a : Any) -> Any",
  "$[10 = (int64= $, a : Any) -> Any]",
  "xxx[10 = (int64= $, a : Any) -> Any]",
  "?$10 = (int64= $, a : Any) -> Any]",
  "?=10 = (int64= $, a : Any) -> Any]",
  "?[10 = (int64= $, a : Any) -> Any$",
  "?[10 = (int64= $, a : Any) -> Any$",
  "10 * (int64= $, ..., a : Any) -> Any",
  "?10 * (int64= $, ..., a : Any) -> Any",
  "?(10 * (int64= $, ..., a : Any) -> Any)",
  "$10 * (int64= $, ..., a : Any) -> Any",
  "=10 * (int64= $, ..., a : Any) -> Any",
  "$[10 * (int64= $, ..., a : Any) -> Any]",
  "xxx[10 * (int64= $, ..., a : Any) -> Any]",
  "?$10 * (int64= $, ..., a : Any) -> Any]",
  "?=10 * (int64= $, ..., a : Any) -> Any]",
  "?[10 * (int64= $, ..., a : Any) -> Any$",
  "?[10 * (int64= $, ..., a : Any) -> Any$",
  "(int64= $, ..., a : Any) -> Any",
  "?(int64= $, ..., a : Any) -> Any",
  "?((int64= $, ..., a : Any) -> Any)",
  "$(int64= $, ..., a : Any) -> Any",
  "=(int64= $, ..., a : Any) -> Any",
  "$[(int64= $, ..., a : Any) -> Any]",
  "xxx[(int64= $, ..., a : Any) -> Any]",
  "?$(int64= $, ..., a : Any) -> Any]",
  "?=(int64= $, ..., a : Any) -> Any]",
  "?[(int64= $, ..., a : Any) -> Any$",
  "?[(int64= $, ..., a : Any) -> Any$",
  "10 $ (int64= $, ..., a : Any) -> Any",
  "?10 $ (int64= $, ..., a : Any) -> Any",
  "?(10 $ (int64= $, ..., a : Any) -> Any)",
  "$10 $ (int64= $, ..., a : Any) -> Any",
  "=10 $ (int64= $, ..., a : Any) -> Any",
  "$[10 $ (int64= $, ..., a : Any) -> Any]",
  "xxx[10 $ (int64= $, ..., a : Any) -> Any]",
  "?$10 $ (int64= $, ..., a : Any) -> Any]",
  "?=10 $ (int64= $, ..., a : Any) -> Any]",
  "?[10 $ (int64= $, ..., a : Any) -> Any$",
  "?[10 $ (int64= $, ..., a : Any) -> Any$",
  "10 = (int64= $, ..., a : Any) -> Any",
  "?10 = (int64= $, ..., a : Any) -> Any",
  "?(10 = (int64= $, ..., a : Any) -> Any)",
  "$10 = (int64= $, ..., a : Any) -> Any",
  "=10 = (int64= $, ..., a : Any) -> Any",
  "$[10 = (int64= $, ..., a : Any) -> Any]",
  "xxx[10 = (int64= $, ..., a : Any) -> Any]",
  "?$10 = (int64= $, ..., a : Any) -> Any]",
  "?=10 = (int64= $, ..., a : Any) -> Any]",
  "?[10 = (int64= $, ..., a : Any) -> Any$",
  "?[10 = (int64= $, ..., a : Any) -> Any$",
  "10 * (int64, int64, $, a : Any) -> Any",
  "?10 * (int64, int64, $, a : Any) -> Any",
  "?(10 * (int64, int64, $, a : Any) -> Any)",
  "$10 * (int64, int64, $, a : Any) -> Any",
  "=10 * (int64, int64, $, a : Any) -> Any",
  "$[10 * (int64, int64, $, a : Any) -> Any]",
  "xxx[10 * (int64, int64, $, a : Any) -> Any]",
  "?$10 * (int64, int64, $, a : Any) -> Any]",
  "?=10 * (int64, int64, $, a : Any) -> Any]",
  "?[10 * (int64, int64, $, a : Any) -> Any$",
  "?[10 * (int64, int64, $, a : Any) -> Any$",
  "(int64, int64, $, a : Any) -> Any",
  "?(int64, int64, $, a : Any) -> Any",
  "?((int64, int64, $, a : Any) -> Any)",
  "$(int64, int64, $, a : Any) -> Any",
  "=(int64, int64, $, a : Any) -> Any",
  "$[(int64, int64, $, a : Any) -> Any]",
  "xxx[(int64, int64, $, a : Any) -> Any]",
  "?$(int64, int64, $, a : Any) -> Any]",
  "?=(int64, int64, $, a : Any) -> Any]",
  "?[(int64, int64, $, a : Any) -> Any$",
  "?[(int64, int64, $, a : Any) -> Any$",
  "10 $ (int64, int64, $, a : Any) -> Any",
  "?10 $ (int64, int64, $, a : Any) -> Any",
  "?(10 $ (int64, int64, $, a : Any) -> Any)",
  "$10 $ (int64, int64, $, a : Any) -> Any",
  "=10 $ (int64, int64, $, a : Any) -> Any",
  "$[10 $ (int64, int64, $, a : Any) -> Any]",
  "xxx[10 $ (int64, int64, $, a : Any) -> Any]",
  "?$10 $ (int64, int64, $, a : Any) -> Any]",
  "?=10 $ (int64, int64, $, a : Any) -> Any]",
  "?[10 $ (int64, int64, $, a : Any) -> Any$",
  "?[10 $ (int64, int64, $, a : Any) -> Any$",
  "10 = (int64, int64, $, a : Any) -> Any",
  "?10 = (int64, int64, $, a : Any) -> Any",
  "?(10 = (int64, int64, $, a : Any) -> Any)",
  "$10 = (int64, int64, $, a : Any) -> Any",
  "=10 = (int64, int64, $, a : Any) -> Any",
  "$[10 = (int64, int64, $, a : Any) -> Any]",
  "xxx[10 = (int64, int64, $, a : Any) -> Any]",
  "?$10 = (int64, int64, $, a : Any) -> Any]",
  "?=10 = (int64, int64, $, a : Any) -> Any]",
  "?[10 = (int64, int64, $, a : Any) -> Any$",
  "?[10 = (int64, int64, $, a : Any) -> Any$",
  "10 * (int64, int64, $, ..., a : Any) -> Any",
  "?10 * (int64, int64, $, ..., a : Any) -> Any",
  "?(10 * (int64, int64, $, ..., a : Any) -> Any)",
  "$10 * (int64, int64, $, ..., a : Any) -> Any",
  "=10 * (int64, int64, $, ..., a : Any) -> Any",
  "$[10 * (int64, int64, $, ..., a : Any) -> Any]",
  "xxx[10 * (int64, int64, $, ..., a : Any) -> Any]",
  "?$10 * (int64, int64, $, ..., a : Any) -> Any]",
  "?=10 * (int64, int64, $, ..., a : Any) -> Any]",
  "?[10 * (int64, int64, $, ..., a : Any) -> Any$",
  "?[10 * (int64, int64, $, ..., a : Any) -> Any$",
  "(int64, int64, $, ..., a : Any) -> Any",
  "?(int64, int64, $, ..., a : Any) -> Any",
  "?((int64, int64, $, ..., a : Any) -> Any)",
  "$(int64, int64, $, ..., a : Any) -> Any",
  "=(int64, int64, $, ..., a : Any) -> Any",
  "$[(int64, int64, $, ..., a : Any) -> Any]",
  "xxx[(int64, int64, $, ..., a : Any) -> Any]",
  "?$(int64, int64, $, ..., a : Any) -> Any]",
  "?=(int64, int64, $, ..., a : Any) -> Any]",
  "?[(int64, int64, $, ..., a : Any) -> Any$",
  "?[(int64, int64, $, ..., a : Any) -> Any$",
  "10 $ (int64, int64, $, ..., a : Any) -> Any",
  "?10 $ (int64, int64, $, ..., a : Any) -> Any",
  "?(10 $ (int64, int64, $, ..., a : Any) -> Any)",
  "$10 $ (int64, int64, $, ..., a : Any) -> Any",
  "=10 $ (int64, int64, $, ..., a : Any) -> Any",
  "$[10 $ (int64, int64, $, ..., a : Any) -> Any]",
  "xxx[10 $ (int64, int64, $, ..., a : Any) -> Any]",
  "?$10 $ (int64, int64, $, ..., a : Any) -> Any]",
  "?=10 $ (int64, int64, $, ..., a : Any) -> Any]",
  "?[10 $ (int64, int64, $, ..., a : Any) -> Any$",
  "?[10 $ (int64, int64, $, ..., a : Any) -> Any$",
  "10 = (int64, int64, $, ..., a : Any) -> Any",
  "?10 = (int64, int64, $, ..., a : Any) -> Any",
  "?(10 = (int64, int64, $, ..., a : Any) -> Any)",
  "$10 = (int64, int64, $, ..., a : Any) -> Any",
  "=10 = (int64, int64, $, ..., a : Any) -> Any",
  "$[10 = (int64, int64, $, ..., a : Any) -> Any]",
  "xxx[10 = (int64, int64, $, ..., a : Any) -> Any]",
  "?$10 = (int64, int64, $, ..., a : Any) -> Any]",
  "?=10 = (int64, int64, $, ..., a : Any) -> Any]",
  "?[10 = (int64, int64, $, ..., a : Any) -> Any$",
  "?[10 = (int64, int64, $, ..., a : Any) -> Any$",
  "10 * (int64, int64$ $, a : Any) -> Any",
  "?10 * (int64, int64$ $, a : Any) -> Any",
  "?(10 * (int64, int64$ $, a : Any) -> Any)",
  "$10 * (int64, int64$ $, a : Any) -> Any",
  "=10 * (int64, int64$ $, a : Any) -> Any",
  "$[10 * (int64, int64$ $, a : Any) -> Any]",
  "xxx[10 * (int64, int64$ $, a : Any) -> Any]",
  "?$10 * (int64, int64$ $, a : Any) -> Any]",
  "?=10 * (int64, int64$ $, a : Any) -> Any]",
  "?[10 * (int64, int64$ $, a : Any) -> Any$",
  "?[10 * (int64, int64$ $, a : Any) -> Any$",
  "(int64, int64$ $, a : Any) -> Any",
  "?(int64, int64$ $, a : Any) -> Any",
  "?((int64, int64$ $, a : Any) -> Any)",
  "$(int64, int64$ $, a : Any) -> Any",
  "=(int64, int64$ $, a : Any) -> Any",
  "$[(int64, int64$ $, a : Any) -> Any]",
  "xxx[(int64, int64$ $, a : Any) -> Any]",
  "?$(int64, int64$ $, a : Any) -> Any]",
  "?=(int64, int64$ $, a : Any) -> Any]",
  "?[(int64, int64$ $, a : Any) -> Any$",
  "?[(int64, int64$ $, a : Any) -> Any$",
  "10 $ (int64, int64$ $, a : Any) -> Any",
  "?10 $ (int64, int64$ $, a : Any) -> Any",
  "?(10 $ (int64, int64$ $, a : Any) -> Any)",
  "$10 $ (int64, int64$ $, a : Any) -> Any",
  "=10 $ (int64, int64$ $, a : Any) -> Any",
  "$[10 $ (int64, int64$ $, a : Any) -> Any]",
  "xxx[10 $ (int64, int64$ $, a : Any) -> Any]",
  "?$10 $ (int64, int64$ $, a : Any) -> Any]",
  "?=10 $ (int64, int64$ $, a : Any) -> Any]",
  "?[10 $ (int64, int64$ $, a : Any) -> Any$",
  "?[10 $ (int64, int64$ $, a : Any) -> Any$",
  "10 = (int64, int64$ $, a : Any) -> Any",
  "?10 = (int64, int64$ $, a : Any) -> Any",
  "?(10 = (int64, int64$ $, a : Any) -> Any)",
  "$10 = (int64, int64$ $, a : Any) -> Any",
  "=10 = (int64, int64$ $, a : Any) -> Any",
  "$[10 = (int64, int64$ $, a : Any) -> Any]",
  "xxx[10 = (int64, int64$ $, a : Any) -> Any]",
  "?$10 = (int64, int64$ $, a : Any) -> Any]",
  "?=10 = (int64, int64$ $, a : Any) -> Any]",
  "?[10 = (int64, int64$ $, a : Any) -> Any$",
  "?[10 = (int64, int64$ $, a : Any) -> Any$",
  "10 * (int64, int64$ $, ..., a : Any) -> Any",
  "?10 * (int64, int64$ $, ..., a : Any) -> Any",
  "?(10 * (int64, int64$ $, ..., a : Any) -> Any)",
  "$10 * (int64, int64$ $, ..., a : Any) -> Any",
  "=10 * (int64, int64$ $, ..., a : Any) -> Any",
  "$[10 * (int64, int64$ $, ..., a : Any) -> Any]",
  "xxx[10 * (int64, int64$ $, ..., a : Any) -> Any]",
  "?$10 * (int64, int64$ $, ..., a : Any) -> Any]",
  "?=10 * (int64, int64$ $, ..., a : Any) -> Any]",
  "?[10 * (int64, int64$ $, ..., a : Any) -> Any$",
  "?[10 * (int64, int64$ $, ..., a : Any) -> Any$",
  "(int64, int64$ $, ..., a : Any) -> Any",
  "?(int64, int64$ $, ..., a : Any) -> Any",
  "?((int64, int64$ $, ..., a : Any) -> Any)",
  "$(int64, int64$ $, ..., a : Any) -> Any",
  "=(int64, int64$ $, ..., a : Any) -> Any",
  "$[(int64, int64$ $, ..., a : Any) -> Any]",
  "xxx[(int64, int64$ $, ..., a : Any) -> Any]",
  "?$(int64, int64$ $, ..., a : Any) -> Any]",
  "?=(int64, int64$ $, ..., a : Any) -> Any]",
  "?[(int64, int64$ $, ..., a : Any) -> Any$",
  "?[(int64, int64$ $, ..., a : Any) -> Any$",
  "10 $ (int64, int64$ $, ..., a : Any) -> Any",
  "?10 $ (int64, int64$ $, ..., a : Any) -> Any",
  "?(10 $ (int64, int64$ $, ..., a : Any) -> Any)",
  "$10 $ (int64, int64$ $, ..., a : Any) -> Any",
  "=10 $ (int64, int64$ $, ..., a : Any) -> Any",
  "$[10 $ (int64, int64$ $, ..., a : Any) -> Any]",
  "xxx[10 $ (int64, int64$ $, ..., a : Any) -> Any]",
  "?$10 $ (int64, int64$ $, ..., a : Any) -> Any]",
  "?=10 $ (int64, int64$ $, ..., a : Any) -> Any]",
  "?[10 $ (int64, int64$ $, ..., a : Any) -> Any$",
  "?[10 $ (int64, int64$ $, ..., a : Any) -> Any$",
  "10 = (int64, int64$ $, ..., a : Any) -> Any",
  "?10 = (int64, int64$ $, ..., a : Any) -> Any",
  "?(10 = (int64, int64$ $, ..., a : Any) -> Any)",
  "$10 = (int64, int64$ $, ..., a : Any) -> Any",
  "=10 = (int64, int64$ $, ..., a : Any) -> Any",
  "$[10 = (int64, int64$ $, ..., a : Any) -> Any]",
  "xxx[10 = (int64, int64$ $, ..., a : Any) -> Any]",
  "?$10 = (int64, int64$ $, ..., a : Any) -> Any]",
  "?=10 = (int64, int64$ $, ..., a : Any) -> Any]",
  "?[10 = (int64, int64$ $, ..., a : Any) -> Any$",
  "?[10 = (int64, int64$ $, ..., a : Any) -> Any$",
  "10 * (int64, int64= $, a : Any) -> Any",
  "?10 * (int64, int64= $, a : Any) -> Any",
  "?(10 * (int64, int64= $, a : Any) -> Any)",
  "$10 * (int64, int64= $, a : Any) -> Any",
  "=10 * (int64, int64= $, a : Any) -> Any",
  "$[10 * (int64, int64= $, a : Any) -> Any]",
  "xxx[10 * (int64, int64= $, a : Any) -> Any]",
  "?$10 * (int64, int64= $, a : Any) -> Any]",
  "?=10 * (int64, int64= $, a : Any) -> Any]",
  "?[10 * (int64, int64= $, a : Any) -> Any$",
  "?[10 * (int64, int64= $, a : Any) -> Any$",
  "(int64, int64= $, a : Any) -> Any",
  "?(int64, int64= $, a : Any) -> Any",
  "?((int64, int64= $, a : Any) -> Any)",
  "$(int64, int64= $, a : Any) -> Any",
  "=(int64, int64= $, a : Any) -> Any",
  "$[(int64, int64= $, a : Any) -> Any]",
  "xxx[(int64, int64= $, a : Any) -> Any]",
  "?$(int64, int64= $, a : Any) -> Any]",
  "?=(int64, int64= $, a : Any) -> Any]",
  "?[(int64, int64= $, a : Any) -> Any$",
  "?[(int64, int64= $, a : Any) -> Any$",
  "10 $ (int64, int64= $, a : Any) -> Any",
  "?10 $ (int64, int64= $, a : Any) -> Any",
  "?(10 $ (int64, int64= $, a : Any) -> Any)",
  "$10 $ (int64, int64= $, a : Any) -> Any",
  "=10 $ (int64, int64= $, a : Any) -> Any",
  "$[10 $ (int64, int64= $, a : Any) -> Any]",
  "xxx[10 $ (int64, int64= $, a : Any) -> Any]",
  "?$10 $ (int64, int64= $, a : Any) -> Any]",
  "?=10 $ (int64, int64= $, a : Any) -> Any]",
  "?[10 $ (int64, int64= $, a : Any) -> Any$",
  "?[10 $ (int64, int64= $, a : Any) -> Any$",
  "10 = (int64, int64= $, a : Any) -> Any",
  "?10 = (int64, int64= $, a : Any) -> Any",
  "?(10 = (int64, int64= $, a : Any) -> Any)",
  "$10 = (int64, int64= $, a : Any) -> Any",
  "=10 = (int64, int64= $, a : Any) -> Any",
  "$[10 = (int64, int64= $, a : Any) -> Any]",
  "xxx[10 = (int64, int64= $, a : Any) -> Any]",
  "?$10 = (int64, int64= $, a : Any) -> Any]",
  "?=10 = (int64, int64= $, a : Any) -> Any]",
  "?[10 = (int64, int64= $, a : Any) -> Any$",
  "?[10 = (int64, int64= $, a : Any) -> Any$",
  "10 * (int64, int64= $, ..., a : Any) -> Any",
  "?10 * (int64, int64= $, ..., a : Any) -> Any",
  "?(10 * (int64, int64= $, ..., a : Any) -> Any)",
  "$10 * (int64, int64= $, ..., a : Any) -> Any",
  "=10 * (int64, int64= $, ..., a : Any) -> Any",
  "$[10 * (int64, int64= $, ..., a : Any) -> Any]",
  "xxx[10 * (int64, int64= $, ..., a : Any) -> Any]",
  "?$10 * (int64, int64= $, ..., a : Any) -> Any]",
  "?=10 * (int64, int64= $, ..., a : Any) -> Any]",
  "?[10 * (int64, int64= $, ..., a : Any) -> Any$",
  "?[10 * (int64, int64= $, ..., a : Any) -> Any$",
  "(int64, int64= $, ..., a : Any) -> Any",
  "?(int64, int64= $, ..., a : Any) -> Any",
  "?((int64, int64= $, ..., a : Any) -> Any)",
  "$(int64, int64= $, ..., a : Any) -> Any",
  "=(int64, int64= $, ..., a : Any) -> Any",
  "$[(int64, int64= $, ..., a : Any) -> Any]",
  "xxx[(int64, int64= $, ..., a : Any) -> Any]",
  "?$(int64, int64= $, ..., a : Any) -> Any]",
  "?=(int64, int64= $, ..., a : Any) -> Any]",
  "?[(int64, int64= $, ..., a : Any) -> Any$",
  "?[(int64, int64= $, ..., a : Any) -> Any$",
  "10 $ (int64, int64= $, ..., a : Any) -> Any",
  "?10 $ (int64, int64= $, ..., a : Any) -> Any",
  "?(10 $ (int64, int64= $, ..., a : Any) -> Any)",
  "$10 $ (int64, int64= $, ..., a : Any) -> Any",
  "=10 $ (int64, int64= $, ..., a : Any) -> Any",
  "$[10 $ (int64, int64= $, ..., a : Any) -> Any]",
  "xxx[10 $ (int64, int64= $, ..., a : Any) -> Any]",
  "?$10 $ (int64, int64= $, ..., a : Any) -> Any]",
  "?=10 $ (int64, int64= $, ..., a : Any) -> Any]",
  "?[10 $ (int64, int64= $, ..., a : Any) -> Any$",
  "?[10 $ (int64, int64= $, ..., a : Any) -> Any$",
  "10 = (int64, int64= $, ..., a : Any) -> Any",
  "?10 = (int64, int64= $, ..., a : Any) -> Any",
  "?(10 = (int64, int64= $, ..., a : Any) -> Any)",
  "$10 = (int64, int64= $, ..., a : Any) -> Any",
  "=10 = (int64, int64= $, ..., a : Any) -> Any",
  "$[10 = (int64, int64= $, ..., a : Any) -> Any]",
  "xxx[10 = (int64, int64= $, ..., a : Any) -> Any]",
  "?$10 = (int64, int64= $, ..., a : Any) -> Any]",
  "?=10 = (int64, int64= $, ..., a : Any) -> Any]",
  "?[10 = (int64, int64= $, ..., a : Any) -> Any$",
  "?[10 = (int64, int64= $, ..., a : Any) -> Any$",
  "10 * (=, a : Any) -> Any",
  "?10 * (=, a : Any) -> Any",
  "?(10 * (=, a : Any) -> Any)",
  "$10 * (=, a : Any) -> Any",
  "=10 * (=, a : Any) -> Any",
  "$[10 * (=, a : Any) -> Any]",
  "xxx[10 * (=, a : Any) -> Any]",
  "?$10 * (=, a : Any) -> Any]",
  "?=10 * (=, a : Any) -> Any]",
  "?[10 * (=, a : Any) -> Any$",
  "?[10 * (=, a : Any) -> Any$",
  "(=, a : Any) -> Any",
  "?(=, a : Any) -> Any",
  "?((=, a : Any) -> Any)",
  "$(=, a : Any) -> Any",
  "=(=, a : Any) -> Any",
  "$[(=, a : Any) -> Any]",
  "xxx[(=, a : Any) -> Any]",
  "?$(=, a : Any) -> Any]",
  "?=(=, a : Any) -> Any]",
  "?[(=, a : Any) -> Any$",
  "?[(=, a : Any) -> Any$",
  "10 $ (=, a : Any) -> Any",
  "?10 $ (=, a : Any) -> Any",
  "?(10 $ (=, a : Any) -> Any)",
  "$10 $ (=, a : Any) -> Any",
  "=10 $ (=, a : Any) -> Any",
  "$[10 $ (=, a : Any) -> Any]",
  "xxx[10 $ (=, a : Any) -> Any]",
  "?$10 $ (=, a : Any) -> Any]",
  "?=10 $ (=, a : Any) -> Any]",
  "?[10 $ (=, a : Any) -> Any$",
  "?[10 $ (=, a : Any) -> Any$",
  "10 = (=, a : Any) -> Any",
  "?10 = (=, a : Any) -> Any",
  "?(10 = (=, a : Any) -> Any)",
  "$10 = (=, a : Any) -> Any",
  "=10 = (=, a : Any) -> Any",
  "$[10 = (=, a : Any) -> Any]",
  "xxx[10 = (=, a : Any) -> Any]",
  "?$10 = (=, a : Any) -> Any]",
  "?=10 = (=, a : Any) -> Any]",
  "?[10 = (=, a : Any) -> Any$",
  "?[10 = (=, a : Any) -> Any$",
  "10 * (=, ..., a : Any) -> Any",
  "?10 * (=, ..., a : Any) -> Any",
  "?(10 * (=, ..., a : Any) -> Any)",
  "$10 * (=, ..., a : Any) -> Any",
  "=10 * (=, ..., a : Any) -> Any",
  "$[10 * (=, ..., a : Any) -> Any]",
  "xxx[10 * (=, ..., a : Any) -> Any]",
  "?$10 * (=, ..., a : Any) -> Any]",
  "?=10 * (=, ..., a : Any) -> Any]",
  "?[10 * (=, ..., a : Any) -> Any$",
  "?[10 * (=, ..., a : Any) -> Any$",
  "(=, ..., a : Any) -> Any",
  "?(=, ..., a : Any) -> Any",
  "?((=, ..., a : Any) -> Any)",
  "$(=, ..., a : Any) -> Any",
  "=(=, ..., a : Any) -> Any",
  "$[(=, ..., a : Any) -> Any]",
  "xxx[(=, ..., a : Any) -> Any]",
  "?$(=, ..., a : Any) -> Any]",
  "?=(=, ..., a : Any) -> Any]",
  "?[(=, ..., a : Any) -> Any$",
  "?[(=, ..., a : Any) -> Any$",
  "10 $ (=, ..., a : Any) -> Any",
  "?10 $ (=, ..., a : Any) -> Any",
  "?(10 $ (=, ..., a : Any) -> Any)",
  "$10 $ (=, ..., a : Any) -> Any",
  "=10 $ (=, ..., a : Any) -> Any",
  "$[10 $ (=, ..., a : Any) -> Any]",
  "xxx[10 $ (=, ..., a : Any) -> Any]",
  "?$10 $ (=, ..., a : Any) -> Any]",
  "?=10 $ (=, ..., a : Any) -> Any]",
  "?[10 $ (=, ..., a : Any) -> Any$",
  "?[10 $ (=, ..., a : Any) -> Any$",
  "10 = (=, ..., a : Any) -> Any",
  "?10 = (=, ..., a : Any) -> Any",
  "?(10 = (=, ..., a : Any) -> Any)",
  "$10 = (=, ..., a : Any) -> Any",
  "=10 = (=, ..., a : Any) -> Any",
  "$[10 = (=, ..., a : Any) -> Any]",
  "xxx[10 = (=, ..., a : Any) -> Any]",
  "?$10 = (=, ..., a : Any) -> Any]",
  "?=10 = (=, ..., a : Any) -> Any]",
  "?[10 = (=, ..., a : Any) -> Any$",
  "?[10 = (=, ..., a : Any) -> Any$",
  "10 * (int64, =, a : Any) -> Any",
  "?10 * (int64, =, a : Any) -> Any",
  "?(10 * (int64, =, a : Any) -> Any)",
  "$10 * (int64, =, a : Any) -> Any",
  "=10 * (int64, =, a : Any) -> Any",
  "$[10 * (int64, =, a : Any) -> Any]",
  "xxx[10 * (int64, =, a : Any) -> Any]",
  "?$10 * (int64, =, a : Any) -> Any]",
  "?=10 * (int64, =, a : Any) -> Any]",
  "?[10 * (int64, =, a : Any) -> Any$",
  "?[10 * (int64, =, a : Any) -> Any$",
  "(int64, =, a : Any) -> Any",
  "?(int64, =, a : Any) -> Any",
  "?((int64, =, a : Any) -> Any)",
  "$(int64, =, a : Any) -> Any",
  "=(int64, =, a : Any) -> Any",
  "$[(int64, =, a : Any) -> Any]",
  "xxx[(int64, =, a : Any) -> Any]",
  "?$(int64, =, a : Any) -> Any]",
  "?=(int64, =, a : Any) -> Any]",
  "?[(int64, =, a : Any) -> Any$",
  "?[(int64, =, a : Any) -> Any$",
  "10 $ (int64, =, a : Any) -> Any",
  "?10 $ (int64, =, a : Any) -> Any",
  "?(10 $ (int64, =, a : Any) -> Any)",
  "$10 $ (int64, =, a : Any) -> Any",
  "=10 $ (int64, =, a : Any) -> Any",
  "$[10 $ (int64, =, a : Any) -> Any]",
  "xxx[10 $ (int64, =, a : Any) -> Any]",
  "?$10 $ (int64, =, a : Any) -> Any]",
  "?=10 $ (int64, =, a : Any) -> Any]",
  "?[10 $ (int64, =, a : Any) -> Any$",
  "?[10 $ (int64, =, a : Any) -> Any$",
  "10 = (int64, =, a : Any) -> Any",
  "?10 = (int64, =, a : Any) -> Any",
  "?(10 = (int64, =, a : Any) -> Any)",
  "$10 = (int64, =, a : Any) -> Any",
  "=10 = (int64, =, a : Any) -> Any",
  "$[10 = (int64, =, a : Any) -> Any]",
  "xxx[10 = (int64, =, a : Any) -> Any]",
  "?$10 = (int64, =, a : Any) -> Any]",
  "?=10 = (int64, =, a : Any) -> Any]",
  "?[10 = (int64, =, a : Any) -> Any$",
  "?[10 = (int64, =, a : Any) -> Any$",
  "10 * (int64, =, ..., a : Any) -> Any",
  "?10 * (int64, =, ..., a : Any) -> Any",
  "?(10 * (int64, =, ..., a : Any) -> Any)",
  "$10 * (int64, =, ..., a : Any) -> Any",
  "=10 * (int64, =, ..., a : Any) -> Any",
  "$[10 * (int64, =, ..., a : Any) -> Any]",
  "xxx[10 * (int64, =, ..., a : Any) -> Any]",
  "?$10 * (int64, =, ..., a : Any) -> Any]",
  "?=10 * (int64, =, ..., a : Any) -> Any]",
  "?[10 * (int64, =, ..., a : Any) -> Any$",
  "?[10 * (int64, =, ..., a : Any) -> Any$",
  "(int64, =, ..., a : Any) -> Any",
  "?(int64, =, ..., a : Any) -> Any",
  "?((int64, =, ..., a : Any) -> Any)",
  "$(int64, =, ..., a : Any) -> Any",
  "=(int64, =, ..., a : Any) -> Any",
  "$[(int64, =, ..., a : Any) -> Any]",
  "xxx[(int64, =, ..., a : Any) -> Any]",
  "?$(int64, =, ..., a : Any) -> Any]",
  "?=(int64, =, ..., a : Any) -> Any]",
  "?[(int64, =, ..., a : Any) -> Any$",
  "?[(int64, =, ..., a : Any) -> Any$",
  "10 $ (int64, =, ..., a : Any) -> Any",
  "?10 $ (int64, =, ..., a : Any) -> Any",
  "?(10 $ (int64, =, ..., a : Any) -> Any)",
  "$10 $ (int64, =, ..., a : Any) -> Any",
  "=10 $ (int64, =, ..., a : Any) -> Any",
  "$[10 $ (int64, =, ..., a : Any) -> Any]",
  "xxx[10 $ (int64, =, ..., a : Any) -> Any]",
  "?$10 $ (int64, =, ..., a : Any) -> Any]",
  "?=10 $ (int64, =, ..., a : Any) -> Any]",
  "?[10 $ (int64, =, ..., a : Any) -> Any$",
  "?[10 $ (int64, =, ..., a : Any) -> Any$",
  "10 = (int64, =, ..., a : Any) -> Any",
  "?10 = (int64, =, ..., a : Any) -> Any",
  "?(10 = (int64, =, ..., a : Any) -> Any)",
  "$10 = (int64, =, ..., a : Any) -> Any",
  "=10 = (int64, =, ..., a : Any) -> Any",
  "$[10 = (int64, =, ..., a : Any) -> Any]",
  "xxx[10 = (int64, =, ..., a : Any) -> Any]",
  "?$10 = (int64, =, ..., a : Any) -> Any]",
  "?=10 = (int64, =, ..., a : Any) -> Any]",
  "?[10 = (int64, =, ..., a : Any) -> Any$",
  "?[10 = (int64, =, ..., a : Any) -> Any$",
  "10 * (int64$ =, a : Any) -> Any",
  "?10 * (int64$ =, a : Any) -> Any",
  "?(10 * (int64$ =, a : Any) -> Any)",
  "$10 * (int64$ =, a : Any) -> Any",
  "=10 * (int64$ =, a : Any) -> Any",
  "$[10 * (int64$ =, a : Any) -> Any]",
  "xxx[10 * (int64$ =, a : Any) -> Any]",
  "?$10 * (int64$ =, a : Any) -> Any]",
  "?=10 * (int64$ =, a : Any) -> Any]",
  "?[10 * (int64$ =, a : Any) -> Any$",
  "?[10 * (int64$ =, a : Any) -> Any$",
  "(int64$ =, a : Any) -> Any",
  "?(int64$ =, a : Any) -> Any",
  "?((int64$ =, a : Any) -> Any)",
  "$(int64$ =, a : Any) -> Any",
  "=(int64$ =, a : Any) -> Any",
  "$[(int64$ =, a : Any) -> Any]",
  "xxx[(int64$ =, a : Any) -> Any]",
  "?$(int64$ =, a : Any) -> Any]",
  "?=(int64$ =, a : Any) -> Any]",
  "?[(int64$ =, a : Any) -> Any$",
  "?[(int64$ =, a : Any) -> Any$",
  "10 $ (int64$ =, a : Any) -> Any",
  "?10 $ (int64$ =, a : Any) -> Any",
  "?(10 $ (int64$ =, a : Any) -> Any)",
  "$10 $ (int64$ =, a : Any) -> Any",
  "=10 $ (int64$ =, a : Any) -> Any",
  "$[10 $ (int64$ =, a : Any) -> Any]",
  "xxx[10 $ (int64$ =, a : Any) -> Any]",
  "?$10 $ (int64$ =, a : Any) -> Any]",
  "?=10 $ (int64$ =, a : Any) -> Any]",
  "?[10 $ (int64$ =, a : Any) -> Any$",
  "?[10 $ (int64$ =, a : Any) -> Any$",
  "10 = (int64$ =, a : Any) -> Any",
  "?10 = (int64$ =, a : Any) -> Any",
  "?(10 = (int64$ =, a : Any) -> Any)",
  "$10 = (int64$ =, a : Any) -> Any",
  "=10 = (int64$ =, a : Any) -> Any",
  "$[10 = (int64$ =, a : Any) -> Any]",
  "xxx[10 = (int64$ =, a : Any) -> Any]",
  "?$10 = (int64$ =, a : Any) -> Any]",
  "?=10 = (int64$ =, a : Any) -> Any]",
  "?[10 = (int64$ =, a : Any) -> Any$",
  "?[10 = (int64$ =, a : Any) -> Any$",
  "10 * (int64$ =, ..., a : Any) -> Any",
  "?10 * (int64$ =, ..., a : Any) -> Any",
  "?(10 * (int64$ =, ..., a : Any) -> Any)",
  "$10 * (int64$ =, ..., a : Any) -> Any",
  "=10 * (int64$ =, ..., a : Any) -> Any",
  "$[10 * (int64$ =, ..., a : Any) -> Any]",
  "xxx[10 * (int64$ =, ..., a : Any) -> Any]",
  "?$10 * (int64$ =, ..., a : Any) -> Any]",
  "?=10 * (int64$ =, ..., a : Any) -> Any]",
  "?[10 * (int64$ =, ..., a : Any) -> Any$",
  "?[10 * (int64$ =, ..., a : Any) -> Any$",
  "(int64$ =, ..., a : Any) -> Any",
  "?(int64$ =, ..., a : Any) -> Any",
  "?((int64$ =, ..., a : Any) -> Any)",
  "$(int64$ =, ..., a : Any) -> Any",
  "=(int64$ =, ..., a : Any) -> Any",
  "$[(int64$ =, ..., a : Any) -> Any]",
  "xxx[(int64$ =, ..., a : Any) -> Any]",
  "?$(int64$ =, ..., a : Any) -> Any]",
  "?=(int64$ =, ..., a : Any) -> Any]",
  "?[(int64$ =, ..., a : Any) -> Any$",
  "?[(int64$ =, ..., a : Any) -> Any$",
  "10 $ (int64$ =, ..., a : Any) -> Any",
  "?10 $ (int64$ =, ..., a : Any) -> Any",
  "?(10 $ (int64$ =, ..., a : Any) -> Any)",
  "$10 $ (int64$ =, ..., a : Any) -> Any",
  "=10 $ (int64$ =, ..., a : Any) -> Any",
  "$[10 $ (int64$ =, ..., a : Any) -> Any]",
  "xxx[10 $ (int64$ =, ..., a : Any) -> Any]",
  "?$10 $ (int64$ =, ..., a : Any) -> Any]",
  "?=10 $ (int64$ =, ..., a : Any) -> Any]",
  "?[10 $ (int64$ =, ..., a : Any) -> Any$",
  "?[10 $ (int64$ =, ..., a : Any) -> Any$",
  "10 = (int64$ =, ..., a : Any) -> Any",
  "?10 = (int64$ =, ..., a : Any) -> Any",
  "?(10 = (int64$ =, ..., a : Any) -> Any)",
  "$10 = (int64$ =, ..., a : Any) -> Any",
  "=10 = (int64$ =, ..., a : Any) -> Any",
  "$[10 = (int64$ =, ..., a : Any) -> Any]",
  "xxx[10 = (int64$ =, ..., a : Any) -> Any]",
  "?$10 = (int64$ =, ..., a : Any) -> Any]",
  "?=10 = (int64$ =, ..., a : Any) -> Any]",
  "?[10 = (int64$ =, ..., a : Any) -> Any$",
  "?[10 = (int64$ =, ..., a : Any) -> Any$",
  "10 * (int64= =, a : Any) -> Any",
  "?10 * (int64= =, a : Any) -> Any",
  "?(10 * (int64= =, a : Any) -> Any)",
  "$10 * (int64= =, a : Any) -> Any",
  "=10 * (int64= =, a : Any) -> Any",
  "$[10 * (int64= =, a : Any) -> Any]",
  "xxx[10 * (int64= =, a : Any) -> Any]",
  "?$10 * (int64= =, a : Any) -> Any]",
  "?=10 * (int64= =, a : Any) -> Any]",
  "?[10 * (int64= =, a : Any) -> Any$",
  "?[10 * (int64= =, a : Any) -> Any$",
  "(int64= =, a : Any) -> Any",
  "?(int64= =, a : Any) -> Any",
  "?((int64= =, a : Any) -> Any)",
  "$(int64= =, a : Any) -> Any",
  "=(int64= =, a : Any) -> Any",
  "$[(int64= =, a : Any) -> Any]",
  "xxx[(int64= =, a : Any) -> Any]",
  "?$(int64= =, a : Any) -> Any]",
  "?=(int64= =, a : Any) -> Any]",
  "?[(int64= =, a : Any) -> Any$",
  "?[(int64= =, a : Any) -> Any$",
  "10 $ (int64= =, a : Any) -> Any",
  "?10 $ (int64= =, a : Any) -> Any",
  "?(10 $ (int64= =, a : Any) -> Any)",
  "$10 $ (int64= =, a : Any) -> Any",
  "=10 $ (int64= =, a : Any) -> Any",
  "$[10 $ (int64= =, a : Any) -> Any]",
  "xxx[10 $ (int64= =, a : Any) -> Any]",
  "?$10 $ (int64= =, a : Any) -> Any]",
  "?=10 $ (int64= =, a : Any) -> Any]",
  "?[10 $ (int64= =, a : Any) -> Any$",
  "?[10 $ (int64= =, a : Any) -> Any$",
  "10 = (int64= =, a : Any) -> Any",
  "?10 = (int64= =, a : Any) -> Any",
  "?(10 = (int64= =, a : Any) -> Any)",
  "$10 = (int64= =, a : Any) -> Any",
  "=10 = (int64= =, a : Any) -> Any",
  "$[10 = (int64= =, a : Any) -> Any]",
  "xxx[10 = (int64= =, a : Any) -> Any]",
  "?$10 = (int64= =, a : Any) -> Any]",
  "?=10 = (int64= =, a : Any) -> Any]",
  "?[10 = (int64= =, a : Any) -> Any$",
  "?[10 = (int64= =, a : Any) -> Any$",
  "10 * (int64= =, ..., a : Any) -> Any",
  "?10 * (int64= =, ..., a : Any) -> Any",
  "?(10 * (int64= =, ..., a : Any) -> Any)",
  "$10 * (int64= =, ..., a : Any) -> Any",
  "=10 * (int64= =, ..., a : Any) -> Any",
  "$[10 * (int64= =, ..., a : Any) -> Any]",
  "xxx[10 * (int64= =, ..., a : Any) -> Any]",
  "?$10 * (int64= =, ..., a : Any) -> Any]",
  "?=10 * (int64= =, ..., a : Any) -> Any]",
  "?[10 * (int64= =, ..., a : Any) -> Any$",
  "?[10 * (int64= =, ..., a : Any) -> Any$",
  "(int64= =, ..., a : Any) -> Any",
  "?(int64= =, ..., a : Any) -> Any",
  "?((int64= =, ..., a : Any) -> Any)",
  "$(int64= =, ..., a : Any) -> Any",
  "=(int64= =, ..., a : Any) -> Any",
  "$[(int64= =, ..., a : Any) -> Any]",
  "xxx[(int64= =, ..., a : Any) -> Any]",
  "?$(int64= =, ..., a : Any) -> Any]",
  "?=(int64= =, ..., a : Any) -> Any]",
  "?[(int64= =, ..., a : Any) -> Any$",
  "?[(int64= =, ..., a : Any) -> Any$",
  "10 $ (int64= =, ..., a : Any) -> Any",
  "?10 $ (int64= =, ..., a : Any) -> Any",
  "?(10 $ (int64= =, ..., a : Any) -> Any)",
  "$10 $ (int64= =, ..., a : Any) -> Any",
  "=10 $ (int64= =, ..., a : Any) -> Any",
  "$[10 $ (int64= =, ..., a : Any) -> Any]",
  "xxx[10 $ (int64= =, ..., a : Any) -> Any]",
  "?$10 $ (int64= =, ..., a : Any) -> Any]",
  "?=10 $ (int64= =, ..., a : Any) -> Any]",
  "?[10 $ (int64= =, ..., a : Any) -> Any$",
  "?[10 $ (int64= =, ..., a : Any) -> Any$",
  "10 = (int64= =, ..., a : Any) -> Any",
  "?10 = (int64= =, ..., a : Any) -> Any",
  "?(10 = (int64= =, ..., a : Any) -> Any)",
  "$10 = (int64= =, ..., a : Any) -> Any",
  "=10 = (int64= =, ..., a : Any) -> Any",
  "$[10 = (int64= =, ..., a : Any) -> Any]",
  "xxx[10 = (int64= =, ..., a : Any) -> Any]",
  "?$10 = (int64= =, ..., a : Any) -> Any]",
  "?=10 = (int64= =, ..., a : Any) -> Any]",
  "?[10 = (int64= =, ..., a : Any) -> Any$",
  "?[10 = (int64= =, ..., a : Any) -> Any$",
  "10 * (int64, int64, =, a : Any) -> Any",
  "?10 * (int64, int64, =, a : Any) -> Any",
  "?(10 * (int64, int64, =, a : Any) -> Any)",
  "$10 * (int64, int64, =, a : Any) -> Any",
  "=10 * (int64, int64, =, a : Any) -> Any",
  "$[10 * (int64, int64, =, a : Any) -> Any]",
  "xxx[10 * (int64, int64, =, a : Any) -> Any]",
  "?$10 * (int64, int64, =, a : Any) -> Any]",
  "?=10 * (int64, int64, =, a : Any) -> Any]",
  "?[10 * (int64, int64, =, a : Any) -> Any$",
  "?[10 * (int64, int64, =, a : Any) -> Any$",
  "(int64, int64, =, a : Any) -> Any",
  "?(int64, int64, =, a : Any) -> Any",
  "?((int64, int64, =, a : Any) -> Any)",
  "$(int64, int64, =, a : Any) -> Any",
  "=(int64, int64, =, a : Any) -> Any",
  "$[(int64, int64, =, a : Any) -> Any]",
  "xxx[(int64, int64, =, a : Any) -> Any]",
  "?$(int64, int64, =, a : Any) -> Any]",
  "?=(int64, int64, =, a : Any) -> Any]",
  "?[(int64, int64, =, a : Any) -> Any$",
  "?[(int64, int64, =, a : Any) -> Any$",
  "10 $ (int64, int64, =, a : Any) -> Any",
  "?10 $ (int64, int64, =, a : Any) -> Any",
  "?(10 $ (int64, int64, =, a : Any) -> Any)",
  "$10 $ (int64, int64, =, a : Any) -> Any",
  "=10 $ (int64, int64, =, a : Any) -> Any",
  "$[10 $ (int64, int64, =, a : Any) -> Any]",
  "xxx[10 $ (int64, int64, =, a : Any) -> Any]",
  "?$10 $ (int64, int64, =, a : Any) -> Any]",
  "?=10 $ (int64, int64, =, a : Any) -> Any]",
  "?[10 $ (int64, int64, =, a : Any) -> Any$",
  "?[10 $ (int64, int64, =, a : Any) -> Any$",
  "10 = (int64, int64, =, a : Any) -> Any",
  "?10 = (int64, int64, =, a : Any) -> Any",
  "?(10 = (int64, int64, =, a : Any) -> Any)",
  "$10 = (int64, int64, =, a : Any) -> Any",
  "=10 = (int64, int64, =, a : Any) -> Any",
  "$[10 = (int64, int64, =, a : Any) -> Any]",
  "xxx[10 = (int64, int64, =, a : Any) -> Any]",
  "?$10 = (int64, int64, =, a : Any) -> Any]",
  "?=10 = (int64, int64, =, a : Any) -> Any]",
  "?[10 = (int64, int64, =, a : Any) -> Any$",
  "?[10 = (int64, int64, =, a : Any) -> Any$",
  "10 * (int64, int64, =, ..., a : Any) -> Any",
  "?10 * (int64, int64, =, ..., a : Any) -> Any",
  "?(10 * (int64, int64, =, ..., a : Any) -> Any)",
  "$10 * (int64, int64, =, ..., a : Any) -> Any",
  "=10 * (int64, int64, =, ..., a : Any) -> Any",
  "$[10 * (int64, int64, =, ..., a : Any) -> Any]",
  "xxx[10 * (int64, int64, =, ..., a : Any) -> Any]",
  "?$10 * (int64, int64, =, ..., a : Any) -> Any]",
  "?=10 * (int64, int64, =, ..., a : Any) -> Any]",
  "?[10 * (int64, int64, =, ..., a : Any) -> Any$",
  "?[10 * (int64, int64, =, ..., a : Any) -> Any$",
  "(int64, int64, =, ..., a : Any) -> Any",
  "?(int64, int64, =, ..., a : Any) -> Any",
  "?((int64, int64, =, ..., a : Any) -> Any)",
  "$(int64, int64, =, ..., a : Any) -> Any",
  "=(int64, int64, =, ..., a : Any) -> Any",
  "$[(int64, int64, =, ..., a : Any) -> Any]",
  "xxx[(int64, int64, =, ..., a : Any) -> Any]",
  "?$(int64, int64, =, ..., a : Any) -> Any]",
  "?=(int64, int64, =, ..., a : Any) -> Any]",
  "?[(int64, int64, =, ..., a : Any) -> Any$",
  "?[(int64, int64, =, ..., a : Any) -> Any$",
  "10 $ (int64, int64, =, ..., a : Any) -> Any",
  "?10 $ (int64, int64, =, ..., a : Any) -> Any",
  "?(10 $ (int64, int64, =, ..., a : Any) -> Any)",
  "$10 $ (int64, int64, =, ..., a : Any) -> Any",
  "=10 $ (int64, int64, =, ..., a : Any) -> Any",
  "$[10 $ (int64, int64, =, ..., a : Any) -> Any]",
  "xxx[10 $ (int64, int64, =, ..., a : Any) -> Any]",
  "?$10 $ (int64, int64, =, ..., a : Any) -> Any]",
  "?=10 $ (int64, int64, =, ..., a : Any) -> Any]",
  "?[10 $ (int64, int64, =, ..., a : Any) -> Any$",
  "?[10 $ (int64, int64, =, ..., a : Any) -> Any$",
  "10 = (int64, int64, =, ..., a : Any) -> Any",
  "?10 = (int64, int64, =, ..., a : Any) -> Any",
  "?(10 = (int64, int64, =, ..., a : Any) -> Any)",
  "$10 = (int64, int64, =, ..., a : Any) -> Any",
  "=10 = (int64, int64, =, ..., a : Any) -> Any",
  "$[10 = (int64, int64, =, ..., a : Any) -> Any]",
  "xxx[10 = (int64, int64, =, ..., a : Any) -> Any]",
  "?$10 = (int64, int64, =, ..., a : Any) -> Any]",
  "?=10 = (int64, int64, =, ..., a : Any) -> Any]",
  "?[10 = (int64, int64, =, ..., a : Any) -> Any$",
  "?[10 = (int64, int64, =, ..., a : Any) -> Any$",
  "10 * (int64, int64$ =, a : Any) -> Any",
  "?10 * (int64, int64$ =, a : Any) -> Any",
  "?(10 * (int64, int64$ =, a : Any) -> Any)",
  "$10 * (int64, int64$ =, a : Any) -> Any",
  "=10 * (int64, int64$ =, a : Any) -> Any",
  "$[10 * (int64, int64$ =, a : Any) -> Any]",
  "xxx[10 * (int64, int64$ =, a : Any) -> Any]",
  "?$10 * (int64, int64$ =, a : Any) -> Any]",
  "?=10 * (int64, int64$ =, a : Any) -> Any]",
  "?[10 * (int64, int64$ =, a : Any) -> Any$",
  "?[10 * (int64, int64$ =, a : Any) -> Any$",
  "(int64, int64$ =, a : Any) -> Any",
  "?(int64, int64$ =, a : Any) -> Any",
  "?((int64, int64$ =, a : Any) -> Any)",
  "$(int64, int64$ =, a : Any) -> Any",
  "=(int64, int64$ =, a : Any) -> Any",
  "$[(int64, int64$ =, a : Any) -> Any]",
  "xxx[(int64, int64$ =, a : Any) -> Any]",
  "?$(int64, int64$ =, a : Any) -> Any]",
  "?=(int64, int64$ =, a : Any) -> Any]",
  "?[(int64, int64$ =, a : Any) -> Any$",
  "?[(int64, int64$ =, a : Any) -> Any$",
  "10 $ (int64, int64$ =, a : Any) -> Any",
  "?10 $ (int64, int64$ =, a : Any) -> Any",
  "?(10 $ (int64, int64$ =, a : Any) -> Any)",
  "$10 $ (int64, int64$ =, a : Any) -> Any",
  "=10 $ (int64, int64$ =, a : Any) -> Any",
  "$[10 $ (int64, int64$ =, a : Any) -> Any]",
  "xxx[10 $ (int64, int64$ =, a : Any) -> Any]",
  "?$10 $ (int64, int64$ =, a : Any) -> Any]",
  "?=10 $ (int64, int64$ =, a : Any) -> Any]",
  "?[10 $ (int64, int64$ =, a : Any) -> Any$",
  "?[10 $ (int64, int64$ =, a : Any) -> Any$",
  "10 = (int64, int64$ =, a : Any) -> Any",
  "?10 = (int64, int64$ =, a : Any) -> Any",
  "?(10 = (int64, int64$ =, a : Any) -> Any)",
  "$10 = (int64, int64$ =, a : Any) -> Any",
  "=10 = (int64, int64$ =, a : Any) -> Any",
  "$[10 = (int64, int64$ =, a : Any) -> Any]",
  "xxx[10 = (int64, int64$ =, a : Any) -> Any]",
  "?$10 = (int64, int64$ =, a : Any) -> Any]",
  "?=10 = (int64, int64$ =, a : Any) -> Any]",
  "?[10 = (int64, int64$ =, a : Any) -> Any$",
  "?[10 = (int64, int64$ =, a : Any) -> Any$",
  "10 * (int64, int64$ =, ..., a : Any) -> Any",
  "?10 * (int64, int64$ =, ..., a : Any) -> Any",
  "?(10 * (int64, int64$ =, ..., a : Any) -> Any)",
  "$10 * (int64, int64$ =, ..., a : Any) -> Any",
  "=10 * (int64, int64$ =, ..., a : Any) -> Any",
  "$[10 * (int64, int64$ =, ..., a : Any) -> Any]",
  "xxx[10 * (int64, int64$ =, ..., a : Any) -> Any]",
  "?$10 * (int64, int64$ =, ..., a : Any) -> Any]",
  "?=10 * (int64, int64$ =, ..., a : Any) -> Any]",
  "?[10 * (int64, int64$ =, ..., a : Any) -> Any$",
  "?[10 * (int64, int64$ =, ..., a : Any) -> Any$",
  "(int64, int64$ =, ..., a : Any) -> Any",
  "?(int64, int64$ =, ..., a : Any) -> Any",
  "?((int64, int64$ =, ..., a : Any) -> Any)",
  "$(int64, int64$ =, ..., a : Any) -> Any",
  "=(int64, int64$ =, ..., a : Any) -> Any",
  "$[(int64, int64$ =, ..., a : Any) -> Any]",
  "xxx[(int64, int64$ =, ..., a : Any) -> Any]",
  "?$(int64, int64$ =, ..., a : Any) -> Any]",
  "?=(int64, int64$ =, ..., a : Any) -> Any]",
  "?[(int64, int64$ =, ..., a : Any) -> Any$",
  "?[(int64, int64$ =, ..., a : Any) -> Any$",
  "10 $ (int64, int64$ =, ..., a : Any) -> Any",
  "?10 $ (int64, int64$ =, ..., a : Any) -> Any",
  "?(10 $ (int64, int64$ =, ..., a : Any) -> Any)",
  "$10 $ (int64, int64$ =, ..., a : Any) -> Any",
  "=10 $ (int64, int64$ =, ..., a : Any) -> Any",
  "$[10 $ (int64, int64$ =, ..., a : Any) -> Any]",
  "xxx[10 $ (int64, int64$ =, ..., a : Any) -> Any]",
  "?$10 $ (int64, int64$ =, ..., a : Any) -> Any]",
  "?=10 $ (int64, int64$ =, ..., a : Any) -> Any]",
  "?[10 $ (int64, int64$ =, ..., a : Any) -> Any$",
  "?[10 $ (int64, int64$ =, ..., a : Any) -> Any$",
  "10 = (int64, int64$ =, ..., a : Any) -> Any",
  "?10 = (int64, int64$ =, ..., a : Any) -> Any",
  "?(10 = (int64, int64$ =, ..., a : Any) -> Any)",
  "$10 = (int64, int64$ =, ..., a : Any) -> Any",
  "=10 = (int64, int64$ =, ..., a : Any) -> Any",
  "$[10 = (int64, int64$ =, ..., a : Any) -> Any]",
  "xxx[10 = (int64, int64$ =, ..., a : Any) -> Any]",
  "?$10 = (int64, int64$ =, ..., a : Any) -> Any]",
  "?=10 = (int64, int64$ =, ..., a : Any) -> Any]",
  "?[10 = (int64, int64$ =, ..., a : Any) -> Any$",
  "?[10 = (int64, int64$ =, ..., a : Any) -> Any$",
  "10 * (int64, int64= =, a : Any) -> Any",
  "?10 * (int64, int64= =, a : Any) -> Any",
  "?(10 * (int64, int64= =, a : Any) -> Any)",
  "$10 * (int64, int64= =, a : Any) -> Any",
  "=10 * (int64, int64= =, a : Any) -> Any",
  "$[10 * (int64, int64= =, a : Any) -> Any]",
  "xxx[10 * (int64, int64= =, a : Any) -> Any]",
  "?$10 * (int64, int64= =, a : Any) -> Any]",
  "?=10 * (int64, int64= =, a : Any) -> Any]",
  "?[10 * (int64, int64= =, a : Any) -> Any$",
  "?[10 * (int64, int64= =, a : Any) -> Any$",
  "(int64, int64= =, a : Any) -> Any",
  "?(int64, int64= =, a : Any) -> Any",
  "?((int64, int64= =, a : Any) -> Any)",
  "$(int64, int64= =, a : Any) -> Any",
  "=(int64, int64= =, a : Any) -> Any",
  "$[(int64, int64= =, a : Any) -> Any]",
  "xxx[(int64, int64= =, a : Any) -> Any]",
  "?$(int64, int64= =, a : Any) -> Any]",
  "?=(int64, int64= =, a : Any) -> Any]",
  "?[(int64, int64= =, a : Any) -> Any$",
  "?[(int64, int64= =, a : Any) -> Any$",
  "10 $ (int64, int64= =, a : Any) -> Any",
  "?10 $ (int64, int64= =, a : Any) -> Any",
  "?(10 $ (int64, int64= =, a : Any) -> Any)",
  "$10 $ (int64, int64= =, a : Any) -> Any",
  "=10 $ (int64, int64= =, a : Any) -> Any",
  "$[10 $ (int64, int64= =, a : Any) -> Any]",
  "xxx[10 $ (int64, int64= =, a : Any) -> Any]",
  "?$10 $ (int64, int64= =, a : Any) -> Any]",
  "?=10 $ (int64, int64= =, a : Any) -> Any]",
  "?[10 $ (int64, int64= =, a : Any) -> Any$",
  "?[10 $ (int64, int64= =, a : Any) -> Any$",
  "10 = (int64, int64= =, a : Any) -> Any",
  "?10 = (int64, int64= =, a : Any) -> Any",
  "?(10 = (int64, int64= =, a : Any) -> Any)",
  "$10 = (int64, int64= =, a : Any) -> Any",
  "=10 = (int64, int64= =, a : Any) -> Any",
  "$[10 = (int64, int64= =, a : Any) -> Any]",
  "xxx[10 = (int64, int64= =, a : Any) -> Any]",
  "?$10 = (int64, int64= =, a : Any) -> Any]",
  "?=10 = (int64, int64= =, a : Any) -> Any]",
  "?[10 = (int64, int64= =, a : Any) -> Any$",
  "?[10 = (int64, int64= =, a : Any) -> Any$",
  "10 * (int64, int64= =, ..., a : Any) -> Any",
  "?10 * (int64, int64= =, ..., a : Any) -> Any",
  "?(10 * (int64, int64= =, ..., a : Any) -> Any)",
  "$10 * (int64, int64= =, ..., a : Any) -> Any",
  "=10 * (int64, int64= =, ..., a : Any) -> Any",
  "$[10 * (int64, int64= =, ..., a : Any) -> Any]",
  "xxx[10 * (int64, int64= =, ..., a : Any) -> Any]",
  "?$10 * (int64, int64= =, ..., a : Any) -> Any]",
  "?=10 * (int64, int64= =, ..., a : Any) -> Any]",
  "?[10 * (int64, int64= =, ..., a : Any) -> Any$",
  "?[10 * (int64, int64= =, ..., a : Any) -> Any$",
  "(int64, int64= =, ..., a : Any) -> Any",
  "?(int64, int64= =, ..., a : Any) -> Any",
  "?((int64, int64= =, ..., a : Any) -> Any)",
  "$(int64, int64= =, ..., a : Any) -> Any",
  "=(int64, int64= =, ..., a : Any) -> Any",
  "$[(int64, int64= =, ..., a : Any) -> Any]",
  "xxx[(int64, int64= =, ..., a : Any) -> Any]",
  "?$(int64, int64= =, ..., a : Any) -> Any]",
  "?=(int64, int64= =, ..., a : Any) -> Any]",
  "?[(int64, int64= =, ..., a : Any) -> Any$",
  "?[(int64, int64= =, ..., a : Any) -> Any$",
  "10 $ (int64, int64= =, ..., a : Any) -> Any",
  "?10 $ (int64, int64= =, ..., a : Any) -> Any",
  "?(10 $ (int64, int64= =, ..., a : Any) -> Any)",
  "$10 $ (int64, int64= =, ..., a : Any) -> Any",
  "=10 $ (int64, int64= =, ..., a : Any) -> Any",
  "$[10 $ (int64, int64= =, ..., a : Any) -> Any]",
  "xxx[10 $ (int64, int64= =, ..., a : Any) -> Any]",
  "?$10 $ (int64, int64= =, ..., a : Any) -> Any]",
  "?=10 $ (int64, int64= =, ..., a : Any) -> Any]",
  "?[10 $ (int64, int64= =, ..., a : Any) -> Any$",
  "?[10 $ (int64, int64= =, ..., a : Any) -> Any$",
  "10 = (int64, int64= =, ..., a : Any) -> Any",
  "?10 = (int64, int64= =, ..., a : Any) -> Any",
  "?(10 = (int64, int64= =, ..., a : Any) -> Any)",
  "$10 = (int64, int64= =, ..., a : Any) -> Any",
  "=10 = (int64, int64= =, ..., a : Any) -> Any",
  "$[10 = (int64, int64= =, ..., a : Any) -> Any]",
  "xxx[10 = (int64, int64= =, ..., a : Any) -> Any]",
  "?$10 = (int64, int64= =, ..., a : Any) -> Any]",
  "?=10 = (int64, int64= =, ..., a : Any) -> Any]",
  "?[10 = (int64, int64= =, ..., a : Any) -> Any$",
  "?[10 = (int64, int64= =, ..., a : Any) -> Any$",
  "$10 * (Any, a: Any) -> Any",
  "=10 * (Any, a: Any) -> Any",
  "$[10 * (Any, a: Any) -> Any]",
  "xxx[10 * (Any, a: Any) -> Any]",
  "?$10 * (Any, a: Any) -> Any]",
  "?=10 * (Any, a: Any) -> Any]",
  "?[10 * (Any, a: Any) -> Any$",
  "?[10 * (Any, a: Any) -> Any$",
  "$(Any, a: Any) -> Any",
  "=(Any, a: Any) -> Any",
  "$[(Any, a: Any) -> Any]",
  "xxx[(Any, a: Any) -> Any]",
  "?$(Any, a: Any) -> Any]",
  "?=(Any, a: Any) -> Any]",
  "?[(Any, a: Any) -> Any$",
  "?[(Any, a: Any) -> Any$",
  "10 $ (Any, a: Any) -> Any",
  "?10 $ (Any, a: Any) -> Any",
  "?(10 $ (Any, a: Any) -> Any)",
  "$10 $ (Any, a: Any) -> Any",
  "=10 $ (Any, a: Any) -> Any",
  "$[10 $ (Any, a: Any) -> Any]",
  "xxx[10 $ (Any, a: Any) -> Any]",
  "?$10 $ (Any, a: Any) -> Any]",
  "?=10 $ (Any, a: Any) -> Any]",
  "?[10 $ (Any, a: Any) -> Any$",
  "?[10 $ (Any, a: Any) -> Any$",
  "10 = (Any, a: Any) -> Any",
  "?10 = (Any, a: Any) -> Any",
  "?(10 = (Any, a: Any) -> Any)",
  "$10 = (Any, a: Any) -> Any",
  "=10 = (Any, a: Any) -> Any",
  "$[10 = (Any, a: Any) -> Any]",
  "xxx[10 = (Any, a: Any) -> Any]",
  "?$10 = (Any, a: Any) -> Any]",
  "?=10 = (Any, a: Any) -> Any]",
  "?[10 = (Any, a: Any) -> Any$",
  "?[10 = (Any, a: Any) -> Any$",
  "$10 * (Any, x: int64, a: Any) -> Any",
  "=10 * (Any, x: int64, a: Any) -> Any",
  "$[10 * (Any, x: int64, a: Any) -> Any]",
  "xxx[10 * (Any, x: int64, a: Any) -> Any]",
  "?$10 * (Any, x: int64, a: Any) -> Any]",
  "?=10 * (Any, x: int64, a: Any) -> Any]",
  "?[10 * (Any, x: int64, a: Any) -> Any$",
  "?[10 * (Any, x: int64, a: Any) -> Any$",
  "$(Any, x: int64, a: Any) -> Any",
  "=(Any, x: int64, a: Any) -> Any",
  "$[(Any, x: int64, a: Any) -> Any]",
  "xxx[(Any, x: int64, a: Any) -> Any]",
  "?$(Any, x: int64, a: Any) -> Any]",
  "?=(Any, x: int64, a: Any) -> Any]",
  "?[(Any, x: int64, a: Any) -> Any$",
  "?[(Any, x: int64, a: Any) -> Any$",
  "10 $ (Any, x: int64, a: Any) -> Any",
  "?10 $ (Any, x: int64, a: Any) -> Any",
  "?(10 $ (Any, x: int64, a: Any) -> Any)",
  "$10 $ (Any, x: int64, a: Any) -> Any",
  "=10 $ (Any, x: int64, a: Any) -> Any",
  "$[10 $ (Any, x: int64, a: Any) -> Any]",
  "xxx[10 $ (Any, x: int64, a: Any) -> Any]",
  "?$10 $ (Any, x: int64, a: Any) -> Any]",
  "?=10 $ (Any, x: int64, a: Any) -> Any]",
  "?[10 $ (Any, x: int64, a: Any) -> Any$",
  "?[10 $ (Any, x: int64, a: Any) -> Any$",
  "10 = (Any, x: int64, a: Any) -> Any",
  "?10 = (Any, x: int64, a: Any) -> Any",
  "?(10 = (Any, x: int64, a: Any) -> Any)",
  "$10 = (Any, x: int64, a: Any) -> Any",
  "=10 = (Any, x: int64, a: Any) -> Any",
  "$[10 = (Any, x: int64, a: Any) -> Any]",
  "xxx[10 = (Any, x: int64, a: Any) -> Any]",
  "?$10 = (Any, x: int64, a: Any) -> Any]",
  "?=10 = (Any, x: int64, a: Any) -> Any]",
  "?[10 = (Any, x: int64, a: Any) -> Any$",
  "?[10 = (Any, x: int64, a: Any) -> Any$",
  "10 * (Any, x: int64$ a: Any) -> Any",
  "?10 * (Any, x: int64$ a: Any) -> Any",
  "?(10 * (Any, x: int64$ a: Any) -> Any)",
  "$10 * (Any, x: int64$ a: Any) -> Any",
  "=10 * (Any, x: int64$ a: Any) -> Any",
  "$[10 * (Any, x: int64$ a: Any) -> Any]",
  "xxx[10 * (Any, x: int64$ a: Any) -> Any]",
  "?$10 * (Any, x: int64$ a: Any) -> Any]",
  "?=10 * (Any, x: int64$ a: Any) -> Any]",
  "?[10 * (Any, x: int64$ a: Any) -> Any$",
  "?[10 * (Any, x: int64$ a: Any) -> Any$",
  "(Any, x: int64$ a: Any) -> Any",
  "?(Any, x: int64$ a: Any) -> Any",
  "?((Any, x: int64$ a: Any) -> Any)",
  "$(Any, x: int64$ a: Any) -> Any",
  "=(Any, x: int64$ a: Any) -> Any",
  "$[(Any, x: int64$ a: Any) -> Any]",
  "xxx[(Any, x: int64$ a: Any) -> Any]",
  "?$(Any, x: int64$ a: Any) -> Any]",
  "?=(Any, x: int64$ a: Any) -> Any]",
  "?[(Any, x: int64$ a: Any) -> Any$",
  "?[(Any, x: int64$ a: Any) -> Any$",
  "10 $ (Any, x: int64$ a: Any) -> Any",
  "?10 $ (Any, x: int64$ a: Any) -> Any",
  "?(10 $ (Any, x: int64$ a: Any) -> Any)",
  "$10 $ (Any, x: int64$ a: Any) -> Any",
  "=10 $ (Any, x: int64$ a: Any) -> Any",
  "$[10 $ (Any, x: int64$ a: Any) -> Any]",
  "xxx[10 $ (Any, x: int64$ a: Any) -> Any]",
  "?$10 $ (Any, x: int64$ a: Any) -> Any]",
  "?=10 $ (Any, x: int64$ a: Any) -> Any]",
  "?[10 $ (Any, x: int64$ a: Any) -> Any$",
  "?[10 $ (Any, x: int64$ a: Any) -> Any$",
  "10 = (Any, x: int64$ a: Any) -> Any",
  "?10 = (Any, x: int64$ a: Any) -> Any",
  "?(10 = (Any, x: int64$ a: Any) -> Any)",
  "$10 = (Any, x: int64$ a: Any) -> Any",
  "=10 = (Any, x: int64$ a: Any) -> Any",
  "$[10 = (Any, x: int64$ a: Any) -> Any]",
  "xxx[10 = (Any, x: int64$ a: Any) -> Any]",
  "?$10 = (Any, x: int64$ a: Any) -> Any]",
  "?=10 = (Any, x: int64$ a: Any) -> Any]",
  "?[10 = (Any, x: int64$ a: Any) -> Any$",
  "?[10 = (Any, x: int64$ a: Any) -> Any$",
  "10 * (Any, x: int64= a: Any) -> Any",
  "?10 * (Any, x: int64= a: Any) -> Any",
  "?(10 * (Any, x: int64= a: Any) -> Any)",
  "$10 * (Any, x: int64= a: Any) -> Any",
  "=10 * (Any, x: int64= a: Any) -> Any",
  "$[10 * (Any, x: int64= a: Any) -> Any]",
  "xxx[10 * (Any, x: int64= a: Any) -> Any]",
  "?$10 * (Any, x: int64= a: Any) -> Any]",
  "?=10 * (Any, x: int64= a: Any) -> Any]",
  "?[10 * (Any, x: int64= a: Any) -> Any$",
  "?[10 * (Any, x: int64= a: Any) -> Any$",
  "(Any, x: int64= a: Any) -> Any",
  "?(Any, x: int64= a: Any) -> Any",
  "?((Any, x: int64= a: Any) -> Any)",
  "$(Any, x: int64= a: Any) -> Any",
  "=(Any, x: int64= a: Any) -> Any",
  "$[(Any, x: int64= a: Any) -> Any]",
  "xxx[(Any, x: int64= a: Any) -> Any]",
  "?$(Any, x: int64= a: Any) -> Any]",
  "?=(Any, x: int64= a: Any) -> Any]",
  "?[(Any, x: int64= a: Any) -> Any$",
  "?[(Any, x: int64= a: Any) -> Any$",
  "10 $ (Any, x: int64= a: Any) -> Any",
  "?10 $ (Any, x: int64= a: Any) -> Any",
  "?(10 $ (Any, x: int64= a: Any) -> Any)",
  "$10 $ (Any, x: int64= a: Any) -> Any",
  "=10 $ (Any, x: int64= a: Any) -> Any",
  "$[10 $ (Any, x: int64= a: Any) -> Any]",
  "xxx[10 $ (Any, x: int64= a: Any) -> Any]",
  "?$10 $ (Any, x: int64= a: Any) -> Any]",
  "?=10 $ (Any, x: int64= a: Any) -> Any]",
  "?[10 $ (Any, x: int64= a: Any) -> Any$",
  "?[10 $ (Any, x: int64= a: Any) -> Any$",
  "10 = (Any, x: int64= a: Any) -> Any",
  "?10 = (Any, x: int64= a: Any) -> Any",
  "?(10 = (Any, x: int64= a: Any) -> Any)",
  "$10 = (Any, x: int64= a: Any) -> Any",
  "=10 = (Any, x: int64= a: Any) -> Any",
  "$[10 = (Any, x: int64= a: Any) -> Any]",
  "xxx[10 = (Any, x: int64= a: Any) -> Any]",
  "?$10 = (Any, x: int64= a: Any) -> Any]",
  "?=10 = (Any, x: int64= a: Any) -> Any]",
  "?[10 = (Any, x: int64= a: Any) -> Any$",
  "?[10 = (Any, x: int64= a: Any) -> Any$",
  "$10 * (Any, x: int64, x: int64, a: Any) -> Any",
  "=10 * (Any, x: int64, x: int64, a: Any) -> Any",
  "$[10 * (Any, x: int64, x: int64, a: Any) -> Any]",
  "xxx[10 * (Any, x: int64, x: int64, a: Any) -> Any]",
  "?$10 * (Any, x: int64, x: int64, a: Any) -> Any]",
  "?=10 * (Any, x: int64, x: int64, a: Any) -> Any]",
  "?[10 * (Any, x: int64, x: int64, a: Any) -> Any$",
  "?[10 * (Any, x: int64, x: int64, a: Any) -> Any$",
  "$(Any, x: int64, x: int64, a: Any) -> Any",
  "=(Any, x: int64, x: int64, a: Any) -> Any",
  "$[(Any, x: int64, x: int64, a: Any) -> Any]",
  "xxx[(Any, x: int64, x: int64, a: Any) -> Any]",
  "?$(Any, x: int64, x: int64, a: Any) -> Any]",
  "?=(Any, x: int64, x: int64, a: Any) -> Any]",
  "?[(Any, x: int64, x: int64, a: Any) -> Any$",
  "?[(Any, x: int64, x: int64, a: Any) -> Any$",
  "10 $ (Any, x: int64, x: int64, a: Any) -> Any",
  "?10 $ (Any, x: int64, x: int64, a: Any) -> Any",
  "?(10 $ (Any, x: int64, x: int64, a: Any) -> Any)",
  "$10 $ (Any, x: int64, x: int64, a: Any) -> Any",
  "=10 $ (Any, x: int64, x: int64, a: Any) -> Any",
  "$[10 $ (Any, x: int64, x: int64, a: Any) -> Any]",
  "xxx[10 $ (Any, x: int64, x: int64, a: Any) -> Any]",
  "?$10 $ (Any, x: int64, x: int64, a: Any) -> Any]",
  "?=10 $ (Any, x: int64, x: int64, a: Any) -> Any]",
  "?[10 $ (Any, x: int64, x: int64, a: Any) -> Any$",
  "?[10 $ (Any, x: int64, x: int64, a: Any) -> Any$",
  "10 = (Any, x: int64, x: int64, a: Any) -> Any",
  "?10 = (Any, x: int64, x: int64, a: Any) -> Any",
  "?(10 = (Any, x: int64, x: int64, a: Any) -> Any)",
  "$10 = (Any, x: int64, x: int64, a: Any) -> Any",
  "=10 = (Any, x: int64, x: int64, a: Any) -> Any",
  "$[10 = (Any, x: int64, x: int64, a: Any) -> Any]",
  "xxx[10 = (Any, x: int64, x: int64, a: Any) -> Any]",
  "?$10 = (Any, x: int64, x: int64, a: Any) -> Any]",
  "?=10 = (Any, x: int64, x: int64, a: Any) -> Any]",
  "?[10 = (Any, x: int64, x: int64, a: Any) -> Any$",
  "?[10 = (Any, x: int64, x: int64, a: Any) -> Any$",
  "10 * (Any, x: int64, x: int64$ a: Any) -> Any",
  "?10 * (Any, x: int64, x: int64$ a: Any) -> Any",
  "?(10 * (Any, x: int64, x: int64$ a: Any) -> Any)",
  "$10 * (Any, x: int64, x: int64$ a: Any) -> Any",
  "=10 * (Any, x: int64, x: int64$ a: Any) -> Any",
  "$[10 * (Any, x: int64, x: int64$ a: Any) -> Any]",
  "xxx[10 * (Any, x: int64, x: int64$ a: Any) -> Any]",
  "?$10 * (Any, x: int64, x: int64$ a: Any) -> Any]",
  "?=10 * (Any, x: int64, x: int64$ a: Any) -> Any]",
  "?[10 * (Any, x: int64, x: int64$ a: Any) -> Any$",
  "?[10 * (Any, x: int64, x: int64$ a: Any) -> Any$",
  "(Any, x: int64, x: int64$ a: Any) -> Any",
  "?(Any, x: int64, x: int64$ a: Any) -> Any",
  "?((Any, x: int64, x: int64$ a: Any) -> Any)",
  "$(Any, x: int64, x: int64$ a: Any) -> Any",
  "=(Any, x: int64, x: int64$ a: Any) -> Any",
  "$[(Any, x: int64, x: int64$ a: Any) -> Any]",
  "xxx[(Any, x: int64, x: int64$ a: Any) -> Any]",
  "?$(Any, x: int64, x: int64$ a: Any) -> Any]",
  "?=(Any, x: int64, x: int64$ a: Any) -> Any]",
  "?[(Any, x: int64, x: int64$ a: Any) -> Any$",
  "?[(Any, x: int64, x: int64$ a: Any) -> Any$",
  "10 $ (Any, x: int64, x: int64$ a: Any) -> Any",
  "?10 $ (Any, x: int64, x: int64$ a: Any) -> Any",
  "?(10 $ (Any, x: int64, x: int64$ a: Any) -> Any)",
  "$10 $ (Any, x: int64, x: int64$ a: Any) -> Any",
  "=10 $ (Any, x: int64, x: int64$ a: Any) -> Any",
  "$[10 $ (Any, x: int64, x: int64$ a: Any) -> Any]",
  "xxx[10 $ (Any, x: int64, x: int64$ a: Any) -> Any]",
  "?$10 $ (Any, x: int64, x: int64$ a: Any) -> Any]",
  "?=10 $ (Any, x: int64, x: int64$ a: Any) -> Any]",
  "?[10 $ (Any, x: int64, x: int64$ a: Any) -> Any$",
  "?[10 $ (Any, x: int64, x: int64$ a: Any) -> Any$",
  "10 = (Any, x: int64, x: int64$ a: Any) -> Any",
  "?10 = (Any, x: int64, x: int64$ a: Any) -> Any",
  "?(10 = (Any, x: int64, x: int64$ a: Any) -> Any)",
  "$10 = (Any, x: int64, x: int64$ a: Any) -> Any",
  "=10 = (Any, x: int64, x: int64$ a: Any) -> Any",
  "$[10 = (Any, x: int64, x: int64$ a: Any) -> Any]",
  "xxx[10 = (Any, x: int64, x: int64$ a: Any) -> Any]",
  "?$10 = (Any, x: int64, x: int64$ a: Any) -> Any]",
  "?=10 = (Any, x: int64, x: int64$ a: Any) -> Any]",
  "?[10 = (Any, x: int64, x: int64$ a: Any) -> Any$",
  "?[10 = (Any, x: int64, x: int64$ a: Any) -> Any$",
  "10 * (Any, x: int64, x: int64= a: Any) -> Any",
  "?10 * (Any, x: int64, x: int64= a: Any) -> Any",
  "?(10 * (Any, x: int64, x: int64= a: Any) -> Any)",
  "$10 * (Any, x: int64, x: int64= a: Any) -> Any",
  "=10 * (Any, x: int64, x: int64= a: Any) -> Any",
  "$[10 * (Any, x: int64, x: int64= a: Any) -> Any]",
  "xxx[10 * (Any, x: int64, x: int64= a: Any) -> Any]",
  "?$10 * (Any, x: int64, x: int64= a: Any) -> Any]",
  "?=10 * (Any, x: int64, x: int64= a: Any) -> Any]",
  "?[10 * (Any, x: int64, x: int64= a: Any) -> Any$",
  "?[10 * (Any, x: int64, x: int64= a: Any) -> Any$",
  "(Any, x: int64, x: int64= a: Any) -> Any",
  "?(Any, x: int64, x: int64= a: Any) -> Any",
  "?((Any, x: int64, x: int64= a: Any) -> Any)",
  "$(Any, x: int64, x: int64= a: Any) -> Any",
  "=(Any, x: int64, x: int64= a: Any) -> Any",
  "$[(Any, x: int64, x: int64= a: Any) -> Any]",
  "xxx[(Any, x: int64, x: int64= a: Any) -> Any]",
  "?$(Any, x: int64, x: int64= a: Any) -> Any]",
  "?=(Any, x: int64, x: int64= a: Any) -> Any]",
  "?[(Any, x: int64, x: int64= a: Any) -> Any$",
  "?[(Any, x: int64, x: int64= a: Any) -> Any$",
  "10 $ (Any, x: int64, x: int64= a: Any) -> Any",
  "?10 $ (Any, x: int64, x: int64= a: Any) -> Any",
  "?(10 $ (Any, x: int64, x: int64= a: Any) -> Any)",
  "$10 $ (Any, x: int64, x: int64= a: Any) -> Any",
  "=10 $ (Any, x: int64, x: int64= a: Any) -> Any",
  "$[10 $ (Any, x: int64, x: int64= a: Any) -> Any]",
  "xxx[10 $ (Any, x: int64, x: int64= a: Any) -> Any]",
  "?$10 $ (Any, x: int64, x: int64= a: Any) -> Any]",
  "?=10 $ (Any, x: int64, x: int64= a: Any) -> Any]",
  "?[10 $ (Any, x: int64, x: int64= a: Any) -> Any$",
  "?[10 $ (Any, x: int64, x: int64= a: Any) -> Any$",
  "10 = (Any, x: int64, x: int64= a: Any) -> Any",
  "?10 = (Any, x: int64, x: int64= a: Any) -> Any",
  "?(10 = (Any, x: int64, x: int64= a: Any) -> Any)",
  "$10 = (Any, x: int64, x: int64= a: Any) -> Any",
  "=10 = (Any, x: int64, x: int64= a: Any) -> Any",
  "$[10 = (Any, x: int64, x: int64= a: Any) -> Any]",
  "xxx[10 = (Any, x: int64, x: int64= a: Any) -> Any]",
  "?$10 = (Any, x: int64, x: int64= a: Any) -> Any]",
  "?=10 = (Any, x: int64, x: int64= a: Any) -> Any]",
  "?[10 = (Any, x: int64, x: int64= a: Any) -> Any$",
  "?[10 = (Any, x: int64, x: int64= a: Any) -> Any$",
  "10 * (Any, $: Any) -> Any",
  "?10 * (Any, $: Any) -> Any",
  "?(10 * (Any, $: Any) -> Any)",
  "$10 * (Any, $: Any) -> Any",
  "=10 * (Any, $: Any) -> Any",
  "$[10 * (Any, $: Any) -> Any]",
  "xxx[10 * (Any, $: Any) -> Any]",
  "?$10 * (Any, $: Any) -> Any]",
  "?=10 * (Any, $: Any) -> Any]",
  "?[10 * (Any, $: Any) -> Any$",
  "?[10 * (Any, $: Any) -> Any$",
  "(Any, $: Any) -> Any",
  "?(Any, $: Any) -> Any",
  "?((Any, $: Any) -> Any)",
  "$(Any, $: Any) -> Any",
  "=(Any, $: Any) -> Any",
  "$[(Any, $: Any) -> Any]",
  "xxx[(Any, $: Any) -> Any]",
  "?$(Any, $: Any) -> Any]",
  "?=(Any, $: Any) -> Any]",
  "?[(Any, $: Any) -> Any$",
  "?[(Any, $: Any) -> Any$",
  "10 $ (Any, $: Any) -> Any",
  "?10 $ (Any, $: Any) -> Any",
  "?(10 $ (Any, $: Any) -> Any)",
  "$10 $ (Any, $: Any) -> Any",
  "=10 $ (Any, $: Any) -> Any",
  "$[10 $ (Any, $: Any) -> Any]",
  "xxx[10 $ (Any, $: Any) -> Any]",
  "?$10 $ (Any, $: Any) -> Any]",
  "?=10 $ (Any, $: Any) -> Any]",
  "?[10 $ (Any, $: Any) -> Any$",
  "?[10 $ (Any, $: Any) -> Any$",
  "10 = (Any, $: Any) -> Any",
  "?10 = (Any, $: Any) -> Any",
  "?(10 = (Any, $: Any) -> Any)",
  "$10 = (Any, $: Any) -> Any",
  "=10 = (Any, $: Any) -> Any",
  "$[10 = (Any, $: Any) -> Any]",
  "xxx[10 = (Any, $: Any) -> Any]",
  "?$10 = (Any, $: Any) -> Any]",
  "?=10 = (Any, $: Any) -> Any]",
  "?[10 = (Any, $: Any) -> Any$",
  "?[10 = (Any, $: Any) -> Any$",
  "10 * (Any, x: int64, $: Any) -> Any",
  "?10 * (Any, x: int64, $: Any) -> Any",
  "?(10 * (Any, x: int64, $: Any) -> Any)",
  "$10 * (Any, x: int64, $: Any) -> Any",
  "=10 * (Any, x: int64, $: Any) -> Any",
  "$[10 * (Any, x: int64, $: Any) -> Any]",
  "xxx[10 * (Any, x: int64, $: Any) -> Any]",
  "?$10 * (Any, x: int64, $: Any) -> Any]",
  "?=10 * (Any, x: int64, $: Any) -> Any]",
  "?[10 * (Any, x: int64, $: Any) -> Any$",
  "?[10 * (Any, x: int64, $: Any) -> Any$",
  "(Any, x: int64, $: Any) -> Any",
  "?(Any, x: int64, $: Any) -> Any",
  "?((Any, x: int64, $: Any) -> Any)",
  "$(Any, x: int64, $: Any) -> Any",
  "=(Any, x: int64, $: Any) -> Any",
  "$[(Any, x: int64, $: Any) -> Any]",
  "xxx[(Any, x: int64, $: Any) -> Any]",
  "?$(Any, x: int64, $: Any) -> Any]",
  "?=(Any, x: int64, $: Any) -> Any]",
  "?[(Any, x: int64, $: Any) -> Any$",
  "?[(Any, x: int64, $: Any) -> Any$",
  "10 $ (Any, x: int64, $: Any) -> Any",
  "?10 $ (Any, x: int64, $: Any) -> Any",
  "?(10 $ (Any, x: int64, $: Any) -> Any)",
  "$10 $ (Any, x: int64, $: Any) -> Any",
  "=10 $ (Any, x: int64, $: Any) -> Any",
  "$[10 $ (Any, x: int64, $: Any) -> Any]",
  "xxx[10 $ (Any, x: int64, $: Any) -> Any]",
  "?$10 $ (Any, x: int64, $: Any) -> Any]",
  "?=10 $ (Any, x: int64, $: Any) -> Any]",
  "?[10 $ (Any, x: int64, $: Any) -> Any$",
  "?[10 $ (Any, x: int64, $: Any) -> Any$",
  "10 = (Any, x: int64, $: Any) -> Any",
  "?10 = (Any, x: int64, $: Any) -> Any",
  "?(10 = (Any, x: int64, $: Any) -> Any)",
  "$10 = (Any, x: int64, $: Any) -> Any",
  "=10 = (Any, x: int64, $: Any) -> Any",
  "$[10 = (Any, x: int64, $: Any) -> Any]",
  "xxx[10 = (Any, x: int64, $: Any) -> Any]",
  "?$10 = (Any, x: int64, $: Any) -> Any]",
  "?=10 = (Any, x: int64, $: Any) -> Any]",
  "?[10 = (Any, x: int64, $: Any) -> Any$",
  "?[10 = (Any, x: int64, $: Any) -> Any$",
  "10 * (Any, x: int64$ $: Any) -> Any",
  "?10 * (Any, x: int64$ $: Any) -> Any",
  "?(10 * (Any, x: int64$ $: Any) -> Any)",
  "$10 * (Any, x: int64$ $: Any) -> Any",
  "=10 * (Any, x: int64$ $: Any) -> Any",
  "$[10 * (Any, x: int64$ $: Any) -> Any]",
  "xxx[10 * (Any, x: int64$ $: Any) -> Any]",
  "?$10 * (Any, x: int64$ $: Any) -> Any]",
  "?=10 * (Any, x: int64$ $: Any) -> Any]",
  "?[10 * (Any, x: int64$ $: Any) -> Any$",
  "?[10 * (Any, x: int64$ $: Any) -> Any$",
  "(Any, x: int64$ $: Any) -> Any",
  "?(Any, x: int64$ $: Any) -> Any",
  "?((Any, x: int64$ $: Any) -> Any)",
  "$(Any, x: int64$ $: Any) -> Any",
  "=(Any, x: int64$ $: Any) -> Any",
  "$[(Any, x: int64$ $: Any) -> Any]",
  "xxx[(Any, x: int64$ $: Any) -> Any]",
  "?$(Any, x: int64$ $: Any) -> Any]",
  "?=(Any, x: int64$ $: Any) -> Any]",
  "?[(Any, x: int64$ $: Any) -> Any$",
  "?[(Any, x: int64$ $: Any) -> Any$",
  "10 $ (Any, x: int64$ $: Any) -> Any",
  "?10 $ (Any, x: int64$ $: Any) -> Any",
  "?(10 $ (Any, x: int64$ $: Any) -> Any)",
  "$10 $ (Any, x: int64$ $: Any) -> Any",
  "=10 $ (Any, x: int64$ $: Any) -> Any",
  "$[10 $ (Any, x: int64$ $: Any) -> Any]",
  "xxx[10 $ (Any, x: int64$ $: Any) -> Any]",
  "?$10 $ (Any, x: int64$ $: Any) -> Any]",
  "?=10 $ (Any, x: int64$ $: Any) -> Any]",
  "?[10 $ (Any, x: int64$ $: Any) -> Any$",
  "?[10 $ (Any, x: int64$ $: Any) -> Any$",
  "10 = (Any, x: int64$ $: Any) -> Any",
  "?10 = (Any, x: int64$ $: Any) -> Any",
  "?(10 = (Any, x: int64$ $: Any) -> Any)",
  "$10 = (Any, x: int64$ $: Any) -> Any",
  "=10 = (Any, x: int64$ $: Any) -> Any",
  "$[10 = (Any, x: int64$ $: Any) -> Any]",
  "xxx[10 = (Any, x: int64$ $: Any) -> Any]",
  "?$10 = (Any, x: int64$ $: Any) -> Any]",
  "?=10 = (Any, x: int64$ $: Any) -> Any]",
  "?[10 = (Any, x: int64$ $: Any) -> Any$",
  "?[10 = (Any, x: int64$ $: Any) -> Any$",
  "10 * (Any, x: int64= $: Any) -> Any",
  "?10 * (Any, x: int64= $: Any) -> Any",
  "?(10 * (Any, x: int64= $: Any) -> Any)",
  "$10 * (Any, x: int64= $: Any) -> Any",
  "=10 * (Any, x: int64= $: Any) -> Any",
  "$[10 * (Any, x: int64= $: Any) -> Any]",
  "xxx[10 * (Any, x: int64= $: Any) -> Any]",
  "?$10 * (Any, x: int64= $: Any) -> Any]",
  "?=10 * (Any, x: int64= $: Any) -> Any]",
  "?[10 * (Any, x: int64= $: Any) -> Any$",
  "?[10 * (Any, x: int64= $: Any) -> Any$",
  "(Any, x: int64= $: Any) -> Any",
  "?(Any, x: int64= $: Any) -> Any",
  "?((Any, x: int64= $: Any) -> Any)",
  "$(Any, x: int64= $: Any) -> Any",
  "=(Any, x: int64= $: Any) -> Any",
  "$[(Any, x: int64= $: Any) -> Any]",
  "xxx[(Any, x: int64= $: Any) -> Any]",
  "?$(Any, x: int64= $: Any) -> Any]",
  "?=(Any, x: int64= $: Any) -> Any]",
  "?[(Any, x: int64= $: Any) -> Any$",
  "?[(Any, x: int64= $: Any) -> Any$",
  "10 $ (Any, x: int64= $: Any) -> Any",
  "?10 $ (Any, x: int64= $: Any) -> Any",
  "?(10 $ (Any, x: int64= $: Any) -> Any)",
  "$10 $ (Any, x: int64= $: Any) -> Any",
  "=10 $ (Any, x: int64= $: Any) -> Any",
  "$[10 $ (Any, x: int64= $: Any) -> Any]",
  "xxx[10 $ (Any, x: int64= $: Any) -> Any]",
  "?$10 $ (Any, x: int64= $: Any) -> Any]",
  "?=10 $ (Any, x: int64= $: Any) -> Any]",
  "?[10 $ (Any, x: int64= $: Any) -> Any$",
  "?[10 $ (Any, x: int64= $: Any) -> Any$",
  "10 = (Any, x: int64= $: Any) -> Any",
  "?10 = (Any, x: int64= $: Any) -> Any",
  "?(10 = (Any, x: int64= $: Any) -> Any)",
  "$10 = (Any, x: int64= $: Any) -> Any",
  "=10 = (Any, x: int64= $: Any) -> Any",
  "$[10 = (Any, x: int64= $: Any) -> Any]",
  "xxx[10 = (Any, x: int64= $: Any) -> Any]",
  "?$10 = (Any, x: int64= $: Any) -> Any]",
  "?=10 = (Any, x: int64= $: Any) -> Any]",
  "?[10 = (Any, x: int64= $: Any) -> Any$",
  "?[10 = (Any, x: int64= $: Any) -> Any$",
  "10 * (Any, x: int64, x: int64, $: Any) -> Any",
  "?10 * (Any, x: int64, x: int64, $: Any) -> Any",
  "?(10 * (Any, x: int64, x: int64, $: Any) -> Any)",
  "$10 * (Any, x: int64, x: int64, $: Any) -> Any",
  "=10 * (Any, x: int64, x: int64, $: Any) -> Any",
  "$[10 * (Any, x: int64, x: int64, $: Any) -> Any]",
  "xxx[10 * (Any, x: int64, x: int64, $: Any) -> Any]",
  "?$10 * (Any, x: int64, x: int64, $: Any) -> Any]",
  "?=10 * (Any, x: int64, x: int64, $: Any) -> Any]",
  "?[10 * (Any, x: int64, x: int64, $: Any) -> Any$",
  "?[10 * (Any, x: int64, x: int64, $: Any) -> Any$",
  "(Any, x: int64, x: int64, $: Any) -> Any",
  "?(Any, x: int64, x: int64, $: Any) -> Any",
  "?((Any, x: int64, x: int64, $: Any) -> Any)",
  "$(Any, x: int64, x: int64, $: Any) -> Any",
  "=(Any, x: int64, x: int64, $: Any) -> Any",
  "$[(Any, x: int64, x: int64, $: Any) -> Any]",
  "xxx[(Any, x: int64, x: int64, $: Any) -> Any]",
  "?$(Any, x: int64, x: int64, $: Any) -> Any]",
  "?=(Any, x: int64, x: int64, $: Any) -> Any]",
  "?[(Any, x: int64, x: int64, $: Any) -> Any$",
  "?[(Any, x: int64, x: int64, $: Any) -> Any$",
  "10 $ (Any, x: int64, x: int64, $: Any) -> Any",
  "?10 $ (Any, x: int64, x: int64, $: Any) -> Any",
  "?(10 $ (Any, x: int64, x: int64, $: Any) -> Any)",
  "$10 $ (Any, x: int64, x: int64, $: Any) -> Any",
  "=10 $ (Any, x: int64, x: int64, $: Any) -> Any",
  "$[10 $ (Any, x: int64, x: int64, $: Any) -> Any]",
  "xxx[10 $ (Any, x: int64, x: int64, $: Any) -> Any]",
  "?$10 $ (Any, x: int64, x: int64, $: Any) -> Any]",
  "?=10 $ (Any, x: int64, x: int64, $: Any) -> Any]",
  "?[10 $ (Any, x: int64, x: int64, $: Any) -> Any$",
  "?[10 $ (Any, x: int64, x: int64, $: Any) -> Any$",
  "10 = (Any, x: int64, x: int64, $: Any) -> Any",
  "?10 = (Any, x: int64, x: int64, $: Any) -> Any",
  "?(10 = (Any, x: int64, x: int64, $: Any) -> Any)",
  "$10 = (Any, x: int64, x: int64, $: Any) -> Any",
  "=10 = (Any, x: int64, x: int64, $: Any) -> Any",
  "$[10 = (Any, x: int64, x: int64, $: Any) -> Any]",
  "xxx[10 = (Any, x: int64, x: int64, $: Any) -> Any]",
  "?$10 = (Any, x: int64, x: int64, $: Any) -> Any]",
  "?=10 = (Any, x: int64, x: int64, $: Any) -> Any]",
  "?[10 = (Any, x: int64, x: int64, $: Any) -> Any$",
  "?[10 = (Any, x: int64, x: int64, $: Any) -> Any$",
  "10 * (Any, x: int64, x: int64$ $: Any) -> Any",
  "?10 * (Any, x: int64, x: int64$ $: Any) -> Any",
  "?(10 * (Any, x: int64, x: int64$ $: Any) -> Any)",
  "$10 * (Any, x: int64, x: int64$ $: Any) -> Any",
  "=10 * (Any, x: int64, x: int64$ $: Any) -> Any",
  "$[10 * (Any, x: int64, x: int64$ $: Any) -> Any]",
  "xxx[10 * (Any, x: int64, x: int64$ $: Any) -> Any]",
  "?$10 * (Any, x: int64, x: int64$ $: Any) -> Any]",
  "?=10 * (Any, x: int64, x: int64$ $: Any) -> Any]",
  "?[10 * (Any, x: int64, x: int64$ $: Any) -> Any$",
  "?[10 * (Any, x: int64, x: int64$ $: Any) -> Any$",
  "(Any, x: int64, x: int64$ $: Any) -> Any",
  "?(Any, x: int64, x: int64$ $: Any) -> Any",
  "?((Any, x: int64, x: int64$ $: Any) -> Any)",
  "$(Any, x: int64, x: int64$ $: Any) -> Any",
  "=(Any, x: int64, x: int64$ $: Any) -> Any",
  "$[(Any, x: int64, x: int64$ $: Any) -> Any]",
  "xxx[(Any, x: int64, x: int64$ $: Any) -> Any]",
  "?$(Any, x: int64, x: int64$ $: Any) -> Any]",
  "?=(Any, x: int64, x: int64$ $: Any) -> Any]",
  "?[(Any, x: int64, x: int64$ $: Any) -> Any$",
  "?[(Any, x: int64, x: int64$ $: Any) -> Any$",
  "10 $ (Any, x: int64, x: int64$ $: Any) -> Any",
  "?10 $ (Any, x: int64, x: int64$ $: Any) -> Any",
  "?(10 $ (Any, x: int64, x: int64$ $: Any) -> Any)",
  "$10 $ (Any, x: int64, x: int64$ $: Any) -> Any",
  "=10 $ (Any, x: int64, x: int64$ $: Any) -> Any",
  "$[10 $ (Any, x: int64, x: int64$ $: Any) -> Any]",
  "xxx[10 $ (Any, x: int64, x: int64$ $: Any) -> Any]",
  "?$10 $ (Any, x: int64, x: int64$ $: Any) -> Any]",
  "?=10 $ (Any, x: int64, x: int64$ $: Any) -> Any]",
  "?[10 $ (Any, x: int64, x: int64$ $: Any) -> Any$",
  "?[10 $ (Any, x: int64, x: int64$ $: Any) -> Any$",
  "10 = (Any, x: int64, x: int64$ $: Any) -> Any",
  "?10 = (Any, x: int64, x: int64$ $: Any) -> Any",
  "?(10 = (Any, x: int64, x: int64$ $: Any) -> Any)",
  "$10 = (Any, x: int64, x: int64$ $: Any) -> Any",
  "=10 = (Any, x: int64, x: int64$ $: Any) -> Any",
  "$[10 = (Any, x: int64, x: int64$ $: Any) -> Any]",
  "xxx[10 = (Any, x: int64, x: int64$ $: Any) -> Any]",
  "?$10 = (Any, x: int64, x: int64$ $: Any) -> Any]",
  "?=10 = (Any, x: int64, x: int64$ $: Any) -> Any]",
  "?[10 = (Any, x: int64, x: int64$ $: Any) -> Any$",
  "?[10 = (Any, x: int64, x: int64$ $: Any) -> Any$",
  "10 * (Any, x: int64, x: int64= $: Any) -> Any",
  "?10 * (Any, x: int64, x: int64= $: Any) -> Any",
  "?(10 * (Any, x: int64, x: int64= $: Any) -> Any)",
  "$10 * (Any, x: int64, x: int64= $: Any) -> Any",
  "=10 * (Any, x: int64, x: int64= $: Any) -> Any",
  "$[10 * (Any, x: int64, x: int64= $: Any) -> Any]",
  "xxx[10 * (Any, x: int64, x: int64= $: Any) -> Any]",
  "?$10 * (Any, x: int64, x: int64= $: Any) -> Any]",
  "?=10 * (Any, x: int64, x: int64= $: Any) -> Any]",
  "?[10 * (Any, x: int64, x: int64= $: Any) -> Any$",
  "?[10 * (Any, x: int64, x: int64= $: Any) -> Any$",
  "(Any, x: int64, x: int64= $: Any) -> Any",
  "?(Any, x: int64, x: int64= $: Any) -> Any",
  "?((Any, x: int64, x: int64= $: Any) -> Any)",
  "$(Any, x: int64, x: int64= $: Any) -> Any",
  "=(Any, x: int64, x: int64= $: Any) -> Any",
  "$[(Any, x: int64, x: int64= $: Any) -> Any]",
  "xxx[(Any, x: int64, x: int64= $: Any) -> Any]",
  "?$(Any, x: int64, x: int64= $: Any) -> Any]",
  "?=(Any, x: int64, x: int64= $: Any) -> Any]",
  "?[(Any, x: int64, x: int64= $: Any) -> Any$",
  "?[(Any, x: int64, x: int64= $: Any) -> Any$",
  "10 $ (Any, x: int64, x: int64= $: Any) -> Any",
  "?10 $ (Any, x: int64, x: int64= $: Any) -> Any",
  "?(10 $ (Any, x: int64, x: int64= $: Any) -> Any)",
  "$10 $ (Any, x: int64, x: int64= $: Any) -> Any",
  "=10 $ (Any, x: int64, x: int64= $: Any) -> Any",
  "$[10 $ (Any, x: int64, x: int64= $: Any) -> Any]",
  "xxx[10 $ (Any, x: int64, x: int64= $: Any) -> Any]",
  "?$10 $ (Any, x: int64, x: int64= $: Any) -> Any]",
  "?=10 $ (Any, x: int64, x: int64= $: Any) -> Any]",
  "?[10 $ (Any, x: int64, x: int64= $: Any) -> Any$",
  "?[10 $ (Any, x: int64, x: int64= $: Any) -> Any$",
  "10 = (Any, x: int64, x: int64= $: Any) -> Any",
  "?10 = (Any, x: int64, x: int64= $: Any) -> Any",
  "?(10 = (Any, x: int64, x: int64= $: Any) -> Any)",
  "$10 = (Any, x: int64, x: int64= $: Any) -> Any",
  "=10 = (Any, x: int64, x: int64= $: Any) -> Any",
  "$[10 = (Any, x: int64, x: int64= $: Any) -> Any]",
  "xxx[10 = (Any, x: int64, x: int64= $: Any) -> Any]",
  "?$10 = (Any, x: int64, x: int64= $: Any) -> Any]",
  "?=10 = (Any, x: int64, x: int64= $: Any) -> Any]",
  "?[10 = (Any, x: int64, x: int64= $: Any) -> Any$",
  "?[10 = (Any, x: int64, x: int64= $: Any) -> Any$",
  "10 * (Any, =: Any) -> Any",
  "?10 * (Any, =: Any) -> Any",
  "?(10 * (Any, =: Any) -> Any)",
  "$10 * (Any, =: Any) -> Any",
  "=10 * (Any, =: Any) -> Any",
  "$[10 * (Any, =: Any) -> Any]",
  "xxx[10 * (Any, =: Any) -> Any]",
  "?$10 * (Any, =: Any) -> Any]",
  "?=10 * (Any, =: Any) -> Any]",
  "?[10 * (Any, =: Any) -> Any$",
  "?[10 * (Any, =: Any) -> Any$",
  "(Any, =: Any) -> Any",
  "?(Any, =: Any) -> Any",
  "?((Any, =: Any) -> Any)",
  "$(Any, =: Any) -> Any",
  "=(Any, =: Any) -> Any",
  "$[(Any, =: Any) -> Any]",
  "xxx[(Any, =: Any) -> Any]",
  "?$(Any, =: Any) -> Any]",
  "?=(Any, =: Any) -> Any]",
  "?[(Any, =: Any) -> Any$",
  "?[(Any, =: Any) -> Any$",
  "10 $ (Any, =: Any) -> Any",
  "?10 $ (Any, =: Any) -> Any",
  "?(10 $ (Any, =: Any) -> Any)",
  "$10 $ (Any, =: Any) -> Any",
  "=10 $ (Any, =: Any) -> Any",
  "$[10 $ (Any, =: Any) -> Any]",
  "xxx[10 $ (Any, =: Any) -> Any]",
  "?$10 $ (Any, =: Any) -> Any]",
  "?=10 $ (Any, =: Any) -> Any]",
  "?[10 $ (Any, =: Any) -> Any$",
  "?[10 $ (Any, =: Any) -> Any$",
  "10 = (Any, =: Any) -> Any",
  "?10 = (Any, =: Any) -> Any",
  "?(10 = (Any, =: Any) -> Any)",
  "$10 = (Any, =: Any) -> Any",
  "=10 = (Any, =: Any) -> Any",
  "$[10 = (Any, =: Any) -> Any]",
  "xxx[10 = (Any, =: Any) -> Any]",
  "?$10 = (Any, =: Any) -> Any]",
  "?=10 = (Any, =: Any) -> Any]",
  "?[10 = (Any, =: Any) -> Any$",
  "?[10 = (Any, =: Any) -> Any$",
  "10 * (Any, x: int64, =: Any) -> Any",
  "?10 * (Any, x: int64, =: Any) -> Any",
  "?(10 * (Any, x: int64, =: Any) -> Any)",
  "$10 * (Any, x: int64, =: Any) -> Any",
  "=10 * (Any, x: int64, =: Any) -> Any",
  "$[10 * (Any, x: int64, =: Any) -> Any]",
  "xxx[10 * (Any, x: int64, =: Any) -> Any]",
  "?$10 * (Any, x: int64, =: Any) -> Any]",
  "?=10 * (Any, x: int64, =: Any) -> Any]",
  "?[10 * (Any, x: int64, =: Any) -> Any$",
  "?[10 * (Any, x: int64, =: Any) -> Any$",
  "(Any, x: int64, =: Any) -> Any",
  "?(Any, x: int64, =: Any) -> Any",
  "?((Any, x: int64, =: Any) -> Any)",
  "$(Any, x: int64, =: Any) -> Any",
  "=(Any, x: int64, =: Any) -> Any",
  "$[(Any, x: int64, =: Any) -> Any]",
  "xxx[(Any, x: int64, =: Any) -> Any]",
  "?$(Any, x: int64, =: Any) -> Any]",
  "?=(Any, x: int64, =: Any) -> Any]",
  "?[(Any, x: int64, =: Any) -> Any$",
  "?[(Any, x: int64, =: Any) -> Any$",
  "10 $ (Any, x: int64, =: Any) -> Any",
  "?10 $ (Any, x: int64, =: Any) -> Any",
  "?(10 $ (Any, x: int64, =: Any) -> Any)",
  "$10 $ (Any, x: int64, =: Any) -> Any",
  "=10 $ (Any, x: int64, =: Any) -> Any",
  "$[10 $ (Any, x: int64, =: Any) -> Any]",
  "xxx[10 $ (Any, x: int64, =: Any) -> Any]",
  "?$10 $ (Any, x: int64, =: Any) -> Any]",
  "?=10 $ (Any, x: int64, =: Any) -> Any]",
  "?[10 $ (Any, x: int64, =: Any) -> Any$",
  "?[10 $ (Any, x: int64, =: Any) -> Any$",
  "10 = (Any, x: int64, =: Any) -> Any",
  "?10 = (Any, x: int64, =: Any) -> Any",
  "?(10 = (Any, x: int64, =: Any) -> Any)",
  "$10 = (Any, x: int64, =: Any) -> Any",
  "=10 = (Any, x: int64, =: Any) -> Any",
  "$[10 = (Any, x: int64, =: Any) -> Any]",
  "xxx[10 = (Any, x: int64, =: Any) -> Any]",
  "?$10 = (Any, x: int64, =: Any) -> Any]",
  "?=10 = (Any, x: int64, =: Any) -> Any]",
  "?[10 = (Any, x: int64, =: Any) -> Any$",
  "?[10 = (Any, x: int64, =: Any) -> Any$",
  "10 * (Any, x: int64$ =: Any) -> Any",
  "?10 * (Any, x: int64$ =: Any) -> Any",
  "?(10 * (Any, x: int64$ =: Any) -> Any)",
  "$10 * (Any, x: int64$ =: Any) -> Any",
  "=10 * (Any, x: int64$ =: Any) -> Any",
  "$[10 * (Any, x: int64$ =: Any) -> Any]",
  "xxx[10 * (Any, x: int64$ =: Any) -> Any]",
  "?$10 * (Any, x: int64$ =: Any) -> Any]",
  "?=10 * (Any, x: int64$ =: Any) -> Any]",
  "?[10 * (Any, x: int64$ =: Any) -> Any$",
  "?[10 * (Any, x: int64$ =: Any) -> Any$",
  "(Any, x: int64$ =: Any) -> Any",
  "?(Any, x: int64$ =: Any) -> Any",
  "?((Any, x: int64$ =: Any) -> Any)",
  "$(Any, x: int64$ =: Any) -> Any",
  "=(Any, x: int64$ =: Any) -> Any",
  "$[(Any, x: int64$ =: Any) -> Any]",
  "xxx[(Any, x: int64$ =: Any) -> Any]",
  "?$(Any, x: int64$ =: Any) -> Any]",
  "?=(Any, x: int64$ =: Any) -> Any]",
  "?[(Any, x: int64$ =: Any) -> Any$",
  "?[(Any, x: int64$ =: Any) -> Any$",
  "10 $ (Any, x: int64$ =: Any) -> Any",
  "?10 $ (Any, x: int64$ =: Any) -> Any",
  "?(10 $ (Any, x: int64$ =: Any) -> Any)",
  "$10 $ (Any, x: int64$ =: Any) -> Any",
  "=10 $ (Any, x: int64$ =: Any) -> Any",
  "$[10 $ (Any, x: int64$ =: Any) -> Any]",
  "xxx[10 $ (Any, x: int64$ =: Any) -> Any]",
  "?$10 $ (Any, x: int64$ =: Any) -> Any]",
  "?=10 $ (Any, x: int64$ =: Any) -> Any]",
  "?[10 $ (Any, x: int64$ =: Any) -> Any$",
  "?[10 $ (Any, x: int64$ =: Any) -> Any$",
  "10 = (Any, x: int64$ =: Any) -> Any",
  "?10 = (Any, x: int64$ =: Any) -> Any",
  "?(10 = (Any, x: int64$ =: Any) -> Any)",
  "$10 = (Any, x: int64$ =: Any) -> Any",
  "=10 = (Any, x: int64$ =: Any) -> Any",
  "$[10 = (Any, x: int64$ =: Any) -> Any]",
  "xxx[10 = (Any, x: int64$ =: Any) -> Any]",
  "?$10 = (Any, x: int64$ =: Any) -> Any]",
  "?=10 = (Any, x: int64$ =: Any) -> Any]",
  "?[10 = (Any, x: int64$ =: Any) -> Any$",
  "?[10 = (Any, x: int64$ =: Any) -> Any$",
  "10 * (Any, x: int64= =: Any) -> Any",
  "?10 * (Any, x: int64= =: Any) -> Any",
  "?(10 * (Any, x: int64= =: Any) -> Any)",
  "$10 * (Any, x: int64= =: Any) -> Any",
  "=10 * (Any, x: int64= =: Any) -> Any",
  "$[10 * (Any, x: int64= =: Any) -> Any]",
  "xxx[10 * (Any, x: int64= =: Any) -> Any]",
  "?$10 * (Any, x: int64= =: Any) -> Any]",
  "?=10 * (Any, x: int64= =: Any) -> Any]",
  "?[10 * (Any, x: int64= =: Any) -> Any$",
  "?[10 * (Any, x: int64= =: Any) -> Any$",
  "(Any, x: int64= =: Any) -> Any",
  "?(Any, x: int64= =: Any) -> Any",
  "?((Any, x: int64= =: Any) -> Any)",
  "$(Any, x: int64= =: Any) -> Any",
  "=(Any, x: int64= =: Any) -> Any",
  "$[(Any, x: int64= =: Any) -> Any]",
  "xxx[(Any, x: int64= =: Any) -> Any]",
  "?$(Any, x: int64= =: Any) -> Any]",
  "?=(Any, x: int64= =: Any) -> Any]",
  "?[(Any, x: int64= =: Any) -> Any$",
  "?[(Any, x: int64= =: Any) -> Any$",
  "10 $ (Any, x: int64= =: Any) -> Any",
  "?10 $ (Any, x: int64= =: Any) -> Any",
  "?(10 $ (Any, x: int64= =: Any) -> Any)",
  "$10 $ (Any, x: int64= =: Any) -> Any",
  "=10 $ (Any, x: int64= =: Any) -> Any",
  "$[10 $ (Any, x: int64= =: Any) -> Any]",
  "xxx[10 $ (Any, x: int64= =: Any) -> Any]",
  "?$10 $ (Any, x: int64= =: Any) -> Any]",
  "?=10 $ (Any, x: int64= =: Any) -> Any]",
  "?[10 $ (Any, x: int64= =: Any) -> Any$",
  "?[10 $ (Any, x: int64= =: Any) -> Any$",
  "10 = (Any, x: int64= =: Any) -> Any",
  "?10 = (Any, x: int64= =: Any) -> Any",
  "?(10 = (Any, x: int64= =: Any) -> Any)",
  "$10 = (Any, x: int64= =: Any) -> Any",
  "=10 = (Any, x: int64= =: Any) -> Any",
  "$[10 = (Any, x: int64= =: Any) -> Any]",
  "xxx[10 = (Any, x: int64= =: Any) -> Any]",
  "?$10 = (Any, x: int64= =: Any) -> Any]",
  "?=10 = (Any, x: int64= =: Any) -> Any]",
  "?[10 = (Any, x: int64= =: Any) -> Any$",
  "?[10 = (Any, x: int64= =: Any) -> Any$",
  "10 * (Any, x: int64, x: int64, =: Any) -> Any",
  "?10 * (Any, x: int64, x: int64, =: Any) -> Any",
  "?(10 * (Any, x: int64, x: int64, =: Any) -> Any)",
  "$10 * (Any, x: int64, x: int64, =: Any) -> Any",
  "=10 * (Any, x: int64, x: int64, =: Any) -> Any",
  "$[10 * (Any, x: int64, x: int64, =: Any) -> Any]",
  "xxx[10 * (Any, x: int64, x: int64, =: Any) -> Any]",
  "?$10 * (Any, x: int64, x: int64, =: Any) -> Any]",
  "?=10 * (Any, x: int64, x: int64, =: Any) -> Any]",
  "?[10 * (Any, x: int64, x: int64, =: Any) -> Any$",
  "?[10 * (Any, x: int64, x: int64, =: Any) -> Any$",
  "(Any, x: int64, x: int64, =: Any) -> Any",
  "?(Any, x: int64, x: int64, =: Any) -> Any",
  "?((Any, x: int64, x: int64, =: Any) -> Any)",
  "$(Any, x: int64, x: int64, =: Any) -> Any",
  "=(Any, x: int64, x: int64, =: Any) -> Any",
  "$[(Any, x: int64, x: int64, =: Any) -> Any]",
  "xxx[(Any, x: int64, x: int64, =: Any) -> Any]",
  "?$(Any, x: int64, x: int64, =: Any) -> Any]",
  "?=(Any, x: int64, x: int64, =: Any) -> Any]",
  "?[(Any, x: int64, x: int64, =: Any) -> Any$",
  "?[(Any, x: int64, x: int64, =: Any) -> Any$",
  "10 $ (Any, x: int64, x: int64, =: Any) -> Any",
  "?10 $ (Any, x: int64, x: int64, =: Any) -> Any",
  "?(10 $ (Any, x: int64, x: int64, =: Any) -> Any)",
  "$10 $ (Any, x: int64, x: int64, =: Any) -> Any",
  "=10 $ (Any, x: int64, x: int64, =: Any) -> Any",
  "$[10 $ (Any, x: int64, x: int64, =: Any) -> Any]",
  "xxx[10 $ (Any, x: int64, x: int64, =: Any) -> Any]",
  "?$10 $ (Any, x: int64, x: int64, =: Any) -> Any]",
  "?=10 $ (Any, x: int64, x: int64, =: Any) -> Any]",
  "?[10 $ (Any, x: int64, x: int64, =: Any) -> Any$",
  "?[10 $ (Any, x: int64, x: int64, =: Any) -> Any$",
  "10 = (Any, x: int64, x: int64, =: Any) -> Any",
  "?10 = (Any, x: int64, x: int64, =: Any) -> Any",
  "?(10 = (Any, x: int64, x: int64, =: Any) -> Any)",
  "$10 = (Any, x: int64, x: int64, =: Any) -> Any",
  "=10 = (Any, x: int64, x: int64, =: Any) -> Any",
  "$[10 = (Any, x: int64, x: int64, =: Any) -> Any]",
  "xxx[10 = (Any, x: int64, x: int64, =: Any) -> Any]",
  "?$10 = (Any, x: int64, x: int64, =: Any) -> Any]",
  "?=10 = (Any, x: int64, x: int64, =: Any) -> Any]",
  "?[10 = (Any, x: int64, x: int64, =: Any) -> Any$",
  "?[10 = (Any, x: int64, x: int64, =: Any) -> Any$",
  "10 * (Any, x: int64, x: int64$ =: Any) -> Any",
  "?10 * (Any, x: int64, x: int64$ =: Any) -> Any",
  "?(10 * (Any, x: int64, x: int64$ =: Any) -> Any)",
  "$10 * (Any, x: int64, x: int64$ =: Any) -> Any",
  "=10 * (Any, x: int64, x: int64$ =: Any) -> Any",
  "$[10 * (Any, x: int64, x: int64$ =: Any) -> Any]",
  "xxx[10 * (Any, x: int64, x: int64$ =: Any) -> Any]",
  "?$10 * (Any, x: int64, x: int64$ =: Any) -> Any]",
  "?=10 * (Any, x: int64, x: int64$ =: Any) -> Any]",
  "?[10 * (Any, x: int64, x: int64$ =: Any) -> Any$",
  "?[10 * (Any, x: int64, x: int64$ =: Any) -> Any$",
  "(Any, x: int64, x: int64$ =: Any) -> Any",
  "?(Any, x: int64, x: int64$ =: Any) -> Any",
  "?((Any, x: int64, x: int64$ =: Any) -> Any)",
  "$(Any, x: int64, x: int64$ =: Any) -> Any",
  "=(Any, x: int64, x: int64$ =: Any) -> Any",
  "$[(Any, x: int64, x: int64$ =: Any) -> Any]",
  "xxx[(Any, x: int64, x: int64$ =: Any) -> Any]",
  "?$(Any, x: int64, x: int64$ =: Any) -> Any]",
  "?=(Any, x: int64, x: int64$ =: Any) -> Any]",
  "?[(Any, x: int64, x: int64$ =: Any) -> Any$",
  "?[(Any, x: int64, x: int64$ =: Any) -> Any$",
  "10 $ (Any, x: int64, x: int64$ =: Any) -> Any",
  "?10 $ (Any, x: int64, x: int64$ =: Any) -> Any",
  "?(10 $ (Any, x: int64, x: int64$ =: Any) -> Any)",
  "$10 $ (Any, x: int64, x: int64$ =: Any) -> Any",
  "=10 $ (Any, x: int64, x: int64$ =: Any) -> Any",
  "$[10 $ (Any, x: int64, x: int64$ =: Any) -> Any]",
  "xxx[10 $ (Any, x: int64, x: int64$ =: Any) -> Any]",
  "?$10 $ (Any, x: int64, x: int64$ =: Any) -> Any]",
  "?=10 $ (Any, x: int64, x: int64$ =: Any) -> Any]",
  "?[10 $ (Any, x: int64, x: int64$ =: Any) -> Any$",
  "?[10 $ (Any, x: int64, x: int64$ =: Any) -> Any$",
  "10 = (Any, x: int64, x: int64$ =: Any) -> Any",
  "?10 = (Any, x: int64, x: int64$ =: Any) -> Any",
  "?(10 = (Any, x: int64, x: int64$ =: Any) -> Any)",
  "$10 = (Any, x: int64, x: int64$ =: Any) -> Any",
  "=10 = (Any, x: int64, x: int64$ =: Any) -> Any",
  "$[10 = (Any, x: int64, x: int64$ =: Any) -> Any]",
  "xxx[10 = (Any, x: int64, x: int64$ =: Any) -> Any]",
  "?$10 = (Any, x: int64, x: int64$ =: Any) -> Any]",
  "?=10 = (Any, x: int64, x: int64$ =: Any) -> Any]",
  "?[10 = (Any, x: int64, x: int64$ =: Any) -> Any$",
  "?[10 = (Any, x: int64, x: int64$ =: Any) -> Any$",
  "10 * (Any, x: int64, x: int64= =: Any) -> Any",
  "?10 * (Any, x: int64, x: int64= =: Any) -> Any",
  "?(10 * (Any, x: int64, x: int64= =: Any) -> Any)",
  "$10 * (Any, x: int64, x: int64= =: Any) -> Any",
  "=10 * (Any, x: int64, x: int64= =: Any) -> Any",
  "$[10 * (Any, x: int64, x: int64= =: Any) -> Any]",
  "xxx[10 * (Any, x: int64, x: int64= =: Any) -> Any]",
  "?$10 * (Any, x: int64, x: int64= =: Any) -> Any]",
  "?=10 * (Any, x: int64, x: int64= =: Any) -> Any]",
  "?[10 * (Any, x: int64, x: int64= =: Any) -> Any$",
  "?[10 * (Any, x: int64, x: int64= =: Any) -> Any$",
  "(Any, x: int64, x: int64= =: Any) -> Any",
  "?(Any, x: int64, x: int64= =: Any) -> Any",
  "?((Any, x: int64, x: int64= =: Any) -> Any)",
  "$(Any, x: int64, x: int64= =: Any) -> Any",
  "=(Any, x: int64, x: int64= =: Any) -> Any",
  "$[(Any, x: int64, x: int64= =: Any) -> Any]",
  "xxx[(Any, x: int64, x: int64= =: Any) -> Any]",
  "?$(Any, x: int64, x: int64= =: Any) -> Any]",
  "?=(Any, x: int64, x: int64= =: Any) -> Any]",
  "?[(Any, x: int64, x: int64= =: Any) -> Any$",
  "?[(Any, x: int64, x: int64= =: Any) -> Any$",
  "10 $ (Any, x: int64, x: int64= =: Any) -> Any",
  "?10 $ (Any, x: int64, x: int64= =: Any) -> Any",
  "?(10 $ (Any, x: int64, x: int64= =: Any) -> Any)",
  "$10 $ (Any, x: int64, x: int64= =: Any) -> Any",
  "=10 $ (Any, x: int64, x: int64= =: Any) -> Any",
  "$[10 $ (Any, x: int64, x: int64= =: Any) -> Any]",
  "xxx[10 $ (Any, x: int64, x: int64= =: Any) -> Any]",
  "?$10 $ (Any, x: int64, x: int64= =: Any) -> Any]",
  "?=10 $ (Any, x: int64, x: int64= =: Any) -> Any]",
  "?[10 $ (Any, x: int64, x: int64= =: Any) -> Any$",
  "?[10 $ (Any, x: int64, x: int64= =: Any) -> Any$",
  "10 = (Any, x: int64, x: int64= =: Any) -> Any",
  "?10 = (Any, x: int64, x: int64= =: Any) -> Any",
  "?(10 = (Any, x: int64, x: int64= =: Any) -> Any)",
  "$10 = (Any, x: int64, x: int64= =: Any) -> Any",
  "=10 = (Any, x: int64, x: int64= =: Any) -> Any",
  "$[10 = (Any, x: int64, x: int64= =: Any) -> Any]",
  "xxx[10 = (Any, x: int64, x: int64= =: Any) -> Any]",
  "?$10 = (Any, x: int64, x: int64= =: Any) -> Any]",
  "?=10 = (Any, x: int64, x: int64= =: Any) -> Any]",
  "?[10 = (Any, x: int64, x: int64= =: Any) -> Any$",
  "?[10 = (Any, x: int64, x: int64= =: Any) -> Any$",
  "10 * (Any, a$ Any) -> Any",
  "?10 * (Any, a$ Any) -> Any",
  "?(10 * (Any, a$ Any) -> Any)",
  "$10 * (Any, a$ Any) -> Any",
  "=10 * (Any, a$ Any) -> Any",
  "$[10 * (Any, a$ Any) -> Any]",
  "xxx[10 * (Any, a$ Any) -> Any]",
  "?$10 * (Any, a$ Any) -> Any]",
  "?=10 * (Any, a$ Any) -> Any]",
  "?[10 * (Any, a$ Any) -> Any$",
  "?[10 * (Any, a$ Any) -> Any$",
  "(Any, a$ Any) -> Any",
  "?(Any, a$ Any) -> Any",
  "?((Any, a$ Any) -> Any)",
  "$(Any, a$ Any) -> Any",
  "=(Any, a$ Any) -> Any",
  "$[(Any, a$ Any) -> Any]",
  "xxx[(Any, a$ Any) -> Any]",
  "?$(Any, a$ Any) -> Any]",
  "?=(Any, a$ Any) -> Any]",
  "?[(Any, a$ Any) -> Any$",
  "?[(Any, a$ Any) -> Any$",
  "10 $ (Any, a$ Any) -> Any",
  "?10 $ (Any, a$ Any) -> Any",
  "?(10 $ (Any, a$ Any) -> Any)",
  "$10 $ (Any, a$ Any) -> Any",
  "=10 $ (Any, a$ Any) -> Any",
  "$[10 $ (Any, a$ Any) -> Any]",
  "xxx[10 $ (Any, a$ Any) -> Any]",
  "?$10 $ (Any, a$ Any) -> Any]",
  "?=10 $ (Any, a$ Any) -> Any]",
  "?[10 $ (Any, a$ Any) -> Any$",
  "?[10 $ (Any, a$ Any) -> Any$",
  "10 = (Any, a$ Any) -> Any",
  "?10 = (Any, a$ Any) -> Any",
  "?(10 = (Any, a$ Any) -> Any)",
  "$10 = (Any, a$ Any) -> Any",
  "=10 = (Any, a$ Any) -> Any",
  "$[10 = (Any, a$ Any) -> Any]",
  "xxx[10 = (Any, a$ Any) -> Any]",
  "?$10 = (Any, a$ Any) -> Any]",
  "?=10 = (Any, a$ Any) -> Any]",
  "?[10 = (Any, a$ Any) -> Any$",
  "?[10 = (Any, a$ Any) -> Any$",
  "10 * (Any, x: int64, a$ Any) -> Any",
  "?10 * (Any, x: int64, a$ Any) -> Any",
  "?(10 * (Any, x: int64, a$ Any) -> Any)",
  "$10 * (Any, x: int64, a$ Any) -> Any",
  "=10 * (Any, x: int64, a$ Any) -> Any",
  "$[10 * (Any, x: int64, a$ Any) -> Any]",
  "xxx[10 * (Any, x: int64, a$ Any) -> Any]",
  "?$10 * (Any, x: int64, a$ Any) -> Any]",
  "?=10 * (Any, x: int64, a$ Any) -> Any]",
  "?[10 * (Any, x: int64, a$ Any) -> Any$",
  "?[10 * (Any, x: int64, a$ Any) -> Any$",
  "(Any, x: int64, a$ Any) -> Any",
  "?(Any, x: int64, a$ Any) -> Any",
  "?((Any, x: int64, a$ Any) -> Any)",
  "$(Any, x: int64, a$ Any) -> Any",
  "=(Any, x: int64, a$ Any) -> Any",
  "$[(Any, x: int64, a$ Any) -> Any]",
  "xxx[(Any, x: int64, a$ Any) -> Any]",
  "?$(Any, x: int64, a$ Any) -> Any]",
  "?=(Any, x: int64, a$ Any) -> Any]",
  "?[(Any, x: int64, a$ Any) -> Any$",
  "?[(Any, x: int64, a$ Any) -> Any$",
  "10 $ (Any, x: int64, a$ Any) -> Any",
  "?10 $ (Any, x: int64, a$ Any) -> Any",
  "?(10 $ (Any, x: int64, a$ Any) -> Any)",
  "$10 $ (Any, x: int64, a$ Any) -> Any",
  "=10 $ (Any, x: int64, a$ Any) -> Any",
  "$[10 $ (Any, x: int64, a$ Any) -> Any]",
  "xxx[10 $ (Any, x: int64, a$ Any) -> Any]",
  "?$10 $ (Any, x: int64, a$ Any) -> Any]",
  "?=10 $ (Any, x: int64, a$ Any) -> Any]",
  "?[10 $ (Any, x: int64, a$ Any) -> Any$",
  "?[10 $ (Any, x: int64, a$ Any) -> Any$",
  "10 = (Any, x: int64, a$ Any) -> Any",
  "?10 = (Any, x: int64, a$ Any) -> Any",
  "?(10 = (Any, x: int64, a$ Any) -> Any)",
  "$10 = (Any, x: int64, a$ Any) -> Any",
  "=10 = (Any, x: int64, a$ Any) -> Any",
  "$[10 = (Any, x: int64, a$ Any) -> Any]",
  "xxx[10 = (Any, x: int64, a$ Any) -> Any]",
  "?$10 = (Any, x: int64, a$ Any) -> Any]",
  "?=10 = (Any, x: int64, a$ Any) -> Any]",
  "?[10 = (Any, x: int64, a$ Any) -> Any$",
  "?[10 = (Any, x: int64, a$ Any) -> Any$",
  "10 * (Any, x: int64$ a$ Any) -> Any",
  "?10 * (Any, x: int64$ a$ Any) -> Any",
  "?(10 * (Any, x: int64$ a$ Any) -> Any)",
  "$10 * (Any, x: int64$ a$ Any) -> Any",
  "=10 * (Any, x: int64$ a$ Any) -> Any",
  "$[10 * (Any, x: int64$ a$ Any) -> Any]",
  "xxx[10 * (Any, x: int64$ a$ Any) -> Any]",
  "?$10 * (Any, x: int64$ a$ Any) -> Any]",
  "?=10 * (Any, x: int64$ a$ Any) -> Any]",
  "?[10 * (Any, x: int64$ a$ Any) -> Any$",
  "?[10 * (Any, x: int64$ a$ Any) -> Any$",
  "(Any, x: int64$ a$ Any) -> Any",
  "?(Any, x: int64$ a$ Any) -> Any",
  "?((Any, x: int64$ a$ Any) -> Any)",
  "$(Any, x: int64$ a$ Any) -> Any",
  "=(Any, x: int64$ a$ Any) -> Any",
  "$[(Any, x: int64$ a$ Any) -> Any]",
  "xxx[(Any, x: int64$ a$ Any) -> Any]",
  "?$(Any, x: int64$ a$ Any) -> Any]",
  "?=(Any, x: int64$ a$ Any) -> Any]",
  "?[(Any, x: int64$ a$ Any) -> Any$",
  "?[(Any, x: int64$ a$ Any) -> Any$",
  "10 $ (Any, x: int64$ a$ Any) -> Any",
  "?10 $ (Any, x: int64$ a$ Any) -> Any",
  "?(10 $ (Any, x: int64$ a$ Any) -> Any)",
  "$10 $ (Any, x: int64$ a$ Any) -> Any",
  "=10 $ (Any, x: int64$ a$ Any) -> Any",
  "$[10 $ (Any, x: int64$ a$ Any) -> Any]",
  "xxx[10 $ (Any, x: int64$ a$ Any) -> Any]",
  "?$10 $ (Any, x: int64$ a$ Any) -> Any]",
  "?=10 $ (Any, x: int64$ a$ Any) -> Any]",
  "?[10 $ (Any, x: int64$ a$ Any) -> Any$",
  "?[10 $ (Any, x: int64$ a$ Any) -> Any$",
  "10 = (Any, x: int64$ a$ Any) -> Any",
  "?10 = (Any, x: int64$ a$ Any) -> Any",
  "?(10 = (Any, x: int64$ a$ Any) -> Any)",
  "$10 = (Any, x: int64$ a$ Any) -> Any",
  "=10 = (Any, x: int64$ a$ Any) -> Any",
  "$[10 = (Any, x: int64$ a$ Any) -> Any]",
  "xxx[10 = (Any, x: int64$ a$ Any) -> Any]",
  "?$10 = (Any, x: int64$ a$ Any) -> Any]",
  "?=10 = (Any, x: int64$ a$ Any) -> Any]",
  "?[10 = (Any, x: int64$ a$ Any) -> Any$",
  "?[10 = (Any, x: int64$ a$ Any) -> Any$",
  "10 * (Any, x: int64= a$ Any) -> Any",
  "?10 * (Any, x: int64= a$ Any) -> Any",
  "?(10 * (Any, x: int64= a$ Any) -> Any)",
  "$10 * (Any, x: int64= a$ Any) -> Any",
  "=10 * (Any, x: int64= a$ Any) -> Any",
  "$[10 * (Any, x: int64= a$ Any) -> Any]",
  "xxx[10 * (Any, x: int64= a$ Any) -> Any]",
  "?$10 * (Any, x: int64= a$ Any) -> Any]",
  "?=10 * (Any, x: int64= a$ Any) -> Any]",
  "?[10 * (Any, x: int64= a$ Any) -> Any$",
  "?[10 * (Any, x: int64= a$ Any) -> Any$",
  "(Any, x: int64= a$ Any) -> Any",
  "?(Any, x: int64= a$ Any) -> Any",
  "?((Any, x: int64= a$ Any) -> Any)",
  "$(Any, x: int64= a$ Any) -> Any",
  "=(Any, x: int64= a$ Any) -> Any",
  "$[(Any, x: int64= a$ Any) -> Any]",
  "xxx[(Any, x: int64= a$ Any) -> Any]",
  "?$(Any, x: int64= a$ Any) -> Any]",
  "?=(Any, x: int64= a$ Any) -> Any]",
  "?[(Any, x: int64= a$ Any) -> Any$",
  "?[(Any, x: int64= a$ Any) -> Any$",
  "10 $ (Any, x: int64= a$ Any) -> Any",
  "?10 $ (Any, x: int64= a$ Any) -> Any",
  "?(10 $ (Any, x: int64= a$ Any) -> Any)",
  "$10 $ (Any, x: int64= a$ Any) -> Any",
  "=10 $ (Any, x: int64= a$ Any) -> Any",
  "$[10 $ (Any, x: int64= a$ Any) -> Any]",
  "xxx[10 $ (Any, x: int64= a$ Any) -> Any]",
  "?$10 $ (Any, x: int64= a$ Any) -> Any]",
  "?=10 $ (Any, x: int64= a$ Any) -> Any]",
  "?[10 $ (Any, x: int64= a$ Any) -> Any$",
  "?[10 $ (Any, x: int64= a$ Any) -> Any$",
  "10 = (Any, x: int64= a$ Any) -> Any",
  "?10 = (Any, x: int64= a$ Any) -> Any",
  "?(10 = (Any, x: int64= a$ Any) -> Any)",
  "$10 = (Any, x: int64= a$ Any) -> Any",
  "=10 = (Any, x: int64= a$ Any) -> Any",
  "$[10 = (Any, x: int64= a$ Any) -> Any]",
  "xxx[10 = (Any, x: int64= a$ Any) -> Any]",
  "?$10 = (Any, x: int64= a$ Any) -> Any]",
  "?=10 = (Any, x: int64= a$ Any) -> Any]",
  "?[10 = (Any, x: int64= a$ Any) -> Any$",
  "?[10 = (Any, x: int64= a$ Any) -> Any$",
  "10 * (Any, x: int64, x: int64, a$ Any) -> Any",
  "?10 * (Any, x: int64, x: int64, a$ Any) -> Any",
  "?(10 * (Any, x: int64, x: int64, a$ Any) -> Any)",
  "$10 * (Any, x: int64, x: int64, a$ Any) -> Any",
  "=10 * (Any, x: int64, x: int64, a$ Any) -> Any",
  "$[10 * (Any, x: int64, x: int64, a$ Any) -> Any]",
  "xxx[10 * (Any, x: int64, x: int64, a$ Any) -> Any]",
  "?$10 * (Any, x: int64, x: int64, a$ Any) -> Any]",
  "?=10 * (Any, x: int64, x: int64, a$ Any) -> Any]",
  "?[10 * (Any, x: int64, x: int64, a$ Any) -> Any$",
  "?[10 * (Any, x: int64, x: int64, a$ Any) -> Any$",
  "(Any, x: int64, x: int64, a$ Any) -> Any",
  "?(Any, x: int64, x: int64, a$ Any) -> Any",
  "?((Any, x: int64, x: int64, a$ Any) -> Any)",
  "$(Any, x: int64, x: int64, a$ Any) -> Any",
  "=(Any, x: int64, x: int64, a$ Any) -> Any",
  "$[(Any, x: int64, x: int64, a$ Any) -> Any]",
  "xxx[(Any, x: int64, x: int64, a$ Any) -> Any]",
  "?$(Any, x: int64, x: int64, a$ Any) -> Any]",
  "?=(Any, x: int64, x: int64, a$ Any) -> Any]",
  "?[(Any, x: int64, x: int64, a$ Any) -> Any$",
  "?[(Any, x: int64, x: int64, a$ Any) -> Any$",
  "10 $ (Any, x: int64, x: int64, a$ Any) -> Any",
  "?10 $ (Any, x: int64, x: int64, a$ Any) -> Any",
  "?(10 $ (Any, x: int64, x: int64, a$ Any) -> Any)",
  "$10 $ (Any, x: int64, x: int64, a$ Any) -> Any",
  "=10 $ (Any, x: int64, x: int64, a$ Any) -> Any",
  "$[10 $ (Any, x: int64, x: int64, a$ Any) -> Any]",
  "xxx[10 $ (Any, x: int64, x: int64, a$ Any) -> Any]",
  "?$10 $ (Any, x: int64, x: int64, a$ Any) -> Any]",
  "?=10 $ (Any, x: int64, x: int64, a$ Any) -> Any]",
  "?[10 $ (Any, x: int64, x: int64, a$ Any) -> Any$",
  "?[10 $ (Any, x: int64, x: int64, a$ Any) -> Any$",
  "10 = (Any, x: int64, x: int64, a$ Any) -> Any",
  "?10 = (Any, x: int64, x: int64, a$ Any) -> Any",
  "?(10 = (Any, x: int64, x: int64, a$ Any) -> Any)",
  "$10 = (Any, x: int64, x: int64, a$ Any) -> Any",
  "=10 = (Any, x: int64, x: int64, a$ Any) -> Any",
  "$[10 = (Any, x: int64, x: int64, a$ Any) -> Any]",
  "xxx[10 = (Any, x: int64, x: int64, a$ Any) -> Any]",
  "?$10 = (Any, x: int64, x: int64, a$ Any) -> Any]",
  "?=10 = (Any, x: int64, x: int64, a$ Any) -> Any]",
  "?[10 = (Any, x: int64, x: int64, a$ Any) -> Any$",
  "?[10 = (Any, x: int64, x: int64, a$ Any) -> Any$",
  "10 * (Any, x: int64, x: int64$ a$ Any) -> Any",
  "?10 * (Any, x: int64, x: int64$ a$ Any) -> Any",
  "?(10 * (Any, x: int64, x: int64$ a$ Any) -> Any)",
  "$10 * (Any, x: int64, x: int64$ a$ Any) -> Any",
  "=10 * (Any, x: int64, x: int64$ a$ Any) -> Any",
  "$[10 * (Any, x: int64, x: int64$ a$ Any) -> Any]",
  "xxx[10 * (Any, x: int64, x: int64$ a$ Any) -> Any]",
  "?$10 * (Any, x: int64, x: int64$ a$ Any) -> Any]",
  "?=10 * (Any, x: int64, x: int64$ a$ Any) -> Any]",
  "?[10 * (Any, x: int64, x: int64$ a$ Any) -> Any$",
  "?[10 * (Any, x: int64, x: int64$ a$ Any) -> Any$",
  "(Any, x: int64, x: int64$ a$ Any) -> Any",
  "?(Any, x: int64, x: int64$ a$ Any) -> Any",
  "?((Any, x: int64, x: int64$ a$ Any) -> Any)",
  "$(Any, x: int64, x: int64$ a$ Any) -> Any",
  "=(Any, x: int64, x: int64$ a$ Any) -> Any",
  "$[(Any, x: int64, x: int64$ a$ Any) -> Any]",
  "xxx[(Any, x: int64, x: int64$ a$ Any) -> Any]",
  "?$(Any, x: int64, x: int64$ a$ Any) -> Any]",
  "?=(Any, x: int64, x: int64$ a$ Any) -> Any]",
  "?[(Any, x: int64, x: int64$ a$ Any) -> Any$",
  "?[(Any, x: int64, x: int64$ a$ Any) -> Any$",
  "10 $ (Any, x: int64, x: int64$ a$ Any) -> Any",
  "?10 $ (Any, x: int64, x: int64$ a$ Any) -> Any",
  "?(10 $ (Any, x: int64, x: int64$ a$ Any) -> Any)",
  "$10 $ (Any, x: int64, x: int64$ a$ Any) -> Any",
  "=10 $ (Any, x: int64, x: int64$ a$ Any) -> Any",
  "$[10 $ (Any, x: int64, x: int64$ a$ Any) -> Any]",
  "xxx[10 $ (Any, x: int64, x: int64$ a$ Any) -> Any]",
  "?$10 $ (Any, x: int64, x: int64$ a$ Any) -> Any]",
  "?=10 $ (Any, x: int64, x: int64$ a$ Any) -> Any]",
  "?[10 $ (Any, x: int64, x: int64$ a$ Any) -> Any$",
  "?[10 $ (Any, x: int64, x: int64$ a$ Any) -> Any$",
  "10 = (Any, x: int64, x: int64$ a$ Any) -> Any",
  "?10 = (Any, x: int64, x: int64$ a$ Any) -> Any",
  "?(10 = (Any, x: int64, x: int64$ a$ Any) -> Any)",
  "$10 = (Any, x: int64, x: int64$ a$ Any) -> Any",
  "=10 = (Any, x: int64, x: int64$ a$ Any) -> Any",
  "$[10 = (Any, x: int64, x: int64$ a$ Any) -> Any]",
  "xxx[10 = (Any, x: int64, x: int64$ a$ Any) -> Any]",
  "?$10 = (Any, x: int64, x: int64$ a$ Any) -> Any]",
  "?=10 = (Any, x: int64, x: int64$ a$ Any) -> Any]",
  "?[10 = (Any, x: int64, x: int64$ a$ Any) -> Any$",
  "?[10 = (Any, x: int64, x: int64$ a$ Any) -> Any$",
  "10 * (Any, x: int64, x: int64= a$ Any) -> Any",
  "?10 * (Any, x: int64, x: int64= a$ Any) -> Any",
  "?(10 * (Any, x: int64, x: int64= a$ Any) -> Any)",
  "$10 * (Any, x: int64, x: int64= a$ Any) -> Any",
  "=10 * (Any, x: int64, x: int64= a$ Any) -> Any",
  "$[10 * (Any, x: int64, x: int64= a$ Any) -> Any]",
  "xxx[10 * (Any, x: int64, x: int64= a$ Any) -> Any]",
  "?$10 * (Any, x: int64, x: int64= a$ Any) -> Any]",
  "?=10 * (Any, x: int64, x: int64= a$ Any) -> Any]",
  "?[10 * (Any, x: int64, x: int64= a$ Any) -> Any$",
  "?[10 * (Any, x: int64, x: int64= a$ Any) -> Any$",
  "(Any, x: int64, x: int64= a$ Any) -> Any",
  "?(Any, x: int64, x: int64= a$ Any) -> Any",
  "?((Any, x: int64, x: int64= a$ Any) -> Any)",
  "$(Any, x: int64, x: int64= a$ Any) -> Any",
  "=(Any, x: int64, x: int64= a$ Any) -> Any",
  "$[(Any, x: int64, x: int64= a$ Any) -> Any]",
  "xxx[(Any, x: int64, x: int64= a$ Any) -> Any]",
  "?$(Any, x: int64, x: int64= a$ Any) -> Any]",
  "?=(Any, x: int64, x: int64= a$ Any) -> Any]",
  "?[(Any, x: int64, x: int64= a$ Any) -> Any$",
  "?[(Any, x: int64, x: int64= a$ Any) -> Any$",
  "10 $ (Any, x: int64, x: int64= a$ Any) -> Any",
  "?10 $ (Any, x: int64, x: int64= a$ Any) -> Any",
  "?(10 $ (Any, x: int64, x: int64= a$ Any) -> Any)",
  "$10 $ (Any, x: int64, x: int64= a$ Any) -> Any",
  "=10 $ (Any, x: int64, x: int64= a$ Any) -> Any",
  "$[10 $ (Any, x: int64, x: int64= a$ Any) -> Any]",
  "xxx[10 $ (Any, x: int64, x: int64= a$ Any) -> Any]",
  "?$10 $ (Any, x: int64, x: int64= a$ Any) -> Any]",
  "?=10 $ (Any, x: int64, x: int64= a$ Any) -> Any]",
  "?[10 $ (Any, x: int64, x: int64= a$ Any) -> Any$",
  "?[10 $ (Any, x: int64, x: int64= a$ Any) -> Any$",
  "10 = (Any, x: int64, x: int64= a$ Any) -> Any",
  "?10 = (Any, x: int64, x: int64= a$ Any) -> Any",
  "?(10 = (Any, x: int64, x: int64= a$ Any) -> Any)",
  "$10 = (Any, x: int64, x: int64= a$ Any) -> Any",
  "=10 = (Any, x: int64, x: int64= a$ Any) -> Any",
  "$[10 = (Any, x: int64, x: int64= a$ Any) -> Any]",
  "xxx[10 = (Any, x: int64, x: int64= a$ Any) -> Any]",
  "?$10 = (Any, x: int64, x: int64= a$ Any) -> Any]",
  "?=10 = (Any, x: int64, x: int64= a$ Any) -> Any]",
  "?[10 = (Any, x: int64, x: int64= a$ Any) -> Any$",
  "?[10 = (Any, x: int64, x: int64= a$ Any) -> Any$",
  "10 * (Any, a: $) -> Any",
  "?10 * (Any, a: $) -> Any",
  "?(10 * (Any, a: $) -> Any)",
  "$10 * (Any, a: $) -> Any",
  "=10 * (Any, a: $) -> Any",
  "$[10 * (Any, a: $) -> Any]",
  "xxx[10 * (Any, a: $) -> Any]",
  "?$10 * (Any, a: $) -> Any]",
  "?=10 * (Any, a: $) -> Any]",
  "?[10 * (Any, a: $) -> Any$",
  "?[10 * (Any, a: $) -> Any$",
  "(Any, a: $) -> Any",
  "?(Any, a: $) -> Any",
  "?((Any, a: $) -> Any)",
  "$(Any, a: $) -> Any",
  "=(Any, a: $) -> Any",
  "$[(Any, a: $) -> Any]",
  "xxx[(Any, a: $) -> Any]",
  "?$(Any, a: $) -> Any]",
  "?=(Any, a: $) -> Any]",
  "?[(Any, a: $) -> Any$",
  "?[(Any, a: $) -> Any$",
  "10 $ (Any, a: $) -> Any",
  "?10 $ (Any, a: $) -> Any",
  "?(10 $ (Any, a: $) -> Any)",
  "$10 $ (Any, a: $) -> Any",
  "=10 $ (Any, a: $) -> Any",
  "$[10 $ (Any, a: $) -> Any]",
  "xxx[10 $ (Any, a: $) -> Any]",
  "?$10 $ (Any, a: $) -> Any]",
  "?=10 $ (Any, a: $) -> Any]",
  "?[10 $ (Any, a: $) -> Any$",
  "?[10 $ (Any, a: $) -> Any$",
  "10 = (Any, a: $) -> Any",
  "?10 = (Any, a: $) -> Any",
  "?(10 = (Any, a: $) -> Any)",
  "$10 = (Any, a: $) -> Any",
  "=10 = (Any, a: $) -> Any",
  "$[10 = (Any, a: $) -> Any]",
  "xxx[10 = (Any, a: $) -> Any]",
  "?$10 = (Any, a: $) -> Any]",
  "?=10 = (Any, a: $) -> Any]",
  "?[10 = (Any, a: $) -> Any$",
  "?[10 = (Any, a: $) -> Any$",
  "10 * (Any, x: int64, a: $) -> Any",
  "?10 * (Any, x: int64, a: $) -> Any",
  "?(10 * (Any, x: int64, a: $) -> Any)",
  "$10 * (Any, x: int64, a: $) -> Any",
  "=10 * (Any, x: int64, a: $) -> Any",
  "$[10 * (Any, x: int64, a: $) -> Any]",
  "xxx[10 * (Any, x: int64, a: $) -> Any]",
  "?$10 * (Any, x: int64, a: $) -> Any]",
  "?=10 * (Any, x: int64, a: $) -> Any]",
  "?[10 * (Any, x: int64, a: $) -> Any$",
  "?[10 * (Any, x: int64, a: $) -> Any$",
  "(Any, x: int64, a: $) -> Any",
  "?(Any, x: int64, a: $) -> Any",
  "?((Any, x: int64, a: $) -> Any)",
  "$(Any, x: int64, a: $) -> Any",
  "=(Any, x: int64, a: $) -> Any",
  "$[(Any, x: int64, a: $) -> Any]",
  "xxx[(Any, x: int64, a: $) -> Any]",
  "?$(Any, x: int64, a: $) -> Any]",
  "?=(Any, x: int64, a: $) -> Any]",
  "?[(Any, x: int64, a: $) -> Any$",
  "?[(Any, x: int64, a: $) -> Any$",
  "10 $ (Any, x: int64, a: $) -> Any",
  "?10 $ (Any, x: int64, a: $) -> Any",
  "?(10 $ (Any, x: int64, a: $) -> Any)",
  "$10 $ (Any, x: int64, a: $) -> Any",
  "=10 $ (Any, x: int64, a: $) -> Any",
  "$[10 $ (Any, x: int64, a: $) -> Any]",
  "xxx[10 $ (Any, x: int64, a: $) -> Any]",
  "?$10 $ (Any, x: int64, a: $) -> Any]",
  "?=10 $ (Any, x: int64, a: $) -> Any]",
  "?[10 $ (Any, x: int64, a: $) -> Any$",
  "?[10 $ (Any, x: int64, a: $) -> Any$",
  "10 = (Any, x: int64, a: $) -> Any",
  "?10 = (Any, x: int64, a: $) -> Any",
  "?(10 = (Any, x: int64, a: $) -> Any)",
  "$10 = (Any, x: int64, a: $) -> Any",
  "=10 = (Any, x: int64, a: $) -> Any",
  "$[10 = (Any, x: int64, a: $) -> Any]",
  "xxx[10 = (Any, x: int64, a: $) -> Any]",
  "?$10 = (Any, x: int64, a: $) -> Any]",
  "?=10 = (Any, x: int64, a: $) -> Any]",
  "?[10 = (Any, x: int64, a: $) -> Any$",
  "?[10 = (Any, x: int64, a: $) -> Any$",
  "10 * (Any, x: int64$ a: $) -> Any",
  "?10 * (Any, x: int64$ a: $) -> Any",
  "?(10 * (Any, x: int64$ a: $) -> Any)",
  "$10 * (Any, x: int64$ a: $) -> Any",
  "=10 * (Any, x: int64$ a: $) -> Any",
  "$[10 * (Any, x: int64$ a: $) -> Any]",
  "xxx[10 * (Any, x: int64$ a: $) -> Any]",
  "?$10 * (Any, x: int64$ a: $) -> Any]",
  "?=10 * (Any, x: int64$ a: $) -> Any]",
  "?[10 * (Any, x: int64$ a: $) -> Any$",
  "?[10 * (Any, x: int64$ a: $) -> Any$",
  "(Any, x: int64$ a: $) -> Any",
  "?(Any, x: int64$ a: $) -> Any",
  "?((Any, x: int64$ a: $) -> Any)",
  "$(Any, x: int64$ a: $) -> Any",
  "=(Any, x: int64$ a: $) -> Any",
  "$[(Any, x: int64$ a: $) -> Any]",
  "xxx[(Any, x: int64$ a: $) -> Any]",
  "?$(Any, x: int64$ a: $) -> Any]",
  "?=(Any, x: int64$ a: $) -> Any]",
  "?[(Any, x: int64$ a: $) -> Any$",
  "?[(Any, x: int64$ a: $) -> Any$",
  "10 $ (Any, x: int64$ a: $) -> Any",
  "?10 $ (Any, x: int64$ a: $) -> Any",
  "?(10 $ (Any, x: int64$ a: $) -> Any)",
  "$10 $ (Any, x: int64$ a: $) -> Any",
  "=10 $ (Any, x: int64$ a: $) -> Any",
  "$[10 $ (Any, x: int64$ a: $) -> Any]",
  "xxx[10 $ (Any, x: int64$ a: $) -> Any]",
  "?$10 $ (Any, x: int64$ a: $) -> Any]",
  "?=10 $ (Any, x: int64$ a: $) -> Any]",
  "?[10 $ (Any, x: int64$ a: $) -> Any$",
  "?[10 $ (Any, x: int64$ a: $) -> Any$",
  "10 = (Any, x: int64$ a: $) -> Any",
  "?10 = (Any, x: int64$ a: $) -> Any",
  "?(10 = (Any, x: int64$ a: $) -> Any)",
  "$10 = (Any, x: int64$ a: $) -> Any",
  "=10 = (Any, x: int64$ a: $) -> Any",
  "$[10 = (Any, x: int64$ a: $) -> Any]",
  "xxx[10 = (Any, x: int64$ a: $) -> Any]",
  "?$10 = (Any, x: int64$ a: $) -> Any]",
  "?=10 = (Any, x: int64$ a: $) -> Any]",
  "?[10 = (Any, x: int64$ a: $) -> Any$",
  "?[10 = (Any, x: int64$ a: $) -> Any$",
  "10 * (Any, x: int64= a: $) -> Any",
  "?10 * (Any, x: int64= a: $) -> Any",
  "?(10 * (Any, x: int64= a: $) -> Any)",
  "$10 * (Any, x: int64= a: $) -> Any",
  "=10 * (Any, x: int64= a: $) -> Any",
  "$[10 * (Any, x: int64= a: $) -> Any]",
  "xxx[10 * (Any, x: int64= a: $) -> Any]",
  "?$10 * (Any, x: int64= a: $) -> Any]",
  "?=10 * (Any, x: int64= a: $) -> Any]",
  "?[10 * (Any, x: int64= a: $) -> Any$",
  "?[10 * (Any, x: int64= a: $) -> Any$",
  "(Any, x: int64= a: $) -> Any",
  "?(Any, x: int64= a: $) -> Any",
  "?((Any, x: int64= a: $) -> Any)",
  "$(Any, x: int64= a: $) -> Any",
  "=(Any, x: int64= a: $) -> Any",
  "$[(Any, x: int64= a: $) -> Any]",
  "xxx[(Any, x: int64= a: $) -> Any]",
  "?$(Any, x: int64= a: $) -> Any]",
  "?=(Any, x: int64= a: $) -> Any]",
  "?[(Any, x: int64= a: $) -> Any$",
  "?[(Any, x: int64= a: $) -> Any$",
  "10 $ (Any, x: int64= a: $) -> Any",
  "?10 $ (Any, x: int64= a: $) -> Any",
  "?(10 $ (Any, x: int64= a: $) -> Any)",
  "$10 $ (Any, x: int64= a: $) -> Any",
  "=10 $ (Any, x: int64= a: $) -> Any",
  "$[10 $ (Any, x: int64= a: $) -> Any]",
  "xxx[10 $ (Any, x: int64= a: $) -> Any]",
  "?$10 $ (Any, x: int64= a: $) -> Any]",
  "?=10 $ (Any, x: int64= a: $) -> Any]",
  "?[10 $ (Any, x: int64= a: $) -> Any$",
  "?[10 $ (Any, x: int64= a: $) -> Any$",
  "10 = (Any, x: int64= a: $) -> Any",
  "?10 = (Any, x: int64= a: $) -> Any",
  "?(10 = (Any, x: int64= a: $) -> Any)",
  "$10 = (Any, x: int64= a: $) -> Any",
  "=10 = (Any, x: int64= a: $) -> Any",
  "$[10 = (Any, x: int64= a: $) -> Any]",
  "xxx[10 = (Any, x: int64= a: $) -> Any]",
  "?$10 = (Any, x: int64= a: $) -> Any]",
  "?=10 = (Any, x: int64= a: $) -> Any]",
  "?[10 = (Any, x: int64= a: $) -> Any$",
  "?[10 = (Any, x: int64= a: $) -> Any$",
  "10 * (Any, x: int64, x: int64, a: $) -> Any",
  "?10 * (Any, x: int64, x: int64, a: $) -> Any",
  "?(10 * (Any, x: int64, x: int64, a: $) -> Any)",
  "$10 * (Any, x: int64, x: int64, a: $) -> Any",
  "=10 * (Any, x: int64, x: int64, a: $) -> Any",
  "$[10 * (Any, x: int64, x: int64, a: $) -> Any]",
  "xxx[10 * (Any, x: int64, x: int64, a: $) -> Any]",
  "?$10 * (Any, x: int64, x: int64, a: $) -> Any]",
  "?=10 * (Any, x: int64, x: int64, a: $) -> Any]",
  "?[10 * (Any, x: int64, x: int64, a: $) -> Any$",
  "?[10 * (Any, x: int64, x: int64, a: $) -> Any$",
  "(Any, x: int64, x: int64, a: $) -> Any",
  "?(Any, x: int64, x: int64, a: $) -> Any",
  "?((Any, x: int64, x: int64, a: $) -> Any)",
  "$(Any, x: int64, x: int64, a: $) -> Any",
  "=(Any, x: int64, x: int64, a: $) -> Any",
  "$[(Any, x: int64, x: int64, a: $) -> Any]",
  "xxx[(Any, x: int64, x: int64, a: $) -> Any]",
  "?$(Any, x: int64, x: int64, a: $) -> Any]",
  "?=(Any, x: int64, x: int64, a: $) -> Any]",
  "?[(Any, x: int64, x: int64, a: $) -> Any$",
  "?[(Any, x: int64, x: int64, a: $) -> Any$",
  "10 $ (Any, x: int64, x: int64, a: $) -> Any",
  "?10 $ (Any, x: int64, x: int64, a: $) -> Any",
  "?(10 $ (Any, x: int64, x: int64, a: $) -> Any)",
  "$10 $ (Any, x: int64, x: int64, a: $) -> Any",
  "=10 $ (Any, x: int64, x: int64, a: $) -> Any",
  "$[10 $ (Any, x: int64, x: int64, a: $) -> Any]",
  "xxx[10 $ (Any, x: int64, x: int64, a: $) -> Any]",
  "?$10 $ (Any, x: int64, x: int64, a: $) -> Any]",
  "?=10 $ (Any, x: int64, x: int64, a: $) -> Any]",
  "?[10 $ (Any, x: int64, x: int64, a: $) -> Any$",
  "?[10 $ (Any, x: int64, x: int64, a: $) -> Any$",
  "10 = (Any, x: int64, x: int64, a: $) -> Any",
  "?10 = (Any, x: int64, x: int64, a: $) -> Any",
  "?(10 = (Any, x: int64, x: int64, a: $) -> Any)",
  "$10 = (Any, x: int64, x: int64, a: $) -> Any",
  "=10 = (Any, x: int64, x: int64, a: $) -> Any",
  "$[10 = (Any, x: int64, x: int64, a: $) -> Any]",
  "xxx[10 = (Any, x: int64, x: int64, a: $) -> Any]",
  "?$10 = (Any, x: int64, x: int64, a: $) -> Any]",
  "?=10 = (Any, x: int64, x: int64, a: $) -> Any]",
  "?[10 = (Any, x: int64, x: int64, a: $) -> Any$",
  "?[10 = (Any, x: int64, x: int64, a: $) -> Any$",
  "10 * (Any, x: int64, x: int64$ a: $) -> Any",
  "?10 * (Any, x: int64, x: int64$ a: $) -> Any",
  "?(10 * (Any, x: int64, x: int64$ a: $) -> Any)",
  "$10 * (Any, x: int64, x: int64$ a: $) -> Any",
  "=10 * (Any, x: int64, x: int64$ a: $) -> Any",
  "$[10 * (Any, x: int64, x: int64$ a: $) -> Any]",
  "xxx[10 * (Any, x: int64, x: int64$ a: $) -> Any]",
  "?$10 * (Any, x: int64, x: int64$ a: $) -> Any]",
  "?=10 * (Any, x: int64, x: int64$ a: $) -> Any]",
  "?[10 * (Any, x: int64, x: int64$ a: $) -> Any$",
  "?[10 * (Any, x: int64, x: int64$ a: $) -> Any$",
  "(Any, x: int64, x: int64$ a: $) -> Any",
  "?(Any, x: int64, x: int64$ a: $) -> Any",
  "?((Any, x: int64, x: int64$ a: $) -> Any)",
  "$(Any, x: int64, x: int64$ a: $) -> Any",
  "=(Any, x: int64, x: int64$ a: $) -> Any",
  "$[(Any, x: int64, x: int64$ a: $) -> Any]",
  "xxx[(Any, x: int64, x: int64$ a: $) -> Any]",
  "?$(Any, x: int64, x: int64$ a: $) -> Any]",
  "?=(Any, x: int64, x: int64$ a: $) -> Any]",
  "?[(Any, x: int64, x: int64$ a: $) -> Any$",
  "?[(Any, x: int64, x: int64$ a: $) -> Any$",
  "10 $ (Any, x: int64, x: int64$ a: $) -> Any",
  "?10 $ (Any, x: int64, x: int64$ a: $) -> Any",
  "?(10 $ (Any, x: int64, x: int64$ a: $) -> Any)",
  "$10 $ (Any, x: int64, x: int64$ a: $) -> Any",
  "=10 $ (Any, x: int64, x: int64$ a: $) -> Any",
  "$[10 $ (Any, x: int64, x: int64$ a: $) -> Any]",
  "xxx[10 $ (Any, x: int64, x: int64$ a: $) -> Any]",
  "?$10 $ (Any, x: int64, x: int64$ a: $) -> Any]",
  "?=10 $ (Any, x: int64, x: int64$ a: $) -> Any]",
  "?[10 $ (Any, x: int64, x: int64$ a: $) -> Any$",
  "?[10 $ (Any, x: int64, x: int64$ a: $) -> Any$",
  "10 = (Any, x: int64, x: int64$ a: $) -> Any",
  "?10 = (Any, x: int64, x: int64$ a: $) -> Any",
  "?(10 = (Any, x: int64, x: int64$ a: $) -> Any)",
  "$10 = (Any, x: int64, x: int64$ a: $) -> Any",
  "=10 = (Any, x: int64, x: int64$ a: $) -> Any",
  "$[10 = (Any, x: int64, x: int64$ a: $) -> Any]",
  "xxx[10 = (Any, x: int64, x: int64$ a: $) -> Any]",
  "?$10 = (Any, x: int64, x: int64$ a: $) -> Any]",
  "?=10 = (Any, x: int64, x: int64$ a: $) -> Any]",
  "?[10 = (Any, x: int64, x: int64$ a: $) -> Any$",
  "?[10 = (Any, x: int64, x: int64$ a: $) -> Any$",
  "10 * (Any, x: int64, x: int64= a: $) -> Any",
  "?10 * (Any, x: int64, x: int64= a: $) -> Any",
  "?(10 * (Any, x: int64, x: int64= a: $) -> Any)",
  "$10 * (Any, x: int64, x: int64= a: $) -> Any",
  "=10 * (Any, x: int64, x: int64= a: $) -> Any",
  "$[10 * (Any, x: int64, x: int64= a: $) -> Any]",
  "xxx[10 * (Any, x: int64, x: int64= a: $) -> Any]",
  "?$10 * (Any, x: int64, x: int64= a: $) -> Any]",
  "?=10 * (Any, x: int64, x: int64= a: $) -> Any]",
  "?[10 * (Any, x: int64, x: int64= a: $) -> Any$",
  "?[10 * (Any, x: int64, x: int64= a: $) -> Any$",
  "(Any, x: int64, x: int64= a: $) -> Any",
  "?(Any, x: int64, x: int64= a: $) -> Any",
  "?((Any, x: int64, x: int64= a: $) -> Any)",
  "$(Any, x: int64, x: int64= a: $) -> Any",
  "=(Any, x: int64, x: int64= a: $) -> Any",
  "$[(Any, x: int64, x: int64= a: $) -> Any]",
  "xxx[(Any, x: int64, x: int64= a: $) -> Any]",
  "?$(Any, x: int64, x: int64= a: $) -> Any]",
  "?=(Any, x: int64, x: int64= a: $) -> Any]",
  "?[(Any, x: int64, x: int64= a: $) -> Any$",
  "?[(Any, x: int64, x: int64= a: $) -> Any$",
  "10 $ (Any, x: int64, x: int64= a: $) -> Any",
  "?10 $ (Any, x: int64, x: int64= a: $) -> Any",
  "?(10 $ (Any, x: int64, x: int64= a: $) -> Any)",
  "$10 $ (Any, x: int64, x: int64= a: $) -> Any",
  "=10 $ (Any, x: int64, x: int64= a: $) -> Any",
  "$[10 $ (Any, x: int64, x: int64= a: $) -> Any]",
  "xxx[10 $ (Any, x: int64, x: int64= a: $) -> Any]",
  "?$10 $ (Any, x: int64, x: int64= a: $) -> Any]",
  "?=10 $ (Any, x: int64, x: int64= a: $) -> Any]",
  "?[10 $ (Any, x: int64, x: int64= a: $) -> Any$",
  "?[10 $ (Any, x: int64, x: int64= a: $) -> Any$",
  "10 = (Any, x: int64, x: int64= a: $) -> Any",
  "?10 = (Any, x: int64, x: int64= a: $) -> Any",
  "?(10 = (Any, x: int64, x: int64= a: $) -> Any)",
  "$10 = (Any, x: int64, x: int64= a: $) -> Any",
  "=10 = (Any, x: int64, x: int64= a: $) -> Any",
  "$[10 = (Any, x: int64, x: int64= a: $) -> Any]",
  "xxx[10 = (Any, x: int64, x: int64= a: $) -> Any]",
  "?$10 = (Any, x: int64, x: int64= a: $) -> Any]",
  "?=10 = (Any, x: int64, x: int64= a: $) -> Any]",
  "?[10 = (Any, x: int64, x: int64= a: $) -> Any$",
  "?[10 = (Any, x: int64, x: int64= a: $) -> Any$",
  "10 * (Any, a: xxx) -> Any",
  "?10 * (Any, a: xxx) -> Any",
  "?(10 * (Any, a: xxx) -> Any)",
  "$10 * (Any, a: xxx) -> Any",
  "=10 * (Any, a: xxx) -> Any",
  "$[10 * (Any, a: xxx) -> Any]",
  "xxx[10 * (Any, a: xxx) -> Any]",
  "?$10 * (Any, a: xxx) -> Any]",
  "?=10 * (Any, a: xxx) -> Any]",
  "?[10 * (Any, a: xxx) -> Any$",
  "?[10 * (Any, a: xxx) -> Any$",
  "(Any, a: xxx) -> Any",
  "?(Any, a: xxx) -> Any",
  "?((Any, a: xxx) -> Any)",
  "$(Any, a: xxx) -> Any",
  "=(Any, a: xxx) -> Any",
  "$[(Any, a: xxx) -> Any]",
  "xxx[(Any, a: xxx) -> Any]",
  "?$(Any, a: xxx) -> Any]",
  "?=(Any, a: xxx) -> Any]",
  "?[(Any, a: xxx) -> Any$",
  "?[(Any, a: xxx) -> Any$",
  "10 $ (Any, a: xxx) -> Any",
  "?10 $ (Any, a: xxx) -> Any",
  "?(10 $ (Any, a: xxx) -> Any)",
  "$10 $ (Any, a: xxx) -> Any",
  "=10 $ (Any, a: xxx) -> Any",
  "$[10 $ (Any, a: xxx) -> Any]",
  "xxx[10 $ (Any, a: xxx) -> Any]",
  "?$10 $ (Any, a: xxx) -> Any]",
  "?=10 $ (Any, a: xxx) -> Any]",
  "?[10 $ (Any, a: xxx) -> Any$",
  "?[10 $ (Any, a: xxx) -> Any$",
  "10 = (Any, a: xxx) -> Any",
  "?10 = (Any, a: xxx) -> Any",
  "?(10 = (Any, a: xxx) -> Any)",
  "$10 = (Any, a: xxx) -> Any",
  "=10 = (Any, a: xxx) -> Any",
  "$[10 = (Any, a: xxx) -> Any]",
  "xxx[10 = (Any, a: xxx) -> Any]",
  "?$10 = (Any, a: xxx) -> Any]",
  "?=10 = (Any, a: xxx) -> Any]",
  "?[10 = (Any, a: xxx) -> Any$",
  "?[10 = (Any, a: xxx) -> Any$",
  "10 * (Any, x: int64, a: xxx) -> Any",
  "?10 * (Any, x: int64, a: xxx) -> Any",
  "?(10 * (Any, x: int64, a: xxx) -> Any)",
  "$10 * (Any, x: int64, a: xxx) -> Any",
  "=10 * (Any, x: int64, a: xxx) -> Any",
  "$[10 * (Any, x: int64, a: xxx) -> Any]",
  "xxx[10 * (Any, x: int64, a: xxx) -> Any]",
  "?$10 * (Any, x: int64, a: xxx) -> Any]",
  "?=10 * (Any, x: int64, a: xxx) -> Any]",
  "?[10 * (Any, x: int64, a: xxx) -> Any$",
  "?[10 * (Any, x: int64, a: xxx) -> Any$",
  "(Any, x: int64, a: xxx) -> Any",
  "?(Any, x: int64, a: xxx) -> Any",
  "?((Any, x: int64, a: xxx) -> Any)",
  "$(Any, x: int64, a: xxx) -> Any",
  "=(Any, x: int64, a: xxx) -> Any",
  "$[(Any, x: int64, a: xxx) -> Any]",
  "xxx[(Any, x: int64, a: xxx) -> Any]",
  "?$(Any, x: int64, a: xxx) -> Any]",
  "?=(Any, x: int64, a: xxx) -> Any]",
  "?[(Any, x: int64, a: xxx) -> Any$",
  "?[(Any, x: int64, a: xxx) -> Any$",
  "10 $ (Any, x: int64, a: xxx) -> Any",
  "?10 $ (Any, x: int64, a: xxx) -> Any",
  "?(10 $ (Any, x: int64, a: xxx) -> Any)",
  "$10 $ (Any, x: int64, a: xxx) -> Any",
  "=10 $ (Any, x: int64, a: xxx) -> Any",
  "$[10 $ (Any, x: int64, a: xxx) -> Any]",
  "xxx[10 $ (Any, x: int64, a: xxx) -> Any]",
  "?$10 $ (Any, x: int64, a: xxx) -> Any]",
  "?=10 $ (Any, x: int64, a: xxx) -> Any]",
  "?[10 $ (Any, x: int64, a: xxx) -> Any$",
  "?[10 $ (Any, x: int64, a: xxx) -> Any$",
  "10 = (Any, x: int64, a: xxx) -> Any",
  "?10 = (Any, x: int64, a: xxx) -> Any",
  "?(10 = (Any, x: int64, a: xxx) -> Any)",
  "$10 = (Any, x: int64, a: xxx) -> Any",
  "=10 = (Any, x: int64, a: xxx) -> Any",
  "$[10 = (Any, x: int64, a: xxx) -> Any]",
  "xxx[10 = (Any, x: int64, a: xxx) -> Any]",
  "?$10 = (Any, x: int64, a: xxx) -> Any]",
  "?=10 = (Any, x: int64, a: xxx) -> Any]",
  "?[10 = (Any, x: int64, a: xxx) -> Any$",
  "?[10 = (Any, x: int64, a: xxx) -> Any$",
  "10 * (Any, x: int64$ a: xxx) -> Any",
  "?10 * (Any, x: int64$ a: xxx) -> Any",
  "?(10 * (Any, x: int64$ a: xxx) -> Any)",
  "$10 * (Any, x: int64$ a: xxx) -> Any",
  "=10 * (Any, x: int64$ a: xxx) -> Any",
  "$[10 * (Any, x: int64$ a: xxx) -> Any]",
  "xxx[10 * (Any, x: int64$ a: xxx) -> Any]",
  "?$10 * (Any, x: int64$ a: xxx) -> Any]",
  "?=10 * (Any, x: int64$ a: xxx) -> Any]",
  "?[10 * (Any, x: int64$ a: xxx) -> Any$",
  "?[10 * (Any, x: int64$ a: xxx) -> Any$",
  "(Any, x: int64$ a: xxx) -> Any",
  "?(Any, x: int64$ a: xxx) -> Any",
  "?((Any, x: int64$ a: xxx) -> Any)",
  "$(Any, x: int64$ a: xxx) -> Any",
  "=(Any, x: int64$ a: xxx) -> Any",
  "$[(Any, x: int64$ a: xxx) -> Any]",
  "xxx[(Any, x: int64$ a: xxx) -> Any]",
  "?$(Any, x: int64$ a: xxx) -> Any]",
  "?=(Any, x: int64$ a: xxx) -> Any]",
  "?[(Any, x: int64$ a: xxx) -> Any$",
  "?[(Any, x: int64$ a: xxx) -> Any$",
  "10 $ (Any, x: int64$ a: xxx) -> Any",
  "?10 $ (Any, x: int64$ a: xxx) -> Any",
  "?(10 $ (Any, x: int64$ a: xxx) -> Any)",
  "$10 $ (Any, x: int64$ a: xxx) -> Any",
  "=10 $ (Any, x: int64$ a: xxx) -> Any",
  "$[10 $ (Any, x: int64$ a: xxx) -> Any]",
  "xxx[10 $ (Any, x: int64$ a: xxx) -> Any]",
  "?$10 $ (Any, x: int64$ a: xxx) -> Any]",
  "?=10 $ (Any, x: int64$ a: xxx) -> Any]",
  "?[10 $ (Any, x: int64$ a: xxx) -> Any$",
  "?[10 $ (Any, x: int64$ a: xxx) -> Any$",
  "10 = (Any, x: int64$ a: xxx) -> Any",
  "?10 = (Any, x: int64$ a: xxx) -> Any",
  "?(10 = (Any, x: int64$ a: xxx) -> Any)",
  "$10 = (Any, x: int64$ a: xxx) -> Any",
  "=10 = (Any, x: int64$ a: xxx) -> Any",
  "$[10 = (Any, x: int64$ a: xxx) -> Any]",
  "xxx[10 = (Any, x: int64$ a: xxx) -> Any]",
  "?$10 = (Any, x: int64$ a: xxx) -> Any]",
  "?=10 = (Any, x: int64$ a: xxx) -> Any]",
  "?[10 = (Any, x: int64$ a: xxx) -> Any$",
  "?[10 = (Any, x: int64$ a: xxx) -> Any$",
  "10 * (Any, x: int64= a: xxx) -> Any",
  "?10 * (Any, x: int64= a: xxx) -> Any",
  "?(10 * (Any, x: int64= a: xxx) -> Any)",
  "$10 * (Any, x: int64= a: xxx) -> Any",
  "=10 * (Any, x: int64= a: xxx) -> Any",
  "$[10 * (Any, x: int64= a: xxx) -> Any]",
  "xxx[10 * (Any, x: int64= a: xxx) -> Any]",
  "?$10 * (Any, x: int64= a: xxx) -> Any]",
  "?=10 * (Any, x: int64= a: xxx) -> Any]",
  "?[10 * (Any, x: int64= a: xxx) -> Any$",
  "?[10 * (Any, x: int64= a: xxx) -> Any$",
  "(Any, x: int64= a: xxx) -> Any",
  "?(Any, x: int64= a: xxx) -> Any",
  "?((Any, x: int64= a: xxx) -> Any)",
  "$(Any, x: int64= a: xxx) -> Any",
  "=(Any, x: int64= a: xxx) -> Any",
  "$[(Any, x: int64= a: xxx) -> Any]",
  "xxx[(Any, x: int64= a: xxx) -> Any]",
  "?$(Any, x: int64= a: xxx) -> Any]",
  "?=(Any, x: int64= a: xxx) -> Any]",
  "?[(Any, x: int64= a: xxx) -> Any$",
  "?[(Any, x: int64= a: xxx) -> Any$",
  "10 $ (Any, x: int64= a: xxx) -> Any",
  "?10 $ (Any, x: int64= a: xxx) -> Any",
  "?(10 $ (Any, x: int64= a: xxx) -> Any)",
  "$10 $ (Any, x: int64= a: xxx) -> Any",
  "=10 $ (Any, x: int64= a: xxx) -> Any",
  "$[10 $ (Any, x: int64= a: xxx) -> Any]",
  "xxx[10 $ (Any, x: int64= a: xxx) -> Any]",
  "?$10 $ (Any, x: int64= a: xxx) -> Any]",
  "?=10 $ (Any, x: int64= a: xxx) -> Any]",
  "?[10 $ (Any, x: int64= a: xxx) -> Any$",
  "?[10 $ (Any, x: int64= a: xxx) -> Any$",
  "10 = (Any, x: int64= a: xxx) -> Any",
  "?10 = (Any, x: int64= a: xxx) -> Any",
  "?(10 = (Any, x: int64= a: xxx) -> Any)",
  "$10 = (Any, x: int64= a: xxx) -> Any",
  "=10 = (Any, x: int64= a: xxx) -> Any",
  "$[10 = (Any, x: int64= a: xxx) -> Any]",
  "xxx[10 = (Any, x: int64= a: xxx) -> Any]",
  "?$10 = (Any, x: int64= a: xxx) -> Any]",
  "?=10 = (Any, x: int64= a: xxx) -> Any]",
  "?[10 = (Any, x: int64= a: xxx) -> Any$",
  "?[10 = (Any, x: int64= a: xxx) -> Any$",
  "10 * (Any, x: int64, x: int64, a: xxx) -> Any",
  "?10 * (Any, x: int64, x: int64, a: xxx) -> Any",
  "?(10 * (Any, x: int64, x: int64, a: xxx) -> Any)",
  "$10 * (Any, x: int64, x: int64, a: xxx) -> Any",
  "=10 * (Any, x: int64, x: int64, a: xxx) -> Any",
  "$[10 * (Any, x: int64, x: int64, a: xxx) -> Any]",
  "xxx[10 * (Any, x: int64, x: int64, a: xxx) -> Any]",
  "?$10 * (Any, x: int64, x: int64, a: xxx) -> Any]",
  "?=10 * (Any, x: int64, x: int64, a: xxx) -> Any]",
  "?[10 * (Any, x: int64, x: int64, a: xxx) -> Any$",
  "?[10 * (Any, x: int64, x: int64, a: xxx) -> Any$",
  "(Any, x: int64, x: int64, a: xxx) -> Any",
  "?(Any, x: int64, x: int64, a: xxx) -> Any",
  "?((Any, x: int64, x: int64, a: xxx) -> Any)",
  "$(Any, x: int64, x: int64, a: xxx) -> Any",
  "=(Any, x: int64, x: int64, a: xxx) -> Any",
  "$[(Any, x: int64, x: int64, a: xxx) -> Any]",
  "xxx[(Any, x: int64, x: int64, a: xxx) -> Any]",
  "?$(Any, x: int64, x: int64, a: xxx) -> Any]",
  "?=(Any, x: int64, x: int64, a: xxx) -> Any]",
  "?[(Any, x: int64, x: int64, a: xxx) -> Any$",
  "?[(Any, x: int64, x: int64, a: xxx) -> Any$",
  "10 $ (Any, x: int64, x: int64, a: xxx) -> Any",
  "?10 $ (Any, x: int64, x: int64, a: xxx) -> Any",
  "?(10 $ (Any, x: int64, x: int64, a: xxx) -> Any)",
  "$10 $ (Any, x: int64, x: int64, a: xxx) -> Any",
  "=10 $ (Any, x: int64, x: int64, a: xxx) -> Any",
  "$[10 $ (Any, x: int64, x: int64, a: xxx) -> Any]",
  "xxx[10 $ (Any, x: int64, x: int64, a: xxx) -> Any]",
  "?$10 $ (Any, x: int64, x: int64, a: xxx) -> Any]",
  "?=10 $ (Any, x: int64, x: int64, a: xxx) -> Any]",
  "?[10 $ (Any, x: int64, x: int64, a: xxx) -> Any$",
  "?[10 $ (Any, x: int64, x: int64, a: xxx) -> Any$",
  "10 = (Any, x: int64, x: int64, a: xxx) -> Any",
  "?10 = (Any, x: int64, x: int64, a: xxx) -> Any",
  "?(10 = (Any, x: int64, x: int64, a: xxx) -> Any)",
  "$10 = (Any, x: int64, x: int64, a: xxx) -> Any",
  "=10 = (Any, x: int64, x: int64, a: xxx) -> Any",
  "$[10 = (Any, x: int64, x: int64, a: xxx) -> Any]",
  "xxx[10 = (Any, x: int64, x: int64, a: xxx) -> Any]",
  "?$10 = (Any, x: int64, x: int64, a: xxx) -> Any]",
  "?=10 = (Any, x: int64, x: int64, a: xxx) -> Any]",
  "?[10 = (Any, x: int64, x: int64, a: xxx) -> Any$",
  "?[10 = (Any, x: int64, x: int64, a: xxx) -> Any$",
  "10 * (Any, x: int64, x: int64$ a: xxx) -> Any",
  "?10 * (Any, x: int64, x: int64$ a: xxx) -> Any",
  "?(10 * (Any, x: int64, x: int64$ a: xxx) -> Any)",
  "$10 * (Any, x: int64, x: int64$ a: xxx) -> Any",
  "=10 * (Any, x: int64, x: int64$ a: xxx) -> Any",
  "$[10 * (Any, x: int64, x: int64$ a: xxx) -> Any]",
  "xxx[10 * (Any, x: int64, x: int64$ a: xxx) -> Any]",
  "?$10 * (Any, x: int64, x: int64$ a: xxx) -> Any]",
  "?=10 * (Any, x: int64, x: int64$ a: xxx) -> Any]",
  "?[10 * (Any, x: int64, x: int64$ a: xxx) -> Any$",
  "?[10 * (Any, x: int64, x: int64$ a: xxx) -> Any$",
  "(Any, x: int64, x: int64$ a: xxx) -> Any",
  "?(Any, x: int64, x: int64$ a: xxx) -> Any",
  "?((Any, x: int64, x: int64$ a: xxx) -> Any)",
  "$(Any, x: int64, x: int64$ a: xxx) -> Any",
  "=(Any, x: int64, x: int64$ a: xxx) -> Any",
  "$[(Any, x: int64, x: int64$ a: xxx) -> Any]",
  "xxx[(Any, x: int64, x: int64$ a: xxx) -> Any]",
  "?$(Any, x: int64, x: int64$ a: xxx) -> Any]",
  "?=(Any, x: int64, x: int64$ a: xxx) -> Any]",
  "?[(Any, x: int64, x: int64$ a: xxx) -> Any$",
  "?[(Any, x: int64, x: int64$ a: xxx) -> Any$",
  "10 $ (Any, x: int64, x: int64$ a: xxx) -> Any",
  "?10 $ (Any, x: int64, x: int64$ a: xxx) -> Any",
  "?(10 $ (Any, x: int64, x: int64$ a: xxx) -> Any)",
  "$10 $ (Any, x: int64, x: int64$ a: xxx) -> Any",
  "=10 $ (Any, x: int64, x: int64$ a: xxx) -> Any",
  "$[10 $ (Any, x: int64, x: int64$ a: xxx) -> Any]",
  "xxx[10 $ (Any, x: int64, x: int64$ a: xxx) -> Any]",
  "?$10 $ (Any, x: int64, x: int64$ a: xxx) -> Any]",
  "?=10 $ (Any, x: int64, x: int64$ a: xxx) -> Any]",
  "?[10 $ (Any, x: int64, x: int64$ a: xxx) -> Any$",
  "?[10 $ (Any, x: int64, x: int64$ a: xxx) -> Any$",
  "10 = (Any, x: int64, x: int64$ a: xxx) -> Any",
  "?10 = (Any, x: int64, x: int64$ a: xxx) -> Any",
  "?(10 = (Any, x: int64, x: int64$ a: xxx) -> Any)",
  "$10 = (Any, x: int64, x: int64$ a: xxx) -> Any",
  "=10 = (Any, x: int64, x: int64$ a: xxx) -> Any",
  "$[10 = (Any, x: int64, x: int64$ a: xxx) -> Any]",
  "xxx[10 = (Any, x: int64, x: int64$ a: xxx) -> Any]",
  "?$10 = (Any, x: int64, x: int64$ a: xxx) -> Any]",
  "?=10 = (Any, x: int64, x: int64$ a: xxx) -> Any]",
  "?[10 = (Any, x: int64, x: int64$ a: xxx) -> Any$",
  "?[10 = (Any, x: int64, x: int64$ a: xxx) -> Any$",
  "10 * (Any, x: int64, x: int64= a: xxx) -> Any",
  "?10 * (Any, x: int64, x: int64= a: xxx) -> Any",
  "?(10 * (Any, x: int64, x: int64= a: xxx) -> Any)",
  "$10 * (Any, x: int64, x: int64= a: xxx) -> Any",
  "=10 * (Any, x: int64, x: int64= a: xxx) -> Any",
  "$[10 * (Any, x: int64, x: int64= a: xxx) -> Any]",
  "xxx[10 * (Any, x: int64, x: int64= a: xxx) -> Any]",
  "?$10 * (Any, x: int64, x: int64= a: xxx) -> Any]",
  "?=10 * (Any, x: int64, x: int64= a: xxx) -> Any]",
  "?[10 * (Any, x: int64, x: int64= a: xxx) -> Any$",
  "?[10 * (Any, x: int64, x: int64= a: xxx) -> Any$",
  "(Any, x: int64, x: int64= a: xxx) -> Any",
  "?(Any, x: int64, x: int64= a: xxx) -> Any",
  "?((Any, x: int64, x: int64= a: xxx) -> Any)",
  "$(Any, x: int64, x: int64= a: xxx) -> Any",
  "=(Any, x: int64, x: int64= a: xxx) -> Any",
  "$[(Any, x: int64, x: int64= a: xxx) -> Any]",
  "xxx[(Any, x: int64, x: int64= a: xxx) -> Any]",
  "?$(Any, x: int64, x: int64= a: xxx) -> Any]",
  "?=(Any, x: int64, x: int64= a: xxx) -> Any]",
  "?[(Any, x: int64, x: int64= a: xxx) -> Any$",
  "?[(Any, x: int64, x: int64= a: xxx) -> Any$",
  "10 $ (Any, x: int64, x: int64= a: xxx) -> Any",
  "?10 $ (Any, x: int64, x: int64= a: xxx) -> Any",
  "?(10 $ (Any, x: int64, x: int64= a: xxx) -> Any)",
  "$10 $ (Any, x: int64, x: int64= a: xxx) -> Any",
  "=10 $ (Any, x: int64, x: int64= a: xxx) -> Any",
  "$[10 $ (Any, x: int64, x: int64= a: xxx) -> Any]",
  "xxx[10 $ (Any, x: int64, x: int64= a: xxx) -> Any]",
  "?$10 $ (Any, x: int64, x: int64= a: xxx) -> Any]",
  "?=10 $ (Any, x: int64, x: int64= a: xxx) -> Any]",
  "?[10 $ (Any, x: int64, x: int64= a: xxx) -> Any$",
  "?[10 $ (Any, x: int64, x: int64= a: xxx) -> Any$",
  "10 = (Any, x: int64, x: int64= a: xxx) -> Any",
  "?10 = (Any, x: int64, x: int64= a: xxx) -> Any",
  "?(10 = (Any, x: int64, x: int64= a: xxx) -> Any)",
  "$10 = (Any, x: int64, x: int64= a: xxx) -> Any",
  "=10 = (Any, x: int64, x: int64= a: xxx) -> Any",
  "$[10 = (Any, x: int64, x: int64= a: xxx) -> Any]",
  "xxx[10 = (Any, x: int64, x: int64= a: xxx) -> Any]",
  "?$10 = (Any, x: int64, x: int64= a: xxx) -> Any]",
  "?=10 = (Any, x: int64, x: int64= a: xxx) -> Any]",
  "?[10 = (Any, x: int64, x: int64= a: xxx) -> Any$",
  "?[10 = (Any, x: int64, x: int64= a: xxx) -> Any$",
  "$10 * (Any, a : Any) -> Any",
  "=10 * (Any, a : Any) -> Any",
  "$[10 * (Any, a : Any) -> Any]",
  "xxx[10 * (Any, a : Any) -> Any]",
  "?$10 * (Any, a : Any) -> Any]",
  "?=10 * (Any, a : Any) -> Any]",
  "?[10 * (Any, a : Any) -> Any$",
  "?[10 * (Any, a : Any) -> Any$",
  "$(Any, a : Any) -> Any",
  "=(Any, a : Any) -> Any",
  "$[(Any, a : Any) -> Any]",
  "xxx[(Any, a : Any) -> Any]",
  "?$(Any, a : Any) -> Any]",
  "?=(Any, a : Any) -> Any]",
  "?[(Any, a : Any) -> Any$",
  "?[(Any, a : Any) -> Any$",
  "10 $ (Any, a : Any) -> Any",
  "?10 $ (Any, a : Any) -> Any",
  "?(10 $ (Any, a : Any) -> Any)",
  "$10 $ (Any, a : Any) -> Any",
  "=10 $ (Any, a : Any) -> Any",
  "$[10 $ (Any, a : Any) -> Any]",
  "xxx[10 $ (Any, a : Any) -> Any]",
  "?$10 $ (Any, a : Any) -> Any]",
  "?=10 $ (Any, a : Any) -> Any]",
  "?[10 $ (Any, a : Any) -> Any$",
  "?[10 $ (Any, a : Any) -> Any$",
  "10 = (Any, a : Any) -> Any",
  "?10 = (Any, a : Any) -> Any",
  "?(10 = (Any, a : Any) -> Any)",
  "$10 = (Any, a : Any) -> Any",
  "=10 = (Any, a : Any) -> Any",
  "$[10 = (Any, a : Any) -> Any]",
  "xxx[10 = (Any, a : Any) -> Any]",
  "?$10 = (Any, a : Any) -> Any]",
  "?=10 = (Any, a : Any) -> Any]",
  "?[10 = (Any, a : Any) -> Any$",
  "?[10 = (Any, a : Any) -> Any$",
  "$10 * (Any, ..., a : Any) -> Any",
  "=10 * (Any, ..., a : Any) -> Any",
  "$[10 * (Any, ..., a : Any) -> Any]",
  "xxx[10 * (Any, ..., a : Any) -> Any]",
  "?$10 * (Any, ..., a : Any) -> Any]",
  "?=10 * (Any, ..., a : Any) -> Any]",
  "?[10 * (Any, ..., a : Any) -> Any$",
  "?[10 * (Any, ..., a : Any) -> Any$",
  "$(Any, ..., a : Any) -> Any",
  "=(Any, ..., a : Any) -> Any",
  "$[(Any, ..., a : Any) -> Any]",
  "xxx[(Any, ..., a : Any) -> Any]",
  "?$(Any, ..., a : Any) -> Any]",
  "?=(Any, ..., a : Any) -> Any]",
  "?[(Any, ..., a : Any) -> Any$",
  "?[(Any, ..., a : Any) -> Any$",
  "10 $ (Any, ..., a : Any) -> Any",
  "?10 $ (Any, ..., a : Any) -> Any",
  "?(10 $ (Any, ..., a : Any) -> Any)",
  "$10 $ (Any, ..., a : Any) -> Any",
  "=10 $ (Any, ..., a : Any) -> Any",
  "$[10 $ (Any, ..., a : Any) -> Any]",
  "xxx[10 $ (Any, ..., a : Any) -> Any]",
  "?$10 $ (Any, ..., a : Any) -> Any]",
  "?=10 $ (Any, ..., a : Any) -> Any]",
  "?[10 $ (Any, ..., a : Any) -> Any$",
  "?[10 $ (Any, ..., a : Any) -> Any$",
  "10 = (Any, ..., a : Any) -> Any",
  "?10 = (Any, ..., a : Any) -> Any",
  "?(10 = (Any, ..., a : Any) -> Any)",
  "$10 = (Any, ..., a : Any) -> Any",
  "=10 = (Any, ..., a : Any) -> Any",
  "$[10 = (Any, ..., a : Any) -> Any]",
  "xxx[10 = (Any, ..., a : Any) -> Any]",
  "?$10 = (Any, ..., a : Any) -> Any]",
  "?=10 = (Any, ..., a : Any) -> Any]",
  "?[10 = (Any, ..., a : Any) -> Any$",
  "?[10 = (Any, ..., a : Any) -> Any$",
  "$10 * (Any, a : Any, ) -> Any",
  "=10 * (Any, a : Any, ) -> Any",
  "$[10 * (Any, a : Any, ) -> Any]",
  "xxx[10 * (Any, a : Any, ) -> Any]",
  "?$10 * (Any, a : Any, ) -> Any]",
  "?=10 * (Any, a : Any, ) -> Any]",
  "?[10 * (Any, a : Any, ) -> Any$",
  "?[10 * (Any, a : Any, ) -> Any$",
  "$(Any, a : Any, ) -> Any",
  "=(Any, a : Any, ) -> Any",
  "$[(Any, a : Any, ) -> Any]",
  "xxx[(Any, a : Any, ) -> Any]",
  "?$(Any, a : Any, ) -> Any]",
  "?=(Any, a : Any, ) -> Any]",
  "?[(Any, a : Any, ) -> Any$",
  "?[(Any, a : Any, ) -> Any$",
  "10 $ (Any, a : Any, ) -> Any",
  "?10 $ (Any, a : Any, ) -> Any",
  "?(10 $ (Any, a : Any, ) -> Any)",
  "$10 $ (Any, a : Any, ) -> Any",
  "=10 $ (Any, a : Any, ) -> Any",
  "$[10 $ (Any, a : Any, ) -> Any]",
  "xxx[10 $ (Any, a : Any, ) -> Any]",
  "?$10 $ (Any, a : Any, ) -> Any]",
  "?=10 $ (Any, a : Any, ) -> Any]",
  "?[10 $ (Any, a : Any, ) -> Any$",
  "?[10 $ (Any, a : Any, ) -> Any$",
  "10 = (Any, a : Any, ) -> Any",
  "?10 = (Any, a : Any, ) -> Any",
  "?(10 = (Any, a : Any, ) -> Any)",
  "$10 = (Any, a : Any, ) -> Any",
  "=10 = (Any, a : Any, ) -> Any",
  "$[10 = (Any, a : Any, ) -> Any]",
  "xxx[10 = (Any, a : Any, ) -> Any]",
  "?$10 = (Any, a : Any, ) -> Any]",
  "?=10 = (Any, a : Any, ) -> Any]",
  "?[10 = (Any, a : Any, ) -> Any$",
  "?[10 = (Any, a : Any, ) -> Any$",
  "$10 * (Any, ..., a : Any, ) -> Any",
  "=10 * (Any, ..., a : Any, ) -> Any",
  "$[10 * (Any, ..., a : Any, ) -> Any]",
  "xxx[10 * (Any, ..., a : Any, ) -> Any]",
  "?$10 * (Any, ..., a : Any, ) -> Any]",
  "?=10 * (Any, ..., a : Any, ) -> Any]",
  "?[10 * (Any, ..., a : Any, ) -> Any$",
  "?[10 * (Any, ..., a : Any, ) -> Any$",
  "$(Any, ..., a : Any, ) -> Any",
  "=(Any, ..., a : Any, ) -> Any",
  "$[(Any, ..., a : Any, ) -> Any]",
  "xxx[(Any, ..., a : Any, ) -> Any]",
  "?$(Any, ..., a : Any, ) -> Any]",
  "?=(Any, ..., a : Any, ) -> Any]",
  "?[(Any, ..., a : Any, ) -> Any$",
  "?[(Any, ..., a : Any, ) -> Any$",
  "10 $ (Any, ..., a : Any, ) -> Any",
  "?10 $ (Any, ..., a : Any, ) -> Any",
  "?(10 $ (Any, ..., a : Any, ) -> Any)",
  "$10 $ (Any, ..., a : Any, ) -> Any",
  "=10 $ (Any, ..., a : Any, ) -> Any",
  "$[10 $ (Any, ..., a : Any, ) -> Any]",
  "xxx[10 $ (Any, ..., a : Any, ) -> Any]",
  "?$10 $ (Any, ..., a : Any, ) -> Any]",
  "?=10 $ (Any, ..., a : Any, ) -> Any]",
  "?[10 $ (Any, ..., a : Any, ) -> Any$",
  "?[10 $ (Any, ..., a : Any, ) -> Any$",
  "10 = (Any, ..., a : Any, ) -> Any",
  "?10 = (Any, ..., a : Any, ) -> Any",
  "?(10 = (Any, ..., a : Any, ) -> Any)",
  "$10 = (Any, ..., a : Any, ) -> Any",
  "=10 = (Any, ..., a : Any, ) -> Any",
  "$[10 = (Any, ..., a : Any, ) -> Any]",
  "xxx[10 = (Any, ..., a : Any, ) -> Any]",
  "?$10 = (Any, ..., a : Any, ) -> Any]",
  "?=10 = (Any, ..., a : Any, ) -> Any]",
  "?[10 = (Any, ..., a : Any, ) -> Any$",
  "?[10 = (Any, ..., a : Any, ) -> Any$",
  "$10 * (Any, a : Any, ...) -> Any",
  "=10 * (Any, a : Any, ...) -> Any",
  "$[10 * (Any, a : Any, ...) -> Any]",
  "xxx[10 * (Any, a : Any, ...) -> Any]",
  "?$10 * (Any, a : Any, ...) -> Any]",
  "?=10 * (Any, a : Any, ...) -> Any]",
  "?[10 * (Any, a : Any, ...) -> Any$",
  "?[10 * (Any, a : Any, ...) -> Any$",
  "$(Any, a : Any, ...) -> Any",
  "=(Any, a : Any, ...) -> Any",
  "$[(Any, a : Any, ...) -> Any]",
  "xxx[(Any, a : Any, ...) -> Any]",
  "?$(Any, a : Any, ...) -> Any]",
  "?=(Any, a : Any, ...) -> Any]",
  "?[(Any, a : Any, ...) -> Any$",
  "?[(Any, a : Any, ...) -> Any$",
  "10 $ (Any, a : Any, ...) -> Any",
  "?10 $ (Any, a : Any, ...) -> Any",
  "?(10 $ (Any, a : Any, ...) -> Any)",
  "$10 $ (Any, a : Any, ...) -> Any",
  "=10 $ (Any, a : Any, ...) -> Any",
  "$[10 $ (Any, a : Any, ...) -> Any]",
  "xxx[10 $ (Any, a : Any, ...) -> Any]",
  "?$10 $ (Any, a : Any, ...) -> Any]",
  "?=10 $ (Any, a : Any, ...) -> Any]",
  "?[10 $ (Any, a : Any, ...) -> Any$",
  "?[10 $ (Any, a : Any, ...) -> Any$",
  "10 = (Any, a : Any, ...) -> Any",
  "?10 = (Any, a : Any, ...) -> Any",
  "?(10 = (Any, a : Any, ...) -> Any)",
  "$10 = (Any, a : Any, ...) -> Any",
  "=10 = (Any, a : Any, ...) -> Any",
  "$[10 = (Any, a : Any, ...) -> Any]",
  "xxx[10 = (Any, a : Any, ...) -> Any]",
  "?$10 = (Any, a : Any, ...) -> Any]",
  "?=10 = (Any, a : Any, ...) -> Any]",
  "?[10 = (Any, a : Any, ...) -> Any$",
  "?[10 = (Any, a : Any, ...) -> Any$",
  "$10 * (Any, ..., a : Any, ...) -> Any",
  "=10 * (Any, ..., a : Any, ...) -> Any",
  "$[10 * (Any, ..., a : Any, ...) -> Any]",
  "xxx[10 * (Any, ..., a : Any, ...) -> Any]",
  "?$10 * (Any, ..., a : Any, ...) -> Any]",
  "?=10 * (Any, ..., a : Any, ...) -> Any]",
  "?[10 * (Any, ..., a : Any, ...) -> Any$",
  "?[10 * (Any, ..., a : Any, ...) -> Any$",
  "$(Any, ..., a : Any, ...) -> Any",
  "=(Any, ..., a : Any, ...) -> Any",
  "$[(Any, ..., a : Any, ...) -> Any]",
  "xxx[(Any, ..., a : Any, ...) -> Any]",
  "?$(Any, ..., a : Any, ...) -> Any]",
  "?=(Any, ..., a : Any, ...) -> Any]",
  "?[(Any, ..., a : Any, ...) -> Any$",
  "?[(Any, ..., a : Any, ...) -> Any$",
  "10 $ (Any, ..., a : Any, ...) -> Any",
  "?10 $ (Any, ..., a : Any, ...) -> Any",
  "?(10 $ (Any, ..., a : Any, ...) -> Any)",
  "$10 $ (Any, ..., a : Any, ...) -> Any",
  "=10 $ (Any, ..., a : Any, ...) -> Any",
  "$[10 $ (Any, ..., a : Any, ...) -> Any]",
  "xxx[10 $ (Any, ..., a : Any, ...) -> Any]",
  "?$10 $ (Any, ..., a : Any, ...) -> Any]",
  "?=10 $ (Any, ..., a : Any, ...) -> Any]",
  "?[10 $ (Any, ..., a : Any, ...) -> Any$",
  "?[10 $ (Any, ..., a : Any, ...) -> Any$",
  "10 = (Any, ..., a : Any, ...) -> Any",
  "?10 = (Any, ..., a : Any, ...) -> Any",
  "?(10 = (Any, ..., a : Any, ...) -> Any)",
  "$10 = (Any, ..., a : Any, ...) -> Any",
  "=10 = (Any, ..., a : Any, ...) -> Any",
  "$[10 = (Any, ..., a : Any, ...) -> Any]",
  "xxx[10 = (Any, ..., a : Any, ...) -> Any]",
  "?$10 = (Any, ..., a : Any, ...) -> Any]",
  "?=10 = (Any, ..., a : Any, ...) -> Any]",
  "?[10 = (Any, ..., a : Any, ...) -> Any$",
  "?[10 = (Any, ..., a : Any, ...) -> Any$",
  "10 * (Any, a : Any, $) -> Any",
  "?10 * (Any, a : Any, $) -> Any",
  "?(10 * (Any, a : Any, $) -> Any)",
  "$10 * (Any, a : Any, $) -> Any",
  "=10 * (Any, a : Any, $) -> Any",
  "$[10 * (Any, a : Any, $) -> Any]",
  "xxx[10 * (Any, a : Any, $) -> Any]",
  "?$10 * (Any, a : Any, $) -> Any]",
  "?=10 * (Any, a : Any, $) -> Any]",
  "?[10 * (Any, a : Any, $) -> Any$",
  "?[10 * (Any, a : Any, $) -> Any$",
  "(Any, a : Any, $) -> Any",
  "?(Any, a : Any, $) -> Any",
  "?((Any, a : Any, $) -> Any)",
  "$(Any, a : Any, $) -> Any",
  "=(Any, a : Any, $) -> Any",
  "$[(Any, a : Any, $) -> Any]",
  "xxx[(Any, a : Any, $) -> Any]",
  "?$(Any, a : Any, $) -> Any]",
  "?=(Any, a : Any, $) -> Any]",
  "?[(Any, a : Any, $) -> Any$",
  "?[(Any, a : Any, $) -> Any$",
  "10 $ (Any, a : Any, $) -> Any",
  "?10 $ (Any, a : Any, $) -> Any",
  "?(10 $ (Any, a : Any, $) -> Any)",
  "$10 $ (Any, a : Any, $) -> Any",
  "=10 $ (Any, a : Any, $) -> Any",
  "$[10 $ (Any, a : Any, $) -> Any]",
  "xxx[10 $ (Any, a : Any, $) -> Any]",
  "?$10 $ (Any, a : Any, $) -> Any]",
  "?=10 $ (Any, a : Any, $) -> Any]",
  "?[10 $ (Any, a : Any, $) -> Any$",
  "?[10 $ (Any, a : Any, $) -> Any$",
  "10 = (Any, a : Any, $) -> Any",
  "?10 = (Any, a : Any, $) -> Any",
  "?(10 = (Any, a : Any, $) -> Any)",
  "$10 = (Any, a : Any, $) -> Any",
  "=10 = (Any, a : Any, $) -> Any",
  "$[10 = (Any, a : Any, $) -> Any]",
  "xxx[10 = (Any, a : Any, $) -> Any]",
  "?$10 = (Any, a : Any, $) -> Any]",
  "?=10 = (Any, a : Any, $) -> Any]",
  "?[10 = (Any, a : Any, $) -> Any$",
  "?[10 = (Any, a : Any, $) -> Any$",
  "10 * (Any, ..., a : Any, $) -> Any",
  "?10 * (Any, ..., a : Any, $) -> Any",
  "?(10 * (Any, ..., a : Any, $) -> Any)",
  "$10 * (Any, ..., a : Any, $) -> Any",
  "=10 * (Any, ..., a : Any, $) -> Any",
  "$[10 * (Any, ..., a : Any, $) -> Any]",
  "xxx[10 * (Any, ..., a : Any, $) -> Any]",
  "?$10 * (Any, ..., a : Any, $) -> Any]",
  "?=10 * (Any, ..., a : Any, $) -> Any]",
  "?[10 * (Any, ..., a : Any, $) -> Any$",
  "?[10 * (Any, ..., a : Any, $) -> Any$",
  "(Any, ..., a : Any, $) -> Any",
  "?(Any, ..., a : Any, $) -> Any",
  "?((Any, ..., a : Any, $) -> Any)",
  "$(Any, ..., a : Any, $) -> Any",
  "=(Any, ..., a : Any, $) -> Any",
  "$[(Any, ..., a : Any, $) -> Any]",
  "xxx[(Any, ..., a : Any, $) -> Any]",
  "?$(Any, ..., a : Any, $) -> Any]",
  "?=(Any, ..., a : Any, $) -> Any]",
  "?[(Any, ..., a : Any, $) -> Any$",
  "?[(Any, ..., a : Any, $) -> Any$",
  "10 $ (Any, ..., a : Any, $) -> Any",
  "?10 $ (Any, ..., a : Any, $) -> Any",
  "?(10 $ (Any, ..., a : Any, $) -> Any)",
  "$10 $ (Any, ..., a : Any, $) -> Any",
  "=10 $ (Any, ..., a : Any, $) -> Any",
  "$[10 $ (Any, ..., a : Any, $) -> Any]",
  "xxx[10 $ (Any, ..., a : Any, $) -> Any]",
  "?$10 $ (Any, ..., a : Any, $) -> Any]",
  "?=10 $ (Any, ..., a : Any, $) -> Any]",
  "?[10 $ (Any, ..., a : Any, $) -> Any$",
  "?[10 $ (Any, ..., a : Any, $) -> Any$",
  "10 = (Any, ..., a : Any, $) -> Any",
  "?10 = (Any, ..., a : Any, $) -> Any",
  "?(10 = (Any, ..., a : Any, $) -> Any)",
  "$10 = (Any, ..., a : Any, $) -> Any",
  "=10 = (Any, ..., a : Any, $) -> Any",
  "$[10 = (Any, ..., a : Any, $) -> Any]",
  "xxx[10 = (Any, ..., a : Any, $) -> Any]",
  "?$10 = (Any, ..., a : Any, $) -> Any]",
  "?=10 = (Any, ..., a : Any, $) -> Any]",
  "?[10 = (Any, ..., a : Any, $) -> Any$",
  "?[10 = (Any, ..., a : Any, $) -> Any$",
  "10 * (Any, ...) -> $",
  "?10 * (Any, ...) -> $",
  "?(10 * (Any, ...) -> $)",
  "$10 * (Any, ...) -> $",
  "=10 * (Any, ...) -> $",
  "$[10 * (Any, ...) -> $]",
  "xxx[10 * (Any, ...) -> $]",
  "?$10 * (Any, ...) -> $]",
  "?=10 * (Any, ...) -> $]",
  "?[10 * (Any, ...) -> $$",
  "?[10 * (Any, ...) -> $$",
  "(Any, ...) -> $",
  "?(Any, ...) -> $",
  "?((Any, ...) -> $)",
  "$(Any, ...) -> $",
  "=(Any, ...) -> $",
  "$[(Any, ...) -> $]",
  "xxx[(Any, ...) -> $]",
  "?$(Any, ...) -> $]",
  "?=(Any, ...) -> $]",
  "?[(Any, ...) -> $$",
  "?[(Any, ...) -> $$",
  "10 $ (Any, ...) -> $",
  "?10 $ (Any, ...) -> $",
  "?(10 $ (Any, ...) -> $)",
  "$10 $ (Any, ...) -> $",
  "=10 $ (Any, ...) -> $",
  "$[10 $ (Any, ...) -> $]",
  "xxx[10 $ (Any, ...) -> $]",
  "?$10 $ (Any, ...) -> $]",
  "?=10 $ (Any, ...) -> $]",
  "?[10 $ (Any, ...) -> $$",
  "?[10 $ (Any, ...) -> $$",
  "10 = (Any, ...) -> $",
  "?10 = (Any, ...) -> $",
  "?(10 = (Any, ...) -> $)",
  "$10 = (Any, ...) -> $",
  "=10 = (Any, ...) -> $",
  "$[10 = (Any, ...) -> $]",
  "xxx[10 = (Any, ...) -> $]",
  "?$10 = (Any, ...) -> $]",
  "?=10 = (Any, ...) -> $]",
  "?[10 = (Any, ...) -> $$",
  "?[10 = (Any, ...) -> $$",
  "10 * $Any, a : Any) -> Any",
  "?10 * $Any, a : Any) -> Any",
  "?(10 * $Any, a : Any) -> Any)",
  "$10 * $Any, a : Any) -> Any",
  "=10 * $Any, a : Any) -> Any",
  "$[10 * $Any, a : Any) -> Any]",
  "xxx[10 * $Any, a : Any) -> Any]",
  "?$10 * $Any, a : Any) -> Any]",
  "?=10 * $Any, a : Any) -> Any]",
  "?[10 * $Any, a : Any) -> Any$",
  "?[10 * $Any, a : Any) -> Any$",
  "$Any, a : Any) -> Any",
  "?$Any, a : Any) -> Any",
  "?($Any, a : Any) -> Any)",
  "$$Any, a : Any) -> Any",
  "=$Any, a : Any) -> Any",
  "$[$Any, a : Any) -> Any]",
  "xxx[$Any, a : Any) -> Any]",
  "?$$Any, a : Any) -> Any]",
  "?=$Any, a : Any) -> Any]",
  "?[$Any, a : Any) -> Any$",
  "?[$Any, a : Any) -> Any$",
  "10 $ $Any, a : Any) -> Any",
  "?10 $ $Any, a : Any) -> Any",
  "?(10 $ $Any, a : Any) -> Any)",
  "$10 $ $Any, a : Any) -> Any",
  "=10 $ $Any, a : Any) -> Any",
  "$[10 $ $Any, a : Any) -> Any]",
  "xxx[10 $ $Any, a : Any) -> Any]",
  "?$10 $ $Any, a : Any) -> Any]",
  "?=10 $ $Any, a : Any) -> Any]",
  "?[10 $ $Any, a : Any) -> Any$",
  "?[10 $ $Any, a : Any) -> Any$",
  "10 = $Any, a : Any) -> Any",
  "?10 = $Any, a : Any) -> Any",
  "?(10 = $Any, a : Any) -> Any)",
  "$10 = $Any, a : Any) -> Any",
  "=10 = $Any, a : Any) -> Any",
  "$[10 = $Any, a : Any) -> Any]",
  "xxx[10 = $Any, a : Any) -> Any]",
  "?$10 = $Any, a : Any) -> Any]",
  "?=10 = $Any, a : Any) -> Any]",
  "?[10 = $Any, a : Any) -> Any$",
  "?[10 = $Any, a : Any) -> Any$",
  "10 * (Any$ a : Any) -> Any",
  "?10 * (Any$ a : Any) -> Any",
  "?(10 * (Any$ a : Any) -> Any)",
  "$10 * (Any$ a : Any) -> Any",
  "=10 * (Any$ a : Any) -> Any",
  "$[10 * (Any$ a : Any) -> Any]",
  "xxx[10 * (Any$ a : Any) -> Any]",
  "?$10 * (Any$ a : Any) -> Any]",
  "?=10 * (Any$ a : Any) -> Any]",
  "?[10 * (Any$ a : Any) -> Any$",
  "?[10 * (Any$ a : Any) -> Any$",
  "(Any$ a : Any) -> Any",
  "?(Any$ a : Any) -> Any",
  "?((Any$ a : Any) -> Any)",
  "$(Any$ a : Any) -> Any",
  "=(Any$ a : Any) -> Any",
  "$[(Any$ a : Any) -> Any]",
  "xxx[(Any$ a : Any) -> Any]",
  "?$(Any$ a : Any) -> Any]",
  "?=(Any$ a : Any) -> Any]",
  "?[(Any$ a : Any) -> Any$",
  "?[(Any$ a : Any) -> Any$",
  "10 $ (Any$ a : Any) -> Any",
  "?10 $ (Any$ a : Any) -> Any",
  "?(10 $ (Any$ a : Any) -> Any)",
  "$10 $ (Any$ a : Any) -> Any",
  "=10 $ (Any$ a : Any) -> Any",
  "$[10 $ (Any$ a : Any) -> Any]",
  "xxx[10 $ (Any$ a : Any) -> Any]",
  "?$10 $ (Any$ a : Any) -> Any]",
  "?=10 $ (Any$ a : Any) -> Any]",
  "?[10 $ (Any$ a : Any) -> Any$",
  "?[10 $ (Any$ a : Any) -> Any$",
  "10 = (Any$ a : Any) -> Any",
  "?10 = (Any$ a : Any) -> Any",
  "?(10 = (Any$ a : Any) -> Any)",
  "$10 = (Any$ a : Any) -> Any",
  "=10 = (Any$ a : Any) -> Any",
  "$[10 = (Any$ a : Any) -> Any]",
  "xxx[10 = (Any$ a : Any) -> Any]",
  "?$10 = (Any$ a : Any) -> Any]",
  "?=10 = (Any$ a : Any) -> Any]",
  "?[10 = (Any$ a : Any) -> Any$",
  "?[10 = (Any$ a : Any) -> Any$",
  "10 * (Any, $ : Any) -> Any",
  "?10 * (Any, $ : Any) -> Any",
  "?(10 * (Any, $ : Any) -> Any)",
  "$10 * (Any, $ : Any) -> Any",
  "=10 * (Any, $ : Any) -> Any",
  "$[10 * (Any, $ : Any) -> Any]",
  "xxx[10 * (Any, $ : Any) -> Any]",
  "?$10 * (Any, $ : Any) -> Any]",
  "?=10 * (Any, $ : Any) -> Any]",
  "?[10 * (Any, $ : Any) -> Any$",
  "?[10 * (Any, $ : Any) -> Any$",
  "(Any, $ : Any) -> Any",
  "?(Any, $ : Any) -> Any",
  "?((Any, $ : Any) -> Any)",
  "$(Any, $ : Any) -> Any",
  "=(Any, $ : Any) -> Any",
  "$[(Any, $ : Any) -> Any]",
  "xxx[(Any, $ : Any) -> Any]",
  "?$(Any, $ : Any) -> Any]",
  "?=(Any, $ : Any) -> Any]",
  "?[(Any, $ : Any) -> Any$",
  "?[(Any, $ : Any) -> Any$",
  "10 $ (Any, $ : Any) -> Any",
  "?10 $ (Any, $ : Any) -> Any",
  "?(10 $ (Any, $ : Any) -> Any)",
  "$10 $ (Any, $ : Any) -> Any",
  "=10 $ (Any, $ : Any) -> Any",
  "$[10 $ (Any, $ : Any) -> Any]",
  "xxx[10 $ (Any, $ : Any) -> Any]",
  "?$10 $ (Any, $ : Any) -> Any]",
  "?=10 $ (Any, $ : Any) -> Any]",
  "?[10 $ (Any, $ : Any) -> Any$",
  "?[10 $ (Any, $ : Any) -> Any$",
  "10 = (Any, $ : Any) -> Any",
  "?10 = (Any, $ : Any) -> Any",
  "?(10 = (Any, $ : Any) -> Any)",
  "$10 = (Any, $ : Any) -> Any",
  "=10 = (Any, $ : Any) -> Any",
  "$[10 = (Any, $ : Any) -> Any]",
  "xxx[10 = (Any, $ : Any) -> Any]",
  "?$10 = (Any, $ : Any) -> Any]",
  "?=10 = (Any, $ : Any) -> Any]",
  "?[10 = (Any, $ : Any) -> Any$",
  "?[10 = (Any, $ : Any) -> Any$",
  "10 * (Any, a : Any$ -> Any",
  "?10 * (Any, a : Any$ -> Any",
  "?(10 * (Any, a : Any$ -> Any)",
  "$10 * (Any, a : Any$ -> Any",
  "=10 * (Any, a : Any$ -> Any",
  "$[10 * (Any, a : Any$ -> Any]",
  "xxx[10 * (Any, a : Any$ -> Any]",
  "?$10 * (Any, a : Any$ -> Any]",
  "?=10 * (Any, a : Any$ -> Any]",
  "?[10 * (Any, a : Any$ -> Any$",
  "?[10 * (Any, a : Any$ -> Any$",
  "(Any, a : Any$ -> Any",
  "?(Any, a : Any$ -> Any",
  "?((Any, a : Any$ -> Any)",
  "$(Any, a : Any$ -> Any",
  "=(Any, a : Any$ -> Any",
  "$[(Any, a : Any$ -> Any]",
  "xxx[(Any, a : Any$ -> Any]",
  "?$(Any, a : Any$ -> Any]",
  "?=(Any, a : Any$ -> Any]",
  "?[(Any, a : Any$ -> Any$",
  "?[(Any, a : Any$ -> Any$",
  "10 $ (Any, a : Any$ -> Any",
  "?10 $ (Any, a : Any$ -> Any",
  "?(10 $ (Any, a : Any$ -> Any)",
  "$10 $ (Any, a : Any$ -> Any",
  "=10 $ (Any, a : Any$ -> Any",
  "$[10 $ (Any, a : Any$ -> Any]",
  "xxx[10 $ (Any, a : Any$ -> Any]",
  "?$10 $ (Any, a : Any$ -> Any]",
  "?=10 $ (Any, a : Any$ -> Any]",
  "?[10 $ (Any, a : Any$ -> Any$",
  "?[10 $ (Any, a : Any$ -> Any$",
  "10 = (Any, a : Any$ -> Any",
  "?10 = (Any, a : Any$ -> Any",
  "?(10 = (Any, a : Any$ -> Any)",
  "$10 = (Any, a : Any$ -> Any",
  "=10 = (Any, a : Any$ -> Any",
  "$[10 = (Any, a : Any$ -> Any]",
  "xxx[10 = (Any, a : Any$ -> Any]",
  "?$10 = (Any, a : Any$ -> Any]",
  "?=10 = (Any, a : Any$ -> Any]",
  "?[10 = (Any, a : Any$ -> Any$",
  "?[10 = (Any, a : Any$ -> Any$",
  "10 * (Any, a : Any) -> $",
  "?10 * (Any, a : Any) -> $",
  "?(10 * (Any, a : Any) -> $)",
  "$10 * (Any, a : Any) -> $",
  "=10 * (Any, a : Any) -> $",
  "$[10 * (Any, a : Any) -> $]",
  "xxx[10 * (Any, a : Any) -> $]",
  "?$10 * (Any, a : Any) -> $]",
  "?=10 * (Any, a : Any) -> $]",
  "?[10 * (Any, a : Any) -> $$",
  "?[10 * (Any, a : Any) -> $$",
  "(Any, a : Any) -> $",
  "?(Any, a : Any) -> $",
  "?((Any, a : Any) -> $)",
  "$(Any, a : Any) -> $",
  "=(Any, a : Any) -> $",
  "$[(Any, a : Any) -> $]",
  "xxx[(Any, a : Any) -> $]",
  "?$(Any, a : Any) -> $]",
  "?=(Any, a : Any) -> $]",
  "?[(Any, a : Any) -> $$",
  "?[(Any, a : Any) -> $$",
  "10 $ (Any, a : Any) -> $",
  "?10 $ (Any, a : Any) -> $",
  "?(10 $ (Any, a : Any) -> $)",
  "$10 $ (Any, a : Any) -> $",
  "=10 $ (Any, a : Any) -> $",
  "$[10 $ (Any, a : Any) -> $]",
  "xxx[10 $ (Any, a : Any) -> $]",
  "?$10 $ (Any, a : Any) -> $]",
  "?=10 $ (Any, a : Any) -> $]",
  "?[10 $ (Any, a : Any) -> $$",
  "?[10 $ (Any, a : Any) -> $$",
  "10 = (Any, a : Any) -> $",
  "?10 = (Any, a : Any) -> $",
  "?(10 = (Any, a : Any) -> $)",
  "$10 = (Any, a : Any) -> $",
  "=10 = (Any, a : Any) -> $",
  "$[10 = (Any, a : Any) -> $]",
  "xxx[10 = (Any, a : Any) -> $]",
  "?$10 = (Any, a : Any) -> $]",
  "?=10 = (Any, a : Any) -> $]",
  "?[10 = (Any, a : Any) -> $$",
  "?[10 = (Any, a : Any) -> $$",
  "$10 * Z_FF5",
  "=10 * Z_FF5",
  "$[10 * Z_FF5]",
  "xxx[10 * Z_FF5]",
  "?$10 * Z_FF5]",
  "?=10 * Z_FF5]",
  "?[10 * Z_FF5$",
  "?[10 * Z_FF5$",
  "$Z_FF5",
  "=Z_FF5",
  "$[Z_FF5]",
  "xxx[Z_FF5]",
  "?$Z_FF5]",
  "?=Z_FF5]",
  "?[Z_FF5$",
  "?[Z_FF5$",
  "10 $ Z_FF5",
  "?10 $ Z_FF5",
  "?(10 $ Z_FF5)",
  "$10 $ Z_FF5",
  "=10 $ Z_FF5",
  "$[10 $ Z_FF5]",
  "xxx[10 $ Z_FF5]",
  "?$10 $ Z_FF5]",
  "?=10 $ Z_FF5]",
  "?[10 $ Z_FF5$",
  "?[10 $ Z_FF5$",
  "10 = Z_FF5",
  "?10 = Z_FF5",
  "?(10 = Z_FF5)",
  "$10 = Z_FF5",
  "=10 = Z_FF5",
  "$[10 = Z_FF5]",
  "xxx[10 = Z_FF5]",
  "?$10 = Z_FF5]",
  "?=10 = Z_FF5]",
  "?[10 = Z_FF5$",
  "?[10 = Z_FF5$",
  "$10 * X(Any)",
  "=10 * X(Any)",
  "$[10 * X(Any])",
  "xxx[10 * X(Any])",
  "?$10 * X(Any])",
  "?=10 * X(Any])",
  "?[10 * X(Any)$",
  "?[10 * X(Any)$",
  "$X(Any)",
  "=X(Any)",
  "$[X(Any])",
  "xxx[X(Any])",
  "?$X(Any])",
  "?=X(Any])",
  "?[X(Any)$",
  "?[X(Any)$",
  "10 $ X(Any)",
  "?10 $ X(Any)",
  "?(10 $ X(Any))",
  "$10 $ X(Any)",
  "=10 $ X(Any)",
  "$[10 $ X(Any])",
  "xxx[10 $ X(Any])",
  "?$10 $ X(Any])",
  "?=10 $ X(Any])",
  "?[10 $ X(Any)$",
  "?[10 $ X(Any)$",
  "10 = X(Any)",
  "?10 = X(Any)",
  "?(10 = X(Any))",
  "$10 = X(Any)",
  "=10 = X(Any)",
  "$[10 = X(Any])",
  "xxx[10 = X(Any])",
  "?$10 = X(Any])",
  "?=10 = X(Any])",
  "?[10 = X(Any)$",
  "?[10 = X(Any)$",
  "10 * X$Any]",
  "?10 * X$Any]",
  "?(10 * X$Any])",
  "$10 * X$Any]",
  "=10 * X$Any]",
  "$[10 * X$Any]]",
  "xxx[10 * X$Any]]",
  "?$10 * X$Any]]",
  "?=10 * X$Any]]",
  "?(10 * X$Any)$",
  "?(10 * X$Any)$",
  "X$Any]",
  "?X$Any]",
  "?(X$Any])",
  "$X$Any]",
  "=X$Any]",
  "$[X$Any]]",
  "xxx[X$Any]]",
  "?$X$Any]]",
  "?=X$Any]]",
  "?(X$Any)$",
  "?(X$Any)$",
  "10 $ X$Any]",
  "?10 $ X$Any]",
  "?(10 $ X$Any])",
  "$10 $ X$Any]",
  "=10 $ X$Any]",
  "$[10 $ X$Any]]",
  "xxx[10 $ X$Any]]",
  "?$10 $ X$Any]]",
  "?=10 $ X$Any]]",
  "?(10 $ X$Any)$",
  "?(10 $ X$Any)$",
  "10 = X$Any]",
  "?10 = X$Any]",
  "?(10 = X$Any])",
  "$10 = X$Any]",
  "=10 = X$Any]",
  "$[10 = X$Any]]",
  "xxx[10 = X$Any]]",
  "?$10 = X$Any]]",
  "?=10 = X$Any]]",
  "?(10 = X$Any)$",
  "?(10 = X$Any)$",
  "10 * X=Any",
  "?10 * X=Any",
  "?(10 * X=Any)",
  "$10 * X=Any",
  "=10 * X=Any",
  "$[10 * X=Any]",
  "xxx[10 * X=Any]",
  "?$10 * X=Any]",
  "?=10 * X=Any]",
  "?[10 * X=Any$",
  "?[10 * X=Any$",
  "X=Any",
  "?X=Any",
  "?(X=Any)",
  "$X=Any",
  "=X=Any",
  "$[X=Any]",
  "xxx[X=Any]",
  "?$X=Any]",
  "?=X=Any]",
  "?[X=Any$",
  "?[X=Any$",
  "10 $ X=Any",
  "?10 $ X=Any",
  "?(10 $ X=Any)",
  "$10 $ X=Any",
  "=10 $ X=Any",
  "$[10 $ X=Any]",
  "xxx[10 $ X=Any]",
  "?$10 $ X=Any]",
  "?=10 $ X=Any]",
  "?[10 $ X=Any$",
  "?[10 $ X=Any$",
  "10 = X=Any",
  "?10 = X=Any",
  "?(10 = X=Any)",
  "$10 = X=Any",
  "=10 = X=Any",
  "$[10 = X=Any]",
  "xxx[10 = X=Any]",
  "?$10 = X=Any]",
  "?=10 = X=Any]",
  "?[10 = X=Any$",
  "?[10 = X=Any$",
  "10 * X[Any$",
  "?10 * X[Any$",
  "?[10 * X(Any$)",
  "$10 * X[Any$",
  "=10 * X[Any$",
  "$[10 * X(Any$)",
  "xxx[10 * X(Any$)",
  "?$10 * X(Any$)",
  "?=10 * X(Any$)",
  "?[10 * X[Any$$",
  "?[10 * X[Any$$",
  "X[Any$",
  "?X[Any$",
  "?[X(Any$)",
  "$X[Any$",
  "=X[Any$",
  "$[X(Any$)",
  "xxx[X(Any$)",
  "?$X(Any$)",
  "?=X(Any$)",
  "?[X[Any$$",
  "?[X[Any$$",
  "10 $ X[Any$",
  "?10 $ X[Any$",
  "?[10 $ X(Any$)",
  "$10 $ X[Any$",
  "=10 $ X[Any$",
  "$[10 $ X(Any$)",
  "xxx[10 $ X(Any$)",
  "?$10 $ X(Any$)",
  "?=10 $ X(Any$)",
  "?[10 $ X[Any$$",
  "?[10 $ X[Any$$",
  "10 = X[Any$",
  "?10 = X[Any$",
  "?[10 = X(Any$)",
  "$10 = X[Any$",
  "=10 = X[Any$",
  "$[10 = X(Any$)",
  "xxx[10 = X(Any$)",
  "?$10 = X(Any$)",
  "?=10 = X(Any$)",
  "?[10 = X[Any$$",
  "?[10 = X[Any$$",
  "10 * X[Any$",
  "?10 * X[Any$",
  "?[10 * X(Any$)",
  "$10 * X[Any$",
  "=10 * X[Any$",
  "$[10 * X(Any$)",
  "xxx[10 * X(Any$)",
  "?$10 * X(Any$)",
  "?=10 * X(Any$)",
  "?[10 * X[Any$$",
  "?[10 * X[Any$$",
  "X[Any$",
  "?X[Any$",
  "?[X(Any$)",
  "$X[Any$",
  "=X[Any$",
  "$[X(Any$)",
  "xxx[X(Any$)",
  "?$X(Any$)",
  "?=X(Any$)",
  "?[X[Any$$",
  "?[X[Any$$",
  "10 $ X[Any$",
  "?10 $ X[Any$",
  "?[10 $ X(Any$)",
  "$10 $ X[Any$",
  "=10 $ X[Any$",
  "$[10 $ X(Any$)",
  "xxx[10 $ X(Any$)",
  "?$10 $ X(Any$)",
  "?=10 $ X(Any$)",
  "?[10 $ X[Any$$",
  "?[10 $ X[Any$$",
  "10 = X[Any$",
  "?10 = X[Any$",
  "?[10 = X(Any$)",
  "$10 = X[Any$",
  "=10 = X[Any$",
  "$[10 = X(Any$)",
  "xxx[10 = X(Any$)",
  "?$10 = X(Any$)",
  "?=10 = X(Any$)",
  "?[10 = X[Any$$",
  "?[10 = X[Any$$",

  /* BEGIN RANDOM */
  "?[DdvP0pL6PQ * ... * WdWR * Fixed * uintptr)",
  "var * var * 332404638 * Fixed * xxx",
  "fixed[123768267] * W * var * {......}",
  "?[categorical[2992862455, 287385671, -1912160504, -5196, 932212418]]",
  "var * 3957054619 * var * Fixed * Ti * 1957830719 * var * var * varvar * U * float64",
  "?[178440563 * ... * fixed[2529695642] * Mxy6SD)",
  "3374755922 * Fixed * ... * NG * var * 1594671069 * var * xxx",
  "fixed_bytes(1342281638, align=xxx)",
  "Y5vtk * var * 905934102 * 1310846899 * fixed[3359714476] * LKgwlEDf * 2822334059 * ; PKI_8Ob8(fixed_string(795133707))",
  "?(E00Lr_HMM * ... * UAzC * var * M4nJaK * , var * var * 1002912577 * PZPD)",
  "?ref(fixed_string(3865363101, 'ucs-10'))",
  "fixed[3622391868] * SUc9IusL * CaPaWth4m * CEwSPY_6Z * HPW * SbvgP * 29709a97707 * 1300296096 * Fixed * VA8O",
  "fixed[1603107638)] * 2764230114 * IRXLe * Odlw6RSx * ... * Fixed * var * 2484716214 * ScalarKind",
  "ZmQxcy * var * 1098824011 * var * var * (,,,)",
  "ref(?x1869144321 * Y3_Dm0 * int32)",
  "categorical[\"a\", 1619552300, 490620000000000000000000000000000000000000, 18772, -84, 'l']",
  "?fixed[2889099917] * ... * var * JgdIXy5M * var * Wr2[categorical['gLsA7BM43V', 16460]]",
  "fixed[2510226978] * 1386472371 * MTaA + Fixed * var * ... * uint64",
  "?var * Fixed * FdN_HERKv * 4204012258 * var * fixed[233758212000000000000000000000000000] * Fixed * FixedBytesKind",
  "?categorical[30987, -16536, -270032755.642, -189, -823800099]",
  "?Fixed * 1998811088 * var * var * var * var * bytes(align=-58)",
  "257051452 * QRgi * Fixed * 4147849906 * var * P4h8i8 * var * 3924618173 * fixed[207255421] * int100",
  "categorical[408786241.474, 2840608325, 44038, 1763886348, \"jOt', 24]",
  "?[OQ78AW32d[categorical[49]%]",
  "categorical[-2049848438, 94, 3599739756, 3577056037, 318483846%634]",
  "?3733890560 * var * 4255463617 * var * Fixed * 1331110955 * var * 3898445876 * === * V840",
  "fixed[2788537974] * 609105307 * 3326611099 * var * 2802274826 * === * 726698562 * Fixed * var * UjsQXDD * float64",
  "categorical[2753441703, -67247918.4053]",
  "var * ... * xxx * 731398846 * 1036083219 * ... * ... * fixed[1657419491] * fixed[816888562] * uint16",
  "?[categorical[1071121364 : 100, 378862641, 48]]",
  "458285686 * fixed[] * uint8",
  "Dg_ * LHH1F_qO * Fixed * fixed[3791178904] * FI(___)",
  "Fixed * Eh * fixed[3099342530] * xxx[2987601612] * 4184075298 * fixed[2661930606] * QJkAi * var * 3172227083 * char",
  "?!2985779530 * var * FixedBytesKind",
  "1846361728 * Rf7jnH * F1vnDDAI * fixed[2337895812) * uint64",
  "fixed[3721367309] * ... * . * 541414917 * 942191038 * bool",
  "YOPZW * 243142180 * 3701263452 * complex_",
  "1293035889 * SQc2jI * 508611408 * var * ... * var * FYqqwF * VKte * uint256",
  "L_ * var * var . Fixed * var * float64",
  "any * 1998314652 * ... * Qs8C * 1987761882 * var * Fixed * fixed[4067017100] * 4267407167 * void",
  "_ * N * lz * var * 1100249655 * _ * Fixed * ScalarKind",
  "fixed[462488877] * * 1962474344 * fixed[] * Fixed * 1399046620 * ... * fixed[3139313879] * var * ZP",
  "1286044356 * LbR6JPFI * 2096958782 * _",
  "categorical[-507014936.36 : float256, -25910, 'xM3Mys0XqH', 4265882500]",
  "?[2957634486 * fixed[1015063449''] * 2153561367 * ... * bytes(align=])",
  "opt[2957634486 * fixed[1015063449''] * 2153561367 * ... * bytes(align=])",
  /* END RANDOM */

  /* BEGIN MANUALLY GENERATED */
  "(..)",
  "(...) ->",
  "(..., ...)",
  "(, ...)",
  "(, int64)",
  "{..}",
  "{...} ->",
  "{, ...}",
  "{..., }",
  "() -> (, ...)",
  "(..., int64, ...) -> complex128",
  "() -> int1024",
  "(scale: float1024) -> int32",
  "(, ...) -> int32",
  "(scale: float64, ..., color: float64, ...) -> int32",
  "(int32, float32, ..., scale: float64, ..., color: float64, ...) -> int32",

  "undefined_t",
  "10 * 20 * 3 * undefined_t",
  "10 * 20 * 3 * {x: int64, y: float64, z: undefined_t}",
  "(int32, float32, ..., scale : undefined_t, color : float64, ...) -> defined_t",
  "(a : ref({b : defined_t, c : (undefined_t, int16) -> float64})) -> defined_t",
  "(a : ref({b : defined_t, c : (defined_t, int16) -> float64})) -> undefined_t",

  "10 * ... * 20 * ... * int64",
  "(a : ref({b : defined_t, c : (var * ... * ... * {a : ref(float64)}, int16) -> float64})) -> (int64, complex128)",

  "categorical[-507014936.36, -25910, 'xM3Mys0XqH', 4265882500, -507014936.36]",

  "2 * !3 * 10 * int64",
  "2 * 3 * !10 * int64",
  "2 * 3 * !10 * !int64",
  "!!2 * 3 * 10 * !int64",
  "!N * 3 * 10 * int64",

  "var(offsets=[]) * {a: T}",
  "var(offsets=[0]) * Some(int64)",
  "var(offsets=[-1]) * Some(int64)",
  "var(offsets=[-1, -1]) * Some(int64)",
  "var(offsets=[0, -1]) * Some(int64)",

  /* Negative dimensions */
  "-2 * 4 * uint8",
  "-9223372036854775808 * uint8",
  "-2 * 4611686018427387904 * uint8",

  /* Overflow */
  "9223372036854775808 * uint8",
  "4611686018427387904 * 2 uint8",
  "2 * 4611686018427387904 * uint8",

  /* END MANUALLY GENERATED */

  NULL
};
